@@ -36,8 +36,7 @@
 // for debugging
 // #define AIO_STATS 1
 
-#undef AIO_MODULE_VERSION
-#define AIO_MODULE_VERSION makeModuleVersion(AIO_MODULE_MAJOR_VERSION, AIO_MODULE_MINOR_VERSION, PRIVATE_MODULE_HEADER)
+static constexpr ts::ModuleVersion AIO_MODULE_INTERNAL_VERSION{AIO_MODULE_PUBLIC_VERSION, ts::ModuleVersion::PRIVATE};
 
 TS_INLINE int
 AIOCallback::ok()
@@ -86,24 +85,19 @@ struct AIOCallbackInternal : public AIOCallback {
 
   int io_complete(int event, void *data);
 
-  AIOCallbackInternal()
-  {
-    aiocb.aio_reqprio = AIO_DEFAULT_PRIORITY;
-    SET_HANDLER(&AIOCallbackInternal::io_complete);
-  }
+  AIOCallbackInternal() { SET_HANDLER(&AIOCallbackInternal::io_complete); }
 };
 
 struct AIO_Reqs {
-  Que(AIOCallback, link) aio_todo;      /* queue for holding non-http requests */
-  Que(AIOCallback, link) http_aio_todo; /* queue for http requests */
-                                        /* Atomic list to temporarily hold the request if the
-                                           lock for a particular queue cannot be acquired */
+  Que(AIOCallback, link) aio_todo; /* queue for AIO operations */
+                                   /* Atomic list to temporarily hold the request if the
+                                      lock for a particular queue cannot be acquired */
   ASLL(AIOCallbackInternal, alink) aio_temp_list;
   ink_mutex aio_mutex;
   ink_cond aio_cond;
   int index           = 0; /* position of this struct in the aio_reqs array */
   int pending         = 0; /* number of outstanding requests on the disk */
-  int queued          = 0; /* total number of aio_todo and http_todo requests */
+  int queued          = 0; /* total number of aio_todo requests */
   int filedes         = 0; /* the file descriptor for the requests */
   int requests_queued = 0;
 };
