@@ -23,6 +23,35 @@
 
 #include "P_Cache.h"
 
+void
+CacheDisk::incrErrors(const AIOCallback *io)
+{
+  if (0 == this->num_errors) {
+    /* This it the first read/write error on this span since ATS started.
+     * Move the newly failing span from "online" to "failing" bucket. */
+    RecIncrGlobalRawStat(cache_rsb, static_cast<int>(cache_span_online_stat), -1);
+    RecIncrGlobalRawStat(cache_rsb, static_cast<int>(cache_span_failing_stat), 1);
+  }
+  this->num_errors++;
+
+  const char *opname = "unknown";
+  int opcode         = io->aiocb.aio_lio_opcode;
+  int fd             = io->aiocb.aio_fildes;
+  switch (io->aiocb.aio_lio_opcode) {
+  case LIO_READ:
+    opname = "READ";
+    RecIncrGlobalRawStat(cache_rsb, static_cast<int>(cache_span_errors_read_stat), 1);
+    break;
+  case LIO_WRITE:
+    opname = "WRITE";
+    RecIncrGlobalRawStat(cache_rsb, static_cast<int>(cache_span_errors_write_stat), 1);
+    break;
+  default:
+    break;
+  }
+  Warning("failed operation: %s (opcode=%d), span: %s (fd=%d)", opname, opcode, path, fd);
+}
+
 int
 CacheDisk::open(char *s, off_t blocks, off_t askip, int ahw_sector_size, int fildes, bool clear)
 {
@@ -32,10 +61,9 @@ CacheDisk::open(char *s, off_t blocks, off_t askip, int ahw_sector_size, int fil
   skip           = askip;
   start          = skip;
   /* we can't use fractions of store blocks. */
-  len                  = blocks;
-  io.aiocb.aio_fildes  = fd;
-  io.aiocb.aio_reqprio = 0;
-  io.action            = this;
+  len                 = blocks;
+  io.aiocb.aio_fildes = fd;
+  io.action           = this;
   // determine header size and hence start point by successive approximation
   uint64_t l;
   for (int i = 0; i < 3; i++) {
@@ -48,28 +76,31 @@ CacheDisk::open(char *s, off_t blocks, off_t askip, int ahw_sector_size, int fil
     start = skip + header_len;
   }
 
-  disk_vols         = (DiskVol **)ats_calloc((l / MIN_VOL_SIZE + 1), sizeof(DiskVol *));
+  disk_vols         = static_cast<DiskVol **>(ats_calloc((l / MIN_VOL_SIZE + 1), sizeof(DiskVol *)));
   header_len        = ROUND_TO_STORE_BLOCK(header_len);
   start             = skip + header_len;
   num_usable_blocks = (off_t(len * STORE_BLOCK_SIZE) - (start - askip)) >> STORE_BLOCK_SHIFT;
 
-  header = (DiskHeader *)ats_memalign(ats_pagesize(), header_len);
+  header = static_cast<DiskHeader *>(ats_memalign(ats_pagesize(), header_len));
   memset(header, 0, header_len);
+
+  // traffic server was asked to clear the cache, i.e., auto clear cache flag is set
   if (clear) {
     if (read_only_p) {
       fprintf(stderr, "Could not read disk header for disk %s", path);
       SET_DISK_BAD(this);
       SET_HANDLER(&CacheDisk::openDone);
-      return openDone(EVENT_IMMEDIATE, 0);
+      return openDone(EVENT_IMMEDIATE, nullptr);
     } else {
       SET_HANDLER(&CacheDisk::clearDone);
       return clearDisk();
     }
   }
 
+  //
   SET_HANDLER(&CacheDisk::openStart);
   io.aiocb.aio_offset = skip;
-  io.aiocb.aio_buf    = (char *)header;
+  io.aiocb.aio_buf    = reinterpret_cast<char *>(header);
   io.aiocb.aio_nbytes = header_len;
   io.thread           = AIO_CALLBACK_THREAD_ANY;
   ink_aio_read(&io);
@@ -80,8 +111,8 @@ CacheDisk::~CacheDisk()
 {
   if (path) {
     ats_free(path);
-    for (int i = 0; i < (int)header->num_volumes; i++) {
-      DiskVolBlockQueue *q = NULL;
+    for (int i = 0; i < static_cast<int>(header->num_volumes); i++) {
+      DiskVolBlockQueue *q = nullptr;
       while (disk_vols[i] && (q = (disk_vols[i]->dpb_queue.pop()))) {
         delete q;
       }
@@ -90,7 +121,7 @@ CacheDisk::~CacheDisk()
     free(header);
   }
   if (free_blocks) {
-    DiskVolBlockQueue *q = NULL;
+    DiskVolBlockQueue *q = nullptr;
     while ((q = (free_blocks->dpb_queue.pop()))) {
       delete q;
     }
@@ -116,14 +147,15 @@ CacheDisk::clearDone(int event, void * /* data ATS_UNUSED */)
 {
   ink_assert(event == AIO_EVENT_DONE);
 
-  if ((size_t)io.aiocb.aio_nbytes != (size_t)io.aio_result) {
+  if (io.aiocb.aio_nbytes != static_cast<size_t>(io.aio_result)) {
     Warning("Could not clear disk header for disk %s: declaring disk bad", path);
+    incrErrors(&io);
     SET_DISK_BAD(this);
   }
   //  update_header();
 
   SET_HANDLER(&CacheDisk::openDone);
-  return openDone(EVENT_IMMEDIATE, 0);
+  return openDone(EVENT_IMMEDIATE, nullptr);
 }
 
 int
@@ -131,11 +163,12 @@ CacheDisk::openStart(int event, void * /* data ATS_UNUSED */)
 {
   ink_assert(event == AIO_EVENT_DONE);
 
-  if ((size_t)io.aiocb.aio_nbytes != (size_t)io.aio_result) {
+  if (io.aiocb.aio_nbytes != static_cast<size_t>(io.aio_result)) {
     Warning("could not read disk header for disk %s: declaring disk bad", path);
+    incrErrors(&io);
     SET_DISK_BAD(this);
     SET_HANDLER(&CacheDisk::openDone);
-    return openDone(EVENT_IMMEDIATE, 0);
+    return openDone(EVENT_IMMEDIATE, nullptr);
   }
 
   if (header->magic != DISK_HEADER_MAGIC || header->num_blocks != static_cast<uint64_t>(len)) {
@@ -168,7 +201,7 @@ CacheDisk::openStart(int event, void * /* data ATS_UNUSED */)
   update_header();
 
   SET_HANDLER(&CacheDisk::openDone);
-  return openDone(EVENT_IMMEDIATE, 0);
+  return openDone(EVENT_IMMEDIATE, nullptr);
 }
 
 int
@@ -200,8 +233,9 @@ CacheDisk::syncDone(int event, void * /* data ATS_UNUSED */)
 {
   ink_assert(event == AIO_EVENT_DONE);
 
-  if ((size_t)io.aiocb.aio_nbytes != (size_t)io.aio_result) {
+  if (io.aiocb.aio_nbytes != static_cast<size_t>(io.aio_result)) {
     Warning("Error writing disk header for disk %s:disk bad", path);
+    incrErrors(&io);
     SET_DISK_BAD(this);
     return EVENT_DONE;
   }
@@ -213,35 +247,35 @@ CacheDisk::syncDone(int event, void * /* data ATS_UNUSED */)
 DiskVolBlock *
 CacheDisk::create_volume(int number, off_t size_in_blocks, int scheme)
 {
-  if (size_in_blocks == 0)
-    return NULL;
+  if (size_in_blocks == 0) {
+    return nullptr;
+  }
 
   DiskVolBlockQueue *q             = free_blocks->dpb_queue.head;
   DiskVolBlockQueue *closest_match = q;
 
-  if (!q)
-    return NULL;
+  if (!q) {
+    return nullptr;
+  }
 
   off_t max_blocks = MAX_VOL_SIZE >> STORE_BLOCK_SHIFT;
   size_in_blocks   = (size_in_blocks <= max_blocks) ? size_in_blocks : max_blocks;
 
   int blocks_per_vol = VOL_BLOCK_SIZE / STORE_BLOCK_SIZE;
   //  ink_assert(!(size_in_blocks % blocks_per_vol));
-  DiskVolBlock *p = 0;
+  DiskVolBlock *p = nullptr;
   for (; q; q = q->link.next) {
-    if ((off_t)q->b->len >= size_in_blocks) {
+    if (static_cast<off_t>(q->b->len) >= size_in_blocks) {
       p            = q->b;
       q->new_block = 1;
       break;
     } else {
-      if (closest_match->b->len < q->b->len)
+      if (closest_match->b->len < q->b->len) {
         closest_match = q;
+      }
     }
   }
 
-  if (!p && !closest_match)
-    return NULL;
-
   if (!p && closest_match) {
     /* allocate from the closest match */
     q            = closest_match;
@@ -259,12 +293,12 @@ CacheDisk::create_volume(int number, off_t size_in_blocks, int scheme)
   free_blocks->size -= p->len;
 
   size_t new_size = p->len - size_in_blocks;
-  if (new_size >= (size_t)blocks_per_vol) {
+  if (new_size >= static_cast<size_t>(blocks_per_vol)) {
     /* create a new volume */
     DiskVolBlock *dpb = &header->vol_info[header->num_diskvol_blks];
     *dpb              = *p;
     dpb->len -= size_in_blocks;
-    dpb->offset += ((off_t)size_in_blocks * STORE_BLOCK_SIZE);
+    dpb->offset += (size_in_blocks * STORE_BLOCK_SIZE);
 
     DiskVolBlockQueue *new_q = new DiskVolBlockQueue();
     new_q->b                 = dpb;
@@ -272,8 +306,9 @@ CacheDisk::create_volume(int number, off_t size_in_blocks, int scheme)
     free_blocks->size += dpb->len;
     free_space += dpb->len;
     header->num_diskvol_blks++;
-  } else
+  } else {
     header->num_free--;
+  }
 
   p->len    = size_in_blocks;
   p->free   = 0;
@@ -344,7 +379,7 @@ CacheDisk::update_header()
   unsigned int n = 0;
   unsigned int i, j;
   if (free_blocks) {
-    DiskVolBlockQueue *q = NULL;
+    DiskVolBlockQueue *q = nullptr;
     while ((q = (free_blocks->dpb_queue.pop()))) {
       delete q;
     }
@@ -408,7 +443,7 @@ CacheDisk::get_diskvol(int vol_number)
       return disk_vols[i];
     }
   }
-  return NULL;
+  return nullptr;
 }
 
 int
