@@ -22,19 +22,20 @@
  */
 
 #include "P_Cache.h"
-#include "ts/I_Layout.h"
-#include "ts/HostLookup.h"
-#include "ts/Tokenizer.h"
+#include "tscore/I_Layout.h"
+#include "tscore/HostLookup.h"
+#include "tscore/Tokenizer.h"
+#include "tscore/Regression.h"
+#include "tscore/Filenames.h"
+#include "tscore/ts_file.h"
 
 extern int gndisks;
 
-matcher_tags CacheHosting_tags = {"hostname", "domain", NULL, NULL, NULL, NULL, 0};
-
 /*************************************************************
  *   Begin class HostMatcher
  *************************************************************/
 
-CacheHostMatcher::CacheHostMatcher(const char *name, CacheType typ) : data_array(NULL), array_len(-1), num_el(-1), type(typ)
+CacheHostMatcher::CacheHostMatcher(const char *name, CacheType typ) : data_array(nullptr), array_len(-1), num_el(-1), type(typ)
 {
   host_lookup = new HostLookup(name);
 }
@@ -67,7 +68,7 @@ CacheHostMatcher::Print()
 void
 CacheHostMatcher::PrintFunc(void *opaque_data)
 {
-  CacheHostRecord *d = (CacheHostRecord *)opaque_data;
+  CacheHostRecord *d = static_cast<CacheHostRecord *>(opaque_data);
   d->Print();
 }
 
@@ -95,21 +96,22 @@ CacheHostMatcher::AllocateSpace(int num_entries)
 //    arg hostname
 //
 void
-CacheHostMatcher::Match(char const *rdata, int rlen, CacheHostResult *result)
+CacheHostMatcher::Match(const char *rdata, int rlen, CacheHostResult *result)
 {
   void *opaque_ptr;
   CacheHostRecord *data_ptr;
   bool r;
 
-  // Check to see if there is any work to do before makeing
-  //   the stirng copy
+  // Check to see if there is any work to do before making
+  //   the string copy
   if (num_el <= 0) {
     return;
   }
 
-  if (rlen == 0)
+  if (rlen == 0) {
     return;
-  char *data = (char *)ats_malloc(rlen + 1);
+  }
+  char *data = static_cast<char *>(ats_malloc(rlen + 1));
   memcpy(data, rdata, rlen);
   *(data + rlen) = '\0';
   HostLookupState s;
@@ -117,8 +119,8 @@ CacheHostMatcher::Match(char const *rdata, int rlen, CacheHostResult *result)
   r = host_lookup->MatchFirst(data, &s, &opaque_ptr);
 
   while (r == true) {
-    ink_assert(opaque_ptr != NULL);
-    data_ptr = (CacheHostRecord *)opaque_ptr;
+    ink_assert(opaque_ptr != nullptr);
+    data_ptr = static_cast<CacheHostRecord *>(opaque_ptr);
     data_ptr->UpdateMatch(result, data);
 
     r = host_lookup->MatchNext(&s, &opaque_ptr);
@@ -151,11 +153,12 @@ CacheHostMatcher::NewEntry(matcher_line *line_info)
 
   // Make sure that the line_info is not bogus
   ink_assert(line_info->dest_entry < MATCHER_MAX_TOKENS);
-  ink_assert(match_data != NULL);
+  ink_assert(match_data != nullptr);
 
   // Remove our consumed label from the parsed line
-  if (line_info->dest_entry < MATCHER_MAX_TOKENS)
-    line_info->line[0][line_info->dest_entry] = NULL;
+  if (line_info->dest_entry < MATCHER_MAX_TOKENS) {
+    line_info->line[0][line_info->dest_entry] = nullptr;
+  }
   line_info->num_el--;
 
   // Fill in the parameter info
@@ -164,7 +167,7 @@ CacheHostMatcher::NewEntry(matcher_line *line_info)
 
   if (errNo) {
     // There was a problem so undo the effects this function
-    memset(cur_d, 0, sizeof(CacheHostRecord));
+    memset(static_cast<void *>(cur_d), 0, sizeof(CacheHostRecord));
     return;
   }
   Debug("cache_hosting", "hostname: %s, host record: %p", match_data, cur_d);
@@ -183,14 +186,9 @@ CacheHostTable::CacheHostTable(Cache *c, CacheType typ)
 {
   ats_scoped_str config_path;
 
-  config_tags = &CacheHosting_tags;
-  ink_assert(config_tags != NULL);
-
   type         = typ;
   cache        = c;
   matcher_name = "[CacheHosting]";
-  ;
-  hostMatch = NULL;
 
   config_path = RecConfigReadConfigPath("proxy.config.cache.hosting_filename");
   ink_release_assert(config_path);
@@ -200,7 +198,7 @@ CacheHostTable::CacheHostTable(Cache *c, CacheType typ)
 
 CacheHostTable::~CacheHostTable()
 {
-  if (hostMatch != NULL) {
+  if (hostMatch != nullptr) {
     delete hostMatch;
   }
 }
@@ -213,7 +211,7 @@ void
 CacheHostTable::Print()
 {
   printf("Control Matcher Table: %s\n", matcher_name);
-  if (hostMatch != NULL) {
+  if (hostMatch != nullptr) {
     hostMatch->Print();
   }
 }
@@ -224,7 +222,7 @@ CacheHostTable::Print()
 //   Queries each table for the Result*
 //
 void
-CacheHostTable::Match(char const *rdata, int rlen, CacheHostResult *result)
+CacheHostTable::Match(const char *rdata, int rlen, CacheHostResult *result)
 {
   hostMatch->Match(rdata, rlen, result);
 }
@@ -233,7 +231,7 @@ int
 CacheHostTable::config_callback(const char * /* name ATS_UNUSED */, RecDataT /* data_type ATS_UNUSED */,
                                 RecData /* data ATS_UNUSED */, void *cookie)
 {
-  CacheHostTable **ppt = (CacheHostTable **)cookie;
+  CacheHostTable **ppt = static_cast<CacheHostTable **>(cookie);
   eventProcessor.schedule_imm(new CacheHostTableConfig(ppt));
   return 0;
 }
@@ -248,17 +246,19 @@ int fstat_wrapper(int fd, struct stat *s);
 int
 CacheHostTable::BuildTableFromString(const char *config_file_path, char *file_buf)
 {
+  Note("%s loading ...", ts::filename::HOSTING);
+
   // Table build locals
   Tokenizer bufTok("\n");
   tok_iter_state i_state;
-  const char *tmp;
-  matcher_line *first = NULL;
-  matcher_line *current;
-  matcher_line *last = NULL;
-  int line_num       = 0;
-  int second_pass    = 0;
-  int numEntries     = 0;
-  const char *errPtr = NULL;
+  const char *tmp       = nullptr;
+  matcher_line *first   = nullptr;
+  matcher_line *current = nullptr;
+  matcher_line *last    = nullptr;
+  int line_num          = 0;
+  int second_pass       = 0;
+  int numEntries        = 0;
+  const char *errPtr    = nullptr;
 
   // type counts
   int hostDomain = 0;
@@ -267,13 +267,14 @@ CacheHostTable::BuildTableFromString(const char *config_file_path, char *file_bu
     // We have an empty file
     /* no hosting customers -- put all the volumes in the
        generic table */
-    if (gen_host_rec.Init(type))
+    if (gen_host_rec.Init(type)) {
       Warning("Problems encountered while initializing the Generic Volume");
+    }
     return 0;
   }
   // First get the number of entries
   tmp = bufTok.iterFirst(&i_state);
-  while (tmp != NULL) {
+  while (tmp != nullptr) {
     line_num++;
 
     // skip all blank spaces at beginning of line
@@ -282,10 +283,10 @@ CacheHostTable::BuildTableFromString(const char *config_file_path, char *file_bu
     }
 
     if (*tmp != '#' && *tmp != '\0') {
-      current = (matcher_line *)ats_malloc(sizeof(matcher_line));
-      errPtr  = parseConfigLine((char *)tmp, current, config_tags);
+      current = static_cast<matcher_line *>(ats_malloc(sizeof(matcher_line)));
+      errPtr  = parseConfigLine(const_cast<char *>(tmp), current, &config_tags);
 
-      if (errPtr != NULL) {
+      if (errPtr != nullptr) {
         RecSignalWarning(REC_SIGNAL_CONFIG_ERROR, "%s discarding %s entry at line %d : %s", matcher_name, config_file_path,
                          line_num, errPtr);
         ats_free(current);
@@ -305,9 +306,10 @@ CacheHostTable::BuildTableFromString(const char *config_file_path, char *file_bu
           ink_assert(0);
         }
 
-        if (first == NULL) {
-          ink_assert(last == NULL);
-          first = last = current;
+        if (first == nullptr) {
+          ink_assert(last == nullptr);
+          first = current;
+          last  = current;
         } else {
           last->next = current;
           last       = current;
@@ -325,10 +327,7 @@ CacheHostTable::BuildTableFromString(const char *config_file_path, char *file_bu
     if (gen_host_rec.Init(type)) {
       Warning("Problems encountered while initializing the Generic Volume");
     }
-
-    if (first != NULL) {
-      ats_free(first);
-    }
+    Note("%s finished loading", ts::filename::HOSTING);
     return 0;
   }
 
@@ -339,11 +338,11 @@ CacheHostTable::BuildTableFromString(const char *config_file_path, char *file_bu
   // Traverse the list and build the records table
   int generic_rec_initd = 0;
   current               = first;
-  while (current != NULL) {
+  while (current != nullptr) {
     second_pass++;
     if ((current->type == MATCH_DOMAIN) || (current->type == MATCH_HOST)) {
       char *match_data = current->line[1][current->dest_entry];
-      ink_assert(match_data != NULL);
+      ink_assert(match_data != nullptr);
 
       if (!strcasecmp(match_data, "*")) {
         // generic volume - initialize the generic hostrecord */
@@ -351,16 +350,18 @@ CacheHostTable::BuildTableFromString(const char *config_file_path, char *file_bu
         ink_assert(current->dest_entry < MATCHER_MAX_TOKENS);
 
         // Remove our consumed label from the parsed line
-        if (current->dest_entry < MATCHER_MAX_TOKENS)
-          current->line[0][current->dest_entry] = NULL;
-        else
+        if (current->dest_entry < MATCHER_MAX_TOKENS) {
+          current->line[0][current->dest_entry] = nullptr;
+        } else {
           Warning("Problems encountered while initializing the Generic Volume");
+        }
 
         current->num_el--;
-        if (!gen_host_rec.Init(current, type))
+        if (!gen_host_rec.Init(current, type)) {
           generic_rec_initd = 1;
-        else
+        } else {
           Warning("Problems encountered while initializing the Generic Volume");
+        }
 
       } else {
         hostMatch->NewEntry(current);
@@ -376,6 +377,8 @@ CacheHostTable::BuildTableFromString(const char *config_file_path, char *file_bu
     ats_free(last);
   }
 
+  Note("%s finished loading", ts::filename::HOSTING);
+
   if (!generic_rec_initd) {
     const char *cache_type = (type == CACHE_HTTP_TYPE) ? "http" : "mixt";
     RecSignalWarning(REC_SIGNAL_CONFIG_ERROR,
@@ -394,20 +397,22 @@ CacheHostTable::BuildTableFromString(const char *config_file_path, char *file_bu
 int
 CacheHostTable::BuildTable(const char *config_file_path)
 {
-  char *file_buf;
-  int ret;
+  std::error_code ec;
+  std::string content{ts::file::load(ts::file::path{config_file_path}, ec)};
 
-  file_buf = readIntoBuffer(config_file_path, matcher_name, NULL);
-
-  if (file_buf == NULL) {
-    Warning("Cannot read the config file: %s", config_file_path);
-    gen_host_rec.Init(type);
-    return 0;
+  if (ec) {
+    switch (ec.value()) {
+    case ENOENT:
+      Warning("Cannot open the config file: %s - %s", config_file_path, strerror(ec.value()));
+      break;
+    default:
+      Error("%s failed to load: %s", config_file_path, strerror(ec.value()));
+      gen_host_rec.Init(type);
+      return 0;
+    }
   }
 
-  ret = BuildTableFromString(config_file_path, file_buf);
-  ats_free(file_buf);
-  return ret;
+  return BuildTableFromString(config_file_path, content.data());
 }
 
 int
@@ -419,7 +424,7 @@ CacheHostRecord::Init(CacheType typ)
 
   num_vols = 0;
   type     = typ;
-  cp       = (CacheVol **)ats_malloc(cp_list_len * sizeof(CacheVol *));
+  cp       = static_cast<CacheVol **>(ats_malloc(cp_list_len * sizeof(CacheVol *)));
   memset(cp, 0, cp_list_len * sizeof(CacheVol *));
   num_cachevols    = 0;
   CacheVol *cachep = cp_list.head;
@@ -435,7 +440,7 @@ CacheHostRecord::Init(CacheType typ)
     RecSignalWarning(REC_SIGNAL_CONFIG_ERROR, "error: No volumes found for Cache Type %d", type);
     return -1;
   }
-  vols        = (Vol **)ats_malloc(num_vols * sizeof(Vol *));
+  vols        = static_cast<Vol **>(ats_malloc(num_vols * sizeof(Vol *)));
   int counter = 0;
   for (i = 0; i < num_cachevols; i++) {
     CacheVol *cachep1 = cp[i];
@@ -461,8 +466,9 @@ CacheHostRecord::Init(matcher_line *line_info, CacheType typ)
   type = typ;
   for (i = 0; i < MATCHER_MAX_TOKENS; i++) {
     char *label = line_info->line[0][i];
-    if (!label)
+    if (!label) {
       continue;
+    }
     char *val;
 
     if (!strcasecmp(label, "volume")) {
@@ -482,7 +488,7 @@ CacheHostRecord::Init(matcher_line *line_info, CacheType typ)
             const char *errptr = "A volume number expected";
             RecSignalWarning(REC_SIGNAL_CONFIG_ERROR, "%s discarding %s entry at line %d :%s", "[CacheHosting]", config_file,
                              line_info->line_num, errptr);
-            if (val != NULL) {
+            if (val != nullptr) {
               ats_free(val);
             }
             return -1;
@@ -491,19 +497,22 @@ CacheHostRecord::Init(matcher_line *line_info, CacheType typ)
         if ((*s < '0') || (*s > '9')) {
           RecSignalWarning(REC_SIGNAL_CONFIG_ERROR, "%s discarding %s entry at line %d : bad token [%c]", "[CacheHosting]",
                            config_file, line_info->line_num, *s);
-          if (val != NULL) {
+          if (val != nullptr) {
             ats_free(val);
           }
           return -1;
         }
         s++;
       }
+      if (val == nullptr) {
+        return -1;
+      }
       s = val;
       num_cachevols++;
-      cp = (CacheVol **)ats_malloc(num_cachevols * sizeof(CacheVol *));
+      cp = static_cast<CacheVol **>(ats_malloc(num_cachevols * sizeof(CacheVol *)));
       memset(cp, 0, num_cachevols * sizeof(CacheVol *));
       num_cachevols = 0;
-      while (1) {
+      while (true) {
         char c = *s;
         if ((c == ',') || (c == '\0')) {
           *s            = '\0';
@@ -526,20 +535,20 @@ CacheHostRecord::Init(matcher_line *line_info, CacheType typ)
           if (!is_vol_present) {
             RecSignalWarning(REC_SIGNAL_CONFIG_ERROR, "%s discarding %s entry at line %d : bad volume number [%d]",
                              "[CacheHosting]", config_file, line_info->line_num, volume_number);
-            if (val != NULL) {
+            if (val != nullptr) {
               ats_free(val);
             }
             return -1;
           }
-          if (c == '\0')
+          if (c == '\0') {
             break;
+          }
           vol_no = s + 1;
         }
         s++;
       }
-      if (val != NULL) {
-        ats_free(val);
-      }
+      ats_free(val);
+
       break;
     }
 
@@ -557,7 +566,7 @@ CacheHostRecord::Init(matcher_line *line_info, CacheType typ)
   if (!num_vols) {
     return -1;
   }
-  vols        = (Vol **)ats_malloc(num_vols * sizeof(Vol *));
+  vols        = static_cast<Vol **>(ats_malloc(num_vols * sizeof(Vol *)));
   int counter = 0;
   for (i = 0; i < num_cachevols; i++) {
     CacheVol *cachep = cp[i];
@@ -586,19 +595,29 @@ void
 ConfigVolumes::read_config_file()
 {
   ats_scoped_str config_path;
-  char *file_buf;
 
   config_path = RecConfigReadConfigPath("proxy.config.cache.volume_filename");
   ink_release_assert(config_path);
 
-  file_buf = readIntoBuffer(config_path, "[CacheVolition]", NULL);
-  if (file_buf == NULL) {
-    Warning("Cannot read the config file: %s", (const char *)config_path);
-    return;
+  Note("%s loading ...", ts::filename::VOLUME);
+
+  std::error_code ec;
+  std::string content{ts::file::load(ts::file::path{config_path}, ec)};
+
+  if (ec) {
+    switch (ec.value()) {
+    case ENOENT:
+      Warning("Cannot open the config file: %s - %s", (const char *)config_path, strerror(ec.value()));
+      break;
+    default:
+      Error("%s failed to load: %s", (const char *)config_path, strerror(ec.value()));
+      return;
+    }
   }
 
-  BuildListFromString(config_path, file_buf);
-  ats_free(file_buf);
+  BuildListFromString(config_path, content.data());
+  Note("volume.config finished loading");
+
   return;
 }
 
@@ -616,9 +635,8 @@ ConfigVolumes::BuildListFromString(char *config_file_path, char *file_buf)
   const char *matcher_name = "[CacheVolition]";
 
   memset(volume_seen, 0, sizeof(volume_seen));
-  num_volumes        = 0;
-  num_stream_volumes = 0;
-  num_http_volumes   = 0;
+  num_volumes      = 0;
+  num_http_volumes = 0;
 
   if (bufTok.Initialize(file_buf, SHARE_TOKS | ALLOW_EMPTY_TOKS) == 0) {
     // We have an empty file
@@ -628,18 +646,19 @@ ConfigVolumes::BuildListFromString(char *config_file_path, char *file_buf)
 
   // First get the number of entries
   tmp = bufTok.iterFirst(&i_state);
-  while (tmp != NULL) {
+  while (tmp != nullptr) {
     line_num++;
 
     char *end;
-    char *line_end    = NULL;
-    const char *err   = NULL;
-    int volume_number = 0;
-    CacheType scheme  = CACHE_NONE_TYPE;
-    int size          = 0;
-    int in_percent    = 0;
+    char *line_end        = nullptr;
+    const char *err       = nullptr;
+    int volume_number     = 0;
+    CacheType scheme      = CACHE_NONE_TYPE;
+    int size              = 0;
+    int in_percent        = 0;
+    bool ramcache_enabled = true;
 
-    while (1) {
+    while (true) {
       // skip all blank spaces at beginning of line
       while (*tmp && isspace(*tmp)) {
         tmp++;
@@ -652,19 +671,20 @@ ConfigVolumes::BuildListFromString(char *config_file_path, char *file_buf)
         break;
       }
 
-      end = (char *)tmp;
-      while (*end && !isspace(*end))
+      end = const_cast<char *>(tmp);
+      while (*end && !isspace(*end)) {
         end++;
+      }
 
-      if (!(*end))
+      if (!(*end)) {
         line_end = end;
-      else {
+      } else {
         line_end = end + 1;
         *end     = '\0';
       }
       char *eq_sign;
 
-      eq_sign = (char *)strchr(tmp, '=');
+      eq_sign = const_cast<char *>(strchr(tmp, '='));
       if (!eq_sign) {
         err = "Unexpected end of line";
         break;
@@ -707,14 +727,15 @@ ConfigVolumes::BuildListFromString(char *config_file_path, char *file_buf)
         tmp += 5;
         size = atoi(tmp);
 
-        while (ParseRules::is_digit(*tmp))
+        while (ParseRules::is_digit(*tmp)) {
           tmp++;
+        }
 
         if (*tmp == '%') {
           // added by YTS Team, yamsat for bug id 59632
           total += size;
           if (size > 100 || total > 100) {
-            err = "Total volume size added upto more than 100 percent, No volumes created";
+            err = "Total volume size added up to more than 100 percent, No volumes created";
             break;
           }
           // ends here
@@ -723,11 +744,24 @@ ConfigVolumes::BuildListFromString(char *config_file_path, char *file_buf)
         } else {
           in_percent = 0;
         }
+      } else if (strcasecmp(tmp, "ramcache") == 0) { // match ramcache
+        tmp += 9;
+        if (!strcasecmp(tmp, "false")) {
+          tmp += 5;
+          ramcache_enabled = false;
+        } else if (!strcasecmp(tmp, "true")) {
+          tmp += 4;
+          ramcache_enabled = true;
+        } else {
+          err = "Unexpected end of line";
+          break;
+        }
       }
 
       // ends here
-      if (end < line_end)
+      if (end < line_end) {
         tmp++;
+      }
     }
 
     if (err) {
@@ -740,21 +774,23 @@ ConfigVolumes::BuildListFromString(char *config_file_path, char *file_buf)
       configp->number    = volume_number;
       if (in_percent) {
         configp->percent    = size;
-        configp->in_percent = 1;
+        configp->in_percent = true;
       } else {
-        configp->in_percent = 0;
+        configp->in_percent = false;
       }
-      configp->scheme = scheme;
-      configp->size   = size;
-      configp->cachep = NULL;
+      configp->scheme           = scheme;
+      configp->size             = size;
+      configp->cachep           = nullptr;
+      configp->ramcache_enabled = ramcache_enabled;
       cp_queue.enqueue(configp);
       num_volumes++;
       if (scheme == CACHE_HTTP_TYPE) {
         num_http_volumes++;
       } else {
-        num_stream_volumes++;
+        ink_release_assert(!"Unexpected non-HTTP cache volume");
       }
-      Debug("cache_hosting", "added volume=%d, scheme=%d, size=%d percent=%d", volume_number, scheme, size, in_percent);
+      Debug("cache_hosting", "added volume=%d, scheme=%d, size=%d percent=%d, ramcache enabled=%d", volume_number, scheme, size,
+            in_percent, ramcache_enabled);
     }
 
     tmp = bufTok.iterNext(&i_state);
@@ -763,14 +799,13 @@ ConfigVolumes::BuildListFromString(char *config_file_path, char *file_buf)
   return;
 }
 
-/* Test the cache volumeing with different configurations */
+/* Test the cache volume with different configurations */
 #define MEGS_128 (128 * 1024 * 1024)
 #define ROUND_TO_VOL_SIZE(_x) (((_x) + (MEGS_128 - 1)) & ~(MEGS_128 - 1))
 extern CacheDisk **gdisks;
 extern Queue<CacheVol> cp_list;
 extern int cp_list_len;
 extern ConfigVolumes config_volumes;
-extern volatile int gnvol;
 
 extern void cplist_init();
 extern int cplist_reconfigure();
@@ -791,7 +826,7 @@ static void restore_state();
 EXCLUSIVE_REGRESSION_TEST(Cache_vol)(RegressionTest *t, int /* atype ATS_UNUSED */, int *status)
 {
   save_state();
-  srand48(time(NULL));
+  srand48(time(nullptr));
   *status = REGRESSION_TEST_PASSED;
   for (int i = 0; i < configs; i++) {
     if (create_config(t, i)) {
@@ -822,14 +857,15 @@ create_config(RegressionTest *t, int num)
       }
       /* create 128 MB volumes */
       for (; blocks >= STORE_BLOCKS_PER_VOL; blocks -= STORE_BLOCKS_PER_VOL) {
-        if (vol_num > 255)
+        if (vol_num > 255) {
           break;
+        }
         ConfigVol *cp  = new ConfigVol();
         cp->number     = vol_num++;
         cp->scheme     = CACHE_HTTP_TYPE;
         cp->size       = 128;
-        cp->in_percent = 0;
-        cp->cachep     = 0;
+        cp->in_percent = false;
+        cp->cachep     = nullptr;
         config_volumes.cp_queue.enqueue(cp);
         config_volumes.num_volumes++;
         config_volumes.num_http_volumes++;
@@ -853,8 +889,8 @@ create_config(RegressionTest *t, int num)
       total_space += vol_blocks;
     }
 
-    // make sure we have atleast 1280 M bytes
-    if (total_space<(10 << 27)>> STORE_BLOCK_SHIFT) {
+    // make sure we have at least 1280 M bytes
+    if (total_space < ((10 << 27) >> STORE_BLOCK_SHIFT)) {
       rprintf(t, "Not enough space for 10 volume\n");
       return 0;
     }
@@ -867,8 +903,8 @@ create_config(RegressionTest *t, int num)
       cp->scheme     = CACHE_HTTP_TYPE;
       cp->size       = 10;
       cp->percent    = 10;
-      cp->in_percent = 1;
-      cp->cachep     = 0;
+      cp->in_percent = true;
+      cp->cachep     = nullptr;
       config_volumes.cp_queue.enqueue(cp);
       config_volumes.num_volumes++;
       config_volumes.num_http_volumes++;
@@ -904,8 +940,9 @@ create_config(RegressionTest *t, int num)
       }
     }
     while (total_space > 0) {
-      if (vol_num > 255)
+      if (vol_num > 255) {
         break;
+      }
       off_t modu = MAX_VOL_SIZE;
       if (total_space < (MAX_VOL_SIZE >> STORE_BLOCK_SHIFT)) {
         modu = total_space * STORE_BLOCK_SIZE;
@@ -925,16 +962,15 @@ create_config(RegressionTest *t, int num)
       cp->scheme     = scheme;
       cp->size       = random_size >> 20;
       cp->percent    = 0;
-      cp->in_percent = 0;
-      cp->cachep     = 0;
+      cp->in_percent = false;
+      cp->cachep     = nullptr;
       config_volumes.cp_queue.enqueue(cp);
       config_volumes.num_volumes++;
       if (cp->scheme == CACHE_HTTP_TYPE) {
         config_volumes.num_http_volumes++;
         rprintf(t, "volume=%d scheme=http size=%d\n", cp->number, cp->size);
       } else {
-        config_volumes.num_stream_volumes++;
-        rprintf(t, "volume=%d scheme=rtsp size=%d\n", cp->number, cp->size);
+        // ToDo: Assert ?
       }
     }
   } break;
@@ -952,8 +988,9 @@ execute_and_verify(RegressionTest *t)
   cplist_reconfigure();
 
   /* compare the volumes */
-  if (cp_list_len != config_volumes.num_volumes)
+  if (cp_list_len != config_volumes.num_volumes) {
     return REGRESSION_TEST_FAILED;
+  }
 
   /* check that the volumes and sizes
      match the configuration */
@@ -1019,10 +1056,10 @@ execute_and_verify(RegressionTest *t)
     CacheDisk *d = gdisks[i];
     if (is_debug_tag_set("cache_hosting")) {
       Debug("cache_hosting", "Disk: %d: Vol Blocks: %u: Free space: %" PRIu64, i, d->header->num_diskvol_blks, d->free_space);
-      for (int j = 0; j < (int)d->header->num_volumes; j++) {
+      for (int j = 0; j < static_cast<int>(d->header->num_volumes); j++) {
         Debug("cache_hosting", "\tVol: %d Size: %" PRIu64, d->disk_vols[j]->vol_number, d->disk_vols[j]->size);
       }
-      for (int j = 0; j < (int)d->header->num_diskvol_blks; j++) {
+      for (int j = 0; j < static_cast<int>(d->header->num_diskvol_blks); j++) {
         Debug("cache_hosting", "\tBlock No: %d Size: %" PRIu64 " Free: %u", d->header->vol_info[j].number,
               d->header->vol_info[j].len, d->header->vol_info[j].free);
       }
@@ -1035,7 +1072,7 @@ int
 ClearConfigVol(ConfigVolumes *configp)
 {
   int i         = 0;
-  ConfigVol *cp = NULL;
+  ConfigVol *cp = nullptr;
   while ((cp = configp->cp_queue.dequeue())) {
     delete cp;
     i++;
@@ -1044,9 +1081,8 @@ ClearConfigVol(ConfigVolumes *configp)
     Warning("failed");
     return 0;
   }
-  configp->num_volumes        = 0;
-  configp->num_http_volumes   = 0;
-  configp->num_stream_volumes = 0;
+  configp->num_volumes      = 0;
+  configp->num_http_volumes = 0;
   return 1;
 }
 
@@ -1054,7 +1090,7 @@ int
 ClearCacheVolList(Queue<CacheVol> *cpl, int len)
 {
   int i        = 0;
-  CacheVol *cp = NULL;
+  CacheVol *cp = nullptr;
   while ((cp = cpl->dequeue())) {
     ats_free(cp->disk_vols);
     ats_free(cp->vols);
@@ -1076,8 +1112,8 @@ save_state()
   saved_cp_list_len = cp_list_len;
   memcpy(&saved_config_volumes, &config_volumes, sizeof(ConfigVolumes));
   saved_gnvol = gnvol;
-  memset(&cp_list, 0, sizeof(Queue<CacheVol>));
-  memset(&config_volumes, 0, sizeof(ConfigVolumes));
+  memset(static_cast<void *>(&cp_list), 0, sizeof(Queue<CacheVol>));
+  memset(static_cast<void *>(&config_volumes), 0, sizeof(ConfigVolumes));
   gnvol = 0;
 }
 
