@@ -21,9 +21,10 @@
   limitations under the License.
  */
 
-#ifndef __P_CACHE_HOSTING_H__
-#define __P_CACHE_HOSTING_H__
+#pragma once
 #include "P_Cache.h"
+#include "tscore/MatcherUtils.h"
+#include "tscore/HostLookup.h"
 
 #define CACHE_MEM_FREE_TIMEOUT HRTIME_SECONDS(1)
 
@@ -45,34 +46,24 @@ struct CacheHostRecord {
     ats_free(cp);
   }
 
-  CacheType type;
-  Vol **vols;
-  volatile int good_num_vols;
-  volatile int num_vols;
-  int num_initialized;
-  unsigned short *vol_hash_table;
-  CacheVol **cp;
-  int num_cachevols;
+  CacheType type                 = CACHE_NONE_TYPE;
+  Vol **vols                     = nullptr;
+  int good_num_vols              = 0;
+  int num_vols                   = 0;
+  int num_initialized            = 0;
+  unsigned short *vol_hash_table = nullptr;
+  CacheVol **cp                  = nullptr;
+  int num_cachevols              = 0;
 
-  CacheHostRecord()
-    : type(CACHE_NONE_TYPE),
-      vols(NULL),
-      good_num_vols(0),
-      num_vols(0),
-      num_initialized(0),
-      vol_hash_table(0),
-      cp(NULL),
-      num_cachevols(0)
-  {
-  }
+  CacheHostRecord() {}
 };
 
 void build_vol_hash_table(CacheHostRecord *cp);
 
 struct CacheHostResult {
-  CacheHostRecord *record;
+  CacheHostRecord *record = nullptr;
 
-  CacheHostResult() : record(NULL) {}
+  CacheHostResult() {}
 };
 
 class CacheHostMatcher
@@ -81,7 +72,7 @@ public:
   CacheHostMatcher(const char *name, CacheType typ);
   ~CacheHostMatcher();
 
-  void Match(char const *rdata, int rlen, CacheHostResult *result);
+  void Match(const char *rdata, int rlen, CacheHostResult *result);
   void AllocateSpace(int num_entries);
   void NewEntry(matcher_line *line_info);
   void Print();
@@ -107,7 +98,7 @@ private:
   HostLookup *host_lookup;     // Data structure to do the lookups
   CacheHostRecord *data_array; // array of all data items
   int array_len;               // the length of the arrays
-  int num_el;                  // the number of itmems in the tree
+  int num_el;                  // the number of items in the tree
   CacheType type;
 };
 
@@ -120,7 +111,7 @@ public:
   ~CacheHostTable();
   int BuildTable(const char *config_file_path);
   int BuildTableFromString(const char *config_file_path, char *str);
-  void Match(char const *rdata, int rlen, CacheHostResult *result);
+  void Match(const char *rdata, int rlen, CacheHostResult *result);
   void Print();
 
   int
@@ -142,22 +133,22 @@ public:
     REC_RegisterConfigUpdateFunc("proxy.config.cache.hosting_filename", CacheHostTable::config_callback, (void *)p);
   }
 
-  CacheType type;
-  Cache *cache;
-  int m_numEntries;
+  CacheType type   = CACHE_HTTP_TYPE;
+  Cache *cache     = nullptr;
+  int m_numEntries = 0;
   CacheHostRecord gen_host_rec;
 
 private:
-  CacheHostMatcher *hostMatch;
-  const matcher_tags *config_tags;
-  const char *matcher_name; // Used for Debug/Warning/Error messages
+  CacheHostMatcher *hostMatch    = nullptr;
+  const matcher_tags config_tags = {"hostname", "domain", nullptr, nullptr, nullptr, nullptr, false};
+  const char *matcher_name       = "unknown"; // Used for Debug/Warning/Error messages
 };
 
 struct CacheHostTableConfig;
 typedef int (CacheHostTableConfig::*CacheHostTabHandler)(int, void *);
 struct CacheHostTableConfig : public Continuation {
   CacheHostTable **ppt;
-  CacheHostTableConfig(CacheHostTable **appt) : Continuation(NULL), ppt(appt)
+  CacheHostTableConfig(CacheHostTable **appt) : Continuation(nullptr), ppt(appt)
   {
     SET_HANDLER((CacheHostTabHandler)&CacheHostTableConfig::mainEvent);
   }
@@ -180,6 +171,7 @@ struct ConfigVol {
   CacheType scheme;
   off_t size;
   bool in_percent;
+  bool ramcache_enabled;
   int percent;
   CacheVol *cachep;
   LINK(ConfigVol, link);
@@ -188,23 +180,19 @@ struct ConfigVol {
 struct ConfigVolumes {
   int num_volumes;
   int num_http_volumes;
-  int num_stream_volumes;
   Queue<ConfigVol> cp_queue;
   void read_config_file();
   void BuildListFromString(char *config_file_path, char *file_buf);
 
   void
-  clear_all(void)
+  clear_all()
   {
     // remove all the volumes from the queue
     for (int i = 0; i < num_volumes; i++) {
       cp_queue.pop();
     }
     // reset count variables
-    num_volumes        = 0;
-    num_http_volumes   = 0;
-    num_stream_volumes = 0;
+    num_volumes      = 0;
+    num_http_volumes = 0;
   }
 };
-
-#endif
