@@ -21,8 +21,9 @@
   limitations under the License.
  */
 
-#ifndef _P_CACHE_VOL_H__
-#define _P_CACHE_VOL_H__
+#pragma once
+
+#include <atomic>
 
 #define CACHE_BLOCK_SHIFT 9
 #define CACHE_BLOCK_SIZE (1 << CACHE_BLOCK_SHIFT) // 512, smallest sector size
@@ -41,7 +42,7 @@
 #define MAX_VOL_SIZE ((off_t)512 * 1024 * 1024 * 1024 * 1024)
 #define STORE_BLOCKS_PER_CACHE_BLOCK (STORE_BLOCK_SIZE / CACHE_BLOCK_SIZE)
 #define MAX_VOL_BLOCKS (MAX_VOL_SIZE / CACHE_BLOCK_SIZE)
-#define MAX_FRAG_SIZE (AGG_SIZE - sizeofDoc) // true max
+#define MAX_FRAG_SIZE (AGG_SIZE - sizeof(Doc)) // true max
 #define LEAVE_FREE DEFAULT_MAX_BUFFER_SIZE
 #define PIN_SCAN_EVERY 16 // scan every 1/16 of disk
 #define VOL_HASH_TABLE_SIZE 32707
@@ -52,13 +53,13 @@
 #define RECOVERY_SIZE EVACUATION_SIZE                // 8MB
 #define AIO_NOT_IN_PROGRESS 0
 #define AIO_AGG_WRITE_IN_PROGRESS -1
-#define AUTO_SIZE_RAM_CACHE -1                             // 1-1 with directory size
-#define DEFAULT_TARGET_FRAGMENT_SIZE (1048576 - sizeofDoc) // 1MB
+#define AUTO_SIZE_RAM_CACHE -1                               // 1-1 with directory size
+#define DEFAULT_TARGET_FRAGMENT_SIZE (1048576 - sizeof(Doc)) // 1MB
 
 #define dir_offset_evac_bucket(_o) (_o / (EVACUATION_BUCKET_SIZE / CACHE_BLOCK_SIZE))
 #define dir_evac_bucket(_e) dir_offset_evac_bucket(dir_offset(_e))
 #define offset_evac_bucket(_d, _o) \
-  dir_offset_evac_bucket((offset_to_vol_offset(_d, _o)
+  dir_offset_evac_bucket((_d->offset_to_vol_offset(_o)
 
 // Documents
 
@@ -66,8 +67,6 @@
 #define DOC_CORRUPT ((uint32_t)0xDEADBABE)
 #define DOC_NO_CHECKSUM ((uint32_t)0xA0B0C0D0)
 
-#define sizeofDoc (((uint32_t)(uintptr_t) & ((Doc *)0)->checksum) + (uint32_t)sizeof(uint32_t))
-
 struct Cache;
 struct Vol;
 struct CacheDisk;
@@ -77,7 +76,7 @@ struct CacheVol;
 
 struct VolHeaderFooter {
   unsigned int magic;
-  VersionNumber version;
+  ts::VersionNumber version;
   time_t create_time;
   off_t write_pos;
   off_t last_write_pos;
@@ -121,58 +120,58 @@ struct EvacuationBlock {
 };
 
 struct Vol : public Continuation {
-  char *path;
+  char *path = nullptr;
   ats_scoped_str hash_text;
   CryptoHash hash_id;
-  int fd;
+  int fd = -1;
 
-  char *raw_dir;
-  Dir *dir;
-  VolHeaderFooter *header;
-  VolHeaderFooter *footer;
-  int segments;
-  off_t buckets;
-  off_t recover_pos;
-  off_t prev_recover_pos;
-  off_t scan_pos;
-  off_t skip;  // start of headers
-  off_t start; // start of data
-  off_t len;
-  off_t data_blocks;
-  int hit_evacuate_window;
+  char *raw_dir           = nullptr;
+  Dir *dir                = nullptr;
+  VolHeaderFooter *header = nullptr;
+  VolHeaderFooter *footer = nullptr;
+  int segments            = 0;
+  off_t buckets           = 0;
+  off_t recover_pos       = 0;
+  off_t prev_recover_pos  = 0;
+  off_t scan_pos          = 0;
+  off_t skip              = 0; // start of headers
+  off_t start             = 0; // start of data
+  off_t len               = 0;
+  off_t data_blocks       = 0;
+  int hit_evacuate_window = 0;
   AIOCallbackInternal io;
 
   Queue<CacheVC, Continuation::Link_link> agg;
   Queue<CacheVC, Continuation::Link_link> stat_cache_vcs;
   Queue<CacheVC, Continuation::Link_link> sync;
-  char *agg_buffer;
-  int agg_todo_size;
-  int agg_buf_pos;
+  char *agg_buffer  = nullptr;
+  int agg_todo_size = 0;
+  int agg_buf_pos   = 0;
 
-  Event *trigger;
+  Event *trigger = nullptr;
 
   OpenDir open_dir;
-  RamCache *ram_cache;
-  int evacuate_size;
-  DLL<EvacuationBlock> *evacuate;
+  RamCache *ram_cache            = nullptr;
+  int evacuate_size              = 0;
+  DLL<EvacuationBlock> *evacuate = nullptr;
   DLL<EvacuationBlock> lookaside[LOOKASIDE_SIZE];
-  CacheVC *doc_evacuator;
+  CacheVC *doc_evacuator = nullptr;
 
-  VolInitInfo *init_info;
+  VolInitInfo *init_info = nullptr;
 
-  CacheDisk *disk;
-  Cache *cache;
-  CacheVol *cache_vol;
-  uint32_t last_sync_serial;
-  uint32_t last_write_serial;
-  uint32_t sector_size;
-  bool recover_wrapped;
-  bool dir_sync_waiting;
-  bool dir_sync_in_progress;
-  bool writing_end_marker;
+  CacheDisk *disk            = nullptr;
+  Cache *cache               = nullptr;
+  CacheVol *cache_vol        = nullptr;
+  uint32_t last_sync_serial  = 0;
+  uint32_t last_write_serial = 0;
+  uint32_t sector_size       = 0;
+  bool recover_wrapped       = false;
+  bool dir_sync_waiting      = false;
+  bool dir_sync_in_progress  = false;
+  bool writing_end_marker    = false;
 
   CacheKey first_fragment_key;
-  int64_t first_fragment_offset;
+  int64_t first_fragment_offset = 0;
   Ptr<IOBufferData> first_fragment_data;
 
   void cancel_trigger();
@@ -245,31 +244,24 @@ struct Vol : public Continuation {
   int within_hit_evacuate_window(Dir *dir);
   uint32_t round_to_approx_size(uint32_t l);
 
-  Vol()
-    : Continuation(new_ProxyMutex()),
-      path(NULL),
-      fd(-1),
-      dir(0),
-      buckets(0),
-      recover_pos(0),
-      prev_recover_pos(0),
-      scan_pos(0),
-      skip(0),
-      start(0),
-      len(0),
-      data_blocks(0),
-      hit_evacuate_window(0),
-      agg_todo_size(0),
-      agg_buf_pos(0),
-      trigger(0),
-      evacuate_size(0),
-      disk(NULL),
-      last_sync_serial(0),
-      last_write_serial(0),
-      recover_wrapped(false),
-      dir_sync_waiting(0),
-      dir_sync_in_progress(0),
-      writing_end_marker(0)
+  // inline functions
+  int headerlen();         // calculates the total length of the vol header and the freelist
+  int direntries();        // total number of dir entries
+  Dir *dir_segment(int s); // returns the first dir in the segment s
+  size_t dirlen();         // calculates the total length of header, directories and footer
+  int vol_out_of_phase_valid(Dir *e);
+
+  int vol_out_of_phase_agg_valid(Dir *e);
+  int vol_out_of_phase_write_valid(Dir *e);
+  int vol_in_phase_valid(Dir *e);
+  int vol_in_phase_agg_buf_valid(Dir *e);
+
+  off_t vol_offset(Dir *e);
+  off_t offset_to_vol_offset(off_t pos);
+  off_t vol_offset_to_offset(off_t pos);
+  off_t vol_relative_length(off_t start_offset);
+
+  Vol() : Continuation(new_ProxyMutex())
   {
     open_dir.mutex = mutex;
     agg_buffer     = (char *)ats_memalign(ats_pagesize(), AGG_SIZE);
@@ -277,7 +269,7 @@ struct Vol : public Continuation {
     SET_HANDLER(&Vol::aggWrite);
   }
 
-  ~Vol() { ats_memalign_free(agg_buffer); }
+  ~Vol() override { ats_memalign_free(agg_buffer); }
 };
 
 struct AIO_Callback_handler : public Continuation {
@@ -287,27 +279,34 @@ struct AIO_Callback_handler : public Continuation {
 };
 
 struct CacheVol {
-  int vol_number;
-  int scheme;
-  off_t size;
-  int num_vols;
-  Vol **vols;
-  DiskVol **disk_vols;
+  int vol_number        = -1;
+  int scheme            = 0;
+  off_t size            = 0;
+  int num_vols          = 0;
+  bool ramcache_enabled = true;
+  Vol **vols            = nullptr;
+  DiskVol **disk_vols   = nullptr;
   LINK(CacheVol, link);
   // per volume stats
-  RecRawStatBlock *vol_rsb;
+  RecRawStatBlock *vol_rsb = nullptr;
 
-  CacheVol() : vol_number(-1), scheme(0), size(0), num_vols(0), vols(NULL), disk_vols(0), vol_rsb(0) {}
+  CacheVol() {}
 };
 
 // Note : hdr() needs to be 8 byte aligned.
-// If you change this, change sizeofDoc above
 struct Doc {
-  uint32_t magic;        // DOC_MAGIC
-  uint32_t len;          // length of this fragment (including hlen & sizeof(Doc), unrounded)
-  uint64_t total_len;    // total length of document
-  CryptoHash first_key;  ///< first key in object.
-  CryptoHash key;        ///< Key for this doc.
+  uint32_t magic;     // DOC_MAGIC
+  uint32_t len;       // length of this fragment (including hlen & sizeof(Doc), unrounded)
+  uint64_t total_len; // total length of document
+#if TS_ENABLE_FIPS == 1
+  // For FIPS CryptoHash is 256 bits vs. 128, and the 'first_key' must be checked first, so
+  // ensure that the new 'first_key' overlaps the old 'first_key' and that the rest of the data layout
+  // is the same by putting 'key' at the ned.
+  CryptoHash first_key; ///< first key in object.
+#else
+  CryptoHash first_key; ///< first key in object.
+  CryptoHash key;       ///< Key for this doc.
+#endif
   uint32_t hlen;         ///< Length of this header.
   uint32_t doc_type : 8; ///< Doc type - indicates the format of this structure and its content.
   uint32_t v_major : 8;  ///< Major version number.
@@ -317,6 +316,9 @@ struct Doc {
   uint32_t write_serial;
   uint32_t pinned; // pinned until
   uint32_t checksum;
+#if TS_ENABLE_FIPS == 1
+  CryptoHash key; ///< Key for this doc.
+#endif
 
   uint32_t data_len();
   uint32_t prefix_len();
@@ -329,7 +331,7 @@ struct Doc {
 // Global Data
 
 extern Vol **gvol;
-extern volatile int gnvol;
+extern std::atomic<int> gnvol;
 extern ClassAllocator<OpenDirEntry> openDirEntryAllocator;
 extern ClassAllocator<EvacuationBlock> evacuationBlockAllocator;
 extern ClassAllocator<EvacuationKey> evacuationKeyAllocator;
@@ -338,94 +340,94 @@ extern unsigned short *vol_hash_table;
 // inline Functions
 
 TS_INLINE int
-vol_headerlen(Vol *d)
+Vol::headerlen()
 {
-  return ROUND_TO_STORE_BLOCK(sizeof(VolHeaderFooter) + sizeof(uint16_t) * (d->segments - 1));
+  return ROUND_TO_STORE_BLOCK(sizeof(VolHeaderFooter) + sizeof(uint16_t) * (this->segments - 1));
+}
+
+TS_INLINE Dir *
+Vol::dir_segment(int s)
+{
+  return (Dir *)(((char *)this->dir) + (s * this->buckets) * DIR_DEPTH * SIZEOF_DIR);
 }
 
 TS_INLINE size_t
-vol_dirlen(Vol *d)
+Vol::dirlen()
 {
-  return vol_headerlen(d) + ROUND_TO_STORE_BLOCK(((size_t)d->buckets) * DIR_DEPTH * d->segments * SIZEOF_DIR) +
+  return this->headerlen() + ROUND_TO_STORE_BLOCK(((size_t)this->buckets) * DIR_DEPTH * this->segments * SIZEOF_DIR) +
          ROUND_TO_STORE_BLOCK(sizeof(VolHeaderFooter));
 }
 
 TS_INLINE int
-vol_direntries(Vol *d)
+Vol::direntries()
 {
-  return d->buckets * DIR_DEPTH * d->segments;
+  return this->buckets * DIR_DEPTH * this->segments;
 }
 
 TS_INLINE int
-vol_out_of_phase_valid(Vol *d, Dir *e)
+Vol::vol_out_of_phase_valid(Dir *e)
 {
-  return (dir_offset(e) - 1 >= ((d->header->agg_pos - d->start) / CACHE_BLOCK_SIZE));
+  return (dir_offset(e) - 1 >= ((this->header->agg_pos - this->start) / CACHE_BLOCK_SIZE));
 }
 
 TS_INLINE int
-vol_out_of_phase_agg_valid(Vol *d, Dir *e)
+Vol::vol_out_of_phase_agg_valid(Dir *e)
 {
-  return (dir_offset(e) - 1 >= ((d->header->agg_pos - d->start + AGG_SIZE) / CACHE_BLOCK_SIZE));
+  return (dir_offset(e) - 1 >= ((this->header->agg_pos - this->start + AGG_SIZE) / CACHE_BLOCK_SIZE));
 }
 
 TS_INLINE int
-vol_out_of_phase_write_valid(Vol *d, Dir *e)
+Vol::vol_out_of_phase_write_valid(Dir *e)
 {
-  return (dir_offset(e) - 1 >= ((d->header->write_pos - d->start) / CACHE_BLOCK_SIZE));
+  return (dir_offset(e) - 1 >= ((this->header->write_pos - this->start) / CACHE_BLOCK_SIZE));
 }
 
 TS_INLINE int
-vol_in_phase_valid(Vol *d, Dir *e)
+Vol::vol_in_phase_valid(Dir *e)
 {
-  return (dir_offset(e) - 1 < ((d->header->write_pos + d->agg_buf_pos - d->start) / CACHE_BLOCK_SIZE));
+  return (dir_offset(e) - 1 < ((this->header->write_pos + this->agg_buf_pos - this->start) / CACHE_BLOCK_SIZE));
 }
 
 TS_INLINE off_t
-vol_offset(Vol *d, Dir *e)
+Vol::vol_offset(Dir *e)
 {
-  return d->start + (off_t)dir_offset(e) * CACHE_BLOCK_SIZE - CACHE_BLOCK_SIZE;
+  return this->start + (off_t)dir_offset(e) * CACHE_BLOCK_SIZE - CACHE_BLOCK_SIZE;
 }
 
 TS_INLINE off_t
-offset_to_vol_offset(Vol *d, off_t pos)
+Vol::offset_to_vol_offset(off_t pos)
 {
-  return ((pos - d->start + CACHE_BLOCK_SIZE) / CACHE_BLOCK_SIZE);
+  return ((pos - this->start + CACHE_BLOCK_SIZE) / CACHE_BLOCK_SIZE);
 }
 
 TS_INLINE off_t
-vol_offset_to_offset(Vol *d, off_t pos)
+Vol::vol_offset_to_offset(off_t pos)
 {
-  return d->start + pos * CACHE_BLOCK_SIZE - CACHE_BLOCK_SIZE;
-}
-
-TS_INLINE Dir *
-vol_dir_segment(Vol *d, int s)
-{
-  return (Dir *)(((char *)d->dir) + (s * d->buckets) * DIR_DEPTH * SIZEOF_DIR);
+  return this->start + pos * CACHE_BLOCK_SIZE - CACHE_BLOCK_SIZE;
 }
 
 TS_INLINE int
-vol_in_phase_agg_buf_valid(Vol *d, Dir *e)
+Vol::vol_in_phase_agg_buf_valid(Dir *e)
 {
-  return (vol_offset(d, e) >= d->header->write_pos && vol_offset(d, e) < (d->header->write_pos + d->agg_buf_pos));
+  return (this->vol_offset(e) >= this->header->write_pos && this->vol_offset(e) < (this->header->write_pos + this->agg_buf_pos));
 }
 // length of the partition not including the offset of location 0.
 TS_INLINE off_t
-vol_relative_length(Vol *v, off_t start_offset)
+Vol::vol_relative_length(off_t start_offset)
 {
-  return (v->len + v->skip) - start_offset;
+  return (this->len + this->skip) - start_offset;
 }
 
 TS_INLINE uint32_t
 Doc::prefix_len()
 {
-  return sizeofDoc + hlen;
+  return sizeof(Doc) + hlen;
 }
 
 TS_INLINE uint32_t
 Doc::data_len()
 {
-  return len - sizeofDoc - hlen;
+  return len - sizeof(Doc) - hlen;
 }
 
 TS_INLINE int
@@ -437,7 +439,7 @@ Doc::single_fragment()
 TS_INLINE char *
 Doc::hdr()
 {
-  return reinterpret_cast<char *>(this) + sizeofDoc;
+  return reinterpret_cast<char *>(this) + sizeof(Doc);
 }
 
 TS_INLINE char *
@@ -458,7 +460,7 @@ evacuation_block_exists(Dir *dir, Vol *p)
   for (; b; b = b->link.next)
     if (dir_offset(&b->dir) == dir_offset(dir))
       return b;
-  return 0;
+  return nullptr;
 }
 
 TS_INLINE void
@@ -466,7 +468,7 @@ Vol::cancel_trigger()
 {
   if (trigger) {
     trigger->cancel_action();
-    trigger = NULL;
+    trigger = nullptr;
   }
 }
 
@@ -476,8 +478,8 @@ new_EvacuationBlock(EThread *t)
   EvacuationBlock *b      = THREAD_ALLOC(evacuationBlockAllocator, t);
   b->init                 = 0;
   b->readers              = 0;
-  b->earliest_evacuator   = 0;
-  b->evac_frags.link.next = 0;
+  b->earliest_evacuator   = nullptr;
+  b->evac_frags.link.next = nullptr;
   return b;
 }
 
@@ -517,5 +519,3 @@ Vol::round_to_approx_size(uint32_t l)
   uint32_t ll = round_to_approx_dir_size(l);
   return ROUND_TO_SECTOR(this, ll);
 }
-
-#endif /* _P_CACHE_VOL_H__ */
