@@ -26,16 +26,16 @@
 
 #include "P_Cache.h"
 #include "I_Tasks.h"
-#include "ts/fastlz.h"
-#if TS_HAS_LIBZ
+#include "tscore/fastlz.h"
+#ifdef HAVE_ZLIB_H
 #include <zlib.h>
 #endif
-#if TS_HAS_LZMA
+#ifdef HAVE_LZMA_H
 #include <lzma.h>
 #endif
 
 #define REQUIRED_COMPRESSION 0.9 // must get to this size or declared incompressible
-#define REQUIRED_SHRINK 0.8      // must get to this size or keep orignal buffer (with padding)
+#define REQUIRED_SHRINK 0.8      // must get to this size or keep original buffer (with padding)
 #define HISTORY_HYSTERIA 10      // extra temporary history
 #define ENTRY_OVERHEAD 256       // per-entry overhead to consider when computing cache value/size
 #define LZMA_BASE_MEMLIMIT (64 * 1024 * 1024)
@@ -49,11 +49,11 @@
 #define REQUEUE_LIMIT 100
 
 struct RamCacheCLFUSEntry {
-  INK_MD5 key;
+  CryptoHash key;
   uint32_t auxkey1;
   uint32_t auxkey2;
   uint64_t hits;
-  uint32_t size; // memory used including paddding in buffer
+  uint32_t size; // memory used including padding in buffer
   uint32_t len;  // actual data length
   uint32_t compressed_len;
   union {
@@ -71,29 +71,30 @@ struct RamCacheCLFUSEntry {
 };
 
 struct RamCacheCLFUS : public RamCache {
-  int64_t max_bytes;
-  int64_t bytes;
-  int64_t objects;
+  int64_t max_bytes = 0;
+  int64_t bytes     = 0;
+  int64_t objects   = 0;
 
   // returns 1 on found/stored, 0 on not found/stored, if provided auxkey1 and auxkey2 must match
-  int get(INK_MD5 *key, Ptr<IOBufferData> *ret_data, uint32_t auxkey1 = 0, uint32_t auxkey2 = 0);
-  int put(INK_MD5 *key, IOBufferData *data, uint32_t len, bool copy = false, uint32_t auxkey1 = 0, uint32_t auxkey2 = 0);
-  int fixup(const INK_MD5 *key, uint32_t old_auxkey1, uint32_t old_auxkey2, uint32_t new_auxkey1, uint32_t new_auxkey2);
-  int64_t size() const;
+  int get(CryptoHash *key, Ptr<IOBufferData> *ret_data, uint32_t auxkey1 = 0, uint32_t auxkey2 = 0) override;
+  int put(CryptoHash *key, IOBufferData *data, uint32_t len, bool copy = false, uint32_t auxkey1 = 0,
+          uint32_t auxkey2 = 0) override;
+  int fixup(const CryptoHash *key, uint32_t old_auxkey1, uint32_t old_auxkey2, uint32_t new_auxkey1, uint32_t new_auxkey2) override;
+  int64_t size() const override;
 
-  void init(int64_t max_bytes, Vol *vol);
+  void init(int64_t max_bytes, Vol *vol) override;
 
   // private
-  Vol *vol; // for stats
-  double average_value;
-  int64_t history;
-  int ibuckets;
-  int nbuckets;
+  Vol *vol             = nullptr; // for stats
+  double average_value = 0;
+  int64_t history      = 0;
+  int ibuckets         = 0;
+  int nbuckets         = 0;
   DList(RamCacheCLFUSEntry, hash_link) * bucket;
   Que(RamCacheCLFUSEntry, lru_link) lru[2];
-  uint16_t *seen;
-  int ncompressed;
-  RamCacheCLFUSEntry *compressed; // first uncompressed lru[0] entry
+  uint16_t *seen                 = nullptr;
+  int ncompressed                = 0;
+  RamCacheCLFUSEntry *compressed = nullptr; // first uncompressed lru[0] entry
   void compress_entries(EThread *thread, int do_at_most = INT_MAX);
   void resize_hashtable();
   void victimize(RamCacheCLFUSEntry *e);
@@ -101,21 +102,7 @@ struct RamCacheCLFUS : public RamCache {
   RamCacheCLFUSEntry *destroy(RamCacheCLFUSEntry *e);
   void requeue_victims(Que(RamCacheCLFUSEntry, lru_link) & victims);
   void tick(); // move CLOCK on history
-  RamCacheCLFUS()
-    : max_bytes(0),
-      bytes(0),
-      objects(0),
-      vol(0),
-      average_value(0),
-      history(0),
-      ibuckets(0),
-      nbuckets(0),
-      bucket(0),
-      seen(0),
-      ncompressed(0),
-      compressed(0)
-  {
-  }
+  RamCacheCLFUS() : bucket(nullptr) {}
 };
 
 int64_t
@@ -154,18 +141,19 @@ RamCacheCLFUSCompressor::mainEvent(int /* event ATS_UNUSED */, Event *e)
   case CACHE_COMPRESSION_FASTLZ:
     break;
   case CACHE_COMPRESSION_LIBZ:
-#if !TS_HAS_LIBZ
+#ifndef HAVE_ZLIB_H
     Warning("libz not available for RAM cache compression");
 #endif
     break;
   case CACHE_COMPRESSION_LIBLZMA:
-#if !TS_HAS_LZMA
+#ifndef HAVE_LZMA_H
     Warning("lzma not available for RAM cache compression");
 #endif
     break;
   }
-  if (cache_config_ram_cache_compress_percent)
+  if (cache_config_ram_cache_compress_percent) {
     rc->compress_entries(e->ethread);
+  }
   return EVENT_CONT;
 }
 
@@ -180,14 +168,15 @@ RamCacheCLFUS::resize_hashtable()
 {
   int anbuckets = bucket_sizes[ibuckets];
   DDebug("ram_cache", "resize hashtable %d", anbuckets);
-  int64_t s = anbuckets * sizeof(DList(RamCacheCLFUSEntry, hash_link));
-  DList(RamCacheCLFUSEntry, hash_link) *new_bucket = (DList(RamCacheCLFUSEntry, hash_link) *)ats_malloc(s);
-  memset(new_bucket, 0, s);
+  int64_t s                                        = anbuckets * sizeof(DList(RamCacheCLFUSEntry, hash_link));
+  DList(RamCacheCLFUSEntry, hash_link) *new_bucket = static_cast<DList(RamCacheCLFUSEntry, hash_link) *>(ats_malloc(s));
+  memset(static_cast<void *>(new_bucket), 0, s);
   if (bucket) {
     for (int64_t i = 0; i < nbuckets; i++) {
-      RamCacheCLFUSEntry *e = 0;
-      while ((e = bucket[i].pop()))
+      RamCacheCLFUSEntry *e = nullptr;
+      while ((e = bucket[i].pop())) {
         new_bucket[e->key.slice32(3) % anbuckets].push(e);
+      }
     }
     ats_free(bucket);
   }
@@ -196,7 +185,7 @@ RamCacheCLFUS::resize_hashtable()
   ats_free(seen);
   if (cache_config_ram_cache_use_seen_filter) {
     int size = bucket_sizes[ibuckets] * sizeof(uint16_t);
-    seen     = (uint16_t *)ats_malloc(size);
+    seen     = static_cast<uint16_t *>(ats_malloc(size));
     memset(seen, 0, size);
   }
 }
@@ -204,12 +193,13 @@ RamCacheCLFUS::resize_hashtable()
 void
 RamCacheCLFUS::init(int64_t abytes, Vol *avol)
 {
-  ink_assert(avol != 0);
+  ink_assert(avol != nullptr);
   vol       = avol;
   max_bytes = abytes;
   DDebug("ram_cache", "initializing ram_cache %" PRId64 " bytes", abytes);
-  if (!max_bytes)
+  if (!max_bytes) {
     return;
+  }
   resize_hashtable();
   if (cache_config_ram_cache_compress) {
     eventProcessor.schedule_every(new RamCacheCLFUSCompressor(this), HRTIME_SECOND, ET_TASK);
@@ -241,13 +231,14 @@ check_accounting(RamCacheCLFUS *c)
 #endif
 
 int
-RamCacheCLFUS::get(INK_MD5 *key, Ptr<IOBufferData> *ret_data, uint32_t auxkey1, uint32_t auxkey2)
+RamCacheCLFUS::get(CryptoHash *key, Ptr<IOBufferData> *ret_data, uint32_t auxkey1, uint32_t auxkey2)
 {
-  if (!max_bytes)
+  if (!max_bytes) {
     return 0;
+  }
   int64_t i             = key->slice32(3) % nbuckets;
   RamCacheCLFUSEntry *e = bucket[i].head;
-  char *b               = 0;
+  char *b               = nullptr;
   while (e) {
     if (e->key == *key && e->auxkey1 == auxkey1 && e->auxkey2 == auxkey2) {
       move_compressed(e);
@@ -259,33 +250,37 @@ RamCacheCLFUS::get(INK_MD5 *key, Ptr<IOBufferData> *ret_data, uint32_t auxkey1,
         e->hits++;
         uint32_t ram_hit_state = RAM_HIT_COMPRESS_NONE;
         if (e->flag_bits.compressed) {
-          b = (char *)ats_malloc(e->len);
+          b = static_cast<char *>(ats_malloc(e->len));
           switch (e->flag_bits.compressed) {
           default:
             goto Lfailed;
           case CACHE_COMPRESSION_FASTLZ: {
-            int l = (int)e->len;
-            if ((l != (int)fastlz_decompress(e->data->data(), e->compressed_len, b, l)))
+            int l = static_cast<int>(e->len);
+            if ((l != fastlz_decompress(e->data->data(), e->compressed_len, b, l))) {
               goto Lfailed;
+            }
             ram_hit_state = RAM_HIT_COMPRESS_FASTLZ;
             break;
           }
-#if TS_HAS_LIBZ
+#ifdef HAVE_ZLIB_H
           case CACHE_COMPRESSION_LIBZ: {
             uLongf l = e->len;
-            if (Z_OK != uncompress((Bytef *)b, &l, (Bytef *)e->data->data(), e->compressed_len))
+            if (Z_OK !=
+                uncompress(reinterpret_cast<Bytef *>(b), &l, reinterpret_cast<Bytef *>(e->data->data()), e->compressed_len)) {
               goto Lfailed;
+            }
             ram_hit_state = RAM_HIT_COMPRESS_LIBZ;
             break;
           }
 #endif
-#if TS_HAS_LZMA
+#ifdef HAVE_LZMA_H
           case CACHE_COMPRESSION_LIBLZMA: {
-            size_t l = (size_t)e->len, ipos = 0, opos = 0;
+            size_t l = static_cast<size_t>(e->len), ipos = 0, opos = 0;
             uint64_t memlimit = e->len * 2 + LZMA_BASE_MEMLIMIT;
-            if (LZMA_OK != lzma_stream_buffer_decode(&memlimit, 0, NULL, (uint8_t *)e->data->data(), &ipos, e->compressed_len,
-                                                     (uint8_t *)b, &opos, l))
+            if (LZMA_OK != lzma_stream_buffer_decode(&memlimit, 0, nullptr, reinterpret_cast<uint8_t *>(e->data->data()), &ipos,
+                                                     e->compressed_len, reinterpret_cast<uint8_t *>(b), &opos, l)) {
               goto Lfailed;
+            }
             ram_hit_state = RAM_HIT_COMPRESS_LIBLZMA;
             break;
           }
@@ -294,7 +289,7 @@ RamCacheCLFUS::get(INK_MD5 *key, Ptr<IOBufferData> *ret_data, uint32_t auxkey1,
           IOBufferData *data = new_xmalloc_IOBufferData(b, e->len);
           data->_mem_type    = DEFAULT_ALLOC;
           if (!e->flag_bits.copy) { // don't bother if we have to copy anyway
-            int64_t delta = ((int64_t)e->compressed_len) - (int64_t)e->size;
+            int64_t delta = (static_cast<int64_t>(e->compressed_len)) - static_cast<int64_t>(e->size);
             bytes += delta;
             CACHE_SUM_DYN_STAT_THREAD(cache_ram_cache_bytes_stat, delta);
             e->size = e->compressed_len;
@@ -338,20 +333,24 @@ void
 RamCacheCLFUS::tick()
 {
   RamCacheCLFUSEntry *e = lru[1].dequeue();
-  if (!e)
+  if (!e) {
     return;
+  }
   e->hits >>= 1;
   if (e->hits) {
     e->hits = REQUEUE_HITS(e->hits);
     lru[1].enqueue(e);
-  } else
+  } else {
     goto Lfree;
-  if (history <= objects + HISTORY_HYSTERIA)
+  }
+  if (history <= objects + HISTORY_HYSTERIA) {
     return;
+  }
   e = lru[1].dequeue();
 Lfree:
-  if (!e) // e may be NULL after e= lru[1].dequeue()
+  if (!e) { // e may be nullptr after e= lru[1].dequeue()
     return;
+  }
   e->flag_bits.lru = 0;
   history--;
   uint32_t b = e->key.slice32(3) % nbuckets;
@@ -365,7 +364,7 @@ RamCacheCLFUS::victimize(RamCacheCLFUSEntry *e)
 {
   objects--;
   DDebug("ram_cache", "put %X %d %d size %d VICTIMIZED", e->key.slice32(3), e->auxkey1, e->auxkey2, e->size);
-  e->data          = NULL;
+  e->data          = nullptr;
   e->flag_bits.lru = 1;
   lru[1].enqueue(e);
   history++;
@@ -375,9 +374,9 @@ void
 RamCacheCLFUS::move_compressed(RamCacheCLFUSEntry *e)
 {
   if (e == compressed) {
-    if (compressed->lru_link.next)
+    if (compressed->lru_link.next) {
       compressed = compressed->lru_link.next;
-    else {
+    } else {
       ncompressed--;
       compressed = compressed->lru_link.prev;
     }
@@ -394,9 +393,10 @@ RamCacheCLFUS::destroy(RamCacheCLFUSEntry *e)
     objects--;
     bytes -= e->size + ENTRY_OVERHEAD;
     CACHE_SUM_DYN_STAT_THREAD(cache_ram_cache_bytes_stat, -(int64_t)e->size);
-    e->data = NULL;
-  } else
+    e->data = nullptr;
+  } else {
     history--;
+  }
   uint32_t b = e->key.slice32(3) % nbuckets;
   bucket[b].remove(e);
   DDebug("ram_cache", "put %X %d %d DESTROYED", e->key.slice32(3), e->auxkey1, e->auxkey2);
@@ -407,9 +407,10 @@ RamCacheCLFUS::destroy(RamCacheCLFUSEntry *e)
 void
 RamCacheCLFUS::compress_entries(EThread *thread, int do_at_most)
 {
-  if (!cache_config_ram_cache_compress)
+  if (!cache_config_ram_cache_compress) {
     return;
-  ink_assert(vol != 0);
+  }
+  ink_assert(vol != nullptr);
   MUTEX_TAKE_LOCK(vol->mutex, thread);
   if (!compressed) {
     compressed  = lru[0].head;
@@ -417,14 +418,16 @@ RamCacheCLFUS::compress_entries(EThread *thread, int do_at_most)
   }
   float target = (cache_config_ram_cache_compress_percent / 100.0) * objects;
   int n        = 0;
-  char *b = 0, *bb = 0;
+  char *b = nullptr, *bb = nullptr;
   while (compressed && target > ncompressed) {
     RamCacheCLFUSEntry *e = compressed;
-    if (e->flag_bits.incompressible || e->flag_bits.compressed)
+    if (e->flag_bits.incompressible || e->flag_bits.compressed) {
       goto Lcontinue;
+    }
     n++;
-    if (do_at_most < n)
+    if (do_at_most < n) {
       break;
+    }
     {
       e->compressed_len = e->size;
       uint32_t l        = 0;
@@ -433,14 +436,14 @@ RamCacheCLFUS::compress_entries(EThread *thread, int do_at_most)
       default:
         goto Lcontinue;
       case CACHE_COMPRESSION_FASTLZ:
-        l = (uint32_t)((double)e->len * 1.05 + 66);
+        l = static_cast<uint32_t>(static_cast<double>(e->len) * 1.05 + 66);
         break;
-#if TS_HAS_LIBZ
+#ifdef HAVE_ZLIB_H
       case CACHE_COMPRESSION_LIBZ:
-        l = (uint32_t)compressBound(e->len);
+        l = static_cast<uint32_t>(compressBound(e->len));
         break;
 #endif
-#if TS_HAS_LZMA
+#ifdef HAVE_LZMA_H
       case CACHE_COMPRESSION_LIBLZMA:
         l = e->len;
         break;
@@ -449,35 +452,40 @@ RamCacheCLFUS::compress_entries(EThread *thread, int do_at_most)
       // store transient data for lock release
       Ptr<IOBufferData> edata = e->data;
       uint32_t elen           = e->len;
-      INK_MD5 key             = e->key;
+      CryptoHash key          = e->key;
       MUTEX_UNTAKE_LOCK(vol->mutex, thread);
-      b           = (char *)ats_malloc(l);
+      b           = static_cast<char *>(ats_malloc(l));
       bool failed = false;
       switch (ctype) {
       default:
         goto Lfailed;
       case CACHE_COMPRESSION_FASTLZ:
-        if (e->len < 16)
+        if (e->len < 16) {
           goto Lfailed;
-        if ((l = fastlz_compress(edata->data(), elen, b)) <= 0)
+        }
+        if ((l = fastlz_compress(edata->data(), elen, b)) <= 0) {
           failed = true;
+        }
         break;
-#if TS_HAS_LIBZ
+#ifdef HAVE_ZLIB_H
       case CACHE_COMPRESSION_LIBZ: {
         uLongf ll = l;
-        if ((Z_OK != compress((Bytef *)b, &ll, (Bytef *)edata->data(), elen)))
+        if ((Z_OK != compress(reinterpret_cast<Bytef *>(b), &ll, reinterpret_cast<Bytef *>(edata->data()), elen))) {
           failed = true;
-        l        = (int)ll;
+        }
+        l = static_cast<int>(ll);
         break;
       }
 #endif
-#if TS_HAS_LZMA
+#ifdef HAVE_LZMA_H
       case CACHE_COMPRESSION_LIBLZMA: {
         size_t pos = 0, ll = l;
-        if (LZMA_OK != lzma_easy_buffer_encode(LZMA_PRESET_DEFAULT, LZMA_CHECK_NONE, NULL, (uint8_t *)edata->data(), elen,
-                                               (uint8_t *)b, &pos, ll))
+        if (LZMA_OK != lzma_easy_buffer_encode(LZMA_PRESET_DEFAULT, LZMA_CHECK_NONE, nullptr,
+                                               reinterpret_cast<uint8_t *>(edata->data()), elen, reinterpret_cast<uint8_t *>(b),
+                                               &pos, ll)) {
           failed = true;
-        l        = (int)pos;
+        }
+        l = static_cast<int>(pos);
         break;
       }
 #endif
@@ -485,40 +493,45 @@ RamCacheCLFUS::compress_entries(EThread *thread, int do_at_most)
       MUTEX_TAKE_LOCK(vol->mutex, thread);
       // see if the entry is till around
       {
-        if (failed)
+        if (failed) {
           goto Lfailed;
+        }
         uint32_t i             = key.slice32(3) % nbuckets;
         RamCacheCLFUSEntry *ee = bucket[i].head;
         while (ee) {
-          if (ee->key == key && ee->data == edata)
+          if (ee->key == key && ee->data == edata) {
             break;
+          }
           ee = ee->hash_link.next;
         }
         if (!ee || ee != e) {
           e = compressed;
+          ats_free(b);
           goto Lcontinue;
         }
       }
-      if (l > REQUIRED_COMPRESSION * e->len)
+      if (l > REQUIRED_COMPRESSION * e->len) {
         e->flag_bits.incompressible = true;
-      if (l > REQUIRED_SHRINK * e->size)
+      }
+      if (l > REQUIRED_SHRINK * e->size) {
         goto Lfailed;
+      }
       if (l < e->len) {
         e->flag_bits.compressed = cache_config_ram_cache_compress;
-        bb                      = (char *)ats_malloc(l);
+        bb                      = static_cast<char *>(ats_malloc(l));
         memcpy(bb, b, l);
         ats_free(b);
         e->compressed_len = l;
-        int64_t delta     = ((int64_t)l) - (int64_t)e->size;
+        int64_t delta     = (static_cast<int64_t>(l)) - static_cast<int64_t>(e->size);
         bytes += delta;
         CACHE_SUM_DYN_STAT_THREAD(cache_ram_cache_bytes_stat, delta);
         e->size = l;
       } else {
         ats_free(b);
         e->flag_bits.compressed = 0;
-        bb                      = (char *)ats_malloc(e->len);
+        bb                      = static_cast<char *>(ats_malloc(e->len));
         memcpy(bb, e->data->data(), e->len);
-        int64_t delta = ((int64_t)e->len) - (int64_t)e->size;
+        int64_t delta = (static_cast<int64_t>(e->len)) - static_cast<int64_t>(e->size);
         bytes += delta;
         CACHE_SUM_DYN_STAT_THREAD(cache_ram_cache_bytes_stat, delta);
         e->size = e->len;
@@ -535,8 +548,9 @@ RamCacheCLFUS::compress_entries(EThread *thread, int do_at_most)
   Lcontinue:;
     DDebug("ram_cache", "compress %X %d %d %d %d %d %d %d", e->key.slice32(3), e->auxkey1, e->auxkey2, e->flag_bits.incompressible,
            e->flag_bits.compressed, e->len, e->compressed_len, ncompressed);
-    if (!e->lru_link.next)
+    if (!e->lru_link.next) {
       break;
+    }
     compressed = e->lru_link.next;
     ncompressed++;
   }
@@ -546,7 +560,7 @@ RamCacheCLFUS::compress_entries(EThread *thread, int do_at_most)
 
 void RamCacheCLFUS::requeue_victims(Que(RamCacheCLFUSEntry, lru_link) & victims)
 {
-  RamCacheCLFUSEntry *victim = 0;
+  RamCacheCLFUSEntry *victim = nullptr;
   while ((victim = victims.dequeue())) {
     bytes += victim->size + ENTRY_OVERHEAD;
     CACHE_SUM_DYN_STAT_THREAD(cache_ram_cache_bytes_stat, victim->size);
@@ -556,19 +570,20 @@ void RamCacheCLFUS::requeue_victims(Que(RamCacheCLFUSEntry, lru_link) & victims)
 }
 
 int
-RamCacheCLFUS::put(INK_MD5 *key, IOBufferData *data, uint32_t len, bool copy, uint32_t auxkey1, uint32_t auxkey2)
+RamCacheCLFUS::put(CryptoHash *key, IOBufferData *data, uint32_t len, bool copy, uint32_t auxkey1, uint32_t auxkey2)
 {
-  if (!max_bytes)
+  if (!max_bytes) {
     return 0;
+  }
   uint32_t i            = key->slice32(3) % nbuckets;
   RamCacheCLFUSEntry *e = bucket[i].head;
   uint32_t size         = copy ? len : data->block_size();
   double victim_value   = 0;
   while (e) {
     if (e->key == *key) {
-      if (e->auxkey1 == auxkey1 && e->auxkey2 == auxkey2)
+      if (e->auxkey1 == auxkey1 && e->auxkey2 == auxkey2) {
         break;
-      else {
+      } else {
         e = destroy(e); // discard when aux keys conflict
         continue;
       }
@@ -581,14 +596,14 @@ RamCacheCLFUS::put(INK_MD5 *key, IOBufferData *data, uint32_t len, bool copy, ui
       move_compressed(e);
       lru[e->flag_bits.lru].remove(e);
       lru[e->flag_bits.lru].enqueue(e);
-      int64_t delta = ((int64_t)size) - (int64_t)e->size;
+      int64_t delta = (static_cast<int64_t>(size)) - static_cast<int64_t>(e->size);
       bytes += delta;
       CACHE_SUM_DYN_STAT_THREAD(cache_ram_cache_bytes_stat, delta);
       if (!copy) {
         e->size = size;
         e->data = data;
       } else {
-        char *b = (char *)ats_malloc(len);
+        char *b = static_cast<char *>(ats_malloc(len));
         memcpy(b, data->data(), len);
         e->data            = new_xmalloc_IOBufferData(b, len);
         e->data->_mem_type = DEFAULT_ALLOC;
@@ -608,11 +623,13 @@ RamCacheCLFUS::put(INK_MD5 *key, IOBufferData *data, uint32_t len, bool copy, ui
     }
   }
   Que(RamCacheCLFUSEntry, lru_link) victims;
-  RamCacheCLFUSEntry *victim = 0;
+  RamCacheCLFUSEntry *victim = nullptr;
   int requeue_limit          = REQUEUE_LIMIT;
-  if (!lru[1].head) // initial fill
-    if (bytes + size <= max_bytes)
+  if (!lru[1].head) { // initial fill
+    if (bytes + size <= max_bytes) {
       goto Linsert;
+    }
+  }
   if (!e && cache_config_ram_cache_use_seen_filter) {
     uint32_t s  = key->slice32(3) % bucket_sizes[ibuckets];
     uint16_t k  = key->slice32(3) >> 16;
@@ -623,13 +640,15 @@ RamCacheCLFUS::put(INK_MD5 *key, IOBufferData *data, uint32_t len, bool copy, ui
       return 0;
     }
   }
-  while (1) {
+  while (true) {
     victim = lru[0].dequeue();
     if (!victim) {
-      if (bytes + size <= max_bytes)
+      if (bytes + size <= max_bytes) {
         goto Linsert;
-      if (e)
+      }
+      if (e) {
         lru[1].enqueue(e);
+      }
       requeue_victims(victims);
       DDebug("ram_cache", "put %X %d %d NO VICTIM", key->slice32(3), auxkey1, auxkey2);
       return 0;
@@ -642,15 +661,16 @@ RamCacheCLFUS::put(INK_MD5 *key, IOBufferData *data, uint32_t len, bool copy, ui
     bytes -= victim->size + ENTRY_OVERHEAD;
     CACHE_SUM_DYN_STAT_THREAD(cache_ram_cache_bytes_stat, -(int64_t)victim->size);
     victims.enqueue(victim);
-    if (victim == compressed)
-      compressed = 0;
-    else
+    if (victim == compressed) {
+      compressed = nullptr;
+    } else {
       ncompressed--;
+    }
     victim_value += CACHE_VALUE(victim);
     tick();
-    if (!e)
+    if (!e) {
       goto Lhistory;
-    else { // e from history
+    } else { // e from history
       DDebug("ram_cache_compare", "put %f %f", victim_value, CACHE_VALUE(e));
       if (bytes + victim->size + size > max_bytes && victim_value > CACHE_VALUE(e)) {
         requeue_victims(victims);
@@ -659,8 +679,9 @@ RamCacheCLFUS::put(INK_MD5 *key, IOBufferData *data, uint32_t len, bool copy, ui
         return 0;
       }
     }
-    if (bytes + size <= max_bytes)
+    if (bytes + size <= max_bytes) {
       goto Linsert;
+    }
   }
 Linsert:
   while ((victim = victims.dequeue())) {
@@ -669,8 +690,9 @@ Linsert:
       CACHE_SUM_DYN_STAT_THREAD(cache_ram_cache_bytes_stat, victim->size);
       victim->hits = REQUEUE_HITS(victim->hits);
       lru[0].enqueue(victim);
-    } else
+    } else {
       victimize(victim);
+    }
   }
   if (e) {
     history--; // move from history
@@ -688,10 +710,10 @@ Linsert:
   }
   check_accounting(this);
   e->flags = 0;
-  if (!copy)
+  if (!copy) {
     e->data = data;
-  else {
-    char *b = (char *)ats_malloc(len);
+  } else {
+    char *b = static_cast<char *>(ats_malloc(len));
     memcpy(b, data->data(), len);
     e->data            = new_xmalloc_IOBufferData(b, len);
     e->data->_mem_type = DEFAULT_ALLOC;
@@ -725,10 +747,11 @@ Lhistory:
 }
 
 int
-RamCacheCLFUS::fixup(const INK_MD5 *key, uint32_t old_auxkey1, uint32_t old_auxkey2, uint32_t new_auxkey1, uint32_t new_auxkey2)
+RamCacheCLFUS::fixup(const CryptoHash *key, uint32_t old_auxkey1, uint32_t old_auxkey2, uint32_t new_auxkey1, uint32_t new_auxkey2)
 {
-  if (!max_bytes)
+  if (!max_bytes) {
     return 0;
+  }
   uint32_t i            = key->slice32(3) % nbuckets;
   RamCacheCLFUSEntry *e = bucket[i].head;
   while (e) {
