@@ -28,14 +28,15 @@
  *
  ****************************************************************************/
 
-#include "ts/ink_platform.h"
-#include "ts/Tokenizer.h"
+#include "tscore/ink_platform.h"
+#include "tscore/Tokenizer.h"
+#include "tscore/Filenames.h"
 
 #ifdef SPLIT_DNS
 #include <sys/types.h>
 #include "P_SplitDNS.h"
-#include "ts/MatcherUtils.h"
-#include "ts/HostLookup.h"
+#include "tscore/MatcherUtils.h"
+#include "tscore/HostLookup.h"
 
 /* --------------------------------------------------------------
    this file is built using "ParentSelection.cc as a template.
@@ -46,7 +47,7 @@
    -------------------------------------------------------------- */
 static const char modulePrefix[] = "[SplitDNS]";
 
-ConfigUpdateHandler<SplitDNSConfig> *SplitDNSConfig::splitDNSUpdate = NULL;
+ConfigUpdateHandler<SplitDNSConfig> *SplitDNSConfig::splitDNSUpdate = nullptr;
 
 static ClassAllocator<DNSRequestData> DNSReqAllocator("DNSRequestDataAllocator");
 
@@ -54,7 +55,7 @@ static ClassAllocator<DNSRequestData> DNSReqAllocator("DNSRequestDataAllocator")
    used by a lot of protocols. We do not have dest ip in most
    cases.
    -------------------------------------------------------------- */
-const matcher_tags sdns_dest_tags = {"dest_host", "dest_domain", NULL, "url_regex", "url", NULL, true};
+const matcher_tags sdns_dest_tags = {"dest_host", "dest_domain", nullptr, "url_regex", "url", nullptr, true};
 
 /* --------------------------------------------------------------
    config Callback Prototypes
@@ -74,16 +75,12 @@ Ptr<ProxyMutex> SplitDNSConfig::dnsHandler_mutex;
 /* --------------------------------------------------------------
    SplitDNSResult::SplitDNSResult()
    -------------------------------------------------------------- */
-inline SplitDNSResult::SplitDNSResult() : r(DNS_SRVR_UNDEFINED), m_line_number(0), m_rec(0), m_wrap_around(false)
-{
-}
+inline SplitDNSResult::SplitDNSResult() {}
 
 /* --------------------------------------------------------------
    SplitDNS::SplitDNS()
    -------------------------------------------------------------- */
-SplitDNS::SplitDNS() : m_DNSSrvrTable(NULL), m_SplitDNSlEnable(0), m_bEnableFastPath(false), m_pxLeafArray(NULL), m_numEle(0)
-{
-}
+SplitDNS::SplitDNS() {}
 
 SplitDNS::~SplitDNS()
 {
@@ -98,7 +95,7 @@ SplitDNS::~SplitDNS()
 SplitDNS *
 SplitDNSConfig::acquire()
 {
-  return (SplitDNS *)configProcessor.get(SplitDNSConfig::m_id);
+  return static_cast<SplitDNS *>(configProcessor.get(SplitDNSConfig::m_id));
 }
 
 /* --------------------------------------------------------------
@@ -116,8 +113,6 @@ SplitDNSConfig::release(SplitDNS *params)
 void
 SplitDNSConfig::startup()
 {
-  dnsHandler_mutex = new_ProxyMutex();
-
   // startup just check gsplit_dns_enabled
   REC_ReadConfigInt32(gsplit_dns_enabled, "proxy.config.dns.splitDNS.enabled");
   SplitDNSConfig::splitDNSUpdate = new ConfigUpdateHandler<SplitDNSConfig>();
@@ -130,26 +125,29 @@ SplitDNSConfig::startup()
 void
 SplitDNSConfig::reconfigure()
 {
-  if (0 == gsplit_dns_enabled)
+  if (0 == gsplit_dns_enabled) {
     return;
+  }
+
+  Note("%s loading ...", ts::filename::SPLITDNS);
 
   SplitDNS *params = new SplitDNS;
 
   params->m_SplitDNSlEnable = gsplit_dns_enabled;
   params->m_DNSSrvrTable    = new DNS_table("proxy.config.dns.splitdns.filename", modulePrefix, &sdns_dest_tags);
 
-  params->m_numEle = params->m_DNSSrvrTable->getEntryCount();
-  if (0 == params->m_DNSSrvrTable || (0 == params->m_numEle)) {
-    Warning("No NAMEDs provided! Disabling SplitDNS");
+  if (nullptr == params->m_DNSSrvrTable || (0 == params->m_DNSSrvrTable->getEntryCount())) {
+    Warning("Failed to load %s - No NAMEDs provided! Disabling SplitDNS", ts::filename::SPLITDNS);
     gsplit_dns_enabled = 0;
     delete params;
     return;
   }
+  params->m_numEle = params->m_DNSSrvrTable->getEntryCount();
 
-  if (0 != params->m_DNSSrvrTable->getHostMatcher() && 0 == params->m_DNSSrvrTable->getReMatcher() &&
-      0 == params->m_DNSSrvrTable->getIPMatcher() && 4 >= params->m_numEle) {
+  if (nullptr != params->m_DNSSrvrTable->getHostMatcher() && nullptr == params->m_DNSSrvrTable->getReMatcher() &&
+      nullptr == params->m_DNSSrvrTable->getIPMatcher() && 4 >= params->m_numEle) {
     HostLookup *pxHL          = params->m_DNSSrvrTable->getHostMatcher()->getHLookup();
-    params->m_pxLeafArray     = (void *)pxHL->getLArray();
+    params->m_pxLeafArray     = pxHL->get_leaf_array();
     params->m_bEnableFastPath = true;
   }
 
@@ -158,6 +156,8 @@ SplitDNSConfig::reconfigure()
   if (is_debug_tag_set("splitdns_config")) {
     SplitDNSConfig::print();
   }
+
+  Note("%s finished loading", ts::filename::SPLITDNS);
 }
 
 /* --------------------------------------------------------------
@@ -196,7 +196,7 @@ SplitDNS::getDNSRecord(const char *hostname)
   }
 
   Debug("splitdns", "Fail to match a valid splitdns rule, fallback to default dns resolver");
-  return NULL;
+  return nullptr;
 }
 
 /* --------------------------------------------------------------
@@ -215,42 +215,42 @@ SplitDNS::findServer(RequestData *rdata, SplitDNSResult *result)
     return;
   }
 
-  result->m_rec         = NULL;
+  result->m_rec         = nullptr;
   result->m_line_number = 0xffffffff;
-  result->m_wrap_around = false;
 
   /* ---------------------------
      the 'alleged' fast path ...
      --------------------------- */
-  if (m_bEnableFastPath) {
-    SplitDNSRecord *data_ptr = 0;
-    char *pHost              = (char *)rdata->get_host();
-    if (0 == pHost) {
+  if (m_bEnableFastPath && m_pxLeafArray) {
+    SplitDNSRecord *data_ptr = nullptr;
+    char *pHost              = const_cast<char *>(rdata->get_host());
+    if (nullptr == pHost) {
       Warning("SplitDNS: No host to match !");
       return;
     }
 
-    int len        = strlen(pHost);
-    HostLeaf *pxHL = (HostLeaf *)m_pxLeafArray;
-    for (int i = 0; i < m_numEle; i++) {
-      if (0 == pxHL)
-        break;
+    int len = strlen(pHost);
+    int n   = std::min(static_cast<size_t>(m_numEle), m_pxLeafArray->size());
+    for (int i = 0; i < n; i++) {
+      const HostLeaf &pxHL = m_pxLeafArray->at(i);
 
-      if (false == pxHL[i].isNot && pxHL[i].len > len)
+      if (false == pxHL.isNot && static_cast<int>(pxHL.match.size()) > len) {
         continue;
+      }
 
-      int idx      = len - pxHL[i].len;
-      char *pH     = &pHost[idx];
-      char *pMatch = (char *)pxHL[i].match;
-      char cNot    = *pMatch;
+      int idx            = len - pxHL.match.size();
+      char *pH           = &pHost[idx];
+      const char *pMatch = pxHL.match.data();
+      char cNot          = *pMatch;
 
-      if ('!' == cNot)
+      if ('!' == cNot) {
         pMatch++;
+      }
 
-      int res = memcmp(pH, pMatch, pxHL[i].len);
+      int res = memcmp(pH, pMatch, pxHL.match.size());
 
       if ((0 != res && '!' == cNot) || (0 == res && '!' != cNot)) {
-        data_ptr = (SplitDNSRecord *)pxHL[i].opaque_data;
+        data_ptr = static_cast<SplitDNSRecord *>(pxHL.opaque_data);
         data_ptr->UpdateMatch(result, rdata);
         break;
       }
@@ -260,7 +260,7 @@ SplitDNS::findServer(RequestData *rdata, SplitDNSResult *result)
   }
 
   rec = result->m_rec;
-  if (rec == NULL) {
+  if (rec == nullptr) {
     result->r = DNS_SRVR_UNDEFINED;
     return;
   } else {
@@ -293,9 +293,7 @@ SplitDNSRecord::ProcessDNSHosts(char *val)
 {
   Tokenizer pTok(",; \t\r");
   int numTok;
-  const char *current;
-  int port = 0;
-  char *tmp;
+  int port  = 0;
   int totsz = 0, sz = 0;
 
   numTok = pTok.Initialize(val, SHARE_TOKS);
@@ -312,10 +310,10 @@ SplitDNSRecord::ProcessDNSHosts(char *val)
      set of servers specified
      ------------------------------------------------ */
   for (int i = 0; i < numTok; i++) {
-    current = pTok[i];
-    tmp     = (char *)strchr(current, ':');
+    const char *current = pTok[i];
+    char *tmp           = const_cast<char *>(strchr(current, ':'));
     // coverity[secure_coding]
-    if (tmp != NULL && sscanf(tmp + 1, "%d", &port) != 1) {
+    if (tmp != nullptr && sscanf(tmp + 1, "%d", &port) != 1) {
       return "Malformed DNS port";
     }
 
@@ -325,10 +323,12 @@ SplitDNSRecord::ProcessDNSHosts(char *val)
        ---------------------------------------- */
     if (tmp) {
       char *scan = tmp + 1;
-      for (; *scan != '\0' && ParseRules::is_digit(*scan); scan++)
+      for (; *scan != '\0' && ParseRules::is_digit(*scan); scan++) {
         ;
-      for (; *scan != '\0' && ParseRules::is_wslfcr(*scan); scan++)
+      }
+      for (; *scan != '\0' && ParseRules::is_wslfcr(*scan); scan++) {
         ;
+      }
 
       if (*scan != '\0') {
         return "Garbage trailing entry or invalid separator";
@@ -337,7 +337,7 @@ SplitDNSRecord::ProcessDNSHosts(char *val)
       if (tmp - current > (MAXDNAME - 1)) {
         return "DNS server name (ip) is too long";
       } else if (tmp - current == 0) {
-        return "server string is emtpy";
+        return "server string is empty";
       }
       *tmp = 0;
     }
@@ -356,7 +356,7 @@ SplitDNSRecord::ProcessDNSHosts(char *val)
   }
 
   m_dnsSrvr_cnt = numTok;
-  return NULL;
+  return nullptr;
 }
 
 /* --------------------------------------------------------------
@@ -384,7 +384,7 @@ SplitDNSRecord::ProcessDefDomain(char *val)
     m_servers.x_def_domain[len] = '\0';
   }
 
-  return NULL;
+  return nullptr;
 }
 
 /* --------------------------------------------------------------
@@ -395,9 +395,8 @@ SplitDNSRecord::ProcessDomainSrchList(char *val)
 {
   Tokenizer pTok(",; \t\r");
   int numTok;
-  int cnt = 0, sz = 0;
-  char *pSp = 0;
-  const char *current;
+  int sz    = 0;
+  char *pSp = nullptr;
 
   numTok = pTok.Initialize(val, SHARE_TOKS);
 
@@ -408,18 +407,19 @@ SplitDNSRecord::ProcessDomainSrchList(char *val)
   pSp = &m_servers.x_domain_srch_list[0];
 
   for (int i = 0; i < numTok; i++) {
-    current = pTok[i];
-    cnt     = sz += strlen(current);
+    const char *current = pTok[i];
+    int cnt             = sz += strlen(current);
 
-    if (MAXDNAME - 1 < sz)
+    if (MAXDNAME - 1 < sz) {
       break;
+    }
 
     memcpy(pSp, current, cnt);
     pSp += (cnt + 1);
   }
 
   m_domain_srch_list = numTok;
-  return NULL;
+  return nullptr;
 }
 
 /* --------------------------------------------------------------
@@ -428,53 +428,50 @@ SplitDNSRecord::ProcessDomainSrchList(char *val)
    matcher_line* line_info - contains parsed label/value pairs
    of the current split.config line
    -------------------------------------------------------------- */
-config_parse_error
+Result
 SplitDNSRecord::Init(matcher_line *line_info)
 {
-  const char *errPtr = NULL;
-  const char *tmp;
-  char *label;
-  char *val;
+  const char *errPtr = nullptr;
 
   this->line_num = line_info->line_num;
   for (int i = 0; i < MATCHER_MAX_TOKENS; i++) {
-    label = line_info->line[0][i];
-    val   = line_info->line[1][i];
+    char *label = line_info->line[0][i];
+    char *val   = line_info->line[1][i];
 
-    if (label == NULL) {
+    if (label == nullptr) {
       continue;
     }
 
     if (strcasecmp(label, "def_domain") == 0) {
-      if (NULL != (errPtr = ProcessDefDomain(val))) {
-        return config_parse_error("%s %s at line %d", modulePrefix, errPtr, line_num);
+      if (nullptr != (errPtr = ProcessDefDomain(val))) {
+        return Result::failure("%s %s at line %d", modulePrefix, errPtr, line_num);
       }
-      line_info->line[0][i] = NULL;
+      line_info->line[0][i] = nullptr;
       line_info->num_el--;
       continue;
     }
 
     if (strcasecmp(label, "search_list") == 0) {
-      if (NULL != (errPtr = ProcessDomainSrchList(val))) {
-        return config_parse_error("%s %s at line %d", modulePrefix, errPtr, line_num);
+      if (nullptr != (errPtr = ProcessDomainSrchList(val))) {
+        return Result::failure("%s %s at line %d", modulePrefix, errPtr, line_num);
       }
-      line_info->line[0][i] = NULL;
+      line_info->line[0][i] = nullptr;
       line_info->num_el--;
       continue;
     }
 
     if (strcasecmp(label, "named") == 0) {
-      if (NULL != (errPtr = ProcessDNSHosts(val))) {
-        return config_parse_error("%s %s at line %d", modulePrefix, errPtr, line_num);
+      if (nullptr != (errPtr = ProcessDNSHosts(val))) {
+        return Result::failure("%s %s at line %d", modulePrefix, errPtr, line_num);
       }
-      line_info->line[0][i] = NULL;
+      line_info->line[0][i] = nullptr;
       line_info->num_el--;
       continue;
     }
   }
 
   if (!ats_is_ip(&m_servers.x_server_ip[0].sa)) {
-    return config_parse_error("%s No server specified in splitdns.config at line %d", modulePrefix, line_num);
+    return Result::failure("%s No server specified in %s at line %d", modulePrefix, ts::filename::SPLITDNS, line_num);
   }
 
   DNSHandler *dnsH  = new DNSHandler;
@@ -482,10 +479,10 @@ SplitDNSRecord::Init(matcher_line *line_info)
 
   memset(res, 0, sizeof(ts_imp_res_state));
   if ((-1 == ink_res_init(res, m_servers.x_server_ip, m_dnsSrvr_cnt, dns_search, m_servers.x_def_domain,
-                          m_servers.x_domain_srch_list, NULL))) {
+                          m_servers.x_domain_srch_list, nullptr))) {
     char ab[INET6_ADDRPORTSTRLEN];
-    return config_parse_error("Failed to build res record for the servers %s ...",
-                              ats_ip_ntop(&m_servers.x_server_ip[0].sa, ab, sizeof ab));
+    return Result::failure("Failed to build res record for the servers %s ...",
+                           ats_ip_ntop(&m_servers.x_server_ip[0].sa, ab, sizeof ab));
   }
 
   dnsH->m_res = res;
@@ -495,19 +492,19 @@ SplitDNSRecord::Init(matcher_line *line_info)
   m_servers.x_dnsH = dnsH;
 
   SET_CONTINUATION_HANDLER(dnsH, &DNSHandler::startEvent_sdns);
-  (eventProcessor.eventthread[ET_DNS][0])->schedule_imm(dnsH);
+  eventProcessor.thread_group[ET_DNS]._thread[0]->schedule_imm(dnsH);
 
   /* -----------------------------------------------------
      Process any modifiers to the directive, if they exist
      ----------------------------------------------------- */
   if (line_info->num_el > 0) {
-    tmp = ProcessModifiers(line_info);
-    if (tmp != NULL) {
-      return config_parse_error("%s %s at line %d in splitdns.config", modulePrefix, tmp, line_num);
+    const char *tmp = ProcessModifiers(line_info);
+    if (tmp != nullptr) {
+      return Result::failure("%s %s at line %d in %s", modulePrefix, tmp, line_num, ts::filename::SPLITDNS);
     }
   }
 
-  return config_parse_error::ok();
+  return Result::ok();
 }
 
 /* --------------------------------------------------------------
@@ -539,13 +536,14 @@ SplitDNSRecord::Print()
 }
 
 void
-ink_split_dns_init(ModuleVersion v)
+ink_split_dns_init(ts::ModuleVersion v)
 {
   static int init_called = 0;
 
-  ink_release_assert(!checkModuleVersion(v, SPLITDNS_MODULE_VERSION));
-  if (init_called)
+  ink_release_assert(v.check(SPLITDNS_MODULE_INTERNAL_VERSION));
+  if (init_called) {
     return;
+  }
 
   init_called = 1;
 }
