@@ -23,7 +23,7 @@
   @section details Details
 
   Continuations have a handleEvent() method to invoke them. Users
-  can determine the behavior of a Continuation by suppling a
+  can determine the behavior of a Continuation by supplying a
   "ContinuationHandler" (member function name) which is invoked
   when events arrive. This function can be changed with the
   "setHandler" method.
@@ -35,16 +35,20 @@
 
 #pragma once
 
-#include "ts/ink_platform.h"
-#include "ts/List.h"
+#include "tscore/ink_platform.h"
+#include "tscore/List.h"
 #include "I_Lock.h"
-#include "ts/ContFlags.h"
+#include "tscore/ContFlags.h"
 
 class Continuation;
 class ContinuationQueue;
 class Processor;
 class ProxyMutex;
 class EThread;
+class Event;
+
+extern EThread *this_ethread();
+extern EThread *this_event_thread();
 
 //////////////////////////////////////////////////////////////////////////////
 //
@@ -97,15 +101,15 @@ public:
     The current continuation handler function.
 
     The current handler should not be set directly. In order to
-    change it, first aquire the Continuation's lock and then use
+    change it, first acquire the Continuation's lock and then use
     the SET_HANDLER macro which takes care of the type casting
     issues.
 
   */
-  ContinuationHandler handler;
+  ContinuationHandler handler = nullptr;
 
 #ifdef DEBUG
-  const char *handler_name;
+  const char *handler_name = nullptr;
 #endif
 
   /**
@@ -115,9 +119,17 @@ public:
     lock is initialized in the constructor and should not be set
     directly.
 
+    TODO:  make this private.
+
   */
   Ptr<ProxyMutex> mutex;
 
+  ProxyMutex *
+  getMutex() const
+  {
+    return mutex.get();
+  }
+
   /**
     Link to other continuations.
 
@@ -133,34 +145,61 @@ public:
   */
   ContFlags control_flags;
 
+  EThread *thread_affinity = nullptr;
+
+  bool
+  setThreadAffinity(EThread *ethread)
+  {
+    if (ethread != nullptr) {
+      thread_affinity = ethread;
+      return true;
+    }
+    return false;
+  }
+
+  EThread *
+  getThreadAffinity()
+  {
+    return thread_affinity;
+  }
+
+  void
+  clearThreadAffinity()
+  {
+    thread_affinity = nullptr;
+  }
+
   /**
     Receives the event code and data for an Event.
 
     This function receives the event code and data for an event and
     forwards them to the current continuation handler. The processor
     calling back the continuation is responsible for acquiring its
-    lock.
+    lock.  If the lock is present and not held, this method will assert.
 
     @param event Event code to be passed at callback (Processor specific).
     @param data General purpose data related to the event code (Processor specific).
     @return State machine and processor specific return code.
 
   */
-  int
-  handleEvent(int event = CONTINUATION_EVENT_NONE, void *data = 0)
+  TS_INLINE int
+  handleEvent(int event = CONTINUATION_EVENT_NONE, void *data = nullptr)
   {
+    // If there is a lock, we must be holding it on entry
+    ink_release_assert(!mutex || mutex->thread_holding == this_ethread());
     return (this->*handler)(event, data);
   }
 
+protected:
   /**
-    Contructor of the Continuation object. It should not be used
+    Constructor of the Continuation object. It should not be used
     directly. Instead create an object of a derived type.
 
     @param amutex Lock to be set for this Continuation.
 
   */
-  Continuation(ProxyMutex *amutex = nullptr);
-  Continuation(Ptr<ProxyMutex> &amutex);
+  explicit Continuation(ProxyMutex *amutex = nullptr);
+  explicit Continuation(Ptr<ProxyMutex> &amutex);
 };
 
 /**
@@ -191,23 +230,13 @@ public:
 #define SET_CONTINUATION_HANDLER(_c, _h) (_c->handler = ((ContinuationHandler)_h))
 #endif
 
-inline Continuation::Continuation(Ptr<ProxyMutex> &amutex)
-  : handler(nullptr),
-#ifdef DEBUG
-    handler_name(nullptr),
-#endif
-    mutex(amutex)
+inline Continuation::Continuation(Ptr<ProxyMutex> &amutex) : mutex(amutex)
 {
   // Pick up the control flags from the creating thread
   this->control_flags.set_flags(get_cont_flags().get_flags());
 }
 
-inline Continuation::Continuation(ProxyMutex *amutex)
-  : handler(nullptr),
-#ifdef DEBUG
-    handler_name(nullptr),
-#endif
-    mutex(amutex)
+inline Continuation::Continuation(ProxyMutex *amutex) : mutex(amutex)
 {
   // Pick up the control flags from the creating thread
   this->control_flags.set_flags(get_cont_flags().get_flags());
