@@ -37,7 +37,7 @@
 #include "tscore/ink_platform.h"
 #include "I_Event.h"
 struct ProtectedQueue {
-  void enqueue(Event *e, bool fast_signal = false);
+  void enqueue(Event *e);
   void signal();
   int try_signal();             // Use non blocking lock and if acquired, signal
   void enqueue_local(Event *e); // Safe when called from the same thread
@@ -54,5 +54,3 @@ struct ProtectedQueue {
 
   ProtectedQueue();
 };
-
-void flush_signals(EThread *t);
