@@ -33,7 +33,7 @@
 #include "I_EThread.h"
 #include "I_EventProcessor.h"
 
-const int DELAY_FOR_RETRY = HRTIME_MSECONDS(10);
+const ink_hrtime DELAY_FOR_RETRY = HRTIME_MSECONDS(10);
 
 TS_INLINE Event *
 EThread::schedule_imm(Continuation *cont, int callback_event, void *cookie)
@@ -44,15 +44,6 @@ EThread::schedule_imm(Continuation *cont, int callback_event, void *cookie)
   return schedule(e->init(cont, 0, 0));
 }
 
-TS_INLINE Event *
-EThread::schedule_imm_signal(Continuation *cont, int callback_event, void *cookie)
-{
-  Event *e          = ::eventAllocator.alloc();
-  e->callback_event = callback_event;
-  e->cookie         = cookie;
-  return schedule(e->init(cont, 0, 0), true);
-}
-
 TS_INLINE Event *
 EThread::schedule_at(Continuation *cont, ink_hrtime t, int callback_event, void *cookie)
 {
@@ -77,20 +68,36 @@ EThread::schedule_every(Continuation *cont, ink_hrtime t, int callback_event, vo
   Event *e          = ::eventAllocator.alloc();
   e->callback_event = callback_event;
   e->cookie         = cookie;
-  return schedule(e->init(cont, get_hrtime() + t, t));
+  if (t < 0) {
+    return schedule(e->init(cont, t, t));
+  } else {
+    return schedule(e->init(cont, get_hrtime() + t, t));
+  }
 }
 
 TS_INLINE Event *
-EThread::schedule(Event *e, bool fast_signal)
+EThread::schedule(Event *e)
 {
   e->ethread = this;
   ink_assert(tt == REGULAR);
-  if (e->continuation->mutex)
+  if (e->continuation->mutex) {
     e->mutex = e->continuation->mutex;
-  else
+  } else {
     e->mutex = e->continuation->mutex = e->ethread->mutex;
+  }
   ink_assert(e->mutex.get());
-  EventQueueExternal.enqueue(e, fast_signal);
+
+  // Make sure client IP debugging works consistently
+  // The continuation that gets scheduled later is not always the
+  // client VC, it can be HttpCacheSM etc. so save the flags
+  e->continuation->control_flags.set_flags(get_cont_flags().get_flags());
+
+  if (e->ethread == this_ethread()) {
+    EventQueueExternal.enqueue_local(e);
+  } else {
+    EventQueueExternal.enqueue(e);
+  }
+
   return e;
 }
 
@@ -127,7 +134,11 @@ EThread::schedule_every_local(Continuation *cont, ink_hrtime t, int callback_eve
   Event *e          = EVENT_ALLOC(eventAllocator, this);
   e->callback_event = callback_event;
   e->cookie         = cookie;
-  return schedule_local(e->init(cont, get_hrtime() + t, t));
+  if (t < 0) {
+    return schedule_local(e->init(cont, t, t));
+  } else {
+    return schedule_local(e->init(cont, get_hrtime() + t, t));
+  }
 }
 
 TS_INLINE Event *
@@ -144,6 +155,11 @@ EThread::schedule_local(Event *e)
     ink_assert(e->ethread == this);
   }
   e->globally_allocated = false;
+
+  // Make sure client IP debugging works consistently
+  // The continuation that gets scheduled later is not always the
+  // client VC, it can be HttpCacheSM etc. so save the flags
+  e->continuation->control_flags.set_flags(get_cont_flags().get_flags());
   EventQueueExternal.enqueue_local(e);
   return e;
 }
@@ -152,8 +168,9 @@ TS_INLINE Event *
 EThread::schedule_spawn(Continuation *c, int ev, void *cookie)
 {
   ink_assert(this != this_ethread()); // really broken to call this from the same thread.
-  if (start_event)
+  if (start_event) {
     free_event(start_event);
+  }
   start_event          = EVENT_ALLOC(eventAllocator, this);
   start_event->ethread = this;
   start_event->mutex   = this->mutex;
@@ -166,7 +183,18 @@ EThread::schedule_spawn(Continuation *c, int ev, void *cookie)
 TS_INLINE EThread *
 this_ethread()
 {
-  return (EThread *)this_thread();
+  return static_cast<EThread *>(this_thread());
+}
+
+TS_INLINE EThread *
+this_event_thread()
+{
+  EThread *ethread = this_ethread();
+  if (ethread != nullptr && ethread->tt == REGULAR) {
+    return ethread;
+  } else {
+    return nullptr;
+  }
 }
 
 TS_INLINE void
