@@ -21,36 +21,28 @@
   limitations under the License.
  */
 
-#ifndef _P_UnixEventProcessor_h_
-#define _P_UnixEventProcessor_h_
+#pragma once
 
-#include "ts/ink_align.h"
+#include <tscore/TSSystemState.h>
+
+#include "tscore/ink_align.h"
 #include "I_EventProcessor.h"
 
 const int LOAD_BALANCE_INTERVAL = 1;
 
-
-TS_INLINE
-EventProcessor::EventProcessor() : n_ethreads(0), n_thread_groups(0), n_dthreads(0), thread_data_used(0)
-{
-  memset(all_ethreads, 0, sizeof(all_ethreads));
-  memset(all_dthreads, 0, sizeof(all_dthreads));
-  memset(n_threads_for_type, 0, sizeof(n_threads_for_type));
-  memset(next_thread_for_type, 0, sizeof(next_thread_for_type));
-}
-
 TS_INLINE off_t
 EventProcessor::allocate(int size)
 {
   static off_t start = INK_ALIGN(offsetof(EThread, thread_private), 16);
-  static off_t loss = start - offsetof(EThread, thread_private);
-  size = INK_ALIGN(size, 16); // 16 byte alignment
+  static off_t loss  = start - offsetof(EThread, thread_private);
+  size               = INK_ALIGN(size, 16); // 16 byte alignment
 
   int old;
   do {
     old = thread_data_used;
-    if (old + loss + size > PER_THREAD_DATA)
+    if (old + loss + size > PER_THREAD_DATA) {
       return -1;
+    }
   } while (!ink_atomic_cas(&thread_data_used, old, old + size));
 
   return (off_t)(old + start);
@@ -60,43 +52,80 @@ TS_INLINE EThread *
 EventProcessor::assign_thread(EventType etype)
 {
   int next;
+  ThreadGroupDescriptor *tg = &thread_group[etype];
 
   ink_assert(etype < MAX_EVENT_TYPES);
-  if (n_threads_for_type[etype] > 1)
-    next = next_thread_for_type[etype]++ % n_threads_for_type[etype];
-  else
+  if (tg->_count > 1) {
+    next = ++tg->_next_round_robin % tg->_count;
+  } else {
     next = 0;
-  return (eventthread[etype][next]);
+  }
+  return tg->_thread[next];
+}
+
+// If thread_holding is the correct type, return it.
+//
+// Otherwise check if there is already an affinity associated with the continuation,
+// return it if the type is the same, return the next available thread of "etype" if
+// the type is different.
+//
+// Only assign new affinity when there is currently none.
+TS_INLINE EThread *
+EventProcessor::assign_affinity_by_type(Continuation *cont, EventType etype)
+{
+  EThread *ethread = cont->mutex->thread_holding;
+  if (!ethread->is_event_type(etype)) {
+    ethread = cont->getThreadAffinity();
+    if (ethread == nullptr || !ethread->is_event_type(etype)) {
+      ethread = assign_thread(etype);
+    }
+  }
+
+  if (cont->getThreadAffinity() == nullptr) {
+    cont->setThreadAffinity(ethread);
+  }
+
+  return ethread;
 }
 
 TS_INLINE Event *
-EventProcessor::schedule(Event *e, EventType etype, bool fast_signal)
+EventProcessor::schedule(Event *e, EventType etype)
 {
   ink_assert(etype < MAX_EVENT_TYPES);
-  e->ethread = assign_thread(etype);
-  if (e->continuation->mutex)
+
+  if (TSSystemState::is_event_system_shut_down()) {
+    return nullptr;
+  }
+
+  EThread *affinity_thread = e->continuation->getThreadAffinity();
+  EThread *curr_thread     = this_ethread();
+  if (affinity_thread != nullptr && affinity_thread->is_event_type(etype)) {
+    e->ethread = affinity_thread;
+  } else {
+    // Is the current thread eligible?
+    if (curr_thread != nullptr && curr_thread->is_event_type(etype)) {
+      e->ethread = curr_thread;
+    } else {
+      e->ethread = assign_thread(etype);
+    }
+    if (affinity_thread == nullptr) {
+      e->continuation->setThreadAffinity(e->ethread);
+    }
+  }
+
+  if (e->continuation->mutex) {
     e->mutex = e->continuation->mutex;
-  else
-    e->mutex = e->continuation->mutex = e->ethread->mutex;
-  e->ethread->EventQueueExternal.enqueue(e, fast_signal);
+  }
+
+  if (curr_thread != nullptr && e->ethread == curr_thread) {
+    e->ethread->EventQueueExternal.enqueue_local(e);
+  } else {
+    e->ethread->EventQueueExternal.enqueue(e);
+  }
+
   return e;
 }
 
-
-TS_INLINE Event *
-EventProcessor::schedule_imm_signal(Continuation *cont, EventType et, int callback_event, void *cookie)
-{
-  Event *e = eventAllocator.alloc();
-
-  ink_assert(et < MAX_EVENT_TYPES);
-#ifdef ENABLE_TIME_TRACE
-  e->start_time = ink_get_hrtime();
-#endif
-  e->callback_event = callback_event;
-  e->cookie = cookie;
-  return schedule(e->init(cont, 0, 0), et, true);
-}
-
 TS_INLINE Event *
 EventProcessor::schedule_imm(Continuation *cont, EventType et, int callback_event, void *cookie)
 {
@@ -104,10 +133,10 @@ EventProcessor::schedule_imm(Continuation *cont, EventType et, int callback_even
 
   ink_assert(et < MAX_EVENT_TYPES);
 #ifdef ENABLE_TIME_TRACE
-  e->start_time = ink_get_hrtime();
+  e->start_time = Thread::get_hrtime();
 #endif
   e->callback_event = callback_event;
-  e->cookie = cookie;
+  e->cookie         = cookie;
   return schedule(e->init(cont, 0, 0), et);
 }
 
@@ -119,7 +148,7 @@ EventProcessor::schedule_at(Continuation *cont, ink_hrtime t, EventType et, int
   ink_assert(t > 0);
   ink_assert(et < MAX_EVENT_TYPES);
   e->callback_event = callback_event;
-  e->cookie = cookie;
+  e->cookie         = cookie;
   return schedule(e->init(cont, t, 0), et);
 }
 
@@ -130,7 +159,7 @@ EventProcessor::schedule_in(Continuation *cont, ink_hrtime t, EventType et, int
 
   ink_assert(et < MAX_EVENT_TYPES);
   e->callback_event = callback_event;
-  e->cookie = cookie;
+  e->cookie         = cookie;
   return schedule(e->init(cont, Thread::get_hrtime() + t, 0), et);
 }
 
@@ -142,12 +171,10 @@ EventProcessor::schedule_every(Continuation *cont, ink_hrtime t, EventType et, i
   ink_assert(t != 0);
   ink_assert(et < MAX_EVENT_TYPES);
   e->callback_event = callback_event;
-  e->cookie = cookie;
-  if (t < 0)
+  e->cookie         = cookie;
+  if (t < 0) {
     return schedule(e->init(cont, t, t), et);
-  else
+  } else {
     return schedule(e->init(cont, Thread::get_hrtime() + t, t), et);
+  }
 }
-
-
-#endif
