@@ -25,9 +25,12 @@
 
   SocketManager.cc
  ****************************************************************************/
-#include "ts/ink_platform.h"
+#include "tscore/ink_platform.h"
 #include "P_EventSystem.h"
 
+#include "tscore/TextBuffer.h"
+#include "tscore/TestBox.h"
+
 SocketManager socketManager;
 
 #if !HAVE_ACCEPT4
@@ -63,10 +66,8 @@ accept4(int sockfd, struct sockaddr *addr, socklen_t *addrlen, int flags)
 int
 SocketManager::accept4(int s, struct sockaddr *addr, socklen_t *addrlen, int flags)
 {
-  int fd;
-
   do {
-    fd = ::accept4(s, addr, addrlen, flags);
+    int fd = ::accept4(s, addr, addrlen, flags);
     if (likely(fd >= 0)) {
       return fd;
     }
@@ -75,9 +76,7 @@ SocketManager::accept4(int s, struct sockaddr *addr, socklen_t *addrlen, int fla
   return -errno;
 }
 
-SocketManager::SocketManager() : pagesize(ats_pagesize())
-{
-}
+SocketManager::SocketManager() : pagesize(ats_pagesize()) {}
 
 SocketManager::~SocketManager()
 {
@@ -96,15 +95,50 @@ SocketManager::close(int s)
 {
   int res;
 
-  if (s == 0)
+  if (s == 0) {
     return -EACCES;
-  else if (s < 0)
+  } else if (s < 0) {
     return -EINVAL;
+  }
 
   do {
     res = ::close(s);
-    if (res == -1)
+    if (res == -1) {
       res = -errno;
+    }
   } while (res == -EINTR);
   return res;
 }
+
+bool
+SocketManager::fastopen_supported()
+{
+  static const unsigned TFO_CLIENT_ENABLE = 1;
+
+  ats_scoped_fd fd(::open("/proc/sys/net/ipv4/tcp_fastopen", O_RDONLY));
+  int value = 0;
+
+  if (fd) {
+    TextBuffer buffer(16);
+
+    buffer.slurp(fd.get());
+    value = atoi(buffer.bufPtr());
+  }
+
+  return value & TFO_CLIENT_ENABLE;
+}
+
+REGRESSION_TEST(socket_fastopen)(RegressionTest *test, int level, int *pstatus)
+{
+  TestBox box(test, pstatus);
+
+  box = REGRESSION_TEST_PASSED;
+
+  if (SocketManager::fastopen_supported()) {
+    box.check(MSG_FASTOPEN != 0, "TCP Fast Open is supported, MSG_FASTOPEN must not be 0");
+  }
+
+  if (::access("/proc/sys/net/ipv4/tcp_fastopen", F_OK) == 0) {
+    box.check(MSG_FASTOPEN != 0, "TCP Fast Open is available, MSG_FASTOPEN must not be 0");
+  }
+}
