@@ -23,11 +23,11 @@
 
 #pragma once
 
-#include "ts/HashFNV.h"
-#include "ts/ink_time.h"
-#include "ts/INK_MD5.h"
-#include "ts/ink_align.h"
-#include "ts/ink_resolver.h"
+#include "tscore/HashFNV.h"
+#include "tscore/ink_time.h"
+#include "tscore/CryptoHash.h"
+#include "tscore/ink_align.h"
+#include "tscore/ink_resolver.h"
 #include "I_EventSystem.h"
 #include "SRV.h"
 #include "P_RefCountCache.h"
@@ -41,9 +41,6 @@
 #define EVENT_SRV_IP_REMOVED (SRV_EVENT_EVENTS_START + 1)
 #define EVENT_SRV_GET_RESPONSE (SRV_EVENT_EVENTS_START + 2)
 
-// TODO: make configurable
-#define HOST_DB_MAX_ROUND_ROBIN_INFO 16
-
 //
 // Data
 //
@@ -54,7 +51,7 @@ struct HostDBContinuation;
 // IP address.
 //
 // Since host information is relatively small, we can afford to have
-// a reasonable size memory cache, and use a (relatively) sparce
+// a reasonable size memory cache, and use a (relatively) sparse
 // disk representation to decrease # of seeks.
 //
 extern int hostdb_enable;
@@ -63,6 +60,9 @@ extern unsigned int hostdb_ip_stale_interval;
 extern unsigned int hostdb_ip_timeout_interval;
 extern unsigned int hostdb_ip_fail_timeout_interval;
 extern unsigned int hostdb_serve_stale_but_revalidate;
+extern unsigned int hostdb_round_robin_max_count;
+
+extern int hostdb_max_iobuf_index;
 
 static inline unsigned int
 makeHostHash(const char *string)
@@ -86,7 +86,7 @@ makeHostHash(const char *string)
 //
 // This structure contains the host information required by
 // the application.  Except for the initial fields it
-// is treated as opacque by the database.
+// is treated as opaque by the database.
 //
 
 union HostDBApplicationInfo {
@@ -148,7 +148,7 @@ struct HostDBInfo : public RefCountObj {
   alloc(int size = 0)
   {
     size += sizeof(HostDBInfo);
-    int iobuffer_index = iobuffer_size_to_index(size);
+    int iobuffer_index = iobuffer_size_to_index(size, hostdb_max_iobuf_index);
     ink_release_assert(iobuffer_index >= 0);
     void *ptr = ioBufAllocator[iobuffer_index].alloc_void();
     memset(ptr, 0, size);
@@ -158,17 +158,17 @@ struct HostDBInfo : public RefCountObj {
   }
 
   void
-  free()
+  free() override
   {
     Debug("hostdb", "freeing %d bytes at [%p]", (1 << (7 + iobuffer_index)), this);
     ioBufAllocator[iobuffer_index].free_void((void *)(this));
   }
 
   // return a version number-- so we can manage compatibility with the marshal/unmarshal
-  static VersionNumber
+  static ts::VersionNumber
   version()
   {
-    return VersionNumber(1, 0);
+    return ts::VersionNumber(1, 0);
   }
 
   static HostDBInfo *
@@ -312,12 +312,13 @@ struct HostDBInfo : public RefCountObj {
   void
   set_failed()
   {
-    if (is_srv)
+    if (is_srv) {
       data.srv.srv_offset = 0;
-    else if (reverse_dns)
+    } else if (reverse_dns) {
       data.hostname_offset = 0;
-    else
+    } else {
       ats_ip_invalidate(ip());
+    }
   }
 
   int iobuffer_index;
@@ -350,13 +351,13 @@ struct HostDBInfo : public RefCountObj {
 
 struct HostDBRoundRobin {
   /** Total number (to compute space used). */
-  short rrcount;
+  short rrcount = 0;
 
   /** Number which have not failed a connect. */
-  short good;
+  short good = 0;
 
-  unsigned short current;
-  ink_time_t timed_rr_ctime;
+  unsigned short current    = 0;
+  ink_time_t timed_rr_ctime = 0;
 
   // This is the equivalent of a variable length array, we can't use a VLA because
   // HostDBInfo is a non-POD type-- so this is the best we can do.
@@ -390,19 +391,19 @@ struct HostDBRoundRobin {
   HostDBInfo *select_next(sockaddr const *addr);
   HostDBInfo *select_best_http(sockaddr const *client_ip, ink_time_t now, int32_t fail_window);
   HostDBInfo *select_best_srv(char *target, InkRand *rand, ink_time_t now, int32_t fail_window);
-  HostDBRoundRobin() : rrcount(0), good(0), current(0), timed_rr_ctime(0) {}
+  HostDBRoundRobin() {}
 };
 
 struct HostDBCache;
+struct HostDBHash;
 
-// Prototype for inline completion functionf or
+// Prototype for inline completion function or
 //  getbyname_imm()
-typedef void (Continuation::*process_hostdb_info_pfn)(HostDBInfo *r);
-typedef void (Continuation::*process_srv_info_pfn)(HostDBInfo *r);
+typedef void (Continuation::*cb_process_result_pfn)(HostDBInfo *r);
 
 Action *iterate(Continuation *cont);
 
-/** The Host Databse access interface. */
+/** The Host Database access interface. */
 struct HostDBProcessor : public Processor {
   friend struct HostDBSync;
   // Public Interface
@@ -425,13 +426,13 @@ struct HostDBProcessor : public Processor {
 
   /// Optional parameters for getby...
   struct Options {
-    typedef Options self;        ///< Self reference type.
-    int port;                    ///< Target service port (default 0 -> don't care)
-    int flags;                   ///< Processing flags (default HOSTDB_DO_NOT_FORCE_DNS)
-    int timeout;                 ///< Timeout value (default 0 -> default timeout)
-    HostResStyle host_res_style; ///< How to query host (default HOST_RES_IPV4)
+    typedef Options self;                                  ///< Self reference type.
+    int port                    = 0;                       ///< Target service port (default 0 -> don't care)
+    int flags                   = HOSTDB_DO_NOT_FORCE_DNS; ///< Processing flags (default HOSTDB_DO_NOT_FORCE_DNS)
+    int timeout                 = 0;                       ///< Timeout value (default 0 -> default timeout)
+    HostResStyle host_res_style = HOST_RES_IPV4;           ///< How to query host (default HOST_RES_IPV4)
 
-    Options() : port(0), flags(HOSTDB_DO_NOT_FORCE_DNS), timeout(0), host_res_style(HOST_RES_IPV4) {}
+    Options() {}
     /// Set the flags.
     self &
     setFlags(int f)
@@ -449,20 +450,16 @@ struct HostDBProcessor : public Processor {
 
   Action *getbynameport_re(Continuation *cont, const char *hostname, int len, Options const &opt = DEFAULT_OPTIONS);
 
-  Action *getSRVbyname_imm(Continuation *cont, process_srv_info_pfn process_srv_info, const char *hostname, int len,
+  Action *getSRVbyname_imm(Continuation *cont, cb_process_result_pfn process_srv_info, const char *hostname, int len,
                            Options const &opt = DEFAULT_OPTIONS);
 
-  Action *getbyname_imm(Continuation *cont, process_hostdb_info_pfn process_hostdb_info, const char *hostname, int len,
+  Action *getbyname_imm(Continuation *cont, cb_process_result_pfn process_hostdb_info, const char *hostname, int len,
                         Options const &opt = DEFAULT_OPTIONS);
 
   Action *iterate(Continuation *cont);
 
   /** Lookup Hostinfo by addr */
-  Action *
-  getbyaddr_re(Continuation *cont, sockaddr const *aip)
-  {
-    return getby(cont, nullptr, 0, aip, false, HOST_RES_NONE, 0);
-  }
+  Action *getbyaddr_re(Continuation *cont, sockaddr const *aip);
 
   /** Set the application information (fire-and-forget). */
   void
@@ -476,7 +473,7 @@ struct HostDBProcessor : public Processor {
   void
   setbyaddr_appinfo(sockaddr const *addr, HostDBApplicationInfo *app)
   {
-    this->setby(0, 0, addr, app);
+    this->setby(nullptr, 0, addr, app);
   }
 
   void
@@ -484,7 +481,7 @@ struct HostDBProcessor : public Processor {
   {
     sockaddr_in addr;
     ats_ip4_set(&addr, ip);
-    this->setby(0, 0, ats_ip_sa_cast(&addr), app);
+    this->setby(nullptr, 0, ats_ip_sa_cast(&addr), app);
   }
 
   /** Configuration. */
@@ -495,14 +492,13 @@ struct HostDBProcessor : public Processor {
   /* hostdb does not use any dedicated event threads
    * currently. Dont pass any value to start
    */
-  int start(int no_of_additional_event_threads = 0, size_t stacksize = DEFAULT_STACKSIZE);
+  int start(int no_of_additional_event_threads = 0, size_t stacksize = DEFAULT_STACKSIZE) override;
 
   // Private
   HostDBCache *cache();
 
 private:
-  Action *getby(Continuation *cont, const char *hostname, int len, sockaddr const *ip, bool aforce_dns, HostResStyle host_res_style,
-                int timeout);
+  Action *getby(Continuation *cont, cb_process_result_pfn cb_process_result, HostDBHash &hash, Options const &opt);
 
 public:
   /** Set something.
@@ -514,7 +510,7 @@ public:
              int len,                   ///< Length of hostname.
              sockaddr const *aip,       ///< Address and/or port.
              HostDBApplicationInfo *app ///< I don't know.
-             );
+  );
 
   void setby_srv(const char *hostname, int len, const char *target, HostDBApplicationInfo *app);
 };
@@ -523,4 +519,4 @@ void run_HostDBTest();
 
 extern inkcoreapi HostDBProcessor hostDBProcessor;
 
-void ink_hostdb_init(ModuleVersion version);
+void ink_hostdb_init(ts::ModuleVersion version);
