@@ -24,6 +24,7 @@
 
 #pragma once
 
+#include "tscore/IpMap.h"
 #include "I_EventSystem.h"
 #include "I_Socks.h"
 struct socks_conf_struct;
@@ -63,10 +64,7 @@ public:
         @c NET_EVENT_ACCEPT_SUCCEED
         or @c NET_EVENT_ACCEPT_FAILED on success and failure resp.
     */
-    bool f_callback_on_open;
-    /** Accept only on the loopback address.
-        Default: @c false.
-     */
+
     bool localhost_only;
     /// Are frequent accepts expected?
     /// Default: @c false.
@@ -79,6 +77,9 @@ public:
     /// Socket transmit buffer size.
     /// 0 => OS default.
     int send_bufsize;
+    /// defer accpet for @c sockopt.
+    /// 0 => OS default.
+    int defer_accept;
     /// Socket options for @c sockopt.
     /// 0 => do not set options.
     uint32_t sockopt_flags;
@@ -96,6 +97,15 @@ public:
     */
     bool f_inbound_transparent;
 
+    /** MPTCP enabled on listener.
+        @internal For logging and metrics purposes to know whether the
+        listener enabled MPTCP or not.
+    */
+    bool f_mptcp;
+
+    /// Proxy Protocol enabled
+    bool f_proxy_protocol;
+
     /// Default constructor.
     /// Instance is constructed with default values.
     AcceptOptions() { this->reset(); }
@@ -151,6 +161,7 @@ public:
 
   */
   virtual Action *main_accept(Continuation *cont, SOCKET listen_socket_in, AcceptOptions const &opt = DEFAULT_ACCEPT_OPTIONS);
+  virtual void stop_accept();
 
   /**
     Open a NetVConnection for connection oriented I/O. Connects
@@ -174,13 +185,12 @@ public:
   inkcoreapi Action *connect_re(Continuation *cont, sockaddr const *addr, NetVCOptions *options = nullptr);
 
   /**
-    Starts the Netprocessor. This has to be called before doing any
-    other net call.
+    Initializes the net processor. This must be called before the event threads are started.
 
-    @param number_of_net_threads is not used. The net processor
-      uses the Event Processor threads for its activity.
   */
-  virtual int start(int number_of_net_threads, size_t stacksize) = 0;
+  virtual void init() = 0;
+
+  virtual void init_socks() = 0;
 
   inkcoreapi virtual NetVConnection *allocate_vc(EThread *) = 0;
 
@@ -188,7 +198,7 @@ public:
   NetProcessor(){};
 
   /** Private destructor. */
-  virtual ~NetProcessor(){};
+  ~NetProcessor() override{};
 
   /** This is MSS for connections we accept (client connections). */
   static int accept_mss;
@@ -212,6 +222,10 @@ public:
   /// Default options instance.
   static AcceptOptions const DEFAULT_ACCEPT_OPTIONS;
 
+  // noncopyable
+  NetProcessor(const NetProcessor &) = delete;
+  NetProcessor &operator=(const NetProcessor &) = delete;
+
 private:
   /** @note Not implemented. */
   virtual int
@@ -220,9 +234,6 @@ private:
     ink_release_assert(!"NetProcessor::stop not implemented");
     return 1;
   }
-
-  NetProcessor(const NetProcessor &);
-  NetProcessor &operator=(const NetProcessor &);
 };
 
 /**
@@ -231,7 +242,7 @@ private:
   object.
 
   @code
-    netProcesors.accept(my_cont, ...);
+    netProcessor.accept(my_cont, ...);
     netProcessor.connect_re(my_cont, ...);
   @endcode
 
@@ -246,3 +257,4 @@ extern inkcoreapi NetProcessor &netProcessor;
 
 */
 extern inkcoreapi NetProcessor &sslNetProcessor;
+extern inkcoreapi NetProcessor &quicNetProcessor;
