@@ -36,12 +36,13 @@
 #include "P_UnixNetState.h"
 #include "P_Connection.h"
 #include "P_NetAccept.h"
+#include "NetEvent.h"
 
 class UnixNetVConnection;
 class NetHandler;
 struct PollDescriptor;
 
-TS_INLINE void
+inline void
 NetVCOptions::reset()
 {
   ip_proto  = USE_TCP;
@@ -66,12 +67,14 @@ NetVCOptions::reset()
 
   etype = ET_NET;
 
-  sni_servername    = nullptr;
-  ssl_servername    = nullptr;
-  clientCertificate = nullptr;
+  sni_servername              = nullptr;
+  ssl_servername              = nullptr;
+  sni_hostname                = nullptr;
+  ssl_client_cert_name        = nullptr;
+  ssl_client_private_key_name = nullptr;
 }
 
-TS_INLINE void
+inline void
 NetVCOptions::set_sock_param(int _recv_bufsize, int _send_bufsize, unsigned long _opt_flags, unsigned long _packet_mark,
                              unsigned long _packet_tos)
 {
@@ -101,13 +104,16 @@ struct OOB_callback : public Continuation {
 
 enum tcp_congestion_control_t { CLIENT_SIDE, SERVER_SIDE };
 
-class UnixNetVConnection : public NetVConnection
+class UnixNetVConnection : public NetVConnection, public NetEvent
 {
 public:
   int64_t outstanding() override;
   VIO *do_io_read(Continuation *c, int64_t nbytes, MIOBuffer *buf) override;
   VIO *do_io_write(Continuation *c, int64_t nbytes, IOBufferReader *buf, bool owner = false) override;
 
+  Continuation *read_vio_cont() override;
+  Continuation *write_vio_cont() override;
+
   bool get_data(int id, void *data) override;
 
   Action *send_OOB(Continuation *cont, char *buf, int len) override;
@@ -135,12 +141,18 @@ public:
     return false;
   }
 
+  const char *
+  get_server_name() const override
+  {
+    return nullptr;
+  }
+
   void do_io_close(int lerrno = -1) override;
   void do_io_shutdown(ShutdownHowTo_t howto) override;
 
   ////////////////////////////////////////////////////////////
   // Set the timeouts associated with this connection.      //
-  // active_timeout is for the total elasped time of        //
+  // active_timeout is for the total elapsed time of        //
   // the connection.                                        //
   // inactivity_timeout is the elapsed time from the time   //
   // a read or a write was scheduled during which the       //
@@ -150,14 +162,17 @@ public:
   // called when handing an  event from this NetVConnection,//
   // or the NetVConnection creation callback.               //
   ////////////////////////////////////////////////////////////
-  void set_active_timeout(ink_hrtime timeout_in) override;
-  void set_inactivity_timeout(ink_hrtime timeout_in) override;
-  void cancel_active_timeout() override;
-  void cancel_inactivity_timeout() override;
+  virtual void set_active_timeout(ink_hrtime timeout_in) override;
+  virtual void set_inactivity_timeout(ink_hrtime timeout_in) override;
+  virtual void set_default_inactivity_timeout(ink_hrtime timeout_in) override;
+  virtual bool is_default_inactivity_timeout() override;
+  virtual void cancel_active_timeout() override;
+  virtual void cancel_inactivity_timeout() override;
   void set_action(Continuation *c) override;
-  void add_to_keep_alive_queue() override;
-  void remove_from_keep_alive_queue() override;
-  bool add_to_active_queue() override;
+  const Action *get_action() const;
+  virtual void add_to_keep_alive_queue() override;
+  virtual void remove_from_keep_alive_queue() override;
+  virtual bool add_to_active_queue() override;
   virtual void remove_from_active_queue();
 
   // The public interface is VIO::reenable()
@@ -206,7 +221,51 @@ public:
     return (true);
   }
 
-  virtual void net_read_io(NetHandler *nh, EThread *lthread);
+  virtual bool
+  trackFirstHandshake()
+  {
+    return false;
+  }
+
+  // NetEvent
+  virtual void net_read_io(NetHandler *nh, EThread *lthread) override;
+  virtual void net_write_io(NetHandler *nh, EThread *lthread) override;
+  virtual void free(EThread *t) override;
+  virtual int
+  close() override
+  {
+    return this->con.close();
+  }
+  virtual int
+  get_fd() override
+  {
+    return this->con.fd;
+  }
+
+  virtual EThread *
+  get_thread() override
+  {
+    return this->thread;
+  }
+
+  virtual int
+  callback(int event = CONTINUATION_EVENT_NONE, void *data = nullptr) override
+  {
+    return this->handleEvent(event, data);
+  }
+
+  virtual Ptr<ProxyMutex> &
+  get_mutex() override
+  {
+    return this->mutex;
+  }
+
+  virtual ContFlags &
+  get_control_flags() override
+  {
+    return this->control_flags;
+  }
+
   virtual int64_t load_buffer_and_write(int64_t towrite, MIOBufferAccessor &buf, int64_t &total_written, int &needs);
   void readDisable(NetHandler *nh);
   void readSignalError(NetHandler *nh, int err);
@@ -223,48 +282,14 @@ public:
   UnixNetVConnection *migrateToCurrentThread(Continuation *c, EThread *t);
 
   Action action_;
-  int closed;
-  NetState read;
-  NetState write;
 
-  LINK(UnixNetVConnection, cop_link);
-  LINKM(UnixNetVConnection, read, ready_link)
-  SLINKM(UnixNetVConnection, read, enable_link)
-  LINKM(UnixNetVConnection, write, ready_link)
-  SLINKM(UnixNetVConnection, write, enable_link)
-  LINK(UnixNetVConnection, keep_alive_queue_link);
-  LINK(UnixNetVConnection, active_queue_link);
-
-  ink_hrtime inactivity_timeout_in;
-  ink_hrtime active_timeout_in;
-  ink_hrtime next_inactivity_timeout_at;
-  ink_hrtime next_activity_timeout_at;
-
-  EventIO ep;
-  NetHandler *nh;
-  unsigned int id;
-
-  union {
-    unsigned int flags;
-#define NET_VC_SHUTDOWN_READ 1
-#define NET_VC_SHUTDOWN_WRITE 2
-    struct {
-      unsigned int got_local_addr : 1;
-      unsigned int shutdown : 2;
-    } f;
-  };
+  unsigned int id = 0;
 
   Connection con;
-  int recursion;
-  ink_hrtime submit_time;
-  OOB_callback *oob_ptr;
-  bool from_accept_thread;
-  NetAccept *accept_object;
-
-  // es - origin_trace associated connections
-  bool origin_trace;
-  const sockaddr *origin_trace_addr;
-  int origin_trace_port;
+  int recursion            = 0;
+  OOB_callback *oob_ptr    = nullptr;
+  bool from_accept_thread  = false;
+  NetAccept *accept_object = nullptr;
 
   int acceptEvent(int event, Event *e);
   int mainEvent(int event, Event *e);
@@ -276,59 +301,39 @@ public:
    */
   virtual int populate(Connection &con, Continuation *c, void *arg);
   virtual void clear();
-  virtual void free(EThread *t);
 
   ink_hrtime get_inactivity_timeout() override;
   ink_hrtime get_active_timeout() override;
 
-  void set_local_addr() override;
+  virtual void set_local_addr() override;
   void set_mptcp_state() override;
-  void set_remote_addr() override;
+  virtual void set_remote_addr() override;
   void set_remote_addr(const sockaddr *) override;
-  int set_tcp_init_cwnd(int init_cwnd) override;
   int set_tcp_congestion_control(int side) override;
   void apply_options() override;
 
   friend void write_to_net_io(NetHandler *, UnixNetVConnection *, EThread *);
-
-  void
-  setOriginTrace(bool t)
-  {
-    origin_trace = t;
-  }
-
-  void
-  setOriginTraceAddr(const sockaddr *addr)
-  {
-    origin_trace_addr = addr;
-  }
-
-  void
-  setOriginTracePort(int port)
-  {
-    origin_trace_port = port;
-  }
 };
 
 extern ClassAllocator<UnixNetVConnection> netVCAllocator;
 
 typedef int (UnixNetVConnection::*NetVConnHandler)(int, void *);
 
-TS_INLINE void
+inline void
 UnixNetVConnection::set_remote_addr()
 {
   ats_ip_copy(&remote_addr, &con.addr);
   this->control_flags.set_flag(ContFlags::DEBUG_OVERRIDE, diags->test_override_ip(remote_addr));
 }
 
-TS_INLINE void
+inline void
 UnixNetVConnection::set_remote_addr(const sockaddr *new_sa)
 {
   ats_ip_copy(&remote_addr, new_sa);
   this->control_flags.set_flag(ContFlags::DEBUG_OVERRIDE, diags->test_override_ip(remote_addr));
 }
 
-TS_INLINE void
+inline void
 UnixNetVConnection::set_local_addr()
 {
   int local_sa_size = sizeof(local_addr);
@@ -339,7 +344,7 @@ UnixNetVConnection::set_local_addr()
 }
 
 // Update the internal VC state variable for MPTCP
-TS_INLINE void
+inline void
 UnixNetVConnection::set_mptcp_state()
 {
   int mptcp_enabled      = -1;
@@ -353,19 +358,19 @@ UnixNetVConnection::set_mptcp_state()
   }
 }
 
-TS_INLINE ink_hrtime
+inline ink_hrtime
 UnixNetVConnection::get_active_timeout()
 {
   return active_timeout_in;
 }
 
-TS_INLINE ink_hrtime
+inline ink_hrtime
 UnixNetVConnection::get_inactivity_timeout()
 {
   return inactivity_timeout_in;
 }
 
-TS_INLINE void
+inline void
 UnixNetVConnection::set_active_timeout(ink_hrtime timeout_in)
 {
   Debug("socket", "Set active timeout=%" PRId64 ", NetVC=%p", timeout_in, this);
@@ -373,15 +378,15 @@ UnixNetVConnection::set_active_timeout(ink_hrtime timeout_in)
   next_activity_timeout_at = (active_timeout_in > 0) ? Thread::get_hrtime() + timeout_in : 0;
 }
 
-TS_INLINE void
+inline void
 UnixNetVConnection::cancel_inactivity_timeout()
 {
   Debug("socket", "Cancel inactive timeout for NetVC=%p", this);
-  inactivity_timeout_in = 0;
-  set_inactivity_timeout(0);
+  inactivity_timeout_in      = 0;
+  next_inactivity_timeout_at = 0;
 }
 
-TS_INLINE void
+inline void
 UnixNetVConnection::cancel_active_timeout()
 {
   Debug("socket", "Cancel active timeout for NetVC=%p", this);
@@ -389,36 +394,28 @@ UnixNetVConnection::cancel_active_timeout()
   next_activity_timeout_at = 0;
 }
 
-TS_INLINE int
-UnixNetVConnection::set_tcp_init_cwnd(int init_cwnd)
-{
-#ifdef TCP_INIT_CWND
-  int rv;
-  uint32_t val = init_cwnd;
-  rv           = setsockopt(con.fd, IPPROTO_TCP, TCP_INIT_CWND, &val, sizeof(val));
-  Debug("socket", "Setting TCP initial congestion window (%d) -> %d", init_cwnd, rv);
-  return rv;
-#else
-  Debug("socket", "Setting TCP initial congestion window %d -> unsupported", init_cwnd);
-  return -1;
-#endif
-}
+inline UnixNetVConnection::~UnixNetVConnection() {}
 
-TS_INLINE UnixNetVConnection::~UnixNetVConnection() {}
-
-TS_INLINE SOCKET
+inline SOCKET
 UnixNetVConnection::get_socket()
 {
   return con.fd;
 }
 
-TS_INLINE void
+inline void
 UnixNetVConnection::set_action(Continuation *c)
 {
   action_ = c;
 }
 
+inline const Action *
+UnixNetVConnection::get_action() const
+{
+  return &action_;
+}
+
 // declarations for local use (within the net module)
 
 void write_to_net(NetHandler *nh, UnixNetVConnection *vc, EThread *thread);
 void write_to_net_io(NetHandler *nh, UnixNetVConnection *vc, EThread *thread);
+void net_activity(UnixNetVConnection *vc, EThread *thread);
