@@ -21,38 +21,42 @@
   limitations under the License.
  */
 
-#include "ts/ink_config.h"
-
 #include "P_SSLCertLookup.h"
+
+#include "tscore/ink_config.h"
+#include "tscore/I_Layout.h"
+#include "tscore/MatcherUtils.h"
+#include "tscore/Regex.h"
+#include "tscore/Trie.h"
+#include "tscore/BufferWriter.h"
+#include "tscore/bwf_std_format.h"
+#include "tscore/TestBox.h"
+
+#include "I_EventSystem.h"
+
 #include "P_SSLUtils.h"
 #include "P_SSLConfig.h"
-#include "I_EventSystem.h"
-#include "ts/I_Layout.h"
-#include "ts/Regex.h"
-#include "ts/Trie.h"
-#include "ts/TestBox.h"
+#include "SSLSessionTicket.h"
+
+#include <unordered_map>
+#include <utility>
+#include <vector>
+#include <algorithm>
 
 struct SSLAddressLookupKey {
-  explicit SSLAddressLookupKey(const IpEndpoint &ip) : sep(0)
+  explicit SSLAddressLookupKey(const IpEndpoint &ip)
   {
-    static const char hextab[16] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};
+    // For IP addresses, the cache key is the hex address with the port concatenated. This makes the
+    // lookup insensitive to address formatting and also allow the longest match semantic to produce
+    // different matches if there is a certificate on the port.
 
-    int nbytes;
-    uint16_t port = ntohs(ip.port());
-
-    // For IP addresses, the cache key is the hex address with the port concatenated. This makes the lookup
-    // insensitive to address formatting and also allow the longest match semantic to produce different matches
-    // if there is a certificate on the port.
-    nbytes = ats_ip_to_hex(&ip.sa, key, sizeof(key));
-    if (port) {
-      sep           = nbytes;
-      key[nbytes++] = '.';
-      key[nbytes++] = hextab[(port >> 12) & 0x000F];
-      key[nbytes++] = hextab[(port >> 8) & 0x000F];
-      key[nbytes++] = hextab[(port >> 4) & 0x000F];
-      key[nbytes++] = hextab[(port)&0x000F];
+    ts::FixedBufferWriter w{key, sizeof(key)};
+    w.print("{}", ts::bwf::Hex_Dump(ip)); // dump as raw hex bytes, don't format as IP address.
+    if (in_port_t port = ip.host_order_port(); port) {
+      sep = static_cast<unsigned char>(w.size());
+      w.print(".{:x}", port);
     }
-    key[nbytes++] = 0;
+    w.write('\0'); // force C-string termination.
   }
 
   const char *
@@ -72,8 +76,8 @@ struct SSLAddressLookupKey {
   }
 
 private:
-  char key[(TS_IP6_SIZE * 2) /* hex addr */ + 1 /* dot */ + 4 /* port */ + 1 /* NULL */];
-  unsigned char sep; // offset of address/port separator
+  char key[(TS_IP6_SIZE * 2) /* hex addr */ + 1 /* dot */ + 4 /* port */ + 1 /* nullptr */];
+  unsigned char sep = 0; // offset of address/port separator
 };
 
 struct SSLContextStorage {
@@ -91,14 +95,15 @@ public:
   /// cert context.
   /// @return @a idx
   int insert(const char *name, int idx);
-  SSLCertContext *lookup(const char *name) const;
+  SSLCertContext *lookup(const char *name);
+  void printWildDomains() const;
   unsigned
   count() const
   {
-    return this->ctx_store.length();
+    return this->ctx_store.size();
   }
   SSLCertContext *
-  get(unsigned i) const
+  get(unsigned i)
   {
     return &this->ctx_store[i];
   }
@@ -109,39 +114,56 @@ private:
       linkage required by @c Trie.
   */
   struct ContextRef {
-    ContextRef() : idx(-1) {}
+    ContextRef() {}
     explicit ContextRef(int n) : idx(n) {}
     void
     Print() const
     {
       Debug("ssl", "Item=%p SSL_CTX=#%d", this, idx);
     }
-    int idx;                ///< Index in the context store.
+    int idx = -1;           ///< Index in the context store.
     LINK(ContextRef, link); ///< Require by @c Trie
   };
 
   /// We can only match one layer with the wildcards
   /// This table stores the wildcarded subdomain
-  InkHashTable *wilddomains;
-
+  std::unordered_map<std::string, int> wilddomains;
   /// Contexts stored by IP address or FQDN
-  InkHashTable *hostnames;
+  std::unordered_map<std::string, int> hostnames;
   /// List for cleanup.
   /// Exactly one pointer to each SSL context is stored here.
-  Vec<SSLCertContext> ctx_store;
+  std::vector<SSLCertContext> ctx_store;
 
   /// Add a context to the clean up list.
   /// @return The index of the added context.
   int store(SSLCertContext const &cc);
 };
 
+namespace
+{
+/** Copy @a src to @a dst, transforming to lower case.
+ *
+ * @param src Input string.
+ * @param dst Output buffer.
+ */
+inline void
+transform_lower(std::string_view src, ts::MemSpan<char> dst)
+{
+  if (src.size() > dst.size() - 1) { // clip @a src, reserving space for the terminal nul.
+    src = std::string_view{src.data(), dst.size() - 1};
+  }
+  auto final = std::transform(src.begin(), src.end(), dst.data(), [](char c) -> char { return std::tolower(c); });
+  *final++   = '\0';
+}
+} // namespace
+
 // Zero out and free the heap space allocated for ticket keys to avoid leaking secrets.
 // The first several bytes stores the number of keys and the rest stores the ticket keys.
 void
 ticket_block_free(void *ptr)
 {
   if (ptr) {
-    ssl_ticket_key_block *key_block_ptr = (ssl_ticket_key_block *)ptr;
+    ssl_ticket_key_block *key_block_ptr = static_cast<ssl_ticket_key_block *>(ptr);
     unsigned num_ticket_keys            = key_block_ptr->num_keys;
     memset(ptr, 0, sizeof(ssl_ticket_key_block) + num_ticket_keys * sizeof(ssl_ticket_key_t));
   }
@@ -154,29 +176,115 @@ ticket_block_alloc(unsigned count)
   ssl_ticket_key_block *ptr;
   size_t nbytes = sizeof(ssl_ticket_key_block) + count * sizeof(ssl_ticket_key_t);
 
-  ptr = (ssl_ticket_key_block *)ats_malloc(nbytes);
+  ptr = static_cast<ssl_ticket_key_block *>(ats_malloc(nbytes));
   memset(ptr, 0, nbytes);
   ptr->num_keys = count;
 
   return ptr;
 }
-
-void
-SSLCertContext::release()
+ssl_ticket_key_block *
+ticket_block_create(char *ticket_key_data, int ticket_key_len)
 {
-  if (keyblock) {
-    ticket_block_free(keyblock);
-    keyblock = NULL;
+  ssl_ticket_key_block *keyblock = nullptr;
+  unsigned num_ticket_keys       = ticket_key_len / sizeof(ssl_ticket_key_t);
+  if (num_ticket_keys == 0) {
+    Error("SSL session ticket key is too short (>= 48 bytes are required)");
+    goto fail;
+  }
+  Debug("ssl", "Create %d ticket key blocks", num_ticket_keys);
+
+  keyblock = ticket_block_alloc(num_ticket_keys);
+
+  // Slurp all the keys in the ticket key file. We will encrypt with the first key, and decrypt
+  // with any key (for rotation purposes).
+  for (unsigned i = 0; i < num_ticket_keys; ++i) {
+    const char *data = (const char *)ticket_key_data + (i * sizeof(ssl_ticket_key_t));
+
+    memcpy(keyblock->keys[i].key_name, data, sizeof(keyblock->keys[i].key_name));
+    memcpy(keyblock->keys[i].hmac_secret, data + sizeof(keyblock->keys[i].key_name), sizeof(keyblock->keys[i].hmac_secret));
+    memcpy(keyblock->keys[i].aes_key, data + sizeof(keyblock->keys[i].key_name) + sizeof(keyblock->keys[i].hmac_secret),
+           sizeof(keyblock->keys[i].aes_key));
   }
 
-  SSLReleaseContext(ctx);
-  ctx = NULL;
+  return keyblock;
+
+fail:
+  ticket_block_free(keyblock);
+  return nullptr;
 }
 
-SSLCertLookup::SSLCertLookup() : ssl_storage(new SSLContextStorage()), ssl_default(NULL), is_valid(true)
+ssl_ticket_key_block *
+ssl_create_ticket_keyblock(const char *ticket_key_path)
 {
+#if TS_HAVE_OPENSSL_SESSION_TICKETS
+  ats_scoped_str ticket_key_data;
+  int ticket_key_len;
+  ssl_ticket_key_block *keyblock = nullptr;
+
+  if (ticket_key_path != nullptr) {
+    ticket_key_data = readIntoBuffer(ticket_key_path, __func__, &ticket_key_len);
+    if (!ticket_key_data) {
+      Error("failed to read SSL session ticket key from %s", (const char *)ticket_key_path);
+      goto fail;
+    }
+    keyblock = ticket_block_create(ticket_key_data, ticket_key_len);
+  } else {
+    // Generate a random ticket key
+    ssl_ticket_key_t key;
+    RAND_bytes(reinterpret_cast<unsigned char *>(&key), sizeof(key));
+    keyblock = ticket_block_create(reinterpret_cast<char *>(&key), sizeof(key));
+  }
+
+  return keyblock;
+
+fail:
+  ticket_block_free(keyblock);
+  return nullptr;
+
+#else  /* !TS_HAVE_OPENSSL_SESSION_TICKETS */
+  (void)ticket_key_path;
+  return nullptr;
+#endif /* TS_HAVE_OPENSSL_SESSION_TICKETS */
 }
 
+SSLCertContext::SSLCertContext(SSLCertContext const &other)
+{
+  opt        = other.opt;
+  userconfig = other.userconfig;
+  keyblock   = other.keyblock;
+  std::lock_guard<std::mutex> lock(other.ctx_mutex);
+  ctx = other.ctx;
+}
+
+SSLCertContext &
+SSLCertContext::operator=(SSLCertContext const &other)
+{
+  if (&other != this) {
+    this->opt        = other.opt;
+    this->userconfig = other.userconfig;
+    this->keyblock   = other.keyblock;
+    std::lock_guard<std::mutex> lock(other.ctx_mutex);
+    this->ctx = other.ctx;
+  }
+  return *this;
+}
+
+shared_SSL_CTX
+SSLCertContext::getCtx()
+{
+  std::lock_guard<std::mutex> lock(ctx_mutex);
+  return ctx;
+}
+
+void
+SSLCertContext::setCtx(shared_SSL_CTX sc)
+{
+  std::lock_guard<std::mutex> lock(ctx_mutex);
+  ctx = std::move(sc);
+}
+
+SSLCertLookup::SSLCertLookup() : ssl_storage(new SSLContextStorage()), ssl_default(nullptr), is_valid(true) {}
+
 SSLCertLookup::~SSLCertLookup()
 {
   delete this->ssl_storage;
@@ -205,7 +313,7 @@ SSLCertLookup::find(const IpEndpoint &address) const
     return this->ssl_storage->lookup(key.get());
   }
 
-  return NULL;
+  return nullptr;
 }
 
 int
@@ -233,39 +341,102 @@ SSLCertLookup::get(unsigned i) const
   return ssl_storage->get(i);
 }
 
-struct ats_wildcard_matcher {
-  ats_wildcard_matcher()
-  {
-    if (regex.compile("^\\*\\.[^\\*.]+") != 0) {
-      Fatal("failed to compile TLS wildcard matching regex");
+SSLContextStorage::SSLContextStorage() {}
+
+SSLContextStorage::~SSLContextStorage() {}
+
+int
+SSLContextStorage::store(SSLCertContext const &cc)
+{
+  this->ctx_store.push_back(cc);
+  return this->ctx_store.size() - 1;
+}
+
+int
+SSLContextStorage::insert(const char *name, SSLCertContext const &cc)
+{
+  int idx = this->store(cc);
+  idx     = this->insert(name, idx);
+  if (idx < 0) {
+    this->ctx_store.pop_back();
+  }
+  return idx;
+}
+
+int
+SSLContextStorage::insert(const char *name, int idx)
+{
+  ats_wildcard_matcher wildcard;
+  char lower_case_name[TS_MAX_HOST_NAME_LEN + 1];
+  transform_lower(name, lower_case_name);
+
+  shared_SSL_CTX ctx = this->ctx_store[idx].getCtx();
+  if (wildcard.match(lower_case_name)) {
+    // Strip the wildcard and store the subdomain
+    const char *subdomain = index(lower_case_name, '*');
+    if (subdomain && subdomain[1] == '.') {
+      subdomain += 2; // Move beyond the '.'
+    } else {
+      subdomain = nullptr;
+    }
+    if (subdomain) {
+      if (auto it = this->wilddomains.find(subdomain); it != this->wilddomains.end()) {
+        Debug("ssl", "previously indexed '%s' with SSL_CTX #%d, cannot index it with SSL_CTX #%d now", lower_case_name, it->second,
+              idx);
+        idx = -1;
+      } else {
+        this->wilddomains.emplace(subdomain, idx);
+        Debug("ssl", "indexed '%s' with SSL_CTX %p [%d]", lower_case_name, ctx.get(), idx);
+      }
+    }
+  } else {
+    if (auto it = this->hostnames.find(lower_case_name); it != this->hostnames.end() && idx != it->second) {
+      Debug("ssl", "previously indexed '%s' with SSL_CTX %d, cannot index it with SSL_CTX #%d now", lower_case_name, it->second,
+            idx);
+      idx = -1;
+    } else {
+      this->hostnames.emplace(lower_case_name, idx);
+      Debug("ssl", "indexed '%s' with SSL_CTX %p [%d]", lower_case_name, ctx.get(), idx);
     }
   }
-
-  ~ats_wildcard_matcher() {}
-  bool
-  match(const char *hostname) const
-  {
-    return regex.match(hostname) != -1;
-  }
-
-private:
-  DFA regex;
-};
-
-static void
-make_to_lower_case(const char *name, char *lower_case_name, int buf_len)
-{
-  int name_len = strlen(name);
-  int i;
-  if (name_len > (buf_len - 1)) {
-    name_len = buf_len - 1;
-  }
-  for (i = 0; i < name_len; i++) {
-    lower_case_name[i] = ParseRules::ink_tolower(name[i]);
-  }
-  lower_case_name[i] = '\0';
+  return idx;
 }
 
+void
+SSLContextStorage::printWildDomains() const
+{
+  for (auto &&it : this->wilddomains) {
+    Debug("ssl", "Stored wilddomain %s", it.first.c_str());
+  }
+}
+
+SSLCertContext *
+SSLContextStorage::lookup(const char *name)
+{
+  // First look for an exact name match
+  if (auto it = this->hostnames.find(name); it != this->hostnames.end()) {
+    return &(this->ctx_store[it->second]);
+  }
+  // Try lower casing it
+  char lower_case_name[TS_MAX_HOST_NAME_LEN + 1];
+  transform_lower(name, lower_case_name);
+  if (auto it_lower = this->hostnames.find(lower_case_name); it_lower != this->hostnames.end()) {
+    return &(this->ctx_store[it_lower->second]);
+  }
+
+  // Then strip off the top domain name and look for a wildcard domain match
+  const char *subdomain = index(lower_case_name, '.');
+  if (subdomain) {
+    ++subdomain; // Move beyond the '.'
+    if (auto it = this->wilddomains.find(subdomain); it != this->wilddomains.end()) {
+      return &(this->ctx_store[it->second]);
+    }
+  }
+  return nullptr;
+}
+
+#if TS_HAS_TESTS
+
 static char *
 reverse_dns_name(const char *hostname, char (&reversed)[TS_MAX_HOST_NAME_LEN + 1])
 {
@@ -279,7 +450,7 @@ reverse_dns_name(const char *hostname, char (&reversed)[TS_MAX_HOST_NAME_LEN + 1
     ssize_t remain = ptr - reversed;
 
     if (remain < (len + 1)) {
-      return NULL;
+      return nullptr;
     }
 
     ptr -= len;
@@ -293,125 +464,11 @@ reverse_dns_name(const char *hostname, char (&reversed)[TS_MAX_HOST_NAME_LEN + 1
       *(--ptr) = '.';
     }
   }
-  make_to_lower_case(ptr, ptr, strlen(ptr) + 1);
+  transform_lower(ptr, {ptr, strlen(ptr) + 1});
 
   return ptr;
 }
 
-SSLContextStorage::SSLContextStorage()
-  : wilddomains(ink_hash_table_create(InkHashTableKeyType_String)), hostnames(ink_hash_table_create(InkHashTableKeyType_String))
-{
-}
-
-bool
-SSLCtxCompare(SSLCertContext const &cc1, SSLCertContext const &cc2)
-{
-  // Either they are both real ctx pointers and cc1 has the smaller pointer
-  // Or only cc2 has a non-null pointer
-  return cc1.ctx < cc2.ctx;
-}
-
-SSLContextStorage::~SSLContextStorage()
-{
-  // First sort the array so we can efficiently detect duplicates
-  // and avoid the double free
-  this->ctx_store.qsort(SSLCtxCompare);
-  SSL_CTX *last_ctx = NULL;
-  for (unsigned i = 0; i < this->ctx_store.length(); ++i) {
-    if (this->ctx_store[i].ctx != last_ctx) {
-      last_ctx = this->ctx_store[i].ctx;
-      this->ctx_store[i].release();
-    }
-  }
-
-  ink_hash_table_destroy(this->hostnames);
-  ink_hash_table_destroy(this->wilddomains);
-}
-
-int
-SSLContextStorage::store(SSLCertContext const &cc)
-{
-  int idx = this->ctx_store.length();
-  this->ctx_store.add(cc);
-  return idx;
-}
-
-int
-SSLContextStorage::insert(const char *name, SSLCertContext const &cc)
-{
-  int idx = this->store(cc);
-  idx     = this->insert(name, idx);
-  if (idx < 0)
-    this->ctx_store.drop();
-  return idx;
-}
-
-int
-SSLContextStorage::insert(const char *name, int idx)
-{
-  ats_wildcard_matcher wildcard;
-  InkHashTableValue value;
-  char lower_case_name[TS_MAX_HOST_NAME_LEN + 1];
-  make_to_lower_case(name, lower_case_name, sizeof(lower_case_name));
-
-  if (wildcard.match(lower_case_name)) {
-    // Strip the wildcard and store the subdomain
-    const char *subdomain = index(lower_case_name, '*');
-    if (subdomain && subdomain[1] == '.') {
-      subdomain += 2; // Move beyond the '.'
-    } else {
-      subdomain = NULL;
-    }
-    if (subdomain) {
-      if (ink_hash_table_lookup(this->wilddomains, subdomain, &value) && reinterpret_cast<InkHashTableValue>(idx) != value) {
-        Warning("previously indexed '%s' with SSL_CTX %p, cannot index it with SSL_CTX #%d now", lower_case_name, value, idx);
-        idx = -1;
-      } else {
-        ink_hash_table_insert(this->wilddomains, subdomain, reinterpret_cast<void *>(static_cast<intptr_t>(idx)));
-        Debug("ssl", "indexed '%s' with SSL_CTX %p [%d]", lower_case_name, this->ctx_store[idx].ctx, idx);
-      }
-    }
-  } else {
-    if (ink_hash_table_lookup(this->hostnames, lower_case_name, &value) && reinterpret_cast<InkHashTableValue>(idx) != value) {
-      Warning("previously indexed '%s' with SSL_CTX %p, cannot index it with SSL_CTX #%d now", lower_case_name, value, idx);
-      idx = -1;
-    } else {
-      ink_hash_table_insert(this->hostnames, lower_case_name, reinterpret_cast<void *>(static_cast<intptr_t>(idx)));
-      Debug("ssl", "indexed '%s' with SSL_CTX %p [%d]", lower_case_name, this->ctx_store[idx].ctx, idx);
-    }
-  }
-  return idx;
-}
-
-SSLCertContext *
-SSLContextStorage::lookup(const char *name) const
-{
-  InkHashTableValue value;
-
-  // First look for an exact name match
-  if (ink_hash_table_lookup(const_cast<InkHashTable *>(this->hostnames), name, &value)) {
-    return &(this->ctx_store[reinterpret_cast<intptr_t>(value)]);
-  }
-  // Try lower casing it
-  char lower_case_name[TS_MAX_HOST_NAME_LEN + 1];
-  make_to_lower_case(name, lower_case_name, sizeof(lower_case_name));
-  if (ink_hash_table_lookup(const_cast<InkHashTable *>(this->hostnames), lower_case_name, &value)) {
-    return &(this->ctx_store[reinterpret_cast<intptr_t>(value)]);
-  }
-
-  // Then strip off the top domain name and look for a wildcard domain match
-  const char *subdomain = index(name, '.');
-  if (subdomain) {
-    ++subdomain; // Move beyond the '.'
-    if (ink_hash_table_lookup(const_cast<InkHashTable *>(this->wilddomains), subdomain, &value)) {
-      return &(this->ctx_store[reinterpret_cast<intptr_t>(value)]);
-    }
-  }
-  return NULL;
-}
-
-#if TS_HAS_TESTS
-
 REGRESSION_TEST(SSLWildcardMatch)(RegressionTest *t, int /* atype ATS_UNUSED */, int *pstatus)
 {
   TestBox box(t, pstatus);
