@@ -19,13 +19,15 @@
   limitations under the License.
  */
 
-#include "ts/ink_config.h"
+#include "tscore/ink_config.h"
 
 #include "P_Net.h"
-#include "ts/I_Layout.h"
-#include "I_RecHttp.h"
+#include "tscore/I_Layout.h"
+#include "records/I_RecHttp.h"
 #include "P_SSLUtils.h"
 #include "P_OCSPStapling.h"
+#include "SSLStats.h"
+#include "P_SSLClientCoordinator.h"
 
 //
 // Global Data
@@ -34,7 +36,7 @@
 SSLNetProcessor ssl_NetProcessor;
 NetProcessor &sslNetProcessor = ssl_NetProcessor;
 
-#ifdef HAVE_OPENSSL_OCSP_STAPLING
+#if TS_USE_TLS_OCSP
 struct OCSPContinuation : public Continuation {
   int
   mainEvent(int /* event ATS_UNUSED */, Event * /* e ATS_UNUSED */)
@@ -46,13 +48,11 @@ struct OCSPContinuation : public Continuation {
 
   OCSPContinuation() : Continuation(new_ProxyMutex()) { SET_HANDLER(&OCSPContinuation::mainEvent); }
 };
-#endif /* HAVE_OPENSSL_OCSP_STAPLING */
+#endif /* TS_USE_TLS_OCSP */
 
 void
-SSLNetProcessor::cleanup(void)
+SSLNetProcessor::cleanup()
 {
-  SSLReleaseContext(client_ctx);
-  client_ctx = NULL;
 }
 
 int
@@ -60,31 +60,31 @@ SSLNetProcessor::start(int, size_t stacksize)
 {
   // This initialization order matters ...
   SSLInitializeLibrary();
-  SSLConfig::startup();
+  SSLClientCoordinator::startup();
+  SSLPostConfigInitialize();
 
-  if (!SSLCertificateConfig::startup())
+  if (!SSLCertificateConfig::startup()) {
     return -1;
+  }
+  SSLTicketKeyConfig::startup();
 
   // Acquire a SSLConfigParams instance *after* we start SSL up.
-  SSLConfig::scoped_config params;
-
-  // Enable client regardless of config file settings as remap file
-  // can cause HTTP layer to connect using SSL. But only if SSL
-  // initialization hasn't failed already.
-  client_ctx = SSLInitClientContext(params);
-  if (!client_ctx) {
-    SSLError("Can't initialize the SSL client, HTTPS in remap rules will not function");
-  }
+  // SSLConfig::scoped_config params;
 
   // Initialize SSL statistics. This depends on an initial set of certificates being loaded above.
   SSLInitializeStatistics();
 
-#ifdef HAVE_OPENSSL_OCSP_STAPLING
+#if TS_USE_TLS_OCSP
   if (SSLConfigParams::ssl_ocsp_enabled) {
-    EventType ET_OCSP = eventProcessor.spawn_event_threads(1, "ET_OCSP", stacksize);
+    // Call the update initially to get things populated
+    Note("Initial OCSP refresh started");
+    ocsp_update();
+    Note("Initial OCSP refresh finished");
+
+    EventType ET_OCSP = eventProcessor.spawn_event_threads("ET_OCSP", 1, stacksize);
     eventProcessor.schedule_every(new OCSPContinuation(), HRTIME_SECONDS(SSLConfigParams::ssl_ocsp_update_period), ET_OCSP);
   }
-#endif /* HAVE_OPENSSL_OCSP_STAPLING */
+#endif /* TS_USE_TLS_OCSP */
 
   // We have removed the difference between ET_SSL threads and ET_NET threads,
   // So just keep on chugging
@@ -92,9 +92,9 @@ SSLNetProcessor::start(int, size_t stacksize)
 }
 
 NetAccept *
-SSLNetProcessor::createNetAccept()
+SSLNetProcessor::createNetAccept(const NetProcessor::AcceptOptions &opt)
 {
-  return (NetAccept *)new SSLNetAccept;
+  return (NetAccept *)new SSLNetAccept(opt);
 }
 
 NetVConnection *
@@ -113,9 +113,7 @@ SSLNetProcessor::allocate_vc(EThread *t)
   return vc;
 }
 
-SSLNetProcessor::SSLNetProcessor() : client_ctx(NULL)
-{
-}
+SSLNetProcessor::SSLNetProcessor() {}
 
 SSLNetProcessor::~SSLNetProcessor()
 {
