@@ -52,7 +52,7 @@ ssl_netvc_cast(int event, void *edata)
     ptr.vio = static_cast<VIO *>(edata);
     return dynamic_cast<SSLNetVConnection *>(ptr.vio->vc_server);
   default:
-    return NULL;
+    return nullptr;
   }
 }
 
@@ -76,7 +76,7 @@ struct SSLNextProtocolTrampoline : public Continuation {
 
     vio   = static_cast<VIO *>(edata);
     netvc = dynamic_cast<SSLNetVConnection *>(vio->vc_server);
-    ink_assert(netvc != NULL);
+    ink_assert(netvc != nullptr);
 
     switch (event) {
     case VC_EVENT_EOS:
@@ -84,8 +84,8 @@ struct SSLNextProtocolTrampoline : public Continuation {
     case VC_EVENT_ACTIVE_TIMEOUT:
     case VC_EVENT_INACTIVITY_TIMEOUT:
       // Cancel the read before we have a chance to delete the continuation
-      netvc->do_io_read(NULL, 0, NULL);
-      netvc->do_io(VIO::CLOSE);
+      netvc->do_io_read(nullptr, 0, nullptr);
+      netvc->do_io_close();
       delete this;
       return EVENT_ERROR;
     case VC_EVENT_READ_COMPLETE:
@@ -97,10 +97,10 @@ struct SSLNextProtocolTrampoline : public Continuation {
     // Cancel the action, so later timeouts and errors don't try to
     // send the event to the Accept object.  After this point, the accept
     // object does not care.
-    netvc->set_action(NULL);
+    netvc->set_action(nullptr);
 
     // Cancel the read before we have a chance to delete the continuation
-    netvc->do_io_read(NULL, 0, NULL);
+    netvc->do_io_read(nullptr, 0, nullptr);
     plugin = netvc->endpoint();
     if (plugin) {
       send_plugin_event(plugin, NET_EVENT_ACCEPT, netvc);
@@ -109,7 +109,7 @@ struct SSLNextProtocolTrampoline : public Continuation {
       send_plugin_event(npnParent->endpoint, NET_EVENT_ACCEPT, netvc);
     } else {
       // No handler, what should we do? Best to just kill the VC while we can.
-      netvc->do_io(VIO::CLOSE);
+      netvc->do_io_close();
     }
 
     delete this;
@@ -125,10 +125,9 @@ SSLNextProtocolAccept::mainEvent(int event, void *edata)
   SSLNetVConnection *netvc = ssl_netvc_cast(event, edata);
 
   Debug("ssl", "[SSLNextProtocolAccept:mainEvent] event %d netvc %p", event, netvc);
-
   switch (event) {
   case NET_EVENT_ACCEPT:
-    ink_release_assert(netvc != NULL);
+    ink_release_assert(netvc != nullptr);
 
     netvc->setTransparentPassThrough(transparent_passthrough);
 
@@ -136,11 +135,13 @@ SSLNextProtocolAccept::mainEvent(int event, void *edata)
     // force the SSLNetVConnection to complete the SSL handshake. Don't tell
     // the endpoint that there is an accept to handle until the read completes
     // and we know which protocol was negotiated.
-    netvc->registerNextProtocolSet(&this->protoset);
-    netvc->do_io(VIO::READ, new SSLNextProtocolTrampoline(this, netvc->mutex), 0, this->buffer, 0);
+    netvc->registerNextProtocolSet(&this->protoset, this->protoenabled);
+    netvc->do_io_read(new SSLNextProtocolTrampoline(this, netvc->mutex), 0, this->buffer);
     return EVENT_CONT;
   default:
-    netvc->do_io(VIO::CLOSE);
+    if (netvc) {
+      netvc->do_io_close();
+    }
     return EVENT_DONE;
   }
 }
@@ -158,18 +159,27 @@ SSLNextProtocolAccept::registerEndpoint(const char *protocol, Continuation *hand
   return this->protoset.registerEndpoint(protocol, handler);
 }
 
-bool
-SSLNextProtocolAccept::unregisterEndpoint(const char *protocol, Continuation *handler)
+void
+SSLNextProtocolAccept::enableProtocols(const SessionProtocolSet &protos)
 {
-  return this->protoset.unregisterEndpoint(protocol, handler);
+  this->protoenabled = protos;
 }
 
 SSLNextProtocolAccept::SSLNextProtocolAccept(Continuation *ep, bool transparent_passthrough)
-  : SessionAccept(NULL), buffer(new_empty_MIOBuffer()), endpoint(ep), transparent_passthrough(transparent_passthrough)
+  : SessionAccept(nullptr),
+    buffer(new_empty_MIOBuffer(SSLConfigParams::ssl_misc_max_iobuffer_size_index)),
+    endpoint(ep),
+    transparent_passthrough(transparent_passthrough)
 {
   SET_HANDLER(&SSLNextProtocolAccept::mainEvent);
 }
 
+SSLNextProtocolSet *
+SSLNextProtocolAccept::getProtoSet()
+{
+  return &this->protoset;
+}
+
 SSLNextProtocolAccept::~SSLNextProtocolAccept()
 {
   free_MIOBuffer(this->buffer);
