@@ -23,12 +23,14 @@
 
 #include <records/I_RecCore.h>
 #include <records/I_RecHttp.h>
-#include <ts/ink_defs.h>
-#include <ts/ink_hash_table.h>
-#include <ts/Tokenizer.h>
-#include <ts/MemView.h>
+#include "tscore/ink_defs.h"
+#include "tscore/TextBuffer.h"
+#include "tscore/Tokenizer.h"
 #include <strings.h>
-#include <ts/ink_inet.h>
+#include "tscore/ink_inet.h"
+#include <string_view>
+#include <unordered_set>
+#include <tscore/IpMapConf.h>
 
 SessionProtocolNameRegistry globalSessionProtocolNameRegistry;
 
@@ -36,39 +38,68 @@ SessionProtocolNameRegistry globalSessionProtocolNameRegistry;
    These are also used for NPN setup.
 */
 
-const char *const TS_ALPN_PROTOCOL_HTTP_0_9 = IP_PROTO_TAG_HTTP_0_9.ptr();
-const char *const TS_ALPN_PROTOCOL_HTTP_1_0 = IP_PROTO_TAG_HTTP_1_0.ptr();
-const char *const TS_ALPN_PROTOCOL_HTTP_1_1 = IP_PROTO_TAG_HTTP_1_1.ptr();
-const char *const TS_ALPN_PROTOCOL_HTTP_2_0 = IP_PROTO_TAG_HTTP_2_0.ptr();
+const char *const TS_ALPN_PROTOCOL_HTTP_0_9      = IP_PROTO_TAG_HTTP_0_9.data();
+const char *const TS_ALPN_PROTOCOL_HTTP_1_0      = IP_PROTO_TAG_HTTP_1_0.data();
+const char *const TS_ALPN_PROTOCOL_HTTP_1_1      = IP_PROTO_TAG_HTTP_1_1.data();
+const char *const TS_ALPN_PROTOCOL_HTTP_2_0      = IP_PROTO_TAG_HTTP_2_0.data();
+const char *const TS_ALPN_PROTOCOL_HTTP_3        = IP_PROTO_TAG_HTTP_3.data();
+const char *const TS_ALPN_PROTOCOL_HTTP_QUIC     = IP_PROTO_TAG_HTTP_QUIC.data();
+const char *const TS_ALPN_PROTOCOL_HTTP_3_D27    = IP_PROTO_TAG_HTTP_3_D27.data();
+const char *const TS_ALPN_PROTOCOL_HTTP_QUIC_D27 = IP_PROTO_TAG_HTTP_QUIC_D27.data();
 
 const char *const TS_ALPN_PROTOCOL_GROUP_HTTP  = "http";
 const char *const TS_ALPN_PROTOCOL_GROUP_HTTP2 = "http2";
 
-const char *const TS_PROTO_TAG_HTTP_1_0 = TS_ALPN_PROTOCOL_HTTP_1_0;
-const char *const TS_PROTO_TAG_HTTP_1_1 = TS_ALPN_PROTOCOL_HTTP_1_1;
-const char *const TS_PROTO_TAG_HTTP_2_0 = TS_ALPN_PROTOCOL_HTTP_2_0;
-const char *const TS_PROTO_TAG_TLS_1_3  = IP_PROTO_TAG_TLS_1_3.ptr();
-const char *const TS_PROTO_TAG_TLS_1_2  = IP_PROTO_TAG_TLS_1_2.ptr();
-const char *const TS_PROTO_TAG_TLS_1_1  = IP_PROTO_TAG_TLS_1_1.ptr();
-const char *const TS_PROTO_TAG_TLS_1_0  = IP_PROTO_TAG_TLS_1_0.ptr();
-const char *const TS_PROTO_TAG_TCP      = IP_PROTO_TAG_TCP.ptr();
-const char *const TS_PROTO_TAG_UDP      = IP_PROTO_TAG_UDP.ptr();
-const char *const TS_PROTO_TAG_IPV4     = IP_PROTO_TAG_IPV4.ptr();
-const char *const TS_PROTO_TAG_IPV6     = IP_PROTO_TAG_IPV6.ptr();
+const char *const TS_PROTO_TAG_HTTP_1_0      = TS_ALPN_PROTOCOL_HTTP_1_0;
+const char *const TS_PROTO_TAG_HTTP_1_1      = TS_ALPN_PROTOCOL_HTTP_1_1;
+const char *const TS_PROTO_TAG_HTTP_2_0      = TS_ALPN_PROTOCOL_HTTP_2_0;
+const char *const TS_PROTO_TAG_HTTP_3        = TS_ALPN_PROTOCOL_HTTP_3;
+const char *const TS_PROTO_TAG_HTTP_QUIC     = TS_ALPN_PROTOCOL_HTTP_QUIC;
+const char *const TS_PROTO_TAG_HTTP_3_D27    = TS_ALPN_PROTOCOL_HTTP_3_D27;
+const char *const TS_PROTO_TAG_HTTP_QUIC_D27 = TS_ALPN_PROTOCOL_HTTP_QUIC_D27;
+const char *const TS_PROTO_TAG_TLS_1_3       = IP_PROTO_TAG_TLS_1_3.data();
+const char *const TS_PROTO_TAG_TLS_1_2       = IP_PROTO_TAG_TLS_1_2.data();
+const char *const TS_PROTO_TAG_TLS_1_1       = IP_PROTO_TAG_TLS_1_1.data();
+const char *const TS_PROTO_TAG_TLS_1_0       = IP_PROTO_TAG_TLS_1_0.data();
+const char *const TS_PROTO_TAG_TCP           = IP_PROTO_TAG_TCP.data();
+const char *const TS_PROTO_TAG_UDP           = IP_PROTO_TAG_UDP.data();
+const char *const TS_PROTO_TAG_IPV4          = IP_PROTO_TAG_IPV4.data();
+const char *const TS_PROTO_TAG_IPV6          = IP_PROTO_TAG_IPV6.data();
 
-InkHashTable *TSProtoTags;
+std::unordered_set<std::string_view> TSProtoTags;
 
 // Precomputed indices for ease of use.
-int TS_ALPN_PROTOCOL_INDEX_HTTP_0_9 = SessionProtocolNameRegistry::INVALID;
-int TS_ALPN_PROTOCOL_INDEX_HTTP_1_0 = SessionProtocolNameRegistry::INVALID;
-int TS_ALPN_PROTOCOL_INDEX_HTTP_1_1 = SessionProtocolNameRegistry::INVALID;
-int TS_ALPN_PROTOCOL_INDEX_HTTP_2_0 = SessionProtocolNameRegistry::INVALID;
+int TS_ALPN_PROTOCOL_INDEX_HTTP_0_9      = SessionProtocolNameRegistry::INVALID;
+int TS_ALPN_PROTOCOL_INDEX_HTTP_1_0      = SessionProtocolNameRegistry::INVALID;
+int TS_ALPN_PROTOCOL_INDEX_HTTP_1_1      = SessionProtocolNameRegistry::INVALID;
+int TS_ALPN_PROTOCOL_INDEX_HTTP_2_0      = SessionProtocolNameRegistry::INVALID;
+int TS_ALPN_PROTOCOL_INDEX_HTTP_3        = SessionProtocolNameRegistry::INVALID;
+int TS_ALPN_PROTOCOL_INDEX_HTTP_QUIC     = SessionProtocolNameRegistry::INVALID;
+int TS_ALPN_PROTOCOL_INDEX_HTTP_3_D27    = SessionProtocolNameRegistry::INVALID;
+int TS_ALPN_PROTOCOL_INDEX_HTTP_QUIC_D27 = SessionProtocolNameRegistry::INVALID;
 
 // Predefined protocol sets for ease of use.
 SessionProtocolSet HTTP_PROTOCOL_SET;
 SessionProtocolSet HTTP2_PROTOCOL_SET;
 SessionProtocolSet DEFAULT_NON_TLS_SESSION_PROTOCOL_SET;
 SessionProtocolSet DEFAULT_TLS_SESSION_PROTOCOL_SET;
+SessionProtocolSet DEFAULT_QUIC_SESSION_PROTOCOL_SET;
+
+static bool
+mptcp_supported()
+{
+  ats_scoped_fd fd(::open("/proc/sys/net/mptcp/mptcp_enabled", O_RDONLY));
+  int value = 0;
+
+  if (fd) {
+    TextBuffer buffer(16);
+
+    buffer.slurp(fd.get());
+    value = atoi(buffer.bufPtr());
+  }
+
+  return value != 0;
+}
 
 void
 RecHttpLoadIp(const char *value_name, IpAddr &ip4, IpAddr &ip6)
@@ -106,6 +137,30 @@ RecHttpLoadIp(const char *value_name, IpAddr &ip4, IpAddr &ip6)
   }
 }
 
+void
+RecHttpLoadIpMap(const char *value_name, IpMap &ipmap)
+{
+  char value[1024];
+  IpAddr laddr;
+  IpAddr raddr;
+  void *payload = nullptr;
+
+  if (REC_ERR_OKAY == RecGetRecordString(value_name, value, sizeof(value))) {
+    Debug("config", "RecHttpLoadIpMap: parsing the name [%s] and value [%s] to an IpMap", value_name, value);
+    Tokenizer tokens(", ");
+    int n_addrs = tokens.Initialize(value);
+    for (int i = 0; i < n_addrs; ++i) {
+      const char *val = tokens[i];
+
+      Debug("config", "RecHttpLoadIpMap: marking the value [%s] to an IpMap entry", val);
+      if (0 == ats_ip_range_parse(val, laddr, raddr)) {
+        ipmap.fill(laddr, raddr, payload);
+      }
+    }
+  }
+  Debug("config", "RecHttpLoadIpMap: parsed %zu IpMap entries", ipmap.count());
+}
+
 const char *const HttpProxyPort::DEFAULT_VALUE = "8080";
 
 const char *const HttpProxyPort::PORTS_CONFIG_NAME = "proxy.config.http.server_ports";
@@ -127,9 +182,12 @@ const char *const HttpProxyPort::OPT_TRANSPARENT_OUTBOUND    = "tr-out";
 const char *const HttpProxyPort::OPT_TRANSPARENT_FULL        = "tr-full";
 const char *const HttpProxyPort::OPT_TRANSPARENT_PASSTHROUGH = "tr-pass";
 const char *const HttpProxyPort::OPT_SSL                     = "ssl";
+const char *const HttpProxyPort::OPT_PROXY_PROTO             = "pp";
 const char *const HttpProxyPort::OPT_PLUGIN                  = "plugin";
 const char *const HttpProxyPort::OPT_BLIND_TUNNEL            = "blind";
 const char *const HttpProxyPort::OPT_COMPRESSED              = "compressed";
+const char *const HttpProxyPort::OPT_MPTCP                   = "mptcp";
+const char *const HttpProxyPort::OPT_QUIC                    = "quic";
 
 // File local constants.
 namespace
@@ -140,7 +198,7 @@ size_t const OPT_OUTBOUND_IP_PREFIX_LEN = strlen(HttpProxyPort::OPT_OUTBOUND_IP_
 size_t const OPT_INBOUND_IP_PREFIX_LEN  = strlen(HttpProxyPort::OPT_INBOUND_IP_PREFIX);
 size_t const OPT_HOST_RES_PREFIX_LEN    = strlen(HttpProxyPort::OPT_HOST_RES_PREFIX);
 size_t const OPT_PROTO_PREFIX_LEN       = strlen(HttpProxyPort::OPT_PROTO_PREFIX);
-}
+} // namespace
 
 namespace
 {
@@ -150,44 +208,44 @@ namespace
 // reference. Might be a problem with Vec<> creating a fixed array
 // rather than allocating on first use (compared to std::vector<>).
 HttpProxyPort::Group GLOBAL_DATA;
-}
+} // namespace
 HttpProxyPort::Group &HttpProxyPort::m_global = GLOBAL_DATA;
 
-HttpProxyPort::HttpProxyPort()
-  : m_fd(ts::NO_FD),
-    m_type(TRANSPORT_DEFAULT),
-    m_port(0),
-    m_family(AF_INET),
-    m_inbound_transparent_p(false),
-    m_outbound_transparent_p(false),
-    m_transparent_passthrough(false)
+HttpProxyPort::HttpProxyPort() : m_fd(ts::NO_FD)
+
 {
   memcpy(m_host_res_preference, host_res_default_preference_order, sizeof(m_host_res_preference));
 }
 
 bool
 HttpProxyPort::hasSSL(Group const &ports)
+{
+  return std::any_of(ports.begin(), ports.end(), [](HttpProxyPort const &port) { return port.isSSL(); });
+}
+
+bool
+HttpProxyPort::hasQUIC(Group const &ports)
 {
   bool zret = false;
-  for (int i = 0, n = ports.length(); i < n && !zret; ++i) {
-    if (ports[i].isSSL()) {
+  for (int i = 0, n = ports.size(); i < n && !zret; ++i) {
+    if (ports[i].isQUIC()) {
       zret = true;
     }
   }
   return zret;
 }
 
-HttpProxyPort *
+const HttpProxyPort *
 HttpProxyPort::findHttp(Group const &ports, uint16_t family)
 {
   bool check_family_p = ats_is_ip(family);
-  self *zret          = nullptr;
-  for (int i = 0, n = ports.length(); i < n && !zret; ++i) {
-    HttpProxyPort &p = ports[i];
+  const self *zret    = nullptr;
+  for (int i = 0, n = ports.size(); i < n && !zret; ++i) {
+    const self &p = ports[i];
     if (p.m_port &&                               // has a valid port
         TRANSPORT_DEFAULT == p.m_type &&          // is normal HTTP
         (!check_family_p || p.m_family == family) // right address family
-        ) {
+    ) {
       zret = &p;
     };
   }
@@ -209,7 +267,7 @@ HttpProxyPort::checkPrefix(const char *src, char const *prefix, size_t prefix_le
 }
 
 bool
-HttpProxyPort::loadConfig(Vec<self> &entries)
+HttpProxyPort::loadConfig(std::vector<self> &entries)
 {
   char *text;
   bool found_p;
@@ -220,23 +278,23 @@ HttpProxyPort::loadConfig(Vec<self> &entries)
   }
   ats_free(text);
 
-  return 0 < entries.length();
+  return 0 < entries.size();
 }
 
 bool
 HttpProxyPort::loadDefaultIfEmpty(Group &ports)
 {
-  if (0 == ports.length()) {
+  if (0 == ports.size()) {
     self::loadValue(ports, DEFAULT_VALUE);
   }
 
-  return 0 < ports.length();
+  return 0 < ports.size();
 }
 
 bool
-HttpProxyPort::loadValue(Vec<self> &ports, const char *text)
+HttpProxyPort::loadValue(std::vector<self> &ports, const char *text)
 {
-  unsigned old_port_length = ports.length(); // remember this.
+  unsigned old_port_length = ports.size(); // remember this.
   if (text && *text) {
     Tokenizer tokens(", ");
     int n_ports = tokens.Initialize(text);
@@ -252,7 +310,7 @@ HttpProxyPort::loadValue(Vec<self> &ports, const char *text)
       }
     }
   }
-  return ports.length() > old_port_length; // we added at least one port.
+  return ports.size() > old_port_length; // we added at least one port.
 }
 
 bool
@@ -265,7 +323,7 @@ HttpProxyPort::processOptions(const char *opts)
   bool bracket_p      = false; // found an open bracket in the input?
   const char *value;           // Temp holder for value of a prefix option.
   IpAddr ip;                   // temp for loading IP addresses.
-  Vec<char *> values;          // Pointers to single option values.
+  std::vector<char *> values;  // Pointers to single option values.
 
   // Make a copy we can modify safely.
   size_t opts_len = strlen(opts) + 1;
@@ -297,8 +355,7 @@ HttpProxyPort::processOptions(const char *opts)
     return zret;
   }
 
-  for (int i = 0, n_items = values.length(); i < n_items; ++i) {
-    const char *item = values[i];
+  for (auto item : values) {
     if (isdigit(item[0])) { // leading digit -> port value
       char *ptr;
       int port = strtoul(item, &ptr, 10);
@@ -344,8 +401,14 @@ HttpProxyPort::processOptions(const char *opts)
       af_set_p = true;
     } else if (0 == strcasecmp(OPT_SSL, item)) {
       m_type = TRANSPORT_SSL;
+#if TS_USE_QUIC == 1
+    } else if (0 == strcasecmp(OPT_QUIC, item)) {
+      m_type = TRANSPORT_QUIC;
+#endif
     } else if (0 == strcasecmp(OPT_PLUGIN, item)) {
       m_type = TRANSPORT_PLUGIN;
+    } else if (0 == strcasecmp(OPT_PROXY_PROTO, item)) {
+      m_proxy_protocol = true;
     } else if (0 == strcasecmp(OPT_TRANSPARENT_INBOUND, item)) {
 #if TS_USE_TPROXY
       m_inbound_transparent_p = true;
@@ -371,6 +434,12 @@ HttpProxyPort::processOptions(const char *opts)
 #else
       Warning("Transparent pass-through requested [%s] in port descriptor '%s' but TPROXY was not configured.", item, opts);
 #endif
+    } else if (0 == strcasecmp(OPT_MPTCP, item)) {
+      if (mptcp_supported()) {
+        m_mptcp = true;
+      } else {
+        Warning("Multipath TCP requested [%s] in port descriptor '%s' but it is not supported by this host.", item, opts);
+      }
     } else if (nullptr != (value = this->checkPrefix(item, OPT_HOST_RES_PREFIX, OPT_HOST_RES_PREFIX_LEN))) {
       this->processFamilyPreference(value);
       host_res_set_p = true;
@@ -378,7 +447,7 @@ HttpProxyPort::processOptions(const char *opts)
       this->processSessionProtocolPreference(value);
       sp_set_p = true;
     } else {
-      Warning("Invalid option '%s' in proxy port configuration '%s'", item, opts);
+      Warning("Invalid option '%s' in proxy port descriptor '%s'", item, opts);
     }
   }
 
@@ -386,9 +455,11 @@ HttpProxyPort::processOptions(const char *opts)
 
   if (af_set_p) {
     if (in_ip_set_p && m_family != m_inbound_ip.family()) {
-      Warning(
-        "Invalid port descriptor '%s' - the inbound adddress family [%s] is not the same type as the explicit family value [%s]",
-        opts, ats_ip_family_name(m_inbound_ip.family()).ptr(), ats_ip_family_name(m_family).ptr());
+      std::string_view iname{ats_ip_family_name(m_inbound_ip.family())};
+      std::string_view fname{ats_ip_family_name(m_family)};
+      Warning("Invalid port descriptor '%s' - the inbound address family [%.*s] is not the same type as the explicit family value "
+              "[%.*s]",
+              opts, static_cast<int>(iname.size()), iname.data(), static_cast<int>(fname.size()), fname.data());
       zret = false;
     }
   } else if (in_ip_set_p) {
@@ -415,7 +486,13 @@ HttpProxyPort::processOptions(const char *opts)
 
   // Set the default session protocols.
   if (!sp_set_p) {
-    m_session_protocol_preference = this->isSSL() ? DEFAULT_TLS_SESSION_PROTOCOL_SET : DEFAULT_NON_TLS_SESSION_PROTOCOL_SET;
+    if (this->isSSL()) {
+      m_session_protocol_preference = DEFAULT_TLS_SESSION_PROTOCOL_SET;
+    } else if (this->isQUIC()) {
+      m_session_protocol_preference = DEFAULT_QUIC_SESSION_PROTOCOL_SET;
+    } else {
+      m_session_protocol_preference = DEFAULT_NON_TLS_SESSION_PROTOCOL_SET;
+    }
   }
 
   return zret;
@@ -451,7 +528,7 @@ SessionProtocolNameRegistry::markIn(const char *value, SessionProtocolSet &sp_se
     } else if (0 == strcasecmp(elt, TS_ALPN_PROTOCOL_GROUP_HTTP2)) {
       sp_set.markIn(HTTP2_PROTOCOL_SET);
     } else { // user defined - register and mark.
-      int idx = globalSessionProtocolNameRegistry.toIndex(elt);
+      int idx = globalSessionProtocolNameRegistry.toIndex(TextView{elt, strlen(elt)});
       sp_set.markIn(idx);
     }
   }
@@ -529,6 +606,8 @@ HttpProxyPort::print(char *out, size_t n)
     zret += snprintf(out + zret, n - zret, ":%s", OPT_BLIND_TUNNEL);
   } else if (TRANSPORT_SSL == m_type) {
     zret += snprintf(out + zret, n - zret, ":%s", OPT_SSL);
+  } else if (TRANSPORT_QUIC == m_type) {
+    zret += snprintf(out + zret, n - zret, ":%s", OPT_QUIC);
   } else if (TRANSPORT_PLUGIN == m_type) {
     zret += snprintf(out + zret, n - zret, ":%s", OPT_PLUGIN);
   } else if (TRANSPORT_COMPRESSED == m_type) {
@@ -538,6 +617,10 @@ HttpProxyPort::print(char *out, size_t n)
     return n;
   }
 
+  if (m_proxy_protocol) {
+    zret += snprintf(out + zret, n - zret, ":%s", OPT_PROXY_PROTO);
+  }
+
   if (m_outbound_transparent_p && m_inbound_transparent_p) {
     zret += snprintf(out + zret, n - zret, ":%s", OPT_TRANSPARENT_FULL);
   } else if (m_inbound_transparent_p) {
@@ -546,6 +629,10 @@ HttpProxyPort::print(char *out, size_t n)
     zret += snprintf(out + zret, n - zret, ":%s", OPT_TRANSPARENT_OUTBOUND);
   }
 
+  if (m_mptcp) {
+    zret += snprintf(out + zret, n - zret, ":%s", OPT_MPTCP);
+  }
+
   if (m_transparent_passthrough) {
     zret += snprintf(out + zret, n - zret, ":%s", OPT_TRANSPARENT_PASSTHROUGH);
   }
@@ -568,6 +655,8 @@ HttpProxyPort::print(char *out, size_t n)
     sp_set.markOut(DEFAULT_NON_TLS_SESSION_PROTOCOL_SET);
   } else if (sp_set == DEFAULT_TLS_SESSION_PROTOCOL_SET && this->isSSL()) {
     sp_set.markOut(DEFAULT_TLS_SESSION_PROTOCOL_SET);
+  } else if (sp_set == DEFAULT_QUIC_SESSION_PROTOCOL_SET && this->isQUIC()) {
+    sp_set.markOut(DEFAULT_QUIC_SESSION_PROTOCOL_SET);
   }
 
   // pull out groups.
@@ -594,13 +683,14 @@ HttpProxyPort::print(char *out, size_t n)
     bool sep_p = !need_colon_p;
     for (int k = 0; k < SessionProtocolSet::MAX; ++k) {
       if (sp_set.contains(k)) {
-        zret += snprintf(out + zret, n - zret, "%s%s", sep_p ? ";" : "", globalSessionProtocolNameRegistry.nameFor(k));
+        auto name{globalSessionProtocolNameRegistry.nameFor(k)};
+        zret += snprintf(out + zret, n - zret, "%s%.*s", sep_p ? ";" : "", static_cast<int>(name.size()), name.data());
         sep_p = true;
       }
     }
   }
 
-  return min(zret, n);
+  return std::min(zret, n);
 }
 
 void
@@ -621,10 +711,15 @@ void
 ts_session_protocol_well_known_name_indices_init()
 {
   // register all the well known protocols and get the indices set.
-  TS_ALPN_PROTOCOL_INDEX_HTTP_0_9 = globalSessionProtocolNameRegistry.toIndexConst(TS_ALPN_PROTOCOL_HTTP_0_9);
-  TS_ALPN_PROTOCOL_INDEX_HTTP_1_0 = globalSessionProtocolNameRegistry.toIndexConst(TS_ALPN_PROTOCOL_HTTP_1_0);
-  TS_ALPN_PROTOCOL_INDEX_HTTP_1_1 = globalSessionProtocolNameRegistry.toIndexConst(TS_ALPN_PROTOCOL_HTTP_1_1);
-  TS_ALPN_PROTOCOL_INDEX_HTTP_2_0 = globalSessionProtocolNameRegistry.toIndexConst(TS_ALPN_PROTOCOL_HTTP_2_0);
+  TS_ALPN_PROTOCOL_INDEX_HTTP_0_9   = globalSessionProtocolNameRegistry.toIndexConst(std::string_view{TS_ALPN_PROTOCOL_HTTP_0_9});
+  TS_ALPN_PROTOCOL_INDEX_HTTP_1_0   = globalSessionProtocolNameRegistry.toIndexConst(std::string_view{TS_ALPN_PROTOCOL_HTTP_1_0});
+  TS_ALPN_PROTOCOL_INDEX_HTTP_1_1   = globalSessionProtocolNameRegistry.toIndexConst(std::string_view{TS_ALPN_PROTOCOL_HTTP_1_1});
+  TS_ALPN_PROTOCOL_INDEX_HTTP_2_0   = globalSessionProtocolNameRegistry.toIndexConst(std::string_view{TS_ALPN_PROTOCOL_HTTP_2_0});
+  TS_ALPN_PROTOCOL_INDEX_HTTP_3     = globalSessionProtocolNameRegistry.toIndexConst(std::string_view{TS_ALPN_PROTOCOL_HTTP_3});
+  TS_ALPN_PROTOCOL_INDEX_HTTP_3_D27 = globalSessionProtocolNameRegistry.toIndexConst(std::string_view{TS_ALPN_PROTOCOL_HTTP_3_D27});
+  TS_ALPN_PROTOCOL_INDEX_HTTP_QUIC  = globalSessionProtocolNameRegistry.toIndexConst(std::string_view{TS_ALPN_PROTOCOL_HTTP_QUIC});
+  TS_ALPN_PROTOCOL_INDEX_HTTP_QUIC_D27 =
+    globalSessionProtocolNameRegistry.toIndexConst(std::string_view{TS_ALPN_PROTOCOL_HTTP_QUIC_D27});
 
   // Now do the predefined protocol sets.
   HTTP_PROTOCOL_SET.markIn(TS_ALPN_PROTOCOL_INDEX_HTTP_0_9);
@@ -633,59 +728,52 @@ ts_session_protocol_well_known_name_indices_init()
   HTTP2_PROTOCOL_SET.markIn(TS_ALPN_PROTOCOL_INDEX_HTTP_2_0);
 
   DEFAULT_TLS_SESSION_PROTOCOL_SET.markAllIn();
+  DEFAULT_TLS_SESSION_PROTOCOL_SET.markOut(TS_ALPN_PROTOCOL_INDEX_HTTP_3);
+  DEFAULT_TLS_SESSION_PROTOCOL_SET.markOut(TS_ALPN_PROTOCOL_INDEX_HTTP_QUIC);
+
+  DEFAULT_QUIC_SESSION_PROTOCOL_SET.markIn(TS_ALPN_PROTOCOL_INDEX_HTTP_3);
+  DEFAULT_QUIC_SESSION_PROTOCOL_SET.markIn(TS_ALPN_PROTOCOL_INDEX_HTTP_QUIC);
+  DEFAULT_QUIC_SESSION_PROTOCOL_SET.markIn(TS_ALPN_PROTOCOL_INDEX_HTTP_3_D27);
+  DEFAULT_QUIC_SESSION_PROTOCOL_SET.markIn(TS_ALPN_PROTOCOL_INDEX_HTTP_QUIC_D27);
 
   DEFAULT_NON_TLS_SESSION_PROTOCOL_SET = HTTP_PROTOCOL_SET;
 
-  TSProtoTags = ink_hash_table_create(InkHashTableKeyType_String);
-  ink_hash_table_insert(TSProtoTags, TS_PROTO_TAG_HTTP_1_0, reinterpret_cast<void *>(const_cast<char *>(TS_PROTO_TAG_HTTP_1_0)));
-  ink_hash_table_insert(TSProtoTags, TS_PROTO_TAG_HTTP_1_1, reinterpret_cast<void *>(const_cast<char *>(TS_PROTO_TAG_HTTP_1_1)));
-  ink_hash_table_insert(TSProtoTags, TS_PROTO_TAG_HTTP_2_0, reinterpret_cast<void *>(const_cast<char *>(TS_PROTO_TAG_HTTP_2_0)));
-  ink_hash_table_insert(TSProtoTags, TS_PROTO_TAG_TLS_1_3, reinterpret_cast<void *>(const_cast<char *>(TS_PROTO_TAG_TLS_1_3)));
-  ink_hash_table_insert(TSProtoTags, TS_PROTO_TAG_TLS_1_2, reinterpret_cast<void *>(const_cast<char *>(TS_PROTO_TAG_TLS_1_2)));
-  ink_hash_table_insert(TSProtoTags, TS_PROTO_TAG_TLS_1_1, reinterpret_cast<void *>(const_cast<char *>(TS_PROTO_TAG_TLS_1_1)));
-  ink_hash_table_insert(TSProtoTags, TS_PROTO_TAG_TLS_1_0, reinterpret_cast<void *>(const_cast<char *>(TS_PROTO_TAG_TLS_1_0)));
-  ink_hash_table_insert(TSProtoTags, TS_PROTO_TAG_TCP, reinterpret_cast<void *>(const_cast<char *>(TS_PROTO_TAG_TCP)));
-  ink_hash_table_insert(TSProtoTags, TS_PROTO_TAG_UDP, reinterpret_cast<void *>(const_cast<char *>(TS_PROTO_TAG_UDP)));
-  ink_hash_table_insert(TSProtoTags, TS_PROTO_TAG_IPV4, reinterpret_cast<void *>(const_cast<char *>(TS_PROTO_TAG_IPV4)));
-  ink_hash_table_insert(TSProtoTags, TS_PROTO_TAG_IPV6, reinterpret_cast<void *>(const_cast<char *>(TS_PROTO_TAG_IPV6)));
+  TSProtoTags.insert(TS_PROTO_TAG_HTTP_1_0);
+  TSProtoTags.insert(TS_PROTO_TAG_HTTP_1_1);
+  TSProtoTags.insert(TS_PROTO_TAG_HTTP_2_0);
+  TSProtoTags.insert(TS_PROTO_TAG_HTTP_3);
+  TSProtoTags.insert(TS_PROTO_TAG_HTTP_QUIC);
+  TSProtoTags.insert(TS_PROTO_TAG_HTTP_3_D27);
+  TSProtoTags.insert(TS_PROTO_TAG_HTTP_QUIC_D27);
+  TSProtoTags.insert(TS_PROTO_TAG_TLS_1_3);
+  TSProtoTags.insert(TS_PROTO_TAG_TLS_1_2);
+  TSProtoTags.insert(TS_PROTO_TAG_TLS_1_1);
+  TSProtoTags.insert(TS_PROTO_TAG_TLS_1_0);
+  TSProtoTags.insert(TS_PROTO_TAG_TCP);
+  TSProtoTags.insert(TS_PROTO_TAG_UDP);
+  TSProtoTags.insert(TS_PROTO_TAG_IPV4);
+  TSProtoTags.insert(TS_PROTO_TAG_IPV6);
 }
 
 const char *
 RecNormalizeProtoTag(const char *tag)
 {
-  InkHashTableValue value;
-
-  if (ink_hash_table_lookup(TSProtoTags, tag, &value)) {
-    return reinterpret_cast<const char *>(value);
-  }
-
-  return nullptr;
-}
-
-SessionProtocolNameRegistry::SessionProtocolNameRegistry() : m_n(0)
-{
-  memset(m_names, 0, sizeof(m_names));
-  memset(&m_flags, 0, sizeof(m_flags));
-}
-
-SessionProtocolNameRegistry::~SessionProtocolNameRegistry()
-{
-  for (size_t i = 0; i < m_n; ++i) {
-    if (m_flags[i] & F_ALLOCATED) {
-      ats_free(const_cast<char *>(m_names[i])); // blech - ats_free won't take a const char*
-    }
-  }
+  auto findResult = TSProtoTags.find(tag);
+  return findResult == TSProtoTags.end() ? nullptr : findResult->data();
 }
 
 int
-SessionProtocolNameRegistry::toIndex(const char *name)
+SessionProtocolNameRegistry::toIndex(ts::TextView name)
 {
   int zret = this->indexFor(name);
   if (INVALID == zret) {
-    if (m_n < static_cast<size_t>(MAX)) {
-      m_names[m_n] = ats_strdup(name);
-      m_flags[m_n] = F_ALLOCATED;
-      zret         = m_n++;
+    if (m_n < MAX) {
+      // Localize the name by copying it in to the arena.
+      auto text = m_arena.alloc(name.size() + 1).rebind<char>();
+      memcpy(text.data(), name.data(), name.size());
+      text.end()[-1] = '\0';
+      m_names[m_n]   = text.view();
+      zret           = m_n++;
     } else {
       ink_release_assert(!"Session protocol name registry overflow");
     }
@@ -694,11 +782,11 @@ SessionProtocolNameRegistry::toIndex(const char *name)
 }
 
 int
-SessionProtocolNameRegistry::toIndexConst(const char *name)
+SessionProtocolNameRegistry::toIndexConst(TextView name)
 {
   int zret = this->indexFor(name);
   if (INVALID == zret) {
-    if (m_n < static_cast<size_t>(MAX)) {
+    if (m_n < MAX) {
       m_names[m_n] = name;
       zret         = m_n++;
     } else {
@@ -709,18 +797,18 @@ SessionProtocolNameRegistry::toIndexConst(const char *name)
 }
 
 int
-SessionProtocolNameRegistry::indexFor(const char *name) const
+SessionProtocolNameRegistry::indexFor(TextView name) const
 {
-  for (size_t i = 0; i < m_n; ++i) {
-    if (0 == strcasecmp(name, m_names[i])) {
-      return i;
-    }
+  const ts::TextView *end = m_names.begin() + m_n;
+  auto spot               = std::find(m_names.begin(), end, name);
+  if (spot != end) {
+    return static_cast<int>(spot - m_names.begin());
   }
   return INVALID;
 }
 
-const char *
+ts::TextView
 SessionProtocolNameRegistry::nameFor(int idx) const
 {
-  return 0 <= idx && idx < static_cast<int>(m_n) ? m_names[idx] : nullptr;
+  return 0 <= idx && idx < m_n ? m_names[idx] : TextView{};
 }
