@@ -1,6 +1,6 @@
 /** @file
 
-  A brief file description
+  Functions for interfacing to management records
 
   @section license License
 
@@ -21,24 +21,12 @@
   limitations under the License.
  */
 
-#include "ts/ink_platform.h"
-#include "ts/ink_string.h"
-#include "ts/Tokenizer.h"
-#include "ts/ink_code.h"
-#include "ts/ink_file.h"
-#include "LocalManager.h"
+#include "tscore/ink_string.h"
+#include "tscore/Tokenizer.h"
+#include "tscore/ink_file.h"
 #include "MgmtUtils.h"
 #include "WebMgmtUtils.h"
-#include "MultiFile.h"
-#include "ts/Regex.h"
-
-/****************************************************************************
- *
- *  WebMgmtUtils.cc - Functions for interfacing to management records
- *
- *
- *
- ****************************************************************************/
+#include "tscore/Regex.h"
 
 // bool varSetFromStr(const char*, const char* )
 //
@@ -60,28 +48,28 @@ varSetFromStr(const char *varName, const char *value)
 
   memset(&data, 0, sizeof(RecData));
 
-  err = RecGetRecordDataType((char *)varName, &varDataType);
+  err = RecGetRecordDataType(const_cast<char *>(varName), &varDataType);
   if (err != REC_ERR_OKAY) {
     return found;
   }
   // Use any empty string if we get a NULL so
   //  sprintf does puke.  However, we need to
   //  switch this back to NULL for STRING types
-  if (value == NULL) {
+  if (value == nullptr) {
     value = "";
   }
 
   switch (varDataType) {
   case RECD_INT:
     if (sscanf(value, "%" PRId64 "", &data.rec_int) == 1) {
-      RecSetRecordInt((char *)varName, data.rec_int, REC_SOURCE_EXPLICIT);
+      RecSetRecordInt(const_cast<char *>(varName), data.rec_int, REC_SOURCE_EXPLICIT);
     } else {
       found = false;
     }
     break;
   case RECD_COUNTER:
     if (sscanf(value, "%" PRId64 "", &data.rec_counter) == 1) {
-      RecSetRecordCounter((char *)varName, data.rec_counter, REC_SOURCE_EXPLICIT);
+      RecSetRecordCounter(const_cast<char *>(varName), data.rec_counter, REC_SOURCE_EXPLICIT);
     } else {
       found = false;
     }
@@ -89,16 +77,16 @@ varSetFromStr(const char *varName, const char *value)
   case RECD_FLOAT:
     // coverity[secure_coding]
     if (sscanf(value, "%f", &data.rec_float) == 1) {
-      RecSetRecordFloat((char *)varName, data.rec_float, REC_SOURCE_EXPLICIT);
+      RecSetRecordFloat(const_cast<char *>(varName), data.rec_float, REC_SOURCE_EXPLICIT);
     } else {
       found = false;
     }
     break;
   case RECD_STRING:
     if (*value == '\0') {
-      RecSetRecordString((char *)varName, NULL, REC_SOURCE_EXPLICIT);
+      RecSetRecordString(const_cast<char *>(varName), nullptr, REC_SOURCE_EXPLICIT);
     } else {
-      RecSetRecordString((char *)varName, (char *)value, REC_SOURCE_EXPLICIT);
+      RecSetRecordString(const_cast<char *>(varName), const_cast<char *>(value), REC_SOURCE_EXPLICIT);
     }
     break;
   case RECD_NULL:
@@ -112,11 +100,11 @@ varSetFromStr(const char *varName, const char *value)
 
 // bool varSetFloat(const char* varName, RecFloat value)
 //
-//  Sets the variable specifed by varName to value.  varName
+//  Sets the variable specified by varName to value.  varName
 //   must be a RecFloat variable.  No conversion is done for
 //   other types unless convert is set to ture. In the case
 //   of convert is ture, type conversion is perform if applicable.
-//   By default, convert is set to be false and can be overrided
+//   By default, convert is set to be false and can be overridden
 //   when the function is called.
 //
 bool
@@ -126,26 +114,31 @@ varSetFloat(const char *varName, RecFloat value, bool convert)
   bool found           = true;
   int err              = REC_ERR_FAIL;
 
-  err = RecGetRecordDataType((char *)varName, &varDataType);
+  err = RecGetRecordDataType(const_cast<char *>(varName), &varDataType);
   if (err != REC_ERR_OKAY) {
     return found;
   }
 
   switch (varDataType) {
   case RECD_FLOAT:
-    RecSetRecordFloat((char *)varName, (RecFloat)value, REC_SOURCE_EXPLICIT);
+    RecSetRecordFloat(const_cast<char *>(varName), value, REC_SOURCE_EXPLICIT);
     break;
+
   case RECD_INT:
     if (convert) {
       value += 0.5; // rounding up
-      RecSetRecordInt((char *)varName, (RecInt)value, REC_SOURCE_EXPLICIT);
+      RecSetRecordInt(const_cast<char *>(varName), static_cast<RecInt>(value), REC_SOURCE_EXPLICIT);
       break;
     }
+    // fallthrough
+
   case RECD_COUNTER:
     if (convert) {
-      RecSetRecordCounter((char *)varName, (RecCounter)value, REC_SOURCE_EXPLICIT);
+      RecSetRecordCounter(const_cast<char *>(varName), static_cast<RecCounter>(value), REC_SOURCE_EXPLICIT);
       break;
     }
+    // fallthrough
+
   case RECD_STRING:
   case RECD_NULL:
   default:
@@ -158,11 +151,11 @@ varSetFloat(const char *varName, RecFloat value, bool convert)
 
 // bool varSetCounter(const char* varName, RecCounter value)
 //
-//  Sets the variable specifed by varName to value.  varName
+//  Sets the variable specified by varName to value.  varName
 //   must be an RecCounter variable.  No conversion is done for
 //   other types unless convert is set to ture. In the case
 //   of convert is ture, type conversion is perform if applicable.
-//   By default, convert is set to be false and can be overrided
+//   By default, convert is set to be false and can be overridden
 //   when the function is called.
 //
 bool
@@ -172,25 +165,30 @@ varSetCounter(const char *varName, RecCounter value, bool convert)
   bool found           = true;
   int err              = REC_ERR_FAIL;
 
-  err = RecGetRecordDataType((char *)varName, &varDataType);
+  err = RecGetRecordDataType(const_cast<char *>(varName), &varDataType);
   if (err != REC_ERR_OKAY) {
     return found;
   }
 
   switch (varDataType) {
   case RECD_COUNTER:
-    RecSetRecordCounter((char *)varName, (RecCounter)value, REC_SOURCE_EXPLICIT);
+    RecSetRecordCounter(const_cast<char *>(varName), value, REC_SOURCE_EXPLICIT);
     break;
+
   case RECD_INT:
     if (convert) {
-      RecSetRecordInt((char *)varName, (RecInt)value, REC_SOURCE_EXPLICIT);
+      RecSetRecordInt(const_cast<char *>(varName), static_cast<RecInt>(value), REC_SOURCE_EXPLICIT);
       break;
     }
+    // fallthrough
+
   case RECD_FLOAT:
     if (convert) {
-      RecSetRecordFloat((char *)varName, (RecFloat)value, REC_SOURCE_EXPLICIT);
+      RecSetRecordFloat(const_cast<char *>(varName), static_cast<RecFloat>(value), REC_SOURCE_EXPLICIT);
       break;
     }
+    // fallthrough
+
   case RECD_STRING:
   case RECD_NULL:
   default:
@@ -203,11 +201,11 @@ varSetCounter(const char *varName, RecCounter value, bool convert)
 
 // bool varSetInt(const char* varName, RecInt value)
 //
-//  Sets the variable specifed by varName to value.  varName
+//  Sets the variable specified by varName to value.  varName
 //   must be an RecInt variable.  No conversion is done for
 //   other types unless convert is set to ture. In the case
 //   of convert is ture, type conversion is perform if applicable.
-//   By default, convert is set to be false and can be overrided
+//   By default, convert is set to be false and can be overridden
 //   when the function is called.
 //
 bool
@@ -217,25 +215,30 @@ varSetInt(const char *varName, RecInt value, bool convert)
   bool found           = true;
   int err              = REC_ERR_FAIL;
 
-  err = RecGetRecordDataType((char *)varName, &varDataType);
+  err = RecGetRecordDataType(const_cast<char *>(varName), &varDataType);
   if (err != REC_ERR_OKAY) {
     return found;
   }
 
   switch (varDataType) {
   case RECD_INT:
-    RecSetRecordInt((char *)varName, (RecInt)value, REC_SOURCE_EXPLICIT);
+    RecSetRecordInt(const_cast<char *>(varName), value, REC_SOURCE_EXPLICIT);
     break;
+
   case RECD_COUNTER:
     if (convert) {
-      RecSetRecordCounter((char *)varName, (RecCounter)value, REC_SOURCE_EXPLICIT);
+      RecSetRecordCounter(const_cast<char *>(varName), static_cast<RecCounter>(value), REC_SOURCE_EXPLICIT);
       break;
     }
+    // fallthrough
+
   case RECD_FLOAT:
     if (convert) {
-      RecSetRecordFloat((char *)varName, (RecFloat)value, REC_SOURCE_EXPLICIT);
+      RecSetRecordFloat(const_cast<char *>(varName), static_cast<RecFloat>(value), REC_SOURCE_EXPLICIT);
       break;
     }
+    // fallthrough
+
   case RECD_STRING:
   case RECD_NULL:
   default:
@@ -248,7 +251,7 @@ varSetInt(const char *varName, RecInt value, bool convert)
 
 // bool varSetData(RecDataT varType, const char *varName, RecData value)
 //
-//  Sets the variable specifed by varName to value. value and varName
+//  Sets the variable specified by varName to value. value and varName
 //   must be varType variables.
 //
 bool
@@ -258,13 +261,13 @@ varSetData(RecDataT varType, const char *varName, RecData value)
 
   switch (varType) {
   case RECD_INT:
-    err = RecSetRecordInt((char *)varName, value.rec_int, REC_SOURCE_EXPLICIT);
+    err = RecSetRecordInt(const_cast<char *>(varName), value.rec_int, REC_SOURCE_EXPLICIT);
     break;
   case RECD_COUNTER:
-    err = RecSetRecordCounter((char *)varName, value.rec_counter, REC_SOURCE_EXPLICIT);
+    err = RecSetRecordCounter(const_cast<char *>(varName), value.rec_counter, REC_SOURCE_EXPLICIT);
     break;
   case RECD_FLOAT:
-    err = RecSetRecordFloat((char *)varName, value.rec_float, REC_SOURCE_EXPLICIT);
+    err = RecSetRecordFloat(const_cast<char *>(varName), value.rec_float, REC_SOURCE_EXPLICIT);
     break;
   default:
     Fatal("unsupport type:%d\n", varType);
@@ -276,7 +279,7 @@ varSetData(RecDataT varType, const char *varName, RecData value)
 //
 //   Sets the *value to value of the varName according varType.
 //
-//  return true if bufVal was succefully set
+//  return true if bufVal was successfully set
 //    and false otherwise
 //
 bool
@@ -293,7 +296,7 @@ varDataFromName(RecDataT varType, const char *varName, RecData *value)
 //
 //   Sets the *value to value of the varName.
 //
-//  return true if bufVal was succefully set
+//  return true if bufVal was successfully set
 //    and false otherwise
 //
 bool
@@ -303,7 +306,7 @@ varCounterFromName(const char *varName, RecCounter *value)
   bool found           = true;
   int err              = REC_ERR_FAIL;
 
-  err = RecGetRecordDataType((char *)varName, &varDataType);
+  err = RecGetRecordDataType(const_cast<char *>(varName), &varDataType);
 
   if (err == REC_ERR_FAIL) {
     return false;
@@ -312,19 +315,19 @@ varCounterFromName(const char *varName, RecCounter *value)
   switch (varDataType) {
   case RECD_INT: {
     RecInt tempInt = 0;
-    RecGetRecordInt((char *)varName, &tempInt);
-    *value = (RecCounter)tempInt;
+    RecGetRecordInt(const_cast<char *>(varName), &tempInt);
+    *value = static_cast<RecCounter>(tempInt);
     break;
   }
   case RECD_COUNTER: {
     *value = 0;
-    RecGetRecordCounter((char *)varName, value);
+    RecGetRecordCounter(const_cast<char *>(varName), value);
     break;
   }
   case RECD_FLOAT: {
     RecFloat tempFloat = 0.0;
-    RecGetRecordFloat((char *)varName, &tempFloat);
-    *value = (RecCounter)tempFloat;
+    RecGetRecordFloat(const_cast<char *>(varName), &tempFloat);
+    *value = static_cast<RecCounter>(tempFloat);
     break;
   }
   case RECD_STRING:
@@ -342,7 +345,7 @@ varCounterFromName(const char *varName, RecCounter *value)
 //
 //   Sets the *value to value of the varName.
 //
-//  return true if bufVal was succefully set
+//  return true if bufVal was successfully set
 //    and false otherwise
 //
 bool
@@ -353,7 +356,7 @@ varFloatFromName(const char *varName, RecFloat *value)
 
   int err = REC_ERR_FAIL;
 
-  err = RecGetRecordDataType((char *)varName, &varDataType);
+  err = RecGetRecordDataType(const_cast<char *>(varName), &varDataType);
 
   if (err == REC_ERR_FAIL) {
     return false;
@@ -362,19 +365,19 @@ varFloatFromName(const char *varName, RecFloat *value)
   switch (varDataType) {
   case RECD_INT: {
     RecInt tempInt = 0;
-    RecGetRecordInt((char *)varName, &tempInt);
-    *value = (RecFloat)tempInt;
+    RecGetRecordInt(const_cast<char *>(varName), &tempInt);
+    *value = static_cast<RecFloat>(tempInt);
     break;
   }
   case RECD_COUNTER: {
     RecCounter tempCounter = 0;
-    RecGetRecordCounter((char *)varName, &tempCounter);
-    *value = (RecFloat)tempCounter;
+    RecGetRecordCounter(const_cast<char *>(varName), &tempCounter);
+    *value = static_cast<RecFloat>(tempCounter);
     break;
   }
   case RECD_FLOAT: {
     *value = 0.0;
-    RecGetRecordFloat((char *)varName, value);
+    RecGetRecordFloat(const_cast<char *>(varName), value);
     break;
   }
   case RECD_STRING:
@@ -392,7 +395,7 @@ varFloatFromName(const char *varName, RecFloat *value)
 //
 //   Sets the *value to value of the varName.
 //
-//  return true if bufVal was succefully set
+//  return true if bufVal was successfully set
 //    and false otherwise
 //
 bool
@@ -402,7 +405,7 @@ varIntFromName(const char *varName, RecInt *value)
   bool found           = true;
   int err              = REC_ERR_FAIL;
 
-  err = RecGetRecordDataType((char *)varName, &varDataType);
+  err = RecGetRecordDataType(const_cast<char *>(varName), &varDataType);
 
   if (err != REC_ERR_OKAY) {
     return false;
@@ -411,19 +414,19 @@ varIntFromName(const char *varName, RecInt *value)
   switch (varDataType) {
   case RECD_INT: {
     *value = 0;
-    RecGetRecordInt((char *)varName, value);
+    RecGetRecordInt(const_cast<char *>(varName), value);
     break;
   }
   case RECD_COUNTER: {
     RecCounter tempCounter = 0;
-    RecGetRecordCounter((char *)varName, &tempCounter);
-    *value = (RecInt)tempCounter;
+    RecGetRecordCounter(const_cast<char *>(varName), &tempCounter);
+    *value = static_cast<RecInt>(tempCounter);
     break;
   }
   case RECD_FLOAT: {
     RecFloat tempFloat = 0.0;
-    RecGetRecordFloat((char *)varName, &tempFloat);
-    *value = (RecInt)tempFloat;
+    RecGetRecordFloat(const_cast<char *>(varName), &tempFloat);
+    *value = static_cast<RecInt>(tempFloat);
     break;
   }
   case RECD_STRING:
@@ -448,7 +451,7 @@ percentStrFromFloat(RecFloat val, char *bufVal)
 {
   int percent;
 
-  percent = (int)((val * 100.0) + 0.5);
+  percent = static_cast<int>((val * 100.0) + 0.5);
   snprintf(bufVal, 4, "%d%%", percent);
 }
 
@@ -516,11 +519,10 @@ bytesFromInt(RecInt bytes, char *bufVal)
   const int64_t gb  = 1073741824;
   const long int mb = 1048576;
   const long int kb = 1024;
-  int bytesP;
   double unitBytes;
 
   if (bytes >= gb) {
-    unitBytes = bytes / (double)gb;
+    unitBytes = bytes / static_cast<double>(gb);
     snprintf(bufVal, 15, "%.1f GB", unitBytes);
   } else {
     // Reduce the precision of the bytes parameter
@@ -528,12 +530,12 @@ bytesFromInt(RecInt bytes, char *bufVal)
     //   has plenty of precision for a regular int
     //   and saves from 64 bit arithmetic which may
     //   be expensive on some processors
-    bytesP = (int)bytes;
+    int bytesP = static_cast<int>(bytes);
     if (bytesP >= mb) {
-      unitBytes = bytes / (double)mb;
+      unitBytes = bytes / static_cast<double>(mb);
       snprintf(bufVal, 15, "%.1f MB", unitBytes);
     } else if (bytesP >= kb) {
-      unitBytes = bytes / (double)kb;
+      unitBytes = bytes / static_cast<double>(kb);
       snprintf(bufVal, 15, "%.1f KB", unitBytes);
     } else {
       snprintf(bufVal, 15, "%d", bytesP);
@@ -546,19 +548,19 @@ bytesFromInt(RecInt bytes, char *bufVal)
 //   Sets the bufVal string to the value of the local manager
 //     named by varName.  bufLen is size of bufVal
 //
-//  return true if bufVal was succefully set
+//  return true if bufVal was successfully set
 //    and false otherwise
 //
 //  EVIL ALERT: overviewRecord::varStrFromName is extremely
 //    similar to this function except in how it gets it's
-//    data.  Changes to this fuction must be propogated
+//    data.  Changes to this function must be propagated
 //    to its twin.  Cut and Paste sucks but there is not
 //    an easy way to merge the functions
 //
 bool
 varStrFromName(const char *varNameConst, char *bufVal, int bufLen)
 {
-  char *varName        = NULL;
+  char *varName        = nullptr;
   RecDataT varDataType = RECD_NULL;
   bool found           = true;
   int varNameLen       = 0;
@@ -614,9 +616,9 @@ varStrFromName(const char *varNameConst, char *bufVal, int bufLen)
   case RECD_COUNTER:
     RecGetRecordCounter(varName, &data.rec_counter);
     if (formatOption == 'b') {
-      bytesFromInt((MgmtInt)data.rec_counter, bufVal);
+      bytesFromInt(static_cast<MgmtInt>(data.rec_counter), bufVal);
     } else if (formatOption == 'm') {
-      MbytesFromInt((MgmtInt)data.rec_counter, bufVal);
+      MbytesFromInt(static_cast<MgmtInt>(data.rec_counter), bufVal);
     } else if (formatOption == 'c') {
       commaStrFromInt(data.rec_counter, bufVal);
     } else {
@@ -633,9 +635,9 @@ varStrFromName(const char *varNameConst, char *bufVal, int bufLen)
     break;
   case RECD_STRING:
     RecGetRecordString_Xmalloc(varName, &data.rec_string);
-    if (data.rec_string == NULL) {
+    if (data.rec_string == nullptr) {
       bufVal[0] = '\0';
-    } else if (strlen(data.rec_string) < (size_t)(bufLen - 1)) {
+    } else if (strlen(data.rec_string) < static_cast<size_t>(bufLen - 1)) {
       ink_strlcpy(bufVal, data.rec_string, bufLen);
     } else {
       ink_strlcpy(bufVal, data.rec_string, bufLen);
@@ -665,7 +667,7 @@ MgmtData::setFromName(const char *varName)
   bool found = true;
   int err;
 
-  err = RecGetRecordDataType((char *)varName, &this->type);
+  err = RecGetRecordDataType(const_cast<char *>(varName), &this->type);
 
   if (err == REC_ERR_FAIL) {
     return found;
@@ -673,16 +675,16 @@ MgmtData::setFromName(const char *varName)
 
   switch (this->type) {
   case RECD_INT:
-    RecGetRecordInt((char *)varName, &this->data.rec_int);
+    RecGetRecordInt(const_cast<char *>(varName), &this->data.rec_int);
     break;
   case RECD_COUNTER:
-    RecGetRecordCounter((char *)varName, &this->data.rec_counter);
+    RecGetRecordCounter(const_cast<char *>(varName), &this->data.rec_counter);
     break;
   case RECD_FLOAT:
-    RecGetRecordFloat((char *)varName, &this->data.rec_float);
+    RecGetRecordFloat(const_cast<char *>(varName), &this->data.rec_float);
     break;
   case RECD_STRING:
-    RecGetRecordString_Xmalloc((char *)varName, &this->data.rec_string);
+    RecGetRecordString_Xmalloc(const_cast<char *>(varName), &this->data.rec_string);
     break;
   case RECD_NULL:
   default:
@@ -748,12 +750,12 @@ MgmtData::compareFromString(const char *str)
     }
     break;
   case RECD_STRING:
-    if (str == NULL || *str == '\0') {
-      if (data.rec_string == NULL) {
+    if (str == nullptr || *str == '\0') {
+      if (data.rec_string == nullptr) {
         compare = true;
       }
     } else {
-      if ((data.rec_string != NULL) && (strcmp(str, data.rec_string) == 0)) {
+      if ((data.rec_string != nullptr) && (strcmp(str, data.rec_string) == 0)) {
         compare = true;
       }
     }
@@ -777,7 +779,7 @@ varType(const char *varName)
   RecDataT data_type;
   int err;
 
-  err = RecGetRecordDataType((char *)varName, &data_type);
+  err = RecGetRecordDataType(const_cast<char *>(varName), &data_type);
 
   if (err == REC_ERR_FAIL) {
     return RECD_NULL;
@@ -787,120 +789,6 @@ varType(const char *varName)
   return data_type;
 }
 
-// InkHashTable* processFormSubmission(char* submission)
-//
-//  A generic way to handle a HTML form submission.
-//  Creates a hash table with name value pairs
-//
-//  CALLEE must deallocate the returned hash table with
-//   ink_hash_table_destroy_and_free_values(InkHashTable *ht_ptr)
-//
-
-InkHashTable *
-processFormSubmission(char *submission)
-{
-  InkHashTable *nameVal = ink_hash_table_create(InkHashTableKeyType_String);
-  Tokenizer updates("&\n\r");
-  Tokenizer pair("=");
-  int numUpdates;
-  char *name;
-  char *value;
-  char *submission_copy;
-  int pairNum;
-
-  if (submission == NULL) {
-    ink_hash_table_destroy(nameVal);
-    return NULL;
-  }
-
-  submission_copy = ats_strdup(submission);
-  numUpdates      = updates.Initialize(submission_copy, SHARE_TOKS);
-
-  for (int i = 0; i < numUpdates; i++) {
-    pairNum = pair.Initialize(updates[i]);
-
-    // We should have gotten either either 1 or 2 tokens
-    //    One token indicates an variable being set to
-    //    blank.  Two indicates the variable being set to
-    //    a value.  If the submission is invalid, just forget
-    //    about it.
-    if (pairNum == 1 || pairNum == 2) {
-      name = ats_strdup(pair[0]);
-      substituteUnsafeChars(name);
-
-      // If the value is blank, store it as a null
-      if (pairNum == 1) {
-        value = NULL;
-      } else {
-        value = ats_strdup(pair[1]);
-        substituteUnsafeChars(value);
-      }
-
-      ink_hash_table_insert(nameVal, name, value);
-      ats_free(name);
-    }
-  }
-  ats_free(submission_copy);
-
-  return nameVal;
-}
-
-// InkHashTable* processFormSubmission_noSubstitute(char* submission)
-//
-//  A generic way to handle a HTML form submission.
-//  Creates a hash table with name value pairs
-//
-//  CALLEE must deallocate the returned hash table with
-//   ink_hash_table_destroy_and_free_values(InkHashTable *ht_ptr)
-//
-//  Note: This function will _not_ substituteUnsafeChars()
-InkHashTable *
-processFormSubmission_noSubstitute(char *submission)
-{
-  InkHashTable *nameVal = ink_hash_table_create(InkHashTableKeyType_String);
-  Tokenizer updates("&\n\r");
-  Tokenizer pair("=");
-  int numUpdates;
-  char *name;
-  char *value;
-  char *submission_copy;
-  int pairNum;
-
-  if (submission == NULL) {
-    ink_hash_table_destroy(nameVal);
-    return NULL;
-  }
-
-  submission_copy = ats_strdup(submission);
-  numUpdates      = updates.Initialize(submission_copy, SHARE_TOKS);
-
-  for (int i = 0; i < numUpdates; i++) {
-    pairNum = pair.Initialize(updates[i]);
-
-    // We should have gotten either either 1 or 2 tokens
-    //    One token indicates an variable being set to
-    //    blank.  Two indicates the variable being set to
-    //    a value.  If the submission is invalid, just forget
-    //    about it.
-    if (pairNum == 1 || pairNum == 2) {
-      name = ats_strdup(pair[0]);
-
-      // If the value is blank, store it as a null
-      if (pairNum == 1) {
-        value = NULL;
-      } else {
-        value = ats_strdup(pair[1]);
-      }
-
-      ink_hash_table_insert(nameVal, name, value);
-      ats_free(name);
-    }
-  }
-  ats_free(submission_copy);
-
-  return nameVal;
-}
-
 //
 // Removes any cr/lf line breaks from the text data
 //
@@ -945,8 +833,8 @@ substituteUnsafeChars(char *buffer)
     if (*read == '%') {
       subStr[0] = *(++read);
       subStr[1] = *(++read);
-      charVal   = strtol(subStr, (char **)NULL, 16);
-      *write    = (char)charVal;
+      charVal   = strtol(subStr, (char **)nullptr, 16);
+      *write    = static_cast<char>(charVal);
       read++;
       write++;
       numSub++;
@@ -1037,7 +925,7 @@ setHostnameVar()
 
   // non-FQ is just the hostname (ie: proxydev)
   firstDot = strchr(ourHostName, '.');
-  if (firstDot != NULL) {
+  if (firstDot != nullptr) {
     *firstDot = '\0';
   }
   varSetFromStr("proxy.node.hostname", ourHostName);
@@ -1045,7 +933,7 @@ setHostnameVar()
   return 0;
 }
 
-// void appendDefautDomain(char* hostname, int bufLength)
+// void appendDefaultDomain(char* hostname, int bufLength)
 //
 //   Appends the pasted in hostname with the default
 //     domain if the hostname is an unqualified name
@@ -1061,7 +949,7 @@ appendDefaultDomain(char *hostname, int bufLength)
 {
   int len                 = strlen(hostname);
   const char msg[]        = "Nodes will be know by their unqualified host name";
-  static int error_before = 0; // Race ok since effect is multple error msg
+  static int error_before = 0; // Race ok since effect is multiple error msg
 
   ink_assert(len < bufLength);
   ink_assert(bufLength >= 64);
@@ -1069,9 +957,9 @@ appendDefaultDomain(char *hostname, int bufLength)
   // Ensure null termination of the result string
   hostname[bufLength - 1] = '\0';
 
-  if (strchr(hostname, '.') == NULL) {
+  if (strchr(hostname, '.') == nullptr) {
     if (_res.defdname[0] != '\0') {
-      if (bufLength - 2 >= (int)(strlen(hostname) + strlen(_res.defdname))) {
+      if (bufLength - 2 >= static_cast<int>(strlen(hostname) + strlen(_res.defdname))) {
         ink_strlcat(hostname, ".", bufLength);
         ink_strlcat(hostname, _res.defdname, bufLength);
       } else {
@@ -1095,10 +983,10 @@ recordValidityCheck(const char *varName, const char *value)
   RecCheckT check_t;
   char *pattern;
 
-  if (RecGetRecordCheckType((char *)varName, &check_t) != REC_ERR_OKAY) {
+  if (RecGetRecordCheckType(const_cast<char *>(varName), &check_t) != REC_ERR_OKAY) {
     return false;
   }
-  if (RecGetRecordCheckExpr((char *)varName, &pattern) != REC_ERR_OKAY) {
+  if (RecGetRecordCheckExpr(const_cast<char *>(varName), &pattern) != REC_ERR_OKAY) {
     return false;
   }
 
@@ -1136,11 +1024,11 @@ recordRegexCheck(const char *pattern, const char *value)
   const char *error;
   int erroffset;
 
-  regex = pcre_compile(pattern, 0, &error, &erroffset, NULL);
+  regex = pcre_compile(pattern, 0, &error, &erroffset, nullptr);
   if (!regex) {
     return false;
   } else {
-    int r = pcre_exec(regex, NULL, value, strlen(value), 0, 0, NULL, 0);
+    int r = pcre_exec(regex, nullptr, value, strlen(value), 0, 0, nullptr, 0);
 
     pcre_free(regex);
     return (r != -1) ? true : false;
@@ -1152,10 +1040,7 @@ recordRegexCheck(const char *pattern, const char *value)
 bool
 recordRangeCheck(const char *pattern, const char *value)
 {
-  int l_limit;
-  int u_limit;
-  int val;
-  char *p = (char *)pattern;
+  char *p = const_cast<char *>(pattern);
   Tokenizer dashTok("-");
 
   if (recordRegexCheck("^[0-9]+$", value)) {
@@ -1163,9 +1048,9 @@ recordRangeCheck(const char *pattern, const char *value)
       p++;
     } // skip to '['
     if (dashTok.Initialize(++p, COPY_TOKS) == 2) {
-      l_limit = atoi(dashTok[0]);
-      u_limit = atoi(dashTok[1]);
-      val     = atoi(value);
+      int l_limit = atoi(dashTok[0]);
+      int u_limit = atoi(dashTok[1]);
+      int val     = atoi(value);
       if (val >= l_limit && val <= u_limit) {
         return true;
       }
@@ -1180,18 +1065,17 @@ recordIPCheck(const char *pattern, const char *value)
   //  regex_t regex;
   //  int result;
   bool check;
-  const char *range_pattern =
-    "\\[[0-9]+\\-[0-9]+\\]\\\\\\.\\[[0-9]+\\-[0-9]+\\]\\\\\\.\\[[0-9]+\\-[0-9]+\\]\\\\\\.\\[[0-9]+\\-[0-9]+\\]";
-  const char *ip_pattern = "[0-9]*[0-9]*[0-9].[0-9]*[0-9]*[0-9].[0-9]*[0-9]*[0-9].[0-9]*[0-9]*[0-9]";
+  const char *range_pattern = R"(\[[0-9]+\-[0-9]+\]\\\.\[[0-9]+\-[0-9]+\]\\\.\[[0-9]+\-[0-9]+\]\\\.\[[0-9]+\-[0-9]+\])";
+  const char *ip_pattern    = "[0-9]*[0-9]*[0-9].[0-9]*[0-9]*[0-9].[0-9]*[0-9]*[0-9].[0-9]*[0-9]*[0-9]";
 
   Tokenizer dotTok1(".");
   Tokenizer dotTok2(".");
-  int i;
 
   check = true;
   if (recordRegexCheck(range_pattern, pattern) && recordRegexCheck(ip_pattern, value)) {
-    if (dotTok1.Initialize((char *)pattern, COPY_TOKS) == 4 && dotTok2.Initialize((char *)value, COPY_TOKS) == 4) {
-      for (i = 0; i < 4 && check; i++) {
+    if (dotTok1.Initialize(const_cast<char *>(pattern), COPY_TOKS) == 4 &&
+        dotTok2.Initialize(const_cast<char *>(value), COPY_TOKS) == 4) {
+      for (int i = 0; i < 4 && check; i++) {
         if (!recordRangeCheck(dotTok1[i], dotTok2[i])) {
           check = false;
         }
@@ -1211,114 +1095,13 @@ recordRestartCheck(const char *varName)
 {
   RecUpdateT update_t;
 
-  if (RecGetRecordUpdateType((char *)varName, &update_t) != REC_ERR_OKAY) {
+  if (RecGetRecordUpdateType(const_cast<char *>(varName), &update_t) != REC_ERR_OKAY) {
     return false;
   }
 
-  if (update_t == RECU_RESTART_TS || update_t == RECU_RESTART_TM || update_t == RECU_RESTART_TC) {
+  if (update_t == RECU_RESTART_TS || update_t == RECU_RESTART_TM) {
     return true;
   }
 
   return false;
 }
-
-//-------------------------------------------------------------------------
-// getFilesInDirectory
-//
-// copied from MultiFiles::WalkFiles - but slightly modified
-// returns -1 if directory does not exit
-// returns 1 if everything is ok
-//-------------------------------------------------------------------------
-int
-getFilesInDirectory(char *managedDir, ExpandingArray *fileList)
-{
-  struct dirent *dirEntry;
-  DIR *dir;
-
-  char *fileName;
-  char *filePath;
-  struct stat fileInfo;
-  //  struct stat records_config_fileInfo;
-  fileEntry *fileListEntry;
-
-  if ((dir = opendir(managedDir)) == NULL) {
-    mgmt_log("[getFilesInDirectory] Unable to open %s directory: %s\n", managedDir, strerror(errno));
-    return -1;
-  }
-  // The fun of Solaris - readdir_r requires a buffer passed into it
-  //   The man page says this obscene expression gives us the proper
-  //     size
-  dirEntry = (struct dirent *)alloca(sizeof(struct dirent) + ink_file_namemax(".") + 1);
-
-  struct dirent *result;
-  while (readdir_r(dir, dirEntry, &result) == 0) {
-    if (!result) {
-      break;
-    }
-    fileName = dirEntry->d_name;
-    if (!fileName || !*fileName) {
-      continue;
-    }
-    filePath = newPathString(managedDir, fileName);
-    if (stat(filePath, &fileInfo) < 0) {
-      mgmt_log("[getFilesInDirectory] Stat of a %s failed : %s\n", fileName, strerror(errno));
-    } else {
-      // Ignore ., .., and any dot files
-      if (fileName && *fileName != '.') {
-        fileListEntry         = (fileEntry *)ats_malloc(sizeof(fileEntry));
-        fileListEntry->c_time = fileInfo.st_ctime;
-        ink_strlcpy(fileListEntry->name, fileName, sizeof(fileListEntry->name));
-        fileList->addEntry(fileListEntry);
-      }
-    }
-    delete[] filePath;
-  }
-
-  closedir(dir);
-
-  fileList->sortWithFunction(fileEntryCmpFunc);
-  return 1;
-}
-
-//-------------------------------------------------------------------------
-// newPathString
-//
-// copied from MultiFile::newPathString
-// Note: uses C++ new/delete for memory allocation/deallocation
-//-------------------------------------------------------------------------
-char *
-newPathString(const char *s1, const char *s2)
-{
-  char *newStr;
-  int srcLen; // is the length of the src rootpath
-  int addLen; // maximum total path length
-
-  // Treat null as an empty path.
-  if (!s2) {
-    s2 = "";
-  }
-  addLen = strlen(s2) + 1;
-  if (*s2 == '/') {
-    // If addpath is rooted, then rootpath is unused.
-    newStr = new char[addLen];
-    ink_strlcpy(newStr, s2, addLen);
-    return newStr;
-  }
-  if (!s1 || !*s1) {
-    // If there's no rootpath return the addpath
-    newStr = new char[addLen];
-    ink_strlcpy(newStr, s2, addLen);
-    return newStr;
-  }
-  srcLen = strlen(s1);
-  newStr = new char[srcLen + addLen + 1];
-  ink_assert(newStr != NULL);
-
-  ink_strlcpy(newStr, s1, srcLen + addLen + 1);
-  if (newStr[srcLen - 1] != '/') {
-    newStr[srcLen++] = '/';
-  }
-  ink_strlcpy(&newStr[srcLen], s2, srcLen + addLen + 1);
-
-  return newStr;
-}
