@@ -53,11 +53,11 @@ mgmt_use_syslog()
 int
 mgmt_readline(int soc, char *buf, int maxlen)
 {
-  int n = 0, rc;
+  int n = 0;
   char c;
 
   for (; n < maxlen; n++) {
-    rc = read_socket(soc, &c, 1);
+    int rc = read_socket(soc, &c, 1);
     if (rc == 1) {
       *buf++ = c;
       if (c == '\n') {
@@ -105,12 +105,12 @@ mgmt_readline(int soc, char *buf, int maxlen)
 int
 mgmt_writeline(int soc, const char *data, int nbytes)
 {
-  int nleft, nwritten, n;
+  int nleft, n;
   const char *tmp = data;
 
   nleft = nbytes;
   while (nleft > 0) {
-    nwritten = write_socket(soc, tmp, nleft);
+    int nwritten = write_socket(soc, tmp, nleft);
     if (nwritten <= 0) { /* Error or nothing written */
       return nwritten;
     }
@@ -141,12 +141,11 @@ mgmt_writeline(int soc, const char *data, int nbytes)
 int
 mgmt_read_pipe(int fd, char *buf, int bytes_to_read)
 {
-  int err        = 0;
   char *p        = buf;
   int bytes_read = 0;
 
   while (bytes_to_read > 0) {
-    err = read_socket(fd, p, bytes_to_read);
+    int err = read_socket(fd, p, bytes_to_read);
     if (err == 0) {
       return err;
     } else if (err < 0) {
@@ -183,12 +182,11 @@ mgmt_read_pipe(int fd, char *buf, int bytes_to_read)
 int
 mgmt_write_pipe(int fd, char *buf, int bytes_to_write)
 {
-  int err           = 0;
   char *p           = buf;
   int bytes_written = 0;
 
   while (bytes_to_write > 0) {
-    err = write_socket(fd, p, bytes_to_write);
+    int err = write_socket(fd, p, bytes_to_write);
     if (err == 0) {
       return err;
     } else if (err < 0) {
@@ -272,7 +270,6 @@ void
 mgmt_fatal(const int lerrno, const char *message_format, ...)
 {
   va_list ap;
-  char extended_format[4096], message[4096];
 
   va_start(ap, message_format);
 
@@ -283,6 +280,7 @@ mgmt_fatal(const int lerrno, const char *message_format, ...)
 
     FatalV(message_format, ap);
   } else {
+    char extended_format[4096], message[4096];
     snprintf(extended_format, sizeof(extended_format), "FATAL ==> %s", message_format);
     vsprintf(message, extended_format, ap);
 
@@ -336,7 +334,7 @@ mgmt_getAddrForIntr(char *intrName, sockaddr *addr, int *mtu)
   int fakeSocket;            // a temporary socket to pass to ioctl
   struct ifconf ifc;         // ifconf information
   char *ifbuf;               // ifconf buffer
-  struct ifreq *ifr, *ifend; // pointer to individual inferface info
+  struct ifreq *ifr, *ifend; // pointer to individual interface info
   int lastlen;
   int len;
 
@@ -353,7 +351,7 @@ mgmt_getAddrForIntr(char *intrName, sockaddr *addr, int *mtu)
   lastlen = 0;
   len     = 128 * sizeof(struct ifreq); // initial buffer size guess
   for (;;) {
-    ifbuf = (char *)ats_malloc(len);
+    ifbuf = static_cast<char *>(ats_malloc(len));
     memset(ifbuf, 0, len); // prevent UMRs
     ifc.ifc_len = len;
     ifc.ifc_buf = ifbuf;
@@ -373,11 +371,11 @@ mgmt_getAddrForIntr(char *intrName, sockaddr *addr, int *mtu)
 
   found = false;
   // Loop through the list of interfaces
-  ifend = (struct ifreq *)(ifc.ifc_buf + ifc.ifc_len);
+  ifend = reinterpret_cast<struct ifreq *>(ifc.ifc_buf + ifc.ifc_len);
   for (ifr = ifc.ifc_req; ifr < ifend;) {
     if (ifr->ifr_addr.sa_family == AF_INET && strcmp(ifr->ifr_name, intrName) == 0) {
       // Get the address of the interface
-      if (ioctl(fakeSocket, SIOCGIFADDR, (char *)ifr) < 0) {
+      if (ioctl(fakeSocket, SIOCGIFADDR, reinterpret_cast<char *>(ifr)) < 0) {
         mgmt_log("[getAddrForIntr] Unable obtain address for network interface %s\n", intrName);
       } else {
         // Only look at the address if it an internet address
@@ -398,7 +396,7 @@ mgmt_getAddrForIntr(char *intrName, sockaddr *addr, int *mtu)
 #if defined(freebsd) || defined(darwin)
     ifr = (struct ifreq *)((char *)&ifr->ifr_addr + ifr->ifr_addr.sa_len);
 #else
-    ifr = (struct ifreq *)(((char *)ifr) + sizeof(*ifr));
+    ifr = reinterpret_cast<struct ifreq *>((reinterpret_cast<char *>(ifr)) + sizeof(*ifr));
 #endif
   }
   ats_free(ifbuf);
@@ -420,7 +418,7 @@ mgmt_sortipaddrs(int num, struct in_addr **list)
 
   min   = (list[0])->s_addr;
   entry = list[0];
-  while (i < num && (tmp = (struct in_addr *)list[i]) != nullptr) {
+  while (i < num && (tmp = list[i]) != nullptr) {
     i++;
     if (min > tmp->s_addr) {
       min   = tmp->s_addr;
