@@ -31,15 +31,12 @@
  *
  */
 
-#ifndef _MGMT_UTILS_H
-#define _MGMT_UTILS_H
+#pragma once
 
-#include "ts/ink_platform.h"
-#include "ts/Diags.h"
+#include "tscore/ink_platform.h"
+#include "tscore/Diags.h"
 
-#include "P_RecCore.h"
-
-void mgmt_blockAllSigs();
+#include "records/P_RecCore.h"
 
 int mgmt_readline(int fd, char *buf, int maxlen);
 int mgmt_writeline(int fd, const char *data, int nbytes);
@@ -51,7 +48,7 @@ void mgmt_use_syslog();
 void mgmt_cleanup();
 
 struct in_addr *mgmt_sortipaddrs(int num, struct in_addr **list);
-bool mgmt_getAddrForIntr(char *intrName, sockaddr *addr, int *mtu = 0);
+bool mgmt_getAddrForIntr(char *intrName, sockaddr *addr, int *mtu = nullptr);
 
 /* the following functions are all DEPRECATED.  The Diags
    interface should be used exclusively in the future */
@@ -61,5 +58,3 @@ void mgmt_fatal(const int lerrno, const char *message_format, ...) TS_NORETURN;
 
 void mgmt_sleep_sec(int);
 void mgmt_sleep_msec(int);
-
-#endif /* _MGMT_UTILS_H */
