@@ -38,13 +38,11 @@
 #include <sys/param.h>
 #include <ts/remap.h>
 
-#include "ts/ink_config.h"
-
 using std::strlen;
 
 struct AuthRequestContext;
 
-typedef bool (*AuthRequestTransform)(AuthRequestContext *auth);
+using AuthRequestTransform = bool (*)(AuthRequestContext *);
 
 const static int MAX_HOST_LENGTH = 4096;
 
@@ -57,12 +55,12 @@ static TSCont AuthOsDnsContinuation;
 
 struct AuthOptions {
   std::string hostname;
-  int hostport;
-  AuthRequestTransform transform;
-  bool force;
+  int hostport                   = -1;
+  AuthRequestTransform transform = nullptr;
+  bool force                     = false;
 
-  AuthOptions() : hostport(-1), transform(NULL), force(false) {}
-  ~AuthOptions() {}
+  AuthOptions()  = default;
+  ~AuthOptions() = default;
 };
 
 // Global options; used when we are in global authorization mode.
@@ -73,7 +71,7 @@ static AuthOptions *AuthGlobalOptions;
 //      TS_EVENT_CONTINUE   Continue the state machine, returning to the ATS event loop
 //      TS_EVENT_NONE       Stop processing (because a nested dispatch occurred)
 //      Anything else       Continue the state machine with this event
-typedef TSEvent (*StateHandler)(struct AuthRequestContext *, void *edata);
+using StateHandler = TSEvent (*)(struct AuthRequestContext *, void *);
 
 struct StateTransition {
   TSEvent event;
@@ -102,8 +100,8 @@ StateContinue(AuthRequestContext *, void *)
 }
 
 // State table for sending the auth proxy response to the client.
-static const StateTransition StateTableSendResponse[] = {{TS_EVENT_HTTP_SEND_RESPONSE_HDR, StateAuthProxySendResponse, NULL},
-                                                         {TS_EVENT_NONE, NULL, NULL}};
+static const StateTransition StateTableSendResponse[] = {{TS_EVENT_HTTP_SEND_RESPONSE_HDR, StateAuthProxySendResponse, nullptr},
+                                                         {TS_EVENT_NONE, nullptr, nullptr}};
 
 // State table for reading the proxy response body content.
 static const StateTransition StateTableProxyReadContent[] = {
@@ -111,9 +109,9 @@ static const StateTransition StateTableProxyReadContent[] = {
   {TS_EVENT_VCONN_READ_COMPLETE, StateAuthProxyReadContent, StateTableProxyReadContent},
   {TS_EVENT_VCONN_EOS, StateAuthProxyCompleteContent, StateTableProxyReadContent},
   {TS_EVENT_HTTP_SEND_RESPONSE_HDR, StateContinue, StateTableSendResponse},
-  {TS_EVENT_ERROR, StateUnauthorized, NULL},
-  {TS_EVENT_IMMEDIATE, StateAuthorized, NULL},
-  {TS_EVENT_NONE, NULL, NULL}};
+  {TS_EVENT_ERROR, StateUnauthorized, nullptr},
+  {TS_EVENT_IMMEDIATE, StateAuthorized, nullptr},
+  {TS_EVENT_NONE, nullptr, nullptr}};
 
 // State table for reading the auth proxy response header.
 static const StateTransition StateTableProxyReadHeader[] = {
@@ -122,52 +120,44 @@ static const StateTransition StateTableProxyReadHeader[] = {
   {TS_EVENT_HTTP_READ_REQUEST_HDR, StateAuthProxyCompleteHeaders, StateTableProxyReadHeader},
   {TS_EVENT_HTTP_SEND_RESPONSE_HDR, StateContinue, StateTableSendResponse},
   {TS_EVENT_HTTP_CONTINUE, StateAuthProxyReadContent, StateTableProxyReadContent},
-  {TS_EVENT_VCONN_EOS, StateUnauthorized, NULL}, // XXX Should we check headers on EOS?
-  {TS_EVENT_ERROR, StateUnauthorized, NULL},
-  {TS_EVENT_IMMEDIATE, StateAuthorized, NULL},
-  {TS_EVENT_NONE, NULL, NULL}};
+  {TS_EVENT_VCONN_EOS, StateUnauthorized, nullptr}, // XXX Should we check headers on EOS?
+  {TS_EVENT_ERROR, StateUnauthorized, nullptr},
+  {TS_EVENT_IMMEDIATE, StateAuthorized, nullptr},
+  {TS_EVENT_NONE, nullptr, nullptr}};
 
 // State table for sending the request to the auth proxy.
 static const StateTransition StateTableProxyRequest[] = {
   {TS_EVENT_VCONN_WRITE_COMPLETE, StateAuthProxyWriteComplete, StateTableProxyReadHeader},
-  {TS_EVENT_ERROR, StateUnauthorized, NULL},
-  {TS_EVENT_NONE, NULL, NULL}};
+  {TS_EVENT_ERROR, StateUnauthorized, nullptr},
+  {TS_EVENT_NONE, nullptr, nullptr}};
 
 // Initial state table.
 static const StateTransition StateTableInit[] = {{TS_EVENT_HTTP_POST_REMAP, StateAuthProxyConnect, StateTableProxyRequest},
-                                                 {TS_EVENT_ERROR, StateUnauthorized, NULL},
-                                                 {TS_EVENT_NONE, NULL, NULL}};
+                                                 {TS_EVENT_ERROR, StateUnauthorized, nullptr},
+                                                 {TS_EVENT_NONE, nullptr, nullptr}};
 
 struct AuthRequestContext {
-  TSHttpTxn txn;        // Original client transaction we are authorizing.
-  TSCont cont;          // Continuation for this state machine.
-  TSVConn vconn;        // Virtual connection to the auth proxy.
-  TSHttpParser hparser; // HTTP response header parser.
-  HttpHeader rheader;   // HTTP response header.
+  TSHttpTxn txn = nullptr; // Original client transaction we are authorizing.
+  TSCont cont   = nullptr; // Continuation for this state machine.
+  TSVConn vconn = nullptr; // Virtual connection to the auth proxy.
+  TSHttpParser hparser;    // HTTP response header parser.
+  HttpHeader rheader;      // HTTP response header.
   HttpIoBuffer iobuf;
-  const char *method; // Client request method (e.g. GET)
-  bool read_body;
+  const char *method = nullptr; // Client request method (e.g. GET)
+  bool read_body     = true;
 
-  const StateTransition *state;
+  const StateTransition *state = nullptr;
 
   AuthRequestContext()
-    : txn(NULL),
-      cont(NULL),
-      vconn(NULL),
-      hparser(TSHttpParserCreate()),
-      rheader(),
-      iobuf(TS_IOBUFFER_SIZE_INDEX_4K),
-      method(NULL),
-      read_body(true),
-      state(NULL)
+    : cont(TSContCreate(dispatch, TSMutexCreate())), hparser(TSHttpParserCreate()), rheader(), iobuf(TS_IOBUFFER_SIZE_INDEX_4K)
+
   {
-    this->cont = TSContCreate(dispatch, TSMutexCreate());
     TSContDataSet(this->cont, this);
   }
 
   ~AuthRequestContext()
   {
-    TSContDataSet(this->cont, NULL);
+    TSContDataSet(this->cont, nullptr);
     TSContDestroy(this->cont);
     TSHttpParserDestroy(this->hparser);
     if (this->vconn) {
@@ -180,7 +170,7 @@ struct AuthRequestContext {
   {
     AuthOptions *opt;
 
-    opt = (AuthOptions *)TSHttpTxnArgGet(this->txn, AuthTaggedRequestArg);
+    opt = static_cast<AuthOptions *>(TSUserArgGet(this->txn, AuthTaggedRequestArg));
     return opt ? opt : AuthGlobalOptions;
   }
 
@@ -208,7 +198,7 @@ AuthRequestContext::destroy(AuthRequestContext *auth)
 int
 AuthRequestContext::dispatch(TSCont cont, TSEvent event, void *edata)
 {
-  AuthRequestContext *auth = (AuthRequestContext *)TSContDataGet(cont);
+  AuthRequestContext *auth = static_cast<AuthRequestContext *>(TSContDataGet(cont));
   const StateTransition *s;
 
 pump:
@@ -219,8 +209,8 @@ pump:
   }
 
   // If we don't have a handler, the state machine is borked.
-  TSReleaseAssert(s != NULL);
-  TSReleaseAssert(s->handler != NULL);
+  TSReleaseAssert(s != nullptr);
+  TSReleaseAssert(s->handler != nullptr);
 
   // Move to the next state. We have to set this *before* invoking the
   // handler because the handler itself can invoke the next handler.
@@ -234,7 +224,7 @@ pump:
     return TS_EVENT_NONE;
   }
   // If there are no more states, the state machine has terminated.
-  if (auth->state == NULL) {
+  if (auth->state == nullptr) {
     AuthRequestContext::destroy(auth);
     return TS_EVENT_NONE;
   }
@@ -270,7 +260,7 @@ AuthChainAuthorizationResponse(AuthRequestContext *auth)
 {
   if (auth->vconn) {
     TSVConnClose(auth->vconn);
-    auth->vconn = NULL;
+    auth->vconn = nullptr;
   }
 
   TSHttpTxnHookAdd(auth->txn, TS_HTTP_SEND_RESPONSE_HDR_HOOK, auth->cont);
@@ -408,7 +398,7 @@ StateAuthProxyConnect(AuthRequestContext *auth, void * /* edata ATS_UNUSED */)
   AuthLogDebug("client request %s a HEAD request", auth->method == TS_HTTP_METHOD_HEAD ? "is" : "is not");
 
   auth->vconn = TSHttpConnect(ip);
-  if (auth->vconn == NULL) {
+  if (auth->vconn == nullptr) {
     return TS_EVENT_ERROR;
   }
   // Transform the client request into an auth proxy request and write it
@@ -425,7 +415,6 @@ static TSEvent
 StateAuthProxyCompleteHeaders(AuthRequestContext *auth, void * /* edata ATS_UNUSED */)
 {
   TSHttpStatus status;
-  unsigned nbytes;
 
   HttpDebugHeader(auth->rheader.buffer, auth->rheader.header);
 
@@ -446,7 +435,7 @@ StateAuthProxyCompleteHeaders(AuthRequestContext *auth, void * /* edata ATS_UNUS
     } else {
       // OK, we have a non-chunked response. If there's any content, let's go and
       // buffer it so that we can send it on to the client.
-      nbytes = HttpGetContentLength(auth->rheader.buffer, auth->rheader.header);
+      unsigned nbytes = HttpGetContentLength(auth->rheader.buffer, auth->rheader.header);
       if (nbytes > 0) {
         AuthLogDebug("content length is %u", nbytes);
         return TS_EVENT_HTTP_CONTINUE;
@@ -512,7 +501,7 @@ StateAuthProxyReadHeaders(AuthRequestContext *auth, void * /* edata ATS_UNUSED *
     TSParseResult result;
 
     ptr = TSIOBufferBlockReadStart(blk, auth->iobuf.reader, &nbytes);
-    if (ptr == NULL || nbytes == 0) {
+    if (ptr == nullptr || nbytes == 0) {
       continue;
     }
 
@@ -602,7 +591,7 @@ StateUnauthorized(AuthRequestContext *auth, void *)
 {
   static const char msg[] = "authorization denied\n";
 
-  TSHttpTxnSetHttpRetStatus(auth->txn, TS_HTTP_STATUS_FORBIDDEN);
+  TSHttpTxnStatusSet(auth->txn, TS_HTTP_STATUS_FORBIDDEN);
   TSHttpTxnErrorBodySet(auth->txn, TSstrdup(msg), sizeof(msg) - 1, TSstrdup("text/plain"));
 
   TSHttpTxnReenable(auth->txn, TS_EVENT_HTTP_ERROR);
@@ -632,14 +621,13 @@ StateAuthorized(AuthRequestContext *auth, void *)
 static bool
 AuthRequestIsTagged(TSHttpTxn txn)
 {
-  return AuthTaggedRequestArg != -1 && TSHttpTxnArgGet(txn, AuthTaggedRequestArg) != NULL;
+  return AuthTaggedRequestArg != -1 && TSUserArgGet(txn, AuthTaggedRequestArg) != nullptr;
 }
 
 static int
 AuthProxyGlobalHook(TSCont /* cont ATS_UNUSED */, TSEvent event, void *edata)
 {
-  AuthRequestContext *auth;
-  TSHttpTxn txn = (TSHttpTxn)edata;
+  TSHttpTxn txn = static_cast<TSHttpTxn>(edata);
 
   AuthLogDebug("handling event=%d edata=%p", (int)event, edata);
 
@@ -662,13 +650,13 @@ AuthProxyGlobalHook(TSCont /* cont ATS_UNUSED */, TSEvent event, void *edata)
     }
     // Hook this request if we are in global authorization mode or if a
     // remap rule tagged it.
-    if (AuthGlobalOptions != NULL || AuthRequestIsTagged(txn)) {
-      auth        = AuthRequestContext::allocate();
-      auth->state = StateTableInit;
-      auth->txn   = txn;
+    if (AuthGlobalOptions != nullptr || AuthRequestIsTagged(txn)) {
+      AuthRequestContext *auth = AuthRequestContext::allocate();
+      auth->state              = StateTableInit;
+      auth->txn                = txn;
       return AuthRequestContext::dispatch(auth->cont, event, edata);
     }
-  // fallthru
+    // fallthrough
 
   default:
     return TS_EVENT_NONE;
@@ -682,11 +670,11 @@ AuthParseOptions(int argc, const char **argv)
   // on some platforms (e.g. Solaris / Illumos). On sane platforms (e.g. linux), it'll get
   // automatically casted back to the const char*, as the struct is defined in <getopt.h>.
   static const struct option longopt[] = {
-    {const_cast<char *>("auth-host"), required_argument, 0, 'h'},
-    {const_cast<char *>("auth-port"), required_argument, 0, 'p'},
-    {const_cast<char *>("auth-transform"), required_argument, 0, 't'},
-    {const_cast<char *>("force-cacheability"), no_argument, 0, 'c'},
-    {0, 0, 0, 0},
+    {const_cast<char *>("auth-host"), required_argument, nullptr, 'h'},
+    {const_cast<char *>("auth-port"), required_argument, nullptr, 'p'},
+    {const_cast<char *>("auth-transform"), required_argument, nullptr, 't'},
+    {const_cast<char *>("force-cacheability"), no_argument, nullptr, 'c'},
+    {nullptr, 0, nullptr, 0},
   };
 
   AuthOptions *options = AuthNew<AuthOptions>();
@@ -696,7 +684,7 @@ AuthParseOptions(int argc, const char **argv)
   for (;;) {
     int opt;
 
-    opt = getopt_long(argc, (char *const *)argv, "", longopt, NULL);
+    opt = getopt_long(argc, const_cast<char *const *>(argv), "", longopt, nullptr);
     switch (opt) {
     case 'h':
       options->hostname = optarg;
@@ -748,9 +736,10 @@ TSPluginInit(int argc, const char *argv[])
     AuthLogError("plugin registration failed");
   }
 
-  TSReleaseAssert(TSHttpArgIndexReserve("AuthProxy", "AuthProxy authorization tag", &AuthTaggedRequestArg) == TS_SUCCESS);
+  TSReleaseAssert(TSUserArgIndexReserve(TS_USER_ARGS_TXN, "AuthProxy", "AuthProxy authorization tag", &AuthTaggedRequestArg) ==
+                  TS_SUCCESS);
 
-  AuthOsDnsContinuation = TSContCreate(AuthProxyGlobalHook, NULL);
+  AuthOsDnsContinuation = TSContCreate(AuthProxyGlobalHook, nullptr);
   AuthGlobalOptions     = AuthParseOptions(argc, argv);
   AuthLogDebug("using authorization proxy at %s:%d", AuthGlobalOptions->hostname.c_str(), AuthGlobalOptions->hostport);
 
@@ -761,9 +750,10 @@ TSPluginInit(int argc, const char *argv[])
 TSReturnCode
 TSRemapInit(TSRemapInterface * /* api ATS_UNUSED */, char * /* err ATS_UNUSED */, int /* errsz ATS_UNUSED */)
 {
-  TSReleaseAssert(TSHttpArgIndexReserve("AuthProxy", "AuthProxy authorization tag", &AuthTaggedRequestArg) == TS_SUCCESS);
+  TSReleaseAssert(TSUserArgIndexReserve(TS_USER_ARGS_TXN, "AuthProxy", "AuthProxy authorization tag", &AuthTaggedRequestArg) ==
+                  TS_SUCCESS);
 
-  AuthOsDnsContinuation = TSContCreate(AuthProxyGlobalHook, NULL);
+  AuthOsDnsContinuation = TSContCreate(AuthProxyGlobalHook, nullptr);
   return TS_SUCCESS;
 }
 
@@ -788,16 +778,16 @@ TSRemapNewInstance(int argc, char *argv[], void **instance, char * /* err ATS_UN
 void
 TSRemapDeleteInstance(void *instance)
 {
-  AuthOptions *options = (AuthOptions *)instance;
+  AuthOptions *options = static_cast<AuthOptions *>(instance);
   AuthDelete(options);
 }
 
 TSRemapStatus
 TSRemapDoRemap(void *instance, TSHttpTxn txn, TSRemapRequestInfo * /* rri ATS_UNUSED */)
 {
-  AuthOptions *options = (AuthOptions *)instance;
+  AuthOptions *options = static_cast<AuthOptions *>(instance);
 
-  TSHttpTxnArgSet(txn, AuthTaggedRequestArg, options);
+  TSUserArgSet(txn, AuthTaggedRequestArg, options);
   TSHttpTxnHookAdd(txn, TS_HTTP_POST_REMAP_HOOK, AuthOsDnsContinuation);
 
   return TSREMAP_NO_REMAP;
