@@ -21,14 +21,16 @@
     See the License for the specific language governing permissions and
     limitations under the License.
 */
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <stdarg.h>
-#include <getopt.h>
+#include <cstdlib>
+#include <cstdio>
+#include <cstring>
+#include <cstdarg>
 
 #include <string>
 #include <unordered_map>
+#include <cinttypes>
+#include <string_view>
+#include <array>
 
 #include "ts/ts.h"
 #include "ts/remap.h"
@@ -37,17 +39,31 @@
 #include "configs.h"
 
 // Global config, if we don't have a remap specific config.
-static BgFetchConfig *gConfig;
+static BgFetchConfig *gConfig = nullptr;
+
+// This is the list of all headers that must be removed when we make the actual background
+// fetch request.
+static const std::array<const std::string_view, 6> FILTER_HEADERS{
+  {{TS_MIME_FIELD_RANGE, static_cast<size_t>(TS_MIME_LEN_RANGE)},
+   {TS_MIME_FIELD_IF_MATCH, static_cast<size_t>(TS_MIME_LEN_IF_MATCH)},
+   {TS_MIME_FIELD_IF_MODIFIED_SINCE, static_cast<size_t>(TS_MIME_LEN_IF_MODIFIED_SINCE)},
+   {TS_MIME_FIELD_IF_NONE_MATCH, static_cast<size_t>(TS_MIME_LEN_IF_NONE_MATCH)},
+   {TS_MIME_FIELD_IF_RANGE, static_cast<size_t>(TS_MIME_LEN_IF_RANGE)},
+   {TS_MIME_FIELD_IF_UNMODIFIED_SINCE, static_cast<size_t>(TS_MIME_LEN_IF_UNMODIFIED_SINCE)}}};
 
 ///////////////////////////////////////////////////////////////////////////
-// Hold the global ackground fetch state. This is currently shared across all
+// Hold the global background fetch state. This is currently shared across all
 // configurations, as a singleton. ToDo: Would it ever make sense to do this
-// per remap rule? Probably not.
+// per remap rule? Maybe for per-remap logging ??
 typedef std::unordered_map<std::string, bool> OutstandingRequests;
 
 class BgFetchState
 {
 public:
+  BgFetchState()                     = default;
+  BgFetchState(BgFetchState const &) = delete;
+  void operator=(BgFetchState const &) = delete;
+
   static BgFetchState &
   getInstance()
   {
@@ -56,11 +72,16 @@ public:
   }
 
   ~BgFetchState() { TSMutexDestroy(_lock); }
+
   void
-  createLog(const char *log_name)
+  createLog(const std::string &log_name)
   {
-    TSDebug(PLUGIN_NAME, "Creating log name %s", log_name);
-    TSAssert(TS_SUCCESS == TSTextLogObjectCreate(log_name, TS_LOG_MODE_ADD_TIMESTAMP, &_log));
+    if (!_log) {
+      TSDebug(PLUGIN_NAME, "Creating log name %s", log_name.c_str());
+      TSAssert(TS_SUCCESS == TSTextLogObjectCreate(log_name.c_str(), TS_LOG_MODE_ADD_TIMESTAMP, &_log));
+    } else {
+      TSError("[%s] A log file was already create, ignoring creation of %s", PLUGIN_NAME, log_name.c_str());
+    }
   }
 
   TSTextLogObject
@@ -106,13 +127,9 @@ public:
   }
 
 private:
-  BgFetchState() : _log(NULL), _lock(TSMutexCreate()) {}
-  BgFetchState(BgFetchState const &);   // Don't Implement
-  void operator=(BgFetchState const &); // Don't implement
-
-  TSTextLogObject _log;
   OutstandingRequests _urls;
-  TSMutex _lock;
+  TSTextLogObject _log = nullptr;
+  TSMutex _lock        = TSMutexCreate();
 };
 
 //////////////////////////////////////////////////////////////////////////////
@@ -120,22 +137,7 @@ private:
 // This is necessary, because the TXN is likely to not be available
 // during the time we fetch from origin.
 struct BgFetchData {
-  BgFetchData()
-    : hdr_loc(TS_NULL_MLOC),
-      url_loc(TS_NULL_MLOC),
-      vc(NULL),
-      req_io_buf(NULL),
-      resp_io_buf(NULL),
-      req_io_buf_reader(NULL),
-      resp_io_buf_reader(NULL),
-      r_vio(NULL),
-      w_vio(NULL),
-      _bytes(0),
-      _cont(NULL)
-  {
-    mbuf = TSMBufferCreate();
-    memset(&client_ip, 0, sizeof(client_ip));
-  }
+  BgFetchData() { memset(&client_ip, 0, sizeof(client_ip)); }
 
   ~BgFetchData()
   {
@@ -147,7 +149,7 @@ struct BgFetchData {
     if (vc) {
       TSError("[%s] Destroyed BgFetchDATA while VC was alive", PLUGIN_NAME);
       TSVConnClose(vc);
-      vc = NULL;
+      vc = nullptr;
     }
 
     // If we got schedule, also clean that up
@@ -155,7 +157,7 @@ struct BgFetchData {
       releaseUrl();
 
       TSContDestroy(_cont);
-      _cont = NULL;
+      _cont = nullptr;
       TSIOBufferReaderFree(req_io_buf_reader);
       TSIOBufferDestroy(req_io_buf);
       TSIOBufferReaderFree(resp_io_buf_reader);
@@ -190,21 +192,25 @@ struct BgFetchData {
   void schedule();
   void log(TSEvent event) const;
 
-  TSMBuffer mbuf;
-  TSMLoc hdr_loc;
-  TSMLoc url_loc;
+  TSMBuffer mbuf = TSMBufferCreate();
+  TSMLoc hdr_loc = TS_NULL_MLOC;
+  TSMLoc url_loc = TS_NULL_MLOC;
+
   struct sockaddr_storage client_ip;
 
   // This is for the actual background fetch / NetVC
-  TSVConn vc;
-  TSIOBuffer req_io_buf, resp_io_buf;
-  TSIOBufferReader req_io_buf_reader, resp_io_buf_reader;
-  TSVIO r_vio, w_vio;
+  TSVConn vc                          = nullptr;
+  TSIOBuffer req_io_buf               = nullptr;
+  TSIOBuffer resp_io_buf              = nullptr;
+  TSIOBufferReader req_io_buf_reader  = nullptr;
+  TSIOBufferReader resp_io_buf_reader = nullptr;
+  TSVIO r_vio                         = nullptr;
+  TSVIO w_vio                         = nullptr;
 
 private:
   std::string _url;
-  int64_t _bytes;
-  TSCont _cont;
+  int64_t _bytes = 0;
+  TSCont _cont   = nullptr;
 };
 
 // This sets up the data and continuation properly, this is done outside
@@ -214,7 +220,7 @@ private:
 // This needs the txnp temporarily, so it can copy the pristine request
 // URL. The txnp is not used once initialize() returns.
 //
-// Upon succesful completion, the struct should be ready to start a
+// Upon successful completion, the struct should be ready to start a
 // background fetch.
 bool
 BgFetchData::initialize(TSMBuffer request, TSMLoc req_hdr, TSHttpTxn txnp)
@@ -247,7 +253,7 @@ BgFetchData::initialize(TSMBuffer request, TSMLoc req_hdr, TSHttpTxn txnp)
       if (TS_SUCCESS == TSUrlClone(mbuf, request, p_url, &url_loc)) {
         TSMLoc c_url = TS_NULL_MLOC;
         int len;
-        char *url = NULL;
+        char *url = nullptr;
 
         // Get the cache key URL (for now), since this has better lookup behavior when using
         // e.g. the cachekey plugin.
@@ -271,10 +277,13 @@ BgFetchData::initialize(TSMBuffer request, TSMLoc req_hdr, TSHttpTxn txnp)
               TSDebug(PLUGIN_NAME, "Set header Host: %.*s", len, hostp);
             }
 
-            // Next, remove any Range: headers from our request.
-            if (remove_header(mbuf, hdr_loc, TS_MIME_FIELD_RANGE, TS_MIME_LEN_RANGE) > 0) {
-              TSDebug(PLUGIN_NAME, "Removed the Range: header from request");
+            // Next, remove the Range headers and IMS (conditional) headers from the request
+            for (auto const &header : FILTER_HEADERS) {
+              if (remove_header(mbuf, hdr_loc, header.data(), header.size()) > 0) {
+                TSDebug(PLUGIN_NAME, "Removed the %s header from request", header.data());
+              }
             }
+
             // Everything went as planned, so we can return true
             ret = true;
           }
@@ -293,7 +302,7 @@ static int cont_bg_fetch(TSCont contp, TSEvent event, void *edata);
 void
 BgFetchData::schedule()
 {
-  TSAssert(NULL == _cont);
+  TSAssert(nullptr == _cont);
 
   // Setup the continuation
   _cont = TSContCreate(cont_bg_fetch, TSMutexCreate());
@@ -306,7 +315,7 @@ BgFetchData::schedule()
   resp_io_buf_reader = TSIOBufferReaderAlloc(resp_io_buf);
 
   // Schedule
-  TSContSchedule(_cont, 0, TS_THREAD_POOL_NET);
+  TSContScheduleOnPool(_cont, 0, TS_THREAD_POOL_NET);
 }
 
 // Log format is:
@@ -361,7 +370,7 @@ cont_bg_fetch(TSCont contp, TSEvent event, void * /* edata ATS_UNUSED */)
     // Debug info for this particular bg fetch (put all debug in here please)
     if (TSIsDebugTagSet(PLUGIN_NAME)) {
       char buf[INET6_ADDRSTRLEN];
-      const sockaddr *sockaddress = (const sockaddr *)&data->client_ip;
+      const sockaddr *sockaddress = reinterpret_cast<const sockaddr *>(&data->client_ip);
 
       switch (sockaddress->sa_family) {
       case AF_INET:
@@ -381,8 +390,8 @@ cont_bg_fetch(TSCont contp, TSEvent event, void * /* edata ATS_UNUSED */)
     }
 
     // Setup the NetVC for background fetch
-    TSAssert(NULL == data->vc);
-    if ((data->vc = TSHttpConnect((sockaddr *)&data->client_ip)) != NULL) {
+    TSAssert(nullptr == data->vc);
+    if ((data->vc = TSHttpConnectWithPluginId(reinterpret_cast<sockaddr *>(&data->client_ip), PLUGIN_NAME, 0)) != nullptr) {
       TSHttpHdrPrint(data->mbuf, data->hdr_loc, data->req_io_buf);
       // We never send a body with the request. ToDo: Do we ever need to support that ?
       TSIOBufferWrite(data->req_io_buf, "\r\n", 2);
@@ -428,7 +437,7 @@ cont_bg_fetch(TSCont contp, TSEvent event, void * /* edata ATS_UNUSED */)
     data->log(event);
 
     // Close, release and cleanup
-    data->vc = NULL;
+    data->vc = nullptr;
     delete data;
     break;
 
@@ -457,7 +466,7 @@ cont_check_cacheable(TSCont contp, TSEvent /* event ATS_UNUSED */, void *edata)
     if (TS_SUCCESS == TSHttpTxnClientReqGet(txnp, &request, &req_hdr)) {
       // Temporarily change the response status to 200 OK, so we can reevaluate cacheability.
       TSHttpHdrStatusSet(response, resp_hdr, TS_HTTP_STATUS_OK);
-      bool cacheable = TSHttpTxnIsCacheable(txnp, NULL, response);
+      bool cacheable = TSHttpTxnIsCacheable(txnp, nullptr, response);
       TSHttpHdrStatusSet(response, resp_hdr, TS_HTTP_STATUS_PARTIAL_CONTENT);
 
       TSDebug(PLUGIN_NAME, "Testing: request / response is cacheable?");
@@ -503,13 +512,13 @@ cont_handle_response(TSCont contp, TSEvent event, void *edata)
   TSHttpTxn txnp        = static_cast<TSHttpTxn>(edata);
   BgFetchConfig *config = static_cast<BgFetchConfig *>(TSContDataGet(contp));
 
-  if (NULL == config) {
+  if (nullptr == config) {
     // something seriously wrong..
     TSError("[%s] Can't get configurations", PLUGIN_NAME);
   } else {
     switch (event) {
     case TS_EVENT_HTTP_READ_RESPONSE_HDR:
-      if (config->getRules()->bgFetchAllowed(txnp)) {
+      if (config->bgFetchAllowed(txnp)) {
         TSMBuffer response;
         TSMLoc resp_hdr;
 
@@ -517,22 +526,20 @@ cont_handle_response(TSCont contp, TSEvent event, void *edata)
           // ToDo: Check the MIME type first, to see if it's a type we care about.
           // ToDo: Such MIME types should probably be per remap rule.
 
-          // Only deal with 206 responses from Origin
-          TSDebug(PLUGIN_NAME, "Testing: response is 206?");
-          if (TS_HTTP_STATUS_PARTIAL_CONTENT == TSHttpHdrStatusGet(response, resp_hdr)) {
+          // Only deal with 206 and possibly 304 responses from Origin
+          TSHttpStatus status = TSHttpHdrStatusGet(response, resp_hdr);
+          TSDebug(PLUGIN_NAME, "Testing: response status code: %d?", status);
+          if (TS_HTTP_STATUS_PARTIAL_CONTENT == status || (config->allow304() && TS_HTTP_STATUS_NOT_MODIFIED == status)) {
             // Everything looks good so far, add a TXN hook for SEND_RESPONSE_HDR
-            TSCont contp = TSContCreate(cont_check_cacheable, NULL);
+            TSCont localcontp = TSContCreate(cont_check_cacheable, nullptr);
 
-            TSHttpTxnHookAdd(txnp, TS_HTTP_SEND_RESPONSE_HDR_HOOK, contp);
+            TSHttpTxnHookAdd(txnp, TS_HTTP_SEND_RESPONSE_HDR_HOOK, localcontp);
           }
           // Release the response MLoc
           TSHandleMLocRelease(response, TS_NULL_MLOC, resp_hdr);
         }
       }
       break;
-    case TS_EVENT_HTTP_TXN_CLOSE:
-      config->release(); // Release the configuration lease when txn closes
-      break;
     default:
       TSError("[%s] Unknown event for this plugin", PLUGIN_NAME);
       TSDebug(PLUGIN_NAME, "unknown event for this plugin");
@@ -551,43 +558,31 @@ void
 TSPluginInit(int argc, const char *argv[])
 {
   TSPluginRegistrationInfo info;
-  static const struct option longopt[] = {{const_cast<char *>("log"), required_argument, NULL, 'l'},
-                                          {const_cast<char *>("config"), required_argument, NULL, 'c'},
-                                          {NULL, no_argument, NULL, '\0'}};
 
   info.plugin_name   = (char *)PLUGIN_NAME;
   info.vendor_name   = (char *)"Apache Software Foundation";
   info.support_email = (char *)"dev@trafficserver.apache.org";
 
   if (TS_SUCCESS != TSPluginRegister(&info)) {
-    TSError("[%s] Plugin registration failed.", PLUGIN_NAME);
+    TSError("[%s] Plugin registration failed", PLUGIN_NAME);
   }
 
-  TSCont cont = TSContCreate(cont_handle_response, NULL);
+  TSCont cont = TSContCreate(cont_handle_response, nullptr);
 
   gConfig = new BgFetchConfig(cont);
-  gConfig->acquire(); // Inc refcount, although this global config should never go out of scope
 
-  while (true) {
-    int opt = getopt_long(argc, (char *const *)argv, "lc", longopt, NULL);
-
-    switch (opt) {
-    case 'l':
-      BgFetchState::getInstance().createLog(optarg);
-      break;
-    case 'c':
-      TSDebug(PLUGIN_NAME, "config file %s..", optarg);
-      gConfig->readConfig(optarg);
-      break;
-    }
-
-    if (opt == -1) {
-      break;
+  if (gConfig->parseOptions(argc, argv)) {
+    // Create the global log file. Note that calling this multiple times currently has no
+    // effect, only one log file is ever created. The BgFetchState is a singleton.
+    if (!gConfig->logFile().empty()) {
+      BgFetchState::getInstance().createLog(gConfig->logFile());
     }
+    TSDebug(PLUGIN_NAME, "Initialized");
+    TSHttpHookAdd(TS_HTTP_READ_RESPONSE_HDR_HOOK, cont);
+  } else {
+    // ToDo: Hmmm, no way to fail a global plugin here?
+    TSDebug(PLUGIN_NAME, "Failed to initialize as global plugin");
   }
-
-  TSDebug(PLUGIN_NAME, "Initialized");
-  TSHttpHookAdd(TS_HTTP_READ_RESPONSE_HDR_HOOK, cont);
 }
 
 ///////////////////////////////////////////////////////////////////////////
@@ -605,7 +600,7 @@ TSRemapInit(TSRemapInterface *api_info, char *errbuf, int errbuf_size)
   }
 
   if (api_info->tsremap_version < TSREMAP_VERSION) {
-    snprintf(errbuf, errbuf_size - 1, "[TSRemapInit] - Incorrect API version %ld.%ld", api_info->tsremap_version >> 16,
+    snprintf(errbuf, errbuf_size, "[TSRemapInit] - Incorrect API version %ld.%ld", api_info->tsremap_version >> 16,
              (api_info->tsremap_version & 0xffff));
     return TS_ERROR;
   }
@@ -620,28 +615,49 @@ TSRemapInit(TSRemapInterface *api_info, char *errbuf, int errbuf_size)
 TSReturnCode
 TSRemapNewInstance(int argc, char *argv[], void **ih, char * /* errbuf */, int /* errbuf_size */)
 {
-  TSCont cont           = TSContCreate(cont_handle_response, NULL);
+  TSCont cont           = TSContCreate(cont_handle_response, nullptr);
   BgFetchConfig *config = new BgFetchConfig(cont);
+  bool success          = true;
 
-  config->acquire(); // Inc refcount
+  // The first two arguments are the "from" and "to" URL string. We need to
+  // skip them, but we also require that there be an option to masquerade as
+  // argv[0], so we increment the argument indexes by 1 rather than by 2.
+  argc--;
+  argv++;
 
-  // Parse the optional rules, wihch becomes a linked list of BgFetchRule's.
-  if (argc > 2) {
-    TSDebug(PLUGIN_NAME, "config file %s", argv[2]);
-    config->readConfig(argv[2]);
+  // This is for backwards compatibility, ugly! ToDo: Remove for ATS v9.0.0 IMO.
+  if (argc > 1 && *argv[1] != '-') {
+    TSDebug(PLUGIN_NAME, "config file %s", argv[1]);
+    if (!config->readConfig(argv[1])) {
+      success = false;
+    }
+  } else {
+    if (config->parseOptions(argc, const_cast<const char **>(argv))) {
+      // Create the global log file. Remember, the BgFetchState is a singleton.
+      if (config->logFile().size()) {
+        BgFetchState::getInstance().createLog(config->logFile());
+      }
+    } else {
+      success = false;
+    }
   }
 
-  *ih = (void *)config;
+  if (success) {
+    *ih = config;
 
-  return TS_SUCCESS;
+    return TS_SUCCESS;
+  }
+
+  // Something went wrong with the configuration setup.
+  delete config;
+  return TS_ERROR;
 }
 
 void
 TSRemapDeleteInstance(void *ih)
 {
   BgFetchConfig *config = static_cast<BgFetchConfig *>(ih);
-
-  config->release();
+  delete config;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -650,7 +666,7 @@ TSRemapDeleteInstance(void *ih)
 TSRemapStatus
 TSRemapDoRemap(void *ih, TSHttpTxn txnp, TSRemapRequestInfo * /* rri */)
 {
-  if (NULL == ih) {
+  if (nullptr == ih) {
     return TSREMAP_NO_REMAP;
   }
 
@@ -660,14 +676,15 @@ TSRemapDoRemap(void *ih, TSHttpTxn txnp, TSRemapRequestInfo * /* rri */)
   if (TS_SUCCESS == TSHttpTxnClientReqGet(txnp, &bufp, &req_hdrs)) {
     TSMLoc field_loc = TSMimeHdrFieldFind(bufp, req_hdrs, TS_MIME_FIELD_RANGE, TS_MIME_LEN_RANGE);
 
+    if (!field_loc) { // Less common case, but also allow If-Range header to trigger, but only if Range not present
+      field_loc = TSMimeHdrFieldFind(bufp, req_hdrs, TS_MIME_FIELD_IF_RANGE, TS_MIME_LEN_IF_RANGE);
+    }
+
     if (field_loc) {
       BgFetchConfig *config = static_cast<BgFetchConfig *>(ih);
 
-      config->acquire(); // Inc refcount
       TSHttpTxnHookAdd(txnp, TS_HTTP_READ_RESPONSE_HDR_HOOK, config->getCont());
-      TSHttpTxnHookAdd(txnp, TS_HTTP_TXN_CLOSE_HOOK, config->getCont());
-
-      TSDebug(PLUGIN_NAME, "background fetch TSRemapDoRemap...");
+      TSDebug(PLUGIN_NAME, "TSRemapDoRemap() added hook, request was Range / If-Range");
       TSHandleMLocRelease(bufp, req_hdrs, field_loc);
     }
     TSHandleMLocRelease(bufp, TS_NULL_MLOC, req_hdrs);
