@@ -21,7 +21,7 @@
   limitations under the License.
  */
 
-#include "ts/ink_defs.h"
+#include "tscore/ink_defs.h"
 
 #include <cstdio>
 #include <cstdlib>
@@ -61,7 +61,7 @@ struct OptionInfo {
 };
 
 static HandlerManager *gHandlerManager = nullptr;
-static Utils::HeaderValueList gWhitelistCookies;
+static Utils::HeaderValueList gAllowlistCookies;
 
 #define DEBUG_TAG "plugin_esi"
 #define PROCESSOR_DEBUG_TAG "plugin_esi_processor"
@@ -97,7 +97,7 @@ struct ContData {
   };
   STATE curr_state;
   TSVIO input_vio;
-  TSIOBufferReader input_reader;
+  TSIOBufferReader input_reader = nullptr;
   TSVIO output_vio;
   TSIOBuffer output_buffer;
   TSIOBufferReader output_reader;
@@ -108,7 +108,7 @@ struct ContData {
   EsiGunzip *esi_gunzip;
   TSCont contp;
   TSHttpTxn txnp;
-  const struct OptionInfo *option_info;
+  const struct OptionInfo *option_info = nullptr;
   char *request_url;
   sockaddr const *client_addr;
   DataType input_type;
@@ -175,7 +175,7 @@ public:
   {
     g_stat_indices[handle] = TSStatCreate(Stats::STAT_NAMES[handle], TS_RECORDDATATYPE_INT, TS_STAT_PERSISTENT, TS_STAT_SYNC_COUNT);
   }
-  //  void increment(int handle, TSMgmtInt step = 1) {
+
   void
   increment(int handle, int step = 1)
   {
@@ -256,7 +256,7 @@ ContData::init()
       data_fetcher = new HttpDataFetcherImpl(contp, client_addr, createDebugTag(FETCHER_DEBUG_TAG, contp, fetcher_tag));
     }
     if (!esi_vars) {
-      esi_vars = new Variables(createDebugTag(VARS_DEBUG_TAG, contp, vars_tag), &TSDebug, &TSError, gWhitelistCookies);
+      esi_vars = new Variables(createDebugTag(VARS_DEBUG_TAG, contp, vars_tag), &TSDebug, &TSError, gAllowlistCookies);
     }
 
     esi_proc = new EsiProcessor(
@@ -290,7 +290,7 @@ ContData::getClientState()
 
   if (!esi_vars) {
     string vars_tag;
-    esi_vars = new Variables(createDebugTag(VARS_DEBUG_TAG, contp, vars_tag), &TSDebug, &TSError, gWhitelistCookies);
+    esi_vars = new Variables(createDebugTag(VARS_DEBUG_TAG, contp, vars_tag), &TSDebug, &TSError, gAllowlistCookies);
   }
   if (!data_fetcher) {
     string fetcher_tag;
@@ -333,7 +333,7 @@ ContData::getClientState()
           if (n_values == 1) {
             value = TSMimeHdrFieldValueStringGet(req_bufp, req_hdr_loc, field_loc, 0, &value_len);
 
-            if (nullptr != value || value_len) {
+            if (nullptr != value && value_len) {
               if (Utils::areEqual(name, name_len, TS_MIME_FIELD_ACCEPT_ENCODING, TS_MIME_LEN_ACCEPT_ENCODING) &&
                   Utils::areEqual(value, value_len, TS_HTTP_VALUE_GZIP, TS_HTTP_LEN_GZIP)) {
                 gzip_output = true;
@@ -342,7 +342,7 @@ ContData::getClientState()
           } else {
             for (int i = 0; i < n_values; ++i) {
               value = TSMimeHdrFieldValueStringGet(req_bufp, req_hdr_loc, field_loc, i, &value_len);
-              if (nullptr != value || value_len) {
+              if (nullptr != value && value_len) {
                 if (Utils::areEqual(name, name_len, TS_MIME_FIELD_ACCEPT_ENCODING, TS_MIME_LEN_ACCEPT_ENCODING) &&
                     Utils::areEqual(value, value_len, TS_HTTP_VALUE_GZIP, TS_HTTP_LEN_GZIP)) {
                   gzip_output = true;
@@ -711,9 +711,10 @@ transformData(TSCont contp)
     }
   }
   if (process_input_complete) {
-    TSDebug(cont_data->debug_tag, "[%s] Completed reading input...", __FUNCTION__);
+    TSDebug(cont_data->debug_tag, "[%s] Completed reading input", __FUNCTION__);
     if (cont_data->input_type == DATA_TYPE_PACKED_ESI) {
-      TSDebug(DEBUG_TAG, "[%s] Going to use packed node list of size %d", __FUNCTION__, (int)cont_data->packed_node_list.size());
+      TSDebug(DEBUG_TAG, "[%s] Going to use packed node list of size %d", __FUNCTION__,
+              static_cast<int>(cont_data->packed_node_list.size()));
       if (cont_data->esi_proc->usePackedNodeList(cont_data->packed_node_list) == EsiProcessor::UNPACK_FAILURE) {
         removeCacheKey(cont_data->txnp);
 
@@ -751,8 +752,9 @@ transformData(TSCont contp)
       EsiProcessor::ReturnCode retval = cont_data->esi_proc->process(out_data, out_data_len);
       TSDebug(cont_data->debug_tag, "[%s] data length: %d, retval: %d", __FUNCTION__, out_data_len, retval);
       if (retval == EsiProcessor::NEED_MORE_DATA) {
-        TSDebug(cont_data->debug_tag, "[%s] ESI processor needs more data; "
-                                      "will wait for all data to be fetched",
+        TSDebug(cont_data->debug_tag,
+                "[%s] ESI processor needs more data; "
+                "will wait for all data to be fetched",
                 __FUNCTION__);
         return 1;
       }
@@ -776,7 +778,7 @@ transformData(TSCont contp)
             out_data     = "";
           } else {
             TSDebug(cont_data->debug_tag, "[%s] Compressed document from size %d to %d bytes via gzip", __FUNCTION__, out_data_len,
-                    (int)cdata.size());
+                    static_cast<int>(cdata.size()));
             out_data_len = cdata.size();
             out_data     = cdata.data();
           }
@@ -823,7 +825,7 @@ transformData(TSCont contp)
 
     if (retval == EsiProcessor::SUCCESS) {
       TSDebug(cont_data->debug_tag, "[%s] ESI processor output document of size %d starting with [%.10s]", __FUNCTION__,
-              (int)out_data.size(), (out_data.size() ? out_data.data() : "(null)"));
+              static_cast<int>(out_data.size()), (out_data.size() ? out_data.data() : "(null)"));
     } else {
       TSError("[esi][%s] ESI processor failed to process document; will return empty document", __FUNCTION__);
       out_data.assign("");
@@ -841,7 +843,7 @@ transformData(TSCont contp)
           TSError("[esi][%s] Error while gzipping content", __FUNCTION__);
         } else {
           TSDebug(cont_data->debug_tag, "[%s] Compressed document from size %d to %d bytes via EsiGzip", __FUNCTION__,
-                  (int)out_data.size(), (int)cdata.size());
+                  static_cast<int>(out_data.size()), static_cast<int>(cdata.size()));
         }
         if (TSIOBufferWrite(TSVIOBufferGet(cont_data->output_vio), cdata.data(), cdata.size()) == TS_ERROR) {
           TSError("[esi][%s] Error while writing bytes to downstream VC", __FUNCTION__);
@@ -916,7 +918,7 @@ transformHandler(TSCont contp, TSEvent event, void *edata)
   is_fetch_event = cont_data->data_fetcher->isFetchEvent(event);
 
   if (cont_data->xform_closed) {
-    TSDebug(cont_debug_tag, "[%s] Transformation closed. Post-processing...", __FUNCTION__);
+    TSDebug(cont_debug_tag, "[%s] Transformation closed, post-processing", __FUNCTION__);
     if (cont_data->curr_state == ContData::PROCESSING_COMPLETE) {
       TSDebug(cont_debug_tag, "[%s] Processing is complete, not processing current event %d", __FUNCTION__, event);
       process_event = false;
@@ -972,13 +974,13 @@ transformHandler(TSCont contp, TSEvent event, void *edata)
       break;
 
     case TS_EVENT_IMMEDIATE:
-      TSDebug(cont_debug_tag, "[%s] handling TS_EVENT_IMMEDIATE...", __FUNCTION__);
+      TSDebug(cont_debug_tag, "[%s] handling TS_EVENT_IMMEDIATE", __FUNCTION__);
       transformData(contp);
       break;
 
     default:
       if (is_fetch_event) {
-        TSDebug(cont_debug_tag, "[%s] Handling fetch event %d...", __FUNCTION__, event);
+        TSDebug(cont_debug_tag, "[%s] Handling fetch event %d", __FUNCTION__, event);
         if (cont_data->data_fetcher->handleFetchEvent(event, edata)) {
           if ((cont_data->curr_state == ContData::FETCHING_DATA) || (cont_data->curr_state == ContData::READING_ESI_DOC)) {
             // there's a small chance that fetcher is ready even before
@@ -998,8 +1000,8 @@ transformHandler(TSCont contp, TSEvent event, void *edata)
     }
   }
 
-  TSDebug(cont_data->debug_tag, "[%s] transformHandler, event: %d, curr_state: %d", __FUNCTION__, (int)event,
-          (int)cont_data->curr_state);
+  TSDebug(cont_data->debug_tag, "[%s] transformHandler, event: %d, curr_state: %d", __FUNCTION__, static_cast<int>(event),
+          static_cast<int>(cont_data->curr_state));
 
   shutdown = (cont_data->xform_closed && (cont_data->curr_state == ContData::PROCESSING_COMPLETE));
   if (shutdown) {
@@ -1008,7 +1010,7 @@ transformHandler(TSCont contp, TSEvent event, void *edata)
       // lock on our continuation which will fail if we destroy
       // ourselves right now
       TSDebug(cont_debug_tag, "[%s] Deferring shutdown as data event was just processed", __FUNCTION__);
-      TSContSchedule(contp, 10, TS_THREAD_POOL_TASK);
+      TSContScheduleOnPool(contp, 10, TS_THREAD_POOL_TASK);
     } else {
       goto lShutdown;
     }
@@ -1017,7 +1019,7 @@ transformHandler(TSCont contp, TSEvent event, void *edata)
   return 1;
 
 lShutdown:
-  TSDebug(cont_data->debug_tag, "[%s] transformation closed; cleaning up data...", __FUNCTION__);
+  TSDebug(cont_data->debug_tag, "[%s] transformation closed; cleaning up data", __FUNCTION__);
   delete cont_data;
   TSContDestroy(contp);
   return 1;
@@ -1125,9 +1127,8 @@ modifyResponseHeader(TSCont contp, TSEvent event, void *edata)
       }
       TSHandleMLocRelease(bufp, hdr_loc, field_loc);
     }
-    if (mod_data->gzip_encoding &&
-        !checkHeaderValue(bufp, hdr_loc, TS_MIME_FIELD_CONTENT_ENCODING, TS_MIME_LEN_CONTENT_ENCODING, TS_HTTP_VALUE_GZIP,
-                          TS_HTTP_LEN_GZIP)) {
+    if (mod_data->gzip_encoding && !checkHeaderValue(bufp, hdr_loc, TS_MIME_FIELD_CONTENT_ENCODING, TS_MIME_LEN_CONTENT_ENCODING,
+                                                     TS_HTTP_VALUE_GZIP, TS_HTTP_LEN_GZIP)) {
       addMimeHeaderField(bufp, hdr_loc, TS_MIME_FIELD_CONTENT_ENCODING, TS_MIME_LEN_CONTENT_ENCODING, TS_HTTP_VALUE_GZIP,
                          TS_HTTP_LEN_GZIP);
     }
@@ -1208,7 +1209,7 @@ maskOsCacheHeaders(TSHttpTxn txnp)
   TSMLoc field_loc;
   const char *name, *value;
   int name_len, value_len, n_field_values;
-  bool os_response_cacheable, is_cache_header, mask_header;
+  bool os_response_cacheable, mask_header;
   string masked_name;
   os_response_cacheable = true;
   for (int i = 0; i < n_mime_headers; ++i) {
@@ -1219,14 +1220,14 @@ maskOsCacheHeaders(TSHttpTxn txnp)
     }
     name = TSMimeHdrFieldNameGet(bufp, hdr_loc, field_loc, &name_len);
     if (name) {
-      mask_header = is_cache_header = false;
-      n_field_values                = TSMimeHdrFieldValuesCount(bufp, hdr_loc, field_loc);
+      mask_header    = false;
+      n_field_values = TSMimeHdrFieldValuesCount(bufp, hdr_loc, field_loc);
       for (int j = 0; j < n_field_values; ++j) {
         value = TSMimeHdrFieldValueStringGet(bufp, hdr_loc, field_loc, j, &value_len);
         if (nullptr == value || !value_len) {
           TSDebug(DEBUG_TAG, "[%s] Error while getting value #%d of header [%.*s]", __FUNCTION__, j, name_len, name);
         } else {
-          is_cache_header = checkForCacheHeader(name, name_len, value, value_len, os_response_cacheable);
+          bool is_cache_header = checkForCacheHeader(name, name_len, value, value_len, os_response_cacheable);
           if (!os_response_cacheable) {
             break;
           }
@@ -1493,15 +1494,15 @@ pthread_key_t threadKey = 0;
 static int
 globalHookHandler(TSCont contp, TSEvent event, void *edata)
 {
-  TSHttpTxn txnp                 = (TSHttpTxn)edata;
+  TSHttpTxn txnp                 = static_cast<TSHttpTxn>(edata);
   bool intercept_header          = false;
   bool head_only                 = false;
   bool intercept_req             = isInterceptRequest(txnp);
-  struct OptionInfo *pOptionInfo = (struct OptionInfo *)TSContDataGet(contp);
+  struct OptionInfo *pOptionInfo = static_cast<struct OptionInfo *>(TSContDataGet(contp));
 
   switch (event) {
   case TS_EVENT_HTTP_READ_REQUEST_HDR:
-    TSDebug(DEBUG_TAG, "[%s] handling read request header event...", __FUNCTION__);
+    TSDebug(DEBUG_TAG, "[%s] handling read request header event", __FUNCTION__);
     if (intercept_req) {
       if (!setupServerIntercept(txnp)) {
         TSError("[esi][%s] Could not setup server intercept", __FUNCTION__);
@@ -1518,14 +1519,8 @@ globalHookHandler(TSCont contp, TSEvent event, void *edata)
     if (!intercept_req) {
       if (event == TS_EVENT_HTTP_READ_RESPONSE_HDR) {
         bool mask_cache_headers = false;
-        TSDebug(DEBUG_TAG, "[%s] handling read response header event...", __FUNCTION__);
-        if (isCacheObjTransformable(txnp, &intercept_header, &head_only)) {
-          // transformable cache object will definitely have a
-          // transformation already as cache_lookup_complete would
-          // have been processed before this
-          TSDebug(DEBUG_TAG, "[%s] xform should already have been added on cache lookup. Not adding now", __FUNCTION__);
-          mask_cache_headers = true;
-        } else if (isTxnTransformable(txnp, false, &intercept_header, &head_only)) {
+        TSDebug(DEBUG_TAG, "[%s] handling read response header event", __FUNCTION__);
+        if (isTxnTransformable(txnp, false, &intercept_header, &head_only)) {
           addTransform(txnp, true, intercept_header, head_only, pOptionInfo);
           Stats::increment(Stats::N_OS_DOCS);
           mask_cache_headers = true;
@@ -1537,10 +1532,10 @@ globalHookHandler(TSCont contp, TSEvent event, void *edata)
           maskOsCacheHeaders(txnp);
         }
       } else {
-        TSDebug(DEBUG_TAG, "[%s] handling cache lookup complete event...", __FUNCTION__);
+        TSDebug(DEBUG_TAG, "[%s] handling cache lookup complete event", __FUNCTION__);
         if (isCacheObjTransformable(txnp, &intercept_header, &head_only)) {
           // we make the assumption above that a transformable cache
-          // object would already have a tranformation. We should revisit
+          // object would already have a transformation. We should revisit
           // that assumption in case we change the statement below
           addTransform(txnp, false, intercept_header, head_only, pOptionInfo);
           Stats::increment(Stats::N_CACHE_DOCS);
@@ -1569,7 +1564,7 @@ loadHandlerConf(const char *file_name, Utils::KeyValueMap &handler_conf)
       conf_lines.push_back(string(buf));
     }
     TSfclose(conf_file);
-    Utils::parseKeyValueConfig(conf_lines, handler_conf, gWhitelistCookies);
+    Utils::parseKeyValueConfig(conf_lines, handler_conf, gAllowlistCookies);
     TSDebug(DEBUG_TAG, "[%s] Loaded handler conf file [%s]", __FUNCTION__, file_name);
   } else {
     TSError("[esi][%s] Failed to open handler config file [%s]", __FUNCTION__, file_name);
@@ -1605,7 +1600,7 @@ esiPluginInit(int argc, const char *argv[], struct OptionInfo *pOptionInfo)
     };
 
     int longindex = 0;
-    while ((c = getopt_long(argc, (char *const *)argv, "npzbf:", longopts, &longindex)) != -1) {
+    while ((c = getopt_long(argc, const_cast<char *const *>(argv), "npzbf:", longopts, &longindex)) != -1) {
       switch (c) {
       case 'n':
         pOptionInfo->packed_node_support = true;
@@ -1644,9 +1639,10 @@ esiPluginInit(int argc, const char *argv[], struct OptionInfo *pOptionInfo)
   }
 
   if (result == 0) {
-    TSDebug(DEBUG_TAG, "[%s] Plugin started%s, "
-                       "packed-node-support: %d, private-response: %d, "
-                       "disable-gzip-output: %d, first-byte-flush: %d ",
+    TSDebug(DEBUG_TAG,
+            "[%s] Plugin started%s, "
+            "packed-node-support: %d, private-response: %d, "
+            "disable-gzip-output: %d, first-byte-flush: %d ",
             __FUNCTION__, bKeySet ? " and key is set" : "", pOptionInfo->packed_node_support, pOptionInfo->private_response,
             pOptionInfo->disable_gzip_output, pOptionInfo->first_byte_flush);
   }
@@ -1663,13 +1659,13 @@ TSPluginInit(int argc, const char *argv[])
   info.support_email = (char *)"dev@trafficserver.apache.org";
 
   if (TSPluginRegister(&info) != TS_SUCCESS) {
-    TSError("[esi][%s] plugin registration failed.", __FUNCTION__);
+    TSError("[esi][%s] plugin registration failed", __FUNCTION__);
     return;
   }
 
-  struct OptionInfo *pOptionInfo = (struct OptionInfo *)TSmalloc(sizeof(struct OptionInfo));
+  struct OptionInfo *pOptionInfo = static_cast<struct OptionInfo *>(TSmalloc(sizeof(struct OptionInfo)));
   if (pOptionInfo == nullptr) {
-    TSError("[esi][%s] malloc %d bytes fail", __FUNCTION__, (int)sizeof(struct OptionInfo));
+    TSError("[esi][%s] malloc %d bytes fail", __FUNCTION__, static_cast<int>(sizeof(struct OptionInfo)));
     return;
   }
   if (esiPluginInit(argc, argv, pOptionInfo) != 0) {
@@ -1716,8 +1712,9 @@ TSReturnCode
 TSRemapNewInstance(int argc, char *argv[], void **ih, char *errbuf, int errbuf_size)
 {
   if (argc < 2) {
-    snprintf(errbuf, errbuf_size, "Unable to create remap instance, "
-                                  "argc: %d < 2",
+    snprintf(errbuf, errbuf_size,
+             "Unable to create remap instance, "
+             "argc: %d < 2",
              argc);
     TSError("[esi]Unable to create remap instance! argc: %d < 2", argc);
     return TS_ERROR;
@@ -1732,10 +1729,10 @@ TSRemapNewInstance(int argc, char *argv[], void **ih, char *errbuf, int errbuf_s
   }
   new_argv[index] = nullptr;
 
-  struct OptionInfo *pOptionInfo = (struct OptionInfo *)TSmalloc(sizeof(struct OptionInfo));
+  struct OptionInfo *pOptionInfo = static_cast<struct OptionInfo *>(TSmalloc(sizeof(struct OptionInfo)));
   if (pOptionInfo == nullptr) {
-    snprintf(errbuf, errbuf_size, "malloc %d bytes fail", (int)sizeof(struct OptionInfo));
-    TSError("[esi][%s] malloc %d bytes fail", __FUNCTION__, (int)sizeof(struct OptionInfo));
+    snprintf(errbuf, errbuf_size, "malloc %d bytes fail", static_cast<int>(sizeof(struct OptionInfo)));
+    TSError("[esi][%s] malloc %d bytes fail", __FUNCTION__, static_cast<int>(sizeof(struct OptionInfo)));
     return TS_ERROR;
   }
   if (esiPluginInit(index, new_argv, pOptionInfo) != 0) {
