@@ -24,7 +24,7 @@
 #include "EsiParser.h"
 #include "Utils.h"
 
-#include <ctype.h>
+#include <cctype>
 
 using std::string;
 using namespace EsiLib;
@@ -265,7 +265,6 @@ EsiParser::_parse(const string &data, int &parse_start_pos, DocNodeList &node_li
 {
   size_t orig_list_size = node_list.size();
   size_t curr_pos, end_pos;
-  const char *data_ptr;
   const char *const data_start_ptr = data.data();
   size_t data_size                 = data.size();
   const EsiNodeInfo *node_info;
@@ -300,12 +299,10 @@ EsiParser::_parse(const string &data, int &parse_start_pos, DocNodeList &node_li
 
     if (is_html_comment_node) {
       _debugLog(_debug_tag, "[%s] Found html comment tag at position %d", __FUNCTION__, curr_pos);
-      data_ptr  = data_start_ptr + curr_pos;
       node_info = &HTML_COMMENT_NODE_INFO;
       ++curr_pos;
     } else {
       curr_pos += ESI_TAG_PREFIX_LEN;
-      data_ptr = data_start_ptr + curr_pos;
 
       for (node_info = ESI_NODES; node_info->type != DocNode::TYPE_UNKNOWN; ++node_info) {
         search_result = _compareData(data, curr_pos, node_info->tag_suffix, node_info->tag_suffix_len);
@@ -336,7 +333,7 @@ EsiParser::_parse(const string &data, int &parse_start_pos, DocNodeList &node_li
         }
       }
       if (node_info->type == DocNode::TYPE_UNKNOWN) {
-        _errorLog("[%s] Unknown ESI tag starting with [%.10s]...", __FUNCTION__, data_ptr - ESI_TAG_PREFIX_LEN);
+        _errorLog("[%s] Unknown ESI tag starting with [%10s]...", __FUNCTION__, data.c_str());
         goto lFail;
       }
     }
@@ -346,8 +343,8 @@ EsiParser::_parse(const string &data, int &parse_start_pos, DocNodeList &node_li
 
     if ((search_result == NO_MATCH) || (search_result == PARTIAL_MATCH)) {
       if (last_chunk) {
-        _errorLog("[%s] ESI tag starting with [%.10s]... has no matching closing tag [%.*s]", __FUNCTION__,
-                  data_ptr - ESI_TAG_PREFIX_LEN, node_info->closing_tag_len, node_info->closing_tag);
+        _errorLog("[%s] ESI tag starting with [%10s]... has no matching closing tag [%.*s]", __FUNCTION__, data.c_str(),
+                  node_info->closing_tag_len, node_info->closing_tag);
         goto lFail;
       } else {
         goto lPartialMatch;
@@ -600,9 +597,7 @@ EsiParser::clear()
   _parse_start_pos = -1;
 }
 
-EsiParser::~EsiParser()
-{
-}
+EsiParser::~EsiParser() = default;
 
 inline void
 EsiParser::_adjustPointers(DocNodeList::iterator node_iter, DocNodeList::iterator end, const char *ext_data_ptr,
