@@ -22,7 +22,7 @@
  */
 
 #include <iostream>
-#include <assert.h>
+#include <cassert>
 #include <string>
 
 #include "EsiProcessor.h"
@@ -44,11 +44,11 @@ static const int FETCHER_STATIC_DATA_SIZE = 30;
 int
 main()
 {
-  Utils::HeaderValueList whitelistCookies;
-  Variables esi_vars("vars", &Debug, &Error, whitelistCookies);
+  Utils::HeaderValueList allowlistCookies;
+  Variables esi_vars("vars", &Debug, &Error, allowlistCookies);
   HandlerManager handler_mgr("handler_mgr", &Debug, &Error);
 
-  pthread_key_create(&threadKey, NULL);
+  pthread_key_create(&threadKey, nullptr);
   Utils::init(&Debug, &Error);
 
   {
@@ -280,9 +280,10 @@ main()
     assert(esi_proc.completeParse() == true);
     assert(esi_proc.process(output_data, output_data_len) == EsiProcessor::SUCCESS);
     assert(output_data_len == 11 + 4 + FETCHER_STATIC_DATA_SIZE + 11 + 4 + FETCHER_STATIC_DATA_SIZE + 9 + FETCHER_STATIC_DATA_SIZE);
-    assert(strncmp(output_data, "foo1 >>>>> Content for URL [url1] <<<<< bar1\n"
-                                "foo2 >>>>> Content for URL [url2] <<<<< bar2\n"
-                                ">>>>> Content for URL [blah bleh] <<<<<",
+    assert(strncmp(output_data,
+                   "foo1 >>>>> Content for URL [url1] <<<<< bar1\n"
+                   "foo2 >>>>> Content for URL [url2] <<<<< bar2\n"
+                   ">>>>> Content for URL [blah bleh] <<<<<",
                    output_data_len) == 0);
   }
 
@@ -303,9 +304,10 @@ main()
     assert(esi_proc.completeParse() == true);
     assert(esi_proc.process(output_data, output_data_len) == EsiProcessor::SUCCESS);
     assert(output_data_len == 11 + 4 + FETCHER_STATIC_DATA_SIZE + 11 + 4 + FETCHER_STATIC_DATA_SIZE + 9 + FETCHER_STATIC_DATA_SIZE);
-    assert(strncmp(output_data, "foo1 >>>>> Content for URL [url1] <<<<< bar1\n"
-                                "foo2 >>>>> Content for URL [url2] <<<<< bar2\n"
-                                ">>>>> Content for URL [blah bleh] <<<<<",
+    assert(strncmp(output_data,
+                   "foo1 >>>>> Content for URL [url1] <<<<< bar1\n"
+                   "foo2 >>>>> Content for URL [url2] <<<<< bar2\n"
+                   ">>>>> Content for URL [blah bleh] <<<<<",
                    output_data_len) == 0);
   }
 
@@ -332,9 +334,10 @@ main()
     assert(esi_proc.completeParse() == true);
     assert(esi_proc.process(output_data, output_data_len) == EsiProcessor::SUCCESS);
     assert(output_data_len == 11 + 4 + FETCHER_STATIC_DATA_SIZE + 11 + 4 + FETCHER_STATIC_DATA_SIZE + 9 + FETCHER_STATIC_DATA_SIZE);
-    assert(strncmp(output_data, "foo1 >>>>> Content for URL [url1] <<<<< bar1\n"
-                                "foo2 >>>>> Content for URL [url2] <<<<< bar2\n"
-                                ">>>>> Content for URL [blah bleh] <<<<<",
+    assert(strncmp(output_data,
+                   "foo1 >>>>> Content for URL [url1] <<<<< bar1\n"
+                   "foo2 >>>>> Content for URL [url2] <<<<< bar2\n"
+                   ">>>>> Content for URL [blah bleh] <<<<<",
                    output_data_len) == 0);
   }
 
@@ -354,9 +357,10 @@ main()
     assert(esi_proc.completeParse(input_data.data(), input_data.size()) == true);
     assert(esi_proc.process(output_data, output_data_len) == EsiProcessor::SUCCESS);
     assert(output_data_len == 11 + 4 + FETCHER_STATIC_DATA_SIZE + 11 + 4 + FETCHER_STATIC_DATA_SIZE + 9 + FETCHER_STATIC_DATA_SIZE);
-    assert(strncmp(output_data, "foo1 >>>>> Content for URL [url1] <<<<< bar1\n"
-                                "foo2 >>>>> Content for URL [url2] <<<<< bar2\n"
-                                ">>>>> Content for URL [blah bleh] <<<<<",
+    assert(strncmp(output_data,
+                   "foo1 >>>>> Content for URL [url1] <<<<< bar1\n"
+                   "foo2 >>>>> Content for URL [url2] <<<<< bar2\n"
+                   ">>>>> Content for URL [blah bleh] <<<<<",
                    output_data_len) == 0);
   }
 
@@ -382,9 +386,10 @@ main()
     assert(esi_proc.completeParse(line6, sizeof(line6) - 1) == true);
     assert(esi_proc.process(output_data, output_data_len) == EsiProcessor::SUCCESS);
     assert(output_data_len == 11 + 4 + FETCHER_STATIC_DATA_SIZE + 11 + 4 + FETCHER_STATIC_DATA_SIZE + 9 + FETCHER_STATIC_DATA_SIZE);
-    assert(strncmp(output_data, "foo1 >>>>> Content for URL [url1] <<<<< bar1\n"
-                                "foo2 >>>>> Content for URL [url2] <<<<< bar2\n"
-                                ">>>>> Content for URL [blah bleh] <<<<<",
+    assert(strncmp(output_data,
+                   "foo1 >>>>> Content for URL [url1] <<<<< bar1\n"
+                   "foo2 >>>>> Content for URL [url2] <<<<< bar2\n"
+                   ">>>>> Content for URL [blah bleh] <<<<<",
                    output_data_len) == 0);
   }
 
@@ -436,9 +441,10 @@ main()
     assert(esi_proc.completeParse(input_data) == true);
     assert(esi_proc.process(output_data, output_data_len) == EsiProcessor::SUCCESS);
     assert(output_data_len == 11 + 4 + FETCHER_STATIC_DATA_SIZE + 11 + 4 + FETCHER_STATIC_DATA_SIZE + 9 + FETCHER_STATIC_DATA_SIZE);
-    assert(strncmp(output_data, "foo1 >>>>> Content for URL [url1] <<<<< bar1\n"
-                                "foo2 >>>>> Content for URL [url2] <<<<< bar2\n"
-                                ">>>>> Content for URL [blah bleh] <<<<<",
+    assert(strncmp(output_data,
+                   "foo1 >>>>> Content for URL [url1] <<<<< bar1\n"
+                   "foo2 >>>>> Content for URL [url2] <<<<< bar2\n"
+                   ">>>>> Content for URL [blah bleh] <<<<<",
                    output_data_len) == 0);
   }
 
@@ -1018,7 +1024,7 @@ main()
     assert(strncmp(output_data, "Special data for include id 2", output_data_len - StubIncludeHandler::FOOTER_SIZE) == 0);
     assert(strncmp(output_data + StubIncludeHandler::DATA_PREFIX_SIZE + 1, StubIncludeHandler::FOOTER,
                    StubIncludeHandler::FOOTER_SIZE) == 0);
-    StubIncludeHandler::FOOTER      = 0;
+    StubIncludeHandler::FOOTER      = nullptr;
     StubIncludeHandler::FOOTER_SIZE = 0;
   }
 
@@ -1041,7 +1047,7 @@ main()
 
     EsiProcessor esi_proc("processor", "parser", "expression", &Debug, &Error, data_fetcher, esi_vars, handler_mgr);
 
-    assert(esi_proc.usePackedNodeList(0, packedNodeList.size()) == EsiProcessor::UNPACK_FAILURE);
+    assert(esi_proc.usePackedNodeList(nullptr, packedNodeList.size()) == EsiProcessor::UNPACK_FAILURE);
   }
 
   {
