@@ -20,11 +20,11 @@
   See the License for the specific language governing permissions and
   limitations under the License.
  */
-#include <stdio.h>
+#include <cstdio>
 #include <iostream>
-#include <assert.h>
+#include <cassert>
 #include <string>
-#include <stdarg.h>
+#include <cstdarg>
 
 #include "print_funcs.h"
 #include "Variables.h"
@@ -70,18 +70,18 @@ fakeDebug(const char *tag, const char *fmt, ...)
 int
 main()
 {
-  pthread_key_create(&threadKey, NULL);
+  pthread_key_create(&threadKey, nullptr);
   Utils::init(&Debug, &Error);
 
   {
     cout << endl << "===================== Test 1" << endl;
-    Utils::HeaderValueList whitelistCookies;
-    whitelistCookies.push_back("c1");
-    whitelistCookies.push_back("c2");
-    whitelistCookies.push_back("c3");
-    whitelistCookies.push_back("c4");
-    whitelistCookies.push_back("c5");
-    Variables esi_vars("vars_test", &Debug, &Error, whitelistCookies);
+    Utils::HeaderValueList allowlistCookies;
+    allowlistCookies.push_back("c1");
+    allowlistCookies.push_back("c2");
+    allowlistCookies.push_back("c3");
+    allowlistCookies.push_back("c4");
+    allowlistCookies.push_back("c5");
+    Variables esi_vars("vars_test", &Debug, &Error, allowlistCookies);
     const char *strings[] = {"Cookie",
                              "; c1=v1; c2=v2; ;   c3; c4=;    c5=v5  ",
                              "Host",
@@ -94,7 +94,7 @@ main()
                              "en-gb , en-us ,  ,",
                              "Accept-Language",
                              "ka-in",
-                             0};
+                             nullptr};
 
     HttpHeaderList headers;
     addToHeaderList(strings, headers);
@@ -135,8 +135,9 @@ main()
     esi_vars.populate(HttpHeader("hosT", -1, "localhost", -1));
     assert(esi_vars.getValue("HTTP_HOST") == "localhost");
 
-    esi_vars.populate(HttpHeader("User-agent", -1, "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.6) "
-                                                   "Gecko/20091201 Firefox/3.5.6 (.NETgecko CLR 3.5.30729)",
+    esi_vars.populate(HttpHeader("User-agent", -1,
+                                 "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.6) "
+                                 "Gecko/20091201 Firefox/3.5.6 (.NETgecko CLR 3.5.30729)",
                                  -1));
 
     /*
@@ -156,7 +157,7 @@ main()
     esi_vars.populate("a=b&c=d&e=f");
 
     Expression esi_expr("vars_test", &Debug, &Error, esi_vars);
-    assert(esi_expr.expand(0) == "");
+    assert(esi_expr.expand(nullptr) == "");
     assert(esi_expr.expand("") == "");
     assert(esi_expr.expand("blah") == "blah");
     assert(esi_expr.expand("blah$(HTTP_HOST") == "");
@@ -313,8 +314,8 @@ main()
   {
     cout << endl << "===================== Test 2" << endl;
     gFakeDebugLog.assign("");
-    Utils::HeaderValueList whitelistCookies;
-    Variables esi_vars("vars_test", &fakeDebug, &Error, whitelistCookies);
+    Utils::HeaderValueList allowlistCookies;
+    Variables esi_vars("vars_test", &fakeDebug, &Error, allowlistCookies);
 
     esi_vars.populate(HttpHeader("Host", -1, "example.com", -1));
     esi_vars.populate(HttpHeader("Referer", -1, "google.com", -1));
@@ -343,17 +344,17 @@ main()
 
   {
     cout << endl << "===================== Test 3" << endl;
-    Utils::HeaderValueList whitelistCookies;
-    whitelistCookies.push_back("age");
-    whitelistCookies.push_back("grade");
-    whitelistCookies.push_back("avg");
-    whitelistCookies.push_back("t1");
-    whitelistCookies.push_back("t2");
-    whitelistCookies.push_back("t3");
-    whitelistCookies.push_back("t4");
-    whitelistCookies.push_back("t5");
-    whitelistCookies.push_back("c1");
-    Variables esi_vars("vars_test", &Debug, &Error, whitelistCookies);
+    Utils::HeaderValueList allowlistCookies;
+    allowlistCookies.push_back("age");
+    allowlistCookies.push_back("grade");
+    allowlistCookies.push_back("avg");
+    allowlistCookies.push_back("t1");
+    allowlistCookies.push_back("t2");
+    allowlistCookies.push_back("t3");
+    allowlistCookies.push_back("t4");
+    allowlistCookies.push_back("t5");
+    allowlistCookies.push_back("c1");
+    Variables esi_vars("vars_test", &Debug, &Error, allowlistCookies);
 
     esi_vars.populate(HttpHeader("Host", -1, "example.com", -1));
     esi_vars.populate(HttpHeader("Referer", -1, "google.com", -1));
@@ -389,15 +390,15 @@ main()
 
   {
     cout << endl << "===================== Test 4" << endl;
-    Utils::HeaderValueList whitelistCookies;
-    whitelistCookies.push_back("FPS");
-    whitelistCookies.push_back("mb");
-    whitelistCookies.push_back("Y");
-    whitelistCookies.push_back("C");
-    whitelistCookies.push_back("F");
-    whitelistCookies.push_back("a");
-    whitelistCookies.push_back("c");
-    Variables esi_vars("vars_test", &Debug, &Error, whitelistCookies);
+    Utils::HeaderValueList allowlistCookies;
+    allowlistCookies.push_back("FPS");
+    allowlistCookies.push_back("mb");
+    allowlistCookies.push_back("Y");
+    allowlistCookies.push_back("C");
+    allowlistCookies.push_back("F");
+    allowlistCookies.push_back("a");
+    allowlistCookies.push_back("c");
+    Variables esi_vars("vars_test", &Debug, &Error, allowlistCookies);
     string cookie_str("FPS=dl; mb=d=OPsv7rvU4FFaAOoIRi75BBuqdMdbMLFuDwQmk6nKrCgno7L4xuN44zm7QBQJRmQSh8ken6GSVk8-&v=1; C=mg=1; "
                       "Y=v=1&n=fmaptagvuff50&l=fc0d94i7/o&p=m2f0000313000400&r=8j&lg=en-US&intl=us; "
                       "F=a=4KvLV9IMvTJnIAqCk25y9Use6hnPALtUf3n78PihlcIqvmzoW.Ax8UyW8_oxtgFNrrdmooqZmPa7WsX4gE."
@@ -439,24 +440,26 @@ main()
 
   {
     cout << endl << "===================== Test 5" << endl;
-    Utils::HeaderValueList whitelistCookies;
-    Variables esi_vars("vars_test", &Debug, &Error, whitelistCookies);
+    Utils::HeaderValueList allowlistCookies;
+    Variables esi_vars("vars_test", &Debug, &Error, allowlistCookies);
     esi_vars.populate(HttpHeader("hdr1", -1, "hval1", -1));
     esi_vars.populate(HttpHeader("Hdr2", -1, "hval2", -1));
     esi_vars.populate(HttpHeader("@Intenal-hdr1", -1, "internal-hval1", -1));
+    esi_vars.populate(HttpHeader("cookie", -1, "x=y", -1));
 
     assert(esi_vars.getValue("HTTP_HEADER{hdr1}") == "hval1");
     assert(esi_vars.getValue("HTTP_HEADER{hdr2}") == "");
     assert(esi_vars.getValue("HTTP_HEADER{Hdr2}") == "hval2");
     assert(esi_vars.getValue("HTTP_HEADER{non-existent}") == "");
     assert(esi_vars.getValue("HTTP_HEADER{@Intenal-hdr1}") == "internal-hval1");
+    assert(esi_vars.getValue("HTTP_HEADER{cookie}") == "");
   }
 
   {
     cout << endl << "===================== Test 6" << endl;
-    Utils::HeaderValueList whitelistCookies;
-    whitelistCookies.push_back("*");
-    Variables esi_vars("vars_test", &Debug, &Error, whitelistCookies);
+    Utils::HeaderValueList allowlistCookies;
+    allowlistCookies.push_back("*");
+    Variables esi_vars("vars_test", &Debug, &Error, allowlistCookies);
 
     esi_vars.populate(HttpHeader("Host", -1, "example.com", -1));
     esi_vars.populate(HttpHeader("Cookie", -1, "age=21; grade=-5; avg=4.3; t1=\" \"; t2=0.0", -1));
