@@ -33,7 +33,6 @@
 ////////////////////////////////////////////////////////////////////////////////////
 // proxy.config.http.cache.open_write_fail_action        1 /////////////////////////
 // proxy.config.cache.enable_read_while_writer           1 /////////////////////////
-// proxy.config.http.redirection_enabled                 1 /////////////////////////
 // proxy.config.http.number_of_redirections             10 /////////////////////////
 // proxy.config.http.redirect_use_orig_cache_key         1 /////////////////////////
 // proxy.config.http.background_fill_active_timeout      0 /////////////////////////
@@ -56,11 +55,11 @@
 #include <ts/remap.h>
 #include <set>
 #include <string>
-#include <string.h>
-#include <stdlib.h>
-#include <stdint.h>
-#include <stdio.h>
-#include <stdarg.h>
+#include <cstring>
+#include <cstdlib>
+#include <cstdint>
+#include <cstdio>
+#include <cstdarg>
 #include <getopt.h>
 #include <netdb.h>
 #include <map>
@@ -74,6 +73,8 @@ static const char *ATS_INTERNAL_MESSAGE = "@Ats-Internal";
 static int OPEN_WRITE_FAIL_MAX_REQ_DELAY_RETRIES = 5;
 static int OPEN_WRITE_FAIL_REQ_DELAY_TIMEOUT     = 500;
 
+static bool global_init = false;
+
 typedef struct _RequestData {
   TSHttpTxn txnp;
   int wl_retry; // write lock failure retry count
@@ -171,7 +172,6 @@ on_immediate(RequestData *req, TSCont &contp)
   TSDebug(DEBUG_TAG, "continuation delayed, scheduling now..for url: %s", req->req_url.c_str());
 
   // add retry_done header to prevent looping
-  std::string value;
   TSMBuffer bufp;
   TSMLoc hdr_loc;
   if (TSHttpTxnClientRespGet(req->txnp, &bufp, &hdr_loc) != TS_SUCCESS) {
@@ -209,7 +209,7 @@ on_send_response_header(RequestData *req, TSHttpTxn &txnp, TSCont &contp)
     if (delay_request) {
       req->wl_retry++;
       TSDebug(DEBUG_TAG, "delaying request, url@%p: {{%s}} on retry: %d time", txnp, req->req_url.c_str(), req->wl_retry);
-      TSContSchedule(contp, OPEN_WRITE_FAIL_REQ_DELAY_TIMEOUT, TS_THREAD_POOL_TASK);
+      TSContScheduleOnPool(contp, OPEN_WRITE_FAIL_REQ_DELAY_TIMEOUT, TS_THREAD_POOL_TASK);
       TSHandleMLocRelease(bufp, TS_NULL_MLOC, hdr_loc);
       return TS_SUCCESS;
     }
@@ -221,74 +221,25 @@ on_send_response_header(RequestData *req, TSHttpTxn &txnp, TSCont &contp)
     req->wl_retry = 0;
   }
 
-  // done..cleanup
-  delete req;
-  TSContDestroy(contp);
-
   TSHandleMLocRelease(bufp, TS_NULL_MLOC, hdr_loc);
   TSHttpTxnReenable(txnp, TS_EVENT_HTTP_CONTINUE);
   return TS_SUCCESS;
 }
 
 static int
-collapsed_cont(TSCont contp, TSEvent event, void *edata)
+on_txn_close(RequestData *req, TSHttpTxn &txnp, TSCont &contp)
 {
-  TSHttpTxn txnp      = static_cast<TSHttpTxn>(edata);
-  RequestData *my_req = static_cast<RequestData *>(TSContDataGet(contp));
-
-  switch (event) {
-  case TS_EVENT_HTTP_OS_DNS: {
-    return on_OS_DNS(my_req, txnp);
-  }
-
-  case TS_EVENT_HTTP_SEND_REQUEST_HDR: {
-    return on_send_request_header(my_req, txnp);
-  }
-
-  case TS_EVENT_HTTP_READ_RESPONSE_HDR: {
-    return on_read_response_header(txnp);
-  }
-  case TS_EVENT_IMMEDIATE:
-  case TS_EVENT_TIMEOUT: {
-    return on_immediate(my_req, contp);
-  }
-  case TS_EVENT_HTTP_SEND_RESPONSE_HDR: {
-    return on_send_response_header(my_req, txnp, contp);
-  }
-  default: {
-    TSDebug(DEBUG_TAG, "Unexpected event: %d", event);
-    break;
-  }
-  }
-
+  // done..cleanup
+  delete req;
+  TSContDestroy(contp);
   TSHttpTxnReenable(txnp, TS_EVENT_HTTP_CONTINUE);
   return TS_SUCCESS;
 }
 
-TSReturnCode
-TSRemapInit(TSRemapInterface * /* api_info */, char * /* errbuf */, int /* errbuf_size */)
-{
-  TSDebug(DEBUG_TAG, "plugin is succesfully initialized");
-  return TS_SUCCESS;
-}
+static int collapsed_cont(TSCont contp, TSEvent event, void *edata);
 
-TSReturnCode
-TSRemapNewInstance(int argc, char *argv[], void ** /* ih */, char * /* errbuf */, int /* errbuf_size */)
-{
-  // basic argv processing..
-  for (int i = 2; i < argc; ++i) {
-    if (strncmp(argv[i], "--delay=", 8) == 0) {
-      OPEN_WRITE_FAIL_REQ_DELAY_TIMEOUT = atoi((char *)(argv[i] + 8));
-    } else if (strncmp(argv[i], "--retries=", 10) == 0) {
-      OPEN_WRITE_FAIL_MAX_REQ_DELAY_RETRIES = atoi((char *)(argv[i] + 10));
-    }
-  }
-
-  return TS_SUCCESS;
-}
-
-TSRemapStatus
-TSRemapDoRemap(void *ih, TSHttpTxn rh, TSRemapRequestInfo *rri)
+void
+setup_transaction_cont(TSHttpTxn rh)
 {
   TSCont cont = TSContCreate(collapsed_cont, TSMutexCreate());
 
@@ -308,6 +259,115 @@ TSRemapDoRemap(void *ih, TSHttpTxn rh, TSRemapRequestInfo *rri)
   TSHttpTxnHookAdd(rh, TS_HTTP_SEND_RESPONSE_HDR_HOOK, cont);
   TSHttpTxnHookAdd(rh, TS_HTTP_READ_RESPONSE_HDR_HOOK, cont);
   TSHttpTxnHookAdd(rh, TS_HTTP_OS_DNS_HOOK, cont);
+  TSHttpTxnHookAdd(rh, TS_HTTP_TXN_CLOSE_HOOK, cont);
+}
+
+static int
+collapsed_cont(TSCont contp, TSEvent event, void *edata)
+{
+  TSHttpTxn txnp      = static_cast<TSHttpTxn>(edata);
+  RequestData *my_req = static_cast<RequestData *>(TSContDataGet(contp));
+
+  switch (event) {
+  case TS_EVENT_HTTP_READ_REQUEST_HDR:
+    // Create per transaction state
+    setup_transaction_cont(txnp);
+    break;
+
+  case TS_EVENT_HTTP_OS_DNS: {
+    return on_OS_DNS(my_req, txnp);
+  }
+
+  case TS_EVENT_HTTP_SEND_REQUEST_HDR: {
+    return on_send_request_header(my_req, txnp);
+  }
+
+  case TS_EVENT_HTTP_READ_RESPONSE_HDR: {
+    return on_read_response_header(txnp);
+  }
+  case TS_EVENT_IMMEDIATE:
+  case TS_EVENT_TIMEOUT: {
+    return on_immediate(my_req, contp);
+  }
+  case TS_EVENT_HTTP_SEND_RESPONSE_HDR: {
+    return on_send_response_header(my_req, txnp, contp);
+  }
+  case TS_EVENT_HTTP_TXN_CLOSE: {
+    return on_txn_close(my_req, txnp, contp);
+  }
+  default: {
+    TSDebug(DEBUG_TAG, "Unexpected event: %d", event);
+    break;
+  }
+  }
+
+  TSHttpTxnReenable(txnp, TS_EVENT_HTTP_CONTINUE);
+  return TS_SUCCESS;
+}
+
+void
+process_args(int argc, const char **argv)
+{
+  // basic argv processing..
+  for (int i = 1; i < argc; ++i) {
+    if (strncmp(argv[i], "--delay=", 8) == 0) {
+      OPEN_WRITE_FAIL_REQ_DELAY_TIMEOUT = atoi(const_cast<char *>(argv[i] + 8));
+    } else if (strncmp(argv[i], "--retries=", 10) == 0) {
+      OPEN_WRITE_FAIL_MAX_REQ_DELAY_RETRIES = atoi(const_cast<char *>(argv[i] + 10));
+    }
+  }
+}
+
+/*
+ * Initialize globally
+ */
+void
+TSPluginInit(int argc, const char *argv[])
+{
+  TSPluginRegistrationInfo info;
+
+  info.plugin_name   = const_cast<char *>(DEBUG_TAG);
+  info.vendor_name   = (char *)"Apache Software Foundation";
+  info.support_email = (char *)"dev@trafficserver.apache.org";
+
+  if (TS_SUCCESS != TSPluginRegister(&info)) {
+    TSError("[%s] Plugin registration failed", DEBUG_TAG);
+  }
+
+  process_args(argc, argv);
+
+  TSCont cont = TSContCreate(collapsed_cont, TSMutexCreate());
+
+  TSDebug(DEBUG_TAG, "Global Initialized");
+  // Set up the per transaction state in the READ_REQUEST event
+  TSHttpHookAdd(TS_HTTP_READ_REQUEST_HDR_HOOK, cont);
+
+  global_init = true;
+}
+
+TSReturnCode
+TSRemapInit(TSRemapInterface * /* api_info */, char * /* errbuf */, int /* errbuf_size */)
+{
+  if (global_init) {
+    TSError("Cannot initialize %s as both global and remap plugin", DEBUG_TAG);
+    return TS_ERROR;
+  } else {
+    TSDebug(DEBUG_TAG, "plugin is successfully initialized for remap");
+    return TS_SUCCESS;
+  }
+}
+
+TSReturnCode
+TSRemapNewInstance(int argc, char *argv[], void ** /* ih */, char * /* errbuf */, int /* errbuf_size */)
+{
+  process_args(argc - 1, const_cast<const char **>(argv + 1));
+  return TS_SUCCESS;
+}
+
+TSRemapStatus
+TSRemapDoRemap(void *ih, TSHttpTxn rh, TSRemapRequestInfo *rri)
+{
+  setup_transaction_cont(rh);
 
   return TSREMAP_NO_REMAP;
 }
