@@ -28,19 +28,16 @@
 #include <ts/remap.h>
 #include <yaml-cpp/yaml.h>
 
-#include <algorithm>
-#include <list>
-#include <map>
-#include <set>
-#include <string.h>
 #include <string>
-#include <unistd.h>
 #include <vector>
+#include <string_view>
+#include <cstddef>
 #include "hash.h"
 
-using namespace std;
+#undef FMT_SV
+#define FMT_SV(SV) static_cast<int>((SV).size()), (SV).data()
 
-// for bucketizing
+using namespace std;
 
 #define MY_NAME "cookie_remap"
 const int OVECCOUNT = 30; // We support $1 - $9 only, and this needs to be 3x that
@@ -65,57 +62,129 @@ ink_free(void *s)
 {
   return TSfree(s);
 }
-///////////////////////////////////////////////////////////////////////////////
-// Class holding one request URL's component, to simplify the code and
-// length calculations (we need all of them).
-//
-struct UrlComponents {
-  UrlComponents() {}
-  ~UrlComponents()
+
+class UrlComponents
+{
+public:
+  UrlComponents(TSRemapRequestInfo *rri, TSHttpTxn txn) : _rri(rri), _txn(txn) {}
+
+  std::string const &
+  path(bool pre_remap)
   {
-    if (url != nullptr)
-      TSfree((void *)url);
+    if (_d[pre_remap].path_str.empty()) {
+      auto urlh = _get_url(pre_remap);
+      // based on RFC2396, matrix params are part of path segments so
+      // we will just
+      // append them to the path
+      _d[pre_remap].path_str = _get_url_comp(urlh, TSUrlPathGet);
+      auto matrix            = _get_url_comp(urlh, TSUrlHttpParamsGet);
+      if (!matrix.empty()) {
+        _d[pre_remap].path_str.append(";", 1).append(matrix);
+      }
+    }
+    return _d[pre_remap].path_str;
   }
 
-  void
-  populate(TSRemapRequestInfo *rri)
+  std::string_view
+  query(bool pre_remap)
   {
-    scheme    = TSUrlSchemeGet(rri->requestBufp, rri->requestUrl, &scheme_len);
-    host      = TSUrlHostGet(rri->requestBufp, rri->requestUrl, &host_len);
-    tspath    = TSUrlPathGet(rri->requestBufp, rri->requestUrl, &tspath_len);
-    query     = TSUrlHttpQueryGet(rri->requestBufp, rri->requestUrl, &query_len);
-    matrix    = TSUrlHttpParamsGet(rri->requestBufp, rri->requestUrl, &matrix_len);
-    port      = TSUrlPortGet(rri->requestBufp, rri->requestUrl);
-    url       = TSUrlStringGet(rri->requestBufp, rri->requestUrl, &url_len);
-    from_path = TSUrlPathGet(rri->requestBufp, rri->mapFromUrl, &from_path_len);
+    if (_d[pre_remap].query.empty()) {
+      _d[pre_remap].query = _get_url_comp(_get_url(pre_remap), TSUrlHttpQueryGet);
+    }
+    return _d[pre_remap].query;
+  }
 
-    // based on RFC2396, matrix params are part of path segments so
-    // we will just
-    // append them to the path
-    path.assign(tspath, tspath_len);
-    if (matrix_len) {
-      path += ';';
-      path.append(matrix, matrix_len);
+  std::string_view
+  from_path()
+  {
+    if (_from_path.empty()) {
+      _UrlHandle urlh{_rri->requestBufp, _rri->mapFromUrl};
+      _from_path = _get_url_comp(urlh, TSUrlPathGet);
+    }
+    return _from_path;
+  }
+
+  std::string_view
+  url(bool pre_remap)
+  {
+    if (_d[pre_remap].url.empty()) {
+      auto urlh = _get_url(pre_remap);
+      int length;
+      auto data         = TSUrlStringGet(urlh.bufp, urlh.urlp, &length);
+      _d[pre_remap].url = std::string_view(data, length);
+    }
+    return _d[pre_remap].url;
+  }
+
+  // No copying/moving.
+  //
+  UrlComponents(UrlComponents const &) = delete;
+  UrlComponents &operator=(UrlComponents const &) = delete;
+
+  ~UrlComponents()
+  {
+    // Not calling TSHandleMLocRelease() for the URL TSMLoc pointers because it doesn't do anything.
+
+    if (_d[0].url.data() != nullptr) {
+      TSfree(const_cast<char *>(_d[0].url.data()));
+    }
+    if (_d[1].url.data() != nullptr) {
+      TSfree(const_cast<char *>(_d[1].url.data()));
     }
   }
 
-  const char *scheme = nullptr;
-  const char *host   = nullptr;
-  const char *tspath = nullptr;
-  std::string path{};
-  const char *query     = nullptr;
-  const char *matrix    = nullptr;
-  const char *url       = nullptr;
-  const char *from_path = nullptr;
-  int port              = 0;
+private:
+  TSRemapRequestInfo *_rri;
+  TSHttpTxn _txn;
 
-  int scheme_len    = 0;
-  int host_len      = 0;
-  int tspath_len    = 0;
-  int query_len     = 0;
-  int matrix_len    = 0;
-  int url_len       = 0;
-  int from_path_len = 0;
+  struct _UrlHandle {
+    TSMBuffer bufp = nullptr;
+    TSMLoc urlp;
+  };
+
+  // Buffer any data that's likely to be used more than once.
+
+  struct _Data {
+    _UrlHandle urlh;
+    std::string path_str;
+    std::string_view url;
+    std::string_view query;
+  };
+
+  // index 0 - remapped
+  // index 1 - pre-remap
+  //
+  _Data _d[2];
+
+  std::string_view _from_path;
+
+  _UrlHandle
+  _get_url(bool pre_remap)
+  {
+    _UrlHandle h = _d[pre_remap].urlh;
+
+    if (!h.bufp) {
+      if (pre_remap) {
+        if (TSHttpTxnPristineUrlGet(_txn, &h.bufp, &h.urlp) != TS_SUCCESS) {
+          TSError("%s: Plugin is unable to get pristine url", MY_NAME);
+          return _UrlHandle();
+        }
+      } else {
+        h.bufp = _rri->requestBufp;
+        h.urlp = _rri->requestUrl;
+      }
+      _d[pre_remap].urlh = h;
+    }
+    return h;
+  }
+
+  static std::string_view
+  _get_url_comp(_UrlHandle urlh, char const *(*comp_func)(TSMBuffer, TSMLoc, int *))
+  {
+    int length;
+    auto data = comp_func(urlh.bufp, urlh.urlp, &length);
+    return std::string_view(data, length);
+  }
 };
 
 void
@@ -130,6 +199,7 @@ enum operation_type { UNKNOWN = -1, EXISTS = 1, NOTEXISTS, REGEXP, STRING, BUCKE
 enum target_type {
   COOKIE = 1,
   URI, // URI = PATH + QUERY
+  PRE_REMAP_URI,
   UNKNOWN_TARGET
 };
 
@@ -159,8 +229,9 @@ dec_to_hex(type _num, char *hdigits)
   const char *hlookup = "0123456789ABCDEF"; // lookup table stores the hex digits into their
   // corresponding index.
 
-  if (_num < 0)
+  if (_num < 0) {
     _num *= -1; // and make _num positive to clear(zero) the sign bit
+  }
 
   char mask = 0x000f; // mask will clear(zero) out all the bits except lowest 4
   // which represent a single hex digit
@@ -174,14 +245,25 @@ dec_to_hex(type _num, char *hdigits)
 void
 urlencode(std::string &str)
 {
-  size_t pos = 0;
-  std::string replacement;
-  for (; pos < str.length(); pos++) {
-    if (!isalnum(str[pos])) {
-      char dec[2];
-      dec_to_hex(str[pos], dec);
-      std::string replacement = "%" + std::string(dec, 2);
-      str.replace(pos, 1, replacement);
+  auto orig = str.size();
+  auto enc  = orig;
+  for (auto c : str) {
+    if (!isalnum(c)) {
+      enc += 2;
+    }
+  }
+  if (enc == orig) {
+    // No changes needed.
+    return;
+  }
+  str.resize(enc);
+  while (orig--) {
+    if (!isalnum(str[orig])) {
+      enc -= 3;
+      dec_to_hex(str[orig], &(str[enc + 1]));
+      str[enc] = '%';
+    } else {
+      str[--enc] = str[orig];
     }
   }
 }
@@ -193,15 +275,11 @@ public:
   subop()
     : cookie(""),
       operation(""),
-      op_type(UNKNOWN),
-      target(UNKNOWN_TARGET),
+
       str_match(""),
-      regex(nullptr),
-      regex_extra(nullptr),
-      regex_ccount(0),
-      bucket(""),
-      how_many(0),
-      out_of(0)
+
+      bucket("")
+
   {
     TSDebug(MY_NAME, "subop constructor called");
   }
@@ -209,11 +287,13 @@ public:
   ~subop()
   {
     TSDebug(MY_NAME, "subop destructor called");
-    if (regex)
+    if (regex) {
       pcre_free(regex);
+    }
 
-    if (regex_extra)
+    if (regex_extra) {
       pcre_free(regex_extra);
+    }
   }
 
   bool
@@ -277,10 +357,13 @@ public:
   void
   setTarget(const std::string &s)
   {
-    if (s == "uri")
+    if (s == "uri") {
       target = URI;
-    else
+    } else if (s == "puri") {
+      target = PRE_REMAP_URI;
+    } else {
       target = COOKIE;
+    }
   }
 
   void
@@ -299,7 +382,7 @@ public:
   void
   setBucket(const std::string &s)
   {
-    int start_pos = s.find("/");
+    int start_pos = s.find('/');
 
     op_type  = BUCKET;
     bucket   = s;
@@ -334,12 +417,13 @@ public:
       return false;
     }
     regex_extra = pcre_study(regex, 0, &error_study);
-    if ((regex_extra == nullptr) && (error_study != 0)) {
+    if ((regex_extra == nullptr) && (error_study != nullptr)) {
       return false;
     }
 
-    if (pcre_fullinfo(regex, regex_extra, PCRE_INFO_CAPTURECOUNT, &regex_ccount) != 0)
+    if (pcre_fullinfo(regex, regex_extra, PCRE_INFO_CAPTURECOUNT, &regex_ccount) != 0) {
       return false;
+    }
 
     return true;
   }
@@ -391,33 +475,34 @@ public:
 private:
   std::string cookie;
   std::string operation;
-  enum operation_type op_type;
-  enum target_type target;
+  enum operation_type op_type = UNKNOWN;
+  enum target_type target     = UNKNOWN_TARGET;
 
   std::string str_match;
 
-  pcre *regex;
-  pcre_extra *regex_extra;
+  pcre *regex             = nullptr;
+  pcre_extra *regex_extra = nullptr;
   std::string regex_string;
-  int regex_ccount;
+  int regex_ccount = 0;
 
   std::string bucket;
-  unsigned int how_many;
-  unsigned int out_of;
+  unsigned int how_many = 0;
+  unsigned int out_of   = 0;
 };
 
-typedef std::vector<const subop *> SubOpQueue;
+using SubOpQueue = std::vector<const subop *>;
 
 //----------------------------------------------------------------------------
 class op
 {
 public:
   op() { TSDebug(MY_NAME, "op constructor called"); }
+
   ~op()
   {
     TSDebug(MY_NAME, "op destructor called");
-    for (SubOpQueue::iterator it = subops.begin(); it != subops.end(); ++it) {
-      delete *it;
+    for (auto &subop : subops) {
+      delete subop;
     }
   }
 
@@ -448,10 +533,11 @@ public:
   void
   setStatus(const std::string &s)
   {
-    if (else_sendto.size() > 0)
+    if (else_sendto.size() > 0) {
       else_status = static_cast<TSHttpStatus>(atoi(s.c_str()));
-    else
+    } else {
       status = static_cast<TSHttpStatus>(atoi(s.c_str()));
+    }
   }
 
   void
@@ -467,15 +553,16 @@ public:
     TSDebug(MY_NAME, "sending to: %s", sendto.c_str());
     TSDebug(MY_NAME, "if these operations match: ");
 
-    for (SubOpQueue::const_iterator it = subops.begin(); it != subops.end(); ++it) {
-      (*it)->printSubOp();
+    for (auto subop : subops) {
+      subop->printSubOp();
     }
-    if (else_sendto.size() > 0)
+    if (else_sendto.size() > 0) {
       TSDebug(MY_NAME, "else: %s", else_sendto.c_str());
+    }
   }
 
   bool
-  process(CookieJar &jar, std::string &dest, TSHttpStatus &retstat, TSRemapRequestInfo *rri) const
+  process(CookieJar &jar, std::string &dest, TSHttpStatus &retstat, TSRemapRequestInfo *rri, UrlComponents &req_url) const
   {
     if (sendto == "") {
       return false; // guessing every operation must have a
@@ -492,16 +579,16 @@ public:
 
     TSDebug(MY_NAME, "starting to process a new operation");
 
-    for (SubOpQueue::const_iterator it = subops.begin(); it != subops.end(); ++it) {
+    for (auto subop : subops) {
       // subop* s = *it;
-      int subop_type     = (*it)->getOpType();
-      target_type target = (*it)->getTargetType();
+      int subop_type     = subop->getOpType();
+      target_type target = subop->getTargetType();
 
-      c = (*it)->getCookieName();
+      c = subop->getCookieName();
       if (c.length()) {
         TSDebug(MY_NAME, "processing cookie: %s", c.c_str());
 
-        size_t period_pos = c.find_first_of(".");
+        size_t period_pos = c.find_first_of('.');
 
         if (period_pos == std::string::npos) { // not a sublevel
                                                // cookie name
@@ -528,9 +615,10 @@ public:
         if (cookie_found == false) { // cookie name or sub-key not found
                                      // inside cookies
           if (subop_type == NOTEXISTS) {
-            TSDebug(MY_NAME, "cookie %s was not "
-                             "found (and we wanted "
-                             "that)",
+            TSDebug(MY_NAME,
+                    "cookie %s was not "
+                    "found (and we wanted "
+                    "that)",
                     object_name.c_str());
             continue; // we can short
                       // circuit more
@@ -544,10 +632,11 @@ public:
           if (subop_type == NOTEXISTS) { // we found the cookie
                                          // but are asking
             // for non existence
-            TSDebug(MY_NAME, "cookie %s was found, "
-                             "but operation "
-                             "requires "
-                             "non-existence",
+            TSDebug(MY_NAME,
+                    "cookie %s was found, "
+                    "but operation "
+                    "requires "
+                    "non-existence",
                     object_name.c_str());
             retval &= 0;
             break;
@@ -565,8 +654,9 @@ public:
         } // handled EXISTS / NOTEXISTS subops
 
         TSDebug(MY_NAME, "processing cookie data: \"%s\"", cookie_data.c_str());
-      } else
+      } else if (target != PRE_REMAP_URI) {
         target = URI;
+      }
 
       // INVARIANT: we now have the data from the cookie (if
       // any) inside
@@ -590,18 +680,18 @@ public:
       std::string request_uri; // only set the value if we
                                // need it; we might
       // match the cookie data instead
-      const std::string &string_to_match(target == URI ? request_uri : cookie_data);
-      if (target == URI) {
-        UrlComponents req_url;
-        req_url.populate(rri);
-
-        TSDebug(MY_NAME, "process req_url.path = %s", req_url.path.c_str());
-        request_uri = req_url.path;
-        if (request_uri.length() && request_uri[0] != '/')
+      bool use_url = (target == URI) || (target == PRE_REMAP_URI);
+      const std::string &string_to_match(use_url ? request_uri : cookie_data);
+      if (use_url) {
+        request_uri = req_url.path(target == PRE_REMAP_URI);
+        TSDebug(MY_NAME, "process req_url.path = %s", request_uri.c_str());
+        if (request_uri.length() && request_uri[0] != '/') {
           request_uri.insert(0, 1, '/');
-        if (req_url.query_len > 0) {
+        }
+        auto query = req_url.query(target == PRE_REMAP_URI);
+        if (query.size() > 0) {
           request_uri += '?';
-          request_uri += std::string(req_url.query, req_url.query_len);
+          request_uri += query;
         }
         object_name = "request uri";
       }
@@ -612,7 +702,7 @@ public:
 
       // OPERATION::string matching
       if (subop_type == STRING) {
-        if (string_to_match == (*it)->getStringMatch()) {
+        if (string_to_match == subop->getStringMatch()) {
           TSDebug(MY_NAME, "string match succeeded");
           continue;
         } else {
@@ -625,7 +715,7 @@ public:
       // OPERATION::regex matching
       if (subop_type == REGEXP) {
         int ovector[OVECCOUNT];
-        int ret = (*it)->regexMatch(string_to_match.c_str(), string_to_match.length(), ovector);
+        int ret = subop->regexMatch(string_to_match.c_str(), string_to_match.length(), ovector);
 
         if (ret >= 0) {
           std::string::size_type pos  = sendto.find('$');
@@ -641,10 +731,11 @@ public:
           // like regex_remap.
 
           TSDebug(MY_NAME, "found %d matches", ret);
-          TSDebug(MY_NAME, "successful regex "
-                           "match of: %s with %s "
-                           "rewriting string: %s",
-                  string_to_match.c_str(), (*it)->getRegexString().c_str(), sendto.c_str());
+          TSDebug(MY_NAME,
+                  "successful regex "
+                  "match of: %s with %s "
+                  "rewriting string: %s",
+                  string_to_match.c_str(), subop->getRegexString().c_str(), sendto.c_str());
 
           // replace the $(1-9) in the sendto url
           // as necessary
@@ -654,16 +745,17 @@ public:
             if (isdigit(sendto[pos + 1])) {
               int ix = sendto[pos + 1] - '0';
 
-              if (ix <= (*it)->getRegexCcount()) { // Just skip an illegal regex group
+              if (ix <= subop->getRegexCcount()) { // Just skip an illegal regex group
                 dest += sendto.substr(ppos, pos - ppos);
                 dest += string_to_match.substr(ovector[ix * 2], ovector[ix * 2 + 1] - ovector[ix * 2]);
                 ppos = pos + 2;
               } else {
-                TSDebug(MY_NAME, "bad "
-                                 "rewriting "
-                                 "string, "
-                                 "for group "
-                                 "%d: %s",
+                TSDebug(MY_NAME,
+                        "bad "
+                        "rewriting "
+                        "string, "
+                        "for group "
+                        "%d: %s",
                         ix, sendto.c_str());
               }
             }
@@ -672,10 +764,11 @@ public:
           dest += sendto.substr(ppos);
           continue; // next subop, please
         } else {
-          TSDebug(MY_NAME, "could not match "
-                           "regular expression "
-                           "%s to %s",
-                  (*it)->getRegexString().c_str(), string_to_match.c_str());
+          TSDebug(MY_NAME,
+                  "could not match "
+                  "regular expression "
+                  "%s to %s",
+                  subop->getRegexString().c_str(), string_to_match.c_str());
           retval &= 0;
           break;
         }
@@ -683,22 +776,24 @@ public:
 
       // OPERATION::bucket ranges
       if (subop_type == BUCKET) {
-        unsigned int taking = (*it)->bucketGetTaking();
-        unsigned int out_of = (*it)->bucketOutOf();
+        unsigned int taking = subop->bucketGetTaking();
+        unsigned int out_of = subop->bucketOutOf();
 
         uint32_t hash;
 
         if (taking == 0 || out_of == 0) {
-          TSDebug(MY_NAME, "taking %d out of %d "
-                           "makes no sense?!",
+          TSDebug(MY_NAME,
+                  "taking %d out of %d "
+                  "makes no sense?!",
                   taking, out_of);
           retval &= 0;
           break;
         }
 
         hash = hash_fnv32_buckets(cookie_data.c_str(), cookie_data.size(), out_of);
-        TSDebug(MY_NAME, "we hashed this to bucket: %u "
-                         "taking: %u out of: %u",
+        TSDebug(MY_NAME,
+                "we hashed this to bucket: %u "
+                "taking: %u out of: %u",
                 hash, taking, out_of);
 
         if (hash < taking) {
@@ -715,9 +810,10 @@ public:
     }
 
     if (retval == 1) {
-      if (dest.size() == 0) // Unless already set by one of
-                            // the operators (e.g. regex)
+      if (dest.size() == 0) { // Unless already set by one of
+                              // the operators (e.g. regex)
         dest = sendto;
+      }
       if (status > 0) {
         retstat = status;
       }
@@ -743,14 +839,12 @@ private:
 };
 
 typedef std::pair<std::string, std::string> StringPair;
-typedef std::vector<StringPair> OpMap;
+using OpMap = std::vector<StringPair>;
 
 //----------------------------------------------------------------------------
 static bool
 build_op(op &o, OpMap const &q)
 {
-  StringPair m;
-
   subop *sub = new subop();
 
   // loop through the array of key->value pairs
@@ -758,6 +852,8 @@ build_op(op &o, OpMap const &q)
     std::string const &key = pr.first;
     std::string const &val = pr.second;
 
+    TSDebug(MY_NAME, "build_op: key=%s val=%s", key.c_str(), val.c_str());
+
     if (key == "cookie") {
       if (!sub->empty()) {
         TSDebug(MY_NAME, "ERROR: you need to define a connector");
@@ -816,7 +912,7 @@ error:
   return false;
 }
 
-typedef std::vector<const op *> OpsQueue;
+using OpsQueue = std::vector<const op *>;
 
 //----------------------------------------------------------------------------
 // init
@@ -857,10 +953,10 @@ TSRemapNewInstance(int argc, char *argv[], void **ih, char *errbuf, int errbuf_s
       }
 
       for (YAML::const_iterator it2 = it->second.begin(); it2 != it->second.end(); ++it2) {
-        const YAML::Node &first  = it2->first;
-        const YAML::Node &second = it2->second;
+        const YAML::Node first  = it2->first;
+        const YAML::Node second = it2->second;
 
-        if (second.Type() != YAML::NodeType::Scalar) {
+        if (second.IsScalar() == false) {
           const string reason = "All op nodes must be of type scalar";
           TSError("Invalid YAML Configuration format for cookie_remap: %s, reason: %s", filename.c_str(), reason.c_str());
           return TS_ERROR;
@@ -886,7 +982,7 @@ TSRemapNewInstance(int argc, char *argv[], void **ih, char *errbuf, int errbuf_s
       }
     }
 
-    TSDebug(MY_NAME, "# of ops: %d", (int)ops->size());
+    TSDebug(MY_NAME, "# of ops: %d", static_cast<int>(ops->size()));
     *ih = static_cast<void *>(ops.release());
   } catch (const YAML::Exception &e) {
     TSError("YAML::Exception %s when parsing YAML config file %s for cookie_remap", e.what(), filename.c_str());
@@ -896,100 +992,182 @@ TSRemapNewInstance(int argc, char *argv[], void **ih, char *errbuf, int errbuf_s
   return TS_SUCCESS;
 }
 
-//----------------------------------------------------------------------------
-// called whenever we need to perform substiturions on a string; used to replace
-// things like
-//  $url, $unmatched_path, $cr_req_url, and $cr_url_encode
-// returns 0 if no substitutions, 1 otw.
-int
-cr_substitutions(std::string &obj, TSRemapRequestInfo *rri)
+namespace
 {
-  int retval = 0;
+std::string
+unmatched_path(UrlComponents &req_url, bool pre_remap)
+{
+  std::string path           = req_url.path(pre_remap);
+  std::string_view from_path = req_url.from_path();
 
-  size_t pos      = obj.find('$');
-  size_t tmp_pos  = 0;
-  size_t last_pos = pos;
+  std::size_t pos = path.find(from_path);
+  if (pos != std::string::npos) {
+    path.erase(pos, from_path.size());
+  }
+  TSDebug(MY_NAME, "from_path: %*s", FMT_SV(from_path));
+  TSDebug(MY_NAME, "%s: %s", pre_remap ? "unmatched_ppath" : "unmatched_path", path.c_str());
 
-  UrlComponents req_url;
-  req_url.populate(rri);
-  TSDebug(MY_NAME, "x req_url.path: %s %zu", req_url.path.c_str(), req_url.path.length());
-  TSDebug(MY_NAME, "x req_url.url: %s %d", std::string(req_url.url, req_url.url_len).c_str(), req_url.url_len);
+  return path;
+}
 
-  while (pos < obj.length()) {
-    if (pos + 1 >= obj.length())
-      break; // trailing ?
+int const sub_req_url_id         = 0;
+int const sub_req_purl_id        = -1;
+int const sub_path_id            = -2;
+int const sub_ppath_id           = -3;
+int const sub_unmatched_path_id  = -4;
+int const sub_unmatched_ppath_id = -5;
+int const sub_url_encode_id      = -6;
 
-    switch (obj[pos + 1]) {
-    case 'p':
-      if (obj.substr(pos + 1, 4) == "path") {
-        obj.replace(pos, 5, req_url.path);
-        pos += req_url.path.length();
+struct CompNext {
+  std::string_view const comp;
+  int const *const next;
+
+  CompNext(std::string_view p, int const *n) : comp(p), next(n) {}
+};
+
+struct {
+  int count = 2;
+  CompNext o1{"ath", &sub_unmatched_path_id};
+  CompNext o2{"path", &sub_unmatched_ppath_id};
+} const sub_unmatched;
+
+struct {
+  int count = 2;
+  CompNext o1{"ath", &sub_path_id};
+  CompNext o2{"path", &sub_ppath_id};
+} const sub_p;
+
+struct {
+  int count = 2;
+  CompNext o1{"url", &sub_req_url_id};
+  CompNext o2{"purl", &sub_req_purl_id};
+} const sub_cr_req;
+
+struct {
+  int count = 2;
+  CompNext o1{"req_", &sub_cr_req.count};
+  CompNext o2{"urlencode(", &sub_url_encode_id};
+} const sub_cr;
+
+struct {
+  int count = 3;
+  CompNext o1{"cr_", &sub_cr.count};
+  CompNext o2{"p", &sub_p.count};
+  CompNext o3{"unmatched_p", &sub_unmatched.count};
+} const sub;
+
+int
+sub_lookup(char const *targ, int targ_len)
+{
+  int count = sub.count;
+  auto opt  = &sub.o1;
+  for (;;) {
+    while ((targ_len < static_cast<int>(opt->comp.size())) || (std::string_view(targ, opt->comp.size()) != opt->comp)) {
+      if (!--count) {
+        return 1; // Failed lookup, return some positive numver.
       }
-      break;
-    case 'u':
-      if (obj.substr(pos + 1, 14) == "unmatched_path") {
-        // we found $unmatched_path inside the sendto
-        // url
-        std::string unmatched_path(req_url.path);
-        TSDebug(MY_NAME, "unmatched_path: %s", unmatched_path.c_str());
-        TSDebug(MY_NAME, "from_path: %s", req_url.from_path);
-
-        tmp_pos = unmatched_path.find(req_url.from_path, 0, req_url.from_path_len); // from patch
-        if (tmp_pos != std::string::npos) {
-          unmatched_path.erase(tmp_pos, req_url.from_path_len);
-        }
-        TSDebug(MY_NAME, "unmatched_path: %s", unmatched_path.c_str());
-        TSDebug(MY_NAME, "obj: %s", obj.c_str());
-        obj.replace(pos, 15, unmatched_path);
-        TSDebug(MY_NAME, "obj: %s", obj.c_str());
-        pos += unmatched_path.length();
-      }
-      break;
-    case 'c':
-      if (obj.substr(pos + 1, 3) == "cr_")
-        switch (obj[pos + 4]) {
-        case 'r':
-          if (obj.substr(pos + 4, 7) == "req_url") {
-            // we found $cr_req_url inside
-            // the sendto url
-            std::string request_url;
-            if (req_url.url && req_url.url_len > 0) {
-              request_url = std::string(req_url.url, req_url.url_len);
-            }
-            TSDebug(MY_NAME, "req_url.url: %s %d", std::string(req_url.url, req_url.url_len).c_str(), req_url.url_len);
-            obj.replace(pos, 11, request_url);
-            pos += request_url.length();
-          }
-          break;
-        case 'u':
-          // note that this allows for variables
-          // nested in the function, but not
-          // for functions nested in the function
-          if (obj.substr(pos + 4, 10) == "urlencode(" && (tmp_pos = obj.find(')', pos + 14)) != std::string::npos) {
-            std::string str = obj.substr(pos + 14, tmp_pos - pos - 14); // the string to
-                                                                        // encode
-            cr_substitutions(str, rri);                                 // expand any
-                                                                        // variables as
-                                                                        // necessary
-                                                                        // before
-                                                                        // encoding
-            urlencode(str);
-            obj.replace(pos, tmp_pos - pos + 1, str); // +1 for the
-                                                      // closing
-                                                      // parens
-            pos += str.length();
-          }
-          break;
-        }
+      ++opt;
+    }
+    count = *opt->next;
+    if (count <= 0) {
       break;
     }
-    if (last_pos == pos)
-      pos++; // don't search the same place again and again
-    last_pos = pos;
-    pos      = obj.find('$', pos);
+    targ += opt->comp.size();
+    targ_len -= opt->comp.size();
+    opt = reinterpret_cast<CompNext const *>(reinterpret_cast<char const *>(opt->next) + offsetof(decltype(sub), o1));
+  }
+  return count;
+}
+
+} // end anonymous namespace
+
+//----------------------------------------------------------------------------
+// called whenever we need to perform substitutions on a string; used to replace
+// things like
+//  $path, $ppath, $unmatched_path, $unmatched_ppath, $cr_req_url, $cr_req_purl, and $cr_url_encode
+void
+cr_substitutions(std::string &obj, UrlComponents &req_url)
+{
+  {
+    auto path = req_url.path(false);
+    TSDebug(MY_NAME, "x req_url.path: %*s %d", FMT_SV(path), static_cast<int>(path.size()));
+    auto url = req_url.url(false);
+    TSDebug(MY_NAME, "x req_url.url: %*s %d", FMT_SV(url), static_cast<int>(url.size()));
   }
 
-  return retval;
+  auto npos = std::string::npos;
+  std::string tmp;
+  std::size_t pos = 0;
+  for (;;) {
+    pos = obj.find('$', pos);
+    if (npos == pos) {
+      break;
+    }
+    std::string_view variable, value;
+    switch (sub_lookup(obj.data() + pos + 1, static_cast<int>(obj.size()) - pos - 1)) {
+    case sub_req_url_id: {
+      variable = "$cr_req_url";
+      value    = req_url.url(false);
+    } break;
+
+    case sub_req_purl_id: {
+      variable = "$cr_req_purl";
+      value    = req_url.url(true);
+    } break;
+
+    case sub_path_id: {
+      variable = "$path";
+      value    = req_url.path(false);
+    } break;
+
+    case sub_ppath_id: {
+      variable = "$ppath";
+      value    = req_url.path(true);
+    } break;
+
+    case sub_unmatched_path_id: {
+      variable = "$unmatched_path";
+      tmp      = unmatched_path(req_url, false);
+      value    = tmp;
+    } break;
+
+    case sub_unmatched_ppath_id: {
+      variable = "$unmatched_ppath";
+      tmp      = unmatched_path(req_url, true);
+      value    = tmp;
+    } break;
+
+    case sub_url_encode_id: {
+      std::size_t bpos = pos + sizeof("cr_urlencode(") - 1;
+      std::size_t epos = obj.find(')', bpos);
+      if (npos == epos) {
+        variable = "$";
+        value    = variable;
+      } else {
+        variable = std::string_view(obj.data() + pos, epos + 1 - pos);
+
+        tmp = obj.substr(bpos, epos - bpos);
+        cr_substitutions(tmp, req_url);
+        urlencode(tmp);
+        value = tmp;
+      }
+    } break;
+
+    default: {
+      variable = "$";
+      value    = variable;
+
+    } break;
+
+    } // end switch
+
+    TSDebug(MY_NAME, "%*s => %*s", FMT_SV(variable), FMT_SV(value));
+
+    obj.replace(pos, variable.size(), value);
+
+    pos += value.size();
+
+  } // end for (;;)
 }
 
 //----------------------------------------------------------------------------
@@ -998,11 +1176,10 @@ cr_substitutions(std::string &obj, TSRemapRequestInfo *rri)
 TSRemapStatus
 TSRemapDoRemap(void *ih, TSHttpTxn txnp, TSRemapRequestInfo *rri)
 {
-  OpsQueue *ops       = (OpsQueue *)ih;
+  OpsQueue *ops       = static_cast<OpsQueue *>(ih);
   TSHttpStatus status = TS_HTTP_STATUS_NONE;
 
-  UrlComponents req_url;
-  req_url.populate(rri);
+  UrlComponents req_url{rri, txnp};
 
   if (ops == (OpsQueue *)nullptr) {
     TSError("serious error with encountered while attempting to "
@@ -1013,9 +1190,10 @@ TSRemapDoRemap(void *ih, TSHttpTxn txnp, TSRemapRequestInfo *rri)
 
   // get any query params..we will append that to the answer (possibly)
   std::string client_req_query_params;
-  if (req_url.query_len) {
+  auto query = req_url.query(false);
+  if (!query.empty()) {
     client_req_query_params = "?";
-    client_req_query_params += std::string(req_url.query, req_url.query_len);
+    client_req_query_params += query;
   }
   TSDebug(MY_NAME, "Query Parameters: %s", client_req_query_params.c_str());
 
@@ -1038,22 +1216,23 @@ TSRemapDoRemap(void *ih, TSHttpTxn txnp, TSRemapRequestInfo *rri)
   CookieJar jar;
   jar.create(temp_cookie);
 
-  for (OpsQueue::iterator it = ops->begin(); it != ops->end(); ++it) {
+  for (auto &op : *ops) {
     TSDebug(MY_NAME, ">>> processing new operation");
-    if ((*it)->process(jar, rewrite_to, status, rri)) {
-      cr_substitutions(rewrite_to, rri);
+    if (op->process(jar, rewrite_to, status, rri, req_url)) {
+      cr_substitutions(rewrite_to, req_url);
 
       size_t pos = 7;                             // 7 because we want to ignore the // in
                                                   // http:// :)
       size_t tmp_pos = rewrite_to.find('?', pos); // we don't want to alter the query string
       do {
         pos = rewrite_to.find("//", pos);
-        if (pos < tmp_pos)
+        if (pos < tmp_pos) {
           rewrite_to.erase(pos, 1); // remove one '/'
+        }
       } while (pos <= rewrite_to.length() && pos < tmp_pos);
 
       // Add Query Parameters if not already present
-      if (!client_req_query_params.empty() && rewrite_to.find("?") == std::string::npos) {
+      if (!client_req_query_params.empty() && rewrite_to.find('?') == std::string::npos) {
         rewrite_to.append(client_req_query_params);
       }
 
@@ -1129,11 +1308,11 @@ TSRemapDoRemap(void *ih, TSHttpTxn txnp, TSRemapRequestInfo *rri)
 void
 TSRemapDeleteInstance(void *ih)
 {
-  OpsQueue *ops = (OpsQueue *)ih;
+  OpsQueue *ops = static_cast<OpsQueue *>(ih);
 
   TSDebug(MY_NAME, "deleting loaded operations");
-  for (OpsQueue::iterator it = ops->begin(); it != ops->end(); ++it) {
-    delete *it;
+  for (auto &op : *ops) {
+    delete op;
   }
 
   delete ops;
