@@ -46,19 +46,56 @@ contentRangeFrom(HttpHeader const &header)
 
   if (!hasContentRange) {
     DEBUG_LOG("invalid response header, no Content-Range");
-  } else if (!bcr.fromStringClosed(rangestr)) {
-    DEBUG_LOG("invalid response header, malformed Content-Range, %s", rangestr);
+  } else {
+    // ensure null termination
+    rangestr[rangelen] = '\0';
+    if (!bcr.fromStringClosed(rangestr)) {
+      DEBUG_LOG("invalid response header, malformed Content-Range, %s", rangestr);
+    }
   }
 
   return bcr;
 }
 
-bool
+int64_t
+contentLengthFrom(HttpHeader const &header)
+{
+  int64_t bytes = 0;
+
+  char constr[1024];
+  int conlen = sizeof(constr);
+
+  // look for expected Content-Length field
+  bool const hasContentLength(header.valueForKey(TS_MIME_FIELD_CONTENT_LENGTH, TS_MIME_LEN_CONTENT_LENGTH, constr, &conlen));
+
+  if (!hasContentLength) {
+    DEBUG_LOG("invalid response header, no Content-Length");
+    bytes = INT64_MAX;
+  } else {
+    // ensure null termination
+    constr[conlen] = '\0';
+    char *endptr   = nullptr;
+    bytes          = std::max((int64_t)0, (int64_t)strtoll(constr, &endptr, 10));
+  }
+
+  return bytes;
+}
+
+// Also reference server header
+enum HeaderState {
+  Good,
+  Fail,
+  Passthru,
+};
+
+HeaderState
 handleFirstServerHeader(Data *const data, TSCont const contp)
 {
   HttpHeader header(data->m_resp_hdrmgr.m_buffer, data->m_resp_hdrmgr.m_lochdr);
 
-  //  DEBUG_LOG("First header\n%s", header.toString().c_str());
+  if (TSIsDebugTagSet(PLUGIN_NAME)) {
+    DEBUG_LOG("First header\n%s", header.toString().c_str());
+  }
 
   data->m_dnstream.setupVioWrite(contp, INT64_MAX);
 
@@ -71,22 +108,27 @@ handleFirstServerHeader(Data *const data, TSCont const contp)
     DEBUG_LOG("Initial response other than 206: %d", header.status());
 
     // Should run TSVIONSetBytes(output_io, hlen + bodybytes);
-    //    int const hlen = TSHttpHdrLengthGet(header.m_buffer, header.m_lochdr);
-    //    TSVIONBytesSet(output_vio, hlen);
+    int64_t const hlen = TSHttpHdrLengthGet(header.m_buffer, header.m_lochdr);
+    int64_t const clen = contentLengthFrom(header);
+    DEBUG_LOG("Passthru bytes: header: %" PRId64 " body: %" PRId64, hlen, clen);
+    if (clen != INT64_MAX) {
+      TSVIONBytesSet(output_vio, hlen + clen);
+    } else {
+      TSVIONBytesSet(output_vio, clen);
+    }
     TSHttpHdrPrint(header.m_buffer, header.m_lochdr, output_buf);
-    transfer_all_bytes(data);
-
-    return false;
+    return HeaderState::Passthru;
   }
 
   ContentRange const blockcr = contentRangeFrom(header);
-  // 206 with bad content range?
+
+  // 206 with bad content range -- should NEVER happen.
   if (!blockcr.isValid()) {
-    static std::string const &msg502 = string502();
+    std::string const msg502 = string502(header.version());
     TSVIONBytesSet(output_vio, msg502.size());
     TSIOBufferWrite(output_buf, msg502.data(), msg502.size());
     TSVIOReenable(output_vio);
-    return false;
+    return HeaderState::Fail;
   }
 
   // set the resource content length from block response
@@ -117,16 +159,18 @@ handleFirstServerHeader(Data *const data, TSCont const contp)
     TSHttpHdrPrint(header.m_buffer, header.m_lochdr, output_buf);
     TSIOBufferWrite(output_buf, bodystr.data(), bodystr.size());
     TSVIOReenable(output_vio);
-
     data->m_upstream.m_read.close();
-
-    return false;
+    return HeaderState::Fail;
   }
 
+  // save data header string
+  data->m_datelen = sizeof(data->m_date);
+  header.valueForKey(TS_MIME_FIELD_DATE, TS_MIME_LEN_DATE, data->m_date, &data->m_datelen);
+
   // save weak cache header identifiers (rfc7232 section 2)
-  data->m_etaglen = sizeof(data->m_etag) - 1;
+  data->m_etaglen = sizeof(data->m_etag);
   header.valueForKey(TS_MIME_FIELD_ETAG, TS_MIME_LEN_ETAG, data->m_etag, &data->m_etaglen);
-  data->m_lastmodifiedlen = sizeof(data->m_lastmodified) - 1;
+  data->m_lastmodifiedlen = sizeof(data->m_lastmodified);
   header.valueForKey(TS_MIME_FIELD_LAST_MODIFIED, TS_MIME_LEN_LAST_MODIFIED, data->m_lastmodified, &data->m_lastmodifiedlen);
 
   // size of the first block payload
@@ -149,7 +193,7 @@ handleFirstServerHeader(Data *const data, TSCont const contp)
       data->m_dnstream.close();
 
       ERROR_LOG("Bad/invalid response content range");
-      return false;
+      return HeaderState::Fail;
     }
 
     header.setKeyVal(TS_MIME_FIELD_CONTENT_RANGE, TS_MIME_LEN_CONTENT_RANGE, rangestr, rangelen);
@@ -173,7 +217,7 @@ handleFirstServerHeader(Data *const data, TSCont const contp)
   data->m_bytessent = hbytes;
   TSVIOReenable(output_vio);
 
-  return true;
+  return HeaderState::Good;
 }
 
 void
@@ -241,9 +285,8 @@ logSliceError(char const *const message, Data const *const data, HttpHeader cons
   header_resp.valueForKey(TS_MIME_FIELD_ETAG, TS_MIME_LEN_ETAG, etagstr, &etaglen);
 
   // last modified
-  char lmstr[1024];
-  int lmlen = sizeof(lmstr);
-  header_resp.valueForKey(TS_MIME_FIELD_LAST_MODIFIED, TS_MIME_LEN_LAST_MODIFIED, lmstr, &lmlen);
+  time_t lmgot = 0;
+  header_resp.timeForKey(TS_MIME_FIELD_LAST_MODIFIED, TS_MIME_LEN_LAST_MODIFIED, &lmgot);
 
   // cc
   char ccstr[2048];
@@ -279,12 +322,12 @@ logSliceError(char const *const message, Data const *const data, HttpHeader cons
             " status_got=\"%d\""
             " cr_got=\"%.*s\""
             " etag_got=\"%.*s\""
-            " lm_got=\"%.*s\""
+            " lm_got=\"%jd\""
             " cc=\"%.*s\""
-            " via=\"%.*s\"",
+            " via=\"%.*s\"  - attempting to recover",
             secs, ms, message, (int)urlplen, urlpstr, uaslen, uasstr, rangelen, rangestr, normlen, normstr, (int)etagexplen,
             etagexpstr, data->m_lastmodifiedlen, data->m_lastmodified, blockbeg, blockend - 1, statusgot, crlen, crstr,
-            (int)etaggotlen, etaggotstr, lmlen, lmstr, cclen, ccstr, vialen, viastr);
+            (int)etaggotlen, etaggotstr, static_cast<intmax_t>(lmgot), cclen, ccstr, vialen, viastr);
 }
 
 bool
@@ -292,48 +335,134 @@ handleNextServerHeader(Data *const data, TSCont const contp)
 {
   // block response header
   HttpHeader header(data->m_resp_hdrmgr.m_buffer, data->m_resp_hdrmgr.m_lochdr);
-  //  DEBUG_LOG("Next Header:\n%s", header.toString().c_str());
-
-  // only process a 206, everything else just aborts
-  if (TS_HTTP_STATUS_PARTIAL_CONTENT != header.status()) {
-    logSliceError("Non 206 internal block response", data, header);
-    return false;
-  }
-
-  // can't parse the content range header, abort -- might be too strict
-  ContentRange const blockcr = contentRangeFrom(header);
-  if (!blockcr.isValid() || blockcr.m_length != data->m_contentlen) {
-    logSliceError("Mismatch/Bad block Content-Range", data, header);
-    return false;
+  if (TSIsDebugTagSet(PLUGIN_NAME)) {
+    DEBUG_LOG("Next Header:\n%s", header.toString().c_str());
   }
 
   bool same = true;
 
-  // prefer the etag but use Last-Modified if we must.
-  char etag[8192];
-  int etaglen = sizeof(etag);
-  header.valueForKey(TS_MIME_FIELD_ETAG, TS_MIME_LEN_ETAG, etag, &etaglen);
+  switch (header.status()) {
+  case TS_HTTP_STATUS_NOT_FOUND:
+    // need to reissue reference slice
+    logSliceError("404 internal block response (asset gone)", data, header);
+    same = false;
+    break;
+  case TS_HTTP_STATUS_PARTIAL_CONTENT:
+    break;
+  default:
+    DEBUG_LOG("Non 206/404 internal block response encountered");
+    return false;
+    break;
+  }
 
-  if (0 < data->m_etaglen || 0 < etaglen) {
-    same = data->m_etaglen == etaglen && 0 == strncmp(etag, data->m_etag, etaglen);
-    if (!same) {
-      logSliceError("Mismatch block Etag", data, header);
+  // can't parse the content range header, abort -- might be too strict
+  ContentRange blockcr;
+
+  if (same) {
+    blockcr = contentRangeFrom(header);
+    if (!blockcr.isValid() || blockcr.m_length != data->m_contentlen) {
+      logSliceError("Mismatch/Bad block Content-Range", data, header);
+      same = false;
     }
-  } else {
-    char lastmodified[8192];
-    int lastmodifiedlen = sizeof(lastmodified);
-    header.valueForKey(TS_MIME_FIELD_LAST_MODIFIED, TS_MIME_LEN_LAST_MODIFIED, lastmodified, &lastmodifiedlen);
-    if (0 < data->m_lastmodifiedlen || 0 != lastmodifiedlen) {
-      same = data->m_lastmodifiedlen == lastmodifiedlen && 0 == strncmp(lastmodified, data->m_lastmodified, lastmodifiedlen);
+  }
+
+  if (same) {
+    // prefer the etag but use Last-Modified if we must.
+    char etag[8192];
+    int etaglen = sizeof(etag);
+    header.valueForKey(TS_MIME_FIELD_ETAG, TS_MIME_LEN_ETAG, etag, &etaglen);
+
+    if (0 < data->m_etaglen || 0 < etaglen) {
+      same = data->m_etaglen == etaglen && 0 == strncmp(etag, data->m_etag, etaglen);
       if (!same) {
-        logSliceError("Mismatch block Last-Modified", data, header);
+        logSliceError("Mismatch block Etag", data, header);
+      }
+    } else {
+      char lastmodified[33];
+      int lastmodifiedlen = sizeof(lastmodified);
+      header.valueForKey(TS_MIME_FIELD_LAST_MODIFIED, TS_MIME_LEN_LAST_MODIFIED, lastmodified, &lastmodifiedlen);
+      if (0 < data->m_lastmodifiedlen || 0 < lastmodifiedlen) {
+        same = data->m_lastmodifiedlen == lastmodifiedlen && 0 == strncmp(lastmodified, data->m_lastmodified, lastmodifiedlen);
+        if (!same) {
+          logSliceError("Mismatch block Last-Modified", data, header);
+        }
       }
     }
   }
 
-  if (!same) {
-    data->m_upstream.close();
-    return false;
+  // Header mismatch
+  if (same) {
+    // If we were in reference block refetch mode and the headers
+    // still match there is a problem
+    if (BlockState::ActiveRef == data->m_blockstate) {
+      ERROR_LOG("Reference block refetched, got the same block back again");
+      return false;
+    }
+  } else {
+    switch (data->m_blockstate) {
+    case BlockState::Active: {
+      data->m_upstream.abort();
+
+      // Refetch the current interior slice
+      data->m_blockstate = BlockState::PendingInt;
+
+      time_t date = 0;
+      header.timeForKey(TS_MIME_FIELD_DATE, TS_MIME_LEN_DATE, &date);
+
+      // Ask for any slice newer than the cached one
+      time_t const dateims = date + 1;
+
+      DEBUG_LOG("Attempting to reissue interior slice block request with IMS header time: %jd", static_cast<intmax_t>(dateims));
+
+      // add special CRR IMS header to the request
+      HttpHeader headerreq(data->m_req_hdrmgr.m_buffer, data->m_req_hdrmgr.m_lochdr);
+      if (!headerreq.setKeyTime(X_CRR_IMS_HEADER.data(), X_CRR_IMS_HEADER.size(), dateims)) {
+        ERROR_LOG("Failed setting '%.*s'", (int)X_CRR_IMS_HEADER.size(), X_CRR_IMS_HEADER.data());
+        return false;
+      }
+
+    } break;
+    case BlockState::ActiveInt: {
+      data->m_upstream.abort();
+
+      // New interior slice still mismatches, refetch the reference slice
+      data->m_blockstate = BlockState::PendingRef;
+
+      // convert reference date header to time_t
+      time_t const date = TSMimeParseDate(data->m_date, data->m_datelen);
+
+      // Ask for any slice newer than the cached one
+      time_t const dateims = date + 1;
+
+      DEBUG_LOG("Attempting to reissue reference slice block request with IMS header time: %jd", static_cast<intmax_t>(dateims));
+
+      // add special CRR IMS header to the request
+      HttpHeader headerreq(data->m_req_hdrmgr.m_buffer, data->m_req_hdrmgr.m_lochdr);
+      if (!headerreq.setKeyTime(X_CRR_IMS_HEADER.data(), X_CRR_IMS_HEADER.size(), dateims)) {
+        ERROR_LOG("Failed setting '%.*s'", (int)X_CRR_IMS_HEADER.size(), X_CRR_IMS_HEADER.data());
+        return false;
+      }
+
+      // Reset for first block
+      if (Config::RefType::First == data->m_config->m_reftype) {
+        data->m_blocknum = 0;
+      } else {
+        data->m_blocknum = data->m_req_range.firstBlockFor(data->m_config->m_blockbytes);
+      }
+
+      return true;
+
+    } break;
+      // Refetch the reference slice
+    case BlockState::ActiveRef: {
+      // In this state the reference changed otherwise the asset is toast
+      // reset the content length (if content length drove the mismatch)
+      data->m_contentlen = blockcr.m_length;
+      return true;
+    } break;
+    default:
+      break;
+    }
   }
 
   data->m_blockexpected = blockcr.rangeSize();
@@ -347,14 +476,13 @@ handleNextServerHeader(Data *const data, TSCont const contp)
 void
 handle_server_resp(TSCont contp, TSEvent event, Data *const data)
 {
-  DEBUG_LOG("%p handle_server_resp: %s", data, TSHttpEventNameLookup(event));
-
-#if defined(COLLECT_STATS)
-  TSStatIntIncrement(stats::Server, 1);
-#endif
-
   switch (event) {
   case TS_EVENT_VCONN_READ_READY: {
+    if (data->m_blockstate == BlockState::Passthru) {
+      transfer_all_bytes(data);
+      return;
+    }
+
     // has block response header been parsed??
     if (!data->m_server_block_header_parsed) {
       int64_t consumed              = 0;
@@ -365,17 +493,34 @@ handle_server_resp(TSCont contp, TSEvent event, Data *const data)
       TSVIONDoneSet(input_vio, TSVIONDoneGet(input_vio) + consumed);
 
       // the server response header didn't fit into the input buffer.
+      // wait for more data from upstream
       if (TS_PARSE_CONT == res) {
         return;
       }
 
-      // very first server response header
       bool headerStat = false;
 
       if (TS_PARSE_DONE == res) {
         if (!data->m_server_first_header_parsed) {
-          headerStat                         = handleFirstServerHeader(data, contp);
+          HeaderState const state = handleFirstServerHeader(data, contp);
+
           data->m_server_first_header_parsed = true;
+          switch (state) {
+          case HeaderState::Fail:
+            data->m_blockstate = BlockState::Fail;
+            headerStat         = false;
+            break;
+          case HeaderState::Passthru: {
+            data->m_blockstate = BlockState::Passthru;
+            transfer_all_bytes(data);
+            DEBUG_LOG("Going into a passthru state");
+            return;
+          } break;
+          case HeaderState::Good:
+          default:
+            headerStat = true;
+            break;
+          }
         } else {
           headerStat = handleNextServerHeader(data, contp);
         }
@@ -386,7 +531,7 @@ handle_server_resp(TSCont contp, TSEvent event, Data *const data)
       // kill the upstream and allow dnstream to clean up
       if (!headerStat) {
         data->m_upstream.abort();
-        data->m_blockstate = Data::BlockState::Fail;
+        data->m_blockstate = BlockState::Fail;
         if (data->m_dnstream.m_write.isOpen()) {
           TSVIOReenable(data->m_dnstream.m_write.m_vio);
         } else {
@@ -395,8 +540,44 @@ handle_server_resp(TSCont contp, TSEvent event, Data *const data)
         return;
       }
 
-      // how much to fast forward into this data block
-      data->m_blockskip = data->m_req_range.skipBytesForBlock(data->m_config->m_blockbytes, data->m_blocknum);
+      // header may have been successfully parsed but with caveats
+      switch (data->m_blockstate) {
+        // request new version of current internal slice
+      case BlockState::PendingInt:
+      case BlockState::PendingRef: {
+        if (!request_block(contp, data)) {
+          data->m_blockstate = BlockState::Fail;
+          if (data->m_dnstream.m_write.isOpen()) {
+            TSVIOReenable(data->m_dnstream.m_write.m_vio);
+          } else {
+            shutdown(contp, data);
+          }
+        }
+        return;
+      } break;
+      case BlockState::ActiveRef: {
+        // Mark the reference block for "skip".
+        int64_t const blockbytes      = data->m_config->m_blockbytes;
+        int64_t const firstblock      = data->m_req_range.firstBlockFor(blockbytes);
+        int64_t const blockpos        = firstblock * blockbytes;
+        int64_t const firstblockbytes = std::min(blockbytes, data->m_contentlen - blockpos);
+        data->m_blockskip             = firstblockbytes;
+
+        // Check if we should abort the client
+        if (data->m_dnstream.isOpen()) {
+          TSVIO const output_vio    = data->m_dnstream.m_write.m_vio;
+          int64_t const output_done = TSVIONDoneGet(output_vio);
+          int64_t const output_sent = data->m_bytessent;
+          if (output_done == output_sent) {
+            data->m_dnstream.abort();
+          }
+        }
+      } break;
+      default: {
+        // how much to normally fast forward into this data block
+        data->m_blockskip = data->m_req_range.skipBytesForBlock(data->m_config->m_blockbytes, data->m_blocknum);
+      } break;
+      }
     }
 
     transfer_content_bytes(data);
@@ -405,13 +586,40 @@ handle_server_resp(TSCont contp, TSEvent event, Data *const data)
     // fprintf(stderr, "%p: TS_EVENT_VCONN_READ_COMPLETE\n", data);
   } break;
   case TS_EVENT_VCONN_EOS: {
-    data->m_blockstate = Data::BlockState::Pending;
+    switch (data->m_blockstate) {
+    case BlockState::ActiveRef:
+    case BlockState::Passthru: {
+      transfer_all_bytes(data);
+      data->m_upstream.close();
+      TSVIO const output_vio = data->m_dnstream.m_write.m_vio;
+      if (nullptr != output_vio) {
+        TSVIOReenable(output_vio);
+      } else {
+        shutdown(contp, data);
+      }
+      return;
+    } break;
+    default:
+      break;
+    }
+
+    // corner condition, good source header + 0 length aborted content
+    // results in no header being read, just an EOS.
+    // trying to delete the upstream will crash ATS (??)
+    if (0 == data->m_blockexpected) {
+      shutdown(contp, data); // this will crash if first block
+      return;
+    }
+
+    transfer_content_bytes(data);
+
     data->m_upstream.close();
+    data->m_blockstate = BlockState::Pending;
 
     // check for block truncation
     if (data->m_blockconsumed < data->m_blockexpected) {
       DEBUG_LOG("%p handle_server_resp truncation: %" PRId64 "\n", data, data->m_blockexpected - data->m_blockconsumed);
-      data->m_blockstate = Data::BlockState::Fail;
+      data->m_blockstate = BlockState::Fail;
       //      shutdown(contp, data);
       return;
     }
@@ -433,29 +641,33 @@ handle_server_resp(TSCont contp, TSEvent event, Data *const data)
       // Don't immediately request the next slice if the client
       // isn't keeping up
 
-      bool start_next_block = true;
+      if (data->m_dnstream.m_write.isOpen()) {
+        bool start_next_block = true;
 
-      // throttle condition
-      if (data->m_config->m_throttle && data->m_dnstream.m_read.isOpen()) {
+        // check throttle condition
         TSVIO const output_vio    = data->m_dnstream.m_write.m_vio;
         int64_t const output_done = TSVIONDoneGet(output_vio);
         int64_t const output_sent = data->m_bytessent;
         int64_t const threshout   = data->m_config->m_blockbytes;
+        int64_t const buffered    = output_sent - output_done;
 
-        if (threshout < (output_sent - output_done)) {
+        if (threshout < buffered) {
           start_next_block = false;
-          DEBUG_LOG("%p handle_server_resp: throttling %" PRId64, data, (output_sent - output_done));
+          DEBUG_LOG("%p handle_server_resp: throttling %" PRId64, data, buffered);
+        }
+
+        if (start_next_block) {
+          if (!request_block(contp, data)) {
+            data->m_blockstate = BlockState::Fail;
+            abort(contp, data);
+            return;
+          }
         }
       }
-
-      if (start_next_block) {
-        request_block(contp, data);
-      }
-
     } else {
       data->m_upstream.close();
-      data->m_blockstate = Data::BlockState::Done;
-      if (!data->m_dnstream.m_read.isOpen()) {
+      data->m_blockstate = BlockState::Done;
+      if (!data->m_dnstream.m_write.isOpen()) {
         shutdown(contp, data);
       }
     }
