@@ -88,21 +88,22 @@
 #define MAX_RX_MATCH 10
 #define WHITESPACE " \t\r\n"
 
-#include <stdint.h>
+#include <cstdint>
 
 #include <vector>
 #include <set>
 #include <regex.h>
-#include <ctype.h>
-#include <assert.h>
-#include <string.h>
+#include <cctype>
+#include <cassert>
+#include <cstring>
 #include <string>
-#include <stdio.h>
+#include <cstdio>
+#include <stdexcept>
 #include "ts/ts.h"
 
 struct edit_t;
-typedef std::set<edit_t> editset_t;
-typedef editset_t::const_iterator edit_p;
+using editset_t = std::set<edit_t>;
+using edit_p    = editset_t::const_iterator;
 
 struct edit_t {
   const size_t start;
@@ -200,18 +201,18 @@ public:
   }
 
   scope_t(const bool u) : uri(u) { ; }
-  virtual ~scope_t() {}
+  virtual ~scope_t() = default;
 };
 
 class rxscope : public scope_t
 {
 private:
   regex_t rx;
-  virtual bool
+  bool
 
-  match(const char *str) const
+  match(const char *str) const override
   {
-    return (regexec(&rx, str, 0, NULL, 0) == 0) ? true : false;
+    return (regexec(&rx, str, 0, nullptr, 0) == 0) ? true : false;
   }
 
 public:
@@ -228,7 +229,7 @@ public:
     TSfree(str);
   }
 
-  virtual ~rxscope() { regfree(&rx); }
+  ~rxscope() override { regfree(&rx); }
 };
 
 class strscope : public scope_t
@@ -236,16 +237,16 @@ class strscope : public scope_t
 private:
   const bool icase;
   char *str;
-  virtual bool
+  bool
 
-  match(const char *p) const
+  match(const char *p) const override
   {
     return ((icase ? strncasecmp : strncmp)(str, p, strlen(str)) == 0) ? true : false;
   }
 
 public:
   strscope(const bool u, const bool i, const char *pattern, int len) : scope_t(u), icase(i) { str = TSstrndup(pattern, len); }
-  virtual ~strscope()
+  ~strscope() override
   {
     if (str) {
       TSfree(str);
@@ -257,8 +258,8 @@ class match_t
 {
 public:
   virtual bool find(const char *, size_t, size_t &, size_t &, const char *, std::string &) const = 0;
-  virtual size_t cont_size() const = 0;
-  virtual ~match_t() {}
+  virtual size_t cont_size() const                                                               = 0;
+  virtual ~match_t()                                                                             = default;
 };
 
 class strmatch : public match_t
@@ -268,8 +269,8 @@ class strmatch : public match_t
   const size_t slen;
 
 public:
-  virtual bool
-  find(const char *buf, size_t len, size_t &found, size_t &found_len, const char *to, std::string &repl) const
+  bool
+  find(const char *buf, size_t len, size_t &found, size_t &found_len, const char *to, std::string &repl) const override
   {
     const char *match = icase ? strcasestr(buf, str) : strstr(buf, str);
     if (match) {
@@ -283,15 +284,15 @@ public:
   }
 
   strmatch(const bool i, const char *pattern, int len) : icase(i), slen(len) { str = TSstrndup(pattern, len); }
-  virtual ~strmatch()
+  ~strmatch() override
   {
     if (str) {
       TSfree(str);
     }
   }
 
-  virtual size_t
-  cont_size() const
+  size_t
+  cont_size() const override
   {
     return slen;
   }
@@ -303,8 +304,8 @@ class rxmatch : public match_t
   regex_t rx;
 
 public:
-  virtual bool
-  find(const char *buf, size_t len, size_t &found, size_t &found_len, const char *tmpl, std::string &repl) const
+  bool
+  find(const char *buf, size_t len, size_t &found, size_t &found_len, const char *tmpl, std::string &repl) const override
   {
     regmatch_t pmatch[MAX_RX_MATCH];
     if (regexec(&rx, buf, MAX_RX_MATCH, pmatch, REG_NOTEOL) == 0) {
@@ -343,8 +344,8 @@ public:
     }
   }
 
-  virtual size_t
-  cont_size() const
+  size_t
+  cont_size() const override
   {
     return match_len;
   }
@@ -362,7 +363,7 @@ public:
     TSfree(str);
   }
 
-  virtual ~rxmatch() { regfree(&rx); }
+  ~rxmatch() override { regfree(&rx); }
 };
 
 #define PARSE_VERIFY(line, x, str) \
@@ -370,7 +371,7 @@ public:
     if (!isspace(*(x - 1)))        \
       x = strcasestr(x + 1, str);  \
     else                           \
-  break
+      break
 
 class rule_t
 {
@@ -382,7 +383,7 @@ private:
   int *refcount;
 
 public:
-  rule_t(const char *line) : scope(NULL), priority(5), from(NULL), to(NULL), refcount(NULL)
+  rule_t(const char *line) : scope(nullptr), priority(5), from(nullptr), to(nullptr), refcount(nullptr)
   {
     const char *scope_spec = strcasestr(line, "scope:");
     const char *from_spec  = strcasestr(line, "from:");
@@ -547,21 +548,21 @@ public:
     }
   }
 };
-typedef std::vector<rule_t> ruleset_t;
-typedef ruleset_t::const_iterator rule_p;
+using ruleset_t = std::vector<rule_t>;
+using rule_p    = ruleset_t::const_iterator;
 
 typedef struct contdata_t {
-  TSCont cont;
-  TSIOBuffer out_buf;
-  TSIOBufferReader out_rd;
-  TSVIO out_vio;
+  TSCont cont             = nullptr;
+  TSIOBuffer out_buf      = nullptr;
+  TSIOBufferReader out_rd = nullptr;
+  TSVIO out_vio           = nullptr;
   ruleset_t rules;
   std::string contbuf;
-  size_t contbuf_sz;
-  int64_t bytes_in;
-  int64_t bytes_out;
+  size_t contbuf_sz = 0;
+  int64_t bytes_in  = 0;
+  int64_t bytes_out = 0;
   /* Use new/delete so destructor does cleanup for us */
-  contdata_t() : cont(NULL), out_buf(NULL), out_rd(NULL), out_vio(NULL), contbuf_sz(0), bytes_in(0), bytes_out(0) {}
+  contdata_t() = default;
   ~contdata_t()
   {
     if (out_rd) {
@@ -593,7 +594,7 @@ process_block(contdata_t *contdata, TSIOBufferReader reader)
   const char *buf;
   TSIOBufferBlock block;
 
-  if (reader == NULL) { // We're just flushing anything we have buffered
+  if (reader == nullptr) { // We're just flushing anything we have buffered
     keep   = 0;
     buf    = contdata->contbuf.c_str();
     buflen = contdata->contbuf.length();
@@ -616,8 +617,8 @@ process_block(contdata_t *contdata, TSIOBufferReader reader)
 
   editset_t edits;
 
-  for (rule_p r = contdata->rules.begin(); r != contdata->rules.end(); ++r) {
-    r->apply(buf, buflen, edits);
+  for (const auto &rule : contdata->rules) {
+    rule.apply(buf, buflen, edits);
   }
 
   for (edit_p p = edits.begin(); p != edits.end(); ++p) {
@@ -672,11 +673,11 @@ streamedit_process(TSCont contp)
   // Loop over edits, and apply them to the stream
   // Retain buffered data at the end
   int64_t ntodo, nbytes;
-  contdata_t *contdata      = (contdata_t *)TSContDataGet(contp);
+  contdata_t *contdata      = static_cast<contdata_t *>(TSContDataGet(contp));
   TSVIO input_vio           = TSVConnWriteVIOGet(contp);
   TSIOBufferReader input_rd = TSVIOReaderGet(input_vio);
 
-  if (contdata->out_buf == NULL) {
+  if (contdata->out_buf == nullptr) {
     contdata->out_buf = TSIOBufferCreate();
     contdata->out_rd  = TSIOBufferReaderAlloc(contdata->out_buf);
     contdata->out_vio = TSVConnWrite(TSTransformOutputVConnGet(contp), contp, contdata->out_rd, INT64_MAX);
@@ -684,8 +685,8 @@ streamedit_process(TSCont contp)
 
   TSIOBuffer in_buf = TSVIOBufferGet(input_vio);
   /* Test for EOS */
-  if (in_buf == NULL) {
-    process_block(contdata, NULL); // flush any buffered data
+  if (in_buf == nullptr) {
+    process_block(contdata, nullptr); // flush any buffered data
     TSVIONBytesSet(contdata->out_vio, contdata->bytes_out);
     TSVIOReenable(contdata->out_vio);
     return;
@@ -736,7 +737,7 @@ streamedit_filter(TSCont contp, TSEvent event, void *edata)
   TSVIO input_vio;
 
   if (TSVConnClosedGet(contp)) {
-    contdata_t *contdata = (contdata_t *)TSContDataGet(contp);
+    contdata_t *contdata = static_cast<contdata_t *>(TSContDataGet(contp));
     delete contdata;
     return TS_SUCCESS;
   }
@@ -759,24 +760,24 @@ streamedit_filter(TSCont contp, TSEvent event, void *edata)
 static int
 streamedit_setup(TSCont contp, TSEvent event, void *edata)
 {
-  TSHttpTxn txn        = (TSHttpTxn)edata;
-  ruleset_t *rules_in  = (ruleset_t *)TSContDataGet(contp);
-  contdata_t *contdata = NULL;
+  TSHttpTxn txn        = static_cast<TSHttpTxn>(edata);
+  ruleset_t *rules_in  = static_cast<ruleset_t *>(TSContDataGet(contp));
+  contdata_t *contdata = nullptr;
 
   assert((event == TS_EVENT_HTTP_READ_RESPONSE_HDR) || (event == TS_EVENT_HTTP_READ_REQUEST_HDR));
 
   /* make a new list comprising those rules that are in scope */
-  for (rule_p r = rules_in->begin(); r != rules_in->end(); ++r) {
-    if (r->in_scope(txn)) {
-      if (contdata == NULL) {
+  for (const auto &r : *rules_in) {
+    if (r.in_scope(txn)) {
+      if (contdata == nullptr) {
         contdata = new contdata_t();
       }
-      contdata->rules.push_back(*r);
-      contdata->set_cont_size(r->cont_size());
+      contdata->rules.push_back(r);
+      contdata->set_cont_size(r.cont_size());
     }
   }
 
-  if (contdata == NULL) {
+  if (contdata == nullptr) {
     /* Nothing to do */
     return TS_SUCCESS;
   }
@@ -802,19 +803,19 @@ read_conf(const char *filename, ruleset_t *&in, ruleset_t *&out)
   char buf[MAX_CONFIG_LINE];
   FILE *file = fopen(filename, "r");
 
-  if (file == NULL) {
+  if (file == nullptr) {
     TSError("[stream-editor] Failed to open %s", filename);
     return;
   }
-  while (fgets(buf, MAX_CONFIG_LINE, file) != NULL) {
+  while (fgets(buf, MAX_CONFIG_LINE, file) != nullptr) {
     try {
       if (!strncasecmp(buf, "[in]", 4)) {
-        if (in == NULL) {
+        if (in == nullptr) {
           in = new ruleset_t();
         }
         in->push_back(rule_t(buf));
       } else if (!strncasecmp(buf, "[out]", 5)) {
-        if (out == NULL) {
+        if (out == nullptr) {
           out = new ruleset_t();
         }
         out->push_back(rule_t(buf));
@@ -831,15 +832,15 @@ TSPluginInit(int argc, const char *argv[])
 {
   TSPluginRegistrationInfo info;
   TSCont inputcont, outputcont;
-  ruleset_t *rewrites_in  = NULL;
-  ruleset_t *rewrites_out = NULL;
+  ruleset_t *rewrites_in  = nullptr;
+  ruleset_t *rewrites_out = nullptr;
 
   info.plugin_name   = (char *)"stream-editor";
   info.vendor_name   = (char *)"Apache Software Foundation";
   info.support_email = (char *)"users@trafficserver.apache.org";
 
   if (TSPluginRegister(&info) != TS_SUCCESS) {
-    TSError("[stream-editor] Plugin registration failed.");
+    TSError("[stream-editor] Plugin registration failed");
     return;
   }
 
@@ -848,11 +849,11 @@ TSPluginInit(int argc, const char *argv[])
     read_conf(*++argv, rewrites_in, rewrites_out);
   }
 
-  if (rewrites_in != NULL) {
-    TSDebug("[stream-editor]", "initialising input filtering");
-    inputcont = TSContCreate(streamedit_setup, NULL);
-    if (inputcont == NULL) {
-      TSError("[stream-editor] failed to initialise input filtering!");
+  if (rewrites_in != nullptr) {
+    TSDebug("[stream-editor]", "initializing input filtering");
+    inputcont = TSContCreate(streamedit_setup, nullptr);
+    if (inputcont == nullptr) {
+      TSError("[stream-editor] failed to initialize input filtering!");
     } else {
       TSContDataSet(inputcont, rewrites_in);
       TSHttpHookAdd(TS_HTTP_READ_REQUEST_HDR_HOOK, inputcont);
@@ -861,11 +862,11 @@ TSPluginInit(int argc, const char *argv[])
     TSDebug("[stream-editor]", "no input filter rules, skipping filter");
   }
 
-  if (rewrites_out != NULL) {
-    TSDebug("[stream-editor]", "initialising output filtering");
-    outputcont = TSContCreate(streamedit_setup, NULL);
-    if (outputcont == NULL) {
-      TSError("[stream-editor] failed to initialise output filtering!");
+  if (rewrites_out != nullptr) {
+    TSDebug("[stream-editor]", "initializing output filtering");
+    outputcont = TSContCreate(streamedit_setup, nullptr);
+    if (outputcont == nullptr) {
+      TSError("[stream-editor] failed to initialize output filtering!");
     } else {
       TSContDataSet(outputcont, rewrites_out);
       TSHttpHookAdd(TS_HTTP_READ_RESPONSE_HDR_HOOK, outputcont);
