@@ -24,23 +24,25 @@
 #include "ts/remap.h"
 
 #include <sys/types.h>
-#include <stdio.h>
-#include <time.h>
-#include <string.h>
+#include <cstdio>
+#include <ctime>
+#include <cstring>
 
-#include <ctype.h>
+#include <cctype>
 #include <unistd.h>
 
 #include <iostream>
 #include <fstream>
 #include <string>
-#include <ctype.h>
+#include <cctype>
+#include <memory>
+#include <sstream>
 
 // Get some specific stuff from libts, yes, we can do that now that we build inside the core.
-#include "ts/ink_platform.h"
-#include "ts/ink_atomic.h"
-#include "ts/ink_time.h"
-#include "ts/ink_inet.h"
+#include "tscore/ink_platform.h"
+#include "tscore/ink_atomic.h"
+#include "tscore/ink_time.h"
+#include "tscore/ink_inet.h"
 
 #ifdef HAVE_PCRE_PCRE_H
 #include <pcre/pcre.h>
@@ -73,49 +75,35 @@ enum ExtraSubstitutions {
 // length calculations (we need all of them).
 //
 struct UrlComponents {
-  UrlComponents()
-    : scheme(NULL),
-      host(NULL),
-      path(NULL),
-      query(NULL),
-      matrix(NULL),
-      port(0),
-      scheme_len(0),
-      host_len(0),
-      path_len(0),
-      query_len(0),
-      matrix_len(0),
-      url_len(0)
-  {
-  }
+  UrlComponents() = default;
 
   void
-  populate(TSRemapRequestInfo *rri)
+  populate(TSMBuffer bufp, TSMLoc url)
   {
-    scheme = TSUrlSchemeGet(rri->requestBufp, rri->requestUrl, &scheme_len);
-    host   = TSUrlHostGet(rri->requestBufp, rri->requestUrl, &host_len);
-    path   = TSUrlPathGet(rri->requestBufp, rri->requestUrl, &path_len);
-    query  = TSUrlHttpQueryGet(rri->requestBufp, rri->requestUrl, &query_len);
-    matrix = TSUrlHttpParamsGet(rri->requestBufp, rri->requestUrl, &matrix_len);
-    port   = TSUrlPortGet(rri->requestBufp, rri->requestUrl);
+    scheme = TSUrlSchemeGet(bufp, url, &scheme_len);
+    host   = TSUrlHostGet(bufp, url, &host_len);
+    path   = TSUrlPathGet(bufp, url, &path_len);
+    query  = TSUrlHttpQueryGet(bufp, url, &query_len);
+    matrix = TSUrlHttpParamsGet(bufp, url, &matrix_len);
+    port   = TSUrlPortGet(bufp, url);
 
     url_len = scheme_len + host_len + path_len + query_len + matrix_len + 32;
   }
 
-  const char *scheme;
-  const char *host;
-  const char *path;
-  const char *query;
-  const char *matrix;
-  int port;
+  const char *scheme = nullptr;
+  const char *host   = nullptr;
+  const char *path   = nullptr;
+  const char *query  = nullptr;
+  const char *matrix = nullptr;
+  int port           = 0;
 
-  int scheme_len;
-  int host_len;
-  int path_len;
-  int query_len;
-  int matrix_len;
+  int scheme_len = 0;
+  int host_len   = 0;
+  int path_len   = 0;
+  int query_len  = 0;
+  int matrix_len = 0;
 
-  int url_len; // Full length, of all components
+  int url_len = 0; // Full length, of all components
 };
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -124,22 +112,6 @@ struct UrlComponents {
 class RemapRegex
 {
 public:
-  RemapRegex()
-    : _num_subs(-1),
-      _rex(NULL),
-      _extra(NULL),
-      _options(0),
-      _order(-1),
-      _lowercase_substitutions(false),
-      _active_timeout(-1),
-      _no_activity_timeout(-1),
-      _connect_timeout(-1),
-      _dns_timeout(-1),
-      _first_override(NULL)
-  {
-    TSDebug(PLUGIN_NAME, "Calling constructor");
-  }
-
   ~RemapRegex()
   {
     TSDebug(PLUGIN_NAME, "Calling destructor");
@@ -168,7 +140,7 @@ public:
     fprintf(stderr, "[%s]:    Regex %d ( %s ): %.2f%%\n", now, ix, _rex_string, 100.0 * _hits / max);
   }
 
-  int compile(const char **error, int *erroffset);
+  int compile(const char *&error, int &erroffset);
 
   // Perform the regular expression matching against a string.
   int
@@ -219,17 +191,11 @@ public:
   {
     return _rex_string;
   }
-  inline const char *
-  substitution() const
+  inline bool
+  regex_empty() const
   {
-    return _subst;
+    return !_rex_string || !*_rex_string;
   }
-  inline int
-  substitutions_used() const
-  {
-    return _num_subs;
-  }
-
   inline TSHttpStatus
   status_option() const
   {
@@ -277,64 +243,56 @@ public:
   }
 
 private:
-  char *_rex_string;
-  char *_subst;
-  int _subst_len;
-  int _num_subs;
-  int _hits;
+  char *_rex_string = nullptr;
+  char *_subst      = nullptr;
+  int _subst_len    = 0;
+  int _num_subs     = -1;
+  int _hits         = 0;
+  int _options      = 0;
+  int _order        = -1;
 
-  pcre *_rex;
-  pcre_extra *_extra;
-  int _options;
+  bool _lowercase_substitutions = false;
+
+  pcre *_rex           = nullptr;
+  pcre_extra *_extra   = nullptr;
+  RemapRegex *_next    = nullptr;
+  TSHttpStatus _status = static_cast<TSHttpStatus>(0);
+
+  int _active_timeout      = -1;
+  int _no_activity_timeout = -1;
+  int _connect_timeout     = -1;
+  int _dns_timeout         = -1;
+
+  Override *_first_override = nullptr;
   int _sub_pos[MAX_SUBS];
   int _sub_ix[MAX_SUBS];
-  RemapRegex *_next;
-  int _order;
-  TSHttpStatus _status;
-  bool _lowercase_substitutions;
-  int _active_timeout;
-  int _no_activity_timeout;
-  int _connect_timeout;
-  int _dns_timeout;
-
-  Override *_first_override;
 };
 
 bool
 RemapRegex::initialize(const std::string &reg, const std::string &sub, const std::string &opt)
 {
-  _status = static_cast<TSHttpStatus>(0);
-
   if (!reg.empty()) {
     _rex_string = TSstrdup(reg.c_str());
-  } else {
-    _rex_string = NULL;
   }
 
   if (!sub.empty()) {
     _subst     = TSstrdup(sub.c_str());
     _subst_len = sub.length();
-  } else {
-    _subst     = NULL;
-    _subst_len = 0;
   }
 
-  _hits = 0;
-
   memset(_sub_pos, 0, sizeof(_sub_pos));
   memset(_sub_ix, 0, sizeof(_sub_ix));
-  _next = NULL;
 
   // Parse options
-  std::string::size_type start = opt.find_first_of("@");
+  std::string::size_type start = opt.find_first_of('@');
   std::string::size_type pos1, pos2;
-  Override *last_override = NULL;
+  Override *last_override = nullptr;
 
   while (start != std::string::npos) {
     std::string opt_val;
 
     ++start;
-    pos1 = opt.find_first_of("=", start);
+    pos1 = opt.find_first_of('=', start);
     pos2 = opt.find_first_of(" \t\n", pos1);
     if (pos2 == std::string::npos) {
       pos2 = opt.length();
@@ -356,30 +314,30 @@ RemapRegex::initialize(const std::string &reg, const std::string &sub, const std
       TSError("[%s] Malformed options: %s", PLUGIN_NAME, opt.c_str());
       break;
     } else if (opt.compare(start, 6, "status") == 0) {
-      _status = static_cast<TSHttpStatus>(strtol(opt_val.c_str(), NULL, 10));
+      _status = static_cast<TSHttpStatus>(strtol(opt_val.c_str(), nullptr, 10));
     } else if (opt.compare(start, 14, "active_timeout") == 0) {
-      _active_timeout = strtol(opt_val.c_str(), NULL, 10);
+      _active_timeout = strtol(opt_val.c_str(), nullptr, 10);
     } else if (opt.compare(start, 19, "no_activity_timeout") == 0) {
-      _no_activity_timeout = strtol(opt_val.c_str(), NULL, 10);
+      _no_activity_timeout = strtol(opt_val.c_str(), nullptr, 10);
     } else if (opt.compare(start, 15, "connect_timeout") == 0) {
-      _connect_timeout = strtol(opt_val.c_str(), NULL, 10);
+      _connect_timeout = strtol(opt_val.c_str(), nullptr, 10);
     } else if (opt.compare(start, 11, "dns_timeout") == 0) {
-      _dns_timeout = strtol(opt_val.c_str(), NULL, 10);
+      _dns_timeout = strtol(opt_val.c_str(), nullptr, 10);
     } else {
       TSOverridableConfigKey key;
       TSRecordDataType type;
       std::string opt_name = opt.substr(start, pos1 - start - 1);
 
       if (TS_SUCCESS == TSHttpTxnConfigFind(opt_name.c_str(), opt_name.length(), &key, &type)) {
-        Override *cur = new Override;
+        std::unique_ptr<Override> cur(new Override);
 
-        TSReleaseAssert(cur);
+        TSReleaseAssert(cur.get());
         switch (type) {
         case TS_RECORDDATATYPE_INT:
-          cur->data.rec_int = strtoll(opt_val.c_str(), NULL, 10);
+          cur->data.rec_int = strtoll(opt_val.c_str(), nullptr, 10);
           break;
         case TS_RECORDDATATYPE_FLOAT:
-          cur->data.rec_float = strtof(opt_val.c_str(), NULL);
+          cur->data.rec_float = strtof(opt_val.c_str(), nullptr);
           break;
         case TS_RECORDDATATYPE_STRING:
           cur->data.rec_string = TSstrdup(opt_val.c_str());
@@ -387,26 +345,24 @@ RemapRegex::initialize(const std::string &reg, const std::string &sub, const std
           break;
         default:
           TSError("[%s] configuration variable '%s' is of an unsupported type", PLUGIN_NAME, opt_name.c_str());
-          delete cur;
           return false;
         }
-        if (cur) {
-          TSDebug(PLUGIN_NAME, "Overridable config %s=%s", opt_name.c_str(), opt_val.c_str());
-          cur->key  = key;
-          cur->type = type;
-          cur->next = NULL;
-          if (NULL == last_override) {
-            _first_override = cur;
-          } else {
-            last_override->next = cur;
-          }
-          last_override = cur;
+        TSDebug(PLUGIN_NAME, "Overridable config %s=%s", opt_name.c_str(), opt_val.c_str());
+        cur->key  = key;
+        cur->type = type;
+        cur->next = nullptr;
+        auto tmp  = cur.get();
+        if (nullptr == last_override) {
+          _first_override = cur.release();
+        } else {
+          last_override->next = cur.release();
         }
+        last_override = tmp;
       } else {
         TSError("[%s] Unknown options: %s", PLUGIN_NAME, opt.c_str());
       }
     }
-    start = opt.find_first_of("@", pos2);
+    start = opt.find_first_of('@', pos2);
   }
 
   return true;
@@ -414,27 +370,36 @@ RemapRegex::initialize(const std::string &reg, const std::string &sub, const std
 
 // Compile and study the regular expression.
 int
-RemapRegex::compile(const char **error, int *erroffset)
+RemapRegex::compile(const char *&error, int &erroffset)
 {
   char *str;
   int ccount;
 
+  // Initialize these in case they are not set.
+  error     = "unknown error";
+  erroffset = -1;
+
   _rex = pcre_compile(_rex_string, // the pattern
                       _options,    // options
-                      error,       // for error message
-                      erroffset,   // for error offset
-                      NULL);       // use default character tables
+                      &error,      // for error message
+                      &erroffset,  // for error offset
+                      nullptr);    // use default character tables
 
-  if (NULL == _rex) {
+  if (nullptr == _rex) {
     return -1;
   }
 
-  _extra = pcre_study(_rex, 0, error);
-  if ((_extra == NULL) && error && (*error != 0)) {
+  _extra = pcre_study(_rex, PCRE_STUDY_EXTRA_NEEDED, &error);
+  if (error != nullptr) {
     return -1;
   }
 
+  // POOMA - also dependent on actual stack size. Crashes with previous value of 2047,
+  _extra->match_limit_recursion = 1750;
+  _extra->flags |= PCRE_EXTRA_MATCH_LIMIT_RECURSION;
+
   if (pcre_fullinfo(_rex, _extra, PCRE_INFO_CAPTURECOUNT, &ccount) != 0) {
+    error = "call to pcre_fullinfo() failed";
     return -1;
   }
 
@@ -487,8 +452,8 @@ RemapRegex::compile(const char **error, int *erroffset)
 
       if (ix > -1) {
         if ((ix < 10) && (ix > ccount)) {
-          TSDebug(PLUGIN_NAME, "Trying to use unavailable substitution, check the regex!");
-          return -1; // No substitutions available other than $0
+          error = "using unavailable captured substring ($n) in substitution";
+          return -1;
         }
 
         _sub_ix[_num_subs]  = ix;
@@ -585,7 +550,7 @@ RemapRegex::substitute(char dest[], const char *src, const int ovector[], const
         p1 += lengths[ix];
       } else {
         char buff[INET6_ADDRSTRLEN];
-        const char *str = NULL;
+        const char *str = nullptr;
         int len         = 0;
 
         switch (ix) {
@@ -657,29 +622,19 @@ RemapRegex::substitute(char dest[], const char *src, const int ovector[], const
 
 // Hold one remap instance
 struct RemapInstance {
-  RemapInstance()
-    : first(NULL),
-      last(NULL),
-      profile(false),
-      method(false),
-      query_string(true),
-      matrix_params(false),
-      host(false),
-      hits(0),
-      misses(0),
-      filename("unknown")
-  {
-  }
+  RemapInstance() : filename("unknown") {}
 
-  RemapRegex *first;
-  RemapRegex *last;
-  bool profile;
-  bool method;
-  bool query_string;
-  bool matrix_params;
-  bool host;
-  int hits;
-  int misses;
+  RemapRegex *first  = nullptr;
+  RemapRegex *last   = nullptr;
+  bool pristine_url  = false;
+  bool profile       = false;
+  bool method        = false;
+  bool query_string  = true;
+  bool matrix_params = false;
+  bool host          = false;
+  int hits           = 0;
+  int misses         = 0;
+  int failures       = 0;
   std::string filename;
 };
 
@@ -717,7 +672,7 @@ TSRemapInit(TSRemapInterface *api_info, char *errbuf, int errbuf_size)
   }
 
   if (api_info->tsremap_version < TSREMAP_VERSION) {
-    snprintf(errbuf, errbuf_size - 1, "[TSRemapInit] - Incorrect API version %ld.%ld", api_info->tsremap_version >> 16,
+    snprintf(errbuf, errbuf_size, "[TSRemapInit] - Incorrect API version %ld.%ld", api_info->tsremap_version >> 16,
              (api_info->tsremap_version & 0xffff));
     return TS_ERROR;
   }
@@ -733,8 +688,6 @@ TSRemapInit(TSRemapInterface *api_info, char *errbuf, int errbuf_size)
 TSReturnCode
 TSRemapNewInstance(int argc, char *argv[], void **ih, char * /* errbuf ATS_UNUSED */, int /* errbuf_sizeATS_UNUSED */)
 {
-  const char *error;
-  int erroffset;
   RemapInstance *ri = new RemapInstance();
 
   std::ifstream f;
@@ -742,7 +695,7 @@ TSRemapNewInstance(int argc, char *argv[], void **ih, char * /* errbuf ATS_UNUSE
   int count  = 0;
 
   *ih = (void *)ri;
-  if (ri == NULL) {
+  if (ri == nullptr) {
     TSError("[%s] Unable to create remap instance", PLUGIN_NAME);
     return TS_ERROR;
   }
@@ -774,6 +727,10 @@ TSRemapNewInstance(int argc, char *argv[], void **ih, char * /* errbuf ATS_UNUSE
       ri->host = true;
     } else if (strncmp(argv[i], "no-host", 7) == 0) {
       ri->host = false;
+    } else if (strcmp(argv[i], "pristine") == 0) {
+      ri->pristine_url = true;
+    } else if (strcmp(argv[i], "no-pristine") == 0) {
+      ri->pristine_url = false;
     } else {
       TSError("[%s] invalid option '%s'", PLUGIN_NAME, argv[i]);
     }
@@ -851,31 +808,43 @@ TSRemapNewInstance(int argc, char *argv[], void **ih, char * /* errbuf ATS_UNUSE
     }
 
     // Got a regex and substitution string
-    RemapRegex *cur = new RemapRegex();
+    std::unique_ptr<RemapRegex> cur(new RemapRegex);
 
     if (!cur->initialize(regex, subst, options)) {
       TSError("[%s] can't create a new regex remap rule", PLUGIN_NAME);
-      delete cur;
       continue;
     }
 
-    if (cur->compile(&error, &erroffset) < 0) {
-      TSError("[%s] PCRE failed in %s (line %d) at offset %d: %s", PLUGIN_NAME, (ri->filename).c_str(), lineno, erroffset, error);
-      delete (cur);
+    const char *error;
+    int erroffset;
+    if (cur->compile(error, erroffset) < 0) {
+      std::ostringstream oss;
+      oss << '[' << PLUGIN_NAME << "] PCRE failed in " << (ri->filename).c_str() << " (line " << lineno << ')';
+      if (erroffset > 0) {
+        oss << " at offset " << erroffset;
+      }
+      oss << ": " << error;
+      if (cur->regex_empty()) {
+        oss << "  (no regular expression)";
+      } else {
+        oss << "  regex: \"" << cur->regex() << '"';
+      }
+      TSError("%s", oss.str().c_str());
     } else {
       TSDebug(PLUGIN_NAME, "Added regex=%s with subs=%s and options `%s'", regex.c_str(), subst.c_str(), options.c_str());
       cur->set_order(++count);
-      if (ri->first == NULL) {
-        ri->first = cur;
+      auto tmp = cur.get();
+      if (ri->first == nullptr) {
+        ri->first = cur.release();
       } else {
-        ri->last->set_next(cur);
+        ri->last->set_next(cur.release());
       }
-      ri->last = cur;
+      ri->last = tmp;
     }
   }
 
   // Make sure we got something...
-  if (ri->first == NULL) {
+  if (ri->first == nullptr) {
     TSError("[%s] no regular expressions from the maps", PLUGIN_NAME);
     return TS_ERROR;
   }
@@ -892,7 +861,7 @@ TSRemapDeleteInstance(void *ih)
 
   if (ri->profile) {
     char now[64];
-    const ink_time_t tim = time(NULL);
+    const ink_time_t tim = time(nullptr);
 
     if (ink_ctime_r(&tim, now)) {
       now[strlen(now) - 1] = '\0';
@@ -904,6 +873,7 @@ TSRemapDeleteInstance(void *ih)
     fprintf(stderr, "[%s]: Profiling information for regex_remap file `%s':\n", now, (ri->filename).c_str());
     fprintf(stderr, "[%s]:    Total hits (matches): %d\n", now, ri->hits);
     fprintf(stderr, "[%s]:    Total missed (no regex matches): %d\n", now, ri->misses);
+    fprintf(stderr, "[%s]:    Total regex internal errors: %d\n", now, ri->failures);
 
     if (ri->hits > 0) { // Avoid divide by zeros...
       int ix = 1;
@@ -919,7 +889,7 @@ TSRemapDeleteInstance(void *ih)
 
   re = ri->first;
   while (re) {
-    RemapRegex::Override * override = re->get_overrides();
+    RemapRegex::Override *override = re->get_overrides();
 
     while (override) {
       RemapRegex::Override *tmp = override;
@@ -944,16 +914,40 @@ TSRemapDeleteInstance(void *ih)
 TSRemapStatus
 TSRemapDoRemap(void *ih, TSHttpTxn txnp, TSRemapRequestInfo *rri)
 {
-  if (NULL == ih) {
+  if (nullptr == ih) {
     TSDebug(PLUGIN_NAME, "Falling back to default URL on regex remap without rules");
     return TSREMAP_NO_REMAP;
   }
+  RemapInstance *ri = static_cast<RemapInstance *>(ih);
+
+  struct SrcUrl {
+    TSMBuffer bufp;
+    TSMLoc loc;
+    bool bad;
+  };
+
+  const SrcUrl src_url([=]() -> SrcUrl {
+    SrcUrl u;
+
+    if (!ri->pristine_url) {
+      u.bufp = rri->requestBufp;
+      u.loc  = rri->requestUrl;
+      u.bad  = false;
+
+    } else {
+      u.bad = TSHttpTxnPristineUrlGet(txnp, &u.bufp, &u.loc) != TS_SUCCESS;
+    }
+    return u;
+  }());
+
+  if (src_url.bad) {
+    return TSREMAP_NO_REMAP;
+  }
 
   // Populate the request url
   UrlComponents req_url;
-  req_url.populate(rri);
+  req_url.populate(src_url.bufp, src_url.loc);
 
-  RemapInstance *ri = (RemapInstance *)ih;
   int ovector[OVECCOUNT];
   int lengths[OVECCOUNT / 2 + 1];
   int dest_len;
@@ -962,7 +956,7 @@ TSRemapDoRemap(void *ih, TSHttpTxn txnp, TSRemapRequestInfo *rri)
   int match_len        = 0;
   char *match_buf;
 
-  match_buf = (char *)alloca(req_url.url_len + 32);
+  match_buf = static_cast<char *>(alloca(req_url.url_len + 32));
 
   if (ri->method) { // Prepend the URI path or URL with the HTTP method
     TSMBuffer mBuf;
@@ -1011,7 +1005,8 @@ TSRemapDoRemap(void *ih, TSHttpTxn txnp, TSRemapRequestInfo *rri)
   // Apply the regular expressions, in order. First one wins.
   while (re) {
     // Since we check substitutions on parse time, we don't need to reset ovector
-    if (re->match(match_buf, match_len, ovector) != -1) {
+    auto match_result = re->match(match_buf, match_len, ovector);
+    if (match_result >= 0) {
       int new_len = re->get_lengths(ovector, lengths, rri, &req_url);
 
       // Set timeouts
@@ -1037,7 +1032,7 @@ TSRemapDoRemap(void *ih, TSHttpTxn txnp, TSRemapRequestInfo *rri)
         lowercase_substitutions = true;
       }
 
-      RemapRegex::Override * override = re->get_overrides();
+      RemapRegex::Override *override = re->get_overrides();
 
       while (override) {
         switch (override->type) {
@@ -1068,7 +1063,7 @@ TSRemapDoRemap(void *ih, TSHttpTxn txnp, TSRemapRequestInfo *rri)
       if (new_len > 0) {
         char *dest;
 
-        dest     = (char *)alloca(new_len + 8);
+        dest     = static_cast<char *>(alloca(new_len + 8));
         dest_len = re->substitute(dest, match_buf, ovector, lengths, txnp, rri, &req_url, lowercase_substitutions);
 
         TSDebug(PLUGIN_NAME, "New URL is estimated to be %d bytes long, or less", new_len);
@@ -1081,12 +1076,12 @@ TSRemapDoRemap(void *ih, TSHttpTxn txnp, TSRemapRequestInfo *rri)
               re->status_option() != TS_HTTP_STATUS_TEMPORARY_REDIRECT &&
               re->status_option() != TS_HTTP_STATUS_PERMANENT_REDIRECT) {
             // Don't set the URL / Location for this.
-            TSHttpTxnSetHttpRetStatus(txnp, re->status_option());
+            TSHttpTxnStatusSet(txnp, re->status_option());
             break;
           }
 
           TSDebug(PLUGIN_NAME, "Redirecting URL, status=%d", re->status_option());
-          TSHttpTxnSetHttpRetStatus(txnp, re->status_option());
+          TSHttpTxnStatusSet(txnp, re->status_option());
           rri->redirect = 1;
         }
 
@@ -1095,18 +1090,23 @@ TSRemapDoRemap(void *ih, TSHttpTxn txnp, TSRemapRequestInfo *rri)
           const char *start = dest;
 
           // Setup the new URL
-          if (TS_PARSE_ERROR == TSUrlParse(rri->requestBufp, rri->requestUrl, &start, start + dest_len)) {
-            TSHttpTxnSetHttpRetStatus(txnp, TS_HTTP_STATUS_INTERNAL_SERVER_ERROR);
+          if (TS_PARSE_ERROR == TSUrlParse(rri->redirect ? src_url.bufp : rri->requestBufp,
+                                           rri->redirect ? src_url.loc : rri->requestUrl, &start, start + dest_len)) {
+            TSHttpTxnStatusSet(txnp, TS_HTTP_STATUS_INTERNAL_SERVER_ERROR);
             TSError("[%s] can't parse substituted URL string", PLUGIN_NAME);
           }
         }
         break;
       }
+    } else if (match_result != -1) {
+      ink_atomic_increment(&(ri->failures), 1);
+      TSError(R"([%s] Bad regular expression result %d from "%s" in file "%s".)", PLUGIN_NAME, match_result, re->regex(),
+              ri->filename.c_str());
     }
 
     // Try the next regex
     re = re->next();
-    if (re == NULL) {
+    if (re == nullptr) {
       retval = TSREMAP_NO_REMAP; // No match
       if (ri->profile) {
         ink_atomic_increment(&(ri->misses), 1);
