@@ -67,9 +67,9 @@ typedef struct invalidate_t {
 } invalidate_t;
 
 typedef struct {
-  invalidate_t *volatile invalidate_list;
+  invalidate_t *invalidate_list;
   char *config_file;
-  volatile time_t last_load;
+  time_t last_load;
   TSTextLogObject log;
 } plugin_state_t;
 
@@ -366,7 +366,7 @@ config_handler(TSCont cont, TSEvent event, void *edata)
     if (iptr) {
       free_cont = TSContCreate(free_handler, TSMutexCreate());
       TSContDataSet(free_cont, (void *)iptr);
-      TSContSchedule(free_cont, FREE_TMOUT, TS_THREAD_POOL_TASK);
+      TSContScheduleOnPool(free_cont, FREE_TMOUT, TS_THREAD_POOL_TASK);
     }
   } else {
     TSDebug(LOG_PREFIX, "No Changes");
@@ -379,7 +379,7 @@ config_handler(TSCont cont, TSEvent event, void *edata)
 
   // Don't reschedule for TS_EVENT_MGMT_UPDATE
   if (event == TS_EVENT_TIMEOUT) {
-    TSContSchedule(cont, CONFIG_TMOUT, TS_THREAD_POOL_TASK);
+    TSContScheduleOnPool(cont, CONFIG_TMOUT, TS_THREAD_POOL_TASK);
   }
   return 0;
 }
@@ -486,7 +486,7 @@ TSPluginInit(int argc, const char *argv[])
   invalidate_t *iptr        = NULL;
   bool disable_timed_reload = false;
 
-  TSDebug(LOG_PREFIX, "Starting plugin init.");
+  TSDebug(LOG_PREFIX, "Starting plugin init");
 
   pstate = (plugin_state_t *)TSmalloc(sizeof(plugin_state_t));
   init_plugin_state_t(pstate);
@@ -504,7 +504,6 @@ TSPluginInit(int argc, const char *argv[])
       break;
     case 'l':
       if (TS_SUCCESS == TSTextLogObjectCreate(optarg, TS_LOG_MODE_ADD_TIMESTAMP, &pstate->log)) {
-        TSTextLogObjectRollingEnabledSet(pstate->log, 1);
         TSTextLogObjectRollingIntervalSecSet(pstate->log, LOG_ROLL_INTERVAL);
         TSTextLogObjectRollingOffsetHrSet(pstate->log, LOG_ROLL_OFFSET);
       }
@@ -518,7 +517,7 @@ TSPluginInit(int argc, const char *argv[])
   }
 
   if (!pstate->config_file) {
-    TSError("[regex_revalidate] Plugin requires a --config option along with a config file name.");
+    TSError("[regex_revalidate] Plugin requires a --config option along with a config file name");
     free_plugin_state_t(pstate);
     return;
   }
@@ -535,12 +534,12 @@ TSPluginInit(int argc, const char *argv[])
   info.support_email = "dev@trafficserver.apache.org";
 
   if (TSPluginRegister(&info) != TS_SUCCESS) {
-    TSError("[regex_revalidate] Plugin registration failed.");
+    TSError("[regex_revalidate] Plugin registration failed");
 
     free_plugin_state_t(pstate);
     return;
   } else {
-    TSDebug(LOG_PREFIX, "Plugin registration succeeded.");
+    TSDebug(LOG_PREFIX, "Plugin registration succeeded");
   }
 
   if (!check_ts_version()) {
@@ -562,8 +561,8 @@ TSPluginInit(int argc, const char *argv[])
   TSMgmtUpdateRegister(config_cont, LOG_PREFIX);
 
   if (!disable_timed_reload) {
-    TSContSchedule(config_cont, CONFIG_TMOUT, TS_THREAD_POOL_TASK);
+    TSContScheduleOnPool(config_cont, CONFIG_TMOUT, TS_THREAD_POOL_TASK);
   }
 
-  TSDebug(LOG_PREFIX, "Plugin Init Complete.");
+  TSDebug(LOG_PREFIX, "Plugin Init Complete");
 }
