@@ -16,30 +16,75 @@
  * limitations under the License.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
+#include <cstdlib>
+#include <cstdio>
+#include <cstdio>
 #include <strings.h>
-#include <string.h>
+#include <sstream>
+#include <cstring>
+#include <atomic>
+#include <memory>
 #include <getopt.h>
+#include <cstdint>
+#include <cinttypes>
+#include <string_view>
+#include <unistd.h>
 
-#include "ts/ts.h"
-#include "ts/ink_defs.h"
+#include <ts/ts.h>
+#include "tscore/ink_defs.h"
+#include "tscpp/util/PostScript.h"
+#include "tscpp/util/TextView.h"
+#include "Cleanup.h"
+
+namespace
+{
+struct BodyBuilder {
+  atscppapi::TSContUniqPtr transform_connp;
+  atscppapi::TSIOBufferUniqPtr output_buffer;
+  // It's important that output_reader comes after output_buffer so it will be deleted first.
+  atscppapi::TSIOBufferReaderUniqPtr output_reader;
+  TSVIO output_vio   = nullptr;
+  bool wrote_prebody = false;
+  bool wrote_body    = false;
+  bool hdr_ready     = false;
+  std::atomic_flag wrote_postbody;
+
+  int64_t nbytes = 0;
+};
+
+struct XDebugTxnAuxData {
+  std::unique_ptr<BodyBuilder> body_builder;
+  unsigned xheaders = 0;
+};
+
+atscppapi::TxnAuxMgrData mgrData;
+
+using AuxDataMgr = atscppapi::TxnAuxDataMgr<XDebugTxnAuxData, mgrData>;
+
+} // end anonymous namespace
+
+#include "xdebug_headers.cc"
+#include "xdebug_transforms.cc"
 
 static struct {
   const char *str;
   int len;
-} xDebugHeader = {NULL, 0};
+} xDebugHeader = {nullptr, 0};
 
 enum {
-  XHEADER_X_CACHE_KEY      = 0x0004u,
-  XHEADER_X_MILESTONES     = 0x0008u,
-  XHEADER_X_CACHE          = 0x0010u,
-  XHEADER_X_GENERATION     = 0x0020u,
-  XHEADER_X_TRANSACTION_ID = 0x0040u,
+  XHEADER_X_CACHE_KEY      = 1u << 2,
+  XHEADER_X_MILESTONES     = 1u << 3,
+  XHEADER_X_CACHE          = 1u << 4,
+  XHEADER_X_GENERATION     = 1u << 5,
+  XHEADER_X_TRANSACTION_ID = 1u << 6,
+  XHEADER_X_DUMP_HEADERS   = 1u << 7,
+  XHEADER_X_REMAP          = 1u << 8,
+  XHEADER_X_PROBE_HEADERS  = 1u << 9,
+  XHEADER_X_PSELECT_KEY    = 1u << 10,
 };
 
-static int XArgIndex             = 0;
-static TSCont XInjectHeadersCont = NULL;
+static TSCont XInjectHeadersCont  = nullptr;
+static TSCont XDeleteDebugHdrCont = nullptr;
 
 // Return the length of a string literal.
 template <int N>
@@ -91,7 +136,7 @@ InjectCacheKeyHeader(TSHttpTxn txn, TSMBuffer buffer, TSMLoc hdr)
   struct {
     char *ptr;
     int len;
-  } strval = {NULL, 0};
+  } strval = {nullptr, 0};
 
   TSDebug("xdebug", "attempting to inject X-Cache-Key header");
 
@@ -104,7 +149,7 @@ InjectCacheKeyHeader(TSHttpTxn txn, TSMBuffer buffer, TSMLoc hdr)
   }
 
   strval.ptr = TSUrlStringGet(buffer, url, &strval.len);
-  if (strval.ptr == NULL || strval.len == 0) {
+  if (strval.ptr == nullptr || strval.len == 0) {
     goto done;
   }
 
@@ -115,7 +160,7 @@ InjectCacheKeyHeader(TSHttpTxn txn, TSMBuffer buffer, TSMLoc hdr)
   }
 
   // Now copy the cache lookup URL into the response header.
-  TSReleaseAssert(TSMimeHdrFieldValueStringInsert(buffer, hdr, dst, 0 /* idx */, strval.ptr, strval.len) == TS_SUCCESS);
+  TSReleaseAssert(TSMimeHdrFieldValueStringInsert(buffer, hdr, dst, -1 /* idx */, strval.ptr, strval.len) == TS_SUCCESS);
 
 done:
   if (dst != TS_NULL_MLOC) {
@@ -152,11 +197,11 @@ InjectCacheHeader(TSHttpTxn txn, TSMBuffer buffer, TSMLoc hdr)
 
   if (TSHttpTxnCacheLookupStatusGet(txn, &status) == TS_ERROR) {
     // If the cache lookup hasn't happened yes, TSHttpTxnCacheLookupStatusGet will fail.
-    TSReleaseAssert(TSMimeHdrFieldValueStringInsert(buffer, hdr, dst, 0 /* idx */, "none", 4) == TS_SUCCESS);
+    TSReleaseAssert(TSMimeHdrFieldValueStringInsert(buffer, hdr, dst, -1 /* idx */, "none", 4) == TS_SUCCESS);
   } else {
-    const char *msg = (status < 0 || status >= (int)countof(names)) ? "unknown" : names[status];
+    const char *msg = (status < 0 || status >= static_cast<int>(countof(names))) ? "unknown" : names[status];
 
-    TSReleaseAssert(TSMimeHdrFieldValueStringInsert(buffer, hdr, dst, 0 /* idx */, msg, -1) == TS_SUCCESS);
+    TSReleaseAssert(TSMimeHdrFieldValueStringInsert(buffer, hdr, dst, -1 /* idx */, msg, -1) == TS_SUCCESS);
   }
 
 done:
@@ -224,10 +269,10 @@ InjectMilestonesHeader(TSHttpTxn txn, TSMBuffer buffer, TSMLoc hdr)
     // state machine the request doesn't traverse.
     TSHttpTxnMilestoneGet(txn, milestones[i].mstype, &time);
     if (time > 0) {
-      double elapsed = (double)(time - epoch) / 1000000000.0;
-      int len        = (int)snprintf(hdrval, sizeof(hdrval), "%s=%1.9lf", milestones[i].msname, elapsed);
+      double elapsed = static_cast<double>(time - epoch) / 1000000000.0;
+      int len        = snprintf(hdrval, sizeof(hdrval), "%s=%1.9lf", milestones[i].msname, elapsed);
 
-      TSReleaseAssert(TSMimeHdrFieldValueStringInsert(buffer, hdr, dst, 0 /* idx */, hdrval, len) == TS_SUCCESS);
+      TSReleaseAssert(TSMimeHdrFieldValueStringInsert(buffer, hdr, dst, -1 /* idx */, hdrval, len) == TS_SUCCESS);
     }
   }
 
@@ -237,6 +282,63 @@ done:
   }
 }
 
+static const char NotFound[] = "Not-Found";
+
+// The returned string must be freed with TSfree() unless it is equal to the constant NotFound.
+//
+static const char *
+getRemapUrlStr(TSHttpTxn txnp, TSReturnCode (*remapUrlGetFunc)(TSHttpTxn, TSMLoc *), int &urlStrLen)
+{
+  TSMLoc urlLoc;
+
+  TSReturnCode rc = remapUrlGetFunc(txnp, &urlLoc);
+  if (rc != TS_SUCCESS) {
+    urlStrLen = sizeof(NotFound) - 1;
+    return NotFound;
+  }
+
+  char *urlStr = TSUrlStringGet(nullptr, urlLoc, &urlStrLen);
+
+  // Be defensive.
+  if ((urlStrLen == 0) and urlStr) {
+    TSError("[xdebug] non-null remap URL string with zero length");
+    TSfree(urlStr);
+    urlStr = nullptr;
+  }
+
+  if (!urlStr) {
+    urlStrLen = sizeof(NotFound) - 1;
+    return NotFound;
+  }
+
+  return urlStr;
+}
+
+static void
+InjectRemapHeader(TSHttpTxn txn, TSMBuffer buffer, TSMLoc hdr)
+{
+  TSMLoc dst = FindOrMakeHdrField(buffer, hdr, "X-Remap", lengthof("X-Remap"));
+
+  if (TS_NULL_MLOC != dst) {
+    int fromUrlStrLen, toUrlStrLen;
+    const char *fromUrlStr = getRemapUrlStr(txn, TSRemapFromUrlGet, fromUrlStrLen);
+    const char *toUrlStr   = getRemapUrlStr(txn, TSRemapToUrlGet, toUrlStrLen);
+
+    char buf[2048];
+    int len = snprintf(buf, sizeof(buf), "from=%*s, to=%*s", fromUrlStrLen, fromUrlStr, toUrlStrLen, toUrlStr);
+
+    if (fromUrlStr != NotFound) {
+      TSfree(const_cast<char *>(fromUrlStr));
+    }
+    if (toUrlStr != NotFound) {
+      TSfree(const_cast<char *>(toUrlStr));
+    }
+
+    TSReleaseAssert(TSMimeHdrFieldValueStringInsert(buffer, hdr, dst, -1 /* idx */, buf, len) == TS_SUCCESS);
+    TSHandleMLocRelease(buffer, hdr, dst);
+  }
+}
+
 static void
 InjectTxnUuidHeader(TSHttpTxn txn, TSMBuffer buffer, TSMLoc hdr)
 {
@@ -245,24 +347,70 @@ InjectTxnUuidHeader(TSHttpTxn txn, TSMBuffer buffer, TSMLoc hdr)
   if (TS_NULL_MLOC != dst) {
     char buf[TS_UUID_STRING_LEN + 22]; // Padded for int64_t (20) + 1 ('-') + 1 ('\0')
     TSUuid uuid = TSProcessUuidGet();
-    int len     = snprintf(buf, sizeof(buf) - 1, "%s-%" PRIu64 "", TSUuidStringGet(uuid), TSHttpTxnIdGet(txn));
+    int len     = snprintf(buf, sizeof(buf), "%s-%" PRIu64 "", TSUuidStringGet(uuid), TSHttpTxnIdGet(txn));
 
-    TSReleaseAssert(TSMimeHdrFieldValueStringInsert(buffer, hdr, dst, 0 /* idx */, buf, len) == TS_SUCCESS);
+    TSReleaseAssert(TSMimeHdrFieldValueStringInsert(buffer, hdr, dst, -1 /* idx */, buf, len) == TS_SUCCESS);
     TSHandleMLocRelease(buffer, hdr, dst);
   }
 }
 
+static void
+InjectParentSelectionKeyHeader(TSHttpTxn txn, TSMBuffer buffer, TSMLoc hdr)
+{
+  TSMLoc url = TS_NULL_MLOC;
+  TSMLoc dst = TS_NULL_MLOC;
+
+  struct {
+    char *ptr;
+    int len;
+  } strval = {nullptr, 0};
+
+  TSDebug("xdebug", "attempting to inject X-ParentSelection-Key header");
+
+  if (TSUrlCreate(buffer, &url) != TS_SUCCESS) {
+    goto done;
+  }
+
+  if (TSHttpTxnParentSelectionUrlGet(txn, buffer, url) != TS_SUCCESS) {
+    goto done;
+  }
+
+  strval.ptr = TSUrlStringGet(buffer, url, &strval.len);
+  if (strval.ptr == nullptr || strval.len == 0) {
+    goto done;
+  }
+
+  // Create a new response header field.
+  dst = FindOrMakeHdrField(buffer, hdr, "X-ParentSelection-Key", lengthof("X-ParentSelection-Key"));
+  if (dst == TS_NULL_MLOC) {
+    goto done;
+  }
+
+  // Now copy the parent selection lookup URL into the response header.
+  TSReleaseAssert(TSMimeHdrFieldValueStringInsert(buffer, hdr, dst, -1 /* idx */, strval.ptr, strval.len) == TS_SUCCESS);
+
+done:
+  if (dst != TS_NULL_MLOC) {
+    TSHandleMLocRelease(buffer, hdr, dst);
+  }
+
+  if (url != TS_NULL_MLOC) {
+    TSHandleMLocRelease(buffer, TS_NULL_MLOC, url);
+  }
+
+  TSfree(strval.ptr);
+}
+
 static int
 XInjectResponseHeaders(TSCont /* contp */, TSEvent event, void *edata)
 {
-  TSHttpTxn txn     = (TSHttpTxn)edata;
-  intptr_t xheaders = 0;
+  TSHttpTxn txn = static_cast<TSHttpTxn>(edata);
   TSMBuffer buffer;
   TSMLoc hdr;
 
   TSReleaseAssert(event == TS_EVENT_HTTP_SEND_RESPONSE_HDR);
 
-  xheaders = (intptr_t)TSHttpTxnArgGet(txn, XArgIndex);
+  unsigned xheaders = AuxDataMgr::data(txn).xheaders;
   if (xheaders == 0) {
     goto done;
   }
@@ -291,26 +439,103 @@ XInjectResponseHeaders(TSCont /* contp */, TSEvent event, void *edata)
     InjectTxnUuidHeader(txn, buffer, hdr);
   }
 
+  if (xheaders & XHEADER_X_REMAP) {
+    InjectRemapHeader(txn, buffer, hdr);
+  }
+
+  // intentionally placed after all injected headers.
+
+  if (xheaders & XHEADER_X_DUMP_HEADERS) {
+    log_headers(txn, buffer, hdr, "ClientResponse");
+  }
+
+  if (xheaders & XHEADER_X_PROBE_HEADERS) {
+    BodyBuilder *data = AuxDataMgr::data(txn).body_builder.get();
+    TSDebug("xdebug_transform", "XInjectResponseHeaders(): client resp header ready");
+    if (data == nullptr) {
+      TSHttpTxnReenable(txn, TS_EVENT_HTTP_ERROR);
+      return TS_ERROR;
+    }
+    data->hdr_ready = true;
+    writePostBody(txn, data);
+  }
+
+  if (xheaders & XHEADER_X_PSELECT_KEY) {
+    InjectParentSelectionKeyHeader(txn, buffer, hdr);
+  }
+
 done:
   TSHttpTxnReenable(txn, TS_EVENT_HTTP_CONTINUE);
   return TS_EVENT_NONE;
 }
 
+static bool
+isFwdFieldValue(std::string_view value, intmax_t &fwdCnt)
+{
+  static const ts::TextView paramName("fwd");
+
+  if (value.size() < paramName.size()) {
+    return false;
+  }
+
+  ts::TextView tvVal(value);
+
+  if (strcasecmp(paramName, tvVal.prefix(paramName.size())) != 0) {
+    return false;
+  }
+
+  tvVal.remove_prefix(paramName.size());
+
+  if (tvVal.size() == 0) {
+    // Value is 'fwd' with no '=<count>'.
+    fwdCnt = -1;
+    return true;
+  }
+
+  const char httpSpace[] = " \t";
+
+  tvVal.ltrim(httpSpace);
+
+  if (tvVal[0] != '=') {
+    return false;
+  }
+
+  tvVal.remove_prefix(1);
+  tvVal.ltrim(httpSpace);
+
+  size_t sz  = tvVal.size();
+  intmax_t i = ts::svtoi(tvVal, &tvVal);
+
+  if ((tvVal.size() != sz) or (i < 0)) {
+    // There were crud characters after the number, or the number was negative.
+    return false;
+  }
+
+  fwdCnt = i;
+
+  return true;
+}
+
 // Scan the client request headers and determine which debug headers they
 // want in the response.
 static int
 XScanRequestHeaders(TSCont /* contp */, TSEvent event, void *edata)
 {
-  TSHttpTxn txn     = (TSHttpTxn)edata;
-  intptr_t xheaders = 0;
+  TSHttpTxn txn     = static_cast<TSHttpTxn>(edata);
+  unsigned xheaders = 0;
+  intmax_t fwdCnt   = 0;
   TSMLoc field, next;
   TSMBuffer buffer;
   TSMLoc hdr;
 
+  // Make sure TSHttpTxnReenable(txn, TS_EVENT_HTTP_CONTINUE) is called before exiting function.
+  //
+  ts::PostScript ps([=]() -> void { TSHttpTxnReenable(txn, TS_EVENT_HTTP_CONTINUE); });
+
   TSReleaseAssert(event == TS_EVENT_HTTP_READ_REQUEST_HDR);
 
   if (TSHttpTxnClientReqGet(txn, &buffer, &hdr) == TS_ERROR) {
-    goto done;
+    return TS_EVENT_NONE;
   }
 
   TSDebug("xdebug", "scanning for %s header values", xDebugHeader.str);
@@ -325,7 +550,7 @@ XScanRequestHeaders(TSCont /* contp */, TSEvent event, void *edata)
       int vsize;
 
       value = TSMimeHdrFieldValueStringGet(buffer, hdr, field, i, &vsize);
-      if (value == NULL || vsize == 0) {
+      if (value == nullptr || vsize == 0) {
         continue;
       }
 
@@ -341,12 +566,46 @@ XScanRequestHeaders(TSCont /* contp */, TSEvent event, void *edata)
         xheaders |= XHEADER_X_GENERATION;
       } else if (header_field_eq("x-transaction-id", value, vsize)) {
         xheaders |= XHEADER_X_TRANSACTION_ID;
+      } else if (header_field_eq("x-remap", value, vsize)) {
+        xheaders |= XHEADER_X_REMAP;
       } else if (header_field_eq("via", value, vsize)) {
         // If the client requests the Via header, enable verbose Via debugging for this transaction.
         TSHttpTxnConfigIntSet(txn, TS_CONFIG_HTTP_INSERT_RESPONSE_VIA_STR, 3);
       } else if (header_field_eq("diags", value, vsize)) {
         // Enable diagnostics for DebugTxn()'s only
         TSHttpTxnDebugSet(txn, 1);
+
+      } else if (header_field_eq("probe", value, vsize)) {
+        xheaders |= XHEADER_X_PROBE_HEADERS;
+
+        auto &auxData = AuxDataMgr::data(txn);
+
+        // prefix request headers and postfix response headers
+        BodyBuilder *data = new BodyBuilder();
+        auxData.body_builder.reset(data);
+
+        TSVConn connp = TSTransformCreate(body_transform, txn);
+        data->transform_connp.reset(connp);
+        TSContDataSet(connp, txn);
+        TSHttpTxnHookAdd(txn, TS_HTTP_RESPONSE_TRANSFORM_HOOK, connp);
+
+        // disable writing to cache because we are injecting data into the body.
+        TSHttpTxnReqCacheableSet(txn, 0);
+        TSHttpTxnRespCacheableSet(txn, 0);
+        TSHttpTxnServerRespNoStoreSet(txn, 1);
+        TSHttpTxnTransformedRespCache(txn, 0);
+        TSHttpTxnUntransformedRespCache(txn, 0);
+
+      } else if (header_field_eq("x-parentselection-key", value, vsize)) {
+        xheaders |= XHEADER_X_PSELECT_KEY;
+
+      } else if (isFwdFieldValue(std::string_view(value, vsize), fwdCnt)) {
+        if (fwdCnt > 0) {
+          // Decrement forward count in X-Debug header.
+          char newVal[128];
+          snprintf(newVal, sizeof(newVal), "fwd=%" PRIiMAX, fwdCnt - 1);
+          TSMimeHdrFieldValueStringSet(buffer, hdr, field, i, newVal, std::strlen(newVal));
+        }
       } else {
         TSDebug("xdebug", "ignoring unrecognized debug tag '%.*s'", vsize, value);
       }
@@ -357,9 +616,6 @@ XScanRequestHeaders(TSCont /* contp */, TSEvent event, void *edata)
     // Get the next duplicate.
     next = TSMimeHdrFieldNextDup(buffer, hdr, field);
 
-    // Destroy the current field that we have. We don't want this to go through and potentially confuse the origin.
-    TSMimeHdrFieldDestroy(buffer, hdr, field);
-
     // Now release our reference.
     TSHandleMLocRelease(buffer, hdr, field);
 
@@ -368,21 +624,58 @@ XScanRequestHeaders(TSCont /* contp */, TSEvent event, void *edata)
   }
 
   if (xheaders) {
-    TSDebug("xdebug", "adding response hook for header mask %p", (void *)xheaders);
+    TSDebug("xdebug", "adding response hook for header mask %p and forward count %" PRIiMAX, reinterpret_cast<void *>(xheaders),
+            fwdCnt);
     TSHttpTxnHookAdd(txn, TS_HTTP_SEND_RESPONSE_HDR_HOOK, XInjectHeadersCont);
-    TSHttpTxnArgSet(txn, XArgIndex, (void *)xheaders);
+    AuxDataMgr::data(txn).xheaders = xheaders;
+
+    if (fwdCnt == 0) {
+      // X-Debug header has to be deleted, but not too soon for other plugins to see it.
+      TSHttpTxnHookAdd(txn, TS_HTTP_CACHE_LOOKUP_COMPLETE_HOOK, XDeleteDebugHdrCont);
+    }
   }
 
-done:
-  TSHttpTxnReenable(txn, TS_EVENT_HTTP_CONTINUE);
+  return TS_EVENT_NONE;
+}
+
+// Continuation function to delete the x-debug header.
+//
+static int
+XDeleteDebugHdr(TSCont /* contp */, TSEvent event, void *edata)
+{
+  TSHttpTxn txn = static_cast<TSHttpTxn>(edata);
+  TSMLoc hdr, field;
+  TSMBuffer buffer;
+
+  // Make sure TSHttpTxnReenable(txn, TS_EVENT_HTTP_CONTINUE) is called before exiting function.
+  //
+  ts::PostScript ps([=]() -> void { TSHttpTxnReenable(txn, TS_EVENT_HTTP_CONTINUE); });
+
+  TSReleaseAssert(event == TS_EVENT_HTTP_CACHE_LOOKUP_COMPLETE);
+
+  if (TSHttpTxnClientReqGet(txn, &buffer, &hdr) == TS_ERROR) {
+    return TS_EVENT_NONE;
+  }
+
+  field = TSMimeHdrFieldFind(buffer, hdr, xDebugHeader.str, xDebugHeader.len);
+  if (field == TS_NULL_MLOC) {
+    return TS_EVENT_NONE;
+  }
+
+  if (TSMimeHdrFieldDestroy(buffer, hdr, field) == TS_ERROR) {
+    TSError("Failure destroying %s header", xDebugHeader.str);
+  }
+
+  TSHandleMLocRelease(buffer, hdr, field);
+
   return TS_EVENT_NONE;
 }
 
 void
 TSPluginInit(int argc, const char *argv[])
 {
-  static const struct option longopt[] = {{const_cast<char *>("header"), required_argument, NULL, 'h'},
-                                          {NULL, no_argument, NULL, '\0'}};
+  static const struct option longopt[] = {{const_cast<char *>("header"), required_argument, nullptr, 'h'},
+                                          {nullptr, no_argument, nullptr, '\0'}};
   TSPluginRegistrationInfo info;
 
   info.plugin_name   = (char *)"xdebug";
@@ -395,7 +688,7 @@ TSPluginInit(int argc, const char *argv[])
 
   // Parse the arguments
   while (true) {
-    int opt = getopt_long(argc, (char *const *)argv, "", longopt, NULL);
+    int opt = getopt_long(argc, const_cast<char *const *>(argv), "", longopt, nullptr);
 
     switch (opt) {
     case 'h':
@@ -408,15 +701,24 @@ TSPluginInit(int argc, const char *argv[])
     }
   }
 
-  if (NULL == xDebugHeader.str) {
+  if (nullptr == xDebugHeader.str) {
     xDebugHeader.str = TSstrdup("X-Debug"); // We malloc this, for consistency for future plugin unload events
   }
   xDebugHeader.len = strlen(xDebugHeader.str);
 
-  // Setup the global hook
-  TSReleaseAssert(TSHttpArgIndexReserve("xdebug", "xdebug header requests", &XArgIndex) == TS_SUCCESS);
-  TSReleaseAssert(XInjectHeadersCont = TSContCreate(XInjectResponseHeaders, NULL));
-  TSHttpHookAdd(TS_HTTP_READ_REQUEST_HDR_HOOK, TSContCreate(XScanRequestHeaders, NULL));
-}
+  // Make xDebugHeader available to other plugins, as a C-style string.
+  //
+  int idx = -1;
+  TSReleaseAssert(TSUserArgIndexReserve(TS_USER_ARGS_GLB, "XDebugHeader", "XDebug header name", &idx) == TS_SUCCESS);
+  TSReleaseAssert(idx >= 0);
+  TSUserArgSet(nullptr, idx, const_cast<char *>(xDebugHeader.str));
 
-// vim: set ts=2 sw=2 et :
+  AuxDataMgr::init("xdebug");
+
+  // Setup the global hook
+  TSReleaseAssert(XInjectHeadersCont = TSContCreate(XInjectResponseHeaders, nullptr));
+  TSReleaseAssert(XDeleteDebugHdrCont = TSContCreate(XDeleteDebugHdr, nullptr));
+  TSHttpHookAdd(TS_HTTP_READ_REQUEST_HDR_HOOK, TSContCreate(XScanRequestHeaders, nullptr));
+
+  gethostname(Hostname, 1024);
+}
