@@ -30,22 +30,21 @@
 
 #include <sys/types.h>
 
-#include "ts/ink_config.h"
-#include "ts/MatcherUtils.h"
-#include "ts/Tokenizer.h"
-#include "Main.h"
+#include "tscore/ink_config.h"
+#include "tscore/MatcherUtils.h"
+#include "tscore/Tokenizer.h"
+#include "tscore/ts_file.h"
 #include "ProxyConfig.h"
 #include "ControlMatcher.h"
 #include "CacheControl.h"
 #include "ParentSelection.h"
-#include "ts/HostLookup.h"
+#include "tscore/HostLookup.h"
 #include "HTTP.h"
 #include "URL.h"
 #include "P_EventSystem.h"
 #include "P_Net.h"
 #include "P_Cache.h"
 #include "P_SplitDNS.h"
-#include "congest/Congestion.h"
 
 /****************************************************************
  *   Place all template instantiations at the bottom of the file
@@ -57,7 +56,7 @@
 char *
 HttpRequestData::get_string()
 {
-  char *str = hdr->url_string_get(NULL);
+  char *str = hdr->url_string_get(nullptr);
 
   if (str) {
     unescapifyStr(str);
@@ -87,54 +86,52 @@ HttpRequestData::get_client_ip()
  *   Begin class HostMatcher
  *************************************************************/
 
-template <class Data, class Result>
-HostMatcher<Data, Result>::HostMatcher(const char *name, const char *filename)
-  : data_array(NULL), array_len(-1), num_el(-1), matcher_name(name), file_name(filename)
+template <class Data, class MatchResult>
+HostMatcher<Data, MatchResult>::HostMatcher(const char *name, const char *filename) : BaseMatcher<Data>(name, filename)
 {
   host_lookup = new HostLookup(name);
 }
 
-template <class Data, class Result> HostMatcher<Data, Result>::~HostMatcher()
+template <class Data, class MatchResult> HostMatcher<Data, MatchResult>::~HostMatcher()
 {
   delete host_lookup;
-  delete[] data_array;
 }
 
 //
-// template <class Data,class Result>
-// void HostMatcher<Data,Result>::Print()
+// template <class Data,class MatchResult>
+// void HostMatcher<Data,MatchResult>::Print()
 //
 //  Debugging Method
 //
-template <class Data, class Result>
+template <class Data, class MatchResult>
 void
-HostMatcher<Data, Result>::Print()
+HostMatcher<Data, MatchResult>::Print()
 {
   printf("\tHost/Domain Matcher with %d elements\n", num_el);
   host_lookup->Print(PrintFunc);
 }
 
 //
-// template <class Data,class Result>
-// void HostMatcher<Data,Result>::PrintFunc(void* opaque_data)
+// template <class Data,class MatchResult>
+// void HostMatcher<Data,MatchResult>::PrintFunc(void* opaque_data)
 //
 //  Debugging Method
 //
-template <class Data, class Result>
+template <class Data, class MatchResult>
 void
-HostMatcher<Data, Result>::PrintFunc(void *opaque_data)
+HostMatcher<Data, MatchResult>::PrintFunc(void *opaque_data)
 {
   Data *d = (Data *)opaque_data;
   d->Print();
 }
 
-// void HostMatcher<Data,Result>::AllocateSpace(int num_entries)
+// void HostMatcher<Data,MatchResult>::AllocateSpace(int num_entries)
 //
 //  Allocates the the HostLeaf and Data arrays
 //
-template <class Data, class Result>
+template <class Data, class MatchResult>
 void
-HostMatcher<Data, Result>::AllocateSpace(int num_entries)
+HostMatcher<Data, MatchResult>::AllocateSpace(int num_entries)
 {
   // Should not have been allocated before
   ink_assert(array_len == -1);
@@ -142,26 +139,25 @@ HostMatcher<Data, Result>::AllocateSpace(int num_entries)
   host_lookup->AllocateSpace(num_entries);
 
   data_array = new Data[num_entries];
-
-  array_len = num_entries;
-  num_el    = 0;
+  array_len  = num_entries;
+  num_el     = 0;
 }
 
-// void HostMatcher<Data,Result>::Match(RequestData* rdata, Result* result)
+// void HostMatcher<Data,MatchResult>::Match(RequestData* rdata, MatchResult* result)
 //
 //  Searches our tree and updates argresult for each element matching
 //    arg hostname
 //
-template <class Data, class Result>
+template <class Data, class MatchResult>
 void
-HostMatcher<Data, Result>::Match(RequestData *rdata, Result *result)
+HostMatcher<Data, MatchResult>::Match(RequestData *rdata, MatchResult *result)
 {
   void *opaque_ptr;
   Data *data_ptr;
   bool r;
 
   // Check to see if there is any work to do before makeing
-  //   the stirng copy
+  //   the string copy
   if (num_el <= 0) {
     return;
   }
@@ -171,7 +167,7 @@ HostMatcher<Data, Result>::Match(RequestData *rdata, Result *result)
   r = host_lookup->MatchFirst(rdata->get_host(), &s, &opaque_ptr);
 
   while (r == true) {
-    ink_assert(opaque_ptr != NULL);
+    ink_assert(opaque_ptr != nullptr);
     data_ptr = (Data *)opaque_ptr;
     data_ptr->UpdateMatch(result, rdata);
 
@@ -180,7 +176,7 @@ HostMatcher<Data, Result>::Match(RequestData *rdata, Result *result)
 }
 
 //
-// config_parse_error HostMatcher<Data,Result>::NewEntry(bool domain_record,
+// Result HostMatcher<Data,MatchResult>::NewEntry(bool domain_record,
 //          char* match_data, char* match_info, int line_num)
 //
 //   Creates a new host/domain record
@@ -189,12 +185,12 @@ HostMatcher<Data, Result>::Match(RequestData *rdata, Result *result)
 //   If not, returns a pointer to malloc allocated error string
 //     that the caller MUST DEALLOCATE
 //
-template <class Data, class Result>
-config_parse_error
-HostMatcher<Data, Result>::NewEntry(matcher_line *line_info)
+template <class Data, class MatchResult>
+Result
+HostMatcher<Data, MatchResult>::NewEntry(matcher_line *line_info)
 {
   Data *cur_d;
-  config_parse_error error = config_parse_error::ok();
+  Result error = Result::ok();
   char *match_data;
 
   // Make sure space has been allocated
@@ -208,18 +204,18 @@ HostMatcher<Data, Result>::NewEntry(matcher_line *line_info)
 
   // Make sure that the line_info is not bogus
   ink_assert(line_info->dest_entry < MATCHER_MAX_TOKENS);
-  ink_assert(match_data != NULL);
+  ink_assert(match_data != nullptr);
 
   // Remove our consumed label from the parsed line
-  line_info->line[0][line_info->dest_entry] = 0;
+  line_info->line[0][line_info->dest_entry] = nullptr;
   line_info->num_el--;
 
   // Fill in the parameter info
   cur_d = data_array + num_el;
   error = cur_d->Init(line_info);
-  if (error) {
+  if (error.failed()) {
     // There was a problem so undo the effects this function
-    memset(cur_d, 0, sizeof(Data));
+    new (cur_d) Data(); // reconstruct
   } else {
     // Fill in the matching info
     host_lookup->NewEntry(match_data, (line_info->type == MATCH_DOMAIN) ? true : false, cur_d);
@@ -234,44 +230,33 @@ HostMatcher<Data, Result>::NewEntry(matcher_line *line_info)
  *************************************************************/
 
 //
-// UrlMatcher<Data,Result>::UrlMatcher()
+// UrlMatcher<Data,MatchResult>::UrlMatcher()
 //
-template <class Data, class Result>
-UrlMatcher<Data, Result>::UrlMatcher(const char *name, const char *filename)
-  : url_ht(NULL),
-    url_str(NULL),
-    url_value(NULL),
-    data_array(NULL),
-    array_len(0),
-    num_el(-1),
-    matcher_name(name),
-    file_name(filename)
+template <class Data, class MatchResult>
+UrlMatcher<Data, MatchResult>::UrlMatcher(const char *name, const char *filename) : BaseMatcher<Data>(name, filename)
 {
-  url_ht = ink_hash_table_create(InkHashTableKeyType_String);
 }
 
 //
-// UrlMatcher<Data,Result>::~UrlMatcher()
+// UrlMatcher<Data,MatchResult>::~UrlMatcher()
 //
-template <class Data, class Result> UrlMatcher<Data, Result>::~UrlMatcher()
+template <class Data, class MatchResult> UrlMatcher<Data, MatchResult>::~UrlMatcher()
 {
-  ink_hash_table_destroy(url_ht);
   for (int i = 0; i < num_el; i++) {
     ats_free(url_str[i]);
   }
   delete[] url_str;
   delete[] url_value;
-  delete[] data_array;
 }
 
 //
-// void UrlMatcher<Data,Result>::Print()
+// void UrlMatcher<Data,MatchResult>::Print()
 //
 //   Debugging function
 //
-template <class Data, class Result>
+template <class Data, class MatchResult>
 void
-UrlMatcher<Data, Result>::Print()
+UrlMatcher<Data, MatchResult>::Print()
 {
   printf("\tUrl Matcher with %d elements\n", num_el);
   for (int i = 0; i < num_el; i++) {
@@ -281,11 +266,11 @@ UrlMatcher<Data, Result>::Print()
 }
 
 //
-// void UrlMatcher<Data,Result>::AllocateSpace(int num_entries)
+// void UrlMatcher<Data,MatchResult>::AllocateSpace(int num_entries)
 //
-template <class Data, class Result>
+template <class Data, class MatchResult>
 void
-UrlMatcher<Data, Result>::AllocateSpace(int num_entries)
+UrlMatcher<Data, MatchResult>::AllocateSpace(int num_entries)
 {
   // Should not have been allocated before
   ink_assert(array_len == -1);
@@ -299,16 +284,15 @@ UrlMatcher<Data, Result>::AllocateSpace(int num_entries)
 }
 
 //
-// config_parse_error UrlMatcher<Data,Result>::NewEntry(matcher_line* line_info)
+// Result UrlMatcher<Data,MatchResult>::NewEntry(matcher_line* line_info)
 //
-template <class Data, class Result>
-config_parse_error
-UrlMatcher<Data, Result>::NewEntry(matcher_line *line_info)
+template <class Data, class MatchResult>
+Result
+UrlMatcher<Data, MatchResult>::NewEntry(matcher_line *line_info)
 {
   Data *cur_d;
   char *pattern;
-  int *value;
-  config_parse_error error = config_parse_error::ok();
+  Result error = Result::ok();
 
   // Make sure space has been allocated
   ink_assert(num_el >= 0);
@@ -320,23 +304,23 @@ UrlMatcher<Data, Result>::NewEntry(matcher_line *line_info)
   pattern = line_info->line[1][line_info->dest_entry];
   // Make sure that the line_info is not bogus
   ink_assert(line_info->dest_entry < MATCHER_MAX_TOKENS);
-  ink_assert(pattern != NULL);
+  ink_assert(pattern != nullptr);
 
-  if (ink_hash_table_lookup(url_ht, pattern, (void **)&value)) {
-    return config_parse_error("%s url expression error (have exist) at line %d position", matcher_name, line_info->line_num);
+  if (url_ht.find(pattern) != url_ht.end()) {
+    return Result::failure("%s url expression error (have exist) at line %d position", matcher_name, line_info->line_num);
   }
 
   // Remove our consumed label from the parsed line
-  line_info->line[0][line_info->dest_entry] = 0;
+  line_info->line[0][line_info->dest_entry] = nullptr;
   line_info->num_el--;
 
   // Fill in the parameter info
   cur_d = data_array + num_el;
   error = cur_d->Init(line_info);
-  if (error) {
+  if (error.failed()) {
     url_str[num_el]   = ats_strdup(pattern);
     url_value[num_el] = num_el;
-    ink_hash_table_insert(url_ht, url_str[num_el], (void *)&url_value[num_el]);
+    url_ht.emplace(url_str[num_el], url_value[num_el]);
     num_el++;
   }
 
@@ -344,17 +328,16 @@ UrlMatcher<Data, Result>::NewEntry(matcher_line *line_info)
 }
 
 //
-// void UrlMatcher<Data,Result>::Match(RD* rdata, Result* result)
+// void UrlMatcher<Data,MatchResult>::Match(RD* rdata, MatchResult* result)
 //
 //   Coduncts a linear search through the regex array and
 //     updates arg result for each regex that matches arg URL
 //
-template <class Data, class Result>
+template <class Data, class MatchResult>
 void
-UrlMatcher<Data, Result>::Match(RequestData *rdata, Result *result)
+UrlMatcher<Data, MatchResult>::Match(RequestData *rdata, MatchResult *result)
 {
   char *url_str;
-  int *value;
 
   // Check to see there is any work to before we copy the
   //   URL
@@ -366,31 +349,30 @@ UrlMatcher<Data, Result>::Match(RequestData *rdata, Result *result)
 
   // Can't do a regex match with a NULL string so
   //  use an empty one instead
-  if (url_str == NULL) {
+  if (url_str == nullptr) {
     url_str = ats_strdup("");
   }
 
-  if (ink_hash_table_lookup(url_ht, url_str, (void **)&value)) {
-    Debug("matcher", "%s Matched %s with url at line %d", matcher_name, url_str, data_array[*value].line_num);
-    data_array[*value].UpdateMatch(result, rdata);
+  if (auto it = url_ht.find(url_str); it != url_ht.end()) {
+    Debug("matcher", "%s Matched %s with url at line %d", matcher_name, url_str, data_array[it->second].line_num);
+    data_array[it->second].UpdateMatch(result, rdata);
   }
 
   ats_free(url_str);
 }
 
 //
-// RegexMatcher<Data,Result>::RegexMatcher()
+// RegexMatcher<Data,MatchResult>::RegexMatcher()
 //
-template <class Data, class Result>
-RegexMatcher<Data, Result>::RegexMatcher(const char *name, const char *filename)
-  : re_array(NULL), re_str(NULL), data_array(NULL), array_len(-1), num_el(-1), matcher_name(name), file_name(filename)
+template <class Data, class MatchResult>
+RegexMatcher<Data, MatchResult>::RegexMatcher(const char *name, const char *filename) : BaseMatcher<Data>(name, filename)
 {
 }
 
 //
-// RegexMatcher<Data,Result>::~RegexMatcher()
+// RegexMatcher<Data,MatchResult>::~RegexMatcher()
 //
-template <class Data, class Result> RegexMatcher<Data, Result>::~RegexMatcher()
+template <class Data, class MatchResult> RegexMatcher<Data, MatchResult>::~RegexMatcher()
 {
   for (int i = 0; i < num_el; i++) {
     pcre_free(re_array[i]);
@@ -398,17 +380,16 @@ template <class Data, class Result> RegexMatcher<Data, Result>::~RegexMatcher()
   }
   delete[] re_str;
   ats_free(re_array);
-  delete[] data_array;
 }
 
 //
-// void RegexMatcher<Data,Result>::Print()
+// void RegexMatcher<Data,MatchResult>::Print()
 //
 //   Debugging function
 //
-template <class Data, class Result>
+template <class Data, class MatchResult>
 void
-RegexMatcher<Data, Result>::Print()
+RegexMatcher<Data, MatchResult>::Print()
 {
   printf("\tRegex Matcher with %d elements\n", num_el);
   for (int i = 0; i < num_el; i++) {
@@ -418,16 +399,16 @@ RegexMatcher<Data, Result>::Print()
 }
 
 //
-// void RegexMatcher<Data,Result>::AllocateSpace(int num_entries)
+// void RegexMatcher<Data,MatchResult>::AllocateSpace(int num_entries)
 //
-template <class Data, class Result>
+template <class Data, class MatchResult>
 void
-RegexMatcher<Data, Result>::AllocateSpace(int num_entries)
+RegexMatcher<Data, MatchResult>::AllocateSpace(int num_entries)
 {
   // Should not have been allocated before
   ink_assert(array_len == -1);
 
-  re_array = (pcre **)ats_malloc(sizeof(pcre *) * num_entries);
+  re_array = static_cast<pcre **>(ats_malloc(sizeof(pcre *) * num_entries));
   memset(re_array, 0, sizeof(pcre *) * num_entries);
 
   data_array = new Data[num_entries];
@@ -440,17 +421,17 @@ RegexMatcher<Data, Result>::AllocateSpace(int num_entries)
 }
 
 //
-// config_parse_error RegexMatcher<Data,Result>::NewEntry(matcher_line* line_info)
+// Result RegexMatcher<Data,MatchResult>::NewEntry(matcher_line* line_info)
 //
-template <class Data, class Result>
-config_parse_error
-RegexMatcher<Data, Result>::NewEntry(matcher_line *line_info)
+template <class Data, class MatchResult>
+Result
+RegexMatcher<Data, MatchResult>::NewEntry(matcher_line *line_info)
 {
   Data *cur_d;
   char *pattern;
   const char *errptr;
   int erroffset;
-  config_parse_error error = config_parse_error::ok();
+  Result error = Result::ok();
 
   // Make sure space has been allocated
   ink_assert(num_el >= 0);
@@ -462,30 +443,30 @@ RegexMatcher<Data, Result>::NewEntry(matcher_line *line_info)
   pattern = line_info->line[1][line_info->dest_entry];
   // Make sure that the line_info is not bogus
   ink_assert(line_info->dest_entry < MATCHER_MAX_TOKENS);
-  ink_assert(pattern != NULL);
+  ink_assert(pattern != nullptr);
 
   // Create the compiled regular expression
-  re_array[num_el] = pcre_compile(pattern, 0, &errptr, &erroffset, NULL);
+  re_array[num_el] = pcre_compile(pattern, 0, &errptr, &erroffset, nullptr);
   if (!re_array[num_el]) {
-    return config_parse_error("%s regular expression error at line %d position %d : %s", matcher_name, line_info->line_num,
-                              erroffset, errptr);
+    return Result::failure("%s regular expression error at line %d position %d : %s", matcher_name, line_info->line_num, erroffset,
+                           errptr);
   }
   re_str[num_el] = ats_strdup(pattern);
 
   // Remove our consumed label from the parsed line
-  line_info->line[0][line_info->dest_entry] = 0;
+  line_info->line[0][line_info->dest_entry] = nullptr;
   line_info->num_el--;
 
   // Fill in the parameter info
   cur_d = data_array + num_el;
   error = cur_d->Init(line_info);
 
-  if (error) {
+  if (error.failed()) {
     // There was a problem so undo the effects this function
     ats_free(re_str[num_el]);
-    re_str[num_el] = NULL;
+    re_str[num_el] = nullptr;
     pcre_free(re_array[num_el]);
-    re_array[num_el] = NULL;
+    re_array[num_el] = nullptr;
   } else {
     num_el++;
   }
@@ -494,14 +475,14 @@ RegexMatcher<Data, Result>::NewEntry(matcher_line *line_info)
 }
 
 //
-// void RegexMatcher<Data,Result>::Match(RequestData* rdata, Result* result)
+// void RegexMatcher<Data,MatchResult>::Match(RequestData* rdata, MatchResult* result)
 //
 //   Coduncts a linear search through the regex array and
 //     updates arg result for each regex that matches arg URL
 //
-template <class Data, class Result>
+template <class Data, class MatchResult>
 void
-RegexMatcher<Data, Result>::Match(RequestData *rdata, Result *result)
+RegexMatcher<Data, MatchResult>::Match(RequestData *rdata, MatchResult *result)
 {
   char *url_str;
   int r;
@@ -516,16 +497,16 @@ RegexMatcher<Data, Result>::Match(RequestData *rdata, Result *result)
 
   // Can't do a regex match with a NULL string so
   //  use an empty one instead
-  if (url_str == NULL) {
+  if (url_str == nullptr) {
     url_str = ats_strdup("");
   }
+
   // INKqa12980
   // The function unescapifyStr() is already called in
   // HttpRequestData::get_string(); therefore, no need to call again here.
-  // unescapifyStr(url_str);
 
   for (int i = 0; i < num_el; i++) {
-    r = pcre_exec(re_array[i], NULL, url_str, strlen(url_str), 0, 0, NULL, 0);
+    r = pcre_exec(re_array[i], nullptr, url_str, strlen(url_str), 0, 0, nullptr, 0);
     if (r > -1) {
       Debug("matcher", "%s Matched %s with regex at line %d", matcher_name, url_str, data_array[i].line_num);
       data_array[i].UpdateMatch(result, rdata);
@@ -538,23 +519,23 @@ RegexMatcher<Data, Result>::Match(RequestData *rdata, Result *result)
 }
 
 //
-// HostRegexMatcher<Data,Result>::HostRegexMatcher()
+// HostRegexMatcher<Data,MatchResult>::HostRegexMatcher()
 //
-template <class Data, class Result>
-HostRegexMatcher<Data, Result>::HostRegexMatcher(const char *name, const char *filename)
-  : RegexMatcher<Data, Result>(name, filename)
+template <class Data, class MatchResult>
+HostRegexMatcher<Data, MatchResult>::HostRegexMatcher(const char *name, const char *filename)
+  : RegexMatcher<Data, MatchResult>(name, filename)
 {
 }
 
 //
-// void HostRegexMatcher<Data,Result>::Match(RequestData* rdata, Result* result)
+// void HostRegexMatcher<Data,MatchResult>::Match(RequestData* rdata, MatchResult* result)
 //
 //   Conducts a linear search through the regex array and
 //     updates arg result for each regex that matches arg host_regex
 //
-template <class Data, class Result>
+template <class Data, class MatchResult>
 void
-HostRegexMatcher<Data, Result>::Match(RequestData *rdata, Result *result)
+HostRegexMatcher<Data, MatchResult>::Match(RequestData *rdata, MatchResult *result)
 {
   const char *url_str;
   int r;
@@ -569,11 +550,11 @@ HostRegexMatcher<Data, Result>::Match(RequestData *rdata, Result *result)
 
   // Can't do a regex match with a NULL string so
   //  use an empty one instead
-  if (url_str == NULL) {
+  if (url_str == nullptr) {
     url_str = "";
   }
   for (int i = 0; i < this->num_el; i++) {
-    r = pcre_exec(this->re_array[i], NULL, url_str, strlen(url_str), 0, 0, NULL, 0);
+    r = pcre_exec(this->re_array[i], nullptr, url_str, strlen(url_str), 0, 0, nullptr, 0);
     if (r != -1) {
       Debug("matcher", "%s Matched %s with regex at line %d", const_cast<char *>(this->matcher_name), url_str,
             this->data_array[i].line_num);
@@ -586,28 +567,19 @@ HostRegexMatcher<Data, Result>::Match(RequestData *rdata, Result *result)
 }
 
 //
-// IpMatcher<Data,Result>::IpMatcher()
+// IpMatcher<Data,MatchResult>::IpMatcher()
 //
-template <class Data, class Result>
-IpMatcher<Data, Result>::IpMatcher(const char *name, const char *filename)
-  : data_array(NULL), array_len(-1), num_el(-1), matcher_name(name), file_name(filename)
+template <class Data, class MatchResult>
+IpMatcher<Data, MatchResult>::IpMatcher(const char *name, const char *filename) : BaseMatcher<Data>(name, filename)
 {
 }
 
 //
-// IpMatcher<Data,Result>::~IpMatcher()
+// void IpMatcher<Data,MatchResult>::AllocateSpace(int num_entries)
 //
-template <class Data, class Result> IpMatcher<Data, Result>::~IpMatcher()
-{
-  delete[] data_array;
-}
-
-//
-// void IpMatcher<Data,Result>::AllocateSpace(int num_entries)
-//
-template <class Data, class Result>
+template <class Data, class MatchResult>
 void
-IpMatcher<Data, Result>::AllocateSpace(int num_entries)
+IpMatcher<Data, MatchResult>::AllocateSpace(int num_entries)
 {
   // Should not have been allocated before
   ink_assert(array_len == -1);
@@ -619,7 +591,7 @@ IpMatcher<Data, Result>::AllocateSpace(int num_entries)
 }
 
 //
-// config_parse_error IpMatcher<Data,Result>::NewEntry(matcher_line* line_info)
+// Result IpMatcher<Data,MatchResult>::NewEntry(matcher_line* line_info)
 //
 //    Inserts a range the ip lookup table.
 //        Creates new table levels as needed
@@ -628,15 +600,15 @@ IpMatcher<Data, Result>::AllocateSpace(int num_entries)
 //     allocated error string which the CALLEE is responsible
 //     for deallocating
 //
-template <class Data, class Result>
-config_parse_error
-IpMatcher<Data, Result>::NewEntry(matcher_line *line_info)
+template <class Data, class MatchResult>
+Result
+IpMatcher<Data, MatchResult>::NewEntry(matcher_line *line_info)
 {
   Data *cur_d;
   const char *errptr;
   char *match_data;
   IpEndpoint addr1, addr2;
-  config_parse_error error = config_parse_error::ok();
+  Result error = Result::ok();
 
   // Make sure space has been allocated
   ink_assert(num_el >= 0);
@@ -649,22 +621,22 @@ IpMatcher<Data, Result>::NewEntry(matcher_line *line_info)
 
   // Make sure that the line_info is not bogus
   ink_assert(line_info->dest_entry < MATCHER_MAX_TOKENS);
-  ink_assert(match_data != NULL);
+  ink_assert(match_data != nullptr);
 
   // Extract the IP range
   errptr = ExtractIpRange(match_data, &addr1.sa, &addr2.sa);
-  if (errptr != NULL) {
-    return config_parse_error("%s %s at %s line %d", matcher_name, errptr, file_name, line_info->line_num);
+  if (errptr != nullptr) {
+    return Result::failure("%s %s at %s line %d", matcher_name, errptr, file_name, line_info->line_num);
   }
 
   // Remove our consumed label from the parsed line
-  line_info->line[0][line_info->dest_entry] = NULL;
+  line_info->line[0][line_info->dest_entry] = nullptr;
   line_info->num_el--;
 
   // Fill in the parameter info
   cur_d = data_array + num_el;
   error = cur_d->Init(line_info);
-  if (!error) {
+  if (!error.failed()) {
     ip_map.mark(&addr1.sa, &addr2.sa, cur_d);
     ++num_el;
   }
@@ -673,40 +645,40 @@ IpMatcher<Data, Result>::NewEntry(matcher_line *line_info)
 }
 
 //
-// void IpMatcherData,Result>::Match(in_addr_t addr, RequestData* rdata, Result* result)
+// void IpMatcherData,MatchResult>::Match(in_addr_t addr, RequestData* rdata, MatchResult* result)
 //
-template <class Data, class Result>
+template <class Data, class MatchResult>
 void
-IpMatcher<Data, Result>::Match(sockaddr const *addr, RequestData *rdata, Result *result)
+IpMatcher<Data, MatchResult>::Match(sockaddr const *addr, RequestData *rdata, MatchResult *result)
 {
   void *raw;
   if (ip_map.contains(addr, &raw)) {
     Data *cur = static_cast<Data *>(raw);
-    ink_assert(cur != 0);
+    ink_assert(cur != nullptr);
     cur->UpdateMatch(result, rdata);
   }
 }
 
-template <class Data, class Result>
+template <class Data, class MatchResult>
 void
-IpMatcher<Data, Result>::Print()
+IpMatcher<Data, MatchResult>::Print()
 {
-  printf("\tIp Matcher with %d elements, %zu ranges.\n", num_el, ip_map.getCount());
-  for (IpMap::iterator spot(ip_map.begin()), limit(ip_map.end()); spot != limit; ++spot) {
+  printf("\tIp Matcher with %d elements, %zu ranges.\n", num_el, ip_map.count());
+  for (auto &spot : ip_map) {
     char b1[INET6_ADDRSTRLEN], b2[INET6_ADDRSTRLEN];
-    printf("\tRange %s - %s ", ats_ip_ntop(spot->min(), b1, sizeof b1), ats_ip_ntop(spot->max(), b2, sizeof b2));
-    static_cast<Data *>(spot->data())->Print();
+    printf("\tRange %s - %s ", ats_ip_ntop(spot.min(), b1, sizeof b1), ats_ip_ntop(spot.max(), b2, sizeof b2));
+    static_cast<Data *>(spot.data())->Print();
   }
 }
 
-template <class Data, class Result>
-ControlMatcher<Data, Result>::ControlMatcher(const char *file_var, const char *name, const matcher_tags *tags, int flags_in)
+template <class Data, class MatchResult>
+ControlMatcher<Data, MatchResult>::ControlMatcher(const char *file_var, const char *name, const matcher_tags *tags, int flags_in)
 {
   flags = flags_in;
   ink_assert(flags & (ALLOW_HOST_TABLE | ALLOW_REGEX_TABLE | ALLOW_URL_TABLE | ALLOW_IP_TABLE));
 
   config_tags = tags;
-  ink_assert(config_tags != NULL);
+  ink_assert(config_tags != nullptr);
 
   matcher_name        = name;
   config_file_path[0] = '\0';
@@ -718,11 +690,11 @@ ControlMatcher<Data, Result>::ControlMatcher(const char *file_var, const char *n
     ink_strlcpy(config_file_path, config_path, sizeof(config_file_path));
   }
 
-  reMatch   = NULL;
-  urlMatch  = NULL;
-  hostMatch = NULL;
-  ipMatch   = NULL;
-  hrMatch   = NULL;
+  reMatch   = nullptr;
+  urlMatch  = nullptr;
+  hostMatch = nullptr;
+  ipMatch   = nullptr;
+  hrMatch   = nullptr;
 
   if (!(flags & DONT_BUILD_TABLE)) {
     m_numEntries = this->BuildTable();
@@ -731,7 +703,7 @@ ControlMatcher<Data, Result>::ControlMatcher(const char *file_var, const char *n
   }
 }
 
-template <class Data, class Result> ControlMatcher<Data, Result>::~ControlMatcher()
+template <class Data, class MatchResult> ControlMatcher<Data, MatchResult>::~ControlMatcher()
 {
   delete reMatch;
   delete urlMatch;
@@ -740,74 +712,74 @@ template <class Data, class Result> ControlMatcher<Data, Result>::~ControlMatche
   delete hrMatch;
 }
 
-// void ControlMatcher<Data, Result>::Print()
+// void ControlMatcher<Data, MatchResult>::Print()
 //
 //   Debugging method
 //
-template <class Data, class Result>
+template <class Data, class MatchResult>
 void
-ControlMatcher<Data, Result>::Print()
+ControlMatcher<Data, MatchResult>::Print()
 {
   printf("Control Matcher Table: %s\n", matcher_name);
-  if (hostMatch != NULL) {
+  if (hostMatch != nullptr) {
     hostMatch->Print();
   }
-  if (reMatch != NULL) {
+  if (reMatch != nullptr) {
     reMatch->Print();
   }
-  if (urlMatch != NULL) {
+  if (urlMatch != nullptr) {
     urlMatch->Print();
   }
-  if (ipMatch != NULL) {
+  if (ipMatch != nullptr) {
     ipMatch->Print();
   }
-  if (hrMatch != NULL) {
+  if (hrMatch != nullptr) {
     hrMatch->Print();
   }
 }
 
-// void ControlMatcher<Data, Result>::Match(RequestData* rdata
-//                                          Result* result)
+// void ControlMatcher<Data, MatchResult>::Match(RequestData* rdata
+//                                          MatchResult* result)
 //
-//   Queries each table for the Result*
+//   Queries each table for the MatchResult*
 //
-template <class Data, class Result>
+template <class Data, class MatchResult>
 void
-ControlMatcher<Data, Result>::Match(RequestData *rdata, Result *result)
+ControlMatcher<Data, MatchResult>::Match(RequestData *rdata, MatchResult *result)
 {
-  if (hostMatch != NULL) {
+  if (hostMatch != nullptr) {
     hostMatch->Match(rdata, result);
   }
-  if (reMatch != NULL) {
+  if (reMatch != nullptr) {
     reMatch->Match(rdata, result);
   }
-  if (urlMatch != NULL) {
+  if (urlMatch != nullptr) {
     urlMatch->Match(rdata, result);
   }
-  if (ipMatch != NULL) {
+  if (ipMatch != nullptr) {
     ipMatch->Match(rdata->get_ip(), rdata, result);
   }
-  if (hrMatch != NULL) {
+  if (hrMatch != nullptr) {
     hrMatch->Match(rdata, result);
   }
 }
 
-// int ControlMatcher::BuildTable() {
+// int ControlMatcher::BuildTable()
 //
 //    Reads the cache.config file and build the records array
 //      from it
 //
-template <class Data, class Result>
+template <class Data, class MatchResult>
 int
-ControlMatcher<Data, Result>::BuildTableFromString(char *file_buf)
+ControlMatcher<Data, MatchResult>::BuildTableFromString(char *file_buf)
 {
   // Table build locals
   Tokenizer bufTok("\n");
   tok_iter_state i_state;
   const char *tmp;
-  matcher_line *first = NULL;
+  matcher_line *first = nullptr;
   matcher_line *current;
-  matcher_line *last = NULL;
+  matcher_line *last = nullptr;
   int line_num       = 0;
   int second_pass    = 0;
   int numEntries     = 0;
@@ -824,9 +796,10 @@ ControlMatcher<Data, Result>::BuildTableFromString(char *file_buf)
     // We have an empty file
     return 0;
   }
+
   // First get the number of entries
   tmp = bufTok.iterFirst(&i_state);
-  while (tmp != NULL) {
+  while (tmp != nullptr) {
     line_num++;
 
     // skip all blank spaces at beginning of line
@@ -837,13 +810,14 @@ ControlMatcher<Data, Result>::BuildTableFromString(char *file_buf)
     if (*tmp != '#' && *tmp != '\0') {
       const char *errptr;
 
-      current = (matcher_line *)ats_malloc(sizeof(matcher_line));
-      errptr  = parseConfigLine((char *)tmp, current, config_tags);
+      current = static_cast<matcher_line *>(ats_malloc(sizeof(matcher_line)));
+      errptr  = parseConfigLine(const_cast<char *>(tmp), current, config_tags);
 
-      if (errptr != NULL) {
+      if (errptr != nullptr) {
         if (config_tags != &socks_server_tags) {
-          config_parse_error error("%s discarding %s entry at line %d : %s", matcher_name, config_file_path, line_num, errptr);
-          SignalError(error.get(), alarmAlready);
+          Result error =
+            Result::failure("%s discarding %s entry at line %d : %s", matcher_name, config_file_path, line_num, errptr);
+          SignalError(error.message(), alarmAlready);
         }
         ats_free(current);
       } else {
@@ -874,8 +848,8 @@ ControlMatcher<Data, Result>::BuildTableFromString(char *file_buf)
           ink_assert(0);
         }
 
-        if (first == NULL) {
-          ink_assert(last == NULL);
+        if (first == nullptr) {
+          ink_assert(last == nullptr);
           first = last = current;
         } else {
           last->next = current;
@@ -893,33 +867,33 @@ ControlMatcher<Data, Result>::BuildTableFromString(char *file_buf)
   }
   // Now allocate space for the record pointers
   if ((flags & ALLOW_REGEX_TABLE) && regex > 0) {
-    reMatch = new RegexMatcher<Data, Result>(matcher_name, config_file_path);
+    reMatch = new RegexMatcher<Data, MatchResult>(matcher_name, config_file_path);
     reMatch->AllocateSpace(regex);
   }
 
   if ((flags & ALLOW_URL_TABLE) && url > 0) {
-    urlMatch = new UrlMatcher<Data, Result>(matcher_name, config_file_path);
+    urlMatch = new UrlMatcher<Data, MatchResult>(matcher_name, config_file_path);
     urlMatch->AllocateSpace(url);
   }
 
   if ((flags & ALLOW_HOST_TABLE) && hostDomain > 0) {
-    hostMatch = new HostMatcher<Data, Result>(matcher_name, config_file_path);
+    hostMatch = new HostMatcher<Data, MatchResult>(matcher_name, config_file_path);
     hostMatch->AllocateSpace(hostDomain);
   }
 
   if ((flags & ALLOW_IP_TABLE) && ip > 0) {
-    ipMatch = new IpMatcher<Data, Result>(matcher_name, config_file_path);
+    ipMatch = new IpMatcher<Data, MatchResult>(matcher_name, config_file_path);
     ipMatch->AllocateSpace(ip);
   }
 
   if ((flags & ALLOW_HOST_REGEX_TABLE) && hostregex > 0) {
-    hrMatch = new HostRegexMatcher<Data, Result>(matcher_name, config_file_path);
+    hrMatch = new HostRegexMatcher<Data, MatchResult>(matcher_name, config_file_path);
     hrMatch->AllocateSpace(hostregex);
   }
   // Traverse the list and build the records table
   current = first;
-  while (current != NULL) {
-    config_parse_error error = config_parse_error::ok();
+  while (current != nullptr) {
+    Result error = Result::ok();
 
     second_pass++;
     if ((flags & ALLOW_HOST_TABLE) && current->type == MATCH_DOMAIN) {
@@ -935,14 +909,13 @@ ControlMatcher<Data, Result>::BuildTableFromString(char *file_buf)
     } else if ((flags & ALLOW_HOST_REGEX_TABLE) && current->type == MATCH_HOST_REGEX) {
       error = hrMatch->NewEntry(current);
     } else {
-      error = config_parse_error("%s discarding %s entry with unknown type at line %d", matcher_name, config_file_path,
-                                 current->line_num);
+      error =
+        Result::failure("%s discarding %s entry with unknown type at line %d", matcher_name, config_file_path, current->line_num);
     }
 
-    // Check to see if there was an error in creating
-    //   the NewEntry
-    if (error) {
-      SignalError(error.get(), alarmAlready);
+    // Check to see if there was an error in creating the NewEntry
+    if (error.failed()) {
+      SignalError(error.message(), alarmAlready);
     }
 
     // Deallocate the parsing structure
@@ -959,52 +932,29 @@ ControlMatcher<Data, Result>::BuildTableFromString(char *file_buf)
   return numEntries;
 }
 
-template <class Data, class Result>
+template <class Data, class MatchResult>
 int
-ControlMatcher<Data, Result>::BuildTable()
+ControlMatcher<Data, MatchResult>::BuildTable()
 {
-  // File I/O Locals
-  char *file_buf;
-  int ret;
-
-  file_buf = readIntoBuffer(config_file_path, matcher_name, NULL);
-
-  if (file_buf == NULL) {
-    return 1;
+  std::error_code ec;
+  std::string content{ts::file::load(ts::file::path{config_file_path}, ec)};
+  if (ec) {
+    switch (ec.value()) {
+    case ENOENT:
+      Warning("ControlMatcher - Cannot open config file: %s - %s", config_file_path, strerror(ec.value()));
+      break;
+    default:
+      Error("ControlMatcher - %s failed to load: %s", config_file_path, strerror(ec.value()));
+      return 1;
+    }
   }
 
-  ret = BuildTableFromString(file_buf);
-  ats_free(file_buf);
-  return ret;
+  return BuildTableFromString(content.data());
 }
 
 /****************************************************************
  *    TEMPLATE INSTANTIATIONS GO HERE
  *
- *  We have to explicitly instantiate the templates so that
- *   everything works on with dec ccx, sun CC, and g++
- *
- *  Details on the different comipilers:
- *
- *  dec ccx: Does not seem to instantiate anything automatically
- *         so it needs all templates manually instantiated
- *
- *  sun CC: Automatic instantiation works but since we make
- *         use of the templates in other files, instantiation
- *         only occurs when those files are compiled, breaking
- *         the dependency system.  Explict instantiation
- *         in this file causes the templates to be reinstantiated
- *         when this file changes.
- *
- *         Also, does not give error messages about template
- *           compliation problems.  Requires the -verbose=template
- *           flage to error messages
- *
- *  g++: Requires instantiation to occur in the same file as the
- *         the implementation.  Instantiating ControlMatcher
- *         automatically instatiatiates the other templates since
- *         ControlMatcher makes use of them
- *
  ****************************************************************/
 
 template class ControlMatcher<ParentRecord, ParentResult>;
@@ -1026,10 +976,3 @@ template class HostMatcher<CacheControlRecord, CacheControlResult>;
 template class RegexMatcher<CacheControlRecord, CacheControlResult>;
 template class UrlMatcher<CacheControlRecord, CacheControlResult>;
 template class IpMatcher<CacheControlRecord, CacheControlResult>;
-
-template class ControlMatcher<CongestionControlRecord, CongestionControlRule>;
-template class HostMatcher<CongestionControlRecord, CongestionControlRule>;
-template class HostRegexMatcher<CongestionControlRecord, CongestionControlRule>;
-template class RegexMatcher<CongestionControlRecord, CongestionControlRule>;
-template class UrlMatcher<CongestionControlRecord, CongestionControlRule>;
-template class IpMatcher<CongestionControlRecord, CongestionControlRule>;
