@@ -84,21 +84,22 @@
 //     ControlMatcher.cc
 //
 
-#ifndef _CONTROL_MATCHER_H_
-#define _CONTROL_MATCHER_H_
+#pragma once
 
-#include "ts/DynArray.h"
-#include "ts/ink_hash_table.h"
-#include "ts/IpMap.h"
-#include "ts/MatcherUtils.h"
+#include "tscore/IpMap.h"
+#include "tscore/Result.h"
+#include "tscore/MatcherUtils.h"
 
-#include "ts/ink_apidefs.h"
-#include "ts/ink_defs.h"
+#include "tscore/ink_apidefs.h"
+#include "tscore/ink_defs.h"
 #include "HTTP.h"
-#include "ts/Regex.h"
+#include "tscore/Regex.h"
+#include "URL.h"
+
+#include <unordered_map>
 
 #ifdef HAVE_CTYPE_H
-#include <ctype.h>
+#include <cctype>
 #endif
 
 #define SignalError(_buf, _already)                         \
@@ -131,118 +132,130 @@ public:
 class HttpRequestData : public RequestData
 {
 public:
-  inkcoreapi char *get_string();
-  inkcoreapi const char *get_host();
-  inkcoreapi sockaddr const *get_ip();
-  inkcoreapi sockaddr const *get_client_ip();
+  inkcoreapi char *get_string() override;
+  inkcoreapi const char *get_host() override;
+  inkcoreapi sockaddr const *get_ip() override;
+  inkcoreapi sockaddr const *get_client_ip() override;
+
+  HttpRequestData()
 
-  HttpRequestData() : hdr(NULL), hostname_str(NULL), api_info(NULL), xact_start(0), incoming_port(0), tag(NULL), internal_txn(false)
   {
     ink_zero(src_ip);
     ink_zero(dest_ip);
   }
 
-  HTTPHdr *hdr;
-  char *hostname_str;
-  HttpApiInfo *api_info;
-  time_t xact_start;
+  HTTPHdr *hdr          = nullptr;
+  char *hostname_str    = nullptr;
+  HttpApiInfo *api_info = nullptr;
+  time_t xact_start     = 0;
   IpEndpoint src_ip;
   IpEndpoint dest_ip;
-  uint16_t incoming_port;
-  char *tag;
-  bool internal_txn;
+  uint16_t incoming_port                = 0;
+  char *tag                             = nullptr;
+  bool internal_txn                     = false;
+  URL **cache_info_lookup_url           = nullptr;
+  URL **cache_info_parent_selection_url = nullptr;
 };
 
-template <class Data, class Result> class UrlMatcher
+// Mixin class for shared info across all templates. This just wraps the
+// shared members such that we don't have to duplicate all these initialixers
+// etc. If someone wants to rewrite all this code to use setters and getters,
+// by all means, please do so. The plumbing is in place :).
+template <class Data> class BaseMatcher
 {
+public:
+  BaseMatcher(const char *name, const char *filename) : matcher_name(name), file_name(filename) {}
+
+  ~BaseMatcher() { delete[] data_array; }
+
+protected:
+  int num_el               = -1;        // number of elements in the table
+  const char *matcher_name = "unknown"; // Used for Debug/Warning/Error messages
+  const char *file_name    = nullptr;   // Used for Debug/Warning/Error messages
+  Data *data_array         = nullptr;   // Array with the Data elements
+  int array_len            = -1;        // length of the arrays (all three are the same length)
+};
+
+template <class Data, class MatchResult> class UrlMatcher : protected BaseMatcher<Data>
+{
+  typedef BaseMatcher<Data> super;
+
 public:
   UrlMatcher(const char *name, const char *filename);
   ~UrlMatcher();
-  void Match(RequestData *rdata, Result *result);
+  void Match(RequestData *rdata, MatchResult *result);
   void AllocateSpace(int num_entries);
-  config_parse_error NewEntry(matcher_line *line_info);
+  Result NewEntry(matcher_line *line_info);
   void Print();
 
-  int
-  getNumElements()
-  {
-    return num_el;
-  }
-  Data *
-  getDataArray()
-  {
-    return data_array;
-  }
+  using super::num_el;
+  using super::matcher_name;
+  using super::file_name;
+  using super::data_array;
+  using super::array_len;
 
-protected:
-  InkHashTable *url_ht;
-  char **url_str;           // array of url strings
-  int *url_value;           // array of posion of url strings
-  Data *data_array;         // data array.  Corresponds to re_array
-  int array_len;            // length of the arrays (all three are the same length)
-  int num_el;               // number of elements in the table
-  const char *matcher_name; // Used for Debug/Warning/Error messages
-  const char *file_name;    // Used for Debug/Warning/Error messages
+private:
+  std::unordered_map<std::string, int> url_ht;
+  char **url_str = nullptr; // array of url strings
+  int *url_value = nullptr; // array of posion of url strings
 };
 
-template <class Data, class Result> class RegexMatcher
+template <class Data, class MatchResult> class RegexMatcher : protected BaseMatcher<Data>
 {
+  typedef BaseMatcher<Data> super;
+
 public:
   RegexMatcher(const char *name, const char *filename);
   ~RegexMatcher();
-  void Match(RequestData *rdata, Result *result);
+  void Match(RequestData *rdata, MatchResult *result);
   void AllocateSpace(int num_entries);
-  config_parse_error NewEntry(matcher_line *line_info);
+  Result NewEntry(matcher_line *line_info);
   void Print();
 
-  int
-  getNumElements()
-  {
-    return num_el;
-  }
-  Data *
-  getDataArray()
-  {
-    return data_array;
-  }
+  using super::num_el;
+  using super::matcher_name;
+  using super::file_name;
+  using super::data_array;
+  using super::array_len;
 
 protected:
-  pcre **re_array;          // array of compiled regexs
-  char **re_str;            // array of uncompiled regex strings
-  Data *data_array;         // data array.  Corresponds to re_array
-  int array_len;            // length of the arrays (all three are the same length)
-  int num_el;               // number of elements in the table
-  const char *matcher_name; // Used for Debug/Warning/Error messages
-  const char *file_name;    // Used for Debug/Warning/Error messages
+  pcre **re_array = nullptr; // array of compiled regexs
+  char **re_str   = nullptr; // array of uncompiled regex strings
 };
 
-template <class Data, class Result> class HostRegexMatcher : public RegexMatcher<Data, Result>
+template <class Data, class MatchResult> class HostRegexMatcher : public RegexMatcher<Data, MatchResult>
 {
+  typedef BaseMatcher<Data> super;
+
 public:
   HostRegexMatcher(const char *name, const char *filename);
-  void Match(RequestData *rdata, Result *result);
+  void Match(RequestData *rdata, MatchResult *result);
+
+  using super::num_el;
+  using super::matcher_name;
+  using super::file_name;
+  using super::data_array;
+  using super::array_len;
 };
 
-template <class Data, class Result> class HostMatcher
+template <class Data, class MatchResult> class HostMatcher : protected BaseMatcher<Data>
 {
+  typedef BaseMatcher<Data> super;
+
 public:
   HostMatcher(const char *name, const char *filename);
   ~HostMatcher();
-  void Match(RequestData *rdata, Result *result);
+  void Match(RequestData *rdata, MatchResult *result);
   void AllocateSpace(int num_entries);
-  config_parse_error NewEntry(matcher_line *line_info);
+  Result NewEntry(matcher_line *line_info);
   void Print();
 
-  int
-  getNumElements()
-  {
-    return num_el;
-  }
-  Data *
-  getDataArray()
-  {
-    return data_array;
-  }
+  using super::num_el;
+  using super::matcher_name;
+  using super::file_name;
+  using super::data_array;
+  using super::array_len;
+
   HostLookup *
   getHLookup()
   {
@@ -251,42 +264,29 @@ public:
 
 private:
   static void PrintFunc(void *opaque_data);
-  HostLookup *host_lookup;  // Data structure to do the lookups
-  Data *data_array;         // array of all data items
-  int array_len;            // the length of the arrays
-  int num_el;               // the numbe of itmems in the tree
-  const char *matcher_name; // Used for Debug/Warning/Error messages
-  const char *file_name;    // Used for Debug/Warning/Error messages
+  HostLookup *host_lookup = nullptr; // Data structure to do the lookups
 };
 
-template <class Data, class Result> class IpMatcher
+template <class Data, class MatchResult> class IpMatcher : protected BaseMatcher<Data>
 {
+  typedef BaseMatcher<Data> super;
+
 public:
   IpMatcher(const char *name, const char *filename);
-  ~IpMatcher();
-  void Match(sockaddr const *ip_addr, RequestData *rdata, Result *result);
+  void Match(sockaddr const *ip_addr, RequestData *rdata, MatchResult *result);
   void AllocateSpace(int num_entries);
-  config_parse_error NewEntry(matcher_line *line_info);
+  Result NewEntry(matcher_line *line_info);
   void Print();
 
-  int
-  getNumElements()
-  {
-    return num_el;
-  }
-  Data *
-  getDataArray()
-  {
-    return data_array;
-  }
+  using super::num_el;
+  using super::matcher_name;
+  using super::file_name;
+  using super::data_array;
+  using super::array_len;
 
+private:
   static void PrintFunc(void *opaque_data);
-  IpMap ip_map;             // Data structure to do lookups
-  Data *data_array;         // array of the data lements with in the table
-  int array_len;            // size of the arrays
-  int num_el;               // number of elements in the table
-  const char *matcher_name; // Used for Debug/Warning/Error messages
-  const char *file_name;    // Used for Debug/Warning/Error messages
+  IpMap ip_map; // Data structure to do lookups
 };
 
 #define ALLOW_HOST_TABLE 1 << 0
@@ -296,7 +296,7 @@ public:
 #define ALLOW_URL_TABLE 1 << 4
 #define DONT_BUILD_TABLE 1 << 5 // for testing
 
-template <class Data, class Result> class ControlMatcher
+template <class Data, class MatchResult> class ControlMatcher
 {
 public:
   // Parameter name must not be deallocated before this
@@ -306,7 +306,7 @@ public:
   ~ControlMatcher();
   int BuildTable();
   int BuildTableFromString(char *str);
-  void Match(RequestData *rdata, Result *result);
+  void Match(RequestData *rdata, MatchResult *result);
   void Print();
 
   int
@@ -314,43 +314,35 @@ public:
   {
     return m_numEntries;
   }
-  HostMatcher<Data, Result> *
+
+  HostMatcher<Data, MatchResult> *
   getHostMatcher()
   {
     return hostMatch;
   }
-  RegexMatcher<Data, Result> *
+
+  RegexMatcher<Data, MatchResult> *
   getReMatcher()
   {
     return reMatch;
   }
-  UrlMatcher<Data, Result> *
-  getUrlMatcher()
-  {
-    return urlMatch;
-  }
-  IpMatcher<Data, Result> *
+
+  IpMatcher<Data, MatchResult> *
   getIPMatcher()
   {
     return ipMatch;
   }
-  HostRegexMatcher<Data, Result> *
-  getHrMatcher()
-  {
-    return hrMatch;
-  }
 
-  // private:
-  RegexMatcher<Data, Result> *reMatch;
-  UrlMatcher<Data, Result> *urlMatch;
-  HostMatcher<Data, Result> *hostMatch;
-  IpMatcher<Data, Result> *ipMatch;
-  HostRegexMatcher<Data, Result> *hrMatch;
-  const matcher_tags *config_tags;
+  // private
+  RegexMatcher<Data, MatchResult> *reMatch;
+  UrlMatcher<Data, MatchResult> *urlMatch;
+  HostMatcher<Data, MatchResult> *hostMatch;
+  IpMatcher<Data, MatchResult> *ipMatch;
+  HostRegexMatcher<Data, MatchResult> *hrMatch;
+
+  const matcher_tags *config_tags = nullptr;
   char config_file_path[PATH_NAME_MAX];
-  int flags;
-  int m_numEntries;
-  const char *matcher_name; // Used for Debug/Warning/Error messages
+  int flags                = 0;
+  int m_numEntries         = 0;
+  const char *matcher_name = "unknown"; // Used for Debug/Warning/Error messages
 };
-
-#endif /* _CONTROL_MATCHER_H_ */
