@@ -74,15 +74,12 @@
 #include "PluginVC.h"
 #include "P_EventSystem.h"
 #include "P_Net.h"
-#include "ts/Regression.h"
+#include "tscore/Regression.h"
 
 #define PVC_LOCK_RETRY_TIME HRTIME_MSECONDS(10)
 #define PVC_DEFAULT_MAX_BYTES 32768
 #define MIN_BLOCK_TRANSFER_BYTES 128
 
-#define EVENT_PTR_LOCKED (void *)0x1
-#define EVENT_PTR_CLOSED (void *)0x2
-
 #define PVC_TYPE ((vc_type == PLUGIN_VC_ACTIVE) ? "Active" : "Passive")
 
 PluginVC::PluginVC(PluginVCCore *core_obj)
@@ -127,7 +124,7 @@ PluginVC::main_handler(int event, void *data)
   ink_assert(!deletable);
   ink_assert(data != nullptr);
 
-  Event *call_event   = (Event *)data;
+  Event *call_event   = static_cast<Event *>(data);
   EThread *my_ethread = mutex->thread_holding;
   ink_release_assert(my_ethread != nullptr);
 
@@ -211,9 +208,6 @@ PluginVC::main_handler(int event, void *data)
   } else if (call_event == inactive_event) {
     if (inactive_timeout_at && inactive_timeout_at < Thread::get_hrtime()) {
       process_timeout(&inactive_event, VC_EVENT_INACTIVITY_TIMEOUT);
-      if (nullptr == inactive_event) {
-        call_event->cancel();
-      }
     }
   } else {
     if (call_event == sm_lock_retry_event) {
@@ -262,8 +256,8 @@ PluginVC::do_io_read(Continuation *c, int64_t nbytes, MIOBuffer *buf)
 
   // Note: we set vio.op last because process_read_side looks at it to
   //  tell if the VConnection is active.
-  read_state.vio.mutex     = c->mutex;
-  read_state.vio._cont     = c;
+  read_state.vio.mutex     = c ? c->mutex : this->mutex;
+  read_state.vio.cont      = c;
   read_state.vio.nbytes    = nbytes;
   read_state.vio.ndone     = 0;
   read_state.vio.vc_server = (VConnection *)this;
@@ -295,7 +289,7 @@ PluginVC::do_io_write(Continuation *c, int64_t nbytes, IOBufferReader *abuffer,
   // Note: we set vio.op last because process_write_side looks at it to
   //  tell if the VConnection is active.
   write_state.vio.mutex     = c ? c->mutex : this->mutex;
-  write_state.vio._cont     = c;
+  write_state.vio.cont      = c;
   write_state.vio.nbytes    = nbytes;
   write_state.vio.ndone     = 0;
   write_state.vio.vc_server = (VConnection *)this;
@@ -316,6 +310,7 @@ PluginVC::reenable(VIO *vio)
 {
   ink_assert(!closed);
   ink_assert(magic == PLUGIN_VC_MAGIC_ALIVE);
+  ink_assert(vio->mutex->thread_holding == this_ethread());
 
   Ptr<ProxyMutex> sm_mutex = vio->mutex;
   SCOPED_MUTEX_LOCK(lock, sm_mutex, this_ethread());
@@ -338,6 +333,7 @@ PluginVC::reenable_re(VIO *vio)
 {
   ink_assert(!closed);
   ink_assert(magic == PLUGIN_VC_MAGIC_ALIVE);
+  ink_assert(vio->mutex->thread_holding == this_ethread());
 
   Debug("pvc", "[%u] %s: reenable_re %s", core_obj->id, PVC_TYPE, (vio->op == VIO::WRITE) ? "Write" : "Read");
 
@@ -432,7 +428,7 @@ PluginVC::transfer_bytes(MIOBuffer *transfer_to, IOBufferReader *transfer_from,
 
   while (act_on > 0) {
     int64_t block_read_avail = transfer_from->block_read_avail();
-    int64_t to_move          = MIN(act_on, block_read_avail);
+    int64_t to_move          = std::min(act_on, block_read_avail);
     int64_t moved            = 0;
 
     if (to_move <= 0) {
@@ -479,25 +475,13 @@ PluginVC::process_write_side(bool other_side_call)
 
   MIOBuffer *core_buffer = (vc_type == PLUGIN_VC_ACTIVE) ? core_obj->a_to_p_buffer : core_obj->p_to_a_buffer;
 
+  Debug("pvc", "[%u] %s: process_write_side", core_obj->id, PVC_TYPE);
   need_write_process = false;
 
+  // Check write_state
   if (write_state.vio.op != VIO::WRITE || closed || write_state.shutdown) {
     return;
   }
-  // Acquire the lock of the write side continuation
-  EThread *my_ethread = mutex->thread_holding;
-  ink_assert(my_ethread != nullptr);
-  MUTEX_TRY_LOCK(lock, write_state.vio.mutex, my_ethread);
-  if (!lock.is_locked()) {
-    Debug("pvc_event", "[%u] %s: process_write_side lock miss, retrying", core_obj->id, PVC_TYPE);
-
-    need_write_process = true;
-    setup_event_cb(PVC_LOCK_RETRY_TIME, &core_lock_retry_event);
-    return;
-  }
-
-  Debug("pvc", "[%u] %s: process_write_side", core_obj->id, PVC_TYPE);
-  need_write_process = false;
 
   // Check the state of our write buffer as well as ntodo
   int64_t ntodo = write_state.vio.ntodo();
@@ -507,12 +491,12 @@ PluginVC::process_write_side(bool other_side_call)
 
   IOBufferReader *reader = write_state.vio.get_reader();
   int64_t bytes_avail    = reader->read_avail();
-  int64_t act_on         = MIN(bytes_avail, ntodo);
+  int64_t act_on         = std::min(bytes_avail, ntodo);
 
   Debug("pvc", "[%u] %s: process_write_side; act_on %" PRId64 "", core_obj->id, PVC_TYPE, act_on);
 
   if (other_side->closed || other_side->read_state.shutdown) {
-    write_state.vio._cont->handleEvent(VC_EVENT_ERROR, &write_state.vio);
+    write_state.vio.cont->handleEvent(VC_EVENT_ERROR, &write_state.vio);
     return;
   }
 
@@ -520,7 +504,7 @@ PluginVC::process_write_side(bool other_side_call)
     if (ntodo > 0) {
       // Notify the continuation that we are "disabling"
       //  ourselves due to to nothing to write
-      write_state.vio._cont->handleEvent(VC_EVENT_WRITE_READY, &write_state.vio);
+      write_state.vio.cont->handleEvent(VC_EVENT_WRITE_READY, &write_state.vio);
     }
     return;
   }
@@ -532,7 +516,7 @@ PluginVC::process_write_side(bool other_side_call)
     Debug("pvc", "[%u] %s: process_write_side no buffer space", core_obj->id, PVC_TYPE);
     return;
   }
-  act_on = MIN(act_on, buf_space);
+  act_on = std::min(act_on, buf_space);
 
   int64_t added = transfer_bytes(core_buffer, reader, act_on);
   if (added < 0) {
@@ -548,9 +532,9 @@ PluginVC::process_write_side(bool other_side_call)
   Debug("pvc", "[%u] %s: process_write_side; added %" PRId64 "", core_obj->id, PVC_TYPE, added);
 
   if (write_state.vio.ntodo() == 0) {
-    write_state.vio._cont->handleEvent(VC_EVENT_WRITE_COMPLETE, &write_state.vio);
+    write_state.vio.cont->handleEvent(VC_EVENT_WRITE_COMPLETE, &write_state.vio);
   } else {
-    write_state.vio._cont->handleEvent(VC_EVENT_WRITE_READY, &write_state.vio);
+    write_state.vio.cont->handleEvent(VC_EVENT_WRITE_READY, &write_state.vio);
   }
 
   update_inactive_time();
@@ -558,6 +542,30 @@ PluginVC::process_write_side(bool other_side_call)
   // Wake up the read side on the other side to process these bytes
   if (!other_side->closed) {
     if (!other_side_call) {
+      /* To clear the `need_read_process`, the mutexes must be obtained:
+       *
+       *   - PluginVC::mutex
+       *   - PluginVC::read_state.vio.mutex
+       *
+       */
+      if (other_side->read_state.vio.op != VIO::READ || other_side->closed || other_side->read_state.shutdown) {
+        // Just return, no touch on `other_side->need_read_process`.
+        return;
+      }
+      // Acquire the lock of the read side continuation
+      EThread *my_ethread = mutex->thread_holding;
+      ink_assert(my_ethread != nullptr);
+      MUTEX_TRY_LOCK(lock, other_side->read_state.vio.mutex, my_ethread);
+      if (!lock.is_locked()) {
+        Debug("pvc_event", "[%u] %s: process_read_side from other side lock miss, retrying", other_side->core_obj->id,
+              ((other_side->vc_type == PLUGIN_VC_ACTIVE) ? "Active" : "Passive"));
+
+        // set need_read_process to enforce the read processing
+        other_side->need_read_process = true;
+        other_side->setup_event_cb(PVC_LOCK_RETRY_TIME, &other_side->core_lock_retry_event);
+        return;
+      }
+
       other_side->process_read_side(true);
     } else {
       other_side->read_state.vio.reenable();
@@ -593,28 +601,11 @@ PluginVC::process_read_side(bool other_side_call)
     core_reader = core_obj->a_to_p_reader;
   }
 
-  need_read_process = false;
-
-  if (read_state.vio.op != VIO::READ || closed) {
-    return;
-  }
-  // Acquire the lock of the read side continuation
-  EThread *my_ethread = mutex->thread_holding;
-  ink_assert(my_ethread != nullptr);
-  MUTEX_TRY_LOCK(lock, read_state.vio.mutex, my_ethread);
-  if (!lock.is_locked()) {
-    Debug("pvc_event", "[%u] %s: process_read_side lock miss, retrying", core_obj->id, PVC_TYPE);
-
-    need_read_process = true;
-    setup_event_cb(PVC_LOCK_RETRY_TIME, &core_lock_retry_event);
-    return;
-  }
-
   Debug("pvc", "[%u] %s: process_read_side", core_obj->id, PVC_TYPE);
   need_read_process = false;
 
-  // Check read_state.shutdown after the lock has been obtained.
-  if (read_state.shutdown) {
+  // Check read_state
+  if (read_state.vio.op != VIO::READ || closed || read_state.shutdown) {
     return;
   }
 
@@ -625,13 +616,13 @@ PluginVC::process_read_side(bool other_side_call)
   }
 
   int64_t bytes_avail = core_reader->read_avail();
-  int64_t act_on      = MIN(bytes_avail, ntodo);
+  int64_t act_on      = std::min(bytes_avail, ntodo);
 
   Debug("pvc", "[%u] %s: process_read_side; act_on %" PRId64 "", core_obj->id, PVC_TYPE, act_on);
 
   if (act_on <= 0) {
     if (other_side->closed || other_side->write_state.shutdown) {
-      read_state.vio._cont->handleEvent(VC_EVENT_EOS, &read_state.vio);
+      read_state.vio.cont->handleEvent(VC_EVENT_EOS, &read_state.vio);
     }
     return;
   }
@@ -641,13 +632,13 @@ PluginVC::process_read_side(bool other_side_call)
   MIOBuffer *output_buffer = read_state.vio.get_writer();
 
   int64_t water_mark = output_buffer->water_mark;
-  water_mark         = MAX(water_mark, PVC_DEFAULT_MAX_BYTES);
+  water_mark         = std::max(water_mark, static_cast<int64_t>(PVC_DEFAULT_MAX_BYTES));
   int64_t buf_space  = water_mark - output_buffer->max_read_avail();
   if (buf_space <= 0) {
     Debug("pvc", "[%u] %s: process_read_side no buffer space", core_obj->id, PVC_TYPE);
     return;
   }
-  act_on = MIN(act_on, buf_space);
+  act_on = std::min(act_on, buf_space);
 
   int64_t added = transfer_bytes(output_buffer, core_reader, act_on);
   if (added <= 0) {
@@ -663,9 +654,9 @@ PluginVC::process_read_side(bool other_side_call)
   Debug("pvc", "[%u] %s: process_read_side; added %" PRId64 "", core_obj->id, PVC_TYPE, added);
 
   if (read_state.vio.ntodo() == 0) {
-    read_state.vio._cont->handleEvent(VC_EVENT_READ_COMPLETE, &read_state.vio);
+    read_state.vio.cont->handleEvent(VC_EVENT_READ_COMPLETE, &read_state.vio);
   } else {
-    read_state.vio._cont->handleEvent(VC_EVENT_READ_READY, &read_state.vio);
+    read_state.vio.cont->handleEvent(VC_EVENT_READ_READY, &read_state.vio);
   }
 
   update_inactive_time();
@@ -674,6 +665,30 @@ PluginVC::process_read_side(bool other_side_call)
   //  intermediate buffer
   if (!other_side->closed) {
     if (!other_side_call) {
+      /* To clear the `need_write_process`, the mutexes must be obtained:
+       *
+       *   - PluginVC::mutex
+       *   - PluginVC::write_state.vio.mutex
+       *
+       */
+      if (other_side->write_state.vio.op != VIO::WRITE || other_side->closed || other_side->write_state.shutdown) {
+        // Just return, no touch on `other_side->need_write_process`.
+        return;
+      }
+      // Acquire the lock of the write side continuation
+      EThread *my_ethread = mutex->thread_holding;
+      ink_assert(my_ethread != nullptr);
+      MUTEX_TRY_LOCK(lock, other_side->write_state.vio.mutex, my_ethread);
+      if (!lock.is_locked()) {
+        Debug("pvc_event", "[%u] %s: process_write_side from other side lock miss, retrying", other_side->core_obj->id,
+              ((other_side->vc_type == PLUGIN_VC_ACTIVE) ? "Active" : "Passive"));
+
+        // set need_write_process to enforce the write processing
+        other_side->need_write_process = true;
+        other_side->setup_event_cb(PVC_LOCK_RETRY_TIME, &other_side->core_lock_retry_event);
+        return;
+      }
+
       other_side->process_write_side(true);
     } else {
       other_side->write_state.vio.reenable();
@@ -751,7 +766,7 @@ PluginVC::process_timeout(Event **e, int event_to_send)
   if (closed) {
     // already closed, ignore the timeout event
     // to avoid handle_event asserting use-after-free
-    *e = nullptr;
+    clear_event(e);
     return;
   }
 
@@ -764,8 +779,8 @@ PluginVC::process_timeout(Event **e, int event_to_send)
       }
       return;
     }
-    *e = nullptr;
-    read_state.vio._cont->handleEvent(event_to_send, &read_state.vio);
+    clear_event(e);
+    read_state.vio.cont->handleEvent(event_to_send, &read_state.vio);
   } else if (write_state.vio.op == VIO::WRITE && !write_state.shutdown && write_state.vio.ntodo() > 0) {
     MUTEX_TRY_LOCK(lock, write_state.vio.mutex, (*e)->ethread);
     if (!lock.is_locked()) {
@@ -775,13 +790,26 @@ PluginVC::process_timeout(Event **e, int event_to_send)
       }
       return;
     }
-    *e = nullptr;
-    write_state.vio._cont->handleEvent(event_to_send, &write_state.vio);
+    clear_event(e);
+    write_state.vio.cont->handleEvent(event_to_send, &write_state.vio);
   } else {
-    *e = nullptr;
+    clear_event(e);
   }
 }
 
+void
+PluginVC::clear_event(Event **e)
+{
+  if (e == nullptr || *e == nullptr) {
+    return;
+  }
+  if (*e == inactive_event) {
+    inactive_event->cancel();
+    inactive_timeout_at = 0;
+  }
+  *e = nullptr;
+}
+
 void
 PluginVC::update_inactive_time()
 {
@@ -858,6 +886,18 @@ PluginVC::set_inactivity_timeout(ink_hrtime timeout_in)
   }
 }
 
+void
+PluginVC::set_default_inactivity_timeout(ink_hrtime timeout_in)
+{
+  set_inactivity_timeout(timeout_in);
+}
+
+bool
+PluginVC::is_default_inactivity_timeout()
+{
+  return false;
+}
+
 void
 PluginVC::cancel_active_timeout()
 {
@@ -898,7 +938,7 @@ bool
 PluginVC::add_to_active_queue()
 {
   // do nothing
-  return false;
+  return true;
 }
 
 SOCKET
@@ -930,10 +970,16 @@ PluginVC::set_remote_addr()
   }
 }
 
-int
-PluginVC::set_tcp_init_cwnd(int /* init_cwnd ATS_UNUSED */)
+void
+PluginVC::set_remote_addr(const sockaddr * /* new_sa ATS_UNUSED */)
 {
-  return -1;
+  return;
+}
+
+void
+PluginVC::set_mptcp_state()
+{
+  return;
 }
 
 int
@@ -957,23 +1003,23 @@ PluginVC::get_data(int id, void *data)
   switch (id) {
   case PLUGIN_VC_DATA_LOCAL:
     if (vc_type == PLUGIN_VC_ACTIVE) {
-      *(void **)data = core_obj->active_data;
+      *static_cast<void **>(data) = core_obj->active_data;
     } else {
-      *(void **)data = core_obj->passive_data;
+      *static_cast<void **>(data) = core_obj->passive_data;
     }
     return true;
   case PLUGIN_VC_DATA_REMOTE:
     if (vc_type == PLUGIN_VC_ACTIVE) {
-      *(void **)data = core_obj->passive_data;
+      *static_cast<void **>(data) = core_obj->passive_data;
     } else {
-      *(void **)data = core_obj->active_data;
+      *static_cast<void **>(data) = core_obj->active_data;
     }
     return true;
   case TS_API_DATA_CLOSED:
     *static_cast<int *>(data) = this->closed;
     return true;
   default:
-    *(void **)data = nullptr;
+    *static_cast<void **>(data) = nullptr;
     return false;
   }
 }
@@ -1003,11 +1049,9 @@ PluginVC::set_data(int id, void *data)
 
 // PluginVCCore
 
-vint32 PluginVCCore::nextid = 0;
+int32_t PluginVCCore::nextid;
 
-PluginVCCore::~PluginVCCore()
-{
-}
+PluginVCCore::~PluginVCCore() = default;
 
 PluginVCCore *
 PluginVCCore::alloc(Continuation *acceptor)
@@ -1250,13 +1294,11 @@ public:
   int main_handler(int event, void *data);
 
 private:
-  unsigned i;
-  unsigned completions_received;
+  unsigned i                    = 0;
+  unsigned completions_received = 0;
 };
 
-PVCTestDriver::PVCTestDriver() : NetTestDriver(), i(0), completions_received(0)
-{
-}
+PVCTestDriver::PVCTestDriver() : NetTestDriver() {}
 
 PVCTestDriver::~PVCTestDriver()
 {
@@ -1271,10 +1313,9 @@ PVCTestDriver::start_tests(RegressionTest *r_arg, int *pstatus_arg)
 
   r       = r_arg;
   pstatus = pstatus_arg;
+  SET_HANDLER(&PVCTestDriver::main_handler);
 
   run_next_test();
-
-  SET_HANDLER(&PVCTestDriver::main_handler);
 }
 
 void
