@@ -27,73 +27,59 @@
 
  */
 
-#include "ts/ink_platform.h"
+#include "tscore/ink_platform.h"
+#include "tscore/Filenames.h"
 #include <dlfcn.h>
-#include "Main.h"
 #include "P_EventSystem.h"
 #include "P_Cache.h"
 #include "ProxyConfig.h"
 #include "ReverseProxy.h"
-#include "ts/MatcherUtils.h"
-#include "ts/Tokenizer.h"
-#include "api/ts/remap.h"
+#include "tscore/MatcherUtils.h"
+#include "tscore/Tokenizer.h"
+#include "ts/remap.h"
 #include "RemapPluginInfo.h"
 #include "RemapProcessor.h"
 #include "UrlRewrite.h"
 #include "UrlMapping.h"
 
-/** Time till we free the old stuff after a reconfiguration. */
-#define URL_REWRITE_TIMEOUT (HRTIME_SECOND * 60)
-
 // Global Ptrs
 static Ptr<ProxyMutex> reconfig_mutex;
-UrlRewrite *rewrite_table = NULL;
-remap_plugin_info *remap_pi_list; // We never reload the remap plugins, just append to 'em.
+UrlRewrite *rewrite_table                             = nullptr;
+thread_local PluginThreadContext *pluginThreadContext = nullptr;
 
 // Tokens for the Callback function
 #define FILE_CHANGED 0
 #define REVERSE_CHANGED 1
 #define TSNAME_CHANGED 2
-#define SYNTH_PORT_CHANGED 3
 #define TRANS_CHANGED 4
 #define URL_REMAP_MODE_CHANGED 8
 #define HTTP_DEFAULT_REDIRECT_CHANGED 9
 
-int url_remap_mode;
-
 //
 // Begin API Functions
 //
-
 int
 init_reverse_proxy()
 {
-  ink_assert(rewrite_table == NULL);
+  ink_assert(rewrite_table == nullptr);
   reconfig_mutex = new_ProxyMutex();
   rewrite_table  = new UrlRewrite();
 
-  if (!rewrite_table->is_valid()) {
-    Fatal("unable to load remap.config");
+  Note("%s loading ...", ts::filename::REMAP);
+  if (!rewrite_table->load()) {
+    Fatal("%s failed to load", ts::filename::REMAP);
   }
+  Note("%s finished loading", ts::filename::REMAP);
 
   REC_RegisterConfigUpdateFunc("proxy.config.url_remap.filename", url_rewrite_CB, (void *)FILE_CHANGED);
   REC_RegisterConfigUpdateFunc("proxy.config.proxy_name", url_rewrite_CB, (void *)TSNAME_CHANGED);
   REC_RegisterConfigUpdateFunc("proxy.config.reverse_proxy.enabled", url_rewrite_CB, (void *)REVERSE_CHANGED);
-  REC_RegisterConfigUpdateFunc("proxy.config.admin.synthetic_port", url_rewrite_CB, (void *)SYNTH_PORT_CHANGED);
   REC_RegisterConfigUpdateFunc("proxy.config.http.referer_default_redirect", url_rewrite_CB, (void *)HTTP_DEFAULT_REDIRECT_CHANGED);
-  return 0;
-}
 
-// TODO: This function needs to be rewritten (or replaced) with something that uses the new
-// Remap Processor properly. Right now, we effectively don't support "remap" rules on a few
-// odd ball configs, for example if you use the "CONNECT" method, or if you set
-// set proxy.config.url_remap.url_remap_mode to "2" (which is a completely undocumented "feature").
-bool
-request_url_remap(HttpTransact::State * /* s ATS_UNUSED */, HTTPHdr * /* request_header ATS_UNUSED */,
-                  char ** /* redirect_url ATS_UNUSED */, unsigned int /* filter_mask ATS_UNUSED */)
-{
-  return false;
-  // return rewrite_table ? rewrite_table->Remap(s, request_header, redirect_url, orig_url, tag, filter_mask) : false;
+  // Hold at least one lease, until we reload the configuration
+  rewrite_table->acquire();
+
+  return 0;
 }
 
 /**
@@ -101,15 +87,15 @@ request_url_remap(HttpTransact::State * /* s ATS_UNUSED */, HTTPHdr * /* request
    according to the rules in remap.config.
 */
 mapping_type
-request_url_remap_redirect(HTTPHdr *request_header, URL *redirect_url)
+request_url_remap_redirect(HTTPHdr *request_header, URL *redirect_url, UrlRewrite *table)
 {
-  return rewrite_table ? rewrite_table->Remap_redirect(request_header, redirect_url) : NONE;
+  return table ? table->Remap_redirect(request_header, redirect_url) : NONE;
 }
 
 bool
-response_url_remap(HTTPHdr *response_header)
+response_url_remap(HTTPHdr *response_header, UrlRewrite *table)
 {
-  return rewrite_table ? rewrite_table->ReverseMap(response_header) : false;
+  return table ? table->ReverseMap(response_header) : false;
 }
 
 //
@@ -119,7 +105,7 @@ response_url_remap(HTTPHdr *response_header)
 
 /** Used to read the remap.config file after the manager signals a change. */
 struct UR_UpdateContinuation;
-typedef int (UR_UpdateContinuation::*UR_UpdContHandler)(int, void *);
+using UR_UpdContHandler = int (UR_UpdateContinuation::*)(int, void *);
 struct UR_UpdateContinuation : public Continuation {
   int
   file_update_handler(int /* etype ATS_UNUSED */, void * /* data ATS_UNUSED */)
@@ -134,6 +120,12 @@ struct UR_UpdateContinuation : public Continuation {
   }
 };
 
+bool
+urlRewriteVerify()
+{
+  return UrlRewrite().load();
+}
+
 /**
   Called when the remap.config file changes. Since it called infrequently,
   we do the load of new file as blocking I/O and lock aquire is also
@@ -143,20 +135,34 @@ struct UR_UpdateContinuation : public Continuation {
 bool
 reloadUrlRewrite()
 {
-  UrlRewrite *newTable;
+  UrlRewrite *newTable, *oldTable;
 
-  Debug("url_rewrite", "remap.config updated, reloading...");
+  Note("%s loading ...", ts::filename::REMAP);
+  Debug("url_rewrite", "%s updated, reloading...", ts::filename::REMAP);
   newTable = new UrlRewrite();
-  if (newTable->is_valid()) {
-    new_Deleter(rewrite_table, URL_REWRITE_TIMEOUT);
-    Debug("url_rewrite", "remap.config done reloading!");
-    ink_atomic_swap(&rewrite_table, newTable);
+  if (newTable->load()) {
+    static const char *msg_format = "%s finished loading";
+
+    // Hold at least one lease, until we reload the configuration
+    newTable->acquire();
+
+    // Swap configurations
+    oldTable = ink_atomic_swap(&rewrite_table, newTable);
+
+    ink_assert(oldTable != nullptr);
+
+    // Release the old one
+    oldTable->release();
+
+    Debug("url_rewrite", msg_format, ts::filename::REMAP);
+    Note(msg_format, ts::filename::REMAP);
     return true;
   } else {
-    static const char *msg = "failed to reload remap.config, not replacing!";
+    static const char *msg_format = "%s failed to load";
+
     delete newTable;
-    Debug("url_rewrite", "%s", msg);
-    Warning("%s", msg);
+    Debug("url_rewrite", msg_format, ts::filename::REMAP);
+    Error(msg_format, ts::filename::REMAP);
     return false;
   }
 }
@@ -164,7 +170,7 @@ reloadUrlRewrite()
 int
 url_rewrite_CB(const char * /* name ATS_UNUSED */, RecDataT /* data_type ATS_UNUSED */, RecData data, void *cookie)
 {
-  int my_token = (int)(long)cookie;
+  int my_token = static_cast<int>((long)cookie);
 
   switch (my_token) {
   case REVERSE_CHANGED:
@@ -177,10 +183,6 @@ url_rewrite_CB(const char * /* name ATS_UNUSED */, RecDataT /* data_type ATS_UNU
     eventProcessor.schedule_imm(new UR_UpdateContinuation(reconfig_mutex), ET_TASK);
     break;
 
-  case SYNTH_PORT_CHANGED:
-    // The AutoConf port does not current change on manager except at restart
-    break;
-
   case URL_REMAP_MODE_CHANGED:
     // You need to restart TS.
     break;
