@@ -65,9 +65,6 @@
 #include "HdrUtils.h"
 #include "Log.h"
 
-#define ART 1
-#define AGIF 2
-
 TransformProcessor transformProcessor;
 
 /*-------------------------------------------------------------------------
@@ -76,9 +73,6 @@ TransformProcessor transformProcessor;
 void
 TransformProcessor::start()
 {
-#ifdef PREFETCH
-  prefetchProcessor.start();
-#endif
 }
 
 /*-------------------------------------------------------------------------
@@ -131,11 +125,11 @@ TransformTerminus::TransformTerminus(TransformVConnection *tvc)
   SET_HANDLER(&TransformTerminus::handle_event);
 }
 
-#define RETRY()                                                  \
-  if (ink_atomic_increment((int *)&m_event_count, 1) < 0) {      \
-    ink_assert(!"not reached");                                  \
-  }                                                              \
-  eventProcessor.schedule_in(this, HRTIME_MSECONDS(10), ET_NET); \
+#define RETRY()                                             \
+  if (ink_atomic_increment((int *)&m_event_count, 1) < 0) { \
+    ink_assert(!"not reached");                             \
+  }                                                         \
+  this_ethread()->schedule_in(this, HRTIME_MSECONDS(10));   \
   return 0;
 
 int
@@ -145,7 +139,7 @@ TransformTerminus::handle_event(int event, void * /* edata ATS_UNUSED */)
 
   m_deletable = ((m_closed != 0) && (m_tvc->m_closed != 0));
 
-  val = ink_atomic_increment((int *)&m_event_count, -1);
+  val = ink_atomic_increment(&m_event_count, -1);
 
   Debug("transform", "[TransformTerminus::handle_event] event_count %d", m_event_count);
 
@@ -213,10 +207,10 @@ TransformTerminus::handle_event(int event, void * /* edata ATS_UNUSED */)
 
       if (m_write_vio.ntodo() > 0) {
         if (towrite > 0) {
-          m_write_vio._cont->handleEvent(VC_EVENT_WRITE_READY, &m_write_vio);
+          m_write_vio.cont->handleEvent(VC_EVENT_WRITE_READY, &m_write_vio);
         }
       } else {
-        m_write_vio._cont->handleEvent(VC_EVENT_WRITE_COMPLETE, &m_write_vio);
+        m_write_vio.cont->handleEvent(VC_EVENT_WRITE_COMPLETE, &m_write_vio);
       }
 
       // We could have closed on the write callback
@@ -226,12 +220,12 @@ TransformTerminus::handle_event(int event, void * /* edata ATS_UNUSED */)
 
       if (m_read_vio.ntodo() > 0) {
         if (m_write_vio.ntodo() <= 0) {
-          m_read_vio._cont->handleEvent(VC_EVENT_EOS, &m_read_vio);
+          m_read_vio.cont->handleEvent(VC_EVENT_EOS, &m_read_vio);
         } else if (towrite > 0) {
-          m_read_vio._cont->handleEvent(VC_EVENT_READ_READY, &m_read_vio);
+          m_read_vio.cont->handleEvent(VC_EVENT_READ_READY, &m_read_vio);
         }
       } else {
-        m_read_vio._cont->handleEvent(VC_EVENT_READ_COMPLETE, &m_read_vio);
+        m_read_vio.cont->handleEvent(VC_EVENT_READ_COMPLETE, &m_read_vio);
       }
     }
   } else {
@@ -254,10 +248,10 @@ TransformTerminus::handle_event(int event, void * /* edata ATS_UNUSED */)
 
         if (!m_called_user) {
           m_called_user = 1;
-          m_tvc->m_cont->handleEvent(ev, nullptr);
+          m_tvc->m_cont->handleEvent(ev, &m_read_vio);
         } else {
-          ink_assert(m_read_vio._cont != nullptr);
-          m_read_vio._cont->handleEvent(ev, &m_read_vio);
+          ink_assert(m_read_vio.cont != nullptr);
+          m_read_vio.cont->handleEvent(ev, &m_read_vio);
         }
       }
 
@@ -281,12 +275,12 @@ TransformTerminus::do_io_read(Continuation *c, int64_t nbytes, MIOBuffer *buf)
   m_read_vio.ndone     = 0;
   m_read_vio.vc_server = this;
 
-  if (ink_atomic_increment((int *)&m_event_count, 1) < 0) {
+  if (ink_atomic_increment(&m_event_count, 1) < 0) {
     ink_assert(!"not reached");
   }
   Debug("transform", "[TransformTerminus::do_io_read] event_count %d", m_event_count);
 
-  eventProcessor.schedule_imm(this, ET_NET);
+  this_ethread()->schedule_imm_local(this);
 
   return &m_read_vio;
 }
@@ -306,12 +300,12 @@ TransformTerminus::do_io_write(Continuation *c, int64_t nbytes, IOBufferReader *
   m_write_vio.ndone     = 0;
   m_write_vio.vc_server = this;
 
-  if (ink_atomic_increment((int *)&m_event_count, 1) < 0) {
+  if (ink_atomic_increment(&m_event_count, 1) < 0) {
     ink_assert(!"not reached");
   }
   Debug("transform", "[TransformTerminus::do_io_write] event_count %d", m_event_count);
 
-  eventProcessor.schedule_imm(this, ET_NET);
+  this_ethread()->schedule_imm_local(this);
 
   return &m_write_vio;
 }
@@ -322,7 +316,7 @@ TransformTerminus::do_io_write(Continuation *c, int64_t nbytes, IOBufferReader *
 void
 TransformTerminus::do_io_close(int error)
 {
-  if (ink_atomic_increment((int *)&m_event_count, 1) < 0) {
+  if (ink_atomic_increment(&m_event_count, 1) < 0) {
     ink_assert(!"not reached");
   }
 
@@ -341,7 +335,7 @@ TransformTerminus::do_io_close(int error)
   m_write_vio.op = VIO::NONE;
   m_write_vio.buffer.clear();
 
-  eventProcessor.schedule_imm(this, ET_NET);
+  this_ethread()->schedule_imm_local(this);
 }
 
 /*-------------------------------------------------------------------------
@@ -370,11 +364,11 @@ TransformTerminus::reenable(VIO *vio)
   ink_assert((vio == &m_read_vio) || (vio == &m_write_vio));
 
   if (m_event_count == 0) {
-    if (ink_atomic_increment((int *)&m_event_count, 1) < 0) {
+    if (ink_atomic_increment(&m_event_count, 1) < 0) {
       ink_assert(!"not reached");
     }
     Debug("transform", "[TransformTerminus::reenable] event_count %d", m_event_count);
-    eventProcessor.schedule_imm(this, ET_NET);
+    this_ethread()->schedule_imm_local(this);
   } else {
     Debug("transform", "[TransformTerminus::reenable] skipping due to "
                        "pending events");
@@ -530,8 +524,7 @@ TransformVConnection::backlog(uint64_t limit)
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
 
-TransformControl::TransformControl()
-  : Continuation(new_ProxyMutex()), m_hooks(), m_tvc(nullptr), m_read_buf(nullptr), m_write_buf(nullptr)
+TransformControl::TransformControl() : Continuation(new_ProxyMutex()), m_hooks()
 {
   SET_HANDLER(&TransformControl::handle_event);
 
@@ -552,11 +545,11 @@ TransformControl::handle_event(int event, void * /* edata ATS_UNUSED */)
     if (http_global_hooks && http_global_hooks->get(TS_HTTP_RESPONSE_TRANSFORM_HOOK)) {
       m_tvc = transformProcessor.open(this, http_global_hooks->get(TS_HTTP_RESPONSE_TRANSFORM_HOOK));
     } else {
-      m_tvc = transformProcessor.open(this, m_hooks.get());
+      m_tvc = transformProcessor.open(this, m_hooks.head());
     }
     ink_assert(m_tvc != nullptr);
 
-    m_write_buf = new_MIOBuffer();
+    m_write_buf = new_MIOBuffer(BUFFER_SIZE_INDEX_32K);
     s           = m_write_buf->end();
     e           = m_write_buf->buf_end();
 
@@ -568,7 +561,7 @@ TransformControl::handle_event(int event, void * /* edata ATS_UNUSED */)
   }
 
   case TRANSFORM_READ_READY: {
-    MIOBuffer *buf = new_empty_MIOBuffer();
+    MIOBuffer *buf = new_empty_MIOBuffer(BUFFER_SIZE_INDEX_32K);
 
     m_read_buf = buf->alloc_reader();
     m_tvc->do_io_read(this, INT64_MAX, buf);
@@ -639,7 +632,7 @@ NullTransform::handle_event(int event, void *edata)
   } else {
     switch (event) {
     case VC_EVENT_ERROR:
-      m_write_vio._cont->handleEvent(VC_EVENT_ERROR, &m_write_vio);
+      m_write_vio.cont->handleEvent(VC_EVENT_ERROR, &m_write_vio);
       break;
     case VC_EVENT_WRITE_COMPLETE:
       ink_assert(m_output_vio == (VIO *)edata);
@@ -659,7 +652,7 @@ NullTransform::handle_event(int event, void *edata)
       ink_assert(m_output_vc != nullptr);
 
       if (!m_output_vio) {
-        m_output_buf    = new_empty_MIOBuffer();
+        m_output_buf    = new_empty_MIOBuffer(BUFFER_SIZE_INDEX_32K);
         m_output_reader = m_output_buf->alloc_reader();
         m_output_vio    = m_output_vc->do_io_write(this, m_write_vio.nbytes, m_output_reader);
       }
@@ -688,8 +681,9 @@ NullTransform::handle_event(int event, void *edata)
         }
 
         if (towrite > 0) {
-          Debug("transform", "[NullTransform::handle_event] "
-                             "writing %" PRId64 " bytes to output",
+          Debug("transform",
+                "[NullTransform::handle_event] "
+                "writing %" PRId64 " bytes to output",
                 towrite);
           m_output_buf->write(m_write_vio.get_reader(), towrite);
 
@@ -701,12 +695,12 @@ NullTransform::handle_event(int event, void *edata)
       if (m_write_vio.ntodo() > 0) {
         if (towrite > 0) {
           m_output_vio->reenable();
-          m_write_vio._cont->handleEvent(VC_EVENT_WRITE_READY, &m_write_vio);
+          m_write_vio.cont->handleEvent(VC_EVENT_WRITE_READY, &m_write_vio);
         }
       } else {
         m_output_vio->nbytes = m_write_vio.ndone;
         m_output_vio->reenable();
-        m_write_vio._cont->handleEvent(VC_EVENT_WRITE_COMPLETE, &m_write_vio);
+        m_write_vio.cont->handleEvent(VC_EVENT_WRITE_COMPLETE, &m_write_vio);
       }
 
       break;
@@ -732,12 +726,12 @@ NullTransform::handle_event(int event, void *edata)
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
 
-#ifdef TS_HAS_TESTS
+#if TS_HAS_TESTS
 void
 TransformTest::run()
 {
   if (is_action_tag_set("transform_test")) {
-    eventProcessor.schedule_imm(new TransformControl(), ET_NET);
+    this_ethread()->schedule_imm_local(new TransformControl());
   }
 }
 #endif
@@ -787,13 +781,17 @@ RangeTransform::handle_event(int event, void *edata)
 
   if (m_closed) {
     if (m_deletable) {
-      Debug("http_trans", "RangeTransform destroy: %p ndone=%" PRId64, this, m_output_vio ? m_output_vio->ndone : 0);
+      if (m_output_vc != nullptr) {
+        Debug("http_trans", "RangeTransform destroy: %p ndone=%" PRId64, this, m_output_vio ? m_output_vio->ndone : 0);
+      } else {
+        Debug("http_trans", "RangeTransform destroy");
+      }
       delete this;
     }
   } else {
     switch (event) {
     case VC_EVENT_ERROR:
-      m_write_vio._cont->handleEvent(VC_EVENT_ERROR, &m_write_vio);
+      m_write_vio.cont->handleEvent(VC_EVENT_ERROR, &m_write_vio);
       break;
     case VC_EVENT_WRITE_COMPLETE:
       ink_assert(m_output_vio == (VIO *)edata);
@@ -804,7 +802,7 @@ RangeTransform::handle_event(int event, void *edata)
       ink_assert(m_output_vc != nullptr);
 
       if (!m_output_vio) {
-        m_output_buf    = new_empty_MIOBuffer();
+        m_output_buf    = new_empty_MIOBuffer(BUFFER_SIZE_INDEX_32K);
         m_output_reader = m_output_buf->alloc_reader();
         m_output_vio    = m_output_vc->do_io_write(this, m_output_cl, m_output_reader);
 
@@ -909,7 +907,7 @@ RangeTransform::transform_to_range()
         //   input data, send VC_EVENT_EOS to let the upstream know
         //   that it can rely on us consuming any more data
         int cb_event = (m_write_vio.ntodo() > 0) ? VC_EVENT_EOS : VC_EVENT_WRITE_COMPLETE;
-        m_write_vio._cont->handleEvent(cb_event, &m_write_vio);
+        m_write_vio.cont->handleEvent(cb_event, &m_write_vio);
         return;
       }
 
@@ -945,7 +943,7 @@ RangeTransform::transform_to_range()
   }
 
   m_output_vio->reenable();
-  m_write_vio._cont->handleEvent(VC_EVENT_WRITE_READY, &m_write_vio);
+  m_write_vio.cont->handleEvent(VC_EVENT_WRITE_READY, &m_write_vio);
 }
 
 /*-------------------------------------------------------------------------
@@ -1020,7 +1018,7 @@ RangeTransform::change_response_header()
 
   status_code = HTTP_STATUS_PARTIAL_CONTENT;
   m_transform_resp->status_set(status_code);
-  reason_phrase = (char *)(http_hdr_reason_lookup(status_code));
+  reason_phrase = const_cast<char *>(http_hdr_reason_lookup(status_code));
   m_transform_resp->reason_set(reason_phrase, strlen(reason_phrase));
 
   if (m_num_range_fields > 1) {
