@@ -30,35 +30,30 @@
 
  ****************************************************************************/
 
-#ifndef _HDR_HEAP_H_
-#define _HDR_HEAP_H_
+#pragma once
 
-#include "ts/Ptr.h"
-#include "ts/ink_defs.h"
-#include "ts/ink_assert.h"
-#include "ts/Arena.h"
+#include "tscore/Ptr.h"
+#include "tscore/ink_defs.h"
+#include "tscore/ink_assert.h"
+#include "tscore/Arena.h"
+#include "tscore/Scalar.h"
 #include "HdrToken.h"
 
 // Objects in the heap must currently be aligned to 8 byte boundaries,
 // so their (address & HDR_PTR_ALIGNMENT_MASK) == 0
 
-#define HDR_PTR_SIZE (sizeof(uint64_t))
-#define HDR_PTR_ALIGNMENT_MASK ((HDR_PTR_SIZE)-1L)
-
-#define ROUND(x, l) (((x) + ((l)-1L)) & ~((l)-1L))
+static constexpr size_t HDR_PTR_SIZE           = sizeof(uint64_t);
+static constexpr size_t HDR_PTR_ALIGNMENT_MASK = HDR_PTR_SIZE - 1L;
+using HdrHeapMarshalBlocks                     = ts::Scalar<HDR_PTR_SIZE>;
 
 // A many of the operations regarding read-only str
-//  heaps are hand unrolled in the code.  Chaning
+//  heaps are hand unrolled in the code.  Changing
 //  this value requires a full pass through HdrBuf.cc
 //  to fix the unrolled operations
-#define HDR_BUF_RONLY_HEAPS 3
+static constexpr unsigned HDR_BUF_RONLY_HEAPS = 3;
 
-#define HDR_HEAP_DEFAULT_SIZE 2048
-#define HDR_STR_HEAP_DEFAULT_SIZE 2048
-
-#define HDR_MAX_ALLOC_SIZE (HDR_HEAP_DEFAULT_SIZE - sizeof(HdrHeap))
-#define HDR_HEAP_HDR_SIZE ROUND(sizeof(HdrHeap), HDR_PTR_SIZE)
-#define STR_HEAP_HDR_SIZE sizeof(HdrStrHeap)
+class CoreUtils;
+class IOBufferBlock;
 
 enum {
   HDR_HEAP_OBJ_EMPTY            = 0,
@@ -135,26 +130,12 @@ enum {
   HDR_BUF_MAGIC_CORRUPT   = 0xbadbadcc
 };
 
-struct StrHeapDesc {
-  StrHeapDesc();
-  Ptr<RefCountObj> m_ref_count_ptr;
-  char *m_heap_start;
-  int32_t m_heap_len;
-  bool m_locked;
-
-  bool
-  contains(const char *str) const
-  {
-    return (str >= m_heap_start && str < (m_heap_start + m_heap_len));
-  }
-};
-
-class IOBufferBlock;
-
 class HdrStrHeap : public RefCountObj
 {
 public:
-  virtual void free();
+  static constexpr int DEFAULT_SIZE = 2048;
+
+  void free() override;
 
   char *allocate(int nbytes);
   char *expand(char *ptr, int old_size, int new_size);
@@ -164,20 +145,37 @@ public:
   char *m_free_start;
   uint32_t m_free_size;
 
+  bool contains(const char *str) const;
+};
+
+inline bool
+HdrStrHeap::contains(const char *str) const
+{
+  return reinterpret_cast<char const *>(this + 1) <= str && str < reinterpret_cast<char const *>(this) + m_heap_size;
+}
+
+struct StrHeapDesc {
+  StrHeapDesc() = default;
+
+  Ptr<RefCountObj> m_ref_count_ptr;
+  char const *m_heap_start = nullptr;
+  int32_t m_heap_len       = 0;
+  bool m_locked            = false;
+
   bool
   contains(const char *str) const
   {
-    return (str >= ((const char *)this + STR_HEAP_HDR_SIZE) && str < ((const char *)this + m_heap_size));
+    return (str >= m_heap_start && str < (m_heap_start + m_heap_len));
   }
 };
 
-class CoreUtils;
-
 class HdrHeap
 {
   friend class CoreUtils;
 
 public:
+  static constexpr int DEFAULT_SIZE = 2048;
+
   void init();
   inkcoreapi void destroy();
 
@@ -205,35 +203,57 @@ public:
   void set_ronly_str_heap_end(int slot, const char *end);
 
   // Lock read only str heaps so that can't be moved around
-  //  by a heap consolidation.  Does NOT lock for Multi-Threaed
+  //  by a heap consolidation.  Does NOT lock for Multi-Threaded
   //  access!
   void
-  lock_ronly_str_heap(int i)
+  lock_ronly_str_heap(unsigned i)
   {
     m_ronly_heap[i].m_locked = true;
-  };
+  }
+
   void
-  unlock_ronly_str_heap(int i)
+  unlock_ronly_str_heap(unsigned i)
   {
     m_ronly_heap[i].m_locked = false;
     // INKqa11238
     // Move slot i to the first available slot in m_ronly_heap[].
     // The move is necessary because the rest of the code assumes
     // heaps are always allocated in order.
-    for (int j = 0; j < i; j++) {
-      if (m_ronly_heap[j].m_heap_start == NULL) {
+    for (unsigned j = 0; j < i; j++) {
+      if (m_ronly_heap[j].m_heap_start == nullptr) {
         // move slot i to slot j
         m_ronly_heap[j].m_ref_count_ptr = m_ronly_heap[i].m_ref_count_ptr;
         m_ronly_heap[j].m_heap_start    = m_ronly_heap[i].m_heap_start;
         m_ronly_heap[j].m_heap_len      = m_ronly_heap[i].m_heap_len;
         m_ronly_heap[j].m_locked        = m_ronly_heap[i].m_locked;
-        m_ronly_heap[i].m_ref_count_ptr = NULL;
-        m_ronly_heap[i].m_heap_start    = NULL;
+        m_ronly_heap[i].m_ref_count_ptr = nullptr;
+        m_ronly_heap[i].m_heap_start    = nullptr;
         m_ronly_heap[i].m_heap_len      = 0;
         m_ronly_heap[i].m_locked        = false;
+        break; // Did the move, time to go.
       }
     }
-  };
+  }
+
+  // Working function to copy strings into a new heap
+  // Unlike the HDR_MOVE_STR macro, this function will call
+  // allocate_str which will update the new_heap to create more space
+  // if there is not originally sufficient space
+  inline std::string_view
+  localize(const std::string_view &string)
+  {
+    auto length = string.length();
+    if (length > 0) {
+      char *new_str = this->allocate_str(length);
+      if (new_str) {
+        memcpy(new_str, string.data(), length);
+      } else {
+        length = 0;
+      }
+      return {new_str, length};
+    }
+    return {nullptr, 0};
+  }
 
   // Sanity Check Functions
   void sanity_check_strs();
@@ -265,7 +285,9 @@ public:
   void coalesce_str_heaps(int incoming_size = 0);
   void evacuate_from_str_heaps(HdrStrHeap *new_heap);
   size_t required_space_for_evacuation();
-  int attach_str_heap(char *h_start, int h_len, RefCountObj *h_ref_obj, int *index);
+  bool attach_str_heap(char const *h_start, int h_len, RefCountObj *h_ref_obj, int *index);
+
+  uint64_t total_used_size() const;
 
   /** Struct to prevent garbage collection on heaps.
       This bumps the reference count to the heap containing the pointer
@@ -278,11 +300,11 @@ public:
     HeapGuard(HdrHeap *heap, const char *str)
     {
       if (heap->m_read_write_heap && heap->m_read_write_heap->contains(str)) {
-        m_ptr = heap->m_read_write_heap;
+        m_ptr = heap->m_read_write_heap.get();
       } else {
-        for (int i = 0; i < HDR_BUF_RONLY_HEAPS; ++i) {
-          if (heap->m_ronly_heap[i].contains(str)) {
-            m_ptr = heap->m_ronly_heap[i].m_ref_count_ptr;
+        for (auto &i : heap->m_ronly_heap) {
+          if (i.contains(str)) {
+            m_ptr = i.m_ref_count_ptr;
             break;
           }
         }
@@ -290,7 +312,7 @@ public:
     }
 
     // There's no need to have a destructor here, the default dtor will take care of
-    // releaseing the (potentially) locked heap.
+    // releasing the (potentially) locked heap.
 
     /// The heap we protect (if any)
     Ptr<RefCountObj> m_ptr;
@@ -302,6 +324,9 @@ public:
   int m_lost_string_space;
 };
 
+static constexpr HdrHeapMarshalBlocks HDR_HEAP_HDR_SIZE{ts::round_up(sizeof(HdrHeap))};
+static constexpr size_t HDR_MAX_ALLOC_SIZE = HdrHeap::DEFAULT_SIZE - HDR_HEAP_HDR_SIZE;
+
 inline void
 HdrHeap::free_string(const char *s, int len)
 {
@@ -318,14 +343,15 @@ HdrHeap::unmarshal_size() const
 
 //
 struct MarshalXlate {
-  char *start;
-  char *end;
-  char *offset;
+  char const *start  = nullptr;
+  char const *end    = nullptr;
+  char const *offset = nullptr;
+  MarshalXlate() {}
 };
 
 struct HeapCheck {
-  char *start;
-  char *end;
+  char const *start;
+  char const *end;
 };
 
 // Nasty macro to do string marshalling
@@ -434,7 +460,7 @@ struct HeapCheck {
 //
 struct HdrHeapSDKHandle {
 public:
-  HdrHeapSDKHandle() : m_heap(NULL) {}
+  HdrHeapSDKHandle() {}
   ~HdrHeapSDKHandle() { clear(); }
   // clear() only deallocates chained SDK return values
   //   The underlying MBuffer is left untouched
@@ -447,13 +473,12 @@ public:
   void set(const HdrHeapSDKHandle *from);
   const char *make_sdk_string(const char *raw_str, int raw_str_len);
 
-  HdrHeap *m_heap;
+  HdrHeap *m_heap = nullptr;
 
-private:
   // In order to prevent gratitous refcounting,
   //  automatic C++ copies are disabled!
-  HdrHeapSDKHandle(const HdrHeapSDKHandle &r);
-  HdrHeapSDKHandle &operator=(const HdrHeapSDKHandle &r);
+  HdrHeapSDKHandle(const HdrHeapSDKHandle &r) = delete;
+  HdrHeapSDKHandle &operator=(const HdrHeapSDKHandle &r) = delete;
 };
 
 inline void
@@ -468,7 +493,7 @@ HdrHeapSDKHandle::destroy()
 inline void
 HdrHeapSDKHandle::clear()
 {
-  m_heap = NULL;
+  m_heap = nullptr;
 }
 
 inline void
@@ -478,7 +503,7 @@ HdrHeapSDKHandle::set(const HdrHeapSDKHandle *from)
   m_heap = from->m_heap;
 }
 
-inkcoreapi HdrHeap *new_HdrHeap(int size = HDR_HEAP_DEFAULT_SIZE);
+HdrStrHeap *new_HdrStrHeap(int requested_size);
+inkcoreapi HdrHeap *new_HdrHeap(int size = HdrHeap::DEFAULT_SIZE);
 
 void hdr_heap_test();
-#endif
