@@ -45,7 +45,8 @@
   -------------------------------------------------------------------------*/
 
 ParseResult
-HTTPHdr::parse_req(HTTPParser *parser, IOBufferReader *r, int *bytes_used, bool eof, bool strict_uri_parsing)
+HTTPHdr::parse_req(HTTPParser *parser, IOBufferReader *r, int *bytes_used, bool eof, bool strict_uri_parsing,
+                   size_t max_request_line_size, size_t max_hdr_field_size)
 {
   const char *start;
   const char *tmp;
@@ -71,11 +72,12 @@ HTTPHdr::parse_req(HTTPParser *parser, IOBufferReader *r, int *bytes_used, bool
     int heap_slot = m_heap->attach_block(r->get_current_block(), start);
 
     m_heap->lock_ronly_str_heap(heap_slot);
-    state = http_parser_parse_req(parser, m_heap, m_http, &tmp, end, false, eof, strict_uri_parsing);
+    state = http_parser_parse_req(parser, m_heap, m_http, &tmp, end, false, eof, strict_uri_parsing, max_request_line_size,
+                                  max_hdr_field_size);
     m_heap->set_ronly_str_heap_end(heap_slot, tmp);
     m_heap->unlock_ronly_str_heap(heap_slot);
 
-    used = (int)(tmp - start);
+    used = static_cast<int>(tmp - start);
     r->consume(used);
     *bytes_used += used;
 
@@ -123,7 +125,7 @@ HTTPHdr::parse_resp(HTTPParser *parser, IOBufferReader *r, int *bytes_used, bool
     m_heap->set_ronly_str_heap_end(heap_slot, tmp);
     m_heap->unlock_ronly_str_heap(heap_slot);
 
-    used = (int)(tmp - start);
+    used = static_cast<int>(tmp - start);
     r->consume(used);
     *bytes_used += used;
 
@@ -150,7 +152,7 @@ HdrHeap::set_ronly_str_heap_end(int slot, const char *end)
   ink_assert(m_ronly_heap[slot].m_heap_start <= end);
   ink_assert(end <= m_ronly_heap[slot].m_heap_start + m_ronly_heap[slot].m_heap_len);
 
-  m_ronly_heap[slot].m_heap_len = (int)(end - m_ronly_heap[slot].m_heap_start);
+  m_ronly_heap[slot].m_heap_len = static_cast<int>(end - m_ronly_heap[slot].m_heap_start);
 }
 
 // void HdrHeap::attach_block(IOBufferBlock* b, const char* use_start)
@@ -170,30 +172,29 @@ HdrHeap::attach_block(IOBufferBlock *b, const char *use_start)
 
 RETRY:
 
-  // It's my contention that since heaps are add to the
-  //   first available slot, one you find an empty slot
-  //   it's not possible that a heap ptr for this block
-  //   exists in a later slot
-  for (int i = 0; i < HDR_BUF_RONLY_HEAPS; i++) {
-    if (m_ronly_heap[i].m_heap_start == nullptr) {
+  // It's my contention that since heaps are add to the first available slot, one you find an empty
+  // slot it's not possible that a heap ptr for this block exists in a later slot
+
+  for (auto &heap : m_ronly_heap) {
+    if (heap.m_heap_start == nullptr) {
       // Add block to heap in this slot
-      m_ronly_heap[i].m_heap_start    = (char *)use_start;
-      m_ronly_heap[i].m_heap_len      = (int)(b->end() - b->start());
-      m_ronly_heap[i].m_ref_count_ptr = b->data.object();
+      heap.m_heap_start    = static_cast<char const *>(use_start);
+      heap.m_heap_len      = static_cast<int>(b->end() - b->start());
+      heap.m_ref_count_ptr = b->data.object();
       //          printf("Attaching block at %X for %d in slot %d\n",
       //                 m_ronly_heap[i].m_heap_start,
       //                 m_ronly_heap[i].m_heap_len,
       //                 i);
-      return i;
-    } else if (m_ronly_heap[i].m_heap_start == b->buf()) {
+      return &heap - m_ronly_heap;
+    } else if (heap.m_heap_start == b->buf()) {
       // This block is already on the heap so just extend
       //   it's range
-      m_ronly_heap[i].m_heap_len = (int)(b->end() - b->buf());
+      heap.m_heap_len = static_cast<int>(b->end() - b->buf());
       //          printf("Extending block at %X to %d in slot %d\n",
       //                 m_ronly_heap[i].m_heap_start,
       //                 m_ronly_heap[i].m_heap_len,
       //                 i);
-      return i;
+      return &heap - m_ronly_heap;
     }
   }
 
