@@ -21,18 +21,16 @@
   limitations under the License.
  */
 
-#ifndef __URL_H__
-#define __URL_H__
+#pragma once
 
-#include "ts/Arena.h"
+#include "tscore/Arena.h"
 #include "HdrToken.h"
 #include "HdrHeap.h"
-#include "ts/INK_MD5.h"
-#include "ts/MMH.h"
+#include "tscore/CryptoHash.h"
 #include "MIME.h"
-#include "ts/TsBuffer.h"
+#include <string_view>
 
-#include "ts/ink_apidefs.h"
+#include "tscore/ink_apidefs.h"
 
 typedef int64_t cache_generation_t;
 
@@ -76,56 +74,22 @@ struct URLImpl : public HdrHeapObjImpl {
   // 6 bytes
 
   uint32_t m_clean : 1;
-  // 8 bytes + 1 bit, will result in padding
+  /// Whether the URI had an absolutely empty path, not even an initial '/'.
+  uint32_t m_path_is_empty : 1;
+  // 8 bytes + 2 bits, will result in padding
 
   // Marshaling Functions
   int marshal(MarshalXlate *str_xlate, int num_xlate);
   void unmarshal(intptr_t offset);
   void move_strings(HdrStrHeap *new_heap);
+  void rehome_strings(HdrHeap *new_heap);
   size_t strings_length();
 
   // Sanity Check Functions
   void check_strings(HeapCheck *heaps, int num_heaps);
 };
 
-/// Crypto Hash context for URLs.
-/// @internal This just forwards on to another specific context but avoids
-/// putting that switch logic in multiple places. The working context is put
-/// in to class local storage (@a _obj) via placement @a new.
-class URLHashContext : public CryptoContext
-{
-public:
-  URLHashContext();
-  /// Update the hash with @a data of @a length bytes.
-  virtual bool update(void const *data, int length);
-  /// Finalize and extract the @a hash.
-  virtual bool finalize(CryptoHash &hash);
-
-  enum HashType {
-    UNSPECIFIED,
-    MD5,
-    MMH,
-  }; ///< What type of hash we really are.
-  static HashType Setting;
-
-  /// Size of storage for placement @c new of hashing context.
-  static size_t const OBJ_SIZE = 256;
-
-protected:
-  char _obj[OBJ_SIZE]; ///< Raw storage for instantiated context.
-};
-
-inline bool
-URLHashContext::update(void const *data, int length)
-{
-  return reinterpret_cast<CryptoContext *>(_obj)->update(data, length);
-}
-
-inline bool
-URLHashContext::finalize(CryptoHash &hash)
-{
-  return reinterpret_cast<CryptoContext *>(_obj)->finalize(hash);
-}
+using URLHashContext = CryptoContext;
 
 extern const char *URL_SCHEME_FILE;
 extern const char *URL_SCHEME_FTP;
@@ -190,11 +154,8 @@ extern int URL_LEN_MMS;
 extern int URL_LEN_MMSU;
 extern int URL_LEN_MMST;
 
-/* Private */
-void url_adjust(MarshalXlate *str_xlate, int num_xlate);
-
 /* Public */
-bool validate_host_name(ts::ConstBuffer addr);
+bool validate_host_name(std::string_view addr);
 void url_init();
 
 URLImpl *url_create(HdrHeap *heap);
@@ -205,19 +166,18 @@ URLImpl *url_copy(URLImpl *s_url, HdrHeap *s_heap, HdrHeap *d_heap, bool inherit
 void url_copy_onto(URLImpl *s_url, HdrHeap *s_heap, URLImpl *d_url, HdrHeap *d_heap, bool inherit_strs = true);
 void url_copy_onto_as_server_url(URLImpl *s_url, HdrHeap *s_heap, URLImpl *d_url, HdrHeap *d_heap, bool inherit_strs = true);
 
-int url_print(URLImpl *u, char *buf, int bufsize, int *bufindex, int *dumpoffset);
+int url_print(URLImpl *u, char *buf, int bufsize, int *bufindex, int *dumpoffset, bool normalized = false);
 void url_describe(HdrHeapObjImpl *raw, bool recurse);
 
 int url_length_get(URLImpl *url);
-char *url_string_get(URLImpl *url, Arena *arena, int *length, HdrHeap *heap);
+char *url_string_get(URLImpl *url, Arena *arena, int *length, HdrHeap *heap, bool normalized = false);
 void url_clear_string_ref(URLImpl *url);
-char *url_string_get_ref(HdrHeap *heap, URLImpl *url, int *length);
+char *url_string_get_ref(HdrHeap *heap, URLImpl *url, int *length, bool normalized = false);
 void url_called_set(URLImpl *url);
 char *url_string_get_buf(URLImpl *url, char *dstbuf, int dstbuf_size, int *length);
 
-const char *url_scheme_get(URLImpl *url, int *length);
-void url_MD5_get(const URLImpl *url, CryptoHash *md5, cache_generation_t generation = -1);
-void url_host_MD5_get(URLImpl *url, CryptoHash *md5);
+void url_CryptoHash_get(const URLImpl *url, CryptoHash *hash, cache_generation_t generation = -1);
+void url_host_CryptoHash_get(URLImpl *url, CryptoHash *hash);
 const char *url_scheme_set(HdrHeap *heap, URLImpl *url, const char *value, int value_wks_idx, int length, bool copy_string);
 
 /* Internet specific */
@@ -236,14 +196,19 @@ void url_params_set(HdrHeap *heap, URLImpl *url, const char *value, int length,
 void url_query_set(HdrHeap *heap, URLImpl *url, const char *value, int length, bool copy_string);
 void url_fragment_set(HdrHeap *heap, URLImpl *url, const char *value, int length, bool copy_string);
 
+constexpr bool USE_STRICT_URI_PARSING = true;
+
 ParseResult url_parse(HdrHeap *heap, URLImpl *url, const char **start, const char *end, bool copy_strings,
-                      bool strict_uri_parsing = false);
-ParseResult url_parse_no_path_component_breakdown(HdrHeap *heap, URLImpl *url, const char **start, const char *end,
-                                                  bool copy_strings);
-ParseResult url_parse_internet(HdrHeap *heap, URLImpl *url, const char **start, const char *end, bool copy_strings);
-ParseResult url_parse_http(HdrHeap *heap, URLImpl *url, const char **start, const char *end, bool copy_strings);
-ParseResult url_parse_http_no_path_component_breakdown(HdrHeap *heap, URLImpl *url, const char **start, const char *end,
-                                                       bool copy_strings);
+                      bool strict_uri_parsing = false, bool verify_host_characters = true);
+
+constexpr bool COPY_STRINGS = true;
+
+ParseResult url_parse_regex(HdrHeap *heap, URLImpl *url, const char **start, const char *end, bool copy_strings);
+ParseResult url_parse_internet(HdrHeap *heap, URLImpl *url, const char **start, const char *end, bool copy_strings,
+                               bool verify_host_characters);
+ParseResult url_parse_http(HdrHeap *heap, URLImpl *url, const char **start, const char *end, bool copy_strings,
+                           bool verify_host_characters);
+ParseResult url_parse_http_regex(HdrHeap *heap, URLImpl *url, const char **start, const char *end, bool copy_strings);
 
 char *url_unescapify(Arena *arena, const char *str, int length);
 
@@ -265,7 +230,7 @@ url_canonicalize_port(int type, int port)
 class URL : public HdrHeapSDKHandle
 {
 public:
-  URLImpl *m_url_impl;
+  URLImpl *m_url_impl = nullptr;
 
   URL();
   ~URL();
@@ -280,17 +245,18 @@ public:
   // Note that URL::destroy() is inherited from HdrHeapSDKHandle.
   void nuke_proxy_stuff();
 
-  int print(char *buf, int bufsize, int *bufindex, int *dumpoffset);
+  int print(char *buf, int bufsize, int *bufindex, int *dumpoffset, bool normalized = false);
 
   int length_get();
   void clear_string_ref();
-  char *string_get(Arena *arena, int *length = NULL);
-  char *string_get_ref(int *length = NULL);
-  char *string_get_buf(char *dstbuf, int dsbuf_size, int *length = NULL);
-  void hash_get(CryptoHash *md5, cache_generation_t generation = -1) const;
-  void host_hash_get(CryptoHash *md5);
+  char *string_get(Arena *arena, int *length = nullptr, bool normalized = false);
+  char *string_get_ref(int *length = nullptr, bool normalized = false);
+  char *string_get_buf(char *dstbuf, int dsbuf_size, int *length = nullptr);
+  void hash_get(CryptoHash *hash, cache_generation_t generation = -1) const;
+  void host_hash_get(CryptoHash *hash);
 
   const char *scheme_get(int *length);
+  const std::string_view scheme_get();
   int scheme_get_wksidx();
   void scheme_set(const char *value, int length);
 
@@ -317,32 +283,59 @@ public:
   const char *fragment_get(int *length);
   void fragment_set(const char *value, int length);
 
+  /**
+   * Parse the given URL string and populate URL state with the parts.
+   *
+   * @param[in] url The URL to parse.
+   *
+   * @return PARSE_RESULT_DONE if parsing was successful, PARSE_RESULT_ERROR
+   * otherwise.
+   */
+  ParseResult parse(std::string_view url);
+
+  /** Same as parse() but do not verify that the host has proper FQDN
+   * characters.
+   *
+   * This is useful for RemapConfig To targets which have "$[0-9]" references
+   * in their host names which will later be substituted for other text.
+   */
+  ParseResult parse_no_host_check(std::string_view url);
+
   ParseResult parse(const char **start, const char *end);
   ParseResult parse(const char *str, int length);
-  ParseResult parse_no_path_component_breakdown(const char *str, int length);
+
+  /** Perform more simplified parsing that is resilient to receiving regular
+   * expressions.
+   *
+   * This simply looks for the first '/' in a URL and considers that the end of
+   * the authority and the beginning of the rest of the URL. This allows for
+   * the '?' character in an authority as a part of a regex without it being
+   * considered a query parameter and, thus, avoids confusing the parser.
+   *
+   * This is only used in RemapConfig and may have no other uses.
+   */
+  ParseResult parse_regex(std::string_view url);
+  ParseResult parse_regex(const char *str, int length);
 
 public:
   static char *unescapify(Arena *arena, const char *str, int length);
+  // No gratuitous copies!
+  URL(const URL &u) = delete;
+  URL &operator=(const URL &u) = delete;
 
 private:
-  // No gratuitous copies!
-  URL(const URL &u);
-  URL &operator=(const URL &u);
+  static constexpr bool VERIFY_HOST_CHARACTERS = true;
 };
 
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
 
-inline URL::URL() : m_url_impl(NULL)
-{
-}
+inline URL::URL() {}
 
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
 
-inline URL::~URL()
-{
-}
+inline URL::~URL() {}
 
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
@@ -374,7 +367,7 @@ URL::create(HdrHeap *heap)
 inline void
 URL::copy(const URL *url)
 {
-  ink_assert(url->valid());
+  ink_assert(url != nullptr && url->valid());
   url_copy_onto(url->m_url_impl, url->m_heap, m_url_impl, m_heap);
 }
 
@@ -395,14 +388,14 @@ URL::copy_shallow(const URL *url)
 inline void
 URL::clear()
 {
-  m_url_impl = NULL;
+  m_url_impl = nullptr;
   HdrHeapSDKHandle::clear();
 }
 
 inline void
 URL::reset()
 {
-  m_url_impl = NULL;
+  m_url_impl = nullptr;
 }
 
 /*-------------------------------------------------------------------------
@@ -419,10 +412,10 @@ URL::nuke_proxy_stuff()
   -------------------------------------------------------------------------*/
 
 inline int
-URL::print(char *buf, int bufsize, int *bufindex, int *dumpoffset)
+URL::print(char *buf, int bufsize, int *bufindex, int *dumpoffset, bool normalized)
 {
   ink_assert(valid());
-  return url_print(m_url_impl, buf, bufsize, bufindex, dumpoffset);
+  return url_print(m_url_impl, buf, bufsize, bufindex, dumpoffset, normalized);
 }
 
 /*-------------------------------------------------------------------------
@@ -439,17 +432,17 @@ URL::length_get()
   -------------------------------------------------------------------------*/
 
 inline char *
-URL::string_get(Arena *arena_or_null_for_malloc, int *length)
+URL::string_get(Arena *arena_or_null_for_malloc, int *length, bool normalized)
 {
   ink_assert(valid());
-  return url_string_get(m_url_impl, arena_or_null_for_malloc, length, m_heap);
+  return url_string_get(m_url_impl, arena_or_null_for_malloc, length, m_heap, normalized);
 }
 
 inline char *
-URL::string_get_ref(int *length)
+URL::string_get_ref(int *length, bool normalized)
 {
   ink_assert(valid());
-  return url_string_get_ref(m_heap, m_url_impl, length);
+  return url_string_get_ref(m_heap, m_url_impl, length, normalized);
 }
 
 inline void
@@ -473,30 +466,44 @@ URL::string_get_buf(char *dstbuf, int dsbuf_size, int *length)
   -------------------------------------------------------------------------*/
 
 inline void
-URL::hash_get(CryptoHash *md5, cache_generation_t generation) const
+URL::hash_get(CryptoHash *hash, cache_generation_t generation) const
 {
   ink_assert(valid());
-  url_MD5_get(m_url_impl, md5, generation);
+  url_CryptoHash_get(m_url_impl, hash, generation);
 }
 
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
 
 inline void
-URL::host_hash_get(CryptoHash *md5)
+URL::host_hash_get(CryptoHash *hash)
 {
   ink_assert(valid());
-  url_host_MD5_get(m_url_impl, md5);
+  url_host_CryptoHash_get(m_url_impl, hash);
 }
 
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
 
+inline const std::string_view
+URL::scheme_get()
+{
+  ink_assert(valid());
+
+  if (m_url_impl->m_scheme_wks_idx >= 0) {
+    return std::string_view{hdrtoken_index_to_wks(m_url_impl->m_scheme_wks_idx),
+                            static_cast<size_t>(hdrtoken_index_to_length(m_url_impl->m_scheme_wks_idx))};
+  } else {
+    return std::string_view{m_url_impl->m_ptr_scheme, m_url_impl->m_len_scheme};
+  }
+}
+
 inline const char *
 URL::scheme_get(int *length)
 {
-  ink_assert(valid());
-  return (url_scheme_get(m_url_impl, length));
+  std::string_view ret = this->scheme_get();
+  *length              = ret.size();
+  return ret.data();
 }
 
 inline int
@@ -714,6 +721,31 @@ URL::fragment_set(const char *value, int length)
   url_fragment_set(m_heap, m_url_impl, value, length, true);
 }
 
+/**
+  Parser doesn't clear URL first, so if you parse over a non-clear URL,
+  the resulting URL may contain some of the previous data.
+
+ */
+inline ParseResult
+URL::parse(std::string_view url)
+{
+  return this->parse(url.data(), static_cast<int>(url.size()));
+}
+
+/**
+  Parser doesn't clear URL first, so if you parse over a non-clear URL,
+  the resulting URL may contain some of the previous data.
+
+ */
+inline ParseResult
+URL::parse_no_host_check(std::string_view url)
+{
+  ink_assert(valid());
+  const char *start = url.data();
+  const char *end   = url.data() + url.length();
+  return url_parse(m_heap, m_url_impl, &start, end, COPY_STRINGS, !USE_STRICT_URI_PARSING, !VERIFY_HOST_CHARACTERS);
+}
+
 /**
   Parser doesn't clear URL first, so if you parse over a non-clear URL,
   the resulting URL may contain some of the previous data.
@@ -723,7 +755,7 @@ inline ParseResult
 URL::parse(const char **start, const char *end)
 {
   ink_assert(valid());
-  return url_parse(m_heap, m_url_impl, start, end, true);
+  return url_parse(m_heap, m_url_impl, start, end, COPY_STRINGS);
 }
 
 /**
@@ -746,13 +778,26 @@ URL::parse(const char *str, int length)
 
  */
 inline ParseResult
-URL::parse_no_path_component_breakdown(const char *str, int length)
+URL::parse_regex(std::string_view url)
+{
+  ink_assert(valid());
+  const char *str = url.data();
+  return url_parse_regex(m_heap, m_url_impl, &str, str + url.length(), COPY_STRINGS);
+}
+
+/**
+  Parser doesn't clear URL first, so if you parse over a non-clear URL,
+  the resulting URL may contain some of the previous data.
+
+ */
+inline ParseResult
+URL::parse_regex(const char *str, int length)
 {
   ink_assert(valid());
   if (length < 0)
     length = (int)strlen(str);
   ink_assert(valid());
-  return url_parse_no_path_component_breakdown(m_heap, m_url_impl, &str, str + length, true);
+  return url_parse_regex(m_heap, m_url_impl, &str, str + length, COPY_STRINGS);
 }
 
 /*-------------------------------------------------------------------------
@@ -763,5 +808,3 @@ URL::unescapify(Arena *arena, const char *str, int length)
 {
   return url_unescapify(arena, str, length);
 }
-
-#endif /* __URL_H__ */
