@@ -30,20 +30,20 @@
 
  ****************************************************************************/
 
-#include <ts/ink_resolver.h>
+#include "tscore/ink_resolver.h"
 #include "Http1ClientSession.h"
-#include "Http1ClientTransaction.h"
+#include "Http1Transaction.h"
 #include "HttpSM.h"
 #include "HttpDebugNames.h"
-#include "HttpServerSession.h"
+#include "Http1ServerSession.h"
 #include "Plugin.h"
 
-#define DebugHttpSsn(fmt, ...) DebugSsn(this, "http_cs", fmt, __VA_ARGS__)
+#define HttpSsnDebug(fmt, ...) SsnDebug(this, "http_cs", fmt, __VA_ARGS__)
 
 #define STATE_ENTER(state_name, event, vio)                                                             \
   do {                                                                                                  \
     /*ink_assert (magic == HTTP_SM_MAGIC_ALIVE);  REMEMBER (event, NULL, reentrancy_count); */          \
-    DebugHttpSsn("[%" PRId64 "] [%s, %s]", con_id, #state_name, HttpDebugNames::get_event_name(event)); \
+    HttpSsnDebug("[%" PRId64 "] [%s, %s]", con_id, #state_name, HttpDebugNames::get_event_name(event)); \
   } while (0)
 
 enum {
@@ -62,25 +62,7 @@ ink_mutex debug_cs_list_mutex;
 
 ClassAllocator<Http1ClientSession> http1ClientSessionAllocator("http1ClientSessionAllocator");
 
-Http1ClientSession::Http1ClientSession()
-  : client_vc(nullptr),
-    magic(HTTP_CS_MAGIC_DEAD),
-    transact_count(0),
-    tcp_init_cwnd_set(false),
-    half_close(false),
-    conn_decrease(false),
-    read_buffer(nullptr),
-    sm_reader(nullptr),
-    read_state(HCS_INIT),
-    ka_vio(nullptr),
-    slave_ka_vio(nullptr),
-    bound_ss(nullptr),
-    released_transactions(0),
-    outbound_port(0),
-    f_outbound_transparent(false),
-    f_transparent_passthrough(false)
-{
-}
+Http1ClientSession::Http1ClientSession() {}
 
 void
 Http1ClientSession::destroy()
@@ -91,8 +73,7 @@ Http1ClientSession::destroy()
   if (!in_destroy) {
     in_destroy = true;
 
-    DebugHttpSsn("[%" PRId64 "] session destroy", con_id);
-    ink_release_assert(!client_vc);
+    HttpSsnDebug("[%" PRId64 "] session destroy", con_id);
     ink_assert(read_buffer);
     ink_release_assert(transact_count == released_transactions);
     do_api_callout(TS_HTTP_SSN_CLOSE_HOOK);
@@ -106,7 +87,14 @@ Http1ClientSession::release_transaction()
 {
   released_transactions++;
   if (transact_count == released_transactions) {
-    destroy();
+    // Make sure we previously called release() or do_io_close() on the session
+    ink_release_assert(read_state != HCS_INIT);
+    if (read_state == HCS_ACTIVE_READER) {
+      // (in)active timeout
+      do_io_close(HTTP_ERRNO);
+    } else {
+      destroy();
+    }
   }
 }
 
@@ -130,15 +118,12 @@ Http1ClientSession::free()
     conn_decrease = false;
   }
 
-  // Clean up the write VIO in case of inactivity timeout
-  this->do_io_write(nullptr, 0, nullptr);
-
   // Free the transaction resources
-  this->trans.super::destroy();
+  this->trans.super_type::destroy();
 
-  if (client_vc) {
-    client_vc->do_io_close();
-    client_vc = nullptr;
+  if (_vc) {
+    _vc->do_io_close();
+    _vc = nullptr;
   }
 
   super::free();
@@ -146,25 +131,28 @@ Http1ClientSession::free()
 }
 
 void
-Http1ClientSession::new_connection(NetVConnection *new_vc, MIOBuffer *iobuf, IOBufferReader *reader, bool backdoor)
+Http1ClientSession::new_connection(NetVConnection *new_vc, MIOBuffer *iobuf, IOBufferReader *reader)
 {
   ink_assert(new_vc != nullptr);
-  ink_assert(client_vc == nullptr);
-  client_vc      = new_vc;
+  ink_assert(_vc == nullptr);
+  _vc            = new_vc;
   magic          = HTTP_CS_MAGIC_ALIVE;
   mutex          = new_vc->mutex;
   trans.mutex    = mutex; // Share this mutex with the transaction
   ssn_start_time = Thread::get_hrtime();
   in_destroy     = false;
 
+  SSLNetVConnection *ssl_vc = dynamic_cast<SSLNetVConnection *>(new_vc);
+  if (ssl_vc != nullptr) {
+    read_from_early_data = ssl_vc->read_from_early_data;
+    Debug("ssl_early_data", "read_from_early_data = %" PRId64, read_from_early_data);
+  }
+
   MUTEX_TRY_LOCK(lock, mutex, this_ethread());
   ink_assert(lock.is_locked());
 
-  // Disable hooks for backdoor connections.
-  this->hooks_on = !backdoor;
-
   // Unique client session identifier.
-  con_id = ProxyClientSession::next_connection_id();
+  con_id = ProxySession::next_connection_id();
 
   schedule_event = nullptr;
 
@@ -202,13 +190,15 @@ Http1ClientSession::new_connection(NetVConnection *new_vc, MIOBuffer *iobuf, IOB
   ink_mutex_release(&debug_cs_list_mutex);
 #endif
 
-  DebugHttpSsn("[%" PRId64 "] session born, netvc %p", con_id, new_vc);
+  HttpSsnDebug("[%" PRId64 "] session born, netvc %p", con_id, new_vc);
 
-  client_vc->set_tcp_congestion_control(CLIENT_SIDE);
+  _vc->set_tcp_congestion_control(CLIENT_SIDE);
 
   read_buffer = iobuf ? iobuf : new_MIOBuffer(HTTP_HEADER_BUFFER_SIZE_INDEX);
-  sm_reader   = reader ? reader : read_buffer->alloc_reader();
-  trans.set_reader(sm_reader);
+  _reader     = reader ? reader : read_buffer->alloc_reader();
+  trans.set_reader(_reader);
+
+  _handle_if_ssl(new_vc);
 
   // INKqa11186: Use a local pointer to the mutex as
   // when we return from do_api_callout, the ClientSession may
@@ -221,55 +211,12 @@ Http1ClientSession::new_connection(NetVConnection *new_vc, MIOBuffer *iobuf, IOB
   lmutex.clear();
 }
 
-VIO *
-Http1ClientSession::do_io_read(Continuation *c, int64_t nbytes, MIOBuffer *buf)
-{
-  return (client_vc) ? client_vc->do_io_read(c, nbytes, buf) : nullptr;
-}
-
-VIO *
-Http1ClientSession::do_io_write(Continuation *c, int64_t nbytes, IOBufferReader *buf, bool owner)
-{
-  /* conditionally set the tcp initial congestion window
-     before our first write. */
-  DebugHttpSsn("tcp_init_cwnd_set %d", (int)tcp_init_cwnd_set);
-  if (!tcp_init_cwnd_set) {
-    tcp_init_cwnd_set = true;
-    set_tcp_init_cwnd();
-  }
-  if (client_vc) {
-    return client_vc->do_io_write(c, nbytes, buf, owner);
-  } else {
-    return nullptr;
-  }
-}
-
-void
-Http1ClientSession::set_tcp_init_cwnd()
-{
-  if (!trans.get_sm())
-    return;
-  int desired_tcp_init_cwnd = trans.get_sm()->t_state.txn_conf->server_tcp_init_cwnd;
-  DebugHttpSsn("desired TCP congestion window is %d", desired_tcp_init_cwnd);
-  if (desired_tcp_init_cwnd == 0) {
-    return;
-  }
-  if (get_netvc()->set_tcp_init_cwnd(desired_tcp_init_cwnd) != 0) {
-    DebugHttpSsn("set_tcp_init_cwnd(%d) failed", desired_tcp_init_cwnd);
-  }
-}
-
-void
-Http1ClientSession::do_io_shutdown(ShutdownHowTo_t howto)
-{
-  client_vc->do_io_shutdown(howto);
-}
-
 void
 Http1ClientSession::do_io_close(int alerrno)
 {
-  if (read_state == HCS_CLOSED)
+  if (read_state == HCS_CLOSED) {
     return; // Don't double call session close
+  }
   if (read_state == HCS_ACTIVE_READER) {
     clear_session_active();
   }
@@ -285,45 +232,42 @@ Http1ClientSession::do_io_close(int alerrno)
     slave_ka_vio = nullptr;
   }
   // Completed the last transaction.  Just shutdown already
-  if (transact_count == released_transactions) {
+  // Or the do_io_close is due to a network error
+  if (transact_count == released_transactions || alerrno == HTTP_ERRNO) {
     half_close = false;
   }
+
   if (half_close && this->trans.get_sm()) {
     read_state = HCS_HALF_CLOSED;
     SET_HANDLER(&Http1ClientSession::state_wait_for_close);
-    DebugHttpSsn("[%" PRId64 "] session half close", con_id);
+    HttpSsnDebug("[%" PRId64 "] session half close", con_id);
 
-    if (client_vc) {
-      // We want the client to know that that we're finished
-      //  writing.  The write shutdown accomplishes this.  Unfortuantely,
-      //  the IO Core semantics don't stop us from getting events
-      //  on the write side of the connection like timeouts so we
-      //  need to zero out the write of the continuation with
-      //  the do_io_write() call (INKqa05309)
-      client_vc->do_io_shutdown(IO_SHUTDOWN_WRITE);
+    if (_vc) {
+      _vc->do_io_shutdown(IO_SHUTDOWN_WRITE);
 
-      ka_vio = client_vc->do_io_read(this, INT64_MAX, read_buffer);
+      ka_vio = _vc->do_io_read(this, INT64_MAX, read_buffer);
       ink_assert(slave_ka_vio != ka_vio);
 
       // Set the active timeout to the same as the inactive time so
       //   that this connection does not hang around forever if
       //   the ua hasn't closed
-      client_vc->set_active_timeout(HRTIME_SECONDS(trans.get_sm()->t_state.txn_conf->keep_alive_no_activity_timeout_in));
+      _vc->set_active_timeout(HRTIME_SECONDS(trans.get_sm()->t_state.txn_conf->keep_alive_no_activity_timeout_in));
     }
 
     // [bug 2610799] Drain any data read.
     // If the buffer is full and the client writes again, we will not receive a
     // READ_READY event.
-    sm_reader->consume(sm_reader->read_avail());
+    _reader->consume(_reader->read_avail());
   } else {
-    read_state = HCS_CLOSED;
-    DebugHttpSsn("[%" PRId64 "] session closed", con_id);
+    HttpSsnDebug("[%" PRId64 "] session closed", con_id);
     HTTP_SUM_DYN_STAT(http_transactions_per_client_con, transact_count);
-    HTTP_DECREMENT_DYN_STAT(http_current_client_connections_stat);
-    conn_decrease = false;
-    if (client_vc) {
-      client_vc->do_io_close();
-      client_vc = nullptr;
+    read_state = HCS_CLOSED;
+
+    // Can go ahead and close the netvc now, but keeping around the session object
+    // until all the transactions are closed
+    if (_vc) {
+      _vc->do_io_close();
+      _vc = nullptr;
     }
   }
   if (transact_count == released_transactions) {
@@ -350,11 +294,11 @@ Http1ClientSession::state_wait_for_close(int event, void *data)
   case VC_EVENT_ACTIVE_TIMEOUT:
   case VC_EVENT_INACTIVITY_TIMEOUT:
     half_close = false;
-    this->do_io_close();
+    this->do_io_close(EHTTP_ERROR);
     break;
   case VC_EVENT_READ_READY:
     // Drain any data read
-    sm_reader->consume(sm_reader->read_avail());
+    _reader->consume(_reader->read_avail());
     break;
 
   default:
@@ -407,7 +351,7 @@ Http1ClientSession::state_slave_keep_alive(int event, void *data)
 int
 Http1ClientSession::state_keep_alive(int event, void *data)
 {
-  // Route the event.  It is either for client vc or
+  // Route the event.  It is either for vc or
   //  the origin server slave vc
   if (data && data == slave_ka_vio) {
     return state_slave_keep_alive(event, data);
@@ -426,7 +370,7 @@ Http1ClientSession::state_keep_alive(int event, void *data)
     break;
 
   case VC_EVENT_EOS:
-    this->do_io_close();
+    this->do_io_close(EHTTP_ERROR);
     break;
 
   case VC_EVENT_READ_COMPLETE:
@@ -438,48 +382,52 @@ Http1ClientSession::state_keep_alive(int event, void *data)
   case VC_EVENT_ACTIVE_TIMEOUT:
   case VC_EVENT_INACTIVITY_TIMEOUT:
     // Keep-alive timed out
-    this->do_io_close();
+    this->do_io_close(EHTTP_ERROR);
     break;
   }
 
   return 0;
 }
-void
-Http1ClientSession::reenable(VIO *vio)
-{
-  client_vc->reenable(vio);
-}
 
-// Called from the Http1ClientTransaction::release
+// Called from the Http1Transaction::release
 void
-Http1ClientSession::release(ProxyClientTransaction *trans)
+Http1ClientSession::release(ProxyTransaction *trans)
 {
   ink_assert(read_state == HCS_ACTIVE_READER || read_state == HCS_INIT);
 
-  // Clean up the write VIO in case of inactivity timeout
-  this->do_io_write(nullptr, 0, nullptr);
+  // When release is called from start() to read the first transaction, get_sm()
+  // will return null.
+  HttpSM *sm = trans->get_sm();
+  if (sm) {
+    MgmtInt ka_in = trans->get_sm()->t_state.txn_conf->keep_alive_no_activity_timeout_in;
+    set_inactivity_timeout(HRTIME_SECONDS(ka_in));
+
+    this->clear_session_active();
+    this->ssn_last_txn_time = Thread::get_hrtime();
+
+    // Timeout events should be delivered to the session
+    this->do_io_write(this, 0, nullptr);
+  }
 
   // Check to see there is remaining data in the
   //  buffer.  If there is, spin up a new state
   //  machine to process it.  Otherwise, issue an
   //  IO to wait for new data
-  bool more_to_read = this->sm_reader->is_read_avail_more_than(0);
+  bool more_to_read = this->_reader->is_read_avail_more_than(0);
   if (more_to_read) {
     trans->destroy();
-    trans->set_restart_immediate(true);
-    DebugHttpSsn("[%" PRId64 "] data already in buffer, starting new transaction", con_id);
+    HttpSsnDebug("[%" PRId64 "] data already in buffer, starting new transaction", con_id);
     new_transaction();
   } else {
-    DebugHttpSsn("[%" PRId64 "] initiating io for next header", con_id);
-    trans->set_restart_immediate(false);
+    HttpSsnDebug("[%" PRId64 "] initiating io for next header", con_id);
     read_state = HCS_KEEP_ALIVE;
     SET_HANDLER(&Http1ClientSession::state_keep_alive);
     ka_vio = this->do_io_read(this, INT64_MAX, read_buffer);
     ink_assert(slave_ka_vio != ka_vio);
 
-    if (client_vc) {
-      client_vc->cancel_active_timeout();
-      client_vc->add_to_keep_alive_queue();
+    if (_vc) {
+      _vc->cancel_active_timeout();
+      _vc->add_to_keep_alive_queue();
     }
     trans->destroy();
   }
@@ -489,32 +437,37 @@ void
 Http1ClientSession::new_transaction()
 {
   // If the client connection terminated during API callouts we're done.
-  if (nullptr == client_vc) {
+  if (nullptr == _vc) {
     this->do_io_close(); // calls the SSN_CLOSE hooks to match the SSN_START hooks.
     return;
   }
 
+  if (!_vc->add_to_active_queue()) {
+    // no room in the active queue close the connection
+    this->do_io_close();
+    return;
+  }
+
   // Defensive programming, make sure nothing persists across
   // connection re-use
   half_close = false;
 
   read_state = HCS_ACTIVE_READER;
 
-  trans.set_parent(this);
+  trans.set_proxy_ssn(this);
   transact_count++;
 
-  client_vc->add_to_active_queue();
-  trans.new_transaction();
+  trans.new_transaction(read_from_early_data > 0 ? true : false);
 }
 
-void
-Http1ClientSession::attach_server_session(HttpServerSession *ssession, bool transaction_done)
+bool
+Http1ClientSession::attach_server_session(Http1ServerSession *ssession, bool transaction_done)
 {
   if (ssession) {
     ink_assert(bound_ss == nullptr);
     ssession->state = HSS_KA_CLIENT_SLAVE;
     bound_ss        = ssession;
-    DebugHttpSsn("[%" PRId64 "] attaching server session [%" PRId64 "] as slave", con_id, ssession->con_id);
+    HttpSsnDebug("[%" PRId64 "] attaching server session [%" PRId64 "] as slave", con_id, ssession->con_id);
     ink_assert(ssession->get_reader()->read_avail() == 0);
     ink_assert(ssession->get_netvc() != this->get_netvc());
 
@@ -526,6 +479,7 @@ Http1ClientSession::attach_server_session(HttpServerSession *ssession, bool tran
     //  the server net conneciton from calling back a dead sm
     SET_HANDLER(&Http1ClientSession::state_keep_alive);
     slave_ka_vio = ssession->do_io_read(this, INT64_MAX, ssession->read_buffer);
+    this->do_io_write(this, 0, nullptr); // Capture the inactivity timeouts
     ink_assert(slave_ka_vio != ka_vio);
 
     // Transfer control of the write side as well
@@ -545,4 +499,73 @@ Http1ClientSession::attach_server_session(HttpServerSession *ssession, bool tran
     bound_ss     = nullptr;
     slave_ka_vio = nullptr;
   }
+  return true;
+}
+
+void
+Http1ClientSession::increment_current_active_client_connections_stat()
+{
+  HTTP_INCREMENT_DYN_STAT(http_current_active_client_connections_stat);
+}
+
+void
+Http1ClientSession::decrement_current_active_client_connections_stat()
+{
+  HTTP_DECREMENT_DYN_STAT(http_current_active_client_connections_stat);
+}
+
+void
+Http1ClientSession::start()
+{
+  // Troll for data to get a new transaction
+  this->release(&trans);
+}
+
+bool
+Http1ClientSession::allow_half_open() const
+{
+  // Only allow half open connections if the not over TLS
+  return (_vc && dynamic_cast<SSLNetVConnection *>(_vc) == nullptr);
+}
+
+void
+Http1ClientSession::set_half_close_flag(bool flag)
+{
+  half_close = flag;
+}
+
+bool
+Http1ClientSession::get_half_close_flag() const
+{
+  return half_close;
+}
+
+bool
+Http1ClientSession::is_chunked_encoding_supported() const
+{
+  return true;
+}
+
+int
+Http1ClientSession::get_transact_count() const
+{
+  return transact_count;
+}
+
+bool
+Http1ClientSession::is_outbound_transparent() const
+{
+  return f_outbound_transparent;
+}
+
+Http1ServerSession *
+Http1ClientSession::get_server_session() const
+{
+  return bound_ss;
+}
+
+const char *
+Http1ClientSession::get_protocol_string() const
+{
+  return "http";
 }
