@@ -28,20 +28,20 @@
 
  ****************************************************************************/
 
-#include "ts/ink_platform.h"
-#include "ts/ink_sprintf.h"
-#include "ts/ink_file.h"
+#include "tscore/ink_platform.h"
+#include "tscore/ink_sprintf.h"
+#include "tscore/ink_file.h"
+#include "tscore/Filenames.h"
 #include "HttpBodyFactory.h"
 #include <unistd.h>
 #include <dirent.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include "URL.h"
-#include <logging/Log.h>
-#include <logging/LogAccess.h>
-#include <logging/LogAccessHttp.h>
+#include "logging/Log.h"
+#include "logging/LogAccess.h"
 #include "HttpCompat.h"
-#include "ts/I_Layout.h"
+#include "tscore/I_Layout.h"
 
 //////////////////////////////////////////////////////////////////////
 // The HttpBodyFactory creates HTTP response page bodies, supported //
@@ -63,16 +63,26 @@ char *
 HttpBodyFactory::fabricate_with_old_api(const char *type, HttpTransact::State *context, int64_t max_buffer_length,
                                         int64_t *resulting_buffer_length, char *content_language_out_buf,
                                         size_t content_language_buf_size, char *content_type_out_buf, size_t content_type_buf_size,
-                                        const char *format, va_list ap)
+                                        int format_size, const char *format)
 {
-  char *buffer            = NULL;
-  const char *lang_ptr    = NULL;
-  const char *charset_ptr = NULL;
+  char *buffer            = nullptr;
+  const char *lang_ptr    = nullptr;
+  const char *charset_ptr = nullptr;
   char url[1024];
-  const char *set               = NULL;
+  const char *set               = nullptr;
   bool found_requested_template = false;
   bool plain_flag               = false;
 
+  ///////////////////////////////////////////////
+  // if suppressing this response, return NULL //
+  ///////////////////////////////////////////////
+  if (is_response_suppressed(context)) {
+    if (enable_logging) {
+      Log::error("BODY_FACTORY: suppressing '%s' response for url '%s'", type, url);
+    }
+    return nullptr;
+  }
+
   lock();
 
   *resulting_buffer_length = 0;
@@ -99,16 +109,6 @@ HttpBodyFactory::fabricate_with_old_api(const char *type, HttpTransact::State *c
       }
     }
   }
-  ///////////////////////////////////////////////
-  // if suppressing this response, return NULL //
-  ///////////////////////////////////////////////
-  if (is_response_suppressed(context)) {
-    if (enable_logging) {
-      Log::error("BODY_FACTORY: suppressing '%s' response for url '%s'", type, url);
-    }
-    unlock();
-    return (NULL);
-  }
   //////////////////////////////////////////////////////////////////////////////////
   // if language-targeting activated, get client Accept-Language & Accept-Charset //
   //////////////////////////////////////////////////////////////////////////////////
@@ -124,28 +124,29 @@ HttpBodyFactory::fabricate_with_old_api(const char *type, HttpTransact::State *c
   ///////////////////////////////////////////
   // check if we don't need to format body //
   ///////////////////////////////////////////
-  if (format) {
-    // The length from ink_bvsprintf includes the trailing NUL, so adjust the final
-    // length accordingly.
-    int l = ink_bvsprintf(NULL, format, ap);
-    if (l <= max_buffer_length) {
-      buffer                   = (char *)ats_malloc(l);
-      *resulting_buffer_length = ink_bvsprintf(buffer, format, ap) - 1;
-      plain_flag               = true;
-    }
+
+  buffer = (format == nullptr) ? nullptr : ats_strndup(format, format_size);
+  if (buffer != nullptr && format_size > 0) {
+    *resulting_buffer_length = format_size > max_buffer_length ? 0 : format_size;
+    plain_flag               = true;
   }
   /////////////////////////////////////////////////////////
   // try to fabricate the desired type of error response //
   /////////////////////////////////////////////////////////
-  if (buffer == NULL) {
+  if (buffer == nullptr) {
     buffer =
       fabricate(&acpt_language_list, &acpt_charset_list, type, context, resulting_buffer_length, &lang_ptr, &charset_ptr, &set);
-    found_requested_template = (buffer != NULL);
+    found_requested_template = (buffer != nullptr);
   }
   /////////////////////////////////////////////////////////////
   // if failed, try to fabricate the default custom response //
   /////////////////////////////////////////////////////////////
-  if (buffer == NULL) {
+  if (buffer == nullptr) {
+    if (is_response_body_precluded(context->http_return_code)) {
+      *resulting_buffer_length = 0;
+      unlock();
+      return nullptr;
+    }
     buffer = fabricate(&acpt_language_list, &acpt_charset_list, "default", context, resulting_buffer_length, &lang_ptr,
                        &charset_ptr, &set);
   }
@@ -160,13 +161,12 @@ HttpBodyFactory::fabricate_with_old_api(const char *type, HttpTransact::State *c
                  set, type, *resulting_buffer_length, max_buffer_length);
     }
     *resulting_buffer_length = 0;
-    buffer                   = (char *)ats_free_null(buffer);
+    buffer                   = static_cast<char *>(ats_free_null(buffer));
   }
   /////////////////////////////////////////////////////////////////////
   // handle return of instantiated template and generate the content //
   // language and content type return values                         //
   /////////////////////////////////////////////////////////////////////
-
   if (buffer) { // got an instantiated template
     if (!plain_flag) {
       snprintf(content_language_out_buf, content_language_buf_size, "%s", lang_ptr);
@@ -191,50 +191,29 @@ HttpBodyFactory::fabricate_with_old_api(const char *type, HttpTransact::State *c
   }
   unlock();
 
-  return (buffer);
+  return buffer;
 }
 
 void
 HttpBodyFactory::dump_template_tables(FILE *fp)
 {
-  RawHashTable *h1, *h2;
-  RawHashTable_Key k1, k2;
-  RawHashTable_Value v1, v2;
-  RawHashTable_Binding *b1, *b2;
-  RawHashTable_IteratorState i1, i2;
-  HttpBodySet *body_set;
-
   lock();
-
-  h1 = table_of_sets;
-
-  if (h1 != NULL) {
-    ///////////////////////////////////////////
-    // loop over set->body-types hash table //
-    ///////////////////////////////////////////
-
-    for (b1 = h1->firstBinding(&i1); b1 != NULL; b1 = h1->nextBinding(&i1)) {
-      k1       = table_of_sets->getKeyFromBinding(b1);
-      v1       = table_of_sets->getValueFromBinding(b1);
-      body_set = (HttpBodySet *)v1;
-
-      if (body_set != NULL) {
-        fprintf(fp, "set %s: name '%s', lang '%s', charset '%s'\n", k1, body_set->set_name, body_set->content_language,
-                body_set->content_charset);
+  if (table_of_sets) {
+    for (const auto &it1 : *table_of_sets.get()) {
+      HttpBodySet *body_set = static_cast<HttpBodySet *>(it1.second);
+      if (body_set) {
+        fprintf(fp, "set %s: name '%s', lang '%s', charset '%s'\n", it1.first.c_str(), body_set->set_name,
+                body_set->content_language, body_set->content_charset);
 
         ///////////////////////////////////////////
         // loop over body-types->body hash table //
         ///////////////////////////////////////////
 
         ink_assert(body_set->is_sane());
-        h2 = body_set->table_of_pages;
-
-        for (b2 = h2->firstBinding(&i2); b2 != NULL; b2 = h2->nextBinding(&i2)) {
-          k2                  = table_of_sets->getKeyFromBinding(b2);
-          v2                  = table_of_sets->getValueFromBinding(b2);
-          HttpBodyTemplate *t = (HttpBodyTemplate *)v2;
-
-          fprintf(fp, "  %-30s: %" PRId64 " bytes\n", k2, t->byte_count);
+        if (body_set->table_of_pages) {
+          for (const auto &it2 : *body_set->table_of_pages.get()) {
+            fprintf(fp, "  %-30s: %" PRId64 " bytes\n", it2.first.c_str(), it2.second->byte_count);
+          }
         }
       }
     }
@@ -253,17 +232,16 @@ static int
 config_callback(const char * /* name ATS_UNUSED */, RecDataT /* data_type ATS_UNUSED */, RecData /* data ATS_UNUSED */,
                 void *cookie)
 {
-  HttpBodyFactory *body_factory = (HttpBodyFactory *)cookie;
+  HttpBodyFactory *body_factory = static_cast<HttpBodyFactory *>(cookie);
   body_factory->reconfigure();
-  return (0);
+  return 0;
 }
 
 void
 HttpBodyFactory::reconfigure()
-//#endif
 {
   RecInt e;
-  RecString s = NULL;
+  RecString s = nullptr;
   bool all_found;
   int rec_err;
 
@@ -311,7 +289,8 @@ HttpBodyFactory::reconfigure()
     }
   }
 
-  Debug("body_factory", "directory_of_template_sets = '%s' (found = %s)", (const char *)directory_of_template_sets, s);
+  Debug("body_factory", "directory_of_template_sets = '%s' ", (const char *)directory_of_template_sets);
+
   ats_free(s);
 
   if (!all_found) {
@@ -350,14 +329,7 @@ HttpBodyFactory::HttpBodyFactory()
   ////////////////////////////////////
   // initialize first-time defaults //
   ////////////////////////////////////
-
-  magic = HTTP_BODY_FACTORY_MAGIC;
-  ink_mutex_init(&mutex, "HttpBodyFactory::lock");
-
-  table_of_sets         = NULL;
-  enable_customizations = 0;
-  enable_logging        = true;
-  callbacks_established = false;
+  ink_mutex_init(&mutex);
 
   //////////////////////////////////////////////////////
   // set up management configuration-change callbacks //
@@ -365,13 +337,13 @@ HttpBodyFactory::HttpBodyFactory()
 
   static const char *config_record_names[] = {
     "proxy.config.body_factory.enable_customizations", "proxy.config.body_factory.enable_logging",
-    "proxy.config.body_factory.template_sets_dir", "proxy.config.body_factory.response_suppression_mode", NULL};
+    "proxy.config.body_factory.template_sets_dir", "proxy.config.body_factory.response_suppression_mode", nullptr};
 
   no_registrations_failed = true;
-  for (i = 0; config_record_names[i] != NULL; i++) {
+  for (i = 0; config_record_names[i] != nullptr; i++) {
     status = REC_RegisterConfigUpdateFunc(config_record_names[i], config_callback, (void *)this);
     if (status != REC_ERR_OKAY) {
-      Warning("couldn't register variable '%s', is records.config up to date?", config_record_names[i]);
+      Warning("couldn't register variable '%s', is %s up to date?", config_record_names[i], ts::filename::RECORDS);
     }
     no_registrations_failed = no_registrations_failed && (status == REC_ERR_OKAY);
   }
@@ -388,7 +360,7 @@ HttpBodyFactory::HttpBodyFactory()
 HttpBodyFactory::~HttpBodyFactory()
 {
   // FIX: need to implement destructor
-  delete table_of_sets;
+  table_of_sets.reset(nullptr);
 }
 
 // LOCKING: must be called with lock taken
@@ -400,15 +372,15 @@ HttpBodyFactory::fabricate(StrList *acpt_language_list, StrList *acpt_charset_li
   char *buffer;
   const char *pType = context->txn_conf->body_factory_template_base;
   const char *set;
-  HttpBodyTemplate *t;
+  HttpBodyTemplate *t = nullptr;
   HttpBodySet *body_set;
   char template_base[PATH_NAME_MAX];
 
   if (set_return) {
     *set_return = "???";
   }
-  *content_language_return = NULL;
-  *content_charset_return  = NULL;
+  *content_language_return = nullptr;
+  *content_charset_return  = nullptr;
 
   Debug("body_factory", "calling fabricate(type '%s')", type);
   *buffer_length_return = 0;
@@ -416,12 +388,12 @@ HttpBodyFactory::fabricate(StrList *acpt_language_list, StrList *acpt_charset_li
   // if error body suppressed, return NULL
   if (is_response_suppressed(context)) {
     Debug("body_factory", "  error suppression enabled, returning NULL template");
-    return (NULL);
+    return nullptr;
   }
   // if custom error pages are disabled, return NULL
   if (!enable_customizations) {
     Debug("body_factory", "  customization disabled, returning NULL template");
-    return (NULL);
+    return nullptr;
   }
 
   // what set should we use (language target if enable_customizations == 2)
@@ -429,25 +401,35 @@ HttpBodyFactory::fabricate(StrList *acpt_language_list, StrList *acpt_charset_li
     set = determine_set_by_language(acpt_language_list, acpt_charset_list);
   } else if (enable_customizations == 3) {
     set = determine_set_by_host(context);
+  } else if (is_response_body_precluded(context->http_return_code)) {
+    return nullptr;
   } else {
     set = "default";
   }
   if (set_return) {
     *set_return = set;
   }
-  if (pType != NULL && 0 != *pType && 0 != strncmp(pType, "NONE", 4)) {
+  if (pType != nullptr && 0 != *pType && 0 != strncmp(pType, "NONE", 4)) {
     sprintf(template_base, "%s_%s", pType, type);
-  } else {
-    sprintf(template_base, "%s", type);
+    t = find_template(set, template_base, &body_set);
+    // Check for default alternate.
+    if (t == nullptr) {
+      sprintf(template_base, "%s_default", pType);
+      t = find_template(set, template_base, &body_set);
+    }
   }
-  // see if we have a custom error page template
-  t = find_template(set, template_base, &body_set);
-  if (t == NULL) {
+
+  // Check for base customizations if specializations didn't work.
+  if (t == nullptr) {
+    if (is_response_body_precluded(context->http_return_code)) {
+      return nullptr;
+    }
     t = find_template(set, type, &body_set); // this executes if the template_base is wrong and doesn't exist
   }
-  if (t == NULL) {
+
+  if (t == nullptr) {
     Debug("body_factory", "  can't find template, returning NULL template");
-    return (NULL);
+    return nullptr;
   }
 
   *content_language_return = body_set->content_language;
@@ -455,7 +437,7 @@ HttpBodyFactory::fabricate(StrList *acpt_language_list, StrList *acpt_charset_li
 
   // build the custom error page
   buffer = t->build_instantiated_buffer(context, buffer_length_return);
-  return (buffer);
+  return buffer;
 }
 
 // LOCKING: must be called with lock taken
@@ -463,19 +445,175 @@ const char *
 HttpBodyFactory::determine_set_by_host(HttpTransact::State *context)
 {
   const char *set;
-  RawHashTable_Value v;
   int host_len = context->hh_info.host_len;
   char host_buffer[host_len + 1];
   strncpy(host_buffer, context->hh_info.request_host, host_len);
   host_buffer[host_len] = '\0';
-  if (table_of_sets->getValue((RawHashTable_Key)host_buffer, &v)) {
-    set = table_of_sets->getKeyFromBinding(table_of_sets->getCurrentBinding((RawHashTable_Key)host_buffer));
+  if (auto it = table_of_sets->find(host_buffer); it != table_of_sets->end()) {
+    set = it->first.c_str();
   } else {
     set = "default";
   }
   return set;
 }
 
+const char *
+HttpBodyFactory::determine_set_by_language(std::unique_ptr<BodySetTable> &table_of_sets, StrList *acpt_language_list,
+                                           StrList *acpt_charset_list, float *Q_best_ptr, int *La_best_ptr, int *Lc_best_ptr,
+                                           int *I_best_ptr)
+{
+  float Q, Ql, Qc, Q_best;
+  int I, Idummy, I_best;
+  int La, Lc, La_best, Lc_best;
+  int is_the_default_set;
+  const char *set_best;
+
+  set_best = "default";
+  Q_best   = 0.00001;
+  La_best  = 0;
+  Lc_best  = INT_MAX;
+  I_best   = INT_MAX;
+
+  Debug("body_factory_determine_set", "  INITIAL: [ set_best='%s', Q=%g, La=%d, Lc=%d, I=%d ]", set_best, Q_best, La_best, Lc_best,
+        I_best);
+
+  // FIX: eliminate this special case (which doesn't work anyway), by properly
+  //      handling empty lists and empty pieces in match_accept_XXX
+
+  // if no Accept-Language or Accept-Charset, just return default
+  if ((acpt_language_list->count == 0) && (acpt_charset_list->count == 0)) {
+    Q_best = 1;
+    Debug("body_factory_determine_set", "  no constraints => returning '%s'", set_best);
+    goto done;
+  }
+
+  if (table_of_sets) {
+    ///////////////////////////////////////////
+    // loop over set->body-types hash table //
+    ///////////////////////////////////////////
+    for (const auto &it : *table_of_sets.get()) {
+      const char *set_name         = it.first.c_str();
+      HttpBodySetRawData *body_set = it.second;
+
+      if ((it.first.empty()) || (body_set->table_of_pages == nullptr)) {
+        continue;
+      }
+
+      //////////////////////////////////////////////////////////////////////
+      // Take this error page language and match it against the           //
+      // Accept-Language string passed in, to evaluate the match          //
+      // quality.  Disable wildcard processing so we use "default"        //
+      // if no set explicitly matches.  We also get back the index        //
+      // of the match and the length of the match.                        //
+      //                                                                  //
+      // We optimize the match in a couple of ways:                       //
+      //   (a) if Q is better ==> wins, else if tie,                      //
+      //   (b) if accept tag length La is bigger ==> wins, else if tie,   //
+      //   (c) if content tag length Lc is smaller ==> wins, else if tie, //
+      //   (d) if index position I is smaller ==> wins                    //
+      //////////////////////////////////////////////////////////////////////
+
+      is_the_default_set = (strcmp(set_name, "default") == 0);
+
+      Debug("body_factory_determine_set", "  --- SET: %-8s (Content-Language '%s', Content-Charset '%s')", set_name,
+            body_set->content_language, body_set->content_charset);
+
+      // if no Accept-Language hdr at all, treat as a wildcard that
+      // slightly prefers "default".
+      if (acpt_language_list->count == 0) {
+        Ql = (is_the_default_set ? 1.0001 : 1.000);
+        La = 0;
+        Lc = INT_MAX;
+        I  = 1;
+        Debug("body_factory_determine_set", "      SET: [%-8s] A-L not present => [ Ql=%g, La=%d, Lc=%d, I=%d ]", set_name, Ql, La,
+              Lc, I);
+      } else {
+        Lc = strlen(body_set->content_language);
+        Ql = HttpCompat::match_accept_language(body_set->content_language, Lc, acpt_language_list, &La, &I, true);
+        Debug("body_factory_determine_set", "      SET: [%-8s] A-L match value => [ Ql=%g, La=%d, Lc=%d, I=%d ]", set_name, Ql, La,
+              Lc, I);
+      }
+
+      /////////////////////////////////////////////////////////////
+      // Take this error page language and match it against the  //
+      // Accept-Charset string passed in, to evaluate the match  //
+      // quality.  Disable wildcard processing so that only      //
+      // explicit values match.  (Many browsers will send along  //
+      // "*" with all lists, and we really don't want to send    //
+      // strange character sets for these people --- we'd rather //
+      // use a more portable "default" set.  The index value we  //
+      // get back isn't used, because it's a little hard to know //
+      // how to tradeoff language indices vs. charset indices.   //
+      // If someone cares, we could surely work charset indices  //
+      // into the sorting computation below.                     //
+      /////////////////////////////////////////////////////////////
+
+      // if no Accept-Charset hdr at all, treat as a wildcard that
+      // slightly prefers "default".
+      if (acpt_charset_list->count == 0) {
+        Qc     = (is_the_default_set ? 1.0001 : 1.000);
+        Idummy = 1;
+        Debug("body_factory_determine_set", "      SET: [%-8s] A-C not present => [ Qc=%g ]", set_name, Qc);
+      } else {
+        Qc = HttpCompat::match_accept_charset(body_set->content_charset, strlen(body_set->content_charset), acpt_charset_list,
+                                              &Idummy, true);
+        Debug("body_factory_determine_set", "      SET: [%-8s] A-C match value => [ Qc=%g ]", set_name, Qc);
+      }
+
+      /////////////////////////////////////////////////////////////////
+      // We get back the Q value, the matching field length, and the //
+      // matching field index.  We sort by largest Q value, but if   //
+      // there is a Q tie, we sub sort on longer matching length,    //
+      // and if there is a tie on Q and L, we sub sort on position   //
+      // index, preferring values earlier in Accept-Language list.   //
+      /////////////////////////////////////////////////////////////////
+
+      Q = std::min(Ql, Qc);
+
+      //////////////////////////////////////////////////////////
+      // normally the Q for default pages should be slightly  //
+      // less than for normal pages, but default pages should //
+      // always match to a slight level, in case everything   //
+      // else doesn't match (matches with Q=0).               //
+      //////////////////////////////////////////////////////////
+
+      if (is_the_default_set) {
+        Q = Q + -0.00005;
+        if (Q < 0.00001) {
+          Q = 0.00001;
+        }
+      }
+
+      Debug("body_factory_determine_set", "      NEW: [ set='%s', Q=%g, La=%d, Lc=%d, I=%d ]", set_name, Q, La, Lc, I);
+      Debug("body_factory_determine_set", "      OLD: [ set='%s', Q=%g, La=%d, Lc=%d, I=%d ]", set_best, Q_best, La_best, Lc_best,
+            I_best);
+
+      if (((Q > Q_best)) || ((Q == Q_best) && (La > La_best)) || ((Q == Q_best) && (La == La_best) && (Lc < Lc_best)) ||
+          ((Q == Q_best) && (La == La_best) && (Lc == Lc_best) && (I < I_best))) {
+        Q_best   = Q;
+        La_best  = La;
+        Lc_best  = Lc;
+        I_best   = I;
+        set_best = set_name;
+
+        Debug("body_factory_determine_set", "   WINNER: [ set_best='%s', Q=%g, La=%d, Lc=%d, I=%d ]", set_best, Q_best, La_best,
+              Lc_best, I_best);
+      } else {
+        Debug("body_factory_determine_set", "    LOSER: [ set_best='%s', Q=%g, La=%d, Lc=%d, I=%d ]", set_best, Q_best, La_best,
+              Lc_best, I_best);
+      }
+    }
+  }
+
+done:
+
+  *Q_best_ptr  = Q_best;
+  *La_best_ptr = La_best;
+  *Lc_best_ptr = Lc_best;
+  *I_best_ptr  = I_best;
+  return set_best;
+}
+
 // LOCKING: must be called with lock taken
 const char *
 HttpBodyFactory::determine_set_by_language(StrList *acpt_language_list, StrList *acpt_charset_list)
@@ -484,49 +622,44 @@ HttpBodyFactory::determine_set_by_language(StrList *acpt_language_list, StrList
   const char *set_best;
   int La_best, Lc_best, I_best;
 
-  set_best = HttpCompat::determine_set_by_language(table_of_sets, acpt_language_list, acpt_charset_list, &Q_best, &La_best,
-                                                   &Lc_best, &I_best);
+  set_best = determine_set_by_language(table_of_sets, acpt_language_list, acpt_charset_list, &Q_best, &La_best, &Lc_best, &I_best);
 
-  return (set_best);
+  return set_best;
 }
 
 // LOCKING: must be called with lock taken
 HttpBodyTemplate *
 HttpBodyFactory::find_template(const char *set, const char *type, HttpBodySet **body_set_return)
 {
-  RawHashTable_Value v;
-
   Debug("body_factory", "calling find_template(%s,%s)", set, type);
 
-  *body_set_return = NULL;
+  *body_set_return = nullptr;
 
-  if (table_of_sets == NULL) {
-    return (NULL);
+  if (table_of_sets == nullptr || !set || !type) {
+    return nullptr;
   }
-  if (table_of_sets->getValue((RawHashTable_Key)set, &v)) {
-    HttpBodySet *body_set        = (HttpBodySet *)v;
-    RawHashTable *table_of_types = body_set->table_of_pages;
-
-    if (table_of_types == NULL) {
-      return (NULL);
+  if (auto it = table_of_sets->find(set); it != table_of_sets->end()) {
+    HttpBodySet *body_set = static_cast<HttpBodySet *>(it->second);
+    if (body_set->table_of_pages == nullptr) {
+      return nullptr;
     }
 
-    if (table_of_types->getValue((RawHashTable_Key)type, &v)) {
-      HttpBodyTemplate *t = (HttpBodyTemplate *)v;
-      if ((t == NULL) || (!t->is_sane())) {
-        return (NULL);
+    if (auto it_page = body_set->table_of_pages->find(type); it_page != body_set->table_of_pages->end()) {
+      HttpBodyTemplate *t = it_page->second;
+      if ((t == nullptr) || (!t->is_sane())) {
+        return nullptr;
       }
       *body_set_return = body_set;
 
       Debug("body_factory", "find_template(%s,%s) -> (file %s, length %" PRId64 ", lang '%s', charset '%s')", set, type,
             t->template_pathname, t->byte_count, body_set->content_language, body_set->content_charset);
 
-      return (t);
+      return t;
     }
   }
   Debug("body_factory", "find_template(%s,%s) -> NULL", set, type);
 
-  return (NULL);
+  return nullptr;
 }
 
 // LOCKING: must be called with lock taken
@@ -544,17 +677,13 @@ HttpBodyFactory::is_response_suppressed(HttpTransact::State *context)
      } else
    */
   if (response_suppression_mode == 0) {
-    return (false);
+    return false;
   } else if (response_suppression_mode == 1) {
-    return (true);
+    return true;
   } else if (response_suppression_mode == 2) {
-    if (context->req_flavor == HttpTransact::REQ_FLAVOR_INTERCEPTED) {
-      return (true);
-    } else {
-      return (false);
-    }
+    return context->request_data.internal_txn;
   } else {
-    return (false);
+    return false;
   }
 }
 
@@ -562,69 +691,43 @@ HttpBodyFactory::is_response_suppressed(HttpTransact::State *context)
 void
 HttpBodyFactory::nuke_template_tables()
 {
-  RawHashTable *h1, *h2;
-  RawHashTable_Value v1, v2;
-  RawHashTable_Binding *b1, *b2;
-  RawHashTable_IteratorState i1, i2;
-  HttpBodySet *body_set;
-  HttpBodyTemplate *hbt;
-
-  h1 = table_of_sets;
-
-  if (h1) {
+  if (table_of_sets) {
     Debug("body_factory", "deleting pre-existing template tables");
   } else {
     Debug("body_factory", "no pre-existing template tables");
   }
-  if (h1 != NULL) {
+  if (table_of_sets) {
     ///////////////////////////////////////////
     // loop over set->body-types hash table //
     ///////////////////////////////////////////
-
-    for (b1 = h1->firstBinding(&i1); b1 != NULL; b1 = h1->nextBinding(&i1)) {
-      v1 = h1->getValueFromBinding(b1);
-
-      body_set = (HttpBodySet *)v1;
+    for (const auto &it : *table_of_sets.get()) {
+      HttpBodySet *body_set = static_cast<HttpBodySet *>(it.second);
       ink_assert(body_set->is_sane());
-      h2 = body_set->table_of_pages;
-
-      if (h2 != NULL) {
-        body_set->table_of_pages = NULL;
-
+      if (body_set->table_of_pages) {
         ///////////////////////////////////////////
         // loop over body-types->body hash table //
         ///////////////////////////////////////////
-
-        for (b2 = h2->firstBinding(&i2); b2 != NULL; b2 = h2->nextBinding(&i2)) {
-          v2 = h2->getValueFromBinding(b2);
-          if (v2) {
-            // need a cast here
-            hbt = (HttpBodyTemplate *)v2;
-            delete hbt;
-          }
+        for (const auto &it_page : *body_set->table_of_pages.get()) {
+          delete it_page.second;
         }
-
-        delete h2;
+        body_set->table_of_pages.reset(nullptr);
       }
-
       delete body_set;
     }
-    delete h1;
+    table_of_sets.reset(nullptr);
   }
-
-  table_of_sets = NULL;
 }
 
 // LOCKING: must be called with lock taken
-RawHashTable *
+std::unique_ptr<HttpBodyFactory::BodySetTable>
 HttpBodyFactory::load_sets_from_directory(char *set_dir)
 {
   DIR *dir;
-  struct dirent *entry_buffer, *result;
-  RawHashTable *new_table_of_sets;
+  struct dirent *dirEntry;
+  std::unique_ptr<HttpBodyFactory::BodySetTable> new_table_of_sets;
 
-  if (set_dir == NULL) {
-    return (NULL);
+  if (set_dir == nullptr) {
+    return nullptr;
   }
 
   Debug("body_factory", "load_sets_from_directory(%s)", set_dir);
@@ -634,20 +737,19 @@ HttpBodyFactory::load_sets_from_directory(char *set_dir)
   //////////////////////////////////////////////////
 
   dir = opendir(set_dir);
-  if (dir == NULL) {
+  if (dir == nullptr) {
     Warning("can't open response template directory '%s' (%s)", set_dir, (strerror(errno) ? strerror(errno) : "unknown reason"));
     Warning("no response templates --- using default error pages");
-    return (NULL);
+    return nullptr;
   }
 
-  new_table_of_sets = new RawHashTable(RawHashTable_KeyType_String);
-  entry_buffer      = (struct dirent *)ats_malloc(sizeof(struct dirent) + MAXPATHLEN + 1);
+  new_table_of_sets.reset(new HttpBodyFactory::BodySetTable);
 
   //////////////////////////////////////////
   // loop over each language subdirectory //
   //////////////////////////////////////////
 
-  while ((readdir_r(dir, entry_buffer, &result) == 0) && (result != NULL)) {
+  while ((dirEntry = readdir(dir))) {
     int status;
     struct stat stat_buf;
     char subdir[MAXPATHLEN + 1];
@@ -656,14 +758,16 @@ HttpBodyFactory::load_sets_from_directory(char *set_dir)
     // ensure a subdirectory, and not starting with '.' //
     //////////////////////////////////////////////////////
 
-    if ((entry_buffer->d_name)[0] == '.') {
+    if ((dirEntry->d_name)[0] == '.') {
       continue;
     }
-    ink_filepath_make(subdir, sizeof(subdir), set_dir, entry_buffer->d_name);
+
+    ink_filepath_make(subdir, sizeof(subdir), set_dir, dirEntry->d_name);
     status = stat(subdir, &stat_buf);
     if (status != 0) {
       continue; // can't stat
     }
+
     if (!S_ISDIR(stat_buf.st_mode)) {
       continue; // not a dir
     }
@@ -672,17 +776,16 @@ HttpBodyFactory::load_sets_from_directory(char *set_dir)
     // at this point, 'subdir' might be a valid template dir //
     ///////////////////////////////////////////////////////////
 
-    HttpBodySet *body_set = load_body_set_from_directory(entry_buffer->d_name, subdir);
-    if (body_set != NULL) {
-      Debug("body_factory", "  %s -> %p", entry_buffer->d_name, body_set);
-      new_table_of_sets->setValue((RawHashTable_Key)(entry_buffer->d_name), (RawHashTable_Value)body_set);
+    HttpBodySet *body_set = load_body_set_from_directory(dirEntry->d_name, subdir);
+    if (body_set != nullptr) {
+      Debug("body_factory", "  %s -> %p", dirEntry->d_name, body_set);
+      new_table_of_sets->emplace(dirEntry->d_name, body_set);
     }
   }
 
-  ats_free(entry_buffer);
   closedir(dir);
 
-  return (new_table_of_sets);
+  return new_table_of_sets;
 }
 
 // LOCKING: must be called with lock taken
@@ -692,8 +795,9 @@ HttpBodyFactory::load_body_set_from_directory(char *set_name, char *tmpl_dir)
   DIR *dir;
   int status;
   struct stat stat_buf;
+  struct dirent *dirEntry;
   char path[MAXPATHLEN + 1];
-  struct dirent *entry_buffer, *result;
+  static const char BASED_DEFAULT[] = "_default";
 
   ////////////////////////////////////////////////
   // ensure we can open tmpl_dir as a directory //
@@ -701,8 +805,8 @@ HttpBodyFactory::load_body_set_from_directory(char *set_name, char *tmpl_dir)
 
   Debug("body_factory", "  load_body_set_from_directory(%s)", tmpl_dir);
   dir = opendir(tmpl_dir);
-  if (dir == NULL) {
-    return (NULL);
+  if (dir == nullptr) {
+    return nullptr;
   }
 
   /////////////////////////////////////////////
@@ -712,8 +816,9 @@ HttpBodyFactory::load_body_set_from_directory(char *set_name, char *tmpl_dir)
   ink_filepath_make(path, sizeof(path), tmpl_dir, ".body_factory_info");
   status = stat(path, &stat_buf);
   if ((status < 0) || !S_ISREG(stat_buf.st_mode)) {
+    Warning("Missing .body_factory_info in %s.  Not loading body_factory templates", tmpl_dir);
     closedir(dir);
-    return (NULL);
+    return nullptr;
   }
   Debug("body_factory", "    found '%s'", path);
 
@@ -726,24 +831,30 @@ HttpBodyFactory::load_body_set_from_directory(char *set_name, char *tmpl_dir)
 
   Debug("body_factory", "  body_set = %p (set_name '%s', lang '%s', charset '%s')", body_set, body_set->set_name,
         body_set->content_language, body_set->content_charset);
-  entry_buffer = (struct dirent *)ats_malloc(sizeof(struct dirent) + MAXPATHLEN + 1);
 
-  while ((readdir_r(dir, entry_buffer, &result) == 0) && (result != NULL)) {
+  while ((dirEntry = readdir(dir))) {
     HttpBodyTemplate *tmpl;
+    size_t d_len = strlen(dirEntry->d_name);
 
     ///////////////////////////////////////////////////////////////
-    // all template files have name of the form <type>#<subtype> //
+    // all template files must have a file name of the form      //
+    // - <type>#<subtype>                                        //
+    // - <base>_<type>#<subtype>                                 //
+    // - <base>_default   [based default]                        //
+    // - default          [global default]                       //
     ///////////////////////////////////////////////////////////////
 
-    if ((strchr(entry_buffer->d_name, '#') == NULL) && (strcmp(entry_buffer->d_name, "default") != 0)) {
+    if (!(nullptr != strchr(dirEntry->d_name, '#') || (0 == strcmp(dirEntry->d_name, "default")) ||
+          (d_len >= sizeof(BASED_DEFAULT) && 0 == strcmp(dirEntry->d_name + d_len - (sizeof(BASED_DEFAULT) - 1), BASED_DEFAULT)))) {
       continue;
     }
 
-    snprintf(path, sizeof(path), "%s/%s", tmpl_dir, entry_buffer->d_name);
+    snprintf(path, sizeof(path), "%s/%s", tmpl_dir, dirEntry->d_name);
     status = stat(path, &stat_buf);
     if (status != 0) {
       continue; // can't stat
     }
+
     if (!S_ISREG(stat_buf.st_mode)) {
       continue; // not a file
     }
@@ -753,16 +864,16 @@ HttpBodyFactory::load_body_set_from_directory(char *set_name, char *tmpl_dir)
     ////////////////////////////////
 
     tmpl = new HttpBodyTemplate();
-    if (!tmpl->load_from_file(tmpl_dir, entry_buffer->d_name)) {
+    if (!tmpl->load_from_file(tmpl_dir, dirEntry->d_name)) {
       delete tmpl;
     } else {
-      Debug("body_factory", "      %s -> %p", entry_buffer->d_name, tmpl);
-      body_set->set_template_by_name(entry_buffer->d_name, tmpl);
+      Debug("body_factory", "      %s -> %p", dirEntry->d_name, tmpl);
+      body_set->set_template_by_name(dirEntry->d_name, tmpl);
     }
   }
-  ats_free(entry_buffer);
+
   closedir(dir);
-  return (body_set);
+  return body_set;
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -775,11 +886,11 @@ HttpBodySet::HttpBodySet()
 {
   magic = HTTP_BODY_SET_MAGIC;
 
-  set_name         = NULL;
-  content_language = NULL;
-  content_charset  = NULL;
+  set_name         = nullptr;
+  content_language = nullptr;
+  content_charset  = nullptr;
 
-  table_of_pages = NULL;
+  table_of_pages = nullptr;
 }
 
 HttpBodySet::~HttpBodySet()
@@ -787,9 +898,7 @@ HttpBodySet::~HttpBodySet()
   ats_free(set_name);
   ats_free(content_language);
   ats_free(content_charset);
-  if (table_of_pages) {
-    delete table_of_pages;
-  }
+  table_of_pages.reset(nullptr);
 }
 
 int
@@ -803,19 +912,15 @@ HttpBodySet::init(char *set, char *dir)
   ink_filepath_make(info_path, sizeof(info_path), dir, ".body_factory_info");
   fd = open(info_path, O_RDONLY);
   if (fd < 0) {
-    return (-1);
+    return -1;
   }
 
   this->set_name = ats_strdup(set);
-
-  if (this->table_of_pages) {
-    delete (this->table_of_pages);
-  }
-  this->table_of_pages = new RawHashTable(RawHashTable_KeyType_String);
+  this->table_of_pages.reset(new TemplateTable);
 
   lineno = 0;
 
-  while (1) {
+  while (true) {
     char *name_s, *name_e, *value_s, *value_e, *hash;
 
     ++lineno;
@@ -910,37 +1015,37 @@ HttpBodySet::init(char *set, char *dir)
   }
 
   close(fd);
-  return (lines_added);
+  return lines_added;
 }
 
 HttpBodyTemplate *
 HttpBodySet::get_template_by_name(const char *name)
 {
-  RawHashTable_Value v;
-
   Debug("body_factory", "    calling get_template_by_name(%s)", name);
 
-  if (table_of_pages == NULL) {
-    return (NULL);
+  if (table_of_pages == nullptr) {
+    return nullptr;
   }
 
-  if (table_of_pages->getValue((RawHashTable_Key)name, &v)) {
-    HttpBodyTemplate *t = (HttpBodyTemplate *)v;
-    if ((t == NULL) || (!t->is_sane())) {
-      return (NULL);
+  if (auto it = table_of_pages->find(name); it != table_of_pages->end()) {
+    HttpBodyTemplate *t = it->second;
+    if ((t == nullptr) || (!t->is_sane())) {
+      return nullptr;
     }
     Debug("body_factory", "    get_template_by_name(%s) -> (file %s, length %" PRId64 ")", name, t->template_pathname,
           t->byte_count);
-    return (t);
+    return t;
   }
   Debug("body_factory", "    get_template_by_name(%s) -> NULL", name);
-  return (NULL);
+  return nullptr;
 }
 
 void
 HttpBodySet::set_template_by_name(const char *name, HttpBodyTemplate *t)
 {
-  table_of_pages->setValue((RawHashTable_Key)name, (RawHashTable_Value)t);
+  if (name) {
+    table_of_pages->emplace(name, t);
+  }
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -953,8 +1058,8 @@ HttpBodyTemplate::HttpBodyTemplate()
 {
   magic             = HTTP_BODY_TEMPLATE_MAGIC;
   byte_count        = 0;
-  template_buffer   = NULL;
-  template_pathname = NULL;
+  template_buffer   = nullptr;
+  template_pathname = nullptr;
 }
 
 HttpBodyTemplate::~HttpBodyTemplate()
@@ -966,7 +1071,7 @@ void
 HttpBodyTemplate::reset()
 {
   ats_free(template_buffer);
-  template_buffer = NULL;
+  template_buffer = nullptr;
   byte_count      = 0;
   ats_free(template_pathname);
 }
@@ -989,10 +1094,10 @@ HttpBodyTemplate::load_from_file(char *dir, char *file)
   // coverity[fs_check_call]
   status = stat(path, &stat_buf);
   if (status != 0) {
-    return (0);
+    return 0;
   }
   if (!S_ISREG(stat_buf.st_mode)) {
-    return (0);
+    return 0;
   }
 
   ///////////////////
@@ -1002,7 +1107,7 @@ HttpBodyTemplate::load_from_file(char *dir, char *file)
   // coverity[toctou]
   fd = open(path, O_RDONLY);
   if (fd < 0) {
-    return (0);
+    return 0;
   }
 
   ////////////////////////////////////////
@@ -1010,7 +1115,7 @@ HttpBodyTemplate::load_from_file(char *dir, char *file)
   ////////////////////////////////////////
 
   new_byte_count                      = stat_buf.st_size;
-  new_template_buffer                 = (char *)ats_malloc(new_byte_count + 1);
+  new_template_buffer                 = static_cast<char *>(ats_malloc(new_byte_count + 1));
   bytes_read                          = read(fd, new_template_buffer, new_byte_count);
   new_template_buffer[new_byte_count] = '\0';
   close(fd);
@@ -1023,7 +1128,7 @@ HttpBodyTemplate::load_from_file(char *dir, char *file)
     Warning("reading template file '%s', got %" PRId64 " bytes instead of %" PRId64 " (%s)", path, bytes_read, new_byte_count,
             (strerror(errno) ? strerror(errno) : "unknown error"));
     ats_free(new_template_buffer);
-    return (0);
+    return 0;
   }
 
   Debug("body_factory", "    read %" PRId64 " bytes from '%s'", new_byte_count, path);
@@ -1037,23 +1142,23 @@ HttpBodyTemplate::load_from_file(char *dir, char *file)
   byte_count        = new_byte_count;
   template_pathname = ats_strdup(path);
 
-  return (1);
+  return 1;
 }
 
 char *
 HttpBodyTemplate::build_instantiated_buffer(HttpTransact::State *context, int64_t *buflen_return)
 {
-  char *buffer = NULL;
+  char *buffer = nullptr;
 
   Debug("body_factory_instantiation", "    before instantiation: [%s]", template_buffer);
 
-  LogAccessHttp la(context->state_machine);
+  LogAccess la(context->state_machine);
 
   buffer = resolve_logfield_string(&la, template_buffer);
 
-  *buflen_return = ((buffer == NULL) ? 0 : strlen(buffer));
+  *buflen_return = ((buffer == nullptr) ? 0 : strlen(buffer));
   Debug("body_factory_instantiation", "    after instantiation: [%s]", buffer);
   Debug("body_factory", "  returning %" PRId64 " byte instantiated buffer", *buflen_return);
 
-  return (buffer);
+  return buffer;
 }
