@@ -30,8 +30,7 @@
 
  ****************************************************************************/
 
-#ifndef _HTTP_CACHE_SM_H_
-#define _HTTP_CACHE_SM_H_
+#pragma once
 
 #include "P_Cache.h"
 #include "ProxyConfig.h"
@@ -41,17 +40,16 @@
 
 class HttpSM;
 class HttpCacheSM;
-class CacheLookupHttpConfig;
 
 struct HttpCacheAction : public Action {
   HttpCacheAction();
-  virtual void cancel(Continuation *c = NULL);
+  void cancel(Continuation *c = nullptr) override;
   void
   init(HttpCacheSM *sm_arg)
   {
     sm = sm_arg;
   };
-  HttpCacheSM *sm;
+  HttpCacheSM *sm = nullptr;
 };
 
 class HttpCacheSM : public Continuation
@@ -60,28 +58,29 @@ public:
   HttpCacheSM();
 
   void
-  init(HttpSM *sm_arg, ProxyMutex *amutex)
+  init(HttpSM *sm_arg, Ptr<ProxyMutex> &amutex)
   {
     master_sm = sm_arg;
     mutex     = amutex;
     captive_action.init(this);
   }
 
-  Action *open_read(const HttpCacheKey *key, URL *url, HTTPHdr *hdr, CacheLookupHttpConfig *params, time_t pin_in_cache);
+  Action *open_read(const HttpCacheKey *key, URL *url, HTTPHdr *hdr, const OverridableHttpConfigParams *params,
+                    time_t pin_in_cache);
 
   Action *open_write(const HttpCacheKey *key, URL *url, HTTPHdr *request, CacheHTTPInfo *old_info, time_t pin_in_cache, bool retry,
                      bool allow_multiple);
 
-  CacheVConnection *cache_read_vc;
-  CacheVConnection *cache_write_vc;
+  CacheVConnection *cache_read_vc  = nullptr;
+  CacheVConnection *cache_write_vc = nullptr;
 
-  bool read_locked;
-  bool write_locked;
+  bool read_locked  = false;
+  bool write_locked = false;
   // Flag to check whether read-while-write is in progress or not
-  bool readwhilewrite_inprogress;
+  bool readwhilewrite_inprogress = false;
 
-  HttpSM *master_sm;
-  Action *pending_action;
+  HttpSM *master_sm      = nullptr;
+  Action *pending_action = nullptr;
 
   // Function to set readwhilewrite_inprogress flag
   inline void
@@ -100,13 +99,13 @@ public:
   bool
   is_ram_cache_hit()
   {
-    return cache_read_vc ? (cache_read_vc->is_ram_cache_hit()) : 0;
+    return cache_read_vc ? (cache_read_vc->is_ram_cache_hit()) : false;
   }
 
   bool
   is_compressed_in_ram()
   {
-    return cache_read_vc ? (cache_read_vc->is_compressed_in_ram()) : 0;
+    return cache_read_vc ? (cache_read_vc->is_compressed_in_ram()) : false;
   }
 
   inline void
@@ -144,8 +143,8 @@ public:
   {
     if (cache_read_vc) {
       HTTP_DECREMENT_DYN_STAT(http_current_cache_connections_stat);
-      cache_read_vc->do_io(VIO::ABORT);
-      cache_read_vc = NULL;
+      cache_read_vc->do_io_close(0); // passing zero as aborting read is not an error
+      cache_read_vc = nullptr;
     }
   }
   inline void
@@ -153,8 +152,8 @@ public:
   {
     if (cache_write_vc) {
       HTTP_DECREMENT_DYN_STAT(http_current_cache_connections_stat);
-      cache_write_vc->do_io(VIO::ABORT);
-      cache_write_vc = NULL;
+      cache_write_vc->do_io_close(0); // passing zero as aborting write is not an error
+      cache_write_vc = nullptr;
     }
   }
   inline void
@@ -162,8 +161,8 @@ public:
   {
     if (cache_write_vc) {
       HTTP_DECREMENT_DYN_STAT(http_current_cache_connections_stat);
-      cache_write_vc->do_io(VIO::CLOSE);
-      cache_write_vc = NULL;
+      cache_write_vc->do_io_close();
+      cache_write_vc = nullptr;
     }
   }
   inline void
@@ -171,8 +170,8 @@ public:
   {
     if (cache_read_vc) {
       HTTP_DECREMENT_DYN_STAT(http_current_cache_connections_stat);
-      cache_read_vc->do_io(VIO::CLOSE);
-      cache_read_vc = NULL;
+      cache_read_vc->do_io_close();
+      cache_read_vc = nullptr;
     }
   }
   inline void
@@ -192,26 +191,24 @@ private:
   int state_cache_open_write(int event, void *data);
 
   HttpCacheAction captive_action;
-  bool open_read_cb;
-  bool open_write_cb;
+  bool open_read_cb  = false;
+  bool open_write_cb = false;
 
   // Open read parameters
-  int open_read_tries;
-  HTTPHdr *read_request_hdr;
-  CacheLookupHttpConfig *read_config;
-  time_t read_pin_in_cache;
+  int open_read_tries                            = 0;
+  HTTPHdr *read_request_hdr                      = nullptr;
+  const OverridableHttpConfigParams *http_params = nullptr;
+  time_t read_pin_in_cache                       = 0;
 
   // Open write parameters
-  bool retry_write;
-  int open_write_tries;
+  bool retry_write     = true;
+  int open_write_tries = 0;
 
   // Common parameters
-  URL *lookup_url;
+  URL *lookup_url = nullptr;
   HttpCacheKey cache_key;
 
   // to keep track of multiple cache lookups
-  int lookup_max_recursive;
-  int current_lookup_level;
+  int lookup_max_recursive = 0;
+  int current_lookup_level = 0;
 };
-
-#endif
