@@ -21,9 +21,8 @@
   limitations under the License.
  */
 
-#include "ts/ink_config.h"
+#include "tscore/ink_config.h"
 #include "P_Net.h"
-#include "Main.h"
 #include "HttpConfig.h"
 #include "HttpSessionAccept.h"
 #include "ReverseProxy.h"
@@ -34,17 +33,55 @@
 #endif
 #include "HttpPages.h"
 #include "HttpTunnel.h"
-#include "ts/Tokenizer.h"
+#include "tscore/Tokenizer.h"
 #include "P_SSLNextProtocolAccept.h"
 #include "ProtocolProbeSessionAccept.h"
 #include "http2/Http2SessionAccept.h"
+#include "HttpConnectionCount.h"
+#include "HttpProxyServerMain.h"
+#if TS_USE_QUIC == 1
+#include "P_QUICNetProcessor.h"
+#include "P_QUICNextProtocolAccept.h"
+#include "http3/Http3SessionAccept.h"
+#endif
 
-HttpSessionAccept *plugin_http_accept             = NULL;
-HttpSessionAccept *plugin_http_transparent_accept = 0;
+#include <vector>
+
+HttpSessionAccept *plugin_http_accept             = nullptr;
+HttpSessionAccept *plugin_http_transparent_accept = nullptr;
 
 static SLL<SSLNextProtocolAccept> ssl_plugin_acceptors;
 static Ptr<ProxyMutex> ssl_plugin_mutex;
 
+std::mutex proxyServerMutex;
+std::condition_variable proxyServerCheck;
+bool et_net_threads_ready = false;
+
+extern int num_of_net_threads;
+extern int num_accept_threads;
+
+/// Global BufferWriter format name functions.
+namespace
+{
+void
+TS_bwf_thread(ts::BufferWriter &w, ts::BWFSpec const &spec)
+{
+  bwformat(w, spec, this_thread());
+}
+void
+TS_bwf_ethread(ts::BufferWriter &w, ts::BWFSpec const &spec)
+{
+  bwformat(w, spec, this_ethread());
+}
+} // namespace
+
+// File / process scope initializations
+static bool HTTP_SERVER_INITIALIZED __attribute__((unused)) = []() -> bool {
+  ts::bwf_register_global("ts-thread", &TS_bwf_thread);
+  ts::bwf_register_global("ts-ethread", &TS_bwf_ethread);
+  return true;
+}();
+
 bool
 ssl_register_protocol(const char *protocol, Continuation *contp)
 {
@@ -59,20 +96,6 @@ ssl_register_protocol(const char *protocol, Continuation *contp)
   return true;
 }
 
-bool
-ssl_unregister_protocol(const char *protocol, Continuation *contp)
-{
-  SCOPED_MUTEX_LOCK(lock, ssl_plugin_mutex, this_ethread());
-
-  for (SSLNextProtocolAccept *ssl = ssl_plugin_acceptors.head; ssl; ssl = ssl_plugin_acceptors.next(ssl)) {
-    // Ignore possible failure because we want to try to unregister
-    // from all SSL ports.
-    ssl->unregisterEndpoint(protocol, contp);
-  }
-
-  return true;
-}
-
 /////////////////////////////////////////////////////////////////
 //
 //  main()
@@ -87,12 +110,12 @@ ssl_unregister_protocol(const char *protocol, Continuation *contp)
 */
 struct HttpProxyAcceptor {
   /// Accept continuation.
-  Continuation *_accept;
+  Continuation *_accept = nullptr;
   /// Options for @c NetProcessor.
   NetProcessor::AcceptOptions _net_opt;
 
   /// Default constructor.
-  HttpProxyAcceptor() : _accept(0) {}
+  HttpProxyAcceptor() {}
 };
 
 /** Global acceptors.
@@ -104,28 +127,42 @@ struct HttpProxyAcceptor {
     @c SSLNextProtocolAccept is a subclass of @c Cont instead of @c
     HttpAccept.
 */
-Vec<HttpProxyAcceptor> HttpProxyAcceptors;
+std::vector<HttpProxyAcceptor> HttpProxyAcceptors;
 
 // Called from InkAPI.cc
 NetProcessor::AcceptOptions
-make_net_accept_options(const HttpProxyPort &port, unsigned nthreads)
+make_net_accept_options(const HttpProxyPort *port, unsigned nthreads)
 {
   NetProcessor::AcceptOptions net;
 
   net.accept_threads = nthreads;
 
-  net.f_inbound_transparent = port.m_inbound_transparent_p;
-  net.ip_family             = port.m_family;
-  net.local_port            = port.m_port;
+  REC_ReadConfigInteger(net.packet_mark, "proxy.config.net.sock_packet_mark_in");
+  REC_ReadConfigInteger(net.packet_tos, "proxy.config.net.sock_packet_tos_in");
+  REC_ReadConfigInteger(net.recv_bufsize, "proxy.config.net.sock_recv_buffer_size_in");
+  REC_ReadConfigInteger(net.send_bufsize, "proxy.config.net.sock_send_buffer_size_in");
+  REC_ReadConfigInteger(net.sockopt_flags, "proxy.config.net.sock_option_flag_in");
+  REC_ReadConfigInteger(net.defer_accept, "proxy.config.net.defer_accept");
 
-  if (port.m_inbound_ip.isValid()) {
-    net.local_ip = port.m_inbound_ip;
-  } else if (AF_INET6 == port.m_family && HttpConfig::m_master.inbound_ip6.isIp6()) {
-    net.local_ip = HttpConfig::m_master.inbound_ip6;
-  } else if (AF_INET == port.m_family && HttpConfig::m_master.inbound_ip4.isIp4()) {
-    net.local_ip = HttpConfig::m_master.inbound_ip4;
+#ifdef TCP_FASTOPEN
+  REC_ReadConfigInteger(net.tfo_queue_length, "proxy.config.net.sock_option_tfo_queue_size_in");
+#endif
+
+  if (port) {
+    net.f_inbound_transparent = port->m_inbound_transparent_p;
+    net.f_mptcp               = port->m_mptcp;
+    net.ip_family             = port->m_family;
+    net.local_port            = port->m_port;
+    net.f_proxy_protocol      = port->m_proxy_protocol;
+
+    if (port->m_inbound_ip.isValid()) {
+      net.local_ip = port->m_inbound_ip;
+    } else if (AF_INET6 == port->m_family && HttpConfig::m_master.inbound_ip6.isIp6()) {
+      net.local_ip = HttpConfig::m_master.inbound_ip6;
+    } else if (AF_INET == port->m_family && HttpConfig::m_master.inbound_ip4.isIp4()) {
+      net.local_ip = HttpConfig::m_master.inbound_ip4;
+    }
   }
-
   return net;
 }
 
@@ -135,11 +172,7 @@ MakeHttpProxyAcceptor(HttpProxyAcceptor &acceptor, HttpProxyPort &port, unsigned
   NetProcessor::AcceptOptions &net_opt = acceptor._net_opt;
   HttpSessionAccept::Options accept_opt;
 
-  net_opt = make_net_accept_options(port, nthreads);
-  REC_ReadConfigInteger(net_opt.recv_bufsize, "proxy.config.net.sock_recv_buffer_size_in");
-  REC_ReadConfigInteger(net_opt.send_bufsize, "proxy.config.net.sock_send_buffer_size_in");
-  REC_ReadConfigInteger(net_opt.packet_mark, "proxy.config.net.sock_packet_mark_in");
-  REC_ReadConfigInteger(net_opt.packet_tos, "proxy.config.net.sock_packet_tos_in");
+  net_opt = make_net_accept_options(&port, nthreads);
 
   accept_opt.f_outbound_transparent = port.m_outbound_transparent_p;
   accept_opt.transport_type         = port.m_type;
@@ -167,7 +200,9 @@ MakeHttpProxyAcceptor(HttpProxyAcceptor &acceptor, HttpProxyPort &port, unsigned
   // XXX the protocol probe should be a configuration option.
 
   ProtocolProbeSessionAccept *probe = new ProtocolProbeSessionAccept();
-  HttpSessionAccept *http           = 0; // don't allocate this unless it will be used.
+  HttpSessionAccept *http           = nullptr; // don't allocate this unless it will be used.
+  probe->proxyPort                  = &port;
+  probe->proxy_protocol_ipmap       = &HttpConfig::m_master.config_proxy_protocol_ipmap;
 
   if (port.m_session_protocol_preference.intersects(HTTP_PROTOCOL_SET)) {
     http = new HttpSessionAccept(accept_opt);
@@ -188,70 +223,105 @@ MakeHttpProxyAcceptor(HttpProxyAcceptor &acceptor, HttpProxyPort &port, unsigned
     // the least important protocol first:
     // http/1.0, http/1.1, h2
 
-    // HTTP
-    if (port.m_session_protocol_preference.contains(TS_ALPN_PROTOCOL_INDEX_HTTP_1_0)) {
-      ssl->registerEndpoint(TS_ALPN_PROTOCOL_HTTP_1_0, http);
-    }
-
-    if (port.m_session_protocol_preference.contains(TS_ALPN_PROTOCOL_INDEX_HTTP_1_1)) {
-      ssl->registerEndpoint(TS_ALPN_PROTOCOL_HTTP_1_1, http);
-    }
-
-    // HTTP2
-    if (port.m_session_protocol_preference.contains(TS_ALPN_PROTOCOL_INDEX_HTTP_2_0)) {
-      Http2SessionAccept *acc = new Http2SessionAccept(accept_opt);
-
-      ssl->registerEndpoint(TS_ALPN_PROTOCOL_HTTP_2_0, acc);
-    }
+    ssl->enableProtocols(port.m_session_protocol_preference);
+    ssl->registerEndpoint(TS_ALPN_PROTOCOL_HTTP_1_0, http);
+    ssl->registerEndpoint(TS_ALPN_PROTOCOL_HTTP_1_1, http);
+    ssl->registerEndpoint(TS_ALPN_PROTOCOL_HTTP_2_0, new Http2SessionAccept(accept_opt));
 
     SCOPED_MUTEX_LOCK(lock, ssl_plugin_mutex, this_ethread());
     ssl_plugin_acceptors.push(ssl);
-
+    ssl->proxyPort   = &port;
     acceptor._accept = ssl;
+#if TS_USE_QUIC == 1
+  } else if (port.isQUIC()) {
+    QUICNextProtocolAccept *quic = new QUICNextProtocolAccept();
+
+    quic->enableProtocols(port.m_session_protocol_preference);
+
+    // HTTP/0.9 over QUIC draft-27 (for interop only, will be removed)
+    quic->registerEndpoint(TS_ALPN_PROTOCOL_HTTP_QUIC_D27, new Http3SessionAccept(accept_opt));
+
+    // HTTP/3 draft-27
+    quic->registerEndpoint(TS_ALPN_PROTOCOL_HTTP_3_D27, new Http3SessionAccept(accept_opt));
+
+    // HTTP/0.9 over QUIC (for interop only, will be removed)
+    quic->registerEndpoint(TS_ALPN_PROTOCOL_HTTP_QUIC, new Http3SessionAccept(accept_opt));
+
+    // HTTP/3
+    quic->registerEndpoint(TS_ALPN_PROTOCOL_HTTP_3, new Http3SessionAccept(accept_opt));
+
+    quic->proxyPort  = &port;
+    acceptor._accept = quic;
+#endif
   } else {
     acceptor._accept = probe;
   }
 }
 
+/// Do all pre-thread initialization / setup.
+void
+prep_HttpProxyServer()
+{
+  httpSessionManager.init();
+}
+
 /** Set up all the accepts and sockets.
  */
 void
-init_HttpProxyServer(int n_accept_threads)
+init_accept_HttpProxyServer(int n_accept_threads)
 {
   HttpProxyPort::Group &proxy_ports = HttpProxyPort::global();
 
   init_reverse_proxy();
-  httpSessionManager.init();
   http_pages_init();
+
 #ifdef USE_HTTP_DEBUG_LISTS
-  ink_mutex_init(&debug_sm_list_mutex, "HttpSM Debug List");
-  ink_mutex_init(&debug_cs_list_mutex, "HttpCS Debug List");
+  ink_mutex_init(&debug_sm_list_mutex);
+  ink_mutex_init(&debug_cs_list_mutex);
 #endif
-  // DI's request to disable/reenable ICP on the fly
-  icp_dynamic_enabled = 1;
 
   // Used to give plugins the ability to create http requests
   //   The equivalent of the connecting to localhost on the  proxy
   //   port but without going through the operating system
   //
-  if (plugin_http_accept == NULL) {
-    plugin_http_accept        = new HttpSessionAccept;
-    plugin_http_accept->mutex = new_ProxyMutex();
+  if (plugin_http_accept == nullptr) {
+    plugin_http_accept = new HttpSessionAccept();
   }
+
   // Same as plugin_http_accept except outbound transparent.
   if (!plugin_http_transparent_accept) {
     HttpSessionAccept::Options ha_opt;
     ha_opt.setOutboundTransparent(true);
-    plugin_http_transparent_accept        = new HttpSessionAccept(ha_opt);
-    plugin_http_transparent_accept->mutex = new_ProxyMutex();
+    plugin_http_transparent_accept = new HttpSessionAccept(ha_opt);
   }
+
   if (!ssl_plugin_mutex) {
     ssl_plugin_mutex = new_ProxyMutex();
   }
 
   // Do the configuration defined ports.
-  for (int i = 0, n = proxy_ports.length(); i < n; ++i) {
-    MakeHttpProxyAcceptor(HttpProxyAcceptors.add(), proxy_ports[i], n_accept_threads);
+  // Assign temporary empty objects of proxy ports size
+  HttpProxyAcceptors.assign(proxy_ports.size(), HttpProxyAcceptor());
+  for (int i = 0, n = proxy_ports.size(); i < n; ++i) {
+    MakeHttpProxyAcceptor(HttpProxyAcceptors.at(i), proxy_ports[i], n_accept_threads);
+  }
+}
+
+/** Increment the counter to keep track of how many et_net threads
+ *  we have started. This function is scheduled at the start of each
+ *  et_net thread using schedule_spawn(). We also check immediately
+ *  after incrementing the counter to see whether all of the et_net
+ *  threads have started such that we can notify main() to call
+ *  start_HttpProxyServer().
+ */
+void
+init_HttpProxyServer()
+{
+  if (eventProcessor.has_tg_started(ET_NET)) {
+    std::unique_lock<std::mutex> lock(proxyServerMutex);
+    et_net_threads_ready = true;
+    lock.unlock();
+    proxyServerCheck.notify_one();
   }
 }
 
@@ -266,17 +336,23 @@ start_HttpProxyServer()
   ///////////////////////////////////
 
   ink_assert(!called_once);
-  ink_assert(proxy_ports.length() == HttpProxyAcceptors.length());
+  ink_assert(proxy_ports.size() == HttpProxyAcceptors.size());
 
-  for (int i = 0, n = proxy_ports.length(); i < n; ++i) {
+  for (int i = 0, n = proxy_ports.size(); i < n; ++i) {
     HttpProxyAcceptor &acceptor = HttpProxyAcceptors[i];
     HttpProxyPort &port         = proxy_ports[i];
     if (port.isSSL()) {
-      if (NULL == sslNetProcessor.main_accept(acceptor._accept, port.m_fd, acceptor._net_opt)) {
+      if (nullptr == sslNetProcessor.main_accept(acceptor._accept, port.m_fd, acceptor._net_opt)) {
         return;
       }
+#if TS_USE_QUIC == 1
+    } else if (port.isQUIC()) {
+      if (nullptr == quic_NetProcessor.main_accept(acceptor._accept, port.m_fd, acceptor._net_opt)) {
+        return;
+      }
+#endif
     } else if (!port.isPlugin()) {
-      if (NULL == netProcessor.main_accept(acceptor._accept, port.m_fd, acceptor._net_opt)) {
+      if (nullptr == netProcessor.main_accept(acceptor._accept, port.m_fd, acceptor._net_opt)) {
         return;
       }
     }
@@ -290,26 +366,20 @@ start_HttpProxyServer()
   }
 #endif
 
+  // Set up stat page for http connection count
+  statPagesManager.register_http("connection_count", register_ShowConnectionCount);
+
   // Alert plugins that connections will be accepted.
   APIHook *hook = lifecycle_hooks->get(TS_LIFECYCLE_PORTS_READY_HOOK);
   while (hook) {
-    hook->invoke(TS_EVENT_LIFECYCLE_PORTS_READY, NULL);
+    hook->invoke(TS_EVENT_LIFECYCLE_PORTS_READY, nullptr);
     hook = hook->next();
   }
 }
 
 void
-start_HttpProxyServerBackDoor(int port, int accept_threads)
+stop_HttpProxyServer()
 {
-  NetProcessor::AcceptOptions opt;
-  HttpSessionAccept::Options ha_opt;
-
-  opt.local_port     = port;
-  opt.accept_threads = accept_threads;
-  opt.localhost_only = true;
-  ha_opt.backdoor    = true;
-  opt.backdoor       = true;
-
-  // The backdoor only binds the loopback interface
-  netProcessor.main_accept(new HttpSessionAccept(ha_opt), NO_FD, opt);
+  sslNetProcessor.stop_accept();
+  netProcessor.stop_accept();
 }
