@@ -21,7 +21,7 @@
   limitations under the License.
  */
 
-#include "ts/ink_platform.h"
+#include "tscore/ink_platform.h"
 
 #include "HttpTransact.h"
 #include "HttpTransactHeaders.h"
@@ -29,11 +29,7 @@
 #include <ctime>
 #include "HTTP.h"
 #include "HttpCompat.h"
-#include "ts/InkErrno.h"
-
-ClassAllocator<CacheLookupHttpConfig> CacheLookupHttpConfigAllocator("CacheLookupHttpConfigAllocator");
-
-CacheLookupHttpConfig global_cache_lookup_config;
+#include "tscore/InkErrno.h"
 
 /**
   Find the pointer and length of an etag, after stripping off any leading
@@ -57,7 +53,7 @@ find_etag(const char *raw_tag_field, int raw_tag_field_len, int *length)
   if ((etag_start < etag_end) && (*etag_start == '"')) {
     ++etag_start;
     --etag_length;
-    quote = (const char *)memchr(etag_start, '"', etag_length);
+    quote = static_cast<const char *>(memchr(etag_start, '"', etag_length));
     if (quote) {
       etag_length = quote - etag_start;
     }
@@ -98,7 +94,7 @@ do_strings_match_strongly(const char *raw_tag_field, int raw_tag_field_len, cons
 
     n = 0;
 
-    if (((int)(tag->len - n) == etag_length) && (strncmp(etag_start, tag->str + n, etag_length) == 0)) {
+    if ((static_cast<int>(tag->len - n) == etag_length) && (strncmp(etag_start, tag->str + n, etag_length) == 0)) {
       return true;
     }
   }
@@ -168,7 +164,7 @@ is_empty(char *s)
 */
 int
 HttpTransactCache::SelectFromAlternates(CacheHTTPInfoVector *cache_vector, HTTPHdr *client_request,
-                                        CacheLookupHttpConfig *http_config_params)
+                                        const OverridableHttpConfigParams *http_config_params)
 {
   time_t current_age, best_age = CacheHighAgeWatermark;
   time_t t_now         = 0;
@@ -184,18 +180,9 @@ HttpTransactCache::SelectFromAlternates(CacheHTTPInfoVector *cache_vector, HTTPH
   Debug("http_match", "[SelectFromAlternates] # alternates = %d", alt_count);
   Debug("http_seq", "[SelectFromAlternates] %d alternates for this cached doc", alt_count);
   if (is_debug_tag_set("http_alts")) {
-    ACQUIRE_PRINT_LOCK()
     fprintf(stderr, "[alts] There are %d alternates for this request header.\n", alt_count);
-    RELEASE_PRINT_LOCK()
-  }
-  // used by ICP to bypass this function
-  if (http_config_params == &global_cache_lookup_config) {
-    return 0;
   }
 
-  if (!client_request->valid()) {
-    return 0;
-  }
   // so that plugins can make cache reads for http
   // docs to check if the doc exists in the cache
   if (!client_request->valid()) {
@@ -216,7 +203,7 @@ HttpTransactCache::SelectFromAlternates(CacheHTTPInfoVector *cache_vector, HTTPH
 
       if (alt_count > 1) {
         if (t_now == 0) {
-          t_now = ink_cluster_time();
+          t_now = ink_local_time();
         }
         current_age = HttpTransactHeaders::calculate_document_age(obj->request_sent_time_get(), obj->response_received_time_get(),
                                                                   cached_response, cached_response->get_date(), t_now);
@@ -225,7 +212,7 @@ HttpTransactCache::SelectFromAlternates(CacheHTTPInfoVector *cache_vector, HTTPH
           current_age = CacheHighAgeWatermark;
         }
       } else {
-        current_age = (time_t)0;
+        current_age = static_cast<time_t>(0);
       }
 
       if (is_debug_tag_set("http_alts")) {
@@ -264,9 +251,7 @@ HttpTransactCache::SelectFromAlternates(CacheHTTPInfoVector *cache_vector, HTTPH
   }
   Debug("http_seq", "[SelectFromAlternates] Chosen alternate # %d", best_index);
   if (is_debug_tag_set("http_alts")) {
-    ACQUIRE_PRINT_LOCK()
     fprintf(stderr, "[alts] and the winner is alternate number %d\n", best_index);
-    RELEASE_PRINT_LOCK()
   }
 
   if ((best_index != -1) && (best_Q > unacceptable_Q)) {
@@ -299,12 +284,12 @@ HttpTransactCache::SelectFromAlternates(CacheHTTPInfoVector *cache_vector, HTTPH
 
 */
 float
-HttpTransactCache::calculate_quality_of_match(CacheLookupHttpConfig *http_config_param, HTTPHdr *client_request,
+HttpTransactCache::calculate_quality_of_match(const OverridableHttpConfigParams *http_config_param, HTTPHdr *client_request,
                                               HTTPHdr *obj_client_request, HTTPHdr *obj_origin_server_response)
 {
   // For PURGE requests, any alternate is good really.
   if (client_request->method_get_wksidx() == HTTP_WKSIDX_PURGE) {
-    return (float)1.0;
+    return static_cast<float>(1.0);
   }
 
   // Now calculate a quality based on all sorts of logic
@@ -501,13 +486,6 @@ HttpTransactCache::calculate_quality_of_match(CacheLookupHttpConfig *http_config
   @return quality (-1: no match, 0..1: poor..good).
 
 */
-static inline bool
-do_content_types_match(char *type1, char *subtype1, char *type2, char *subtype2)
-{
-  return ((is_asterisk(type1) || is_empty(type1) || (strcasecmp(type1, type2) == 0)) &&
-          (is_asterisk(subtype1) || is_empty(subtype1) || (strcasecmp(subtype1, subtype2) == 0)));
-}
-
 float
 HttpTransactCache::calculate_quality_of_accept_match(MIMEField *accept_field, MIMEField *content_field)
 {
@@ -541,6 +519,9 @@ HttpTransactCache::calculate_quality_of_accept_match(MIMEField *accept_field, MI
   // Parse the type and subtype of the Content-Type field.
   HttpCompat::parse_mime_type(c_param->str, c_type, c_subtype, sizeof(c_type), sizeof(c_subtype));
 
+  // Special case for webp because Safari is has Accept: */*, but doesn't support webp
+  bool content_type_webp = ((strcasecmp("webp", c_subtype) == 0) && (strcasecmp("image", c_type) == 0));
+
   // Now loop over Accept field values.
   // TODO: Should we check the return value (count) from this?
   accept_field->value_get_comma_list(&a_values_list);
@@ -564,19 +545,25 @@ HttpTransactCache::calculate_quality_of_accept_match(MIMEField *accept_field, MI
     char a_type[32], a_subtype[32];
     HttpCompat::parse_mime_type(a_param->str, a_type, a_subtype, sizeof(a_type), sizeof(a_subtype));
 
-    //      printf("matching Content-type; '%s/%s' with Accept value '%s/%s'\n",
-    //             c_type,c_subtype,a_type,a_subtype);
+    Debug("http_match", "matching Content-type; '%s/%s' with Accept value '%s/%s'\n", c_type, c_subtype, a_type, a_subtype);
 
-    // Is there a wildcard in the type or subtype?
-    if (is_asterisk(a_type)) {
-      wildcard_type_present = true;
-      wildcard_type_q       = HttpCompat::find_Q_param_in_strlist(&a_param_list);
-    } else if (is_asterisk(a_subtype) && (strcasecmp(a_type, c_type) == 0)) {
-      wildcard_subtype_present = true;
-      wildcard_subtype_q       = HttpCompat::find_Q_param_in_strlist(&a_param_list);
-    } else {
-      // No wildcard. Do explicit matching of accept and content values.
-      if (do_content_types_match(a_type, a_subtype, c_type, c_subtype)) {
+    bool wildcard_found = true;
+    // Only do wildcard checks if the content type is not image/webp
+    if (content_type_webp == false) {
+      // Is there a wildcard in the type or subtype?
+      if (is_asterisk(a_type)) {
+        wildcard_type_present = true;
+        wildcard_type_q       = HttpCompat::find_Q_param_in_strlist(&a_param_list);
+      } else if (is_asterisk(a_subtype) && (strcasecmp(a_type, c_type) == 0)) {
+        wildcard_subtype_present = true;
+        wildcard_subtype_q       = HttpCompat::find_Q_param_in_strlist(&a_param_list);
+      } else {
+        wildcard_found = false;
+      }
+    }
+    if (content_type_webp == true || wildcard_found == false) {
+      // No wildcard or the content type is image/webp. Do explicit matching of accept and content values.
+      if ((strcasecmp(a_type, c_type) == 0) && (strcasecmp(a_subtype, c_subtype) == 0)) {
         float tq;
         tq = HttpCompat::find_Q_param_in_strlist(&a_param_list);
         q  = (tq > q ? tq : q);
@@ -597,6 +584,7 @@ HttpTransactCache::calculate_quality_of_accept_match(MIMEField *accept_field, MI
   if ((q == -1.0) && (wildcard_type_present == true)) {
     q = wildcard_type_q;
   }
+
   return (q);
 }
 
@@ -645,13 +633,13 @@ HttpTransactCache::calculate_quality_of_accept_charset_match(MIMEField *accept_f
     ca_raw = cached_accept_field->value_get(&ca_raw_len);
     if (a_raw && ca_raw && a_raw_len == ca_raw_len && !strncmp(a_raw, ca_raw, a_raw_len)) {
       Debug("http_alternate", "Exact match for ACCEPT CHARSET");
-      return (float)1.001; // slightly higher weight to this guy
+      return static_cast<float>(1.001); // slightly higher weight to this guy
     }
   }
   // return match if either ac or ct is missing
   // this check is different from accept-encoding
   if (accept_field == nullptr || content_field == nullptr) {
-    return (float)1.0;
+    return static_cast<float>(1.0);
   }
   // get the charset of this content-type //
   c_raw = content_field->value_get(&c_raw_len);
@@ -672,7 +660,7 @@ HttpTransactCache::calculate_quality_of_accept_charset_match(MIMEField *accept_f
     HttpCompat::parse_semicolon_list(&a_param_list, a_raw, a_raw_len);
 
     if (a_param_list.head) {
-      a_charset     = (char *)a_param_list.head->str;
+      a_charset     = const_cast<char *>(a_param_list.head->str);
       a_charset_len = a_param_list.head->len;
     } else {
       continue;
@@ -757,14 +745,14 @@ does_encoding_match(char *enc1, const char *enc2)
   return false;
 }
 
-ContentEncoding
-HttpTransactCache::match_gzip(MIMEField *accept_field)
+bool
+HttpTransactCache::match_content_encoding(MIMEField *accept_field, const char *encoding_identifier)
 {
   Str *a_value;
   const char *a_raw;
   StrList a_values_list;
   if (!accept_field) {
-    return NO_GZIP;
+    return false;
   }
   // TODO: Should we check the return value (count) here?
   accept_field->value_get_comma_list(&a_values_list);
@@ -775,17 +763,17 @@ HttpTransactCache::match_gzip(MIMEField *accept_field)
     a_raw = a_value->str;
     HttpCompat::parse_semicolon_list(&a_param_list, a_raw);
     if (a_param_list.head) {
-      a_encoding = (char *)a_param_list.head->str;
+      a_encoding = const_cast<char *>(a_param_list.head->str);
     } else {
       continue;
     }
     float q;
     q = HttpCompat::find_Q_param_in_strlist(&a_param_list);
-    if (q != 0 && does_encoding_match(a_encoding, "gzip")) {
-      return GZIP;
+    if (q != 0 && does_encoding_match(a_encoding, encoding_identifier)) {
+      return true;
     }
   }
-  return NO_GZIP;
+  return false;
 }
 
 // TODO: This used to take a length for c_raw, but that was never used, so removed it from the prototype.
@@ -813,7 +801,7 @@ match_accept_content_encoding(const char *c_raw, MIMEField *accept_field, bool *
     // break Accept-Encoding piece into semi-colon separated parts //
     HttpCompat::parse_semicolon_list(&a_param_list, a_raw);
     if (a_param_list.head) {
-      a_encoding = (char *)a_param_list.head->str;
+      a_encoding = const_cast<char *>(a_param_list.head->str);
     } else {
       continue;
     }
@@ -857,13 +845,13 @@ HttpTransactCache::calculate_quality_of_accept_encoding_match(MIMEField *accept_
     ca_raw = cached_accept_field->value_get(&ca_raw_len);
     if (a_raw && ca_raw && a_raw_len == ca_raw_len && !strncmp(a_raw, ca_raw, a_raw_len)) {
       Debug("http_alternate", "Exact match for ACCEPT ENCODING");
-      return (float)1.001; // slightly higher weight to this guy
+      return static_cast<float>(1.001); // slightly higher weight to this guy
     }
   }
   // return match if both ae and ce are missing
   // this check is different from accept charset
   if (accept_field == nullptr && content_field == nullptr) {
-    return (float)1.0;
+    return static_cast<float>(1.0);
   }
   // if no Content-Encoding, treat as "identity" //
   if (!content_field) {
@@ -895,7 +883,7 @@ HttpTransactCache::calculate_quality_of_accept_encoding_match(MIMEField *accept_
   //   The 1.1 spec says servers MAY assume that clients will accept   //
   //   any encoding if no header is sent.  Unforntunately, this does   //
   //   not work 1.0 clients & is particularly thorny when the proxy    //
-  //   created the enconding as the result of a transform.  Http 1.1   //
+  //   created the encoding as the result of a transform.  Http 1.1   //
   //   purists would say that if proxy encodes something it's really   //
   //   a transfer-encoding and not a content-encoding but again this   //
   //   causes problems with 1.0 clients                                //
@@ -903,12 +891,12 @@ HttpTransactCache::calculate_quality_of_accept_encoding_match(MIMEField *accept_
   if (!accept_field) {
     if (is_identity_encoding) {
       if (!cached_accept_field) {
-        return ((float)1.0);
+        return (static_cast<float>(1.0));
       } else {
-        return ((float)0.001);
+        return (static_cast<float>(0.001));
       }
     } else {
-      return ((float)-1.0);
+      return (static_cast<float>(-1.0));
     }
   }
 
@@ -918,8 +906,8 @@ HttpTransactCache::calculate_quality_of_accept_encoding_match(MIMEField *accept_
   if (!content_field) {
     if (!match_accept_content_encoding("identity", accept_field, &wildcard_present, &wildcard_q, &q)) {
       // CE was not returned, and AE does not have identity
-      if (match_gzip(accept_field) == GZIP && match_gzip(cached_accept_field) == GZIP) {
-        return (float)1.0;
+      if (match_content_encoding(accept_field, "gzip") and match_content_encoding(cached_accept_field, "gzip")) {
+        return 1.0f;
       }
       goto encoding_wildcard;
     }
@@ -957,17 +945,17 @@ encoding_wildcard:
   // still okay, but otherwise, this is just not a match at all.         //
   /////////////////////////////////////////////////////////////////////////
   if ((q == -1.0) && is_identity_encoding) {
-    if (match_gzip(accept_field) == GZIP) {
-      if (match_gzip(cached_accept_field) == GZIP) {
-        return (float)1.0;
+    if (match_content_encoding(accept_field, "gzip")) {
+      if (match_content_encoding(cached_accept_field, "gzip")) {
+        return 1.0f;
       } else {
         // always try to fetch GZIP content if we have not tried sending AE before
-        return (float)-1.0;
+        return -1.0f;
       }
-    } else if (cached_accept_field && match_gzip(cached_accept_field) != GZIP) {
-      return (float)0.001;
+    } else if (cached_accept_field && !match_content_encoding(cached_accept_field, "gzip")) {
+      return 0.001f;
     } else {
-      return (float)-1.0;
+      return -1.0f;
     }
   }
   //      q = (float)-1.0;
@@ -1030,14 +1018,14 @@ match_accept_content_language(const char *c_raw, MIMEField *accept_field, bool *
     float tq = HttpCompat::find_Q_param_in_strlist(&a_param_list);
 
     /////////////////////////////////////////////////////////////////////
-    // This algorithm is a bit wierd --- the resulting Q factor is     //
+    // This algorithm is a bit weird --- the resulting Q factor is     //
     // the Q value corresponding to the LONGEST range field that       //
     // matched, or if none matched, then the Q value of any asterisk.  //
     // Also, if the lang value is "", meaning that no Content-Language //
     // was specified, this document matches all accept headers.        //
     /////////////////////////////////////////////////////////////////////
     if (a_param_list.head) {
-      a_range         = (char *)a_param_list.head->str;
+      a_range         = const_cast<char *>(a_param_list.head->str);
       *a_range_length = a_param_list.head->len;
     } else {
       continue;
@@ -1049,11 +1037,6 @@ match_accept_content_language(const char *c_raw, MIMEField *accept_field, bool *
       return true;
     } else if (does_language_range_match(a_range, c_raw)) {
       *q = tq;
-      // This is disabled, so removed max_a_range_length from prototype
-      //          if (*a_range_length > *max_a_range_length) {
-      //              *q = tq;
-      //              *max_a_range_length = *a_range_length;
-      //          }
       return true;
     } else {
     }
@@ -1063,7 +1046,7 @@ match_accept_content_language(const char *c_raw, MIMEField *accept_field, bool *
 }
 
 // FIX: This code is icky, and i suspect wrong in places, particularly
-//      beacuse parts of match_accept_content_language are commented out.
+//      because parts of match_accept_content_language are commented out.
 //      It looks like lots of hacks were done.  The code should probably
 //      be updated to use the code in HttpCompat::match_accept_language.
 
@@ -1088,7 +1071,7 @@ HttpTransactCache::calculate_quality_of_accept_language_match(MIMEField *accept_
     ca_raw = cached_accept_field->value_get(&ca_raw_len);
     if (a_raw && ca_raw && a_raw_len == ca_raw_len && !strncmp(a_raw, ca_raw, a_raw_len)) {
       Debug("http_alternate", "Exact match for ACCEPT LANGUAGE");
-      return (float)1.001; // slightly higher weight to this guy
+      return static_cast<float>(1.001); // slightly higher weight to this guy
     }
   }
 
@@ -1144,8 +1127,8 @@ language_wildcard:
 
 */
 Variability_t
-HttpTransactCache::CalcVariability(CacheLookupHttpConfig *http_config_params, HTTPHdr *client_request, HTTPHdr *obj_client_request,
-                                   HTTPHdr *obj_origin_server_response)
+HttpTransactCache::CalcVariability(const OverridableHttpConfigParams *http_config_params, HTTPHdr *client_request,
+                                   HTTPHdr *obj_client_request, HTTPHdr *obj_origin_server_response)
 {
   ink_assert(http_config_params != nullptr);
   ink_assert(client_request != nullptr);
@@ -1153,115 +1136,77 @@ HttpTransactCache::CalcVariability(CacheLookupHttpConfig *http_config_params, HT
   ink_assert(obj_origin_server_response != nullptr);
 
   Variability_t variability = VARIABILITY_NONE;
-  if (http_config_params->cache_enable_default_vary_headers || obj_origin_server_response->presence(MIME_PRESENCE_VARY)) {
-    ///////////////////////////////////////////////////////////////////////
-    // If the origin server sent a Vary header in the response, use that //
-    // Vary, otherwise use the default. Ivry adds: However if the origin //
-    // server was a non-compliant 1.1 and did not send a Vary header,    //
-    // treat as 1.0 with no Vary header.                                 //
-    ///////////////////////////////////////////////////////////////////////
+  if (obj_origin_server_response->presence(MIME_PRESENCE_VARY)) {
     StrList vary_list;
-    int num_vary_values = obj_origin_server_response->value_get_comma_list(MIME_FIELD_VARY, MIME_LEN_VARY, &vary_list);
 
-    if (num_vary_values <= 0) { // no vary hdr, so use defaults if enabled
-      const char *vary_values = nullptr;
-      const char *content_type;
-      int content_type_len;
-      char type[32], subtype[32];
-
-      content_type = obj_origin_server_response->value_get(MIME_FIELD_CONTENT_TYPE, MIME_LEN_CONTENT_TYPE, &content_type_len);
-
-      if (content_type) {
-        HttpCompat::parse_mime_type_with_len(content_type, content_type_len, type, subtype, sizeof(type), sizeof(subtype));
-      } else {
-        type[0]    = '\0';
-        subtype[0] = '\0';
+    if (obj_origin_server_response->value_get_comma_list(MIME_FIELD_VARY, MIME_LEN_VARY, &vary_list) > 0) {
+      if (is_debug_tag_set("http_match") && vary_list.head) {
+        Debug("http_match", "Vary list of %d elements", vary_list.count);
+        vary_list.dump(stderr);
       }
 
-      Debug("http_match", "      type = '%s', subtype = '%s'", type, subtype);
-
-      if (http_config_params->cache_enable_default_vary_headers) {
-        if (strcasecmp(type, "text") == 0) {
-          Debug("http_match", "      Using default text vary headers");
-          vary_values = http_config_params->cache_vary_default_text;
-        } else if (strcasecmp(type, "image") == 0) {
-          Debug("http_match", "      Using default image vary headers");
-          vary_values = http_config_params->cache_vary_default_images;
-        } else {
-          Debug("http_match", "      Using default other vary headers");
-          vary_values = http_config_params->cache_vary_default_other;
+      // for each field that varies, see if current & original hdrs match //
+      for (Str *field = vary_list.head; field != nullptr; field = field->next) {
+        if (field->len == 0) {
+          continue;
         }
-      }
-      // convert the comma-sep string from the config var into a list
-      HttpCompat::parse_comma_list(&vary_list, (vary_values ? vary_values : ""));
-    }
 
-    if (is_debug_tag_set("http_match") && (vary_list.head)) {
-      Debug("http_match", "Vary list of %d elements", vary_list.count);
-      vary_list.dump(stderr);
-    }
+        /////////////////////////////////////////////////////////////
+        // If the field name is unhandled, we should probably do a //
+        // string comparison on the values of this extension field //
+        // but currently we just treat it equivalent to a '*'.     //
+        /////////////////////////////////////////////////////////////
 
-    // for each field that varies, see if current & original hdrs match //
-    for (Str *field = vary_list.head; field != nullptr; field = field->next) {
-      if (field->len == 0) {
-        continue;
-      }
+        Debug("http_match", "Vary: %s", field->str);
+        if (((field->str[0] == '*') && (field->str[1] == NUL))) {
+          Debug("http_match", "Wildcard variability --- object not served from cache");
+          variability = VARIABILITY_ALL;
+          break;
+        }
+        ////////////////////////////////////////////////////////////////////////////////////////
+        // Special case: if 'proxy.config.http.global_user_agent_header' set                  //
+        // we should ignore Vary: User-Agent.                                                 //
+        ////////////////////////////////////////////////////////////////////////////////////////
+        if (http_config_params->global_user_agent_header && !strcasecmp(const_cast<char *>(field->str), "User-Agent")) {
+          continue;
+        }
 
-      /////////////////////////////////////////////////////////////
-      // If the field name is unhandled, we should probably do a //
-      // string comparison on the values of this extension field //
-      // but currently we just treat it equivalent to a '*'.     //
-      /////////////////////////////////////////////////////////////
+        // Disable Vary mismatch checking for Accept-Encoding.  This is only safe to
+        // set if you are promising to fix any Accept-Encoding/Content-Encoding mismatches.
+        if (http_config_params->ignore_accept_encoding_mismatch && !strcasecmp(const_cast<char *>(field->str), "Accept-Encoding")) {
+          continue;
+        }
 
-      Debug("http_match", "Vary: %s", field->str);
-      if (((field->str[0] == '*') && (field->str[1] == NUL))) {
-        Debug("http_match", "Wildcard variability --- object not served from cache");
-        variability = VARIABILITY_ALL;
-        break;
-      }
-      ////////////////////////////////////////////////////////////////////////////////////////
-      // Special case: if 'proxy.config.http.global_user_agent_header' set                  //
-      // we should ignore Vary: User-Agent.                                                 //
-      ////////////////////////////////////////////////////////////////////////////////////////
-      if (http_config_params->cache_global_user_agent_header && !strcasecmp((char *)field->str, "User-Agent")) {
-        continue;
-      }
+        ///////////////////////////////////////////////////////////////////
+        // Take the current vary field and look up the headers in        //
+        // the current client, and the original client.  The cached      //
+        // object varies unless BOTH the current client and the original //
+        // client contain the header, and the header values are equal.   //
+        // We relax this to allow a match if NEITHER have the header.    //
+        //                                                               //
+        // While header "equality" appears to be header-specific, the    //
+        // RFC2068 spec implies that matching only needs to account for  //
+        // differences in whitespace and support for multiple headers    //
+        // with the same name.  Case is presumably also insignificant.   //
+        // Other variations (such as q=1 vs. a field with no q factor)   //
+        // mean that the values DO NOT match.                            //
+        ///////////////////////////////////////////////////////////////////
 
-      // Disable Vary mismatch checking for Accept-Encoding.  This is only safe to
-      // set if you are promising to fix any Accept-Encoding/Content-Encoding mismatches.
-      if (http_config_params->ignore_accept_encoding_mismatch && !strcasecmp((char *)field->str, "Accept-Encoding")) {
-        continue;
-      }
+        ink_assert(strlen(field->str) == field->len);
 
-      ///////////////////////////////////////////////////////////////////
-      // Take the current vary field and look up the headers in        //
-      // the current client, and the original client.  The cached      //
-      // object varies unless BOTH the current client and the original //
-      // client contain the header, and the header values are equal.   //
-      // We relax this to allow a match if NEITHER have the header.    //
-      //                                                               //
-      // While header "equality" appears to be header-specific, the    //
-      // RFC2068 spec implies that matching only needs to account for  //
-      // differences in whitespace and support for multiple headers    //
-      // with the same name.  Case is presumably also insignificant.   //
-      // Other variations (such as q=1 vs. a field with no q factor)   //
-      // mean that the values DO NOT match.                            //
-      ///////////////////////////////////////////////////////////////////
+        char *field_name_str = const_cast<char *>(hdrtoken_string_to_wks(field->str, field->len));
+        if (field_name_str == nullptr) {
+          field_name_str = const_cast<char *>(field->str);
+        }
 
-      ink_assert(strlen(field->str) == field->len);
+        MIMEField *cached_hdr_field  = obj_client_request->field_find(field_name_str, field->len);
+        MIMEField *current_hdr_field = client_request->field_find(field_name_str, field->len);
 
-      char *field_name_str = (char *)hdrtoken_string_to_wks(field->str, field->len);
-      if (field_name_str == nullptr) {
-        field_name_str = (char *)field->str;
-      }
-
-      MIMEField *cached_hdr_field  = obj_client_request->field_find(field_name_str, field->len);
-      MIMEField *current_hdr_field = client_request->field_find(field_name_str, field->len);
-
-      // Header values match? //
-      if (!HttpCompat::do_header_values_rfc2068_14_43_match(cached_hdr_field, current_hdr_field)) {
-        variability = VARIABILITY_SOME;
-        break;
+        // Header values match? //
+        if (!HttpCompat::do_vary_header_values_match(cached_hdr_field, current_hdr_field)) {
+          variability = VARIABILITY_SOME;
+          break;
+        }
       }
     }
   }
@@ -1303,6 +1248,32 @@ HttpTransactCache::match_response_to_request_conditionals(HTTPHdr *request, HTTP
     return response->status_get();
   }
 
+  // If-None-Match: may match weakly //
+  if (request->presence(MIME_PRESENCE_IF_NONE_MATCH)) {
+    int raw_etags_len, comma_sep_tag_list_len;
+    const char *raw_etags          = response->value_get(MIME_FIELD_ETAG, MIME_LEN_ETAG, &raw_etags_len);
+    const char *comma_sep_tag_list = nullptr;
+
+    if (raw_etags) {
+      comma_sep_tag_list = request->value_get(MIME_FIELD_IF_NONE_MATCH, MIME_LEN_IF_NONE_MATCH, &comma_sep_tag_list_len);
+      if (!comma_sep_tag_list) {
+        comma_sep_tag_list     = "";
+        comma_sep_tag_list_len = 0;
+      }
+
+      ////////////////////////////////////////////////////////////////////////
+      // If we have an etag and a if-none-match, we are talking to someone  //
+      // who is doing a 1.1 revalidate. Since this is a GET request with no //
+      // sub-ranges, we can do a weak validation.                           //
+      ////////////////////////////////////////////////////////////////////////
+      if (do_strings_match_weakly(raw_etags, raw_etags_len, comma_sep_tag_list, comma_sep_tag_list_len)) {
+        return HTTP_STATUS_NOT_MODIFIED;
+      } else {
+        return response->status_get();
+      }
+    }
+  }
+
   // If-Modified-Since //
   if (request->presence(MIME_PRESENCE_IF_MODIFIED_SINCE)) {
     if (response->presence(MIME_PRESENCE_LAST_MODIFIED)) {
@@ -1331,38 +1302,6 @@ HttpTransactCache::match_response_to_request_conditionals(HTTPHdr *request, HTTP
 
       response_code = HTTP_STATUS_NOT_MODIFIED;
     }
-
-    // we cannot return NOT_MODIFIED yet, need to check If-none-match
-    if (!request->presence(MIME_PRESENCE_IF_NONE_MATCH)) {
-      return response_code;
-    }
-  }
-
-  // If-None-Match: may match weakly //
-  if (request->presence(MIME_PRESENCE_IF_NONE_MATCH)) {
-    int raw_etags_len, comma_sep_tag_list_len;
-    const char *raw_etags          = response->value_get(MIME_FIELD_ETAG, MIME_LEN_ETAG, &raw_etags_len);
-    const char *comma_sep_tag_list = nullptr;
-
-    if (raw_etags) {
-      comma_sep_tag_list = request->value_get(MIME_FIELD_IF_NONE_MATCH, MIME_LEN_IF_NONE_MATCH, &comma_sep_tag_list_len);
-      if (!comma_sep_tag_list) {
-        comma_sep_tag_list     = "";
-        comma_sep_tag_list_len = 0;
-      }
-
-      ////////////////////////////////////////////////////////////////////////
-      // If we have an etag and a if-none-match, we are talking to someone  //
-      // who is doing a 1.1 revalidate. Since this is a GET request with no //
-      // sub-ranges, we can do a weak validation.                           //
-      ////////////////////////////////////////////////////////////////////////
-      if (do_strings_match_weakly(raw_etags, raw_etags_len, comma_sep_tag_list, comma_sep_tag_list_len)) {
-        // the response already failed If-modified-since (if one exists)
-        return HTTP_STATUS_NOT_MODIFIED;
-      } else {
-        return response->status_get();
-      }
-    }
   }
 
   // There is no If-none-match, and If-modified-since failed,
@@ -1371,24 +1310,6 @@ HttpTransactCache::match_response_to_request_conditionals(HTTPHdr *request, HTTP
     return response_code;
   }
 
-  // return PRECONDITIONAL_FAILED if either If-unmodified-since
-  // or If-match fails
-  // BUT, return the original response code only if both pass
-
-  // If-Unmodified-Since //
-  if (request->presence(MIME_PRESENCE_IF_UNMODIFIED_SINCE)) {
-    // lm_value is zero if Last-modified not exists
-    ink_time_t lm_value = response->get_last_modified();
-
-    // Condition fails if Last-modified not exists
-    if ((request->get_if_unmodified_since() < lm_value) || (lm_value == 0)) {
-      return HTTP_STATUS_PRECONDITION_FAILED;
-    } else {
-      // we cannot return yet, need to check If-match
-      response_code = response->status_get();
-    }
-  }
-
   // If-Match: must match strongly //
   if (request->presence(MIME_PRESENCE_IF_MATCH)) {
     int raw_etags_len, comma_sep_tag_list_len;
@@ -1410,13 +1331,25 @@ HttpTransactCache::match_response_to_request_conditionals(HTTPHdr *request, HTTP
     }
 
     if (do_strings_match_strongly(raw_etags, raw_etags_len, comma_sep_tag_list, comma_sep_tag_list_len)) {
-      // at the point, the response passed both If-unmodified-since
-      // and If-match, so we can return the original response code
       return response->status_get();
     } else {
       return HTTP_STATUS_PRECONDITION_FAILED;
     }
   }
+
+  // If-Unmodified-Since //
+  if (request->presence(MIME_PRESENCE_IF_UNMODIFIED_SINCE)) {
+    // lm_value is zero if Last-modified not exists
+    ink_time_t lm_value = response->get_last_modified();
+
+    // Condition fails if Last-modified not exists
+    if ((request->get_if_unmodified_since() < lm_value) || (lm_value == 0)) {
+      return HTTP_STATUS_PRECONDITION_FAILED;
+    } else {
+      response_code = response->status_get();
+    }
+  }
+
   // There is no If-match, and If-unmodified-since passed,
   // so return the original response code
   if (response_code != HTTP_STATUS_NONE) {
@@ -1451,13 +1384,13 @@ HttpTransactCache::match_response_to_request_conditionals(HTTPHdr *request, HTTP
         return HTTP_STATUS_RANGE_NOT_SATISFIABLE;
       }
     }
-    // this a Date, similar to If-Unmodified-Since
+    // this a Date, similar to If-Unmodified-Since but must be an exact match
     else {
       // lm_value is zero if Last-modified not exists
       ink_time_t lm_value = response->get_last_modified();
 
       // condition fails if Last-modified not exists
-      if ((request->get_if_range_date() < lm_value) || (lm_value == 0)) {
+      if ((request->get_if_range_date() != lm_value) || (lm_value == 0)) {
         return HTTP_STATUS_RANGE_NOT_SATISFIABLE;
       } else {
         return response->status_get();
@@ -1467,95 +1400,3 @@ HttpTransactCache::match_response_to_request_conditionals(HTTPHdr *request, HTTP
 
   return response->status_get();
 }
-
-/*---------------------------------------------------
- *        class CacheLookupHttpConfig
- *---------------------------------------------------*/
-int
-CacheLookupHttpConfig::marshal_length()
-{
-  int len = (int)sizeof(int32_t);
-  len += (cache_vary_default_text ? strlen(cache_vary_default_text) + 1 : 1);
-  len += (cache_vary_default_images ? strlen(cache_vary_default_images) + 1 : 1);
-  len += (cache_vary_default_other ? strlen(cache_vary_default_other) + 1 : 1);
-  return len;
-}
-
-int
-CacheLookupHttpConfig::marshal(char *buf, int length)
-{
-  int32_t i32_tmp;
-  char *p = buf;
-  int len;
-
-  if ((length -= sizeof(int32_t)) < 0) {
-    return -1;
-  }
-
-  i32_tmp = (int32_t)cache_enable_default_vary_headers;
-  memcpy(p, &i32_tmp, sizeof(int32_t));
-  p += sizeof(int32_t);
-
-  len = (cache_vary_default_text ? strlen(cache_vary_default_text) + 1 : 1);
-  if ((length -= len) < 0) {
-    return -1;
-  }
-  ink_strlcpy(p, (cache_vary_default_text ? cache_vary_default_text : ""), length);
-  p += len;
-
-  len = (cache_vary_default_images ? strlen(cache_vary_default_images) + 1 : 1);
-  if ((length -= len) < 0) {
-    return -1;
-  }
-  ink_strlcpy(p, (cache_vary_default_images ? cache_vary_default_images : ""), length);
-  p += len;
-
-  len = (cache_vary_default_other ? strlen(cache_vary_default_other) + 1 : 1);
-  if ((length -= len) < 0) {
-    return -1;
-  }
-  ink_strlcpy(p, (cache_vary_default_other ? cache_vary_default_other : ""), length);
-  p += len;
-
-  return (p - buf);
-}
-
-int
-CacheLookupHttpConfig::unmarshal(Arena *arena, const char *buf, int buflen)
-{
-  const char *p = buf;
-  int length    = buflen;
-  int len;
-  int32_t i32_tmp;
-
-  if ((length -= sizeof(int32_t)) < 0) {
-    return -1;
-  }
-
-  memcpy(&i32_tmp, p, sizeof(int32_t));
-  cache_enable_default_vary_headers = (bool)i32_tmp;
-  p += sizeof(int32_t);
-
-  len = strlen(p) + 1;
-  if ((length -= len) < 0) {
-    return -1;
-  }
-  cache_vary_default_text = arena->str_store(((len == 2) ? "" : p), len - 1);
-  p += len;
-
-  len = strlen(p) + 1;
-  if ((length -= len) < 0) {
-    return -1;
-  }
-  cache_vary_default_images = arena->str_store(((len == 2) ? "" : p), len - 1);
-  p += len;
-
-  len = strlen(p) + 1;
-  if ((length -= len) < 0) {
-    return -1;
-  }
-  cache_vary_default_other = arena->str_store(((len == 2) ? "" : p), len - 1);
-  p += len;
-
-  return (p - buf);
-}
