@@ -19,6 +19,7 @@
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
+
  */
 
 #include "RemapPlugins.h"
@@ -26,13 +27,14 @@
 ClassAllocator<RemapPlugins> pluginAllocator("RemapPluginsAlloc");
 
 TSRemapStatus
-RemapPlugins::run_plugin(remap_plugin_info *plugin)
+RemapPlugins::run_plugin(RemapPluginInst *plugin)
 {
+  ink_assert(_s);
+
   TSRemapStatus plugin_retcode;
   TSRemapRequestInfo rri;
-  url_mapping *map = _s->url_map.getMapping();
-  URL *map_from    = _s->url_map.getFromURL();
-  URL *map_to      = _s->url_map.getToURL();
+  URL *map_from = _s->url_map.getFromURL();
+  URL *map_to   = _s->url_map.getToURL();
 
   // This is the equivalent of TSHttpTxnClientReqGet(), which every remap plugin would
   // have to call.
@@ -46,33 +48,12 @@ RemapPlugins::run_plugin(remap_plugin_info *plugin)
 
   rri.redirect = 0;
 
-// These are made to reflect the "defaults" that will be used in
-// the case where the plugins don't modify them. It's semi-weird
-// that the "from" and "to" URLs changes when chaining happens, but
-// it is necessary to get predictable behavior.
-#if 0
-  if (_cur == 0) {
-    rri.remap_from_host = map_from->host_get(&rri.remap_from_host_size);
-    rri.remap_from_port = map_from->port_get();
-    rri.remap_from_path = map_from->path_get(&rri.remap_from_path_size);
-    rri.from_scheme = map_from->scheme_get(&rri.from_scheme_len);
-  } else {
-    rri.remap_from_host = _request_url->host_get(&rri.remap_from_host_size);
-    rri.remap_from_port = _request_url->port_get();
-    rri.remap_from_path = _request_url->path_get(&rri.remap_from_path_size);
-    rri.from_scheme = _request_url->scheme_get(&rri.from_scheme_len);
-  }
-#endif
-
-  void *ih = map->get_instance(_cur);
-
   // Prepare State for the future
-  if (_s && _cur == 0) {
-    _s->fp_tsremap_os_response = plugin->fp_tsremap_os_response;
-    _s->remap_plugin_instance  = ih;
+  if (_cur == 0) {
+    _s->os_response_plugin_inst = plugin;
   }
 
-  plugin_retcode = plugin->fp_tsremap_do_remap(ih, _s ? reinterpret_cast<TSHttpTxn>(_s->state_machine) : nullptr, &rri);
+  plugin_retcode = plugin->doRemap(reinterpret_cast<TSHttpTxn>(_s->state_machine), &rri);
   // TODO: Deal with negative return codes here
   if (plugin_retcode < 0) {
     plugin_retcode = TSREMAP_NO_REMAP;
@@ -95,59 +76,45 @@ RemapPlugins::run_plugin(remap_plugin_info *plugin)
     there actually *is* something to do).
 
 */
-int
+bool
 RemapPlugins::run_single_remap()
 {
   url_mapping *map             = _s->url_map.getMapping();
-  remap_plugin_info *plugin    = map->get_plugin(_cur); // get the nth plugin in our list of plugins
+  RemapPluginInst *plugin      = map->get_plugin_instance(_cur); // get the nth plugin in our list of plugins
   TSRemapStatus plugin_retcode = TSREMAP_NO_REMAP;
-
+  bool zret                    = true; // default - last iteration.
   Debug("url_rewrite", "running single remap rule id %d for the %d%s time", map->map_id, _cur,
         _cur == 1 ? "st" : _cur == 2 ? "nd" : _cur == 3 ? "rd" : "th");
 
+  if (0 == _cur) {
+    Debug("url_rewrite", "setting the remapped url by copying from mapping rule");
+    url_rewrite_remap_request(_s->url_map, _request_url, _s->hdr_info.client_request.method_get_wksidx());
+  }
+
   // There might not be a plugin if we are a regular non-plugin map rule. In that case, we will fall through
   // and do the default mapping and then stop.
   if (plugin) {
     plugin_retcode = run_plugin(plugin);
   }
 
-  _cur++;
+  ++_cur;
 
-  // If the plugin redirected, we need to end the remap chain now.
-  if (_s->remap_redirect) {
-    return 1;
-  }
+  // If the plugin redirected, we need to end the remap chain now. Otherwise see what's next.
+  if (!_s->remap_redirect) {
+    if (TSREMAP_DID_REMAP_STOP == plugin_retcode || TSREMAP_DID_REMAP == plugin_retcode) {
+      ++_rewritten;
+    }
 
-  if (TSREMAP_NO_REMAP == plugin_retcode || TSREMAP_NO_REMAP_STOP == plugin_retcode) {
-    // After running the first plugin, rewrite the request URL. This is doing the default rewrite rule
-    // to handle the case where no plugin ever rewrites.
-    //
-    // XXX we could probably optimize this a bit more by keeping a flag and only rewriting the request URL
-    // if no plugin has rewritten it already.
-    if (_cur == 1) {
-      Debug("url_rewrite", "plugin did not change host, port or path, copying from mapping rule");
-      url_rewrite_remap_request(_s->url_map, _request_url, _s->hdr_info.client_request.method_get_wksidx());
+    if (TSREMAP_NO_REMAP_STOP == plugin_retcode || TSREMAP_DID_REMAP_STOP == plugin_retcode) {
+      Debug("url_rewrite", "breaking remap plugin chain since last plugin said we should stop after %d rewrites", _rewritten);
+    } else if (_cur >= map->plugin_instance_count()) {
+      Debug("url_rewrite", "completed all remap plugins for rule id %d, changed by %d plugins", map->map_id, _rewritten);
+    } else {
+      Debug("url_rewrite", "completed single remap, attempting another via immediate callback");
+      zret = false; // not done yet.
     }
   }
-
-  if (TSREMAP_NO_REMAP_STOP == plugin_retcode || TSREMAP_DID_REMAP_STOP == plugin_retcode) {
-    Debug("url_rewrite", "breaking remap plugin chain since last plugin said we should stop");
-    return 1;
-  }
-
-  if (_cur > MAX_REMAP_PLUGIN_CHAIN) {
-    Error("called %s more than %u times; stopping this remap insanity now", __func__, MAX_REMAP_PLUGIN_CHAIN);
-    return 1;
-  }
-
-  if (_cur >= map->_plugin_count) {
-    // Normally, we would callback into this function but we dont have anything more to do!
-    Debug("url_rewrite", "completed all remap plugins for rule id %d", map->map_id);
-    return 1;
-  }
-
-  Debug("url_rewrite", "completed single remap, attempting another via immediate callback");
-  return 0;
+  return zret;
 }
 
 int
@@ -158,8 +125,6 @@ RemapPlugins::run_remap(int event, Event *e)
   ink_assert(action.continuation);
   ink_assert(action.continuation);
 
-  int ret = 0;
-
   /* make sure we weren't cancelled */
   if (action.cancelled) {
     mutex.clear();
@@ -170,14 +135,14 @@ RemapPlugins::run_remap(int event, Event *e)
   switch (event) {
   case EVENT_IMMEDIATE:
     Debug("url_rewrite", "handling immediate event inside RemapPlugins::run_remap");
-    ret = run_single_remap();
     /**
-     * If ret !=0 then we are done with this processor and we call back into the SM;
-     * otherwise, we call this function again immediately (which really isn't immediate)
-     * thru the eventProcessor, thus forcing another run of run_single_remap() which will
+     * If @c run_single_remap returns @c true then we are done with this processor and we call back
+     * into the SM; otherwise, we call this function again immediately (which really isn't
+     * immediate) thru the eventProcessor, thus forcing another run of run_single_remap() which will
      * then operate on _request_url, etc performing additional remaps (mainly another plugin run)
+     *
      **/
-    if (ret) {
+    if (run_single_remap()) {
       action.continuation->handleEvent(EVENT_REMAP_COMPLETE, nullptr);
       mutex.clear();
       action.mutex.clear();
