@@ -19,72 +19,48 @@
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
+
  */
 
-#include "ts/ink_defs.h"
+#include "tscore/ink_defs.h"
 #include "UrlMapping.h"
-#include "I_RecCore.h"
-#include "ts/ink_cap.h"
+#include "records/I_RecCore.h"
+#include "tscore/ink_cap.h"
 
 /**
  *
-**/
+ **/
 bool
-url_mapping::add_plugin(remap_plugin_info *i, void *ih)
+url_mapping::add_plugin_instance(RemapPluginInst *i)
 {
-  _plugin_list.push_back(i);
-  _instance_data.push_back(ih);
-
+  _plugin_inst_list.push_back(i);
   return true;
 }
 
 /**
  *
-**/
-remap_plugin_info *
-url_mapping::get_plugin(std::size_t index) const
+ **/
+RemapPluginInst *
+url_mapping::get_plugin_instance(std::size_t index) const
 {
-  Debug("url_rewrite", "get_plugin says we have %zu plugins and asking for plugin %zu", plugin_count(), index);
-  if (index < _plugin_list.size()) {
-    return _plugin_list[index];
-  }
-  return nullptr;
-}
-
-void *
-url_mapping::get_instance(std::size_t index) const
-{
-  if (index < _instance_data.size()) {
-    return _instance_data[index];
+  Debug("url_rewrite", "get_plugin says we have %zu plugins and asking for plugin %zu", _plugin_inst_list.size(), index);
+  if (index < _plugin_inst_list.size()) {
+    return _plugin_inst_list[index];
   }
   return nullptr;
 }
 
 /**
  *
-**/
-void
-url_mapping::delete_instance(unsigned int index)
-{
-  void *ih             = get_instance(index);
-  remap_plugin_info *p = get_plugin(index);
-
-  if (ih && p && p->fp_tsremap_delete_instance) {
-    p->fp_tsremap_delete_instance(ih);
-  }
-}
-
-/**
- *
-**/
+ **/
 url_mapping::~url_mapping()
 {
   referer_info *r;
   redirect_tag_str *rc;
   acl_filter_rule *afr;
 
-  tag                 = (char *)ats_free_null(tag);
-  filter_redirect_url = (char *)ats_free_null(filter_redirect_url);
+  tag                 = static_cast<char *>(ats_free_null(tag));
+  filter_redirect_url = static_cast<char *>(ats_free_null(filter_redirect_url));
 
   while ((r = referer_list) != nullptr) {
     referer_list = r->next;
@@ -96,13 +72,6 @@ url_mapping::~url_mapping()
     delete rc;
   }
 
-  // Delete all instance data, this gets ugly because to delete the instance data, we also
-  // must know which plugin this is associated with. Hence, looping with index instead of a
-  // normal iterator. ToDo: Maybe we can combine them into another container.
-  for (std::size_t i = 0; i < plugin_count(); ++i) {
-    delete_instance(i);
-  }
-
   // Delete filters
   while ((afr = filter) != nullptr) {
     filter = afr->next;
@@ -111,7 +80,7 @@ url_mapping::~url_mapping()
 
   // Destroy the URLs
   fromURL.destroy();
-  toUrl.destroy();
+  toURL.destroy();
 }
 
 void
@@ -119,28 +88,29 @@ url_mapping::Print()
 {
   char from_url_buf[131072], to_url_buf[131072];
 
-  fromURL.string_get_buf(from_url_buf, (int)sizeof(from_url_buf));
-  toUrl.string_get_buf(to_url_buf, (int)sizeof(to_url_buf));
+  fromURL.string_get_buf(from_url_buf, static_cast<int>(sizeof(from_url_buf)));
+  toURL.string_get_buf(to_url_buf, static_cast<int>(sizeof(to_url_buf)));
   printf("\t %s %s=> %s %s <%s> [plugins %s enabled; running with %zu plugins]\n", from_url_buf, unique ? "(unique)" : "",
-         to_url_buf, homePageRedirect ? "(R)" : "", tag ? tag : "", plugin_count() > 0 ? "are" : "not", plugin_count());
+         to_url_buf, homePageRedirect ? "(R)" : "", tag ? tag : "", _plugin_inst_list.size() > 0 ? "are" : "not",
+         _plugin_inst_list.size());
 }
 
 /**
  *
-**/
+ **/
 redirect_tag_str *
 redirect_tag_str::parse_format_redirect_url(char *url)
 {
   char *c;
-  redirect_tag_str *r, **rr;
+  redirect_tag_str *r;
   redirect_tag_str *list = nullptr;
-  char type              = 0;
 
   if (url && *url) {
-    for (rr = &list; *(c = url) != 0;) {
+    for (redirect_tag_str **rr = &list; *(c = url) != 0;) {
+      char type = 0;
       for (type = 's'; *c; c++) {
         if (c[0] == '%') {
-          char tmp_type = (char)tolower((int)c[1]);
+          char tmp_type = static_cast<char>(tolower(static_cast<int>(c[1])));
           if (tmp_type == 'r' || tmp_type == 'f' || tmp_type == 't' || tmp_type == 'o') {
             if (url == c) {
               type = tmp_type;
@@ -162,7 +132,6 @@ redirect_tag_str::parse_format_redirect_url(char *url)
         }
         (*rr = r)->next = nullptr;
         rr              = &(r->next);
-        // printf("\t***********'%c' - '%s'*******\n",r->type,r->chunk_str ? r->chunk_str : "<NULL>");
       } else {
         break; /* memory allocation error */
       }
@@ -173,7 +142,7 @@ redirect_tag_str::parse_format_redirect_url(char *url)
 
 /**
  *
-**/
+ **/
 referer_info::referer_info(char *_ref, bool *error_flag, char *errmsgbuf, int errmsgbuf_size)
   : next(nullptr), referer(nullptr), referer_size(0), any(false), negative(false), regx_valid(false)
 {
@@ -213,7 +182,7 @@ referer_info::referer_info(char *_ref, bool *error_flag, char *errmsgbuf, int er
 
 /**
  *
-**/
+ **/
 referer_info::~referer_info()
 {
   ats_free(referer);
