@@ -95,79 +95,99 @@ typedef enum {
   TS_HPACK_STATIC_TABLE_ENTRY_NUM
 } TS_HPACK_STATIC_TABLE_ENTRY;
 
-const static struct {
+struct StaticTable {
+  StaticTable(const char *n, const char *v) : name(n), value(v), name_size(strlen(name)), value_size(strlen(value)) {}
   const char *name;
   const char *value;
-} STATIC_TABLE[] = {{"", ""},
-                    {":authority", ""},
-                    {":method", "GET"},
-                    {":method", "POST"},
-                    {":path", "/"},
-                    {":path", "/index.html"},
-                    {":scheme", "http"},
-                    {":scheme", "https"},
-                    {":status", "200"},
-                    {":status", "204"},
-                    {":status", "206"},
-                    {":status", "304"},
-                    {":status", "400"},
-                    {":status", "404"},
-                    {":status", "500"},
-                    {"accept-charset", ""},
-                    {"accept-encoding", "gzip, deflate"},
-                    {"accept-language", ""},
-                    {"accept-ranges", ""},
-                    {"accept", ""},
-                    {"access-control-allow-origin", ""},
-                    {"age", ""},
-                    {"allow", ""},
-                    {"authorization", ""},
-                    {"cache-control", ""},
-                    {"content-disposition", ""},
-                    {"content-encoding", ""},
-                    {"content-language", ""},
-                    {"content-length", ""},
-                    {"content-location", ""},
-                    {"content-range", ""},
-                    {"content-type", ""},
-                    {"cookie", ""},
-                    {"date", ""},
-                    {"etag", ""},
-                    {"expect", ""},
-                    {"expires", ""},
-                    {"from", ""},
-                    {"host", ""},
-                    {"if-match", ""},
-                    {"if-modified-since", ""},
-                    {"if-none-match", ""},
-                    {"if-range", ""},
-                    {"if-unmodified-since", ""},
-                    {"last-modified", ""},
-                    {"link", ""},
-                    {"location", ""},
-                    {"max-forwards", ""},
-                    {"proxy-authenticate", ""},
-                    {"proxy-authorization", ""},
-                    {"range", ""},
-                    {"referer", ""},
-                    {"refresh", ""},
-                    {"retry-after", ""},
-                    {"server", ""},
-                    {"set-cookie", ""},
-                    {"strict-transport-security", ""},
-                    {"transfer-encoding", ""},
-                    {"user-agent", ""},
-                    {"vary", ""},
-                    {"via", ""},
-                    {"www-authenticate", ""}};
+  const int name_size;
+  const int value_size;
+};
+
+static const StaticTable STATIC_TABLE[] = {{"", ""},
+                                           {":authority", ""},
+                                           {":method", "GET"},
+                                           {":method", "POST"},
+                                           {":path", "/"},
+                                           {":path", "/index.html"},
+                                           {":scheme", "http"},
+                                           {":scheme", "https"},
+                                           {":status", "200"},
+                                           {":status", "204"},
+                                           {":status", "206"},
+                                           {":status", "304"},
+                                           {":status", "400"},
+                                           {":status", "404"},
+                                           {":status", "500"},
+                                           {"accept-charset", ""},
+                                           {"accept-encoding", "gzip, deflate"},
+                                           {"accept-language", ""},
+                                           {"accept-ranges", ""},
+                                           {"accept", ""},
+                                           {"access-control-allow-origin", ""},
+                                           {"age", ""},
+                                           {"allow", ""},
+                                           {"authorization", ""},
+                                           {"cache-control", ""},
+                                           {"content-disposition", ""},
+                                           {"content-encoding", ""},
+                                           {"content-language", ""},
+                                           {"content-length", ""},
+                                           {"content-location", ""},
+                                           {"content-range", ""},
+                                           {"content-type", ""},
+                                           {"cookie", ""},
+                                           {"date", ""},
+                                           {"etag", ""},
+                                           {"expect", ""},
+                                           {"expires", ""},
+                                           {"from", ""},
+                                           {"host", ""},
+                                           {"if-match", ""},
+                                           {"if-modified-since", ""},
+                                           {"if-none-match", ""},
+                                           {"if-range", ""},
+                                           {"if-unmodified-since", ""},
+                                           {"last-modified", ""},
+                                           {"link", ""},
+                                           {"location", ""},
+                                           {"max-forwards", ""},
+                                           {"proxy-authenticate", ""},
+                                           {"proxy-authorization", ""},
+                                           {"range", ""},
+                                           {"referer", ""},
+                                           {"refresh", ""},
+                                           {"retry-after", ""},
+                                           {"server", ""},
+                                           {"set-cookie", ""},
+                                           {"strict-transport-security", ""},
+                                           {"transfer-encoding", ""},
+                                           {"user-agent", ""},
+                                           {"vary", ""},
+                                           {"via", ""},
+                                           {"www-authenticate", ""}};
+
+/**
+  Threshold for total HdrHeap size which used by HPAK Dynamic Table.
+  The HdrHeap is filled by MIMEHdrImpl and MIMEFieldBlockImpl like below.
+  This threshold allow to allocate 3 HdrHeap at maximum.
+
+                     +------------------+-----------------------------+
+   HdrHeap 1 (2048): | MIMEHdrImpl(592) | MIMEFieldBlockImpl(528) x 2 |
+                     +------------------+-----------------------------+--...--+
+   HdrHeap 2 (4096): | MIMEFieldBlockImpl(528) x 7                            |
+                     +------------------------------------------------+--...--+--...--+
+   HdrHeap 3 (8192): | MIMEFieldBlockImpl(528) x 15                                   |
+                     +------------------------------------------------+--...--+--...--+
+*/
+static constexpr uint32_t HPACK_HDR_HEAP_THRESHOLD = sizeof(MIMEHdrImpl) + sizeof(MIMEFieldBlockImpl) * (2 + 7 + 15);
 
 /******************
  * Local functions
  ******************/
 static inline bool
-hpack_field_is_literal(HpackFieldType ftype)
+hpack_field_is_literal(HpackField ftype)
 {
-  return ftype == HPACK_FIELD_INDEXED_LITERAL || ftype == HPACK_FIELD_NOINDEX_LITERAL || ftype == HPACK_FIELD_NEVERINDEX_LITERAL;
+  return ftype == HpackField::INDEXED_LITERAL || ftype == HpackField::NOINDEX_LITERAL || ftype == HpackField::NEVERINDEX_LITERAL;
 }
 
 //
@@ -175,27 +195,27 @@ hpack_field_is_literal(HpackFieldType ftype)
 // kind of field it is. Field types are specified in the high 4 bits
 // and all bits are defined, so there's no way to get an invalid field type.
 //
-HpackFieldType
+HpackField
 hpack_parse_field_type(uint8_t ftype)
 {
   if (ftype & 0x80) {
-    return HPACK_FIELD_INDEX;
+    return HpackField::INDEX;
   }
 
   if (ftype & 0x40) {
-    return HPACK_FIELD_INDEXED_LITERAL;
+    return HpackField::INDEXED_LITERAL;
   }
 
   if (ftype & 0x20) {
-    return HPACK_FIELD_TABLESIZE_UPDATE;
+    return HpackField::TABLESIZE_UPDATE;
   }
 
   if (ftype & 0x10) {
-    return HPACK_FIELD_NEVERINDEX_LITERAL;
+    return HpackField::NEVERINDEX_LITERAL;
   }
 
   ink_assert((ftype & 0xf0) == 0x0);
-  return HPACK_FIELD_NOINDEX_LITERAL;
+  return HpackField::NOINDEX_LITERAL;
 }
 
 /************************
@@ -214,9 +234,9 @@ HpackLookupResult
 HpackIndexingTable::lookup(const char *name, int name_len, const char *value, int value_len) const
 {
   HpackLookupResult result;
-  const int entry_num = TS_HPACK_STATIC_TABLE_ENTRY_NUM + _dynamic_table->length();
+  const unsigned int entry_num = TS_HPACK_STATIC_TABLE_ENTRY_NUM + _dynamic_table->length();
 
-  for (int index = 1; index < entry_num; ++index) {
+  for (unsigned int index = 1; index < entry_num; ++index) {
     const char *table_name, *table_value;
     int table_name_len = 0, table_value_len = 0;
 
@@ -224,8 +244,8 @@ HpackIndexingTable::lookup(const char *name, int name_len, const char *value, in
       // static table
       table_name      = STATIC_TABLE[index].name;
       table_value     = STATIC_TABLE[index].value;
-      table_name_len  = strlen(table_name);
-      table_value_len = strlen(table_value);
+      table_name_len  = STATIC_TABLE[index].name_size;
+      table_value_len = STATIC_TABLE[index].value_size;
     } else {
       // dynamic table
       const MIMEField *m_field = _dynamic_table->get_header_field(index - TS_HPACK_STATIC_TABLE_ENTRY_NUM);
@@ -238,19 +258,19 @@ HpackIndexingTable::lookup(const char *name, int name_len, const char *value, in
     if (ptr_len_casecmp(name, name_len, table_name, table_name_len) == 0) {
       if ((value_len == table_value_len) && (memcmp(value, table_value, value_len) == 0)) {
         result.index      = index;
-        result.match_type = HPACK_EXACT_MATCH;
+        result.match_type = HpackMatch::EXACT;
         break;
       } else if (!result.index) {
         result.index      = index;
-        result.match_type = HPACK_NAME_MATCH;
+        result.match_type = HpackMatch::NAME;
       }
     }
   }
-  if (result.match_type != HPACK_NO_MATCH) {
+  if (result.match_type != HpackMatch::NONE) {
     if (result.index < TS_HPACK_STATIC_TABLE_ENTRY_NUM) {
-      result.index_type = HPACK_INDEX_TYPE_STATIC;
+      result.index_type = HpackIndex::STATIC;
     } else {
-      result.index_type = HPACK_INDEX_TYPE_DYNAMIC;
+      result.index_type = HpackIndex::DYNAMIC;
     }
   }
 
@@ -267,8 +287,8 @@ HpackIndexingTable::get_header_field(uint32_t index, MIMEFieldWrapper &field) co
 
   if (index < TS_HPACK_STATIC_TABLE_ENTRY_NUM) {
     // static table
-    field.name_set(STATIC_TABLE[index].name, strlen(STATIC_TABLE[index].name));
-    field.value_set(STATIC_TABLE[index].value, strlen(STATIC_TABLE[index].value));
+    field.name_set(STATIC_TABLE[index].name, STATIC_TABLE[index].name_size);
+    field.value_set(STATIC_TABLE[index].value, STATIC_TABLE[index].value_size);
   } else if (index < TS_HPACK_STATIC_TABLE_ENTRY_NUM + _dynamic_table->length()) {
     // dynamic table
     const MIMEField *m_field = _dynamic_table->get_header_field(index - TS_HPACK_STATIC_TABLE_ENTRY_NUM);
@@ -307,16 +327,34 @@ HpackIndexingTable::size() const
   return _dynamic_table->size();
 }
 
-bool
+void
 HpackIndexingTable::update_maximum_size(uint32_t new_size)
 {
-  return _dynamic_table->update_maximum_size(new_size);
+  _dynamic_table->update_maximum_size(new_size);
+}
+
+//
+// HpackDynamicTable
+//
+HpackDynamicTable::~HpackDynamicTable()
+{
+  this->_headers.clear();
+
+  this->_mhdr->fields_clear();
+  this->_mhdr->destroy();
+  delete this->_mhdr;
+
+  if (this->_mhdr_old != nullptr) {
+    this->_mhdr_old->fields_clear();
+    this->_mhdr_old->destroy();
+    delete this->_mhdr_old;
+  }
 }
 
 const MIMEField *
 HpackDynamicTable::get_header_field(uint32_t index) const
 {
-  return _headers.get(index);
+  return this->_headers.at(index);
 }
 
 void
@@ -332,28 +370,25 @@ HpackDynamicTable::add_header_field(const MIMEField *field)
     // It is not an error to attempt to add an entry that is larger than
     // the maximum size; an attempt to add an entry larger than the entire
     // table causes the table to be emptied of all existing entries.
-    _headers.clear();
-    _mhdr->fields_clear();
-    _current_size = 0;
-  } else {
-    _current_size += header_size;
-    while (_current_size > _maximum_size) {
-      int last_name_len, last_value_len;
-      MIMEField *last_field = _headers.last();
+    this->_headers.clear();
+    this->_mhdr->fields_clear();
 
-      last_field->name_get(&last_name_len);
-      last_field->value_get(&last_value_len);
-      _current_size -= ADDITIONAL_OCTETS + last_name_len + last_value_len;
-
-      _headers.remove_index(_headers.length() - 1);
-      _mhdr->field_delete(last_field, false);
+    if (this->_mhdr_old) {
+      this->_mhdr_old->fields_clear();
+      this->_mhdr_old->destroy();
+      delete this->_mhdr_old;
+      this->_mhdr_old = nullptr;
     }
 
-    MIMEField *new_field = _mhdr->field_create(name, name_len);
-    new_field->value_set(_mhdr->m_heap, _mhdr->m_mime, value, value_len);
-    _mhdr->field_attach(new_field);
-    // XXX Because entire Vec instance is copied, Its too expensive!
-    _headers.insert(0, new_field);
+    this->_current_size = 0;
+  } else {
+    this->_current_size += header_size;
+    this->_evict_overflowed_entries();
+
+    MIMEField *new_field = this->_mhdr->field_create(name, name_len);
+    new_field->value_set(this->_mhdr->m_heap, this->_mhdr->m_mime, value, value_len);
+    this->_mhdr->field_attach(new_field);
+    this->_headers.push_front(new_field);
   }
 }
 
@@ -376,118 +411,70 @@ HpackDynamicTable::size() const
 // are evicted from the end of the header table until the size of the
 // header table is less than or equal to the maximum size.
 //
-bool
+void
 HpackDynamicTable::update_maximum_size(uint32_t new_size)
 {
-  while (_current_size > new_size) {
-    if (_headers.n <= 0) {
-      return false;
-    }
-    int last_name_len, last_value_len;
-    MIMEField *last_field = _headers.last();
-
-    last_field->name_get(&last_name_len);
-    last_field->value_get(&last_value_len);
-    _current_size -= ADDITIONAL_OCTETS + last_name_len + last_value_len;
-
-    _headers.remove_index(_headers.length() - 1);
-    _mhdr->field_delete(last_field, false);
-  }
-
-  _maximum_size = new_size;
-  return true;
+  this->_maximum_size = new_size;
+  this->_evict_overflowed_entries();
 }
 
 uint32_t
 HpackDynamicTable::length() const
 {
-  return _headers.length();
+  return this->_headers.size();
 }
 
-//
-// [RFC 7541] 5.1. Integer representation
-//
-int64_t
-encode_integer(uint8_t *buf_start, const uint8_t *buf_end, uint32_t value, uint8_t n)
+void
+HpackDynamicTable::_evict_overflowed_entries()
 {
-  if (buf_start >= buf_end) {
-    return -1;
+  if (this->_current_size <= this->_maximum_size) {
+    // Do nothing
+    return;
   }
 
-  uint8_t *p = buf_start;
+  for (auto h = this->_headers.rbegin(); h != this->_headers.rend(); ++h) {
+    int name_len, value_len;
+    (*h)->name_get(&name_len);
+    (*h)->value_get(&value_len);
 
-  if (value < (static_cast<uint32_t>(1 << n) - 1)) {
-    *(p++) = value;
+    this->_current_size -= ADDITIONAL_OCTETS + name_len + value_len;
+
+    if (this->_mhdr_old && this->_mhdr_old->fields_count() != 0) {
+      this->_mhdr_old->field_delete(*h, false);
+    } else {
+      this->_mhdr->field_delete(*h, false);
+    }
+
+    this->_headers.pop_back();
+
+    if (this->_current_size <= this->_maximum_size) {
+      break;
+    }
+  }
+
+  this->_mime_hdr_gc();
+}
+
+/**
+   When HdrHeap size of current MIMEHdr exceeds the threshold, allocate new MIMEHdr and HdrHeap.
+   The old MIMEHdr and HdrHeap will be freed, when all MIMEFiled are deleted by HPACK Entry Eviction.
+ */
+void
+HpackDynamicTable::_mime_hdr_gc()
+{
+  if (this->_mhdr_old == nullptr) {
+    if (this->_mhdr->m_heap->total_used_size() >= HPACK_HDR_HEAP_THRESHOLD) {
+      this->_mhdr_old = this->_mhdr;
+      this->_mhdr     = new MIMEHdr();
+      this->_mhdr->create();
+    }
   } else {
-    *(p++) = (1 << n) - 1;
-    value -= (1 << n) - 1;
-    while (value >= 128) {
-      if (p >= buf_end) {
-        return -1;
-      }
-      *(p++) = (value & 0x7F) | 0x80;
-      value  = value >> 7;
+    if (this->_mhdr_old->fields_count() == 0) {
+      this->_mhdr_old->destroy();
+      delete this->_mhdr_old;
+      this->_mhdr_old = nullptr;
     }
-    if (p + 1 >= buf_end) {
-      return -1;
-    }
-    *(p++) = value;
   }
-  return p - buf_start;
-}
-
-int64_t
-encode_string(uint8_t *buf_start, const uint8_t *buf_end, const char *value, size_t value_len)
-{
-  uint8_t *p       = buf_start;
-  bool use_huffman = true;
-  char *data       = nullptr;
-  int64_t data_len = 0;
-
-  // TODO Choose whether to use Huffman encoding wisely
-
-  if (use_huffman && value_len) {
-    data = static_cast<char *>(ats_malloc(value_len * 4));
-    if (data == nullptr) {
-      return -1;
-    }
-    data_len = huffman_encode(reinterpret_cast<uint8_t *>(data), reinterpret_cast<const uint8_t *>(value), value_len);
-  }
-
-  // Length
-  const int64_t len = encode_integer(p, buf_end, data_len, 7);
-  if (len == -1) {
-    if (use_huffman) {
-      ats_free(data);
-    }
-
-    return -1;
-  }
-
-  if (use_huffman) {
-    *p |= 0x80;
-  }
-  p += len;
-
-  if (buf_end < p || buf_end - p < data_len) {
-    if (use_huffman) {
-      ats_free(data);
-    }
-
-    return -1;
-  }
-
-  // Value
-  if (data_len) {
-    memcpy(p, data, data_len);
-    p += data_len;
-  }
-
-  if (use_huffman) {
-    ats_free(data);
-  }
-
-  return p - buf_start;
 }
 
 int64_t
@@ -500,7 +487,7 @@ encode_indexed_header_field(uint8_t *buf_start, const uint8_t *buf_end, uint32_t
   uint8_t *p = buf_start;
 
   // Index
-  const int64_t len = encode_integer(p, buf_end, index, 7);
+  const int64_t len = xpack_encode_integer(p, buf_end, index, 7);
   if (len == -1) {
     return -1;
   }
@@ -519,7 +506,7 @@ encode_indexed_header_field(uint8_t *buf_start, const uint8_t *buf_end, uint32_t
 
 int64_t
 encode_literal_header_field_with_indexed_name(uint8_t *buf_start, const uint8_t *buf_end, const MIMEFieldWrapper &header,
-                                              uint32_t index, HpackIndexingTable &indexing_table, HpackFieldType type)
+                                              uint32_t index, HpackIndexingTable &indexing_table, HpackField type)
 {
   uint8_t *p = buf_start;
   int64_t len;
@@ -528,16 +515,16 @@ encode_literal_header_field_with_indexed_name(uint8_t *buf_start, const uint8_t
   ink_assert(hpack_field_is_literal(type));
 
   switch (type) {
-  case HPACK_FIELD_INDEXED_LITERAL:
+  case HpackField::INDEXED_LITERAL:
     indexing_table.add_header_field(header.field_get());
     prefix = 6;
     flag   = 0x40;
     break;
-  case HPACK_FIELD_NOINDEX_LITERAL:
+  case HpackField::NOINDEX_LITERAL:
     prefix = 4;
     flag   = 0x00;
     break;
-  case HPACK_FIELD_NEVERINDEX_LITERAL:
+  case HpackField::NEVERINDEX_LITERAL:
     prefix = 4;
     flag   = 0x10;
     break;
@@ -546,7 +533,8 @@ encode_literal_header_field_with_indexed_name(uint8_t *buf_start, const uint8_t
   }
 
   // Index
-  len = encode_integer(p, buf_end, index, prefix);
+  *p  = 0;
+  len = xpack_encode_integer(p, buf_end, index, prefix);
   if (len == -1) {
     return -1;
   }
@@ -561,7 +549,7 @@ encode_literal_header_field_with_indexed_name(uint8_t *buf_start, const uint8_t
   // Value String
   int value_len;
   const char *value = header.value_get(&value_len);
-  len               = encode_string(p, buf_end, value, value_len);
+  len               = xpack_encode_string(p, buf_end, value, value_len);
   if (len == -1) {
     return -1;
   }
@@ -573,7 +561,7 @@ encode_literal_header_field_with_indexed_name(uint8_t *buf_start, const uint8_t
 
 int64_t
 encode_literal_header_field_with_new_name(uint8_t *buf_start, const uint8_t *buf_end, const MIMEFieldWrapper &header,
-                                          HpackIndexingTable &indexing_table, HpackFieldType type)
+                                          HpackIndexingTable &indexing_table, HpackField type)
 {
   uint8_t *p = buf_start;
   int64_t len;
@@ -582,14 +570,14 @@ encode_literal_header_field_with_new_name(uint8_t *buf_start, const uint8_t *buf
   ink_assert(hpack_field_is_literal(type));
 
   switch (type) {
-  case HPACK_FIELD_INDEXED_LITERAL:
+  case HpackField::INDEXED_LITERAL:
     indexing_table.add_header_field(header.field_get());
     flag = 0x40;
     break;
-  case HPACK_FIELD_NOINDEX_LITERAL:
+  case HpackField::NOINDEX_LITERAL:
     flag = 0x00;
     break;
-  case HPACK_FIELD_NEVERINDEX_LITERAL:
+  case HpackField::NEVERINDEX_LITERAL:
     flag = 0x10;
     break;
   default:
@@ -611,7 +599,7 @@ encode_literal_header_field_with_new_name(uint8_t *buf_start, const uint8_t *buf
   }
 
   // Name String
-  len = encode_string(p, buf_end, lower_name, name_len);
+  len = xpack_encode_string(p, buf_end, lower_name, name_len);
   if (len == -1) {
     return -1;
   }
@@ -620,7 +608,7 @@ encode_literal_header_field_with_new_name(uint8_t *buf_start, const uint8_t *buf
   // Value String
   int value_len;
   const char *value = header.value_get(&value_len);
-  len               = encode_string(p, buf_end, value, value_len);
+  len               = xpack_encode_string(p, buf_end, value, value_len);
   if (len == -1) {
     return -1;
   }
@@ -634,95 +622,15 @@ encode_literal_header_field_with_new_name(uint8_t *buf_start, const uint8_t *buf
 int64_t
 encode_dynamic_table_size_update(uint8_t *buf_start, const uint8_t *buf_end, uint32_t size)
 {
-  const int64_t len = encode_integer(buf_start, buf_end, size, 5);
+  buf_start[0]      = 0x20;
+  const int64_t len = xpack_encode_integer(buf_start, buf_end, size, 5);
   if (len == -1) {
     return -1;
   }
-  buf_start[0] |= 0x20;
 
   return len;
 }
 
-//
-// [RFC 7541] 5.1. Integer representation
-//
-int64_t
-decode_integer(uint32_t &dst, const uint8_t *buf_start, const uint8_t *buf_end, uint8_t n)
-{
-  if (buf_start >= buf_end) {
-    return HPACK_ERROR_COMPRESSION_ERROR;
-  }
-
-  const uint8_t *p = buf_start;
-
-  dst = (*p & ((1 << n) - 1));
-  if (dst == static_cast<uint32_t>(1 << n) - 1) {
-    int m = 0;
-    do {
-      if (++p >= buf_end) {
-        return HPACK_ERROR_COMPRESSION_ERROR;
-      }
-
-      uint32_t added_value = *p & 0x7f;
-      if ((UINT32_MAX >> m) < added_value) {
-        // Excessively large integer encodings - in value or octet
-        // length - MUST be treated as a decoding error.
-        return HPACK_ERROR_COMPRESSION_ERROR;
-      }
-      dst += added_value << m;
-      m += 7;
-    } while (*p & 0x80);
-  }
-
-  return p - buf_start + 1;
-}
-
-//
-// [RFC 7541] 5.2. String Literal Representation
-// return content from String Data (Length octets) with huffman decoding if it is encoded
-//
-int64_t
-decode_string(Arena &arena, char **str, uint32_t &str_length, const uint8_t *buf_start, const uint8_t *buf_end)
-{
-  if (buf_start >= buf_end) {
-    return HPACK_ERROR_COMPRESSION_ERROR;
-  }
-
-  const uint8_t *p            = buf_start;
-  bool isHuffman              = *p & 0x80;
-  uint32_t encoded_string_len = 0;
-  int64_t len                 = 0;
-
-  len = decode_integer(encoded_string_len, p, buf_end, 7);
-  if (len == HPACK_ERROR_COMPRESSION_ERROR) {
-    return HPACK_ERROR_COMPRESSION_ERROR;
-  }
-  p += len;
-
-  if ((p + encoded_string_len) > buf_end) {
-    return HPACK_ERROR_COMPRESSION_ERROR;
-  }
-
-  if (isHuffman) {
-    // Allocate temporary area twice the size of before decoded data
-    *str = arena.str_alloc(encoded_string_len * 2);
-
-    len = huffman_decode(*str, p, encoded_string_len);
-    if (len < 0) {
-      return HPACK_ERROR_COMPRESSION_ERROR;
-    }
-    str_length = len;
-  } else {
-    *str = arena.str_alloc(encoded_string_len);
-
-    memcpy(*str, reinterpret_cast<const char *>(p), encoded_string_len);
-
-    str_length = encoded_string_len;
-  }
-
-  return p + encoded_string_len - buf_start;
-}
-
 //
 // [RFC 7541] 6.1. Indexed Header Field Representation
 //
@@ -730,11 +638,11 @@ int64_t
 decode_indexed_header_field(MIMEFieldWrapper &header, const uint8_t *buf_start, const uint8_t *buf_end,
                             HpackIndexingTable &indexing_table)
 {
-  uint32_t index = 0;
+  uint64_t index = 0;
   int64_t len    = 0;
 
-  len = decode_integer(index, buf_start, buf_end, 7);
-  if (len == HPACK_ERROR_COMPRESSION_ERROR) {
+  len = xpack_decode_integer(index, buf_start, buf_end, 7);
+  if (len == XPACK_ERROR_COMPRESSION_ERROR) {
     return HPACK_ERROR_COMPRESSION_ERROR;
   }
 
@@ -766,22 +674,22 @@ decode_literal_header_field(MIMEFieldWrapper &header, const uint8_t *buf_start,
 {
   const uint8_t *p         = buf_start;
   bool isIncremental       = false;
-  uint32_t index           = 0;
+  uint64_t index           = 0;
   int64_t len              = 0;
-  HpackFieldType ftype     = hpack_parse_field_type(*p);
+  HpackField ftype         = hpack_parse_field_type(*p);
   bool has_http2_violation = false;
 
-  if (ftype == HPACK_FIELD_INDEXED_LITERAL) {
-    len           = decode_integer(index, p, buf_end, 6);
+  if (ftype == HpackField::INDEXED_LITERAL) {
+    len           = xpack_decode_integer(index, p, buf_end, 6);
     isIncremental = true;
-  } else if (ftype == HPACK_FIELD_NEVERINDEX_LITERAL) {
-    len = decode_integer(index, p, buf_end, 4);
+  } else if (ftype == HpackField::NEVERINDEX_LITERAL) {
+    len = xpack_decode_integer(index, p, buf_end, 4);
   } else {
-    ink_assert(ftype == HPACK_FIELD_NOINDEX_LITERAL);
-    len = decode_integer(index, p, buf_end, 4);
+    ink_assert(ftype == HpackField::NOINDEX_LITERAL);
+    len = xpack_decode_integer(index, p, buf_end, 4);
   }
 
-  if (len == HPACK_ERROR_COMPRESSION_ERROR) {
+  if (len == XPACK_ERROR_COMPRESSION_ERROR) {
     return HPACK_ERROR_COMPRESSION_ERROR;
   }
 
@@ -796,10 +704,10 @@ decode_literal_header_field(MIMEFieldWrapper &header, const uint8_t *buf_start,
     }
   } else {
     char *name_str        = nullptr;
-    uint32_t name_str_len = 0;
+    uint64_t name_str_len = 0;
 
-    len = decode_string(arena, &name_str, name_str_len, p, buf_end);
-    if (len == HPACK_ERROR_COMPRESSION_ERROR) {
+    len = xpack_decode_string(arena, &name_str, name_str_len, p, buf_end);
+    if (len == XPACK_ERROR_COMPRESSION_ERROR) {
       return HPACK_ERROR_COMPRESSION_ERROR;
     }
 
@@ -818,10 +726,10 @@ decode_literal_header_field(MIMEFieldWrapper &header, const uint8_t *buf_start,
 
   // Decode header field value
   char *value_str        = nullptr;
-  uint32_t value_str_len = 0;
+  uint64_t value_str_len = 0;
 
-  len = decode_string(arena, &value_str, value_str_len, p, buf_end);
-  if (len == HPACK_ERROR_COMPRESSION_ERROR) {
+  len = xpack_decode_string(arena, &value_str, value_str_len, p, buf_end);
+  if (len == XPACK_ERROR_COMPRESSION_ERROR) {
     return HPACK_ERROR_COMPRESSION_ERROR;
   }
 
@@ -864,9 +772,9 @@ update_dynamic_table_size(const uint8_t *buf_start, const uint8_t *buf_end, Hpac
   }
 
   // Update header table size if its required.
-  uint32_t size = 0;
-  int64_t len   = decode_integer(size, buf_start, buf_end, 5);
-  if (len == HPACK_ERROR_COMPRESSION_ERROR) {
+  uint64_t size = 0;
+  int64_t len   = xpack_decode_integer(size, buf_start, buf_end, 5);
+  if (len == XPACK_ERROR_COMPRESSION_ERROR) {
     return HPACK_ERROR_COMPRESSION_ERROR;
   }
 
@@ -874,9 +782,7 @@ update_dynamic_table_size(const uint8_t *buf_start, const uint8_t *buf_end, Hpac
     return HPACK_ERROR_COMPRESSION_ERROR;
   }
 
-  if (indexing_table.update_maximum_size(size) == false) {
-    return HPACK_ERROR_COMPRESSION_ERROR;
-  }
+  indexing_table.update_maximum_size(size);
 
   return len;
 }
@@ -899,10 +805,10 @@ hpack_decode_header_block(HpackIndexingTable &indexing_table, HTTPHdr *hdr, cons
     // decode a header field encoded by HPACK
     MIMEField *field = mime_field_create(heap, hh->m_fields_impl);
     MIMEFieldWrapper header(field, heap, hh->m_fields_impl);
-    HpackFieldType ftype = hpack_parse_field_type(*cursor);
+    HpackField ftype = hpack_parse_field_type(*cursor);
 
     switch (ftype) {
-    case HPACK_FIELD_INDEX:
+    case HpackField::INDEX:
       read_bytes = decode_indexed_header_field(header, cursor, in_buf_end, indexing_table);
       if (read_bytes == HPACK_ERROR_COMPRESSION_ERROR) {
         return HPACK_ERROR_COMPRESSION_ERROR;
@@ -910,9 +816,9 @@ hpack_decode_header_block(HpackIndexingTable &indexing_table, HTTPHdr *hdr, cons
       cursor += read_bytes;
       header_field_started = true;
       break;
-    case HPACK_FIELD_INDEXED_LITERAL:
-    case HPACK_FIELD_NOINDEX_LITERAL:
-    case HPACK_FIELD_NEVERINDEX_LITERAL:
+    case HpackField::INDEXED_LITERAL:
+    case HpackField::NOINDEX_LITERAL:
+    case HpackField::NEVERINDEX_LITERAL:
       read_bytes = decode_literal_header_field(header, cursor, in_buf_end, indexing_table);
       if (read_bytes == HPACK_ERROR_COMPRESSION_ERROR) {
         return HPACK_ERROR_COMPRESSION_ERROR;
@@ -924,7 +830,7 @@ hpack_decode_header_block(HpackIndexingTable &indexing_table, HTTPHdr *hdr, cons
       cursor += read_bytes;
       header_field_started = true;
       break;
-    case HPACK_FIELD_TABLESIZE_UPDATE:
+    case HpackField::TABLESIZE_UPDATE:
       if (header_field_started) {
         return HPACK_ERROR_COMPRESSION_ERROR;
       }
@@ -984,7 +890,7 @@ hpack_encode_header_block(HpackIndexingTable &indexing_table, uint8_t *out_buf,
 
   MIMEFieldIter field_iter;
   for (MIMEField *field = hdr->iter_get_first(&field_iter); field != nullptr; field = hdr->iter_get_next(&field_iter)) {
-    HpackFieldType field_type;
+    HpackField field_type;
     MIMEFieldWrapper header(field, hdr->m_heap, hdr->m_http->m_fields_impl);
     int name_len;
     int value_len;
@@ -995,20 +901,20 @@ hpack_encode_header_block(HpackIndexingTable &indexing_table, uint8_t *out_buf,
     // - Short Cookie header should not be indexed because of low entropy
     if ((ptr_len_casecmp(name, name_len, MIME_FIELD_COOKIE, MIME_LEN_COOKIE) == 0 && value_len < 20) ||
         (ptr_len_casecmp(name, name_len, MIME_FIELD_AUTHORIZATION, MIME_LEN_AUTHORIZATION) == 0)) {
-      field_type = HPACK_FIELD_NEVERINDEX_LITERAL;
+      field_type = HpackField::NEVERINDEX_LITERAL;
     } else {
-      field_type = HPACK_FIELD_INDEXED_LITERAL;
+      field_type = HpackField::INDEXED_LITERAL;
     }
     const HpackLookupResult result = indexing_table.lookup(header);
     switch (result.match_type) {
-    case HPACK_NO_MATCH:
+    case HpackMatch::NONE:
       written = encode_literal_header_field_with_new_name(cursor, out_buf_end, header, indexing_table, field_type);
       break;
-    case HPACK_NAME_MATCH:
+    case HpackMatch::NAME:
       written =
         encode_literal_header_field_with_indexed_name(cursor, out_buf_end, header, result.index, indexing_table, field_type);
       break;
-    case HPACK_EXACT_MATCH:
+    case HpackMatch::EXACT:
       written = encode_indexed_header_field(cursor, out_buf_end, result.index);
       break;
     default:
