@@ -45,10 +45,19 @@ const unsigned HTTP2_LEN_STATUS    = countof(":status") - 1;
 static size_t HTTP2_LEN_STATUS_VALUE_STR         = 3;
 static const uint32_t HTTP2_MAX_TABLE_SIZE_LIMIT = 64 * 1024;
 
+namespace
+{
+struct Http2HeaderName {
+  const char *name = nullptr;
+  int name_len     = 0;
+};
+
+Http2HeaderName http2_connection_specific_headers[5] = {};
+} // namespace
+
 // Statistics
 RecRawStatBlock *http2_rsb;
-static const char *const HTTP2_STAT_CURRENT_CLIENT_SESSION_NAME    = "proxy.process.http2.current_client_sessions"; // DEPRECATED
-static const char *const HTTP2_STAT_CURRENT_CLIENT_CONNECTION_NAME = "proxy.process.http2.current_client_connections";
+static const char *const HTTP2_STAT_CURRENT_CLIENT_CONNECTION_NAME        = "proxy.process.http2.current_client_connections";
 static const char *const HTTP2_STAT_CURRENT_ACTIVE_CLIENT_CONNECTION_NAME = "proxy.process.http2.current_active_client_connections";
 static const char *const HTTP2_STAT_CURRENT_CLIENT_STREAM_NAME            = "proxy.process.http2.current_client_streams";
 static const char *const HTTP2_STAT_TOTAL_CLIENT_STREAM_NAME              = "proxy.process.http2.total_client_streams";
@@ -71,8 +80,6 @@ static const char *const HTTP2_STAT_MAX_PING_FRAMES_PER_MINUTE_EXCEEDED_NAME =
   "proxy.process.http2.max_ping_frames_per_minute_exceeded";
 static const char *const HTTP2_STAT_MAX_PRIORITY_FRAMES_PER_MINUTE_EXCEEDED_NAME =
   "proxy.process.http2.max_priority_frames_per_minute_exceeded";
-static const char *const HTTP2_STAT_MAX_RST_STREAM_FRAMES_PER_MINUTE_EXCEEDED_NAME =
-  "proxy.process.http2.max_rst_stream_frames_per_minute_exceeded";
 static const char *const HTTP2_STAT_INSUFFICIENT_AVG_WINDOW_UPDATE_NAME = "proxy.process.http2.insufficient_avg_window_update";
 
 union byte_pointer {
@@ -100,6 +107,7 @@ write_and_advance(byte_pointer &dst, uint32_t src)
 {
   byte_addressable_value<uint32_t> pval;
 
+  // cppcheck-suppress unreadVariable ; it's an union and be read as pval.bytes
   pval.value = htonl(src);
   memcpy(dst.u8, pval.bytes, sizeof(pval.bytes));
   dst.u8 += sizeof(pval.bytes);
@@ -110,6 +118,7 @@ write_and_advance(byte_pointer &dst, uint16_t src)
 {
   byte_addressable_value<uint16_t> pval;
 
+  // cppcheck-suppress unreadVariable ; it's an union and be read as pval.bytes
   pval.value = htons(src);
   memcpy(dst.u8, pval.bytes, sizeof(pval.bytes));
   dst.u8 += sizeof(pval.bytes);
@@ -230,6 +239,7 @@ http2_write_frame_header(const Http2FrameHeader &hdr, IOVec iov)
   }
 
   byte_addressable_value<uint32_t> length;
+  // cppcheck-suppress unreadVariable ; it's an union and be read as pval.bytes
   length.value = htonl(hdr.length);
   // MSB length.bytes[0] is unused.
   write_and_advance(ptr, length.bytes[1]);
@@ -411,37 +421,23 @@ http2_convert_header_from_2_to_1_1(HTTPHdr *headers)
   ink_assert(http_hdr_type_get(headers->m_http) != HTTP_TYPE_UNKNOWN);
 
   if (http_hdr_type_get(headers->m_http) == HTTP_TYPE_REQUEST) {
-    const char *scheme, *authority, *path, *method;
-    int scheme_len, authority_len, path_len, method_len;
+    const char *scheme, *authority, *path;
+    int scheme_len, authority_len, path_len;
 
     // Get values of :scheme, :authority and :path to assemble requested URL
-    if ((field = headers->field_find(HTTP2_VALUE_SCHEME, HTTP2_LEN_SCHEME)) != nullptr &&
-        field->value_is_valid(is_control_BIT | is_ws_BIT)) {
+    if ((field = headers->field_find(HTTP2_VALUE_SCHEME, HTTP2_LEN_SCHEME)) != nullptr && field->value_is_valid()) {
       scheme = field->value_get(&scheme_len);
-
-      const char *scheme_wks;
-
-      int scheme_wks_idx = hdrtoken_tokenize(scheme, scheme_len, &scheme_wks);
-
-      if (!(scheme_wks_idx > 0 && hdrtoken_wks_to_token_type(scheme_wks) == HDRTOKEN_TYPE_SCHEME)) {
-        // unkown scheme, validate the scheme
-        if (!validate_scheme({scheme, static_cast<size_t>(scheme_len)})) {
-          return PARSE_RESULT_ERROR;
-        }
-      }
     } else {
       return PARSE_RESULT_ERROR;
     }
 
-    if ((field = headers->field_find(HTTP2_VALUE_AUTHORITY, HTTP2_LEN_AUTHORITY)) != nullptr &&
-        field->value_is_valid(is_control_BIT | is_ws_BIT)) {
+    if ((field = headers->field_find(HTTP2_VALUE_AUTHORITY, HTTP2_LEN_AUTHORITY)) != nullptr && field->value_is_valid()) {
       authority = field->value_get(&authority_len);
     } else {
       return PARSE_RESULT_ERROR;
     }
 
-    if ((field = headers->field_find(HTTP2_VALUE_PATH, HTTP2_LEN_PATH)) != nullptr &&
-        field->value_is_valid(is_control_BIT | is_ws_BIT)) {
+    if ((field = headers->field_find(HTTP2_VALUE_PATH, HTTP2_LEN_PATH)) != nullptr && field->value_is_valid()) {
       path = field->value_get(&path_len);
     } else {
       return PARSE_RESULT_ERROR;
@@ -461,9 +457,9 @@ http2_convert_header_from_2_to_1_1(HTTPHdr *headers)
     arena.str_free(url);
 
     // Get value of :method
-    if ((field = headers->field_find(HTTP2_VALUE_METHOD, HTTP2_LEN_METHOD)) != nullptr &&
-        field->value_is_valid(is_control_BIT | is_ws_BIT)) {
-      method = field->value_get(&method_len);
+    if ((field = headers->field_find(HTTP2_VALUE_METHOD, HTTP2_LEN_METHOD)) != nullptr && field->value_is_valid()) {
+      int method_len;
+      const char *method = field->value_get(&method_len);
 
       int method_wks_idx = hdrtoken_tokenize(method, method_len);
       http_hdr_method_set(headers->m_heap, headers->m_http, method, method_wks_idx, method_len, false);
@@ -487,11 +483,14 @@ http2_convert_header_from_2_to_1_1(HTTPHdr *headers)
     headers->field_delete(HTTP2_VALUE_AUTHORITY, HTTP2_LEN_AUTHORITY);
     headers->field_delete(HTTP2_VALUE_PATH, HTTP2_LEN_PATH);
   } else {
-    int status_len;
-    const char *status;
+    // Set HTTP Version 1.1
+    int32_t version = HTTP_VERSION(1, 1);
+    http_hdr_version_set(headers->m_http, version);
 
+    // Set status from :status
     if ((field = headers->field_find(HTTP2_VALUE_STATUS, HTTP2_LEN_STATUS)) != nullptr) {
-      status = field->value_get(&status_len);
+      int status_len;
+      const char *status = field->value_get(&status_len);
       headers->status_set(http_parse_status(status, status + status_len));
     } else {
       return PARSE_RESULT_ERROR;
@@ -503,8 +502,8 @@ http2_convert_header_from_2_to_1_1(HTTPHdr *headers)
 
   // Check validity of all names and values
   MIMEFieldIter iter;
-  for (const MIMEField *field = headers->iter_get_first(&iter); field != nullptr; field = headers->iter_get_next(&iter)) {
-    if (!field->name_is_valid(is_control_BIT | is_ws_BIT) || !field->value_is_valid()) {
+  for (auto *mf = headers->iter_get_first(&iter); mf != nullptr; mf = headers->iter_get_next(&iter)) {
+    if (!mf->name_is_valid() || !mf->value_is_valid()) {
       return PARSE_RESULT_ERROR;
     }
   }
@@ -512,84 +511,146 @@ http2_convert_header_from_2_to_1_1(HTTPHdr *headers)
   return PARSE_RESULT_DONE;
 }
 
+/**
+  Initialize HTTPHdr for HTTP/2
+
+  Reserve HTTP/2 Pseudo-Header Fields in front of HTTPHdr. Value of these header fields will be set by
+  `http2_convert_header_from_1_1_to_2()`. When a HTTPHdr for HTTP/2 headers is created, this should be called immediately.
+  Because all pseudo-header fields MUST appear in the header block before regular header fields.
+ */
 void
-http2_generate_h2_header_from_1_1(HTTPHdr *headers, HTTPHdr *h2_headers)
+http2_init_pseudo_headers(HTTPHdr &hdr)
 {
-  h2_headers->create(http_hdr_type_get(headers->m_http));
+  switch (http_hdr_type_get(hdr.m_http)) {
+  case HTTP_TYPE_REQUEST: {
+    MIMEField *method = hdr.field_create(HTTP2_VALUE_METHOD, HTTP2_LEN_METHOD);
+    hdr.field_attach(method);
 
-  if (http_hdr_type_get(headers->m_http) == HTTP_TYPE_RESPONSE) {
-    // Add ':status' header field
-    char status_str[HTTP2_LEN_STATUS_VALUE_STR + 1];
-    snprintf(status_str, sizeof(status_str), "%d", headers->status_get());
-    MIMEField *status_field = h2_headers->field_create(HTTP2_VALUE_STATUS, HTTP2_LEN_STATUS);
-    status_field->value_set(h2_headers->m_heap, h2_headers->m_mime, status_str, HTTP2_LEN_STATUS_VALUE_STR);
-    h2_headers->field_attach(status_field);
+    MIMEField *scheme = hdr.field_create(HTTP2_VALUE_SCHEME, HTTP2_LEN_SCHEME);
+    hdr.field_attach(scheme);
 
-  } else if (http_hdr_type_get(headers->m_http) == HTTP_TYPE_REQUEST) {
-    MIMEField *field;
-    const char *value;
-    int value_len;
+    MIMEField *authority = hdr.field_create(HTTP2_VALUE_AUTHORITY, HTTP2_LEN_AUTHORITY);
+    hdr.field_attach(authority);
 
-    // Add ':authority' header field
-    field = h2_headers->field_create(HTTP2_VALUE_AUTHORITY, HTTP2_LEN_AUTHORITY);
-    value = headers->host_get(&value_len);
-    if (headers->is_port_in_header()) {
-      int port            = headers->port_get();
-      char *host_and_port = (char *)ats_malloc(value_len + 8);
-      value_len           = snprintf(host_and_port, value_len + 8, "%.*s:%d", value_len, value, port);
-      field->value_set(h2_headers->m_heap, h2_headers->m_mime, host_and_port, value_len);
-      ats_free(host_and_port);
+    MIMEField *path = hdr.field_create(HTTP2_VALUE_PATH, HTTP2_LEN_PATH);
+    hdr.field_attach(path);
+
+    break;
+  }
+  case HTTP_TYPE_RESPONSE: {
+    MIMEField *status = hdr.field_create(HTTP2_VALUE_STATUS, HTTP2_LEN_STATUS);
+    hdr.field_attach(status);
+
+    break;
+  }
+  default:
+    ink_abort("HTTP_TYPE_UNKNOWN");
+  }
+}
+
+/**
+  Convert HTTP/1.1 HTTPHdr to HTTP/2
+
+  Assuming HTTP/2 Pseudo-Header Fields are reserved by `http2_init_pseudo_headers()`.
+ */
+ParseResult
+http2_convert_header_from_1_1_to_2(HTTPHdr *headers)
+{
+  switch (http_hdr_type_get(headers->m_http)) {
+  case HTTP_TYPE_REQUEST: {
+    // :method
+    if (MIMEField *field = headers->field_find(HTTP2_VALUE_METHOD, HTTP2_LEN_METHOD); field != nullptr) {
+      int value_len;
+      const char *value = headers->method_get(&value_len);
+
+      field->value_set(headers->m_heap, headers->m_mime, value, value_len);
     } else {
-      field->value_set(h2_headers->m_heap, h2_headers->m_mime, value, value_len);
+      ink_abort("initialize HTTP/2 pseudo-headers");
+      return PARSE_RESULT_ERROR;
     }
-    h2_headers->field_attach(field);
 
-    // Add ':method' header field
-    field = h2_headers->field_create(HTTP2_VALUE_METHOD, HTTP2_LEN_METHOD);
-    value = headers->method_get(&value_len);
-    field->value_set(h2_headers->m_heap, h2_headers->m_mime, value, value_len);
-    h2_headers->field_attach(field);
+    // :scheme
+    if (MIMEField *field = headers->field_find(HTTP2_VALUE_SCHEME, HTTP2_LEN_SCHEME); field != nullptr) {
+      int value_len;
+      const char *value = headers->scheme_get(&value_len);
 
-    // Add ':path' header field
-    field      = h2_headers->field_create(HTTP2_VALUE_PATH, HTTP2_LEN_PATH);
-    value      = headers->path_get(&value_len);
-    char *path = (char *)ats_malloc(value_len + 1);
-    path[0]    = '/';
-    memcpy(path + 1, value, value_len);
-    field->value_set(h2_headers->m_heap, h2_headers->m_mime, path, value_len + 1);
-    ats_free(path);
-    h2_headers->field_attach(field);
+      if (value != nullptr) {
+        field->value_set(headers->m_heap, headers->m_mime, value, value_len);
+      } else {
+        field->value_set(headers->m_heap, headers->m_mime, URL_SCHEME_HTTPS, URL_LEN_HTTPS);
+      }
+    } else {
+      ink_abort("initialize HTTP/2 pseudo-headers");
+      return PARSE_RESULT_ERROR;
+    }
 
-    // Add ':scheme' header field
-    field = h2_headers->field_create(HTTP2_VALUE_SCHEME, HTTP2_LEN_SCHEME);
-    value = headers->scheme_get(&value_len);
-    field->value_set(h2_headers->m_heap, h2_headers->m_mime, value, value_len);
-    h2_headers->field_attach(field);
+    // :authority
+    if (MIMEField *field = headers->field_find(HTTP2_VALUE_AUTHORITY, HTTP2_LEN_AUTHORITY); field != nullptr) {
+      int value_len;
+      const char *value = headers->host_get(&value_len);
+
+      if (headers->is_port_in_header()) {
+        int port            = headers->port_get();
+        char *host_and_port = static_cast<char *>(ats_malloc(value_len + 8));
+        value_len           = snprintf(host_and_port, value_len + 8, "%.*s:%d", value_len, value, port);
+
+        field->value_set(headers->m_heap, headers->m_mime, host_and_port, value_len);
+        ats_free(host_and_port);
+      } else {
+        field->value_set(headers->m_heap, headers->m_mime, value, value_len);
+      }
+    } else {
+      ink_abort("initialize HTTP/2 pseudo-headers");
+      return PARSE_RESULT_ERROR;
+    }
+
+    // :path
+    if (MIMEField *field = headers->field_find(HTTP2_VALUE_PATH, HTTP2_LEN_PATH); field != nullptr) {
+      int value_len;
+      const char *value = headers->path_get(&value_len);
+      char *path        = static_cast<char *>(ats_malloc(value_len + 1));
+      path[0]           = '/';
+      memcpy(path + 1, value, value_len);
+
+      field->value_set(headers->m_heap, headers->m_mime, path, value_len + 1);
+      ats_free(path);
+    } else {
+      ink_abort("initialize HTTP/2 pseudo-headers");
+      return PARSE_RESULT_ERROR;
+    }
+
+    // TODO: remove host/Host header
+    // [RFC 7540] 8.1.2.3. Clients that generate HTTP/2 requests directly SHOULD use the ":authority" pseudo-header field instead
+    // of the Host header field.
+
+    break;
+  }
+  case HTTP_TYPE_RESPONSE: {
+    // :status
+    if (MIMEField *field = headers->field_find(HTTP2_VALUE_STATUS, HTTP2_LEN_STATUS); field != nullptr) {
+      // ink_small_itoa() requires 5+ buffer length
+      char status_str[HTTP2_LEN_STATUS_VALUE_STR + 3];
+      mime_format_int(status_str, headers->status_get(), sizeof(status_str));
+
+      field->value_set(headers->m_heap, headers->m_mime, status_str, HTTP2_LEN_STATUS_VALUE_STR);
+    } else {
+      ink_abort("initialize HTTP/2 pseudo-headers");
+      return PARSE_RESULT_ERROR;
+    }
+    break;
+  }
+  default:
+    ink_abort("HTTP_TYPE_UNKNOWN");
   }
 
-  // Copy headers
   // Intermediaries SHOULD remove connection-specific header fields.
-  MIMEFieldIter field_iter;
-  for (MIMEField *field = headers->iter_get_first(&field_iter); field != nullptr; field = headers->iter_get_next(&field_iter)) {
-    const char *name;
-    int name_len;
-    const char *value;
-    int value_len;
-    name = field->name_get(&name_len);
-    if ((name_len == MIME_LEN_CONNECTION && strncasecmp(name, MIME_FIELD_CONNECTION, name_len) == 0) ||
-        (name_len == MIME_LEN_KEEP_ALIVE && strncasecmp(name, MIME_FIELD_KEEP_ALIVE, name_len) == 0) ||
-        (name_len == MIME_LEN_PROXY_CONNECTION && strncasecmp(name, MIME_FIELD_PROXY_CONNECTION, name_len) == 0) ||
-        (name_len == MIME_LEN_TRANSFER_ENCODING && strncasecmp(name, MIME_FIELD_TRANSFER_ENCODING, name_len) == 0) ||
-        (name_len == MIME_LEN_UPGRADE && strncasecmp(name, MIME_FIELD_UPGRADE, name_len) == 0)) {
-      continue;
+  for (auto &h : http2_connection_specific_headers) {
+    if (MIMEField *field = headers->field_find(h.name, h.name_len); field != nullptr) {
+      headers->field_delete(field);
     }
-    MIMEField *newfield;
-    name     = field->name_get(&name_len);
-    newfield = h2_headers->field_create(name, name_len);
-    value    = field->value_get(&value_len);
-    newfield->value_set(h2_headers->m_heap, h2_headers->m_mime, value, value_len);
-    h2_headers->field_attach(newfield);
   }
+
+  return PARSE_RESULT_DONE;
 }
 
 Http2ErrorCode
@@ -603,6 +664,7 @@ http2_encode_header_blocks(HTTPHdr *in, uint8_t *out, uint32_t out_len, uint32_t
   if (maximum_table_size == hpack_get_maximum_table_size(handle)) {
     maximum_table_size = -1;
   }
+
   // TODO: It would be better to split Cookie header value
   int64_t result = hpack_encode_header_block(handle, out, out_len, in, maximum_table_size);
   if (result < 0) {
@@ -662,13 +724,6 @@ http2_decode_header_blocks(HTTPHdr *hdr, const uint8_t *buf_start, const uint32_
         return Http2ErrorCode::HTTP2_ERROR_PROTOCOL_ERROR;
       }
     }
-    // Check whether header field name is lower case
-    // This check should be here but it will fail because WKSs in MIMEField is old fashioned.
-    // for (uint32_t i = 0; i < len; ++i) {
-    //   if (ParseRules::is_upalpha(value[i])) {
-    //     return Http2ErrorCode::HTTP2_ERROR_PROTOCOL_ERROR;
-    //   }
-    // }
   }
 
   // rfc7540,sec8.1.2.2: Any message containing connection-specific header
@@ -728,31 +783,33 @@ http2_decode_header_blocks(HTTPHdr *hdr, const uint8_t *buf_start, const uint32_
 }
 
 // Initialize this subsystem with librecords configs (for now)
-uint32_t Http2::max_concurrent_streams_in        = 100;
-uint32_t Http2::min_concurrent_streams_in        = 10;
-uint32_t Http2::max_active_streams_in            = 0;
-bool Http2::throttling                           = false;
-uint32_t Http2::stream_priority_enabled          = 0;
-uint32_t Http2::initial_window_size              = 65535;
-uint32_t Http2::max_frame_size                   = 16384;
-uint32_t Http2::header_table_size                = 4096;
-uint32_t Http2::max_header_list_size             = 4294967295;
-uint32_t Http2::accept_no_activity_timeout       = 120;
-uint32_t Http2::no_activity_timeout_in           = 120;
-uint32_t Http2::active_timeout_in                = 0;
-uint32_t Http2::push_diary_size                  = 256;
-uint32_t Http2::zombie_timeout_in                = 0;
-float Http2::stream_error_rate_threshold         = 0.1;
-uint32_t Http2::max_settings_per_frame           = 7;
-uint32_t Http2::max_settings_per_minute          = 14;
-uint32_t Http2::max_settings_frames_per_minute   = 14;
-uint32_t Http2::max_ping_frames_per_minute       = 60;
-uint32_t Http2::max_priority_frames_per_minute   = 120;
-uint32_t Http2::max_rst_stream_frames_per_minute = 200;
-float Http2::min_avg_window_update               = 2560.0;
-uint32_t Http2::con_slow_log_threshold           = 0;
-uint32_t Http2::stream_slow_log_threshold        = 0;
-uint32_t Http2::header_table_size_limit          = 65536;
+uint32_t Http2::max_concurrent_streams_in      = 100;
+uint32_t Http2::min_concurrent_streams_in      = 10;
+uint32_t Http2::max_active_streams_in          = 0;
+bool Http2::throttling                         = false;
+uint32_t Http2::stream_priority_enabled        = 0;
+uint32_t Http2::initial_window_size            = 65535;
+uint32_t Http2::max_frame_size                 = 16384;
+uint32_t Http2::header_table_size              = 4096;
+uint32_t Http2::max_header_list_size           = 4294967295;
+uint32_t Http2::accept_no_activity_timeout     = 120;
+uint32_t Http2::no_activity_timeout_in         = 120;
+uint32_t Http2::active_timeout_in              = 0;
+uint32_t Http2::push_diary_size                = 256;
+uint32_t Http2::zombie_timeout_in              = 0;
+float Http2::stream_error_rate_threshold       = 0.1;
+uint32_t Http2::max_settings_per_frame         = 7;
+uint32_t Http2::max_settings_per_minute        = 14;
+uint32_t Http2::max_settings_frames_per_minute = 14;
+uint32_t Http2::max_ping_frames_per_minute     = 60;
+uint32_t Http2::max_priority_frames_per_minute = 120;
+float Http2::min_avg_window_update             = 2560.0;
+uint32_t Http2::con_slow_log_threshold         = 0;
+uint32_t Http2::stream_slow_log_threshold      = 0;
+uint32_t Http2::header_table_size_limit        = 65536;
+uint32_t Http2::write_buffer_block_size        = 262144;
+float Http2::write_size_threshold              = 0.5;
+uint32_t Http2::write_time_threshold           = 100;
 
 void
 Http2::init()
@@ -776,11 +833,13 @@ Http2::init()
   REC_EstablishStaticConfigInt32U(max_settings_frames_per_minute, "proxy.config.http2.max_settings_frames_per_minute");
   REC_EstablishStaticConfigInt32U(max_ping_frames_per_minute, "proxy.config.http2.max_ping_frames_per_minute");
   REC_EstablishStaticConfigInt32U(max_priority_frames_per_minute, "proxy.config.http2.max_priority_frames_per_minute");
-  REC_EstablishStaticConfigInt32U(max_rst_stream_frames_per_minute, "proxy.config.http2.max_rst_stream_frames_per_minute");
   REC_EstablishStaticConfigFloat(min_avg_window_update, "proxy.config.http2.min_avg_window_update");
   REC_EstablishStaticConfigInt32U(con_slow_log_threshold, "proxy.config.http2.connection.slow.log.threshold");
   REC_EstablishStaticConfigInt32U(stream_slow_log_threshold, "proxy.config.http2.stream.slow.log.threshold");
   REC_EstablishStaticConfigInt32U(header_table_size_limit, "proxy.config.http2.header_table_size_limit");
+  REC_EstablishStaticConfigInt32U(write_buffer_block_size, "proxy.config.http2.write_buffer_block_size");
+  REC_EstablishStaticConfigFloat(write_size_threshold, "proxy.config.http2.write_size_threshold");
+  REC_EstablishStaticConfigInt32U(write_time_threshold, "proxy.config.http2.write_time_threshold");
 
   // If any settings is broken, ATS should not start
   ink_release_assert(http2_settings_parameter_is_valid({HTTP2_SETTINGS_MAX_CONCURRENT_STREAMS, max_concurrent_streams_in}));
@@ -798,8 +857,6 @@ Http2::init()
 
   // Setup statistics
   http2_rsb = RecAllocateRawStatBlock(static_cast<int>(HTTP2_N_STATS));
-  RecRegisterRawStat(http2_rsb, RECT_PROCESS, HTTP2_STAT_CURRENT_CLIENT_SESSION_NAME, RECD_INT, RECP_NON_PERSISTENT,
-                     static_cast<int>(HTTP2_STAT_CURRENT_CLIENT_SESSION_COUNT), RecRawStatSyncSum);
   RecRegisterRawStat(http2_rsb, RECT_PROCESS, HTTP2_STAT_CURRENT_CLIENT_CONNECTION_NAME, RECD_INT, RECP_NON_PERSISTENT,
                      static_cast<int>(HTTP2_STAT_CURRENT_CLIENT_SESSION_COUNT), RecRawStatSyncSum);
   HTTP2_CLEAR_DYN_STAT(HTTP2_STAT_CURRENT_CLIENT_SESSION_COUNT);
@@ -843,10 +900,29 @@ Http2::init()
                      static_cast<int>(HTTP2_STAT_MAX_PING_FRAMES_PER_MINUTE_EXCEEDED), RecRawStatSyncSum);
   RecRegisterRawStat(http2_rsb, RECT_PROCESS, HTTP2_STAT_MAX_PRIORITY_FRAMES_PER_MINUTE_EXCEEDED_NAME, RECD_INT, RECP_PERSISTENT,
                      static_cast<int>(HTTP2_STAT_MAX_PRIORITY_FRAMES_PER_MINUTE_EXCEEDED), RecRawStatSyncSum);
-  RecRegisterRawStat(http2_rsb, RECT_PROCESS, HTTP2_STAT_MAX_RST_STREAM_FRAMES_PER_MINUTE_EXCEEDED_NAME, RECD_INT, RECP_PERSISTENT,
-                     static_cast<int>(HTTP2_STAT_MAX_RST_STREAM_FRAMES_PER_MINUTE_EXCEEDED), RecRawStatSyncSum);
   RecRegisterRawStat(http2_rsb, RECT_PROCESS, HTTP2_STAT_INSUFFICIENT_AVG_WINDOW_UPDATE_NAME, RECD_INT, RECP_PERSISTENT,
                      static_cast<int>(HTTP2_STAT_INSUFFICIENT_AVG_WINDOW_UPDATE), RecRawStatSyncSum);
+
+  http2_init();
+}
+
+/**
+  mime_init() needs to be called
+ */
+void
+http2_init()
+{
+  ink_assert(MIME_FIELD_CONNECTION != nullptr);
+  ink_assert(MIME_FIELD_KEEP_ALIVE != nullptr);
+  ink_assert(MIME_FIELD_PROXY_CONNECTION != nullptr);
+  ink_assert(MIME_FIELD_TRANSFER_ENCODING != nullptr);
+  ink_assert(MIME_FIELD_UPGRADE != nullptr);
+
+  http2_connection_specific_headers[0] = {MIME_FIELD_CONNECTION, MIME_LEN_CONNECTION};
+  http2_connection_specific_headers[1] = {MIME_FIELD_KEEP_ALIVE, MIME_LEN_KEEP_ALIVE};
+  http2_connection_specific_headers[2] = {MIME_FIELD_PROXY_CONNECTION, MIME_LEN_PROXY_CONNECTION};
+  http2_connection_specific_headers[3] = {MIME_FIELD_TRANSFER_ENCODING, MIME_LEN_TRANSFER_ENCODING};
+  http2_connection_specific_headers[4] = {MIME_FIELD_UPGRADE, MIME_LEN_UPGRADE};
 }
 
 #if TS_HAS_TESTS
