@@ -23,10 +23,12 @@
 
 #pragma once
 
+#include <atomic>
 #include "HTTP2.h"
 #include "HPACK.h"
 #include "Http2Stream.h"
 #include "Http2DependencyTree.h"
+#include "Http2FrequencyCounter.h"
 
 class Http2ClientSession;
 
@@ -113,35 +115,14 @@ private:
 class Http2ConnectionState : public Continuation
 {
 public:
-  Http2ConnectionState()
-    : Continuation(NULL),
-      ua_session(NULL),
-      dependency_tree(NULL),
-      stream_list(),
-      latest_streamid_in(0),
-      latest_streamid_out(0),
-      stream_requests(0),
-      client_streams_in_count(0),
-      client_streams_out_count(0),
-      total_client_streams_count(0),
-      stream_error_count(0),
-      _client_rwnd(HTTP2_INITIAL_WINDOW_SIZE),
-      _server_rwnd(0),
-      continued_stream_id(0),
-      _scheduled(false),
-      fini_received(false),
-      recursion(0),
-      fini_event(nullptr)
-  {
-    SET_HANDLER(&Http2ConnectionState::main_event_handler);
-  }
+  Http2ConnectionState() : stream_list() { SET_HANDLER(&Http2ConnectionState::main_event_handler); }
 
   ProxyError rx_error_code;
   ProxyError tx_error_code;
-  Http2ClientSession *ua_session;
-  HpackHandle *local_hpack_handle;
-  HpackHandle *remote_hpack_handle;
-  DependencyTree *dependency_tree;
+  Http2ClientSession *ua_session   = nullptr;
+  HpackHandle *local_hpack_handle  = nullptr;
+  HpackHandle *remote_hpack_handle = nullptr;
+  DependencyTree *dependency_tree  = nullptr;
 
   // Settings.
   Http2ConnectionSettings server_settings;
@@ -154,12 +135,23 @@ public:
 
     local_hpack_handle  = new HpackHandle(HTTP2_HEADER_TABLE_SIZE);
     remote_hpack_handle = new HpackHandle(HTTP2_HEADER_TABLE_SIZE);
-    dependency_tree     = new DependencyTree(Http2::max_concurrent_streams_in);
+    if (Http2::stream_priority_enabled) {
+      dependency_tree = new DependencyTree(Http2::max_concurrent_streams_in);
+    }
   }
 
   void
   destroy()
   {
+    if (in_destroy) {
+      schedule_zombie_event();
+      return;
+    }
+    in_destroy = true;
+    if (shutdown_cont_event) {
+      shutdown_cont_event->cancel();
+      shutdown_cont_event = nullptr;
+    }
     cleanup_streams();
 
     delete local_hpack_handle;
@@ -173,7 +165,11 @@ public:
     if (fini_event) {
       fini_event->cancel();
     }
-    mutex = NULL; // magic happens - assigning to NULL frees the ProxyMutex
+    if (zombie_event) {
+      zombie_event->cancel();
+    }
+    // release the mutex after the events are cancelled and sessions are destroyed.
+    mutex = nullptr; // magic happens - assigning to nullptr frees the ProxyMutex
   }
 
   // Event handlers
@@ -185,9 +181,8 @@ public:
   Http2Stream *find_stream(Http2StreamId id) const;
   void restart_streams();
   bool delete_stream(Http2Stream *stream);
-  void release_stream(Http2Stream *stream);
+  void release_stream();
   void cleanup_streams();
-
   void restart_receiving(Http2Stream *stream);
   void update_initial_rwnd(Http2WindowSize new_size);
 
@@ -238,11 +233,18 @@ public:
     return client_streams_in_count;
   }
 
+  void
+  decrement_stream_count()
+  {
+    --total_client_streams_count;
+  }
+
   double
   get_stream_error_rate() const
   {
     int total = get_stream_requests();
-    if (total > 0) {
+
+    if (total >= (1 / Http2::stream_error_rate_threshold)) {
       return (double)stream_error_count / (double)total;
     } else {
       return 0;
@@ -261,7 +263,7 @@ public:
   void send_data_frames(Http2Stream *stream);
   Http2SendDataFrameResult send_a_data_frame(Http2Stream *stream, size_t &payload_length);
   void send_headers_frame(Http2Stream *stream);
-  void send_push_promise_frame(Http2Stream *stream, URL &url, const MIMEField *accept_encoding);
+  bool send_push_promise_frame(Http2Stream *stream, URL &url, const MIMEField *accept_encoding);
   void send_rst_stream_frame(Http2StreamId id, Http2ErrorCode ec);
   void send_settings_frame(const Http2ConnectionSettings &new_settings);
   void send_ping_frame(Http2StreamId id, uint8_t flag, const uint8_t *opaque_data);
@@ -271,7 +273,7 @@ public:
   bool
   is_state_closed() const
   {
-    return ua_session == NULL || fini_received;
+    return ua_session == nullptr || fini_received;
   }
 
   bool
@@ -290,6 +292,40 @@ public:
     }
   }
 
+  Http2ShutdownState
+  get_shutdown_state() const
+  {
+    return shutdown_state;
+  }
+
+  void
+  set_shutdown_state(Http2ShutdownState state, Http2ErrorCode reason = Http2ErrorCode::HTTP2_ERROR_NO_ERROR)
+  {
+    shutdown_state  = state;
+    shutdown_reason = reason;
+  }
+
+  // noncopyable
+  Http2ConnectionState(const Http2ConnectionState &) = delete;
+  Http2ConnectionState &operator=(const Http2ConnectionState &) = delete;
+
+  Event *
+  get_zombie_event()
+  {
+    return zombie_event;
+  }
+
+  void
+  schedule_zombie_event()
+  {
+    if (Http2::zombie_timeout_in) { // If we have zombie debugging enabled
+      if (zombie_event) {
+        zombie_event->cancel();
+      }
+      zombie_event = this_ethread()->schedule_in(this, HRTIME_SECONDS(Http2::zombie_timeout_in));
+    }
+  }
+
   void increment_received_settings_count(uint32_t count);
   uint32_t get_received_settings_count();
   void increment_received_settings_frame_count();
@@ -307,9 +343,6 @@ public:
   Http2ErrorCode decrement_server_rwnd(size_t amount);
 
 private:
-  Http2ConnectionState(const Http2ConnectionState &);            // noncopyable
-  Http2ConnectionState &operator=(const Http2ConnectionState &); // noncopyable
-
   unsigned _adjust_concurrent_stream();
 
   // NOTE: 'stream_list' has only active streams.
@@ -319,41 +352,33 @@ private:
   //   If given Stream Identifier is not found in stream_list and it is greater
   //   than latest_streamid_in, the state of Stream is IDLE.
   Queue<Http2Stream> stream_list;
-  Http2StreamId latest_streamid_in;
-  Http2StreamId latest_streamid_out;
-  int stream_requests;
+  Http2StreamId latest_streamid_in  = 0;
+  Http2StreamId latest_streamid_out = 0;
+  std::atomic<int> stream_requests  = 0;
 
   // Counter for current active streams which is started by client
-  uint32_t client_streams_in_count;
+  std::atomic<uint32_t> client_streams_in_count = 0;
 
-  // Counter for current acive streams which is started by server
-  uint32_t client_streams_out_count;
+  // Counter for current active streams which is started by server
+  std::atomic<uint32_t> client_streams_out_count = 0;
 
   // Counter for current active streams and streams in the process of shutting down
-  uint32_t total_client_streams_count;
+  std::atomic<uint32_t> total_client_streams_count = 0;
 
   // Counter for stream errors ATS sent
-  uint32_t stream_error_count;
+  uint32_t stream_error_count = 0;
 
   // Connection level window size
-  ssize_t _client_rwnd;
-  ssize_t _server_rwnd;
+  ssize_t _client_rwnd = HTTP2_INITIAL_WINDOW_SIZE;
+  ssize_t _server_rwnd = 0;
 
   std::vector<size_t> _recent_rwnd_increment = {SIZE_MAX, SIZE_MAX, SIZE_MAX, SIZE_MAX, SIZE_MAX};
   int _recent_rwnd_increment_index           = 0;
 
-  // Counter for settings received within last 60 seconds
-  // Each item holds a count for 30 seconds.
-  uint16_t settings_count[2]            = {0};
-  ink_hrtime settings_count_last_update = 0;
-  // Counters for frames received within last 60 seconds
-  // Each item in an array holds a count for 30 seconds.
-  uint16_t settings_frame_count[2]            = {0};
-  ink_hrtime settings_frame_count_last_update = 0;
-  uint16_t ping_frame_count[2]                = {0};
-  ink_hrtime ping_frame_count_last_update     = 0;
-  uint16_t priority_frame_count[2]            = {0};
-  ink_hrtime priority_frame_count_last_update = 0;
+  Http2FrequencyCounter _received_settings_counter;
+  Http2FrequencyCounter _received_settings_frame_counter;
+  Http2FrequencyCounter _received_ping_frame_counter;
+  Http2FrequencyCounter _received_priority_frame_counter;
 
   // NOTE: Id of stream which MUST receive CONTINUATION frame.
   //   - [RFC 7540] 6.2 HEADERS
@@ -362,12 +387,14 @@ private:
   //   - [RFC 7540] 6.10 CONTINUATION
   //     "If the END_HEADERS bit is not set, this frame MUST be followed by
   //     another CONTINUATION frame."
-  Http2StreamId continued_stream_id;
-  bool _scheduled;
-  bool fini_received;
-  int recursion;
+  Http2StreamId continued_stream_id = 0;
+  bool _scheduled                   = false;
+  bool fini_received                = false;
+  bool in_destroy                   = false;
+  int recursion                     = 0;
   Http2ShutdownState shutdown_state = HTTP2_SHUTDOWN_NONE;
   Http2ErrorCode shutdown_reason    = Http2ErrorCode::HTTP2_ERROR_MAX;
   Event *shutdown_cont_event        = nullptr;
-  Event *fini_event;
+  Event *fini_event                 = nullptr;
+  Event *zombie_event               = nullptr;
 };
