@@ -23,13 +23,14 @@
 
 #pragma once
 
-#include "ts/ink_platform.h"
-#include "ts/Diags.h"
+#include "tscore/ink_platform.h"
+#include "tscore/Diags.h"
 #include "LogFormat.h"
 #include "LogLimits.h"
 #include "LogAccess.h"
 
 class LogObject;
+class LogConfig;
 class LogBufferIterator;
 
 #define LOG_SEGMENT_COOKIE 0xaceface
@@ -87,7 +88,6 @@ struct LogBufferHeader {
 
   // some helper functions to return the header strings
 
-  char *fmt_name(); // not used
   char *fmt_fieldlist();
   char *fmt_printf();
   char *src_hostname();
@@ -96,15 +96,15 @@ struct LogBufferHeader {
 
 union LB_State {
   LB_State() : ival(0) {}
-  LB_State(volatile LB_State &vs) { ival = vs.ival; }
+  LB_State(LB_State &vs) { ival = vs.ival; }
   LB_State &
-  operator=(volatile LB_State &vs)
+  operator=(LB_State &vs)
   {
     ival = vs.ival;
     return *this;
   }
 
-  int64_t ival;
+  int64_t ival; // ival is used to help do an atomic CAS for struct s
   struct {
     uint32_t offset;           // buffer offset(bytes in buffer)
     uint16_t num_entries;      // number of entries in buffer
@@ -130,11 +130,13 @@ public:
     LB_BUFFER_TOO_SMALL
   };
 
-  LogBuffer(LogObject *owner, size_t size, size_t buf_align = LB_DEFAULT_ALIGN, size_t write_align = INK_MIN_ALIGN);
+  LogBuffer(const LogConfig *cfg, LogObject *owner, size_t size, size_t buf_align = LB_DEFAULT_ALIGN,
+            size_t write_align = INK_MIN_ALIGN);
   LogBuffer(LogObject *owner, LogBufferHeader *header);
   ~LogBuffer();
 
-  char &operator[](int idx)
+  char &
+  operator[](int idx)
   {
     ink_assert(idx >= 0);
     ink_assert((size_t)idx < m_size);
@@ -182,18 +184,18 @@ public:
   LINK(LogBuffer, link);
 
   // static variables
-  static vint32 M_ID;
+  static int32_t M_ID;
 
   // static functions
   static size_t max_entry_bytes();
   static int to_ascii(LogEntryHeader *entry, LogFormatType type, char *buf, int max_len, const char *symbol_str, char *printf_str,
-                      unsigned buffer_version, const char *alt_format = NULL);
+                      unsigned buffer_version, const char *alt_format = nullptr);
   static int resolve_custom_entry(LogFieldList *fieldlist, char *printf_str, char *read_from, char *write_to, int write_to_len,
-                                  long timestamp, long timestamp_us, unsigned buffer_version, LogFieldList *alt_fieldlist = NULL,
-                                  char *alt_printf_str = NULL);
+                                  long timestamp, long timestamp_us, unsigned buffer_version, LogFieldList *alt_fieldlist = nullptr,
+                                  char *alt_printf_str = nullptr);
 
   static void
-  destroy(LogBuffer *lb)
+  destroy(LogBuffer *&lb)
   {
     // ink_atomic_increment() returns the previous value, so when it was 1, we are
     // the thread that decremented to zero and should delete ...
@@ -201,6 +203,7 @@ public:
 
     if (refcnt == 1) {
       delete lb;
+      lb = nullptr;
     }
 
     ink_release_assert(refcnt >= 0);
@@ -221,18 +224,22 @@ private:
 
   uint32_t m_id; // unique buffer id (for debugging)
 public:
-  volatile LB_State m_state; // buffer state
-  volatile int m_references; // oustanding checkout_write references.
+  LB_State m_state; // buffer state
+  int m_references; // oustanding checkout_write references.
+
+  // noncopyable
+  // -- member functions that are not allowed --
+  LogBuffer(const LogBuffer &rhs) = delete;
+  LogBuffer &operator=(const LogBuffer &rhs) = delete;
+
 private:
   // private functions
-  size_t _add_buffer_header();
+  size_t _add_buffer_header(const LogConfig *cfg);
   unsigned add_header_str(const char *str, char *buf_ptr, unsigned buf_len);
   void freeLogBuffer();
 
   // -- member functions that are not allowed --
   LogBuffer();
-  LogBuffer(const LogBuffer &rhs);
-  LogBuffer &operator=(const LogBuffer &rhs);
 
   friend class LogBufferIterator;
 };
@@ -257,9 +264,9 @@ public:
   ~LogBufferList();
 
   void add(LogBuffer *lb);
-  LogBuffer *get(void);
+  LogBuffer *get();
   int
-  get_size(void)
+  get_size()
   {
     return m_size;
   }
@@ -279,6 +286,11 @@ public:
 
   LogEntryHeader *next();
 
+  // noncopyable
+  // -- member functions not allowed --
+  LogBufferIterator(const LogBufferIterator &) = delete;
+  LogBufferIterator &operator=(const LogBufferIterator &) = delete;
+
 private:
   bool m_in_network_order;
   char *m_next;
@@ -287,8 +299,6 @@ private:
 
   // -- member functions not allowed --
   LogBufferIterator();
-  LogBufferIterator(const LogBufferIterator &);
-  LogBufferIterator &operator=(const LogBufferIterator &);
 };
 
 /*-------------------------------------------------------------------------
@@ -299,7 +309,7 @@ private:
   -------------------------------------------------------------------------*/
 
 inline LogBufferIterator::LogBufferIterator(LogBufferHeader *header, bool in_network_order)
-  : m_in_network_order(in_network_order), m_next(0), m_iter_entry_count(0), m_buffer_entry_count(0)
+  : m_in_network_order(in_network_order), m_next(nullptr), m_iter_entry_count(0), m_buffer_entry_count(0)
 {
   ink_assert(header);
 
@@ -320,6 +330,4 @@ inline LogBufferIterator::LogBufferIterator(LogBufferHeader *header, bool in_net
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
 
-inline LogBufferIterator::~LogBufferIterator()
-{
-}
+inline LogBufferIterator::~LogBufferIterator() {}
