@@ -27,8 +27,9 @@
  This file implements the LogField object, which is the central
  representation of a logging field.
  ***************************************************************************/
-#include "ts/ink_platform.h"
+#include "tscore/ink_platform.h"
 
+#include "MIME.h"
 #include "LogUtils.h"
 #include "LogField.h"
 #include "LogBuffer.h"
@@ -75,17 +76,17 @@ LogSlice::LogSlice(char *str)
   m_start  = 0;
   m_end    = INT_MAX;
 
-  if ((a = strchr(str, '[')) == NULL) {
+  if ((a = strchr(str, '[')) == nullptr) {
     return;
   }
 
   *a++ = '\0';
-  if ((b = strchr(a, ':')) == NULL) {
+  if ((b = strchr(a, ':')) == nullptr) {
     return;
   }
 
   *b++ = '\0';
-  if ((c = strchr(b, ']')) == NULL) {
+  if ((c = strchr(b, ']')) == nullptr) {
     return;
   }
 
@@ -168,7 +169,7 @@ struct cmp_str {
     return ptr_len_casecmp(a._ptr, a._size, b._ptr, b._size) < 0;
   }
 };
-}
+} // namespace
 
 typedef std::map<ts::ConstBuffer, TSMilestonesType, cmp_str> milestone_map;
 static milestone_map m_milestone_map;
@@ -201,10 +202,12 @@ static const milestone milestones[] = {
   {"TS_MILESTONE_SM_FINISH", TS_MILESTONE_SM_FINISH},
   {"TS_MILESTONE_PLUGIN_ACTIVE", TS_MILESTONE_PLUGIN_ACTIVE},
   {"TS_MILESTONE_PLUGIN_TOTAL", TS_MILESTONE_PLUGIN_TOTAL},
+  {"TS_MILESTONE_TLS_HANDSHAKE_START", TS_MILESTONE_TLS_HANDSHAKE_START},
+  {"TS_MILESTONE_TLS_HANDSHAKE_END", TS_MILESTONE_TLS_HANDSHAKE_END},
 };
 
 void
-LogField::init_milestone_container(void)
+LogField::init_milestone_container()
 {
   if (m_milestone_map.empty()) {
     for (unsigned i = 0; i < countof(milestones); ++i) {
@@ -222,33 +225,33 @@ LogField::LogField(const char *name, const char *symbol, Type type, MarshalFunc
     m_container(NO_CONTAINER),
     m_marshal_func(marshal),
     m_unmarshal_func(unmarshal),
-    m_unmarshal_func_map(NULL),
+    m_unmarshal_func_map(nullptr),
     m_agg_op(NO_AGGREGATE),
     m_agg_cnt(0),
     m_agg_val(0),
     m_milestone1(TS_MILESTONE_LAST_ENTRY),
     m_milestone2(TS_MILESTONE_LAST_ENTRY),
     m_time_field(false),
-    m_alias_map(0),
+    m_alias_map(nullptr),
     m_set_func(_setfunc)
 {
-  ink_assert(m_name != NULL);
-  ink_assert(m_symbol != NULL);
+  ink_assert(m_name != nullptr);
+  ink_assert(m_symbol != nullptr);
   ink_assert(m_type >= 0 && m_type < N_TYPES);
-  ink_assert(m_marshal_func != (MarshalFunc)NULL);
+  ink_assert(m_marshal_func != (MarshalFunc) nullptr);
 
   m_time_field = (strcmp(m_symbol, "cqts") == 0 || strcmp(m_symbol, "cqth") == 0 || strcmp(m_symbol, "cqtq") == 0 ||
                   strcmp(m_symbol, "cqtn") == 0 || strcmp(m_symbol, "cqtd") == 0 || strcmp(m_symbol, "cqtt") == 0);
 }
 
 LogField::LogField(const char *name, const char *symbol, Type type, MarshalFunc marshal, UnmarshalFuncWithMap unmarshal,
-                   Ptr<LogFieldAliasMap> map, SetFunc _setfunc)
+                   const Ptr<LogFieldAliasMap> &map, SetFunc _setfunc)
   : m_name(ats_strdup(name)),
     m_symbol(ats_strdup(symbol)),
     m_type(type),
     m_container(NO_CONTAINER),
     m_marshal_func(marshal),
-    m_unmarshal_func(NULL),
+    m_unmarshal_func(nullptr),
     m_unmarshal_func_map(unmarshal),
     m_agg_op(NO_AGGREGATE),
     m_agg_cnt(0),
@@ -259,10 +262,10 @@ LogField::LogField(const char *name, const char *symbol, Type type, MarshalFunc
     m_alias_map(map),
     m_set_func(_setfunc)
 {
-  ink_assert(m_name != NULL);
-  ink_assert(m_symbol != NULL);
+  ink_assert(m_name != nullptr);
+  ink_assert(m_symbol != nullptr);
   ink_assert(m_type >= 0 && m_type < N_TYPES);
-  ink_assert(m_marshal_func != (MarshalFunc)NULL);
+  ink_assert(m_marshal_func != (MarshalFunc) nullptr);
   ink_assert(m_alias_map);
 
   m_time_field = (strcmp(m_symbol, "cqts") == 0 || strcmp(m_symbol, "cqth") == 0 || strcmp(m_symbol, "cqtq") == 0 ||
@@ -314,20 +317,20 @@ LogField::LogField(const char *field, Container container, SetFunc _setfunc)
     m_symbol(ats_strdup(container_names[container])),
     m_type(LogField::STRING),
     m_container(container),
-    m_marshal_func(NULL),
-    m_unmarshal_func(NULL),
-    m_unmarshal_func_map(NULL),
+    m_marshal_func(nullptr),
+    m_unmarshal_func(nullptr),
+    m_unmarshal_func_map(nullptr),
     m_agg_op(NO_AGGREGATE),
     m_agg_cnt(0),
     m_agg_val(0),
     m_milestone1(TS_MILESTONE_LAST_ENTRY),
     m_milestone2(TS_MILESTONE_LAST_ENTRY),
     m_time_field(false),
-    m_alias_map(0),
-    m_set_func(_setfunc)
+    m_alias_map(nullptr),
+    m_set_func(nullptr)
 {
-  ink_assert(m_name != NULL);
-  ink_assert(m_symbol != NULL);
+  ink_assert(m_name != nullptr);
+  ink_assert(m_symbol != nullptr);
   ink_assert(m_type >= 0 && m_type < N_TYPES);
 
   m_time_field = (strcmp(m_symbol, "cqts") == 0 || strcmp(m_symbol, "cqth") == 0 || strcmp(m_symbol, "cqtq") == 0 ||
@@ -345,7 +348,7 @@ LogField::LogField(const char *field, Container container, SetFunc _setfunc)
   case ESSH:
   case ECSSH:
   case SCFG:
-    m_unmarshal_func = (UnmarshalFunc) & (LogAccess::unmarshal_str);
+    m_unmarshal_func = reinterpret_cast<UnmarshalFunc>(&(LogAccess::unmarshal_str));
     break;
 
   case ICFG:
@@ -396,8 +399,8 @@ LogField::LogField(const LogField &rhs)
     m_alias_map(rhs.m_alias_map),
     m_set_func(rhs.m_set_func)
 {
-  ink_assert(m_name != NULL);
-  ink_assert(m_symbol != NULL);
+  ink_assert(m_name != nullptr);
+  ink_assert(m_symbol != nullptr);
   ink_assert(m_type >= 0 && m_type < N_TYPES);
 }
 
@@ -422,7 +425,7 @@ unsigned
 LogField::marshal_len(LogAccess *lad)
 {
   if (m_container == NO_CONTAINER) {
-    return (lad->*m_marshal_func)(NULL);
+    return (lad->*m_marshal_func)(nullptr);
   }
 
   switch (m_container) {
@@ -431,42 +434,68 @@ LogField::marshal_len(LogAccess *lad)
   case PQH:
   case SSH:
   case CSSH:
-    return lad->marshal_http_header_field(m_container, m_name, NULL);
+    return lad->marshal_http_header_field(m_container, m_name, nullptr);
 
   case ECQH:
   case EPSH:
   case EPQH:
   case ESSH:
   case ECSSH:
-    return lad->marshal_http_header_field_escapify(m_container, m_name, NULL);
+    return lad->marshal_http_header_field_escapify(m_container, m_name, nullptr);
 
   case ICFG:
-    return lad->marshal_config_int_var(m_name, NULL);
+    return lad->marshal_config_int_var(m_name, nullptr);
 
   case SCFG:
-    return lad->marshal_config_str_var(m_name, NULL);
+    return lad->marshal_config_str_var(m_name, nullptr);
 
   case RECORD:
-    return lad->marshal_record(m_name, NULL);
+    return lad->marshal_record(m_name, nullptr);
 
   case MS:
-    return lad->marshal_milestone(m_milestone1, NULL);
+    return lad->marshal_milestone(m_milestone1, nullptr);
 
   case MSDMS:
-    return lad->marshal_milestone_diff(m_milestone1, m_milestone2, NULL);
+    return lad->marshal_milestone_diff(m_milestone1, m_milestone2, nullptr);
 
   default:
     return 0;
   }
 }
 
+bool
+LogField::isContainerUpdateFieldSupported(Container container)
+{
+  switch (container) {
+  case CQH:
+  case PSH:
+  case PQH:
+  case SSH:
+  case CSSH:
+  case ECQH:
+  case EPSH:
+  case EPQH:
+  case ESSH:
+  case ECSSH:
+  case SCFG:
+    return true;
+  default:
+    return false;
+  }
+}
+
 void
 LogField::updateField(LogAccess *lad, char *buf, int len)
 {
   if (m_container == NO_CONTAINER) {
     return (lad->*m_set_func)(buf, len);
+  } else {
+    if (isContainerUpdateFieldSupported(m_container)) {
+      return set_http_header_field(lad, m_container, this->m_name, buf, len);
+    } else {
+      // no set function defined for the container
+    }
   }
-  // else...// future enhancement
 }
 
 /*-------------------------------------------------------------------------
@@ -522,7 +551,7 @@ LogField::marshal(LogAccess *lad, char *buf)
 unsigned
 LogField::marshal_agg(char *buf)
 {
-  ink_assert(buf != NULL);
+  ink_assert(buf != nullptr);
   int64_t avg = 0;
 
   switch (m_agg_op) {
@@ -566,9 +595,9 @@ unsigned
 LogField::unmarshal(char **buf, char *dest, int len)
 {
   if (!m_alias_map) {
-    if (m_unmarshal_func == (UnmarshalFunc)LogAccess::unmarshal_str ||
-        m_unmarshal_func == (UnmarshalFunc)LogAccess::unmarshal_http_text) {
-      UnmarshalFuncWithSlice func = (UnmarshalFuncWithSlice)m_unmarshal_func;
+    if (m_unmarshal_func == reinterpret_cast<UnmarshalFunc>(LogAccess::unmarshal_str) ||
+        m_unmarshal_func == reinterpret_cast<UnmarshalFunc>(LogAccess::unmarshal_http_text)) {
+      UnmarshalFuncWithSlice func = reinterpret_cast<UnmarshalFuncWithSlice>(m_unmarshal_func);
       return (*func)(buf, dest, len, &m_slice);
     }
     return (*m_unmarshal_func)(buf, dest, len);
@@ -645,8 +674,9 @@ LogField::update_aggregate(int64_t val)
     return;
   }
 
-  Debug("log-agg", "Aggregate field %s updated with val %" PRId64 ", "
-                   "new val = %" PRId64 ", cnt = %" PRId64 "",
+  Debug("log-agg",
+        "Aggregate field %s updated with val %" PRId64 ", "
+        "new val = %" PRId64 ", cnt = %" PRId64 "",
         m_symbol, val, m_agg_val, m_agg_cnt);
 }
 
@@ -655,7 +685,7 @@ LogField::valid_container_name(char *name)
 {
   for (unsigned i = 1; i < countof(container_names); i++) {
     if (strcmp(name, container_names[i]) == 0) {
-      return (LogField::Container)i;
+      return static_cast<LogField::Container>(i);
     }
   }
 
@@ -667,7 +697,7 @@ LogField::valid_aggregate_name(char *name)
 {
   for (unsigned i = 1; i < countof(aggregate_names); i++) {
     if (strcmp(name, aggregate_names[i]) == 0) {
-      return (LogField::Aggregate)i;
+      return static_cast<LogField::Aggregate>(i);
     }
   }
 
@@ -675,14 +705,14 @@ LogField::valid_aggregate_name(char *name)
 }
 
 bool
-LogField::fieldlist_contains_aggregates(char *fieldlist)
+LogField::fieldlist_contains_aggregates(const char *fieldlist)
 {
-  char *match;
+  const char *match;
 
   for (unsigned i = 1; i < countof(aggregate_names); i++) {
-    if ((match = strstr(fieldlist, aggregate_names[i])) != NULL) {
+    if ((match = strstr(fieldlist, aggregate_names[i])) != nullptr) {
       // verify that the aggregate string is not part of a container field name.
-      if ((strchr(fieldlist, '{') == NULL) && (strchr(match, '}') == NULL)) {
+      if ((strchr(fieldlist, '{') == nullptr) && (strchr(match, '}') == nullptr)) {
         return true;
       }
     }
@@ -691,6 +721,12 @@ LogField::fieldlist_contains_aggregates(char *fieldlist)
   return false;
 }
 
+void
+LogField::set_http_header_field(LogAccess *lad, LogField::Container container, char *field, char *buf, int len)
+{
+  return lad->set_http_header_field(container, field, buf, len);
+}
+
 /*-------------------------------------------------------------------------
   LogFieldList
 
@@ -698,9 +734,7 @@ LogField::fieldlist_contains_aggregates(char *fieldlist)
   heap with "new" and that each element is on at most ONE list.  To enforce
   this, items are copied by default, using the copy ctor.
   -------------------------------------------------------------------------*/
-LogFieldList::LogFieldList() : m_marshal_len(0)
-{
-}
+LogFieldList::LogFieldList() = default;
 
 LogFieldList::~LogFieldList()
 {
@@ -715,12 +749,13 @@ LogFieldList::clear()
     delete f; // safe given the semantics stated above
   }
   m_marshal_len = 0;
+  _badSymbols.clear();
 }
 
 void
 LogFieldList::add(LogField *field, bool copy)
 {
-  ink_assert(field != NULL);
+  ink_assert(field != nullptr);
 
   if (copy) {
     m_field_list.enqueue(new LogField(*field));
@@ -741,19 +776,18 @@ LogFieldList::find_by_name(const char *name) const
       return f;
     }
   }
-  return NULL;
+  return nullptr;
 }
 
 LogField *
 LogFieldList::find_by_symbol(const char *symbol) const
 {
-  LogField *field = 0;
+  LogField *field = nullptr;
 
-  if (Log::field_symbol_hash) {
-    if (ink_hash_table_lookup(Log::field_symbol_hash, (char *)symbol, (InkHashTableValue *)&field) && field) {
-      Debug("log-field-hash", "Field %s found", field->symbol());
-      return field;
-    }
+  if (auto it = Log::field_symbol_hash.find(symbol); it != Log::field_symbol_hash.end() && it->second) {
+    field = it->second;
+    Debug("log-field-hash", "Field %s found", field->symbol());
+    return field;
   }
   // trusty old method
 
@@ -763,7 +797,7 @@ LogFieldList::find_by_symbol(const char *symbol) const
     }
   }
 
-  return NULL;
+  return nullptr;
 }
 
 unsigned
@@ -772,7 +806,9 @@ LogFieldList::marshal_len(LogAccess *lad)
   int bytes = 0;
   for (LogField *f = first(); f; f = next(f)) {
     if (f->type() != LogField::sINT) {
-      bytes += f->marshal_len(lad);
+      const int len = f->marshal_len(lad);
+      ink_release_assert(len >= INK_MIN_ALIGN);
+      bytes += len;
     }
   }
   return m_marshal_len + bytes;
@@ -820,3 +856,12 @@ LogFieldList::display(FILE *fd)
     f->display(fd);
   }
 }
+
+void
+LogFieldList::addBadSymbol(std::string_view badSymbol)
+{
+  if (_badSymbols.size() > 0) {
+    _badSymbols += ", ";
+  }
+  _badSymbols += badSymbol;
+}
