@@ -27,26 +27,29 @@
 
  ***************************************************************************/
 
-#include "ts/ink_platform.h"
-#include "ts/SimpleTokenizer.h"
-#include "ts/ink_file.h"
+#include <vector>
+#include <string>
+#include <algorithm>
 
-#include <errno.h>
+#include "tscore/ink_platform.h"
+#include "tscore/SimpleTokenizer.h"
+#include "tscore/ink_file.h"
+
+#include <cerrno>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <libgen.h>
 
 #include "P_EventSystem.h"
 #include "I_Machine.h"
-#include "LogSock.h"
 
-#include "ts/BaseLogFile.h"
+#include "tscore/BaseLogFile.h"
 #include "LogField.h"
 #include "LogFilter.h"
 #include "LogFormat.h"
 #include "LogBuffer.h"
 #include "LogFile.h"
-#include "LogHost.h"
 #include "LogObject.h"
 #include "LogUtils.h"
 #include "LogConfig.h"
@@ -60,18 +63,19 @@
   -------------------------------------------------------------------------*/
 
 LogFile::LogFile(const char *name, const char *header, LogFileFormat format, uint64_t signature, size_t ascii_buffer_size,
-                 size_t max_line_size)
+                 size_t max_line_size, int pipe_buffer_size)
   : m_file_format(format),
     m_name(ats_strdup(name)),
     m_header(ats_strdup(header)),
     m_signature(signature),
-    m_max_line_size(max_line_size)
+    m_max_line_size(max_line_size),
+    m_pipe_buffer_size(pipe_buffer_size)
 {
   if (m_file_format != LOG_FILE_PIPE) {
     m_log = new BaseLogFile(name, m_signature);
     m_log->set_hostname(Machine::instance()->hostname);
   } else {
-    m_log = NULL;
+    m_log = nullptr;
   }
 
   m_fd                = -1;
@@ -94,6 +98,7 @@ LogFile::LogFile(const LogFile &copy)
     m_signature(copy.m_signature),
     m_ascii_buffer_size(copy.m_ascii_buffer_size),
     m_max_line_size(copy.m_max_line_size),
+    m_pipe_buffer_size(copy.m_pipe_buffer_size),
     m_fd(copy.m_fd)
 {
   ink_release_assert(m_ascii_buffer_size >= m_max_line_size);
@@ -101,7 +106,7 @@ LogFile::LogFile(const LogFile &copy)
   if (copy.m_log) {
     m_log = new BaseLogFile(*(copy.m_log));
   } else {
-    m_log = NULL;
+    m_log = nullptr;
   }
 
   Debug("log-file", "exiting LogFile copy constructor, m_name=%s, this=%p", m_name, this);
@@ -113,6 +118,12 @@ LogFile::LogFile(const LogFile &copy)
 LogFile::~LogFile()
 {
   Debug("log-file", "entering LogFile destructor, this=%p", this);
+
+  // close_file() checks whether a file is open before attempting to close, so
+  // this is safe to call even if a file had not been opened. Further, calling
+  // close_file() here ensures that we do not leak file descriptors.
+  close_file();
+
   delete m_log;
   ats_free(m_header);
   ats_free(m_name);
@@ -155,7 +166,7 @@ int
 LogFile::open_file()
 {
   // whatever we want to open should have a name
-  ink_assert(m_name != NULL);
+  ink_assert(m_name != nullptr);
 
   // is_open() takes into account if we're using BaseLogFile or a naked fd
   if (is_open()) {
@@ -182,6 +193,30 @@ LogFile::open_file()
       Debug("log-file", "no readers for pipe %s", m_name);
       return LOG_FILE_NO_PIPE_READERS;
     }
+
+#ifdef F_GETPIPE_SZ
+    // adjust pipe size if necessary
+    if (m_pipe_buffer_size) {
+      long pipe_size = static_cast<long>(fcntl(m_fd, F_GETPIPE_SZ));
+      if (pipe_size == -1) {
+        Error("Get pipe size failed for pipe %s: %s", m_name, strerror(errno));
+      } else {
+        Debug("log-file", "Previous buffer size for pipe %s: %ld", m_name, pipe_size);
+      }
+
+      int ret = fcntl(m_fd, F_SETPIPE_SZ, m_pipe_buffer_size);
+      if (ret == -1) {
+        Error("Set pipe size failed for pipe %s to size %d: %s", m_name, m_pipe_buffer_size, strerror(errno));
+      }
+
+      pipe_size = static_cast<long>(fcntl(m_fd, F_GETPIPE_SZ));
+      if (pipe_size == -1) {
+        Error("Get pipe size after setting it failed for pipe %s: %s", m_name, strerror(errno));
+      } else {
+        Debug("log-file", "New buffer size for pipe %s: %ld", m_name, pipe_size);
+      }
+    }
+#endif // F_GETPIPE_SZ
   } else {
     if (m_log) {
       int status = m_log->open_file(Log::config->logfile_perm);
@@ -193,14 +228,6 @@ LogFile::open_file()
     }
   }
 
-  int e = do_filesystem_checks();
-  if (e != 0) {
-    m_fd = -1; // reset to error condition
-    delete m_log;
-    m_log = NULL;
-    return LOG_FILE_FILESYSTEM_CHECKS_FAILED;
-  }
-
   //
   // If we've opened the file and it didn't already exist, then this is a
   // "new" file and we need to make some initializations.  This is the
@@ -231,18 +258,100 @@ LogFile::close_file()
 {
   if (is_open()) {
     if (m_file_format == LOG_FILE_PIPE) {
-      ::close(m_fd);
-      Debug("log-file", "LogFile %s (fd=%d) is closed", m_name, m_fd);
+      if (::close(m_fd)) {
+        Error("Error closing LogFile %s: %s.", m_name, strerror(errno));
+      } else {
+        Debug("log-file", "LogFile %s (fd=%d) is closed", m_name, m_fd);
+        RecIncrRawStat(log_rsb, this_thread()->mutex->thread_holding, log_stat_log_files_open_stat, -1);
+      }
       m_fd = -1;
     } else if (m_log) {
-      m_log->close_file();
-      Debug("log-file", "LogFile %s is closed", m_log->get_name());
+      if (m_log->close_file()) {
+        Error("Error closing LogFile %s: %s.", m_log->get_name(), strerror(errno));
+      } else {
+        Debug("log-file", "LogFile %s is closed", m_log->get_name());
+        RecIncrRawStat(log_rsb, this_thread()->mutex->thread_holding, log_stat_log_files_open_stat, -1);
+      }
     } else {
       Warning("LogFile %s is open but was not closed", m_name);
     }
   }
+}
 
-  RecIncrRawStat(log_rsb, this_thread()->mutex->thread_holding, log_stat_log_files_open_stat, -1);
+struct RolledFile {
+  RolledFile(const std::string &name, time_t mtime) : _name(name), _mtime(mtime) {}
+  std::string _name;
+  time_t _mtime;
+};
+
+/**
+ * @brief trim rolled files to max number of rolled files, older first
+ *
+ * @param rolling_max_count - limit to which rolled files will be trimmed
+ * @return true if success, false if failure
+ */
+bool
+LogFile::trim_rolled(size_t rolling_max_count)
+{
+  /* man: "dirname() may modify the contents of path, so it may be
+   * desirable to pass a copy when calling one of these functions." */
+  char *name = ats_strdup(m_name);
+  std::string logfile_dir(::dirname((name)));
+  ats_free(name);
+
+  /* Check logging directory access */
+  int err;
+  do {
+    err = access(logfile_dir.c_str(), R_OK | W_OK | X_OK);
+  } while ((err < 0) && (errno == EINTR));
+
+  if (err < 0) {
+    Error("Error accessing logging directory %s: %s.", logfile_dir.c_str(), strerror(errno));
+    return false;
+  }
+
+  /* Open logging directory */
+  DIR *ld = ::opendir(logfile_dir.c_str());
+  if (ld == nullptr) {
+    Error("Error opening logging directory %s to collect trim candidates: %s.", logfile_dir.c_str(), strerror(errno));
+    return false;
+  }
+
+  /* Collect the rolled file names from the logging directory that match the specified log file name */
+  std::vector<RolledFile> rolled;
+  char path[MAXPATHLEN];
+  struct dirent *entry;
+  while ((entry = readdir(ld))) {
+    struct stat sbuf;
+    snprintf(path, MAXPATHLEN, "%s/%s", logfile_dir.c_str(), entry->d_name);
+    int sret = ::stat(path, &sbuf);
+    if (sret != -1 && S_ISREG(sbuf.st_mode)) {
+      int name_len = strlen(m_name);
+      int path_len = strlen(path);
+      if (path_len > name_len && 0 == ::strncmp(m_name, path, name_len) && LogFile::rolled_logfile(entry->d_name)) {
+        rolled.push_back(RolledFile(path, sbuf.st_mtime));
+      }
+    }
+  }
+
+  ::closedir(ld);
+
+  bool result = true;
+  std::sort(rolled.begin(), rolled.end(), [](const RolledFile &a, const RolledFile &b) { return a._mtime > b._mtime; });
+  if (rolling_max_count < rolled.size()) {
+    for (auto it = rolled.begin() + rolling_max_count; it != rolled.end(); it++) {
+      const RolledFile &file = *it;
+      if (unlink(file._name.c_str()) < 0) {
+        Error("unable to auto-delete rolled logfile %s: %s.", file._name.c_str(), strerror(errno));
+        result = false;
+      } else {
+        Debug("log-file", "rolled logfile, %s, was auto-deleted", file._name.c_str());
+      }
+    }
+  }
+
+  rolled.clear();
+  return result;
 }
 
 /*-------------------------------------------------------------------------
@@ -252,10 +361,31 @@ LogFile::close_file()
  * Return 1 if file rolled, 0 otherwise
 -------------------------------------------------------------------------*/
 int
-LogFile::roll(long interval_start, long interval_end)
+LogFile::roll(long interval_start, long interval_end, bool reopen_after_rolling)
 {
   if (m_log) {
-    return m_log->roll(interval_start, interval_end);
+    // Due to commit 346b419 the BaseLogFile::close_file() is no longer called within BaseLogFile::roll().
+    // For diagnostic log files, the rolling is implemented by renaming and destroying the BaseLogFile object
+    // and then creating a new BaseLogFile object with the original filename. Due to possible race conditions
+    // the old/new object swap happens within lock/unlock calls within Diags.cc.
+    // For logging log files, the rolling is implemented by renaming the original file and closing it.
+    // Afterwards, the LogFile object will re-open a new file with the original file name using the original object.
+    // There is no need to protect against contention since the open/close/writes are all executed under a
+    // single log flush thread.
+    // Since these two methods of using BaseLogFile are not compatible, we perform the logging log file specific
+    // close file operation here within the containing LogFile object.
+    if (m_log->roll(interval_start, interval_end)) {
+      if (m_log->close_file()) {
+        Error("Error closing LogFile %s: %s.", m_log->get_name(), strerror(errno));
+      }
+
+      if (reopen_after_rolling) {
+        /* If we re-open now log file will be created even if there is nothing being logged */
+        m_log->open_file();
+      }
+
+      return 1;
+    }
   }
 
   return 0;
@@ -273,14 +403,14 @@ LogFile::preproc_and_try_delete(LogBuffer *lb)
   int ret = -1;
   LogBufferHeader *buffer_header;
 
-  if (lb == NULL) {
+  if (lb == nullptr) {
     Note("Cannot write LogBuffer to LogFile %s; LogBuffer is NULL", m_name);
     return -1;
   }
 
   ink_atomic_increment(&lb->m_references, 1);
 
-  if ((buffer_header = lb->header()) == NULL) {
+  if ((buffer_header = lb->header()) == nullptr) {
     Note("Cannot write LogBuffer to LogFile %s; LogBufferHeader is NULL", m_name);
     goto done;
   }
@@ -352,7 +482,7 @@ done:
 int
 LogFile::write_ascii_logbuffer(LogBufferHeader *buffer_header, int fd, const char *path, const char *alt_format)
 {
-  ink_assert(buffer_header != NULL);
+  ink_assert(buffer_header != nullptr);
   ink_assert(fd >= 0);
 
   char fmt_buf[LOG_MAX_FORMATTED_BUFFER];
@@ -369,7 +499,7 @@ LogFile::write_ascii_logbuffer(LogBufferHeader *buffer_header, int fd, const cha
 
   switch (buffer_header->version) {
   case LOG_SEGMENT_VERSION:
-    format_type = (LogFormatType)buffer_header->format_type;
+    format_type = static_cast<LogFormatType>(buffer_header->format_type);
 
     fieldlist_str = buffer_header->fmt_fieldlist();
     printf_str    = buffer_header->fmt_printf();
@@ -416,10 +546,11 @@ LogFile::write_ascii_logbuffer(LogBufferHeader *buffer_header, int fd, const cha
 int
 LogFile::write_ascii_logbuffer3(LogBufferHeader *buffer_header, const char *alt_format)
 {
-  Debug("log-file", "entering LogFile::write_ascii_logbuffer3 for %s "
-                    "(this=%p)",
+  Debug("log-file",
+        "entering LogFile::write_ascii_logbuffer3 for %s "
+        "(this=%p)",
         m_name, this);
-  ink_assert(buffer_header != NULL);
+  ink_assert(buffer_header != nullptr);
 
   ProxyMutex *mutex = this_thread()->mutex.get();
   LogBufferIterator iter(buffer_header);
@@ -435,7 +566,7 @@ LogFile::write_ascii_logbuffer3(LogBufferHeader *buffer_header, const char *alt_
 
   switch (buffer_header->version) {
   case LOG_SEGMENT_VERSION:
-    format_type   = (LogFormatType)buffer_header->format_type;
+    format_type   = static_cast<LogFormatType>(buffer_header->format_type);
     fieldlist_str = buffer_header->fmt_fieldlist();
     printf_str    = buffer_header->fmt_printf();
     break;
@@ -452,9 +583,9 @@ LogFile::write_ascii_logbuffer3(LogBufferHeader *buffer_header, const char *alt_
     fmt_buf_bytes   = 0;
 
     if (m_file_format == LOG_FILE_PIPE) {
-      ascii_buffer = (char *)malloc(m_max_line_size);
+      ascii_buffer = static_cast<char *>(ats_malloc(m_max_line_size));
     } else {
-      ascii_buffer = (char *)malloc(m_ascii_buffer_size);
+      ascii_buffer = static_cast<char *>(ats_malloc(m_ascii_buffer_size));
     }
 
     // fill the buffer with as many records as possible
@@ -545,7 +676,7 @@ LogFile::writeln(char *data, int len, int fd, const char *path)
 #else
     wvec[0].iov_base = (void *)data;
 #endif
-    wvec[0].iov_len = (size_t)len;
+    wvec[0].iov_len = static_cast<size_t>(len);
 
     if (data[len - 1] != '\n') {
 #if defined(solaris)
@@ -553,11 +684,11 @@ LogFile::writeln(char *data, int len, int fd, const char *path)
 #else
       wvec[1].iov_base = (void *)"\n";
 #endif
-      wvec[1].iov_len = (size_t)1;
+      wvec[1].iov_len = static_cast<size_t>(1);
       vcnt++;
     }
 
-    if ((bytes_this_write = (int)::writev(fd, (const struct iovec *)wvec, vcnt)) < 0) {
+    if ((bytes_this_write = static_cast<int>(::writev(fd, (const struct iovec *)wvec, vcnt))) < 0) {
       Warning("An error was encountered in writing to %s: %s.", ((path) ? path : "logfile"), strerror(errno));
     } else {
       total_bytes = bytes_this_write;
@@ -586,7 +717,7 @@ LogFile::check_fd()
     //
     // It's time to see if the file really exists.  If we can't see
     // the file (via access), then we'll close our descriptor and
-    // attept to re-open it, which will create the file if it's not
+    // attempt to re-open it, which will create the file if it's not
     // there.
     //
     if (m_name && !LogFile::exists(m_name)) {
@@ -643,7 +774,3 @@ LogFile::get_fd()
     return -1;
   }
 }
-
-/***************************************************************************
- LogFileList IS NOT USED
-****************************************************************************/
