@@ -26,22 +26,20 @@
 
 
  ***************************************************************************/
-#include "ts/ink_config.h"
+#include "tscore/ink_config.h"
 
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
+#include <cstdio>
+#include <cstring>
+#include <cstdlib>
 
-#include "ts/INK_MD5.h"
-
-#include "ts/SimpleTokenizer.h"
+#include "tscore/SimpleTokenizer.h"
+#include "tscore/CryptoHash.h"
 
 #include "LogUtils.h"
 #include "LogFile.h"
 #include "LogField.h"
 #include "LogFilter.h"
 #include "LogFormat.h"
-#include "LogHost.h"
 #include "LogBuffer.h"
 #include "LogObject.h"
 #include "LogConfig.h"
@@ -58,7 +56,7 @@ bool LogFormat::m_tagging_on = false;
 bool
 LogFormat::setup(const char *name, const char *format_str, unsigned interval_sec)
 {
-  if (name == NULL) {
+  if (name == nullptr) {
     Note("missing log format name");
     return false;
   }
@@ -66,7 +64,7 @@ LogFormat::setup(const char *name, const char *format_str, unsigned interval_sec
   if (format_str) {
     const char *tag                = " %<phn>";
     const size_t m_format_str_size = strlen(format_str) + (m_tagging_on ? strlen(tag) : 0) + 1;
-    m_format_str                   = (char *)ats_malloc(m_format_str_size);
+    m_format_str                   = static_cast<char *>(ats_malloc(m_format_str_size));
     ink_strlcpy(m_format_str, format_str, m_format_str_size);
     if (m_tagging_on) {
       Note("Log tagging enabled, adding %%<phn> field at the end of "
@@ -75,8 +73,8 @@ LogFormat::setup(const char *name, const char *format_str, unsigned interval_sec
       ink_strlcat(m_format_str, tag, m_format_str_size);
     };
 
-    char *printf_str    = NULL;
-    char *fieldlist_str = NULL;
+    char *printf_str    = nullptr;
+    char *fieldlist_str = nullptr;
     int nfields         = parse_format_string(m_format_str, &printf_str, &fieldlist_str);
     if (nfields > (m_tagging_on ? 1 : 0)) {
       init_variables(name, fieldlist_str, printf_str, interval_sec);
@@ -109,14 +107,14 @@ LogFormat::id_from_name(const char *name)
   int32_t id = 0;
   if (name) {
     CryptoHash hash;
-    MD5Context().hash_immediate(hash, name, static_cast<int>(strlen(name)));
+    CryptoContext().hash_immediate(hash, name, static_cast<int>(strlen(name)));
 #if defined(linux)
     /* Mask most signficant bit so that return value of this function
      * is not sign extended to be a negative number.
      * This problem is only known to occur on Linux which
      * is a 32-bit OS.
      */
-    id = (int32_t)hash.fold() & 0x7fffffff;
+    id = static_cast<int32_t>(hash.fold()) & 0x7fffffff;
 #else
     id = (int32_t)hash.fold();
 #endif
@@ -141,12 +139,12 @@ LogFormat::init_variables(const char *name, const char *fieldlist_str, const cha
     m_valid = false;
   } else {
     if (m_aggregate) {
-      m_agg_marshal_space = (char *)ats_malloc(m_field_count * INK_MIN_ALIGN);
+      m_agg_marshal_space = static_cast<char *>(ats_malloc(m_field_count * INK_MIN_ALIGN));
     }
 
     if (m_name_str) {
       ats_free(m_name_str);
-      m_name_str = NULL;
+      m_name_str = nullptr;
       m_name_id  = 0;
     }
     if (name) {
@@ -156,7 +154,7 @@ LogFormat::init_variables(const char *name, const char *fieldlist_str, const cha
 
     if (m_fieldlist_str) {
       ats_free(m_fieldlist_str);
-      m_fieldlist_str = NULL;
+      m_fieldlist_str = nullptr;
       m_fieldlist_id  = 0;
     }
     if (fieldlist_str) {
@@ -184,16 +182,16 @@ LogFormat::init_variables(const char *name, const char *fieldlist_str, const cha
 LogFormat::LogFormat(const char *name, const char *format_str, unsigned interval_sec)
   : m_interval_sec(0),
     m_interval_next(0),
-    m_agg_marshal_space(NULL),
+    m_agg_marshal_space(nullptr),
     m_valid(false),
-    m_name_str(NULL),
+    m_name_str(nullptr),
     m_name_id(0),
-    m_fieldlist_str(NULL),
+    m_fieldlist_str(nullptr),
     m_fieldlist_id(0),
     m_field_count(0),
-    m_printf_str(NULL),
+    m_printf_str(nullptr),
     m_aggregate(false),
-    m_format_str(NULL)
+    m_format_str(nullptr)
 {
   setup(name, format_str, interval_sec);
 
@@ -202,32 +200,6 @@ LogFormat::LogFormat(const char *name, const char *format_str, unsigned interval
   m_format_type = format_str ? LOG_FORMAT_CUSTOM : LOG_FORMAT_TEXT;
 }
 
-//-----------------------------------------------------------------------------
-// This constructor is used only in Log::match_logobject
-//
-// It is awkward because it does not take a format_str but a fieldlist_str
-// and a printf_str. These should be derived from a format_str but a
-// LogBufferHeader does not store the format_str, if it did, we could probably
-// delete this.
-//
-LogFormat::LogFormat(const char *name, const char *fieldlist_str, const char *printf_str, unsigned interval_sec)
-  : m_interval_sec(0),
-    m_interval_next(0),
-    m_agg_marshal_space(NULL),
-    m_valid(false),
-    m_name_str(NULL),
-    m_name_id(0),
-    m_fieldlist_str(NULL),
-    m_fieldlist_id(0),
-    m_field_count(0),
-    m_printf_str(NULL),
-    m_aggregate(false),
-    m_format_str(NULL)
-{
-  init_variables(name, fieldlist_str, printf_str, interval_sec);
-  m_format_type = LOG_FORMAT_CUSTOM;
-}
-
 /*-------------------------------------------------------------------------
   LogFormat::LogFormat
 
@@ -238,23 +210,23 @@ LogFormat::LogFormat(const LogFormat &rhs)
   : RefCountObj(rhs),
     m_interval_sec(0),
     m_interval_next(0),
-    m_agg_marshal_space(NULL),
+    m_agg_marshal_space(nullptr),
     m_valid(rhs.m_valid),
-    m_name_str(NULL),
+    m_name_str(nullptr),
     m_name_id(0),
-    m_fieldlist_str(NULL),
+    m_fieldlist_str(nullptr),
     m_fieldlist_id(0),
     m_field_count(0),
-    m_printf_str(NULL),
+    m_printf_str(nullptr),
     m_aggregate(false),
-    m_format_str(NULL),
+    m_format_str(nullptr),
     m_format_type(rhs.m_format_type)
 {
   if (m_valid) {
     if (m_format_type == LOG_FORMAT_TEXT) {
       m_name_str = ats_strdup(rhs.m_name_str);
     } else {
-      m_format_str = rhs.m_format_str ? ats_strdup(rhs.m_format_str) : 0;
+      m_format_str = rhs.m_format_str ? ats_strdup(rhs.m_format_str) : nullptr;
       init_variables(rhs.m_name_str, rhs.m_fieldlist_str, rhs.m_printf_str, rhs.m_interval_sec);
     }
   }
@@ -294,9 +266,9 @@ LogFormat::format_from_specification(char *spec, char **file_name, char **file_h
   int format_id;
   char *format_name, *format_str;
 
-  ink_assert(file_name != NULL);
-  ink_assert(file_header != NULL);
-  ink_assert(file_type != NULL);
+  ink_assert(file_name != nullptr);
+  ink_assert(file_header != nullptr);
+  ink_assert(file_type != nullptr);
 
   SimpleTokenizer tok(spec, ':');
 
@@ -309,15 +281,15 @@ LogFormat::format_from_specification(char *spec, char **file_name, char **file_h
   // First should be the "format" keyword that says this is a format spec.
   //
   token = tok.getNext();
-  if (token == NULL) {
+  if (token == nullptr) {
     Debug("log-format", "token expected");
-    return NULL;
+    return nullptr;
   }
   if (strcasecmp(token, "format") == 0) {
     Debug("log-format", "this is a format");
   } else {
     Debug("log-format", "should be 'format'");
-    return NULL;
+    return nullptr;
   }
 
   //
@@ -325,27 +297,27 @@ LogFormat::format_from_specification(char *spec, char **file_name, char **file_h
   // whether we should care about this format or not.
   //
   token = tok.getNext();
-  if (token == NULL) {
+  if (token == nullptr) {
     Debug("log-format", "token expected");
-    return NULL;
+    return nullptr;
   }
   if (!strcasecmp(token, "disabled")) {
     Debug("log-format", "format not enabled, skipping ...");
-    return NULL;
+    return nullptr;
   } else if (!strcasecmp(token, "enabled")) {
     Debug("log-format", "enabled format");
   } else {
     Debug("log-format", "should be 'enabled' or 'disabled', not %s", token);
-    return NULL;
+    return nullptr;
   }
 
   //
   // Next should be the numeric format identifier
   //
   token = tok.getNext();
-  if (token == NULL) {
+  if (token == nullptr) {
     Debug("log-format", "token expected");
-    return NULL;
+    return nullptr;
   }
   format_id = atoi(token);
   // NOW UNUSED !!!
@@ -354,9 +326,9 @@ LogFormat::format_from_specification(char *spec, char **file_name, char **file_h
   // Next should be the format name
   //
   token = tok.getNext();
-  if (token == NULL) {
+  if (token == nullptr) {
     Debug("log-format", "token expected");
-    return NULL;
+    return nullptr;
   }
   format_name = token;
 
@@ -364,9 +336,9 @@ LogFormat::format_from_specification(char *spec, char **file_name, char **file_h
   // Next should be the printf-stlye format symbol string
   //
   token = tok.getNext();
-  if (token == NULL) {
+  if (token == nullptr) {
     Debug("log-format", "token expected");
-    return NULL;
+    return nullptr;
   }
   format_str = token;
 
@@ -374,9 +346,9 @@ LogFormat::format_from_specification(char *spec, char **file_name, char **file_h
   // Next should be the file name for the log
   //
   token = tok.getNext();
-  if (token == NULL) {
+  if (token == nullptr) {
     Debug("log-format", "token expected");
-    return NULL;
+    return nullptr;
   }
   *file_name = ats_strdup(token);
 
@@ -384,9 +356,9 @@ LogFormat::format_from_specification(char *spec, char **file_name, char **file_h
   // Next should be the file type, either "ASCII" or "BINARY"
   //
   token = tok.getNext();
-  if (token == NULL) {
+  if (token == nullptr) {
     Debug("log-format", "token expected");
-    return NULL;
+    return nullptr;
   }
   if (!strcasecmp(token, "ASCII")) {
     *file_type = LOG_FILE_ASCII;
@@ -394,29 +366,29 @@ LogFormat::format_from_specification(char *spec, char **file_name, char **file_h
     *file_type = LOG_FILE_BINARY;
   } else {
     Debug("log-format", "%s is not a valid file format (ASCII or BINARY)", token);
-    return NULL;
+    return nullptr;
   }
 
   //
   // the rest should be the file header
   //
   token = tok.getRest();
-  if (token == NULL) {
+  if (token == nullptr) {
     Debug("log-format", "token expected");
-    return NULL;
+    return nullptr;
   }
   // set header to NULL if "none" was specified (a NULL header means
   // "write no header" to the rest of the logging system)
   //
-  *file_header = strcmp(token, "none") == 0 ? NULL : ats_strdup(token);
+  *file_header = strcmp(token, "none") == 0 ? nullptr : ats_strdup(token);
 
   Debug("log-format", "custom:%d:%s:%s:%s:%d:%s", format_id, format_name, format_str, *file_name, *file_type, token);
 
   format = new LogFormat(format_name, format_str);
-  ink_assert(format != NULL);
+  ink_assert(format != nullptr);
   if (!format->valid()) {
     delete format;
-    return NULL;
+    return nullptr;
   }
 
   return format;
@@ -440,11 +412,11 @@ LogFormat::parse_symbol_string(const char *symbol_string, LogFieldList *field_li
   LogField::Container container;
   LogField::Aggregate aggregate;
 
-  if (symbol_string == NULL) {
+  if (symbol_string == nullptr) {
     return 0;
   }
-  ink_assert(field_list != NULL);
-  ink_assert(contains_aggregates != NULL);
+  ink_assert(field_list != nullptr);
+  ink_assert(contains_aggregates != nullptr);
 
   *contains_aggregates = false; // we'll change if it does
 
@@ -454,7 +426,7 @@ LogFormat::parse_symbol_string(const char *symbol_string, LogFieldList *field_li
   sym_str = ats_strdup(symbol_string);
   symbol  = strtok_r(sym_str, ",", &saveptr);
 
-  while (symbol != NULL) {
+  while (symbol != nullptr) {
     //
     // See if there is an aggregate operator, which will contain "()"
     //
@@ -503,9 +475,9 @@ LogFormat::parse_symbol_string(const char *symbol_string, LogFieldList *field_li
     //
     else if (*symbol == '{') {
       Debug("log-format", "Container symbol: %s", symbol);
-      f              = NULL;
+      f              = nullptr;
       char *name_end = strchr(symbol, '}');
-      if (name_end != NULL) {
+      if (name_end != nullptr) {
         name      = symbol + 1;
         *name_end = 0;            // changes '}' to '\0'
         sym       = name_end + 1; // start of container symbol
@@ -516,7 +488,7 @@ LogFormat::parse_symbol_string(const char *symbol_string, LogFieldList *field_li
           Note("Invalid container specification: %s", sym);
         } else {
           f = new LogField(name, container);
-          ink_assert(f != NULL);
+          ink_assert(f != nullptr);
           if (slice.m_enable) {
             f->m_slice = slice;
             Debug("log-slice", "symbol = %s, [%d:%d]", sym, f->m_slice.m_start, f->m_slice.m_end);
@@ -538,7 +510,7 @@ LogFormat::parse_symbol_string(const char *symbol_string, LogFieldList *field_li
       LogSlice slice(symbol);
       Debug("log-format", "Regular field symbol: %s", symbol);
       f = Log::global_field_list.find_by_symbol(symbol);
-      if (f != NULL) {
+      if (f != nullptr) {
         LogField *cpy = new LogField(*f);
         if (slice.m_enable) {
           cpy->m_slice = slice;
@@ -551,13 +523,14 @@ LogFormat::parse_symbol_string(const char *symbol_string, LogFieldList *field_li
         Note("The log format symbol %s was not found in the "
              "list of known symbols.",
              symbol);
+        field_list->addBadSymbol(symbol);
       }
     }
 
     //
     // Get the next symbol
     //
-    symbol = strtok_r(NULL, ",", &saveptr);
+    symbol = strtok_r(nullptr, ",", &saveptr);
   }
 
   ats_free(sym_str);
@@ -570,11 +543,11 @@ LogFormat::parse_symbol_string(const char *symbol_string, LogFieldList *field_li
 // 1) Octal representation: '\abc', for example: '\060'
 //    0 < (a*8^2 + b*8 + c) < 255
 //
-// 2) Hex representation: '\xab', for exampe: '\x3A'
+// 2) Hex representation: '\xab', for example: '\x3A'
 //    0 < (a*16 + b) < 255
 //
 // Return -1 if the beginning four characters are not valid
-// escape sequence, otherwise reutrn unsigned char value of the
+// escape sequence, otherwise return unsigned char value of the
 // escape sequence in the string.
 //
 // NOTE: The value of escape sequence should be greater than 0
@@ -599,9 +572,9 @@ LogFormat::parse_escape_string(const char *str, int len)
     return -1;
   }
 
-  a = (unsigned char)str[start + 1];
-  b = (unsigned char)str[start + 2];
-  c = (unsigned char)str[start + 3];
+  a = static_cast<unsigned char>(str[start + 1]);
+  b = static_cast<unsigned char>(str[start + 2]);
+  c = static_cast<unsigned char>(str[start + 3]);
 
   if (isdigit(a) && isdigit(b)) {
     sum = (a - '0') * 64 + (b - '0') * 8 + (c - '0');
@@ -657,11 +630,11 @@ LogFormat::parse_escape_string(const char *str, int len)
 int
 LogFormat::parse_format_string(const char *format_str, char **printf_str, char **fields_str)
 {
-  ink_assert(printf_str != NULL);
-  ink_assert(fields_str != NULL);
+  ink_assert(printf_str != nullptr);
+  ink_assert(fields_str != nullptr);
 
-  if (format_str == NULL) {
-    *printf_str = *fields_str = NULL;
+  if (format_str == nullptr) {
+    *printf_str = *fields_str = nullptr;
     return 0;
   }
   //
@@ -670,9 +643,9 @@ LogFormat::parse_format_string(const char *format_str, char **printf_str, char *
   // each is guaranteed to be smaller (or the same size) as the format
   // string.
   //
-  unsigned len = (unsigned)::strlen(format_str);
-  *printf_str  = (char *)ats_malloc(len + 1);
-  *fields_str  = (char *)ats_malloc(len + 1);
+  unsigned len = static_cast<unsigned>(::strlen(format_str));
+  *printf_str  = static_cast<char *>(ats_malloc(len + 1));
+  *fields_str  = static_cast<char *>(ats_malloc(len + 1));
 
   unsigned printf_pos  = 0;
   unsigned fields_pos  = 0;
@@ -719,10 +692,10 @@ LogFormat::parse_format_string(const char *format_str, char **printf_str, char *
 
         if (escape_char == '\\') {
           start += 1;
-          (*printf_str)[printf_pos++] = (char)escape_char;
+          (*printf_str)[printf_pos++] = static_cast<char>(escape_char);
         } else if (escape_char >= 0) {
           start += 3;
-          (*printf_str)[printf_pos++] = (char)escape_char;
+          (*printf_str)[printf_pos++] = static_cast<char>(escape_char);
         } else {
           memcpy(&(*printf_str)[printf_pos], &format_str[start], stop - start + 1);
           printf_pos += stop - start + 1;
@@ -737,10 +710,10 @@ LogFormat::parse_format_string(const char *format_str, char **printf_str, char *
 
       if (escape_char == '\\') {
         start += 1;
-        (*printf_str)[printf_pos++] = (char)escape_char;
+        (*printf_str)[printf_pos++] = static_cast<char>(escape_char);
       } else if (escape_char >= 0) {
         start += 3;
-        (*printf_str)[printf_pos++] = (char)escape_char;
+        (*printf_str)[printf_pos++] = static_cast<char>(escape_char);
       } else {
         (*printf_str)[printf_pos++] = format_str[start];
       }
@@ -785,9 +758,7 @@ LogFormat::display(FILE *fd)
   LogFormatList
   -------------------------------------------------------------------------*/
 
-LogFormatList::LogFormatList()
-{
-}
+LogFormatList::LogFormatList() = default;
 
 LogFormatList::~LogFormatList()
 {
@@ -806,7 +777,7 @@ LogFormatList::clear()
 void
 LogFormatList::add(LogFormat *format, bool copy)
 {
-  ink_assert(format != NULL);
+  ink_assert(format != nullptr);
 
   if (copy) {
     m_format_list.enqueue(new LogFormat(*format));
@@ -823,7 +794,7 @@ LogFormatList::find_by_name(const char *name) const
       return f;
     }
   }
-  return NULL;
+  return nullptr;
 }
 
 unsigned
