@@ -21,20 +21,35 @@
   limitations under the License.
  */
 
-#include <ts/TestBox.h>
+#include "tscore/TestBox.h"
 #include "LogUtils.cc"
 #include <string.h>
 
+#if 0
+// Stub
+EThread *
+this_ethread()
+{
+  return nullptr;
+}
+#endif
+
 REGRESSION_TEST(LogUtils_pure_escapify_url)(RegressionTest *t, int /* atype ATS_UNUSED */, int *pstatus)
 {
   TestBox box(t, pstatus);
   box = REGRESSION_TEST_PASSED;
 
   char input[][32] = {
-    " ", "%", "% ", "%20",
+    " ",
+    "%",
+    "% ",
+    "%20",
   };
   const char *expected[] = {
-    "%20", "%25", "%25%20", "%2520",
+    "%20",
+    "%25",
+    "%25%20",
+    "%2520",
   };
   char output[128];
   int output_len;
@@ -52,10 +67,16 @@ REGRESSION_TEST(LogUtils_escapify_url)(RegressionTest *t, int /* atype ATS_UNUSE
   box = REGRESSION_TEST_PASSED;
 
   char input[][32] = {
-    " ", "%", "% ", "%20",
+    " ",
+    "%",
+    "% ",
+    "%20",
   };
   const char *expected[] = {
-    "%20", "%25", "%25%20", "%20",
+    "%20",
+    "%25",
+    "%25%20",
+    "%20",
   };
   char output[128];
   int output_len;
